/*
 ***********************************************************************************************************************
 *
 *  Copyright (c) 2016-2019 Advanced Micro Devices, Inc. All Rights Reserved.
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in all
 *  copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 *  SOFTWARE.
 *
 **********************************************************************************************************************/

#pragma once

#include "pal.h"

namespace GpuUtil
{

/// Namespace to encapsulate all constants and binary data related to the TextWriter Font.
namespace TextWriterFont
{

constexpr Pal::uint32 LetterWidth  = 10; ///< Each letter is 10 pixels wide.
constexpr Pal::uint32 LetterHeight = 16; ///< Each letter is 16 pixels tall.

/**
 ***********************************************************************************************************************
 * @brief Binary data that represents the pixels enabled in a 10x16 rectangle for ASCII characters.
 *
 * Every byte contains four contiguous pixels in the X direction of the font image.
 *     The pixels are set up like a bit mask, i.e. 0x44332211
 *     If the pixel is equal to 0, there is no color there.
 *     If the pixel is equal to 1, there is foreground color there.
 *     If the pixel is equal to 2, there is background color there.
 ***********************************************************************************************************************
 */
constexpr Pal::uint8 FontData[] =
{
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0xa0, 0x02, 0x00, 0x00, 0x60, 0x02, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0x60, 0x02, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0x60, 0x02, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0x60, 0x02, 0x00,
    0x00, 0xa0, 0x02, 0x00, 0x00, 0x60, 0x02, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0xa0, 0x02, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0xa8, 0x2a, 0x00, 0x00, 0x98, 0x26, 0x00,
    0x00, 0x98, 0x26, 0x00, 0x00, 0x98, 0x26, 0x00,
    0x00, 0x98, 0x29, 0x00, 0x00, 0xa8, 0x0a, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xa8, 0x2a, 0x00,
    0x00, 0x98, 0x26, 0x00, 0x00, 0x9a, 0xa6, 0x00,
    0x00, 0x56, 0x95, 0x00, 0x00, 0x9a, 0xa6, 0x00,
    0x80, 0x9a, 0xa9, 0x00, 0x80, 0x55, 0x95, 0x00,
    0x80, 0x9a, 0xa9, 0x00, 0x00, 0x98, 0x09, 0x00,
    0x00, 0xa8, 0x09, 0x00, 0x00, 0x80, 0x0a, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xa8, 0x2a, 0x00,
    0x00, 0x5a, 0x25, 0x00, 0x00, 0x66, 0x26, 0x00,
    0x00, 0x66, 0x2a, 0x00, 0x00, 0x5a, 0x2a, 0x00,
    0x00, 0x68, 0xa5, 0x00, 0x00, 0x60, 0x9a, 0x00,
    0x00, 0x6a, 0x9a, 0x00, 0x00, 0x66, 0x96, 0x00,
    0x00, 0x5a, 0xa9, 0x00, 0x00, 0xa8, 0x0a, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xaa, 0x2a, 0x00,
    0x80, 0x56, 0x26, 0x00, 0x80, 0x69, 0x26, 0x00,
    0x80, 0x56, 0x29, 0x00, 0x00, 0xaa, 0x09, 0x00,
    0x00, 0x60, 0x2a, 0x00, 0x00, 0x68, 0xa6, 0x00,
    0x00, 0x9a, 0x99, 0x00, 0x00, 0xa6, 0x99, 0x00,
    0x00, 0xa6, 0x95, 0x00, 0x00, 0xaa, 0xaa, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xa8, 0x0a, 0x00,
    0x00, 0x5a, 0x09, 0x00, 0x00, 0xa6, 0x09, 0x00,
    0x00, 0x9a, 0x09, 0x00, 0x00, 0x58, 0x09, 0x00,
    0x00, 0x5a, 0xaa, 0x00, 0x00, 0xa6, 0x99, 0x00,
    0x00, 0xa6, 0x95, 0x00, 0x00, 0xa6, 0xa6, 0x00,
    0x00, 0x56, 0x99, 0x00, 0x00, 0xaa, 0xaa, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x80, 0x0a, 0x00, 0x00, 0x80, 0x09, 0x00,
    0x00, 0xa0, 0x09, 0x00, 0x00, 0x60, 0x0a, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0xa0, 0x02, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x2a, 0x00, 0x00, 0x80, 0x26, 0x00,
    0x00, 0xa0, 0x29, 0x00, 0x00, 0x60, 0x0a, 0x00,
    0x00, 0x68, 0x02, 0x00, 0x00, 0x98, 0x02, 0x00,
    0x00, 0x98, 0x00, 0x00, 0x00, 0x98, 0x00, 0x00,
    0x00, 0x98, 0x02, 0x00, 0x00, 0x58, 0x02, 0x00,
    0x00, 0x68, 0x0a, 0x00, 0x00, 0x60, 0x29, 0x00,
    0x00, 0xa0, 0x25, 0x00, 0x00, 0x80, 0x2a, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x2a, 0x00, 0x00, 0x00, 0xa6, 0x02, 0x00,
    0x00, 0x5a, 0x02, 0x00, 0x00, 0x68, 0x0a, 0x00,
    0x00, 0xa0, 0x09, 0x00, 0x00, 0x80, 0x09, 0x00,
    0x00, 0x80, 0x09, 0x00, 0x00, 0x80, 0x09, 0x00,
    0x00, 0x80, 0x09, 0x00, 0x00, 0xa0, 0x09, 0x00,
    0x00, 0x60, 0x09, 0x00, 0x00, 0x6a, 0x0a, 0x00,
    0x00, 0x96, 0x02, 0x00, 0x00, 0xa6, 0x00, 0x00,
    0x00, 0x2a, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0xa0, 0x02, 0x00, 0x00, 0x60, 0x02, 0x00,
    0x00, 0x6a, 0xaa, 0x00, 0x00, 0x56, 0x96, 0x00,
    0x00, 0x6a, 0xa9, 0x00, 0x00, 0x9a, 0x29, 0x00,
    0x00, 0x96, 0x26, 0x00, 0x00, 0xaa, 0x2a, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0xa0, 0x02, 0x00, 0x00, 0x60, 0x02, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0x6a, 0xaa, 0x00,
    0x00, 0x56, 0x95, 0x00, 0x00, 0x6a, 0xaa, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0xa0, 0x02, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0xa0, 0x02, 0x00, 0x00, 0x60, 0x0a, 0x00,
    0x00, 0x60, 0x09, 0x00, 0x00, 0x60, 0x0a, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0xa0, 0x02, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xaa, 0x2a, 0x00,
    0x00, 0x56, 0x25, 0x00, 0x00, 0xaa, 0x2a, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0xa0, 0x02, 0x00, 0x00, 0x60, 0x02, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0xa0, 0x02, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0xaa, 0x00, 0x00, 0x00, 0x96, 0x00,
    0x00, 0x00, 0xa6, 0x00, 0x00, 0x80, 0x26, 0x00,
    0x00, 0x80, 0x29, 0x00, 0x00, 0xa0, 0x09, 0x00,
    0x00, 0x60, 0x0a, 0x00, 0x00, 0x68, 0x02, 0x00,
    0x00, 0x98, 0x02, 0x00, 0x00, 0x9a, 0x00, 0x00,
    0x00, 0xa6, 0x00, 0x00, 0x00, 0x2a, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xa8, 0x2a, 0x00,
    0x00, 0x5a, 0x25, 0x00, 0x00, 0xa6, 0xa6, 0x00,
    0x00, 0xa6, 0x96, 0x00, 0x00, 0xa6, 0x99, 0x00,
    0x00, 0x66, 0x9a, 0x00, 0x00, 0x96, 0x9a, 0x00,
    0x00, 0x96, 0xa6, 0x00, 0x00, 0x96, 0x26, 0x00,
    0x00, 0x5a, 0x29, 0x00, 0x00, 0xa8, 0x0a, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xa8, 0x02, 0x00,
    0x00, 0x58, 0x02, 0x00, 0x00, 0x68, 0x02, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0x60, 0x02, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0x60, 0x02, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0x60, 0x02, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0xa0, 0x02, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xa8, 0x2a, 0x00,
    0x00, 0x58, 0x25, 0x00, 0x00, 0xa8, 0x26, 0x00,
    0x00, 0x00, 0x26, 0x00, 0x00, 0x80, 0x26, 0x00,
    0x00, 0xa0, 0x29, 0x00, 0x00, 0x68, 0x0a, 0x00,
    0x00, 0x98, 0x02, 0x00, 0x00, 0x9a, 0xaa, 0x00,
    0x00, 0x56, 0x95, 0x00, 0x00, 0xaa, 0xaa, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xaa, 0x2a, 0x00,
    0x00, 0x56, 0x25, 0x00, 0x00, 0xaa, 0x26, 0x00,
    0x00, 0x80, 0x26, 0x00, 0x00, 0xa0, 0x25, 0x00,
    0x00, 0x60, 0x25, 0x00, 0x00, 0xa0, 0x26, 0x00,
    0x00, 0x00, 0x26, 0x00, 0x00, 0xaa, 0x26, 0x00,
    0x00, 0x56, 0x29, 0x00, 0x00, 0xaa, 0x0a, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x2a, 0x00,
    0x00, 0xa0, 0x25, 0x00, 0x00, 0x60, 0x25, 0x00,
    0x00, 0x68, 0x25, 0x00, 0x00, 0x9a, 0x25, 0x00,
    0x00, 0xa6, 0xa5, 0x00, 0x00, 0x56, 0x95, 0x00,
    0x00, 0xaa, 0xa5, 0x00, 0x00, 0x80, 0x25, 0x00,
    0x00, 0x80, 0x25, 0x00, 0x00, 0x80, 0x2a, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xa8, 0x2a, 0x00,
    0x00, 0x5a, 0x25, 0x00, 0x00, 0xa6, 0x2a, 0x00,
    0x00, 0xa6, 0x2a, 0x00, 0x00, 0x56, 0x25, 0x00,
    0x00, 0xa6, 0xa6, 0x00, 0x00, 0x2a, 0x9a, 0x00,
    0x00, 0x2a, 0x9a, 0x00, 0x00, 0xa6, 0xa6, 0x00,
    0x00, 0x5a, 0x25, 0x00, 0x00, 0xa8, 0x2a, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xa8, 0x2a, 0x00,
    0x00, 0x58, 0x25, 0x00, 0x00, 0x9a, 0x2a, 0x00,
    0x00, 0xa6, 0x2a, 0x00, 0x00, 0x66, 0x25, 0x00,
    0x00, 0x96, 0xa6, 0x00, 0x00, 0xa6, 0x9a, 0x00,
    0x00, 0xa6, 0x9a, 0x00, 0x00, 0x96, 0xa6, 0x00,
    0x00, 0x5a, 0x25, 0x00, 0x00, 0xa8, 0x2a, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xaa, 0x2a, 0x00,
    0x00, 0x56, 0x25, 0x00, 0x00, 0xaa, 0x26, 0x00,
    0x00, 0x80, 0x25, 0x00, 0x00, 0x80, 0x29, 0x00,
    0x00, 0xa0, 0x09, 0x00, 0x00, 0x60, 0x0a, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0x68, 0x02, 0x00,
    0x00, 0x98, 0x02, 0x00, 0x00, 0xa8, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xa8, 0x2a, 0x00,
    0x00, 0x5a, 0x25, 0x00, 0x00, 0xa6, 0x26, 0x00,
    0x00, 0x96, 0x26, 0x00, 0x00, 0x5a, 0x25, 0x00,
    0x00, 0x9a, 0x25, 0x00, 0x00, 0xa6, 0xa6, 0x00,
    0x00, 0x26, 0x9a, 0x00, 0x00, 0xa6, 0xa6, 0x00,
    0x00, 0x5a, 0x25, 0x00, 0x00, 0xa8, 0x2a, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xa8, 0x2a, 0x00,
    0x00, 0x5a, 0x25, 0x00, 0x00, 0xa6, 0x26, 0x00,
    0x00, 0x26, 0xa6, 0x00, 0x00, 0xa6, 0x96, 0x00,
    0x00, 0x9a, 0x95, 0x00, 0x00, 0x68, 0xa6, 0x00,
    0x00, 0xa0, 0x26, 0x00, 0x00, 0xaa, 0x26, 0x00,
    0x00, 0x56, 0x29, 0x00, 0x00, 0xaa, 0x0a, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0xa0, 0x02, 0x00, 0x00, 0x60, 0x02, 0x00,
    0x00, 0xa0, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0xa0, 0x02, 0x00, 0x00, 0x60, 0x02, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0xa0, 0x02, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0xa0, 0x02, 0x00, 0x00, 0x60, 0x02, 0x00,
    0x00, 0xa0, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0xa0, 0x02, 0x00, 0x00, 0x60, 0x0a, 0x00,
    0x00, 0x60, 0x09, 0x00, 0x00, 0x60, 0x0a, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0xa0, 0x02, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0xaa, 0x00, 0x00, 0xa0, 0x96, 0x00,
    0x00, 0x6a, 0xa9, 0x00, 0x00, 0x96, 0x0a, 0x00,
    0x00, 0xa6, 0x02, 0x00, 0x00, 0x5a, 0x2a, 0x00,
    0x00, 0xa8, 0xa5, 0x00, 0x00, 0x80, 0x9a, 0x00,
    0x00, 0x00, 0xa8, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0x00,
    0x00, 0x56, 0x95, 0x00, 0x00, 0xaa, 0xaa, 0x00,
    0x00, 0xaa, 0x2a, 0x00, 0x00, 0x56, 0x25, 0x00,
    0x00, 0xaa, 0x2a, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0xaa, 0x00, 0x00, 0x00, 0x96, 0x02, 0x00,
    0x00, 0x5a, 0x2a, 0x00, 0x00, 0xa8, 0xa5, 0x00,
    0x00, 0x80, 0x96, 0x00, 0x00, 0xa8, 0xa9, 0x00,
    0x00, 0x5a, 0x0a, 0x00, 0x00, 0xa6, 0x02, 0x00,
    0x00, 0x2a, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0xa8, 0x0a, 0x00, 0x00, 0x5a, 0x29, 0x00,
    0x00, 0xa6, 0xa6, 0x00, 0x00, 0x2a, 0x96, 0x00,
    0x00, 0x80, 0xa6, 0x00, 0x00, 0x80, 0x25, 0x00,
    0x00, 0xa0, 0x29, 0x00, 0x00, 0x60, 0x0a, 0x00,
    0x00, 0xa0, 0x02, 0x00, 0x00, 0xa0, 0x0a, 0x00,
    0x00, 0x60, 0x09, 0x00, 0x00, 0xa0, 0x0a, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xa8, 0x2a, 0x00,
    0x00, 0x5a, 0xa5, 0x00, 0x00, 0xa6, 0x9a, 0x00,
    0x80, 0xa6, 0x95, 0x00, 0x80, 0x69, 0x9a, 0x00,
    0x80, 0x69, 0x9a, 0x00, 0x80, 0x69, 0x96, 0x00,
    0x80, 0xa6, 0xa9, 0x00, 0x00, 0xa6, 0x2a, 0x00,
    0x00, 0x5a, 0x25, 0x00, 0x00, 0xa8, 0x2a, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xa0, 0x02, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0x68, 0x0a, 0x00,
    0x00, 0x98, 0x09, 0x00, 0x00, 0x98, 0x29, 0x00,
    0x00, 0x9a, 0x26, 0x00, 0x00, 0x56, 0x25, 0x00,
    0x00, 0xa6, 0xa6, 0x00, 0x80, 0x26, 0x9a, 0x00,
    0x80, 0x29, 0x98, 0x00, 0x80, 0x0a, 0xa8, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xaa, 0x2a, 0x00,
    0x00, 0x56, 0x25, 0x00, 0x00, 0xa6, 0x26, 0x00,
    0x00, 0xa6, 0x26, 0x00, 0x00, 0xa6, 0x25, 0x00,
    0x00, 0x56, 0xa5, 0x00, 0x00, 0xa6, 0x9a, 0x00,
    0x00, 0x26, 0x9a, 0x00, 0x00, 0xa6, 0x96, 0x00,
    0x00, 0x56, 0xa5, 0x00, 0x00, 0xaa, 0x2a, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xa8, 0x2a, 0x00,
    0x00, 0x5a, 0xa5, 0x00, 0x00, 0xa6, 0x9a, 0x00,
    0x00, 0x26, 0xa8, 0x00, 0x00, 0x26, 0x00, 0x00,
    0x00, 0x26, 0x00, 0x00, 0x00, 0x26, 0x00, 0x00,
    0x00, 0xa6, 0xa8, 0x00, 0x00, 0x96, 0x9a, 0x00,
    0x00, 0x5a, 0xa5, 0x00, 0x00, 0xa8, 0x2a, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xaa, 0x2a, 0x00,
    0x00, 0x56, 0x25, 0x00, 0x00, 0xa6, 0xa6, 0x00,
    0x00, 0x26, 0x9a, 0x00, 0x00, 0x26, 0x98, 0x00,
    0x00, 0x26, 0x98, 0x00, 0x00, 0x26, 0x9a, 0x00,
    0x00, 0x26, 0x96, 0x00, 0x00, 0xa6, 0xa6, 0x00,
    0x00, 0x56, 0x29, 0x00, 0x00, 0xaa, 0x0a, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0x00,
    0x00, 0x56, 0x95, 0x00, 0x00, 0xa6, 0xaa, 0x00,
    0x00, 0x26, 0x00, 0x00, 0x00, 0xa6, 0x2a, 0x00,
    0x00, 0x56, 0x25, 0x00, 0x00, 0xa6, 0x2a, 0x00,
    0x00, 0x26, 0x00, 0x00, 0x00, 0xa6, 0xaa, 0x00,
    0x00, 0x56, 0x95, 0x00, 0x00, 0xaa, 0xaa, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0x00,
    0x00, 0x56, 0x95, 0x00, 0x00, 0xa6, 0xaa, 0x00,
    0x00, 0x26, 0x00, 0x00, 0x00, 0xa6, 0x0a, 0x00,
    0x00, 0x56, 0x09, 0x00, 0x00, 0xa6, 0x0a, 0x00,
    0x00, 0x26, 0x00, 0x00, 0x00, 0x26, 0x00, 0x00,
    0x00, 0x26, 0x00, 0x00, 0x00, 0x2a, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xa8, 0x2a, 0x00,
    0x00, 0x5a, 0xa5, 0x00, 0x00, 0xa6, 0x9a, 0x00,
    0x00, 0x26, 0xa8, 0x00, 0x00, 0x26, 0x00, 0x00,
    0x00, 0x26, 0xaa, 0x00, 0x00, 0x26, 0x96, 0x00,
    0x00, 0xa6, 0x9a, 0x00, 0x00, 0x96, 0x9a, 0x00,
    0x00, 0x5a, 0xa5, 0x00, 0x00, 0xa8, 0x2a, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x2a, 0xa8, 0x00,
    0x00, 0x26, 0x98, 0x00, 0x00, 0x26, 0x98, 0x00,
    0x00, 0x26, 0x98, 0x00, 0x00, 0xa6, 0x9a, 0x00,
    0x00, 0x56, 0x95, 0x00, 0x00, 0xa6, 0x9a, 0x00,
    0x00, 0x26, 0x98, 0x00, 0x00, 0x26, 0x98, 0x00,
    0x00, 0x26, 0x98, 0x00, 0x00, 0x2a, 0xa8, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xaa, 0x2a, 0x00,
    0x00, 0x56, 0x25, 0x00, 0x00, 0x6a, 0x2a, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0x60, 0x02, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0x60, 0x02, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0x6a, 0x2a, 0x00,
    0x00, 0x56, 0x25, 0x00, 0x00, 0xaa, 0x2a, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xa0, 0xaa, 0x00,
    0x00, 0x60, 0x95, 0x00, 0x00, 0xa0, 0xa9, 0x00,
    0x00, 0x80, 0x09, 0x00, 0x00, 0x80, 0x09, 0x00,
    0x00, 0x80, 0x09, 0x00, 0x00, 0x80, 0x09, 0x00,
    0x00, 0x80, 0x09, 0x00, 0x00, 0xaa, 0x09, 0x00,
    0x00, 0x56, 0x09, 0x00, 0x00, 0xaa, 0x0a, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x2a, 0x2a, 0x00,
    0x00, 0xa6, 0x26, 0x00, 0x00, 0xa6, 0x29, 0x00,
    0x00, 0x66, 0x0a, 0x00, 0x00, 0x96, 0x02, 0x00,
    0x00, 0x66, 0x0a, 0x00, 0x00, 0x66, 0x09, 0x00,
    0x00, 0xa6, 0x29, 0x00, 0x00, 0xa6, 0xa6, 0x00,
    0x00, 0x26, 0x96, 0x00, 0x00, 0x2a, 0xaa, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x2a, 0x00, 0x00,
    0x00, 0x26, 0x00, 0x00, 0x00, 0x26, 0x00, 0x00,
    0x00, 0x26, 0x00, 0x00, 0x00, 0x26, 0x00, 0x00,
    0x00, 0x26, 0x00, 0x00, 0x00, 0x26, 0x00, 0x00,
    0x00, 0x26, 0x00, 0x00, 0x00, 0xa6, 0xaa, 0x00,
    0x00, 0x56, 0x95, 0x00, 0x00, 0xaa, 0xaa, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x2a, 0xa8, 0x00,
    0x00, 0x26, 0x9a, 0x00, 0x00, 0xa6, 0x96, 0x00,
    0x00, 0x96, 0x96, 0x00, 0x00, 0x96, 0x99, 0x00,
    0x00, 0x66, 0x99, 0x00, 0x00, 0x66, 0x9a, 0x00,
    0x00, 0xa6, 0x9a, 0x00, 0x00, 0x26, 0x98, 0x00,
    0x00, 0x26, 0x98, 0x00, 0x00, 0x2a, 0xa8, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x2a, 0xa8, 0x00,
    0x00, 0xa6, 0x98, 0x00, 0x00, 0x96, 0x98, 0x00,
    0x00, 0x96, 0x9a, 0x00, 0x00, 0x66, 0x9a, 0x00,
    0x00, 0x66, 0x99, 0x00, 0x00, 0xa6, 0x99, 0x00,
    0x00, 0xa6, 0x95, 0x00, 0x00, 0xa6, 0x96, 0x00,
    0x00, 0x26, 0x9a, 0x00, 0x00, 0x2a, 0xa8, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xa8, 0x2a, 0x00,
    0x00, 0x5a, 0xa5, 0x00, 0x00, 0xa6, 0x96, 0x00,
    0x80, 0x26, 0x9a, 0x00, 0x80, 0x29, 0x98, 0x00,
    0x80, 0x29, 0x98, 0x00, 0x80, 0x26, 0x98, 0x00,
    0x00, 0x26, 0x9a, 0x00, 0x00, 0xa6, 0xa6, 0x00,
    0x00, 0x5a, 0x25, 0x00, 0x00, 0xa8, 0x2a, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xaa, 0x2a, 0x00,
    0x00, 0x56, 0xa5, 0x00, 0x00, 0xa6, 0x9a, 0x00,
    0x00, 0x26, 0x9a, 0x00, 0x00, 0xa6, 0x96, 0x00,
    0x00, 0x56, 0xa5, 0x00, 0x00, 0xa6, 0x2a, 0x00,
    0x00, 0x26, 0x00, 0x00, 0x00, 0x26, 0x00, 0x00,
    0x00, 0x26, 0x00, 0x00, 0x00, 0x2a, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xa8, 0x2a, 0x00,
    0x00, 0x5a, 0xa5, 0x00, 0x00, 0xa6, 0x96, 0x00,
    0x80, 0x26, 0x9a, 0x00, 0x80, 0x29, 0x98, 0x00,
    0x80, 0x29, 0x98, 0x00, 0x80, 0x26, 0x98, 0x00,
    0x00, 0x26, 0x9a, 0x00, 0x00, 0xa6, 0xa6, 0x00,
    0x00, 0x5a, 0x25, 0x00, 0x00, 0x68, 0xaa, 0x00,
    0x00, 0xa0, 0x95, 0x00, 0x00, 0x80, 0xaa, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xaa, 0x2a, 0x00,
    0x00, 0x56, 0xa5, 0x00, 0x00, 0xa6, 0x96, 0x00,
    0x00, 0x26, 0x9a, 0x00, 0x00, 0xa6, 0xa6, 0x00,
    0x00, 0x56, 0x25, 0x00, 0x00, 0xa6, 0x29, 0x00,
    0x00, 0xa6, 0x26, 0x00, 0x00, 0x26, 0xa6, 0x00,
    0x00, 0x26, 0x9a, 0x00, 0x00, 0x2a, 0xa8, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xa8, 0x2a, 0x00,
    0x00, 0x5a, 0x25, 0x00, 0x00, 0xa6, 0x2a, 0x00,
    0x00, 0xa6, 0x02, 0x00, 0x00, 0x5a, 0x2a, 0x00,
    0x00, 0x68, 0xa5, 0x00, 0x00, 0xa0, 0x96, 0x00,
    0x00, 0x2a, 0x9a, 0x00, 0x00, 0xa6, 0x96, 0x00,
    0x00, 0x56, 0xa5, 0x00, 0x00, 0xaa, 0x2a, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x80, 0xaa, 0xaa, 0x00,
    0x80, 0x55, 0x95, 0x00, 0x80, 0x6a, 0xaa, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0x60, 0x02, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0x60, 0x02, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0x60, 0x02, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0xa0, 0x02, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x2a, 0xa8, 0x00,
    0x00, 0x26, 0x98, 0x00, 0x00, 0x26, 0x98, 0x00,
    0x00, 0x26, 0x98, 0x00, 0x00, 0x26, 0x98, 0x00,
    0x00, 0x26, 0x98, 0x00, 0x00, 0x26, 0x98, 0x00,
    0x00, 0x26, 0x9a, 0x00, 0x00, 0xa6, 0x96, 0x00,
    0x00, 0x5a, 0xa5, 0x00, 0x00, 0xa8, 0x2a, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x2a, 0xa8, 0x00,
    0x00, 0x26, 0x98, 0x00, 0x00, 0x26, 0x9a, 0x00,
    0x00, 0xa6, 0xa6, 0x00, 0x00, 0x9a, 0x26, 0x00,
    0x00, 0x98, 0x26, 0x00, 0x00, 0x98, 0x29, 0x00,
    0x00, 0x68, 0x09, 0x00, 0x00, 0x60, 0x09, 0x00,
    0x00, 0x60, 0x0a, 0x00, 0x00, 0xa0, 0x02, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x80, 0x0a, 0xa8, 0x00,
    0x80, 0xa9, 0x9a, 0x00, 0x80, 0x66, 0x9a, 0x00,
    0x00, 0x66, 0x99, 0x00, 0x00, 0x66, 0x99, 0x00,
    0x00, 0x96, 0x99, 0x00, 0x00, 0x96, 0xa5, 0x00,
    0x00, 0x96, 0x26, 0x00, 0x00, 0x96, 0x26, 0x00,
    0x00, 0xa6, 0x26, 0x00, 0x00, 0x2a, 0x2a, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x2a, 0x2a, 0x00,
    0x00, 0xa6, 0x26, 0x00, 0x00, 0x9a, 0x26, 0x00,
    0x00, 0x98, 0x29, 0x00, 0x00, 0x68, 0x09, 0x00,
    0x00, 0x68, 0x09, 0x00, 0x00, 0x58, 0x29, 0x00,
    0x00, 0x9a, 0x25, 0x00, 0x00, 0x96, 0xa6, 0x00,
    0x00, 0xa6, 0x9a, 0x00, 0x00, 0x2a, 0xa8, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x2a, 0xa8, 0x00,
    0x00, 0xa6, 0x9a, 0x00, 0x00, 0x96, 0xa6, 0x00,
    0x00, 0x9a, 0x26, 0x00, 0x00, 0x58, 0x29, 0x00,
    0x00, 0x68, 0x09, 0x00, 0x00, 0x60, 0x0a, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0x60, 0x02, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0xa0, 0x02, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0x00,
    0x00, 0x56, 0x95, 0x00, 0x00, 0xaa, 0xa6, 0x00,
    0x00, 0x80, 0x29, 0x00, 0x00, 0xa0, 0x09, 0x00,
    0x00, 0x68, 0x0a, 0x00, 0x00, 0x58, 0x02, 0x00,
    0x00, 0x9a, 0x02, 0x00, 0x00, 0xa6, 0xaa, 0x00,
    0x00, 0x56, 0x95, 0x00, 0x00, 0xaa, 0xaa, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0xa8, 0x2a, 0x00, 0x00, 0x58, 0x25, 0x00,
    0x00, 0x98, 0x2a, 0x00, 0x00, 0x98, 0x00, 0x00,
    0x00, 0x98, 0x00, 0x00, 0x00, 0x98, 0x00, 0x00,
    0x00, 0x98, 0x00, 0x00, 0x00, 0x98, 0x00, 0x00,
    0x00, 0x98, 0x00, 0x00, 0x00, 0x98, 0x00, 0x00,
    0x00, 0x98, 0x2a, 0x00, 0x00, 0x58, 0x25, 0x00,
    0x00, 0xa8, 0x2a, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x2a, 0x00, 0x00, 0x00, 0xa6, 0x00, 0x00,
    0x00, 0x96, 0x00, 0x00, 0x00, 0x9a, 0x00, 0x00,
    0x00, 0x98, 0x02, 0x00, 0x00, 0x68, 0x02, 0x00,
    0x00, 0x60, 0x0a, 0x00, 0x00, 0xa0, 0x09, 0x00,
    0x00, 0x80, 0x29, 0x00, 0x00, 0x80, 0x26, 0x00,
    0x00, 0x00, 0x26, 0x00, 0x00, 0x00, 0x2a, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0xaa, 0x2a, 0x00, 0x00, 0x56, 0x25, 0x00,
    0x00, 0xaa, 0x26, 0x00, 0x00, 0x00, 0x26, 0x00,
    0x00, 0x00, 0x26, 0x00, 0x00, 0x00, 0x26, 0x00,
    0x00, 0x00, 0x26, 0x00, 0x00, 0x00, 0x26, 0x00,
    0x00, 0x00, 0x26, 0x00, 0x00, 0x00, 0x26, 0x00,
    0x00, 0xaa, 0x26, 0x00, 0x00, 0x56, 0x25, 0x00,
    0x00, 0xaa, 0x2a, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xa0, 0x02, 0x00,
    0x00, 0x60, 0x0a, 0x00, 0x00, 0x68, 0x29, 0x00,
    0x00, 0x9a, 0x25, 0x00, 0x00, 0xa6, 0x26, 0x00,
    0x00, 0x2a, 0x2a, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0xaa, 0xaa, 0x00, 0x00, 0x56, 0x95, 0x00,
    0x00, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0xa8, 0x00, 0x00, 0x00, 0x98, 0x02, 0x00,
    0x00, 0x68, 0x02, 0x00, 0x00, 0x60, 0x02, 0x00,
    0x00, 0xa0, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xa8, 0x2a, 0x00,
    0x00, 0x58, 0x25, 0x00, 0x00, 0xa8, 0xa6, 0x00,
    0x00, 0xa8, 0x96, 0x00, 0x00, 0x5a, 0x95, 0x00,
    0x00, 0xa6, 0x9a, 0x00, 0x00, 0xa6, 0x96, 0x00,
    0x00, 0x56, 0x95, 0x00, 0x00, 0xaa, 0xaa, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x2a, 0x00, 0x00, 0x00, 0x26, 0x00, 0x00,
    0x00, 0x26, 0x00, 0x00, 0x00, 0xa6, 0x2a, 0x00,
    0x00, 0x66, 0x25, 0x00, 0x00, 0x96, 0xa6, 0x00,
    0x00, 0xa6, 0x9a, 0x00, 0x00, 0x26, 0x98, 0x00,
    0x00, 0x26, 0x9a, 0x00, 0x00, 0xa6, 0xa6, 0x00,
    0x00, 0x56, 0x25, 0x00, 0x00, 0xaa, 0x2a, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xa8, 0x2a, 0x00,
    0x00, 0x5a, 0xa5, 0x00, 0x00, 0x96, 0x9a, 0x00,
    0x00, 0xa6, 0xa8, 0x00, 0x00, 0x26, 0x00, 0x00,
    0x00, 0xa6, 0x00, 0x00, 0x00, 0x96, 0x2a, 0x00,
    0x00, 0x5a, 0x25, 0x00, 0x00, 0xa8, 0x2a, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0xa8, 0x00, 0x00, 0x00, 0x98, 0x00,
    0x00, 0x00, 0x98, 0x00, 0x00, 0xa8, 0x9a, 0x00,
    0x00, 0x5a, 0x99, 0x00, 0x00, 0xa6, 0x96, 0x00,
    0x00, 0x26, 0x96, 0x00, 0x00, 0x26, 0x96, 0x00,
    0x00, 0x26, 0x96, 0x00, 0x00, 0xa6, 0x96, 0x00,
    0x00, 0x5a, 0x95, 0x00, 0x00, 0xa8, 0xaa, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xa8, 0x2a, 0x00,
    0x00, 0x5a, 0x25, 0x00, 0x00, 0xa6, 0xa6, 0x00,
    0x00, 0xa6, 0x9a, 0x00, 0x00, 0x56, 0x95, 0x00,
    0x00, 0xa6, 0xaa, 0x00, 0x00, 0xa6, 0x2a, 0x00,
    0x00, 0x5a, 0x25, 0x00, 0x00, 0xa8, 0x2a, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0xa0, 0xaa, 0x00, 0x00, 0x60, 0x95, 0x00,
    0x00, 0x68, 0xaa, 0x00, 0x00, 0x98, 0x02, 0x00,
    0x00, 0x9a, 0x0a, 0x00, 0x00, 0x56, 0x09, 0x00,
    0x00, 0x9a, 0x0a, 0x00, 0x00, 0x98, 0x00, 0x00,
    0x00, 0x98, 0x00, 0x00, 0x00, 0x98, 0x00, 0x00,
    0x00, 0x98, 0x00, 0x00, 0x00, 0xa8, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xa8, 0xaa, 0x00,
    0x00, 0x5a, 0x95, 0x00, 0x00, 0xa6, 0xa6, 0x00,
    0x00, 0xa6, 0x26, 0x00, 0x00, 0xa6, 0x25, 0x00,
    0x00, 0x5a, 0x2a, 0x00, 0x00, 0xa6, 0x2a, 0x00,
    0x00, 0x5a, 0xa5, 0x00, 0x00, 0xa6, 0x9a, 0x00,
    0x00, 0xa6, 0xa6, 0x00, 0x00, 0x5a, 0x29, 0x00,
    0x00, 0xa8, 0x0a, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x2a, 0x00, 0x00, 0x00, 0x26, 0x00, 0x00,
    0x00, 0x26, 0x00, 0x00, 0x00, 0xa6, 0x2a, 0x00,
    0x00, 0x66, 0x25, 0x00, 0x00, 0x96, 0x26, 0x00,
    0x00, 0xa6, 0xa6, 0x00, 0x00, 0x26, 0x96, 0x00,
    0x00, 0x26, 0x96, 0x00, 0x00, 0x26, 0x96, 0x00,
    0x00, 0x26, 0x96, 0x00, 0x00, 0x2a, 0xaa, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0xa0, 0x02, 0x00, 0x00, 0x60, 0x02, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0xa8, 0x02, 0x00,
    0x00, 0x58, 0x02, 0x00, 0x00, 0x68, 0x02, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0x60, 0x02, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0x68, 0x2a, 0x00,
    0x00, 0x58, 0x25, 0x00, 0x00, 0xa8, 0x2a, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x80, 0x2a, 0x00, 0x00, 0x80, 0x25, 0x00,
    0x00, 0x80, 0x25, 0x00, 0x00, 0xa8, 0x2a, 0x00,
    0x00, 0x58, 0x25, 0x00, 0x00, 0xa8, 0x25, 0x00,
    0x00, 0x80, 0x25, 0x00, 0x00, 0x80, 0x25, 0x00,
    0x00, 0x80, 0x25, 0x00, 0x00, 0x80, 0x25, 0x00,
    0x00, 0x80, 0x25, 0x00, 0x00, 0xaa, 0x29, 0x00,
    0x00, 0xa6, 0x09, 0x00, 0x00, 0x5a, 0x0a, 0x00,
    0x00, 0xa8, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x2a, 0x00, 0x00, 0x00, 0x26, 0x00, 0x00,
    0x00, 0x26, 0x00, 0x00, 0x00, 0x26, 0x2a, 0x00,
    0x00, 0xa6, 0x26, 0x00, 0x00, 0xa6, 0x29, 0x00,
    0x00, 0x66, 0x0a, 0x00, 0x00, 0x56, 0x0a, 0x00,
    0x00, 0xa6, 0x29, 0x00, 0x00, 0xa6, 0xa6, 0x00,
    0x00, 0x26, 0x96, 0x00, 0x00, 0x2a, 0xaa, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0xaa, 0x02, 0x00, 0x00, 0x56, 0x02, 0x00,
    0x00, 0x6a, 0x02, 0x00, 0x00, 0x60, 0x02, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0x60, 0x02, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0x60, 0x02, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0x6a, 0x2a, 0x00,
    0x00, 0x56, 0x25, 0x00, 0x00, 0xaa, 0x2a, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0x00,
    0x00, 0x56, 0x95, 0x00, 0x00, 0x66, 0x99, 0x00,
    0x00, 0x66, 0x9a, 0x00, 0x00, 0x66, 0x9a, 0x00,
    0x00, 0x66, 0x9a, 0x00, 0x00, 0x66, 0x9a, 0x00,
    0x00, 0x66, 0x9a, 0x00, 0x00, 0xaa, 0xaa, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xaa, 0x2a, 0x00,
    0x00, 0x66, 0x25, 0x00, 0x00, 0x96, 0x26, 0x00,
    0x00, 0xa6, 0x26, 0x00, 0x00, 0x26, 0x26, 0x00,
    0x00, 0x26, 0x26, 0x00, 0x00, 0x26, 0x26, 0x00,
    0x00, 0x26, 0x26, 0x00, 0x00, 0x2a, 0x2a, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xa8, 0x0a, 0x00,
    0x00, 0x5a, 0x29, 0x00, 0x00, 0xa6, 0xa6, 0x00,
    0x00, 0x26, 0x9a, 0x00, 0x00, 0x26, 0x98, 0x00,
    0x00, 0x26, 0x9a, 0x00, 0x00, 0xa6, 0xa6, 0x00,
    0x00, 0x5a, 0x25, 0x00, 0x00, 0xa8, 0x2a, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xaa, 0x2a, 0x00,
    0x00, 0x66, 0xa5, 0x00, 0x00, 0x96, 0x96, 0x00,
    0x00, 0xa6, 0x9a, 0x00, 0x00, 0x26, 0x98, 0x00,
    0x00, 0x26, 0x9a, 0x00, 0x00, 0xa6, 0x96, 0x00,
    0x00, 0x56, 0xa5, 0x00, 0x00, 0xa6, 0x2a, 0x00,
    0x00, 0x26, 0x00, 0x00, 0x00, 0x2a, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xa8, 0xaa, 0x00,
    0x00, 0x5a, 0x99, 0x00, 0x00, 0xa6, 0x96, 0x00,
    0x00, 0x26, 0x96, 0x00, 0x00, 0x26, 0x96, 0x00,
    0x00, 0x26, 0x96, 0x00, 0x00, 0xa6, 0x96, 0x00,
    0x00, 0x5a, 0x99, 0x00, 0x00, 0xa8, 0x9a, 0x00,
    0x00, 0x00, 0x98, 0x00, 0x00, 0x00, 0xa8, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0x00,
    0x00, 0x56, 0x95, 0x00, 0x00, 0x56, 0x9a, 0x00,
    0x00, 0x96, 0xaa, 0x00, 0x00, 0x96, 0x00, 0x00,
    0x00, 0x96, 0x00, 0x00, 0x00, 0x96, 0x00, 0x00,
    0x00, 0x96, 0x00, 0x00, 0x00, 0xaa, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xa8, 0x2a, 0x00,
    0x00, 0x5a, 0x25, 0x00, 0x00, 0xa6, 0x26, 0x00,
    0x00, 0x96, 0x2a, 0x00, 0x00, 0x6a, 0x25, 0x00,
    0x00, 0xaa, 0xa6, 0x00, 0x00, 0xa6, 0x96, 0x00,
    0x00, 0x56, 0xa5, 0x00, 0x00, 0xaa, 0x2a, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xa0, 0x02, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0x6a, 0x2a, 0x00,
    0x00, 0x56, 0x25, 0x00, 0x00, 0x6a, 0x2a, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0x60, 0x02, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0x60, 0x2a, 0x00,
    0x00, 0x60, 0x25, 0x00, 0x00, 0xa0, 0x2a, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x2a, 0xaa, 0x00,
    0x00, 0x26, 0x96, 0x00, 0x00, 0x26, 0x96, 0x00,
    0x00, 0x26, 0x96, 0x00, 0x00, 0x26, 0x96, 0x00,
    0x00, 0x26, 0x96, 0x00, 0x00, 0xa6, 0x96, 0x00,
    0x00, 0x5a, 0x95, 0x00, 0x00, 0xa8, 0xaa, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x2a, 0xa8, 0x00,
    0x00, 0x26, 0x9a, 0x00, 0x00, 0xa6, 0xa6, 0x00,
    0x00, 0x9a, 0x26, 0x00, 0x00, 0x98, 0x26, 0x00,
    0x00, 0x98, 0x29, 0x00, 0x00, 0x68, 0x09, 0x00,
    0x00, 0x60, 0x0a, 0x00, 0x00, 0xa0, 0x02, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x80, 0x0a, 0xa8, 0x00,
    0x80, 0xa9, 0x9a, 0x00, 0x80, 0x69, 0x9a, 0x00,
    0x80, 0x66, 0x99, 0x00, 0x00, 0x66, 0x99, 0x00,
    0x00, 0x96, 0x99, 0x00, 0x00, 0x96, 0xa6, 0x00,
    0x00, 0x9a, 0x26, 0x00, 0x00, 0xa8, 0x2a, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x2a, 0x2a, 0x00,
    0x00, 0xa6, 0x26, 0x00, 0x00, 0x9a, 0x26, 0x00,
    0x00, 0x58, 0x29, 0x00, 0x00, 0x68, 0x0a, 0x00,
    0x00, 0x58, 0x29, 0x00, 0x00, 0x9a, 0x26, 0x00,
    0x00, 0xa6, 0x26, 0x00, 0x00, 0x2a, 0x2a, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x2a, 0xa8, 0x00,
    0x00, 0x26, 0x9a, 0x00, 0x00, 0xa6, 0xa6, 0x00,
    0x00, 0x9a, 0x26, 0x00, 0x00, 0x98, 0x26, 0x00,
    0x00, 0x58, 0x29, 0x00, 0x00, 0x68, 0x09, 0x00,
    0x00, 0x60, 0x09, 0x00, 0x00, 0x60, 0x0a, 0x00,
    0x00, 0x6a, 0x02, 0x00, 0x00, 0x96, 0x02, 0x00,
    0x00, 0xaa, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xaa, 0x2a, 0x00,
    0x00, 0x56, 0x25, 0x00, 0x00, 0xaa, 0x26, 0x00,
    0x00, 0xa0, 0x29, 0x00, 0x00, 0x68, 0x0a, 0x00,
    0x00, 0x9a, 0x02, 0x00, 0x00, 0x96, 0xaa, 0x00,
    0x00, 0x56, 0x95, 0x00, 0x00, 0xaa, 0xaa, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x80, 0x2a, 0x00, 0x00, 0xa0, 0x25, 0x00,
    0x00, 0x68, 0x2a, 0x00, 0x00, 0x58, 0x02, 0x00,
    0x00, 0x58, 0x02, 0x00, 0x00, 0x98, 0x02, 0x00,
    0x00, 0x9a, 0x00, 0x00, 0x00, 0x96, 0x00, 0x00,
    0x00, 0x9a, 0x00, 0x00, 0x00, 0x98, 0x00, 0x00,
    0x00, 0x98, 0x02, 0x00, 0x00, 0x58, 0x2a, 0x00,
    0x00, 0x68, 0x25, 0x00, 0x00, 0xa0, 0x2a, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0xa0, 0x02, 0x00, 0x00, 0x60, 0x02, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0x60, 0x02, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0x60, 0x02, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0x60, 0x02, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0x60, 0x02, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0x60, 0x02, 0x00,
    0x00, 0x60, 0x02, 0x00, 0x00, 0xa0, 0x02, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0xaa, 0x00, 0x00, 0x00, 0x96, 0x0a, 0x00,
    0x00, 0x6a, 0x09, 0x00, 0x00, 0xa0, 0x09, 0x00,
    0x00, 0x80, 0x09, 0x00, 0x00, 0x80, 0x29, 0x00,
    0x00, 0x80, 0xa5, 0x00, 0x00, 0x80, 0x96, 0x00,
    0x00, 0x80, 0xa9, 0x00, 0x00, 0x80, 0x09, 0x00,
    0x00, 0x80, 0x09, 0x00, 0x00, 0xa8, 0x09, 0x00,
    0x00, 0x58, 0x09, 0x00, 0x00, 0xa8, 0x0a, 0x00,
    0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
    0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
    0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
    0xaa, 0x56, 0x9a, 0xaa, 0xaa, 0xa6, 0x95, 0xaa,
    0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
    0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
    0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
    0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
};

} // TextWriterFont
} // GpuUtil
