/*
 ***********************************************************************************************************************
 *
 *  Copyright (c) 2019 Advanced Micro Devices, Inc. All Rights Reserved.
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in all
 *  copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 *  SOFTWARE.
 *
 **********************************************************************************************************************/

#pragma once

//
// Make sure the necessary endian defines are there.
//
#ifndef LITTLEENDIAN_CPU
#error "LITTLEENDIAN_CPU must be defined"
#endif

union alloc_DEBUG_DATA {
    struct {
        unsigned int GDS_DEBUG_REG5_write_dis                                     :  1;
        unsigned int GDS_DEBUG_REG5_dec_error                                     :  1;
        unsigned int GDS_DEBUG_REG5_alloc_opco_error                              :  1;
        unsigned int GDS_DEBUG_REG5_dealloc_opco_error                            :  1;
        unsigned int GDS_DEBUG_REG5_wrap_opco_error                               :  1;
        unsigned int Reserved0                                                    :  3;
        unsigned int GDS_DEBUG_REG5_error_ds_address                              : 14;
        unsigned int Reserved1                                                    : 10;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_ATS_CNTL {
    struct {
        unsigned int DISABLE_ATC                                                  :  1;
        unsigned int DISABLE_PRI                                                  :  1;
        unsigned int DISABLE_PASID                                                :  1;
        unsigned int                                                              :  5;
        unsigned int CREDITS_ATS_RPB                                              :  6;
        unsigned int                                                              :  6;
        unsigned int INVALIDATION_LOG_KEEP_ORDER                                  :  1;
        unsigned int TRANS_LOG_KEEP_ORDER                                         :  1;
        unsigned int TRANS_EXE_RETURN                                             :  2;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_ATS_DEFAULT_PAGE_LOW {
    struct {
        unsigned int DEFAULT_PAGE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_ATS_FAULT_CNTL {
    struct {
        unsigned int FAULT_REGISTER_LOG                                           :  9;
        unsigned int                                                              :  1;
        unsigned int FAULT_INTERRUPT_TABLE                                        :  9;
        unsigned int                                                              :  1;
        unsigned int FAULT_CRASH_TABLE                                            :  9;
        unsigned int                                                              :  3;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_ATS_FAULT_STATUS_ADDR {
    struct {
        unsigned int PAGE_ADDR                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_ATS_FAULT_STATUS_INFO {
    struct {
        unsigned int FAULT_TYPE                                                   :  9;
        unsigned int                                                              :  1;
        unsigned int VMID                                                         :  5;
        unsigned int EXTRA_INFO                                                   :  1;
        unsigned int EXTRA_INFO2                                                  :  1;
        unsigned int INVALIDATION                                                 :  1;
        unsigned int PAGE_REQUEST                                                 :  1;
        unsigned int STATUS                                                       :  5;
        unsigned int PAGE_ADDR_HIGH                                               :  4;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_ATS_FAULT_STATUS_INFO2 {
    struct {
        unsigned int VF                                                           :  1;
        unsigned int                                                              :  8;
        unsigned int MMHUB_INV_VMID                                               :  5;
        unsigned int                                                              : 18;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  1;
        unsigned int VFID                                                         :  4;
        unsigned int                                                              : 27;
    } gfx09;
    struct {
        unsigned int                                                              :  1;
        unsigned int VFID                                                         :  5;
        unsigned int                                                              : 26;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_ATS_GFX_ATCL2_STATUS {
    struct {
        unsigned int POWERED_DOWN                                                 :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_ATS_MMHUB_ATCL2_STATUS {
    struct {
        unsigned int POWERED_DOWN                                                 :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_ATS_SDPPORT_CNTL {
    struct {
        unsigned int ATS_INV_SELF_ACTIVATE                                        :  1;
        unsigned int ATS_INV_CFG_MODE                                             :  2;
        unsigned int ATS_INV_HALT_THRESHOLD                                       :  4;
        unsigned int UTCL2_TRANS_SELF_ACTIVATE                                    :  1;
        unsigned int UTCL2_TRANS_QUICK_COMACK                                     :  1;
        unsigned int UTCL2_TRANS_HALT_THRESHOLD                                   :  4;
        unsigned int UTCL2_TRANS_PASSIVE_MODE                                     :  1;
        unsigned int UTCL2_GFX_RDY_MODE                                           :  1;
        unsigned int UTCL2_MMHUB_RDY_MODE                                         :  1;
        unsigned int UTCL2_GFX_SDPVDCI_RDRSPCKEN                                  :  1;
        unsigned int UTCL2_GFX_SDPVDCI_RDRSPCKENRCV                               :  1;
        unsigned int UTCL2_GFX_SDPVDCI_RDRSPDATACKEN                              :  1;
        unsigned int UTCL2_GFX_SDPVDCI_RDRSPDATACKENRCV                           :  1;
        unsigned int UTCL2_GFX_SDPVDCI_WRRSPCKEN                                  :  1;
        unsigned int UTCL2_GFX_SDPVDCI_WRRSPCKENRCV                               :  1;
        unsigned int UTCL2_GFX_SDPVDCI_REQCKEN                                    :  1;
        unsigned int UTCL2_GFX_SDPVDCI_REQCKENRCV                                 :  1;
        unsigned int UTCL2_GFX_SDPVDCI_ORIGDATACKEN                               :  1;
        unsigned int UTCL2_GFX_SDPVDCI_ORIGDATACKENRCV                            :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_ATS_STATUS {
    struct {
        unsigned int BUSY                                                         :  1;
        unsigned int CRASHED                                                      :  1;
        unsigned int DEADLOCK_DETECTION                                           :  1;
        unsigned int FLUSH_INVALIDATION_OUTSTANDING                               :  3;
        unsigned int NONFLUSH_INVALIDATION_OUTSTANDING                            :  3;
        unsigned int                                                              : 23;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_ATS_VMID_SNAPSHOT_GFX_STAT {
    struct {
        unsigned int VMID0                                                        :  1;
        unsigned int VMID1                                                        :  1;
        unsigned int VMID2                                                        :  1;
        unsigned int VMID3                                                        :  1;
        unsigned int VMID4                                                        :  1;
        unsigned int VMID5                                                        :  1;
        unsigned int VMID6                                                        :  1;
        unsigned int VMID7                                                        :  1;
        unsigned int VMID8                                                        :  1;
        unsigned int VMID9                                                        :  1;
        unsigned int VMID10                                                       :  1;
        unsigned int VMID11                                                       :  1;
        unsigned int VMID12                                                       :  1;
        unsigned int VMID13                                                       :  1;
        unsigned int VMID14                                                       :  1;
        unsigned int VMID15                                                       :  1;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_ATS_VMID_SNAPSHOT_MMHUB_STAT {
    struct {
        unsigned int VMID0                                                        :  1;
        unsigned int VMID1                                                        :  1;
        unsigned int VMID2                                                        :  1;
        unsigned int VMID3                                                        :  1;
        unsigned int VMID4                                                        :  1;
        unsigned int VMID5                                                        :  1;
        unsigned int VMID6                                                        :  1;
        unsigned int VMID7                                                        :  1;
        unsigned int VMID8                                                        :  1;
        unsigned int VMID9                                                        :  1;
        unsigned int VMID10                                                       :  1;
        unsigned int VMID11                                                       :  1;
        unsigned int VMID12                                                       :  1;
        unsigned int VMID13                                                       :  1;
        unsigned int VMID14                                                       :  1;
        unsigned int VMID15                                                       :  1;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_ATS_VMID_STATUS {
    struct {
        unsigned int VMID0_OUTSTANDING                                            :  1;
        unsigned int VMID1_OUTSTANDING                                            :  1;
        unsigned int VMID2_OUTSTANDING                                            :  1;
        unsigned int VMID3_OUTSTANDING                                            :  1;
        unsigned int VMID4_OUTSTANDING                                            :  1;
        unsigned int VMID5_OUTSTANDING                                            :  1;
        unsigned int VMID6_OUTSTANDING                                            :  1;
        unsigned int VMID7_OUTSTANDING                                            :  1;
        unsigned int VMID8_OUTSTANDING                                            :  1;
        unsigned int VMID9_OUTSTANDING                                            :  1;
        unsigned int VMID10_OUTSTANDING                                           :  1;
        unsigned int VMID11_OUTSTANDING                                           :  1;
        unsigned int VMID12_OUTSTANDING                                           :  1;
        unsigned int VMID13_OUTSTANDING                                           :  1;
        unsigned int VMID14_OUTSTANDING                                           :  1;
        unsigned int VMID15_OUTSTANDING                                           :  1;
        unsigned int VMID16_OUTSTANDING                                           :  1;
        unsigned int VMID17_OUTSTANDING                                           :  1;
        unsigned int VMID18_OUTSTANDING                                           :  1;
        unsigned int VMID19_OUTSTANDING                                           :  1;
        unsigned int VMID20_OUTSTANDING                                           :  1;
        unsigned int VMID21_OUTSTANDING                                           :  1;
        unsigned int VMID22_OUTSTANDING                                           :  1;
        unsigned int VMID23_OUTSTANDING                                           :  1;
        unsigned int VMID24_OUTSTANDING                                           :  1;
        unsigned int VMID25_OUTSTANDING                                           :  1;
        unsigned int VMID26_OUTSTANDING                                           :  1;
        unsigned int VMID27_OUTSTANDING                                           :  1;
        unsigned int VMID28_OUTSTANDING                                           :  1;
        unsigned int VMID29_OUTSTANDING                                           :  1;
        unsigned int VMID30_OUTSTANDING                                           :  1;
        unsigned int VMID31_OUTSTANDING                                           :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_L2_PERFCOUNTER0_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_L2_PERFCOUNTER1_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_L2_PERFCOUNTER_HI {
    struct {
        unsigned int COUNTER_HI                                                   : 16;
        unsigned int COMPARE_VALUE                                                : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_L2_PERFCOUNTER_LO {
    struct {
        unsigned int COUNTER_LO                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_L2_PERFCOUNTER_RSLT_CNTL {
    struct {
        unsigned int PERF_COUNTER_SELECT                                          :  4;
        unsigned int                                                              :  4;
        unsigned int START_TRIGGER                                                :  8;
        unsigned int STOP_TRIGGER                                                 :  8;
        unsigned int ENABLE_ANY                                                   :  1;
        unsigned int CLEAR_ALL                                                    :  1;
        unsigned int STOP_ALL_ON_SATURATE                                         :  1;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_PERFCOUNTER0_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_PERFCOUNTER1_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_PERFCOUNTER2_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_PERFCOUNTER3_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_PERFCOUNTER_HI {
    struct {
        unsigned int COUNTER_HI                                                   : 16;
        unsigned int COMPARE_VALUE                                                : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_PERFCOUNTER_LO {
    struct {
        unsigned int COUNTER_LO                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_PERFCOUNTER_RSLT_CNTL {
    struct {
        unsigned int PERF_COUNTER_SELECT                                          :  4;
        unsigned int                                                              :  4;
        unsigned int START_TRIGGER                                                :  8;
        unsigned int STOP_TRIGGER                                                 :  8;
        unsigned int ENABLE_ANY                                                   :  1;
        unsigned int CLEAR_ALL                                                    :  1;
        unsigned int STOP_ALL_ON_SATURATE                                         :  1;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_TRANS_FAULT_RSPCNTRL {
    struct {
        unsigned int VMID0                                                        :  1;
        unsigned int VMID1                                                        :  1;
        unsigned int VMID2                                                        :  1;
        unsigned int VMID3                                                        :  1;
        unsigned int VMID4                                                        :  1;
        unsigned int VMID5                                                        :  1;
        unsigned int VMID6                                                        :  1;
        unsigned int VMID7                                                        :  1;
        unsigned int VMID8                                                        :  1;
        unsigned int VMID9                                                        :  1;
        unsigned int VMID10                                                       :  1;
        unsigned int VMID11                                                       :  1;
        unsigned int VMID12                                                       :  1;
        unsigned int VMID13                                                       :  1;
        unsigned int VMID14                                                       :  1;
        unsigned int VMID15                                                       :  1;
        unsigned int VMID16                                                       :  1;
        unsigned int VMID17                                                       :  1;
        unsigned int VMID18                                                       :  1;
        unsigned int VMID19                                                       :  1;
        unsigned int VMID20                                                       :  1;
        unsigned int VMID21                                                       :  1;
        unsigned int VMID22                                                       :  1;
        unsigned int VMID23                                                       :  1;
        unsigned int VMID24                                                       :  1;
        unsigned int VMID25                                                       :  1;
        unsigned int VMID26                                                       :  1;
        unsigned int VMID27                                                       :  1;
        unsigned int VMID28                                                       :  1;
        unsigned int VMID29                                                       :  1;
        unsigned int VMID30                                                       :  1;
        unsigned int VMID31                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_VMID0_PASID_MAPPING {
    struct {
        unsigned int PASID                                                        : 16;
        unsigned int                                                              : 14;
        unsigned int NO_INVALIDATION                                              :  1;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_VMID1_PASID_MAPPING {
    struct {
        unsigned int PASID                                                        : 16;
        unsigned int                                                              : 14;
        unsigned int NO_INVALIDATION                                              :  1;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_VMID2_PASID_MAPPING {
    struct {
        unsigned int PASID                                                        : 16;
        unsigned int                                                              : 14;
        unsigned int NO_INVALIDATION                                              :  1;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_VMID3_PASID_MAPPING {
    struct {
        unsigned int PASID                                                        : 16;
        unsigned int                                                              : 14;
        unsigned int NO_INVALIDATION                                              :  1;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_VMID4_PASID_MAPPING {
    struct {
        unsigned int PASID                                                        : 16;
        unsigned int                                                              : 14;
        unsigned int NO_INVALIDATION                                              :  1;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_VMID5_PASID_MAPPING {
    struct {
        unsigned int PASID                                                        : 16;
        unsigned int                                                              : 14;
        unsigned int NO_INVALIDATION                                              :  1;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_VMID6_PASID_MAPPING {
    struct {
        unsigned int PASID                                                        : 16;
        unsigned int                                                              : 14;
        unsigned int NO_INVALIDATION                                              :  1;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_VMID7_PASID_MAPPING {
    struct {
        unsigned int PASID                                                        : 16;
        unsigned int                                                              : 14;
        unsigned int NO_INVALIDATION                                              :  1;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_VMID8_PASID_MAPPING {
    struct {
        unsigned int PASID                                                        : 16;
        unsigned int                                                              : 14;
        unsigned int NO_INVALIDATION                                              :  1;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_VMID9_PASID_MAPPING {
    struct {
        unsigned int PASID                                                        : 16;
        unsigned int                                                              : 14;
        unsigned int NO_INVALIDATION                                              :  1;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_VMID10_PASID_MAPPING {
    struct {
        unsigned int PASID                                                        : 16;
        unsigned int                                                              : 14;
        unsigned int NO_INVALIDATION                                              :  1;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_VMID11_PASID_MAPPING {
    struct {
        unsigned int PASID                                                        : 16;
        unsigned int                                                              : 14;
        unsigned int NO_INVALIDATION                                              :  1;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_VMID12_PASID_MAPPING {
    struct {
        unsigned int PASID                                                        : 16;
        unsigned int                                                              : 14;
        unsigned int NO_INVALIDATION                                              :  1;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_VMID13_PASID_MAPPING {
    struct {
        unsigned int PASID                                                        : 16;
        unsigned int                                                              : 14;
        unsigned int NO_INVALIDATION                                              :  1;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_VMID14_PASID_MAPPING {
    struct {
        unsigned int PASID                                                        : 16;
        unsigned int                                                              : 14;
        unsigned int NO_INVALIDATION                                              :  1;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_VMID15_PASID_MAPPING {
    struct {
        unsigned int PASID                                                        : 16;
        unsigned int                                                              : 14;
        unsigned int NO_INVALIDATION                                              :  1;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_VMID16_PASID_MAPPING {
    struct {
        unsigned int PASID                                                        : 16;
        unsigned int                                                              : 14;
        unsigned int NO_INVALIDATION                                              :  1;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_VMID17_PASID_MAPPING {
    struct {
        unsigned int PASID                                                        : 16;
        unsigned int                                                              : 14;
        unsigned int NO_INVALIDATION                                              :  1;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_VMID18_PASID_MAPPING {
    struct {
        unsigned int PASID                                                        : 16;
        unsigned int                                                              : 14;
        unsigned int NO_INVALIDATION                                              :  1;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_VMID19_PASID_MAPPING {
    struct {
        unsigned int PASID                                                        : 16;
        unsigned int                                                              : 14;
        unsigned int NO_INVALIDATION                                              :  1;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_VMID20_PASID_MAPPING {
    struct {
        unsigned int PASID                                                        : 16;
        unsigned int                                                              : 14;
        unsigned int NO_INVALIDATION                                              :  1;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_VMID21_PASID_MAPPING {
    struct {
        unsigned int PASID                                                        : 16;
        unsigned int                                                              : 14;
        unsigned int NO_INVALIDATION                                              :  1;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_VMID22_PASID_MAPPING {
    struct {
        unsigned int PASID                                                        : 16;
        unsigned int                                                              : 14;
        unsigned int NO_INVALIDATION                                              :  1;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_VMID23_PASID_MAPPING {
    struct {
        unsigned int PASID                                                        : 16;
        unsigned int                                                              : 14;
        unsigned int NO_INVALIDATION                                              :  1;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_VMID24_PASID_MAPPING {
    struct {
        unsigned int PASID                                                        : 16;
        unsigned int                                                              : 14;
        unsigned int NO_INVALIDATION                                              :  1;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_VMID25_PASID_MAPPING {
    struct {
        unsigned int PASID                                                        : 16;
        unsigned int                                                              : 14;
        unsigned int NO_INVALIDATION                                              :  1;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_VMID26_PASID_MAPPING {
    struct {
        unsigned int PASID                                                        : 16;
        unsigned int                                                              : 14;
        unsigned int NO_INVALIDATION                                              :  1;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_VMID27_PASID_MAPPING {
    struct {
        unsigned int PASID                                                        : 16;
        unsigned int                                                              : 14;
        unsigned int NO_INVALIDATION                                              :  1;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_VMID28_PASID_MAPPING {
    struct {
        unsigned int PASID                                                        : 16;
        unsigned int                                                              : 14;
        unsigned int NO_INVALIDATION                                              :  1;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_VMID29_PASID_MAPPING {
    struct {
        unsigned int PASID                                                        : 16;
        unsigned int                                                              : 14;
        unsigned int NO_INVALIDATION                                              :  1;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_VMID30_PASID_MAPPING {
    struct {
        unsigned int PASID                                                        : 16;
        unsigned int                                                              : 14;
        unsigned int NO_INVALIDATION                                              :  1;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_VMID31_PASID_MAPPING {
    struct {
        unsigned int PASID                                                        : 16;
        unsigned int                                                              : 14;
        unsigned int NO_INVALIDATION                                              :  1;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATC_VMID_PASID_MAPPING_UPDATE_STATUS {
    struct {
        unsigned int VMID0_REMAPPING_FINISHED                                     :  1;
        unsigned int VMID1_REMAPPING_FINISHED                                     :  1;
        unsigned int VMID2_REMAPPING_FINISHED                                     :  1;
        unsigned int VMID3_REMAPPING_FINISHED                                     :  1;
        unsigned int VMID4_REMAPPING_FINISHED                                     :  1;
        unsigned int VMID5_REMAPPING_FINISHED                                     :  1;
        unsigned int VMID6_REMAPPING_FINISHED                                     :  1;
        unsigned int VMID7_REMAPPING_FINISHED                                     :  1;
        unsigned int VMID8_REMAPPING_FINISHED                                     :  1;
        unsigned int VMID9_REMAPPING_FINISHED                                     :  1;
        unsigned int VMID10_REMAPPING_FINISHED                                    :  1;
        unsigned int VMID11_REMAPPING_FINISHED                                    :  1;
        unsigned int VMID12_REMAPPING_FINISHED                                    :  1;
        unsigned int VMID13_REMAPPING_FINISHED                                    :  1;
        unsigned int VMID14_REMAPPING_FINISHED                                    :  1;
        unsigned int VMID15_REMAPPING_FINISHED                                    :  1;
        unsigned int VMID16_REMAPPING_FINISHED                                    :  1;
        unsigned int VMID17_REMAPPING_FINISHED                                    :  1;
        unsigned int VMID18_REMAPPING_FINISHED                                    :  1;
        unsigned int VMID19_REMAPPING_FINISHED                                    :  1;
        unsigned int VMID20_REMAPPING_FINISHED                                    :  1;
        unsigned int VMID21_REMAPPING_FINISHED                                    :  1;
        unsigned int VMID22_REMAPPING_FINISHED                                    :  1;
        unsigned int VMID23_REMAPPING_FINISHED                                    :  1;
        unsigned int VMID24_REMAPPING_FINISHED                                    :  1;
        unsigned int VMID25_REMAPPING_FINISHED                                    :  1;
        unsigned int VMID26_REMAPPING_FINISHED                                    :  1;
        unsigned int VMID27_REMAPPING_FINISHED                                    :  1;
        unsigned int VMID28_REMAPPING_FINISHED                                    :  1;
        unsigned int VMID29_REMAPPING_FINISHED                                    :  1;
        unsigned int VMID30_REMAPPING_FINISHED                                    :  1;
        unsigned int VMID31_REMAPPING_FINISHED                                    :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ATS_IH_CREDIT {
    struct {
        unsigned int CREDIT_VALUE                                                 :  2;
        unsigned int                                                              : 14;
        unsigned int IH_CLIENT_ID                                                 :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union BCI_DEBUG_READ {
    struct {
        unsigned int DATA                                                         : 24;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_BLEND0_CONTROL {
    struct {
        unsigned int COLOR_SRCBLEND                                               :  5;
        unsigned int COLOR_COMB_FCN                                               :  3;
        unsigned int COLOR_DESTBLEND                                              :  5;
        unsigned int                                                              :  3;
        unsigned int ALPHA_SRCBLEND                                               :  5;
        unsigned int ALPHA_COMB_FCN                                               :  3;
        unsigned int ALPHA_DESTBLEND                                              :  5;
        unsigned int SEPARATE_ALPHA_BLEND                                         :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int DISABLE_ROP3                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_BLEND1_CONTROL {
    struct {
        unsigned int COLOR_SRCBLEND                                               :  5;
        unsigned int COLOR_COMB_FCN                                               :  3;
        unsigned int COLOR_DESTBLEND                                              :  5;
        unsigned int                                                              :  3;
        unsigned int ALPHA_SRCBLEND                                               :  5;
        unsigned int ALPHA_COMB_FCN                                               :  3;
        unsigned int ALPHA_DESTBLEND                                              :  5;
        unsigned int SEPARATE_ALPHA_BLEND                                         :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int DISABLE_ROP3                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_BLEND2_CONTROL {
    struct {
        unsigned int COLOR_SRCBLEND                                               :  5;
        unsigned int COLOR_COMB_FCN                                               :  3;
        unsigned int COLOR_DESTBLEND                                              :  5;
        unsigned int                                                              :  3;
        unsigned int ALPHA_SRCBLEND                                               :  5;
        unsigned int ALPHA_COMB_FCN                                               :  3;
        unsigned int ALPHA_DESTBLEND                                              :  5;
        unsigned int SEPARATE_ALPHA_BLEND                                         :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int DISABLE_ROP3                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_BLEND3_CONTROL {
    struct {
        unsigned int COLOR_SRCBLEND                                               :  5;
        unsigned int COLOR_COMB_FCN                                               :  3;
        unsigned int COLOR_DESTBLEND                                              :  5;
        unsigned int                                                              :  3;
        unsigned int ALPHA_SRCBLEND                                               :  5;
        unsigned int ALPHA_COMB_FCN                                               :  3;
        unsigned int ALPHA_DESTBLEND                                              :  5;
        unsigned int SEPARATE_ALPHA_BLEND                                         :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int DISABLE_ROP3                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_BLEND4_CONTROL {
    struct {
        unsigned int COLOR_SRCBLEND                                               :  5;
        unsigned int COLOR_COMB_FCN                                               :  3;
        unsigned int COLOR_DESTBLEND                                              :  5;
        unsigned int                                                              :  3;
        unsigned int ALPHA_SRCBLEND                                               :  5;
        unsigned int ALPHA_COMB_FCN                                               :  3;
        unsigned int ALPHA_DESTBLEND                                              :  5;
        unsigned int SEPARATE_ALPHA_BLEND                                         :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int DISABLE_ROP3                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_BLEND5_CONTROL {
    struct {
        unsigned int COLOR_SRCBLEND                                               :  5;
        unsigned int COLOR_COMB_FCN                                               :  3;
        unsigned int COLOR_DESTBLEND                                              :  5;
        unsigned int                                                              :  3;
        unsigned int ALPHA_SRCBLEND                                               :  5;
        unsigned int ALPHA_COMB_FCN                                               :  3;
        unsigned int ALPHA_DESTBLEND                                              :  5;
        unsigned int SEPARATE_ALPHA_BLEND                                         :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int DISABLE_ROP3                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_BLEND6_CONTROL {
    struct {
        unsigned int COLOR_SRCBLEND                                               :  5;
        unsigned int COLOR_COMB_FCN                                               :  3;
        unsigned int COLOR_DESTBLEND                                              :  5;
        unsigned int                                                              :  3;
        unsigned int ALPHA_SRCBLEND                                               :  5;
        unsigned int ALPHA_COMB_FCN                                               :  3;
        unsigned int ALPHA_DESTBLEND                                              :  5;
        unsigned int SEPARATE_ALPHA_BLEND                                         :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int DISABLE_ROP3                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_BLEND7_CONTROL {
    struct {
        unsigned int COLOR_SRCBLEND                                               :  5;
        unsigned int COLOR_COMB_FCN                                               :  3;
        unsigned int COLOR_DESTBLEND                                              :  5;
        unsigned int                                                              :  3;
        unsigned int ALPHA_SRCBLEND                                               :  5;
        unsigned int ALPHA_COMB_FCN                                               :  3;
        unsigned int ALPHA_DESTBLEND                                              :  5;
        unsigned int SEPARATE_ALPHA_BLEND                                         :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int DISABLE_ROP3                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_BLEND_ALPHA {
    struct {
        unsigned int BLEND_ALPHA                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_BLEND_BLUE {
    struct {
        unsigned int BLEND_BLUE                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_BLEND_GREEN {
    struct {
        unsigned int BLEND_GREEN                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_BLEND_RED {
    struct {
        unsigned int BLEND_RED                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_CACHE_EVICT_POINTS {
    struct {
        unsigned int CM_CACHE_EVICT_POINT                                         :  8;
        unsigned int FC_CACHE_EVICT_POINT                                         :  8;
        unsigned int DCC_CACHE_EVICT_POINT                                        :  8;
        unsigned int CC_CACHE_EVICT_POINT                                         :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_CGTT_SCLK_CTRL {
    struct {
        unsigned int ON_DELAY                                                     :  4;
        unsigned int OFF_HYSTERESIS                                               :  8;
        unsigned int                                                              :  4;
        unsigned int SOFT_STALL_OVERRIDE7                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE6                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE5                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE4                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE3                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE2                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE1                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE0                                         :  1;
        unsigned int SOFT_OVERRIDE7                                               :  1;
        unsigned int SOFT_OVERRIDE6                                               :  1;
        unsigned int SOFT_OVERRIDE5                                               :  1;
        unsigned int SOFT_OVERRIDE4                                               :  1;
        unsigned int SOFT_OVERRIDE3                                               :  1;
        unsigned int SOFT_OVERRIDE2                                               :  1;
        unsigned int SOFT_OVERRIDE1                                               :  1;
        unsigned int SOFT_OVERRIDE0                                               :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR0_ATTRIB {
    struct {
        unsigned int                                                              : 12;
        unsigned int NUM_SAMPLES                                                  :  3;
        unsigned int NUM_FRAGMENTS                                                :  2;
        unsigned int FORCE_DST_ALPHA_1                                            :  1;
        unsigned int                                                              : 14;
    } core;
    struct {
        unsigned int MIP0_DEPTH                                                   : 11;
        unsigned int META_LINEAR                                                  :  1;
        unsigned int                                                              :  6;
        unsigned int COLOR_SW_MODE                                                :  5;
        unsigned int FMASK_SW_MODE                                                :  5;
        unsigned int RESOURCE_TYPE                                                :  2;
        unsigned int RB_ALIGNED                                                   :  1;
        unsigned int PIPE_ALIGNED                                                 :  1;
    } gfx09;
    struct {
        unsigned int TILE_MODE_INDEX                                              :  5;
        unsigned int FMASK_TILE_MODE_INDEX                                        :  5;
        unsigned int FMASK_BANK_HEIGHT                                            :  2;
        unsigned int                                                              :  6;
        unsigned int DISABLE_FMASK_NOFETCH_OPT                                    :  1;
        unsigned int LIMIT_COLOR_FETCH_TO_256B_MAX                                :  1;
        unsigned int                                                              : 12;
    } gfx10Core;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR0_ATTRIB2 {
    struct {
        unsigned int MIP0_HEIGHT                                                  : 14;
        unsigned int MIP0_WIDTH                                                   : 14;
        unsigned int MAX_MIP                                                      :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR0_ATTRIB3 {
    struct {
        unsigned int MIP0_DEPTH                                                   : 13;
        unsigned int META_LINEAR                                                  :  1;
        unsigned int COLOR_SW_MODE                                                :  5;
        unsigned int FMASK_SW_MODE                                                :  5;
        unsigned int RESOURCE_TYPE                                                :  2;
        unsigned int                                                              :  6;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 26;
        unsigned int CMASK_PIPE_ALIGNED                                           :  1;
        unsigned int                                                              :  3;
        unsigned int DCC_PIPE_ALIGNED                                             :  1;
        unsigned int                                                              :  1;
    } gfx101Plus;
    struct {
        unsigned int                                                              : 27;
        unsigned int RESOURCE_LEVEL                                               :  3;
        unsigned int                                                              :  2;
    } gfx10Core;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR0_BASE {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR0_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR0_CLEAR_WORD0 {
    struct {
        unsigned int CLEAR_WORD0                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR0_CLEAR_WORD1 {
    struct {
        unsigned int CLEAR_WORD1                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR0_CMASK {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR0_CMASK_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR0_CMASK_SLICE {
    struct {
        unsigned int TILE_MAX                                                     : 14;
        unsigned int                                                              : 18;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR0_DCC_BASE {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR0_DCC_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR0_DCC_CONTROL {
    struct {
        unsigned int OVERWRITE_COMBINER_DISABLE                                   :  1;
        unsigned int KEY_CLEAR_ENABLE                                             :  1;
        unsigned int MAX_UNCOMPRESSED_BLOCK_SIZE                                  :  2;
        unsigned int MIN_COMPRESSED_BLOCK_SIZE                                    :  1;
        unsigned int MAX_COMPRESSED_BLOCK_SIZE                                    :  2;
        unsigned int COLOR_TRANSFORM                                              :  2;
        unsigned int INDEPENDENT_64B_BLOCKS                                       :  1;
        unsigned int LOSSY_RGB_PRECISION                                          :  4;
        unsigned int LOSSY_ALPHA_PRECISION                                        :  4;
        unsigned int                                                              : 14;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 18;
        unsigned int DISABLE_CONSTANT_ENCODE_REG                                  :  1;
        unsigned int ENABLE_CONSTANT_ENCODE_REG_WRITE                             :  1;
        unsigned int                                                              : 12;
    } gfx09_1xPlus;
    struct {
        unsigned int                                                              : 20;
        unsigned int INDEPENDENT_128B_BLOCKS                                      :  1;
        unsigned int                                                              : 11;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR0_FMASK {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR0_FMASK_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR0_FMASK_SLICE {
    struct {
        unsigned int TILE_MAX                                                     : 22;
        unsigned int                                                              : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR0_INFO {
    struct {
        unsigned int ENDIAN                                                       :  2;
        unsigned int FORMAT                                                       :  5;
        unsigned int                                                              :  1;
        unsigned int NUMBER_TYPE                                                  :  3;
        unsigned int COMP_SWAP                                                    :  2;
        unsigned int FAST_CLEAR                                                   :  1;
        unsigned int COMPRESSION                                                  :  1;
        unsigned int BLEND_CLAMP                                                  :  1;
        unsigned int BLEND_BYPASS                                                 :  1;
        unsigned int SIMPLE_FLOAT                                                 :  1;
        unsigned int ROUND_MODE                                                   :  1;
        unsigned int                                                              :  1;
        unsigned int BLEND_OPT_DONT_RD_DST                                        :  3;
        unsigned int BLEND_OPT_DISCARD_PIXEL                                      :  3;
        unsigned int FMASK_COMPRESSION_DISABLE                                    :  1;
        unsigned int FMASK_COMPRESS_1FRAG_ONLY                                    :  1;
        unsigned int DCC_ENABLE                                                   :  1;
        unsigned int CMASK_ADDR_TYPE                                              :  2;
        unsigned int                                                              :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  7;
        unsigned int LINEAR_GENERAL                                               :  1;
        unsigned int                                                              : 24;
    } gfx10;
    struct {
        unsigned int                                                              : 31;
        unsigned int ALT_TILE_MODE                                                :  1;
    } gfx101;
    struct {
        unsigned int                                                              : 19;
        unsigned int CMASK_IS_LINEAR                                              :  1;
        unsigned int                                                              : 12;
    } gfx10Core;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR0_PITCH {
    struct {
        unsigned int TILE_MAX                                                     : 11;
        unsigned int                                                              :  9;
        unsigned int FMASK_TILE_MAX                                               : 11;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR0_SLICE {
    struct {
        unsigned int TILE_MAX                                                     : 22;
        unsigned int                                                              : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR0_VIEW {
    struct {
        unsigned int SLICE_START                                                  : 11;
        unsigned int                                                              :  2;
        unsigned int SLICE_MAX                                                    : 11;
        unsigned int MIP_LEVEL                                                    :  4;
        unsigned int                                                              :  4;
    } gfx09;
    struct {
        unsigned int SLICE_START                                                  : 13;
        unsigned int SLICE_MAX                                                    : 13;
        unsigned int MIP_LEVEL                                                    :  4;
        unsigned int                                                              :  2;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR1_ATTRIB {
    struct {
        unsigned int                                                              : 12;
        unsigned int NUM_SAMPLES                                                  :  3;
        unsigned int NUM_FRAGMENTS                                                :  2;
        unsigned int FORCE_DST_ALPHA_1                                            :  1;
        unsigned int                                                              : 14;
    } core;
    struct {
        unsigned int MIP0_DEPTH                                                   : 11;
        unsigned int META_LINEAR                                                  :  1;
        unsigned int                                                              :  6;
        unsigned int COLOR_SW_MODE                                                :  5;
        unsigned int FMASK_SW_MODE                                                :  5;
        unsigned int RESOURCE_TYPE                                                :  2;
        unsigned int RB_ALIGNED                                                   :  1;
        unsigned int PIPE_ALIGNED                                                 :  1;
    } gfx09;
    struct {
        unsigned int TILE_MODE_INDEX                                              :  5;
        unsigned int FMASK_TILE_MODE_INDEX                                        :  5;
        unsigned int FMASK_BANK_HEIGHT                                            :  2;
        unsigned int                                                              :  6;
        unsigned int DISABLE_FMASK_NOFETCH_OPT                                    :  1;
        unsigned int LIMIT_COLOR_FETCH_TO_256B_MAX                                :  1;
        unsigned int                                                              : 12;
    } gfx10Core;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR1_ATTRIB2 {
    struct {
        unsigned int MIP0_HEIGHT                                                  : 14;
        unsigned int MIP0_WIDTH                                                   : 14;
        unsigned int MAX_MIP                                                      :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR1_ATTRIB3 {
    struct {
        unsigned int MIP0_DEPTH                                                   : 13;
        unsigned int META_LINEAR                                                  :  1;
        unsigned int COLOR_SW_MODE                                                :  5;
        unsigned int FMASK_SW_MODE                                                :  5;
        unsigned int RESOURCE_TYPE                                                :  2;
        unsigned int                                                              :  6;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 26;
        unsigned int CMASK_PIPE_ALIGNED                                           :  1;
        unsigned int                                                              :  3;
        unsigned int DCC_PIPE_ALIGNED                                             :  1;
        unsigned int                                                              :  1;
    } gfx101Plus;
    struct {
        unsigned int                                                              : 27;
        unsigned int RESOURCE_LEVEL                                               :  3;
        unsigned int                                                              :  2;
    } gfx10Core;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR1_BASE {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR1_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR1_CLEAR_WORD0 {
    struct {
        unsigned int CLEAR_WORD0                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR1_CLEAR_WORD1 {
    struct {
        unsigned int CLEAR_WORD1                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR1_CMASK {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR1_CMASK_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR1_CMASK_SLICE {
    struct {
        unsigned int TILE_MAX                                                     : 14;
        unsigned int                                                              : 18;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR1_DCC_BASE {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR1_DCC_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR1_DCC_CONTROL {
    struct {
        unsigned int OVERWRITE_COMBINER_DISABLE                                   :  1;
        unsigned int KEY_CLEAR_ENABLE                                             :  1;
        unsigned int MAX_UNCOMPRESSED_BLOCK_SIZE                                  :  2;
        unsigned int MIN_COMPRESSED_BLOCK_SIZE                                    :  1;
        unsigned int MAX_COMPRESSED_BLOCK_SIZE                                    :  2;
        unsigned int COLOR_TRANSFORM                                              :  2;
        unsigned int INDEPENDENT_64B_BLOCKS                                       :  1;
        unsigned int LOSSY_RGB_PRECISION                                          :  4;
        unsigned int LOSSY_ALPHA_PRECISION                                        :  4;
        unsigned int                                                              : 14;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 18;
        unsigned int DISABLE_CONSTANT_ENCODE_REG                                  :  1;
        unsigned int ENABLE_CONSTANT_ENCODE_REG_WRITE                             :  1;
        unsigned int                                                              : 12;
    } gfx09_1xPlus;
    struct {
        unsigned int                                                              : 20;
        unsigned int INDEPENDENT_128B_BLOCKS                                      :  1;
        unsigned int                                                              : 11;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR1_FMASK {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR1_FMASK_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR1_FMASK_SLICE {
    struct {
        unsigned int TILE_MAX                                                     : 22;
        unsigned int                                                              : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR1_INFO {
    struct {
        unsigned int ENDIAN                                                       :  2;
        unsigned int FORMAT                                                       :  5;
        unsigned int                                                              :  1;
        unsigned int NUMBER_TYPE                                                  :  3;
        unsigned int COMP_SWAP                                                    :  2;
        unsigned int FAST_CLEAR                                                   :  1;
        unsigned int COMPRESSION                                                  :  1;
        unsigned int BLEND_CLAMP                                                  :  1;
        unsigned int BLEND_BYPASS                                                 :  1;
        unsigned int SIMPLE_FLOAT                                                 :  1;
        unsigned int ROUND_MODE                                                   :  1;
        unsigned int                                                              :  1;
        unsigned int BLEND_OPT_DONT_RD_DST                                        :  3;
        unsigned int BLEND_OPT_DISCARD_PIXEL                                      :  3;
        unsigned int FMASK_COMPRESSION_DISABLE                                    :  1;
        unsigned int FMASK_COMPRESS_1FRAG_ONLY                                    :  1;
        unsigned int DCC_ENABLE                                                   :  1;
        unsigned int CMASK_ADDR_TYPE                                              :  2;
        unsigned int                                                              :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  7;
        unsigned int LINEAR_GENERAL                                               :  1;
        unsigned int                                                              : 24;
    } gfx10;
    struct {
        unsigned int                                                              : 31;
        unsigned int ALT_TILE_MODE                                                :  1;
    } gfx101;
    struct {
        unsigned int                                                              : 19;
        unsigned int CMASK_IS_LINEAR                                              :  1;
        unsigned int                                                              : 12;
    } gfx10Core;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR1_PITCH {
    struct {
        unsigned int TILE_MAX                                                     : 11;
        unsigned int                                                              :  9;
        unsigned int FMASK_TILE_MAX                                               : 11;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR1_SLICE {
    struct {
        unsigned int TILE_MAX                                                     : 22;
        unsigned int                                                              : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR1_VIEW {
    struct {
        unsigned int SLICE_START                                                  : 11;
        unsigned int                                                              :  2;
        unsigned int SLICE_MAX                                                    : 11;
        unsigned int MIP_LEVEL                                                    :  4;
        unsigned int                                                              :  4;
    } gfx09;
    struct {
        unsigned int SLICE_START                                                  : 13;
        unsigned int SLICE_MAX                                                    : 13;
        unsigned int MIP_LEVEL                                                    :  4;
        unsigned int                                                              :  2;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR2_ATTRIB {
    struct {
        unsigned int                                                              : 12;
        unsigned int NUM_SAMPLES                                                  :  3;
        unsigned int NUM_FRAGMENTS                                                :  2;
        unsigned int FORCE_DST_ALPHA_1                                            :  1;
        unsigned int                                                              : 14;
    } core;
    struct {
        unsigned int MIP0_DEPTH                                                   : 11;
        unsigned int META_LINEAR                                                  :  1;
        unsigned int                                                              :  6;
        unsigned int COLOR_SW_MODE                                                :  5;
        unsigned int FMASK_SW_MODE                                                :  5;
        unsigned int RESOURCE_TYPE                                                :  2;
        unsigned int RB_ALIGNED                                                   :  1;
        unsigned int PIPE_ALIGNED                                                 :  1;
    } gfx09;
    struct {
        unsigned int TILE_MODE_INDEX                                              :  5;
        unsigned int FMASK_TILE_MODE_INDEX                                        :  5;
        unsigned int FMASK_BANK_HEIGHT                                            :  2;
        unsigned int                                                              :  6;
        unsigned int DISABLE_FMASK_NOFETCH_OPT                                    :  1;
        unsigned int LIMIT_COLOR_FETCH_TO_256B_MAX                                :  1;
        unsigned int                                                              : 12;
    } gfx10Core;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR2_ATTRIB2 {
    struct {
        unsigned int MIP0_HEIGHT                                                  : 14;
        unsigned int MIP0_WIDTH                                                   : 14;
        unsigned int MAX_MIP                                                      :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR2_ATTRIB3 {
    struct {
        unsigned int MIP0_DEPTH                                                   : 13;
        unsigned int META_LINEAR                                                  :  1;
        unsigned int COLOR_SW_MODE                                                :  5;
        unsigned int FMASK_SW_MODE                                                :  5;
        unsigned int RESOURCE_TYPE                                                :  2;
        unsigned int                                                              :  6;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 26;
        unsigned int CMASK_PIPE_ALIGNED                                           :  1;
        unsigned int                                                              :  3;
        unsigned int DCC_PIPE_ALIGNED                                             :  1;
        unsigned int                                                              :  1;
    } gfx101Plus;
    struct {
        unsigned int                                                              : 27;
        unsigned int RESOURCE_LEVEL                                               :  3;
        unsigned int                                                              :  2;
    } gfx10Core;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR2_BASE {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR2_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR2_CLEAR_WORD0 {
    struct {
        unsigned int CLEAR_WORD0                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR2_CLEAR_WORD1 {
    struct {
        unsigned int CLEAR_WORD1                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR2_CMASK {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR2_CMASK_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR2_CMASK_SLICE {
    struct {
        unsigned int TILE_MAX                                                     : 14;
        unsigned int                                                              : 18;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR2_DCC_BASE {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR2_DCC_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR2_DCC_CONTROL {
    struct {
        unsigned int OVERWRITE_COMBINER_DISABLE                                   :  1;
        unsigned int KEY_CLEAR_ENABLE                                             :  1;
        unsigned int MAX_UNCOMPRESSED_BLOCK_SIZE                                  :  2;
        unsigned int MIN_COMPRESSED_BLOCK_SIZE                                    :  1;
        unsigned int MAX_COMPRESSED_BLOCK_SIZE                                    :  2;
        unsigned int COLOR_TRANSFORM                                              :  2;
        unsigned int INDEPENDENT_64B_BLOCKS                                       :  1;
        unsigned int LOSSY_RGB_PRECISION                                          :  4;
        unsigned int LOSSY_ALPHA_PRECISION                                        :  4;
        unsigned int                                                              : 14;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 18;
        unsigned int DISABLE_CONSTANT_ENCODE_REG                                  :  1;
        unsigned int ENABLE_CONSTANT_ENCODE_REG_WRITE                             :  1;
        unsigned int                                                              : 12;
    } gfx09_1xPlus;
    struct {
        unsigned int                                                              : 20;
        unsigned int INDEPENDENT_128B_BLOCKS                                      :  1;
        unsigned int                                                              : 11;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR2_FMASK {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR2_FMASK_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR2_FMASK_SLICE {
    struct {
        unsigned int TILE_MAX                                                     : 22;
        unsigned int                                                              : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR2_INFO {
    struct {
        unsigned int ENDIAN                                                       :  2;
        unsigned int FORMAT                                                       :  5;
        unsigned int                                                              :  1;
        unsigned int NUMBER_TYPE                                                  :  3;
        unsigned int COMP_SWAP                                                    :  2;
        unsigned int FAST_CLEAR                                                   :  1;
        unsigned int COMPRESSION                                                  :  1;
        unsigned int BLEND_CLAMP                                                  :  1;
        unsigned int BLEND_BYPASS                                                 :  1;
        unsigned int SIMPLE_FLOAT                                                 :  1;
        unsigned int ROUND_MODE                                                   :  1;
        unsigned int                                                              :  1;
        unsigned int BLEND_OPT_DONT_RD_DST                                        :  3;
        unsigned int BLEND_OPT_DISCARD_PIXEL                                      :  3;
        unsigned int FMASK_COMPRESSION_DISABLE                                    :  1;
        unsigned int FMASK_COMPRESS_1FRAG_ONLY                                    :  1;
        unsigned int DCC_ENABLE                                                   :  1;
        unsigned int CMASK_ADDR_TYPE                                              :  2;
        unsigned int                                                              :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  7;
        unsigned int LINEAR_GENERAL                                               :  1;
        unsigned int                                                              : 24;
    } gfx10;
    struct {
        unsigned int                                                              : 31;
        unsigned int ALT_TILE_MODE                                                :  1;
    } gfx101;
    struct {
        unsigned int                                                              : 19;
        unsigned int CMASK_IS_LINEAR                                              :  1;
        unsigned int                                                              : 12;
    } gfx10Core;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR2_PITCH {
    struct {
        unsigned int TILE_MAX                                                     : 11;
        unsigned int                                                              :  9;
        unsigned int FMASK_TILE_MAX                                               : 11;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR2_SLICE {
    struct {
        unsigned int TILE_MAX                                                     : 22;
        unsigned int                                                              : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR2_VIEW {
    struct {
        unsigned int SLICE_START                                                  : 11;
        unsigned int                                                              :  2;
        unsigned int SLICE_MAX                                                    : 11;
        unsigned int MIP_LEVEL                                                    :  4;
        unsigned int                                                              :  4;
    } gfx09;
    struct {
        unsigned int SLICE_START                                                  : 13;
        unsigned int SLICE_MAX                                                    : 13;
        unsigned int MIP_LEVEL                                                    :  4;
        unsigned int                                                              :  2;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR3_ATTRIB {
    struct {
        unsigned int                                                              : 12;
        unsigned int NUM_SAMPLES                                                  :  3;
        unsigned int NUM_FRAGMENTS                                                :  2;
        unsigned int FORCE_DST_ALPHA_1                                            :  1;
        unsigned int                                                              : 14;
    } core;
    struct {
        unsigned int MIP0_DEPTH                                                   : 11;
        unsigned int META_LINEAR                                                  :  1;
        unsigned int                                                              :  6;
        unsigned int COLOR_SW_MODE                                                :  5;
        unsigned int FMASK_SW_MODE                                                :  5;
        unsigned int RESOURCE_TYPE                                                :  2;
        unsigned int RB_ALIGNED                                                   :  1;
        unsigned int PIPE_ALIGNED                                                 :  1;
    } gfx09;
    struct {
        unsigned int TILE_MODE_INDEX                                              :  5;
        unsigned int FMASK_TILE_MODE_INDEX                                        :  5;
        unsigned int FMASK_BANK_HEIGHT                                            :  2;
        unsigned int                                                              :  6;
        unsigned int DISABLE_FMASK_NOFETCH_OPT                                    :  1;
        unsigned int LIMIT_COLOR_FETCH_TO_256B_MAX                                :  1;
        unsigned int                                                              : 12;
    } gfx10Core;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR3_ATTRIB2 {
    struct {
        unsigned int MIP0_HEIGHT                                                  : 14;
        unsigned int MIP0_WIDTH                                                   : 14;
        unsigned int MAX_MIP                                                      :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR3_ATTRIB3 {
    struct {
        unsigned int MIP0_DEPTH                                                   : 13;
        unsigned int META_LINEAR                                                  :  1;
        unsigned int COLOR_SW_MODE                                                :  5;
        unsigned int FMASK_SW_MODE                                                :  5;
        unsigned int RESOURCE_TYPE                                                :  2;
        unsigned int                                                              :  6;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 26;
        unsigned int CMASK_PIPE_ALIGNED                                           :  1;
        unsigned int                                                              :  3;
        unsigned int DCC_PIPE_ALIGNED                                             :  1;
        unsigned int                                                              :  1;
    } gfx101Plus;
    struct {
        unsigned int                                                              : 27;
        unsigned int RESOURCE_LEVEL                                               :  3;
        unsigned int                                                              :  2;
    } gfx10Core;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR3_BASE {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR3_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR3_CLEAR_WORD0 {
    struct {
        unsigned int CLEAR_WORD0                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR3_CLEAR_WORD1 {
    struct {
        unsigned int CLEAR_WORD1                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR3_CMASK {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR3_CMASK_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR3_CMASK_SLICE {
    struct {
        unsigned int TILE_MAX                                                     : 14;
        unsigned int                                                              : 18;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR3_DCC_BASE {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR3_DCC_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR3_DCC_CONTROL {
    struct {
        unsigned int OVERWRITE_COMBINER_DISABLE                                   :  1;
        unsigned int KEY_CLEAR_ENABLE                                             :  1;
        unsigned int MAX_UNCOMPRESSED_BLOCK_SIZE                                  :  2;
        unsigned int MIN_COMPRESSED_BLOCK_SIZE                                    :  1;
        unsigned int MAX_COMPRESSED_BLOCK_SIZE                                    :  2;
        unsigned int COLOR_TRANSFORM                                              :  2;
        unsigned int INDEPENDENT_64B_BLOCKS                                       :  1;
        unsigned int LOSSY_RGB_PRECISION                                          :  4;
        unsigned int LOSSY_ALPHA_PRECISION                                        :  4;
        unsigned int                                                              : 14;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 18;
        unsigned int DISABLE_CONSTANT_ENCODE_REG                                  :  1;
        unsigned int ENABLE_CONSTANT_ENCODE_REG_WRITE                             :  1;
        unsigned int                                                              : 12;
    } gfx09_1xPlus;
    struct {
        unsigned int                                                              : 20;
        unsigned int INDEPENDENT_128B_BLOCKS                                      :  1;
        unsigned int                                                              : 11;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR3_FMASK {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR3_FMASK_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR3_FMASK_SLICE {
    struct {
        unsigned int TILE_MAX                                                     : 22;
        unsigned int                                                              : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR3_INFO {
    struct {
        unsigned int ENDIAN                                                       :  2;
        unsigned int FORMAT                                                       :  5;
        unsigned int                                                              :  1;
        unsigned int NUMBER_TYPE                                                  :  3;
        unsigned int COMP_SWAP                                                    :  2;
        unsigned int FAST_CLEAR                                                   :  1;
        unsigned int COMPRESSION                                                  :  1;
        unsigned int BLEND_CLAMP                                                  :  1;
        unsigned int BLEND_BYPASS                                                 :  1;
        unsigned int SIMPLE_FLOAT                                                 :  1;
        unsigned int ROUND_MODE                                                   :  1;
        unsigned int                                                              :  1;
        unsigned int BLEND_OPT_DONT_RD_DST                                        :  3;
        unsigned int BLEND_OPT_DISCARD_PIXEL                                      :  3;
        unsigned int FMASK_COMPRESSION_DISABLE                                    :  1;
        unsigned int FMASK_COMPRESS_1FRAG_ONLY                                    :  1;
        unsigned int DCC_ENABLE                                                   :  1;
        unsigned int CMASK_ADDR_TYPE                                              :  2;
        unsigned int                                                              :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  7;
        unsigned int LINEAR_GENERAL                                               :  1;
        unsigned int                                                              : 24;
    } gfx10;
    struct {
        unsigned int                                                              : 31;
        unsigned int ALT_TILE_MODE                                                :  1;
    } gfx101;
    struct {
        unsigned int                                                              : 19;
        unsigned int CMASK_IS_LINEAR                                              :  1;
        unsigned int                                                              : 12;
    } gfx10Core;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR3_PITCH {
    struct {
        unsigned int TILE_MAX                                                     : 11;
        unsigned int                                                              :  9;
        unsigned int FMASK_TILE_MAX                                               : 11;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR3_SLICE {
    struct {
        unsigned int TILE_MAX                                                     : 22;
        unsigned int                                                              : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR3_VIEW {
    struct {
        unsigned int SLICE_START                                                  : 11;
        unsigned int                                                              :  2;
        unsigned int SLICE_MAX                                                    : 11;
        unsigned int MIP_LEVEL                                                    :  4;
        unsigned int                                                              :  4;
    } gfx09;
    struct {
        unsigned int SLICE_START                                                  : 13;
        unsigned int SLICE_MAX                                                    : 13;
        unsigned int MIP_LEVEL                                                    :  4;
        unsigned int                                                              :  2;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR4_ATTRIB {
    struct {
        unsigned int                                                              : 12;
        unsigned int NUM_SAMPLES                                                  :  3;
        unsigned int NUM_FRAGMENTS                                                :  2;
        unsigned int FORCE_DST_ALPHA_1                                            :  1;
        unsigned int                                                              : 14;
    } core;
    struct {
        unsigned int MIP0_DEPTH                                                   : 11;
        unsigned int META_LINEAR                                                  :  1;
        unsigned int                                                              :  6;
        unsigned int COLOR_SW_MODE                                                :  5;
        unsigned int FMASK_SW_MODE                                                :  5;
        unsigned int RESOURCE_TYPE                                                :  2;
        unsigned int RB_ALIGNED                                                   :  1;
        unsigned int PIPE_ALIGNED                                                 :  1;
    } gfx09;
    struct {
        unsigned int TILE_MODE_INDEX                                              :  5;
        unsigned int FMASK_TILE_MODE_INDEX                                        :  5;
        unsigned int FMASK_BANK_HEIGHT                                            :  2;
        unsigned int                                                              :  6;
        unsigned int DISABLE_FMASK_NOFETCH_OPT                                    :  1;
        unsigned int LIMIT_COLOR_FETCH_TO_256B_MAX                                :  1;
        unsigned int                                                              : 12;
    } gfx10Core;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR4_ATTRIB2 {
    struct {
        unsigned int MIP0_HEIGHT                                                  : 14;
        unsigned int MIP0_WIDTH                                                   : 14;
        unsigned int MAX_MIP                                                      :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR4_ATTRIB3 {
    struct {
        unsigned int MIP0_DEPTH                                                   : 13;
        unsigned int META_LINEAR                                                  :  1;
        unsigned int COLOR_SW_MODE                                                :  5;
        unsigned int FMASK_SW_MODE                                                :  5;
        unsigned int RESOURCE_TYPE                                                :  2;
        unsigned int                                                              :  6;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 26;
        unsigned int CMASK_PIPE_ALIGNED                                           :  1;
        unsigned int                                                              :  3;
        unsigned int DCC_PIPE_ALIGNED                                             :  1;
        unsigned int                                                              :  1;
    } gfx101Plus;
    struct {
        unsigned int                                                              : 27;
        unsigned int RESOURCE_LEVEL                                               :  3;
        unsigned int                                                              :  2;
    } gfx10Core;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR4_BASE {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR4_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR4_CLEAR_WORD0 {
    struct {
        unsigned int CLEAR_WORD0                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR4_CLEAR_WORD1 {
    struct {
        unsigned int CLEAR_WORD1                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR4_CMASK {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR4_CMASK_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR4_CMASK_SLICE {
    struct {
        unsigned int TILE_MAX                                                     : 14;
        unsigned int                                                              : 18;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR4_DCC_BASE {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR4_DCC_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR4_DCC_CONTROL {
    struct {
        unsigned int OVERWRITE_COMBINER_DISABLE                                   :  1;
        unsigned int KEY_CLEAR_ENABLE                                             :  1;
        unsigned int MAX_UNCOMPRESSED_BLOCK_SIZE                                  :  2;
        unsigned int MIN_COMPRESSED_BLOCK_SIZE                                    :  1;
        unsigned int MAX_COMPRESSED_BLOCK_SIZE                                    :  2;
        unsigned int COLOR_TRANSFORM                                              :  2;
        unsigned int INDEPENDENT_64B_BLOCKS                                       :  1;
        unsigned int LOSSY_RGB_PRECISION                                          :  4;
        unsigned int LOSSY_ALPHA_PRECISION                                        :  4;
        unsigned int                                                              : 14;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 18;
        unsigned int DISABLE_CONSTANT_ENCODE_REG                                  :  1;
        unsigned int ENABLE_CONSTANT_ENCODE_REG_WRITE                             :  1;
        unsigned int                                                              : 12;
    } gfx09_1xPlus;
    struct {
        unsigned int                                                              : 20;
        unsigned int INDEPENDENT_128B_BLOCKS                                      :  1;
        unsigned int                                                              : 11;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR4_FMASK {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR4_FMASK_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR4_FMASK_SLICE {
    struct {
        unsigned int TILE_MAX                                                     : 22;
        unsigned int                                                              : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR4_INFO {
    struct {
        unsigned int ENDIAN                                                       :  2;
        unsigned int FORMAT                                                       :  5;
        unsigned int                                                              :  1;
        unsigned int NUMBER_TYPE                                                  :  3;
        unsigned int COMP_SWAP                                                    :  2;
        unsigned int FAST_CLEAR                                                   :  1;
        unsigned int COMPRESSION                                                  :  1;
        unsigned int BLEND_CLAMP                                                  :  1;
        unsigned int BLEND_BYPASS                                                 :  1;
        unsigned int SIMPLE_FLOAT                                                 :  1;
        unsigned int ROUND_MODE                                                   :  1;
        unsigned int                                                              :  1;
        unsigned int BLEND_OPT_DONT_RD_DST                                        :  3;
        unsigned int BLEND_OPT_DISCARD_PIXEL                                      :  3;
        unsigned int FMASK_COMPRESSION_DISABLE                                    :  1;
        unsigned int FMASK_COMPRESS_1FRAG_ONLY                                    :  1;
        unsigned int DCC_ENABLE                                                   :  1;
        unsigned int CMASK_ADDR_TYPE                                              :  2;
        unsigned int                                                              :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  7;
        unsigned int LINEAR_GENERAL                                               :  1;
        unsigned int                                                              : 24;
    } gfx10;
    struct {
        unsigned int                                                              : 31;
        unsigned int ALT_TILE_MODE                                                :  1;
    } gfx101;
    struct {
        unsigned int                                                              : 19;
        unsigned int CMASK_IS_LINEAR                                              :  1;
        unsigned int                                                              : 12;
    } gfx10Core;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR4_PITCH {
    struct {
        unsigned int TILE_MAX                                                     : 11;
        unsigned int                                                              :  9;
        unsigned int FMASK_TILE_MAX                                               : 11;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR4_SLICE {
    struct {
        unsigned int TILE_MAX                                                     : 22;
        unsigned int                                                              : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR4_VIEW {
    struct {
        unsigned int SLICE_START                                                  : 11;
        unsigned int                                                              :  2;
        unsigned int SLICE_MAX                                                    : 11;
        unsigned int MIP_LEVEL                                                    :  4;
        unsigned int                                                              :  4;
    } gfx09;
    struct {
        unsigned int SLICE_START                                                  : 13;
        unsigned int SLICE_MAX                                                    : 13;
        unsigned int MIP_LEVEL                                                    :  4;
        unsigned int                                                              :  2;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR5_ATTRIB {
    struct {
        unsigned int                                                              : 12;
        unsigned int NUM_SAMPLES                                                  :  3;
        unsigned int NUM_FRAGMENTS                                                :  2;
        unsigned int FORCE_DST_ALPHA_1                                            :  1;
        unsigned int                                                              : 14;
    } core;
    struct {
        unsigned int MIP0_DEPTH                                                   : 11;
        unsigned int META_LINEAR                                                  :  1;
        unsigned int                                                              :  6;
        unsigned int COLOR_SW_MODE                                                :  5;
        unsigned int FMASK_SW_MODE                                                :  5;
        unsigned int RESOURCE_TYPE                                                :  2;
        unsigned int RB_ALIGNED                                                   :  1;
        unsigned int PIPE_ALIGNED                                                 :  1;
    } gfx09;
    struct {
        unsigned int TILE_MODE_INDEX                                              :  5;
        unsigned int FMASK_TILE_MODE_INDEX                                        :  5;
        unsigned int FMASK_BANK_HEIGHT                                            :  2;
        unsigned int                                                              :  6;
        unsigned int DISABLE_FMASK_NOFETCH_OPT                                    :  1;
        unsigned int LIMIT_COLOR_FETCH_TO_256B_MAX                                :  1;
        unsigned int                                                              : 12;
    } gfx10Core;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR5_ATTRIB2 {
    struct {
        unsigned int MIP0_HEIGHT                                                  : 14;
        unsigned int MIP0_WIDTH                                                   : 14;
        unsigned int MAX_MIP                                                      :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR5_ATTRIB3 {
    struct {
        unsigned int MIP0_DEPTH                                                   : 13;
        unsigned int META_LINEAR                                                  :  1;
        unsigned int COLOR_SW_MODE                                                :  5;
        unsigned int FMASK_SW_MODE                                                :  5;
        unsigned int RESOURCE_TYPE                                                :  2;
        unsigned int                                                              :  6;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 26;
        unsigned int CMASK_PIPE_ALIGNED                                           :  1;
        unsigned int                                                              :  3;
        unsigned int DCC_PIPE_ALIGNED                                             :  1;
        unsigned int                                                              :  1;
    } gfx101Plus;
    struct {
        unsigned int                                                              : 27;
        unsigned int RESOURCE_LEVEL                                               :  3;
        unsigned int                                                              :  2;
    } gfx10Core;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR5_BASE {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR5_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR5_CLEAR_WORD0 {
    struct {
        unsigned int CLEAR_WORD0                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR5_CLEAR_WORD1 {
    struct {
        unsigned int CLEAR_WORD1                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR5_CMASK {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR5_CMASK_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR5_CMASK_SLICE {
    struct {
        unsigned int TILE_MAX                                                     : 14;
        unsigned int                                                              : 18;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR5_DCC_BASE {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR5_DCC_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR5_DCC_CONTROL {
    struct {
        unsigned int OVERWRITE_COMBINER_DISABLE                                   :  1;
        unsigned int KEY_CLEAR_ENABLE                                             :  1;
        unsigned int MAX_UNCOMPRESSED_BLOCK_SIZE                                  :  2;
        unsigned int MIN_COMPRESSED_BLOCK_SIZE                                    :  1;
        unsigned int MAX_COMPRESSED_BLOCK_SIZE                                    :  2;
        unsigned int COLOR_TRANSFORM                                              :  2;
        unsigned int INDEPENDENT_64B_BLOCKS                                       :  1;
        unsigned int LOSSY_RGB_PRECISION                                          :  4;
        unsigned int LOSSY_ALPHA_PRECISION                                        :  4;
        unsigned int                                                              : 14;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 18;
        unsigned int DISABLE_CONSTANT_ENCODE_REG                                  :  1;
        unsigned int ENABLE_CONSTANT_ENCODE_REG_WRITE                             :  1;
        unsigned int                                                              : 12;
    } gfx09_1xPlus;
    struct {
        unsigned int                                                              : 20;
        unsigned int INDEPENDENT_128B_BLOCKS                                      :  1;
        unsigned int                                                              : 11;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR5_FMASK {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR5_FMASK_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR5_FMASK_SLICE {
    struct {
        unsigned int TILE_MAX                                                     : 22;
        unsigned int                                                              : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR5_INFO {
    struct {
        unsigned int ENDIAN                                                       :  2;
        unsigned int FORMAT                                                       :  5;
        unsigned int                                                              :  1;
        unsigned int NUMBER_TYPE                                                  :  3;
        unsigned int COMP_SWAP                                                    :  2;
        unsigned int FAST_CLEAR                                                   :  1;
        unsigned int COMPRESSION                                                  :  1;
        unsigned int BLEND_CLAMP                                                  :  1;
        unsigned int BLEND_BYPASS                                                 :  1;
        unsigned int SIMPLE_FLOAT                                                 :  1;
        unsigned int ROUND_MODE                                                   :  1;
        unsigned int                                                              :  1;
        unsigned int BLEND_OPT_DONT_RD_DST                                        :  3;
        unsigned int BLEND_OPT_DISCARD_PIXEL                                      :  3;
        unsigned int FMASK_COMPRESSION_DISABLE                                    :  1;
        unsigned int FMASK_COMPRESS_1FRAG_ONLY                                    :  1;
        unsigned int DCC_ENABLE                                                   :  1;
        unsigned int CMASK_ADDR_TYPE                                              :  2;
        unsigned int                                                              :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  7;
        unsigned int LINEAR_GENERAL                                               :  1;
        unsigned int                                                              : 24;
    } gfx10;
    struct {
        unsigned int                                                              : 31;
        unsigned int ALT_TILE_MODE                                                :  1;
    } gfx101;
    struct {
        unsigned int                                                              : 19;
        unsigned int CMASK_IS_LINEAR                                              :  1;
        unsigned int                                                              : 12;
    } gfx10Core;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR5_PITCH {
    struct {
        unsigned int TILE_MAX                                                     : 11;
        unsigned int                                                              :  9;
        unsigned int FMASK_TILE_MAX                                               : 11;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR5_SLICE {
    struct {
        unsigned int TILE_MAX                                                     : 22;
        unsigned int                                                              : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR5_VIEW {
    struct {
        unsigned int SLICE_START                                                  : 11;
        unsigned int                                                              :  2;
        unsigned int SLICE_MAX                                                    : 11;
        unsigned int MIP_LEVEL                                                    :  4;
        unsigned int                                                              :  4;
    } gfx09;
    struct {
        unsigned int SLICE_START                                                  : 13;
        unsigned int SLICE_MAX                                                    : 13;
        unsigned int MIP_LEVEL                                                    :  4;
        unsigned int                                                              :  2;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR6_ATTRIB {
    struct {
        unsigned int                                                              : 12;
        unsigned int NUM_SAMPLES                                                  :  3;
        unsigned int NUM_FRAGMENTS                                                :  2;
        unsigned int FORCE_DST_ALPHA_1                                            :  1;
        unsigned int                                                              : 14;
    } core;
    struct {
        unsigned int MIP0_DEPTH                                                   : 11;
        unsigned int META_LINEAR                                                  :  1;
        unsigned int                                                              :  6;
        unsigned int COLOR_SW_MODE                                                :  5;
        unsigned int FMASK_SW_MODE                                                :  5;
        unsigned int RESOURCE_TYPE                                                :  2;
        unsigned int RB_ALIGNED                                                   :  1;
        unsigned int PIPE_ALIGNED                                                 :  1;
    } gfx09;
    struct {
        unsigned int TILE_MODE_INDEX                                              :  5;
        unsigned int FMASK_TILE_MODE_INDEX                                        :  5;
        unsigned int FMASK_BANK_HEIGHT                                            :  2;
        unsigned int                                                              :  6;
        unsigned int DISABLE_FMASK_NOFETCH_OPT                                    :  1;
        unsigned int LIMIT_COLOR_FETCH_TO_256B_MAX                                :  1;
        unsigned int                                                              : 12;
    } gfx10Core;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR6_ATTRIB2 {
    struct {
        unsigned int MIP0_HEIGHT                                                  : 14;
        unsigned int MIP0_WIDTH                                                   : 14;
        unsigned int MAX_MIP                                                      :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR6_ATTRIB3 {
    struct {
        unsigned int MIP0_DEPTH                                                   : 13;
        unsigned int META_LINEAR                                                  :  1;
        unsigned int COLOR_SW_MODE                                                :  5;
        unsigned int FMASK_SW_MODE                                                :  5;
        unsigned int RESOURCE_TYPE                                                :  2;
        unsigned int                                                              :  6;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 26;
        unsigned int CMASK_PIPE_ALIGNED                                           :  1;
        unsigned int                                                              :  3;
        unsigned int DCC_PIPE_ALIGNED                                             :  1;
        unsigned int                                                              :  1;
    } gfx101Plus;
    struct {
        unsigned int                                                              : 27;
        unsigned int RESOURCE_LEVEL                                               :  3;
        unsigned int                                                              :  2;
    } gfx10Core;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR6_BASE {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR6_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR6_CLEAR_WORD0 {
    struct {
        unsigned int CLEAR_WORD0                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR6_CLEAR_WORD1 {
    struct {
        unsigned int CLEAR_WORD1                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR6_CMASK {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR6_CMASK_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR6_CMASK_SLICE {
    struct {
        unsigned int TILE_MAX                                                     : 14;
        unsigned int                                                              : 18;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR6_DCC_BASE {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR6_DCC_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR6_DCC_CONTROL {
    struct {
        unsigned int OVERWRITE_COMBINER_DISABLE                                   :  1;
        unsigned int KEY_CLEAR_ENABLE                                             :  1;
        unsigned int MAX_UNCOMPRESSED_BLOCK_SIZE                                  :  2;
        unsigned int MIN_COMPRESSED_BLOCK_SIZE                                    :  1;
        unsigned int MAX_COMPRESSED_BLOCK_SIZE                                    :  2;
        unsigned int COLOR_TRANSFORM                                              :  2;
        unsigned int INDEPENDENT_64B_BLOCKS                                       :  1;
        unsigned int LOSSY_RGB_PRECISION                                          :  4;
        unsigned int LOSSY_ALPHA_PRECISION                                        :  4;
        unsigned int                                                              : 14;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 18;
        unsigned int DISABLE_CONSTANT_ENCODE_REG                                  :  1;
        unsigned int ENABLE_CONSTANT_ENCODE_REG_WRITE                             :  1;
        unsigned int                                                              : 12;
    } gfx09_1xPlus;
    struct {
        unsigned int                                                              : 20;
        unsigned int INDEPENDENT_128B_BLOCKS                                      :  1;
        unsigned int                                                              : 11;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR6_FMASK {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR6_FMASK_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR6_FMASK_SLICE {
    struct {
        unsigned int TILE_MAX                                                     : 22;
        unsigned int                                                              : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR6_INFO {
    struct {
        unsigned int ENDIAN                                                       :  2;
        unsigned int FORMAT                                                       :  5;
        unsigned int                                                              :  1;
        unsigned int NUMBER_TYPE                                                  :  3;
        unsigned int COMP_SWAP                                                    :  2;
        unsigned int FAST_CLEAR                                                   :  1;
        unsigned int COMPRESSION                                                  :  1;
        unsigned int BLEND_CLAMP                                                  :  1;
        unsigned int BLEND_BYPASS                                                 :  1;
        unsigned int SIMPLE_FLOAT                                                 :  1;
        unsigned int ROUND_MODE                                                   :  1;
        unsigned int                                                              :  1;
        unsigned int BLEND_OPT_DONT_RD_DST                                        :  3;
        unsigned int BLEND_OPT_DISCARD_PIXEL                                      :  3;
        unsigned int FMASK_COMPRESSION_DISABLE                                    :  1;
        unsigned int FMASK_COMPRESS_1FRAG_ONLY                                    :  1;
        unsigned int DCC_ENABLE                                                   :  1;
        unsigned int CMASK_ADDR_TYPE                                              :  2;
        unsigned int                                                              :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  7;
        unsigned int LINEAR_GENERAL                                               :  1;
        unsigned int                                                              : 24;
    } gfx10;
    struct {
        unsigned int                                                              : 31;
        unsigned int ALT_TILE_MODE                                                :  1;
    } gfx101;
    struct {
        unsigned int                                                              : 19;
        unsigned int CMASK_IS_LINEAR                                              :  1;
        unsigned int                                                              : 12;
    } gfx10Core;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR6_PITCH {
    struct {
        unsigned int TILE_MAX                                                     : 11;
        unsigned int                                                              :  9;
        unsigned int FMASK_TILE_MAX                                               : 11;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR6_SLICE {
    struct {
        unsigned int TILE_MAX                                                     : 22;
        unsigned int                                                              : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR6_VIEW {
    struct {
        unsigned int SLICE_START                                                  : 11;
        unsigned int                                                              :  2;
        unsigned int SLICE_MAX                                                    : 11;
        unsigned int MIP_LEVEL                                                    :  4;
        unsigned int                                                              :  4;
    } gfx09;
    struct {
        unsigned int SLICE_START                                                  : 13;
        unsigned int SLICE_MAX                                                    : 13;
        unsigned int MIP_LEVEL                                                    :  4;
        unsigned int                                                              :  2;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR7_ATTRIB {
    struct {
        unsigned int                                                              : 12;
        unsigned int NUM_SAMPLES                                                  :  3;
        unsigned int NUM_FRAGMENTS                                                :  2;
        unsigned int FORCE_DST_ALPHA_1                                            :  1;
        unsigned int                                                              : 14;
    } core;
    struct {
        unsigned int MIP0_DEPTH                                                   : 11;
        unsigned int META_LINEAR                                                  :  1;
        unsigned int                                                              :  6;
        unsigned int COLOR_SW_MODE                                                :  5;
        unsigned int FMASK_SW_MODE                                                :  5;
        unsigned int RESOURCE_TYPE                                                :  2;
        unsigned int RB_ALIGNED                                                   :  1;
        unsigned int PIPE_ALIGNED                                                 :  1;
    } gfx09;
    struct {
        unsigned int TILE_MODE_INDEX                                              :  5;
        unsigned int FMASK_TILE_MODE_INDEX                                        :  5;
        unsigned int FMASK_BANK_HEIGHT                                            :  2;
        unsigned int                                                              :  6;
        unsigned int DISABLE_FMASK_NOFETCH_OPT                                    :  1;
        unsigned int LIMIT_COLOR_FETCH_TO_256B_MAX                                :  1;
        unsigned int                                                              : 12;
    } gfx10Core;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR7_ATTRIB2 {
    struct {
        unsigned int MIP0_HEIGHT                                                  : 14;
        unsigned int MIP0_WIDTH                                                   : 14;
        unsigned int MAX_MIP                                                      :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR7_ATTRIB3 {
    struct {
        unsigned int MIP0_DEPTH                                                   : 13;
        unsigned int META_LINEAR                                                  :  1;
        unsigned int COLOR_SW_MODE                                                :  5;
        unsigned int FMASK_SW_MODE                                                :  5;
        unsigned int RESOURCE_TYPE                                                :  2;
        unsigned int                                                              :  6;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 26;
        unsigned int CMASK_PIPE_ALIGNED                                           :  1;
        unsigned int                                                              :  3;
        unsigned int DCC_PIPE_ALIGNED                                             :  1;
        unsigned int                                                              :  1;
    } gfx101Plus;
    struct {
        unsigned int                                                              : 27;
        unsigned int RESOURCE_LEVEL                                               :  3;
        unsigned int                                                              :  2;
    } gfx10Core;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR7_BASE {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR7_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR7_CLEAR_WORD0 {
    struct {
        unsigned int CLEAR_WORD0                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR7_CLEAR_WORD1 {
    struct {
        unsigned int CLEAR_WORD1                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR7_CMASK {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR7_CMASK_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR7_CMASK_SLICE {
    struct {
        unsigned int TILE_MAX                                                     : 14;
        unsigned int                                                              : 18;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR7_DCC_BASE {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR7_DCC_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR7_DCC_CONTROL {
    struct {
        unsigned int OVERWRITE_COMBINER_DISABLE                                   :  1;
        unsigned int KEY_CLEAR_ENABLE                                             :  1;
        unsigned int MAX_UNCOMPRESSED_BLOCK_SIZE                                  :  2;
        unsigned int MIN_COMPRESSED_BLOCK_SIZE                                    :  1;
        unsigned int MAX_COMPRESSED_BLOCK_SIZE                                    :  2;
        unsigned int COLOR_TRANSFORM                                              :  2;
        unsigned int INDEPENDENT_64B_BLOCKS                                       :  1;
        unsigned int LOSSY_RGB_PRECISION                                          :  4;
        unsigned int LOSSY_ALPHA_PRECISION                                        :  4;
        unsigned int                                                              : 14;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 18;
        unsigned int DISABLE_CONSTANT_ENCODE_REG                                  :  1;
        unsigned int ENABLE_CONSTANT_ENCODE_REG_WRITE                             :  1;
        unsigned int                                                              : 12;
    } gfx09_1xPlus;
    struct {
        unsigned int                                                              : 20;
        unsigned int INDEPENDENT_128B_BLOCKS                                      :  1;
        unsigned int                                                              : 11;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR7_FMASK {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR7_FMASK_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR7_FMASK_SLICE {
    struct {
        unsigned int TILE_MAX                                                     : 22;
        unsigned int                                                              : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR7_INFO {
    struct {
        unsigned int ENDIAN                                                       :  2;
        unsigned int FORMAT                                                       :  5;
        unsigned int                                                              :  1;
        unsigned int NUMBER_TYPE                                                  :  3;
        unsigned int COMP_SWAP                                                    :  2;
        unsigned int FAST_CLEAR                                                   :  1;
        unsigned int COMPRESSION                                                  :  1;
        unsigned int BLEND_CLAMP                                                  :  1;
        unsigned int BLEND_BYPASS                                                 :  1;
        unsigned int SIMPLE_FLOAT                                                 :  1;
        unsigned int ROUND_MODE                                                   :  1;
        unsigned int                                                              :  1;
        unsigned int BLEND_OPT_DONT_RD_DST                                        :  3;
        unsigned int BLEND_OPT_DISCARD_PIXEL                                      :  3;
        unsigned int FMASK_COMPRESSION_DISABLE                                    :  1;
        unsigned int FMASK_COMPRESS_1FRAG_ONLY                                    :  1;
        unsigned int DCC_ENABLE                                                   :  1;
        unsigned int CMASK_ADDR_TYPE                                              :  2;
        unsigned int                                                              :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  7;
        unsigned int LINEAR_GENERAL                                               :  1;
        unsigned int                                                              : 24;
    } gfx10;
    struct {
        unsigned int                                                              : 31;
        unsigned int ALT_TILE_MODE                                                :  1;
    } gfx101;
    struct {
        unsigned int                                                              : 19;
        unsigned int CMASK_IS_LINEAR                                              :  1;
        unsigned int                                                              : 12;
    } gfx10Core;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR7_PITCH {
    struct {
        unsigned int TILE_MAX                                                     : 11;
        unsigned int                                                              :  9;
        unsigned int FMASK_TILE_MAX                                               : 11;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR7_SLICE {
    struct {
        unsigned int TILE_MAX                                                     : 22;
        unsigned int                                                              : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR7_VIEW {
    struct {
        unsigned int SLICE_START                                                  : 11;
        unsigned int                                                              :  2;
        unsigned int SLICE_MAX                                                    : 11;
        unsigned int MIP_LEVEL                                                    :  4;
        unsigned int                                                              :  4;
    } gfx09;
    struct {
        unsigned int SLICE_START                                                  : 13;
        unsigned int SLICE_MAX                                                    : 13;
        unsigned int MIP_LEVEL                                                    :  4;
        unsigned int                                                              :  2;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR_CONTROL {
    struct {
        unsigned int DISABLE_DUAL_QUAD                                            :  1;
        unsigned int                                                              :  2;
        unsigned int DEGAMMA_ENABLE                                               :  1;
        unsigned int MODE                                                         :  3;
        unsigned int                                                              :  9;
        unsigned int ROP3                                                         :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COVERAGE_OUT_CONTROL {
    struct {
        unsigned int COVERAGE_OUT_ENABLE                                          :  1;
        unsigned int COVERAGE_OUT_MRT                                             :  3;
        unsigned int COVERAGE_OUT_CHANNEL                                         :  2;
        unsigned int                                                              :  2;
        unsigned int COVERAGE_OUT_SAMPLES                                         :  4;
        unsigned int                                                              : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_DCC_CONFIG {
    struct {
        unsigned int OVERWRITE_COMBINER_DEPTH                                     :  5;
        unsigned int OVERWRITE_COMBINER_DISABLE                                   :  1;
        unsigned int OVERWRITE_COMBINER_CC_POP_DISABLE                            :  1;
        unsigned int                                                              :  1;
        unsigned int FC_RDLAT_KEYID_FIFO_DEPTH                                    :  8;
        unsigned int                                                              : 16;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 16;
        unsigned int READ_RETURN_SKID_FIFO_DEPTH                                  :  7;
        unsigned int                                                              :  1;
        unsigned int DCC_CACHE_EVICT_POINT                                        :  4;
        unsigned int DCC_CACHE_NUM_TAGS                                           :  4;
    } gfx09;
    struct {
        unsigned int                                                              :  7;
        unsigned int DISABLE_CONSTANT_ENCODE                                      :  1;
        unsigned int                                                              : 24;
    } gfx09_1xPlus;
    struct {
        unsigned int                                                              : 16;
        unsigned int READ_RETURN_SKID_FIFO_DEPTH                                  :  9;
        unsigned int                                                              :  7;
    } gfx10;
    struct {
        unsigned int                                                              : 26;
        unsigned int DCC_CACHE_NUM_TAGS                                           :  6;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_DCC_CONTROL {
    struct {
        unsigned int OVERWRITE_COMBINER_DISABLE                                   :  1;
        unsigned int                                                              :  1;
        unsigned int OVERWRITE_COMBINER_WATERMARK                                 :  5;
        unsigned int                                                              : 25;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  1;
        unsigned int OVERWRITE_COMBINER_MRT_SHARING_DISABLE                       :  1;
        unsigned int                                                              : 30;
    } gfx09;
    struct {
        unsigned int                                                              :  8;
        unsigned int DISABLE_CONSTANT_ENCODE_AC01                                 :  1;
        unsigned int DISABLE_CONSTANT_ENCODE_SINGLE                               :  1;
        unsigned int DISABLE_CONSTANT_ENCODE_REG                                  :  1;
        unsigned int                                                              :  1;
        unsigned int DISABLE_ELIMFC_SKIP_OF_AC01                                  :  1;
        unsigned int DISABLE_ELIMFC_SKIP_OF_SINGLE                                :  1;
        unsigned int ENABLE_ELIMFC_SKIP_OF_REG                                    :  1;
        unsigned int                                                              : 17;
    } gfx09_1xPlus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_HW_CONTROL {
    struct {
        unsigned int                                                              : 19;
        unsigned int FORCE_NEEDS_DST                                              :  1;
        unsigned int FORCE_ALWAYS_TOGGLE                                          :  1;
        unsigned int DISABLE_BLEND_OPT_RESULT_EQ_DEST                             :  1;
        unsigned int DISABLE_FULL_WRITE_MASK                                      :  1;
        unsigned int DISABLE_RESOLVE_OPT_FOR_SINGLE_FRAG                          :  1;
        unsigned int DISABLE_BLEND_OPT_DONT_RD_DST                                :  1;
        unsigned int DISABLE_BLEND_OPT_BYPASS                                     :  1;
        unsigned int DISABLE_BLEND_OPT_DISCARD_PIXEL                              :  1;
        unsigned int DISABLE_BLEND_OPT_WHEN_DISABLED_SRCALPHA_IS_USED             :  1;
        unsigned int PRIORITIZE_FC_WR_OVER_FC_RD_ON_CMASK_CONFLICT                :  1;
        unsigned int PRIORITIZE_FC_EVICT_OVER_FOP_RD_ON_BANK_CONFLICT             :  1;
        unsigned int DISABLE_CC_IB_SERIALIZER_STATE_OPT                           :  1;
        unsigned int DISABLE_PIXEL_IN_QUAD_FIX_FOR_LINEAR_SURFACE                 :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 18;
        unsigned int DISABLE_INTNORM_LE11BPC_CLAMPING                             :  1;
        unsigned int                                                              : 13;
    } core;
    struct {
        unsigned int CM_CACHE_EVICT_POINT                                         :  4;
        unsigned int                                                              :  2;
        unsigned int FC_CACHE_EVICT_POINT                                         :  4;
        unsigned int                                                              :  2;
        unsigned int CC_CACHE_EVICT_POINT                                         :  4;
        unsigned int ALLOW_MRT_WITH_DUAL_SOURCE                                   :  1;
        unsigned int                                                              : 15;
    } gfx09;
    struct {
        unsigned int ALLOW_MRT_WITH_DUAL_SOURCE                                   :  1;
        unsigned int                                                              : 31;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_HW_CONTROL_1 {
    struct {
        unsigned int CM_CACHE_NUM_TAGS                                            :  5;
        unsigned int                                                              : 27;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  5;
        unsigned int FC_CACHE_NUM_TAGS                                            :  6;
        unsigned int CC_CACHE_NUM_TAGS                                            :  6;
        unsigned int CM_TILE_FIFO_DEPTH                                           :  9;
        unsigned int RMI_CREDITS                                                  :  6;
    } gfx09;
    struct {
        unsigned int                                                              :  5;
        unsigned int FC_CACHE_NUM_TAGS                                            :  6;
        unsigned int CC_CACHE_NUM_TAGS                                            :  6;
        unsigned int CM_TILE_FIFO_DEPTH                                           :  9;
        unsigned int RMI_CREDITS                                                  :  6;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_HW_CONTROL_2 {
    struct {
        unsigned int CC_EVEN_ODD_FIFO_DEPTH                                       :  8;
        unsigned int FC_RDLAT_TILE_FIFO_DEPTH                                     :  7;
        unsigned int FC_RDLAT_QUAD_FIFO_DEPTH                                     :  8;
        unsigned int                                                              :  9;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 24;
        unsigned int DRR_ASSUMED_FIFO_DEPTH_DIV8                                  :  4;
        unsigned int CHICKEN_BITS                                                 :  4;
    } gfx09;
    struct {
        unsigned int                                                              : 24;
        unsigned int DRR_ASSUMED_FIFO_DEPTH_DIV8                                  :  6;
        unsigned int CHICKEN_BITS                                                 :  2;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_HW_CONTROL_3 {
    struct {
        unsigned int DISABLE_SLOW_MODE_EMPTY_HALF_QUAD_KILL                       :  1;
        unsigned int RAM_ADDRESS_CONFLICTS_DISALLOWED                             :  1;
        unsigned int DISABLE_FAST_CLEAR_FETCH_OPT                                 :  1;
        unsigned int DISABLE_QUAD_MARKER_DROP_STOP                                :  1;
        unsigned int DISABLE_OVERWRITE_COMBINER_CAM_CLR                           :  1;
        unsigned int DISABLE_CC_CACHE_OVWR_STATUS_ACCUM                           :  1;
        unsigned int                                                              :  1;
        unsigned int DISABLE_CC_CACHE_PANIC_GATING                                :  1;
        unsigned int                                                              :  1;
        unsigned int SPLIT_ALL_FAST_MODE_TRANSFERS                                :  1;
        unsigned int DISABLE_SHADER_BLEND_OPTS                                    :  1;
        unsigned int DISABLE_CMASK_LAST_QUAD_INSERTION                            :  1;
        unsigned int DISABLE_ROP3_FIXES_OF_BUG_511967                             :  1;
        unsigned int DISABLE_ROP3_FIXES_OF_BUG_520657                             :  1;
        unsigned int DISABLE_OC_FIXES_OF_BUG_522542                               :  1;
        unsigned int FORCE_RMI_LAST_HIGH                                          :  1;
        unsigned int FORCE_RMI_CLKEN_HIGH                                         :  1;
        unsigned int DISABLE_EARLY_WRACKS_CC                                      :  1;
        unsigned int DISABLE_EARLY_WRACKS_FC                                      :  1;
        unsigned int DISABLE_EARLY_WRACKS_DC                                      :  1;
        unsigned int DISABLE_EARLY_WRACKS_CM                                      :  1;
        unsigned int DISABLE_NACK_PROCESSING_CC                                   :  1;
        unsigned int DISABLE_NACK_PROCESSING_FC                                   :  1;
        unsigned int DISABLE_NACK_PROCESSING_DC                                   :  1;
        unsigned int DISABLE_NACK_PROCESSING_CM                                   :  1;
        unsigned int DISABLE_NACK_COLOR_RD_WR_OPT                                 :  1;
        unsigned int DISABLE_BLENDER_CLOCK_GATING                                 :  1;
        unsigned int                                                              :  5;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  6;
        unsigned int DISABLE_CC_CACHE_OVWR_KEY_MOD                                :  1;
        unsigned int                                                              :  1;
        unsigned int DISABLE_OVERWRITE_COMBINER_TARGET_MASK_VALIDATION            :  1;
        unsigned int                                                              : 18;
        unsigned int                                                              :  1;
        unsigned int COLOR_CACHE_PREFETCH_NUM_CLS                                 :  2;
        unsigned int                                                              :  2;
    } gfx09;
    struct {
        unsigned int                                                              : 30;
        unsigned int DISABLE_FMASK_NOFETCH_OPT                                    :  1;
        unsigned int                                                              :  1;
    } gfx10;
    struct {
        unsigned int                                                              : 27;
        unsigned int                                                              :  1;
        unsigned int                                                              :  4;
    } gfx101;
    struct {
        unsigned int                                                              : 31;
        unsigned int DISABLE_FMASK_NOFETCH_OPT_BC                                 :  1;
    } gfx10Core;
    struct {
        unsigned int                                                              :  8;
        unsigned int DISABLE_OVERWRITE_COMBINER_TARGET_MASK_VALIDATION            :  1;
        unsigned int                                                              : 23;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_HW_CONTROL_4 {
    struct {
        unsigned int COLOR_CACHE_FETCH_NUM_CLS_LOG2                               :  3;
        unsigned int FMASK_CACHE_FETCH_NUM_CLS_LOG2                               :  2;
        unsigned int DISABLE_USE_OF_QUAD_SCOREBOARD                               :  1;
        unsigned int DISABLE_CMASK_CLOCK_GATING                                   :  1;
        unsigned int DISABLE_FMASK_CLOCK_GATING                                   :  1;
        unsigned int DISABLE_COLOR_CLOCK_GATING                                   :  1;
        unsigned int DISABLE_QSB_AA_MODE                                          :  1;
        unsigned int DISABLE_QSB_WAIT_FOR_SCORE                                   :  1;
        unsigned int DISABLE_QSB_FRAG_GT0                                         :  1;
        unsigned int REVERSE_KEYXFR_RD_PRIORITY                                   :  1;
        unsigned int DISABLE_KEYXFR_HIT_RETURNS                                   :  1;
        unsigned int                                                              :  1;
        unsigned int DISABLE_MA_WAIT_FOR_LAST                                     :  1;
        unsigned int DISABLE_QSB_SPECULATIVE                                      :  1;
        unsigned int QSB_WAIT_FOR_SCORE                                           :  5;
        unsigned int                                                              :  2;
        unsigned int FC_QSB_FIFO_DEPTH                                            :  8;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 22;
        unsigned int DISABLE_TILE_FGCG                                            :  1;
        unsigned int DISABLE_LQUAD_FGCG                                           :  1;
        unsigned int                                                              :  8;
    } gfx101Plus;
    struct {
        unsigned int                                                              : 14;
        unsigned int DISABLE_BC_COLOR_CACHE_PREFETCH                              :  1;
        unsigned int                                                              : 17;
    } gfx10Core;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_HW_MEM_ARBITER_RD {
    struct {
        unsigned int MODE                                                         :  2;
        unsigned int IGNORE_URGENT_AGE                                            :  4;
        unsigned int BREAK_GROUP_AGE                                              :  4;
        unsigned int WEIGHT_CC                                                    :  2;
        unsigned int WEIGHT_FC                                                    :  2;
        unsigned int WEIGHT_CM                                                    :  2;
        unsigned int WEIGHT_DC                                                    :  2;
        unsigned int WEIGHT_DECAY_REQS                                            :  2;
        unsigned int WEIGHT_DECAY_NOREQS                                          :  2;
        unsigned int WEIGHT_IGNORE_NUM_TIDS                                       :  1;
        unsigned int SCALE_AGE                                                    :  3;
        unsigned int SCALE_WEIGHT                                                 :  3;
        unsigned int SEND_LASTS_WITHIN_GROUPS                                     :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_HW_MEM_ARBITER_WR {
    struct {
        unsigned int MODE                                                         :  2;
        unsigned int IGNORE_URGENT_AGE                                            :  4;
        unsigned int BREAK_GROUP_AGE                                              :  4;
        unsigned int WEIGHT_CC                                                    :  2;
        unsigned int WEIGHT_FC                                                    :  2;
        unsigned int WEIGHT_CM                                                    :  2;
        unsigned int WEIGHT_DC                                                    :  2;
        unsigned int WEIGHT_DECAY_REQS                                            :  2;
        unsigned int WEIGHT_DECAY_NOREQS                                          :  2;
        unsigned int WEIGHT_IGNORE_BYTE_MASK                                      :  1;
        unsigned int SCALE_AGE                                                    :  3;
        unsigned int SCALE_WEIGHT                                                 :  3;
        unsigned int SEND_LASTS_WITHIN_GROUPS                                     :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_MRT0_EPITCH {
    struct {
        unsigned int EPITCH                                                       : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_MRT1_EPITCH {
    struct {
        unsigned int EPITCH                                                       : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_MRT2_EPITCH {
    struct {
        unsigned int EPITCH                                                       : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_MRT3_EPITCH {
    struct {
        unsigned int EPITCH                                                       : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_MRT4_EPITCH {
    struct {
        unsigned int EPITCH                                                       : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_MRT5_EPITCH {
    struct {
        unsigned int EPITCH                                                       : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_MRT6_EPITCH {
    struct {
        unsigned int EPITCH                                                       : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_MRT7_EPITCH {
    struct {
        unsigned int EPITCH                                                       : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              :  1;
        unsigned int PERF_SEL1                                                    :  9;
        unsigned int                                                              :  1;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    :  9;
        unsigned int                                                              :  1;
        unsigned int PERF_SEL3                                                    :  9;
        unsigned int                                                              :  5;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 19;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 19;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 19;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_PERFCOUNTER_FILTER {
    struct {
        unsigned int OP_FILTER_ENABLE                                             :  1;
        unsigned int OP_FILTER_SEL                                                :  3;
        unsigned int FORMAT_FILTER_ENABLE                                         :  1;
        unsigned int FORMAT_FILTER_SEL                                            :  5;
        unsigned int CLEAR_FILTER_ENABLE                                          :  1;
        unsigned int CLEAR_FILTER_SEL                                             :  1;
        unsigned int MRT_FILTER_ENABLE                                            :  1;
        unsigned int MRT_FILTER_SEL                                               :  3;
        unsigned int                                                              :  1;
        unsigned int NUM_SAMPLES_FILTER_ENABLE                                    :  1;
        unsigned int NUM_SAMPLES_FILTER_SEL                                       :  3;
        unsigned int NUM_FRAGMENTS_FILTER_ENABLE                                  :  1;
        unsigned int NUM_FRAGMENTS_FILTER_SEL                                     :  2;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_RMI_BC_GL2_CACHE_CONTROL {
    struct {
        unsigned int CMASK_WR_POLICY                                              :  2;
        unsigned int FMASK_WR_POLICY                                              :  2;
        unsigned int DCC_WR_POLICY                                                :  2;
        unsigned int COLOR_WR_POLICY                                              :  2;
        unsigned int                                                              :  8;
        unsigned int CMASK_RD_POLICY                                              :  2;
        unsigned int FMASK_RD_POLICY                                              :  2;
        unsigned int DCC_RD_POLICY                                                :  2;
        unsigned int COLOR_RD_POLICY                                              :  2;
        unsigned int                                                              :  7;
        unsigned int VOLAT                                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_RMI_GL2_CACHE_CONTROL {
    struct {
        unsigned int CMASK_WR_POLICY                                              :  2;
        unsigned int FMASK_WR_POLICY                                              :  2;
        unsigned int DCC_WR_POLICY                                                :  2;
        unsigned int COLOR_WR_POLICY                                              :  2;
        unsigned int                                                              :  8;
        unsigned int CMASK_RD_POLICY                                              :  2;
        unsigned int FMASK_RD_POLICY                                              :  2;
        unsigned int DCC_RD_POLICY                                                :  2;
        unsigned int COLOR_RD_POLICY                                              :  2;
        unsigned int                                                              :  8;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 30;
        unsigned int FMASK_BIG_PAGE                                               :  1;
        unsigned int COLOR_BIG_PAGE                                               :  1;
    } gfx101Plus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_SHADER_MASK {
    struct {
        unsigned int OUTPUT0_ENABLE                                               :  4;
        unsigned int OUTPUT1_ENABLE                                               :  4;
        unsigned int OUTPUT2_ENABLE                                               :  4;
        unsigned int OUTPUT3_ENABLE                                               :  4;
        unsigned int OUTPUT4_ENABLE                                               :  4;
        unsigned int OUTPUT5_ENABLE                                               :  4;
        unsigned int OUTPUT6_ENABLE                                               :  4;
        unsigned int OUTPUT7_ENABLE                                               :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_STUTTER_CONTROL_CMASK_RDLAT {
    struct {
        unsigned int THRESHOLD                                                    :  8;
        unsigned int TIMEOUT                                                      :  8;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_STUTTER_CONTROL_COLOR_RDLAT {
    struct {
        unsigned int THRESHOLD                                                    :  8;
        unsigned int TIMEOUT                                                      :  8;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_STUTTER_CONTROL_FMASK_RDLAT {
    struct {
        unsigned int THRESHOLD                                                    :  8;
        unsigned int TIMEOUT                                                      :  8;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_TARGET_MASK {
    struct {
        unsigned int TARGET0_ENABLE                                               :  4;
        unsigned int TARGET1_ENABLE                                               :  4;
        unsigned int TARGET2_ENABLE                                               :  4;
        unsigned int TARGET3_ENABLE                                               :  4;
        unsigned int TARGET4_ENABLE                                               :  4;
        unsigned int TARGET5_ENABLE                                               :  4;
        unsigned int TARGET6_ENABLE                                               :  4;
        unsigned int TARGET7_ENABLE                                               :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CC_RB_BACKEND_DISABLE {
    struct {
        unsigned int                                                              : 16;
        unsigned int BACKEND_DISABLE                                              :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CC_RB_DAISY_CHAIN {
    struct {
        unsigned int RB_0                                                         :  4;
        unsigned int RB_1                                                         :  4;
        unsigned int RB_2                                                         :  4;
        unsigned int RB_3                                                         :  4;
        unsigned int RB_4                                                         :  4;
        unsigned int RB_5                                                         :  4;
        unsigned int RB_6                                                         :  4;
        unsigned int RB_7                                                         :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CC_RB_REDUNDANCY {
    struct {
        unsigned int                                                              :  8;
        unsigned int FAILED_RB0                                                   :  4;
        unsigned int EN_REDUNDANCY0                                               :  1;
        unsigned int                                                              :  3;
        unsigned int FAILED_RB1                                                   :  4;
        unsigned int EN_REDUNDANCY1                                               :  1;
        unsigned int                                                              : 11;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CC_RMI_REDUNDANCY {
    struct {
        unsigned int                                                              :  1;
        unsigned int REPAIR_EN_IN_0                                               :  1;
        unsigned int REPAIR_EN_IN_1                                               :  1;
        unsigned int REPAIR_RMI_OVERRIDE                                          :  1;
        unsigned int REPAIR_ID_SWAP                                               :  1;
        unsigned int                                                              : 27;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHA_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHA_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHA_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHA_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHA_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHA_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHA_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHA_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHA_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHA_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHA_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHA_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHA_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHCG_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHCG_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHCG_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHCG_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHCG_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHCG_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHCG_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHCG_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHCG_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHCG_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHCG_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHCG_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHCG_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHC_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHC_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHC_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHC_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHC_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHC_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHC_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHC_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHC_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHC_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHC_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHC_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHC_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CH_ARB_CTRL {
    struct {
        unsigned int NUM_MEM_PIPES                                                :  3;
        unsigned int                                                              : 29;
    } gfx101;
    struct {
        unsigned int                                                              :  3;
        unsigned int UC_IO_WR_PATH                                                :  1;
        unsigned int                                                              : 28;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CH_ARB_STATUS {
    struct {
        unsigned int REQ_ARB_BUSY                                                 :  1;
        unsigned int RET_ARB_BUSY                                                 :  1;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CH_DRAM_BURST_CTRL {
    struct {
        unsigned int MAX_DRAM_BURST                                               :  3;
        unsigned int BURST_DISABLE                                                :  1;
        unsigned int GATHER_64B_MEMORY_BURST_DISABLE                              :  1;
        unsigned int GATHER_64B_IO_BURST_DISABLE                                  :  1;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CH_DRAM_BURST_MASK {
    struct {
        unsigned int DRAM_BURST_ADDR_MASK                                         :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CH_PIPE_STEER {
    struct {
        unsigned int PIPE0                                                        :  2;
        unsigned int PIPE1                                                        :  2;
        unsigned int PIPE2                                                        :  2;
        unsigned int PIPE3                                                        :  2;
        unsigned int                                                              : 24;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  8;
        unsigned int PIPE4                                                        :  2;
        unsigned int PIPE5                                                        :  2;
        unsigned int PIPE6                                                        :  2;
        unsigned int PIPE7                                                        :  2;
        unsigned int PIPE8                                                        :  2;
        unsigned int PIPE9                                                        :  2;
        unsigned int PIPE10                                                       :  2;
        unsigned int PIPE11                                                       :  2;
        unsigned int PIPE12                                                       :  2;
        unsigned int PIPE13                                                       :  2;
        unsigned int PIPE14                                                       :  2;
        unsigned int PIPE15                                                       :  2;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CH_VC5_ENABLE {
    struct {
        unsigned int                                                              :  1;
        unsigned int UTCL2_VC5_ENABLE                                             :  1;
        unsigned int                                                              : 30;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COHER_DEST_BASE_0 {
    struct {
        unsigned int DEST_BASE_256B                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COHER_DEST_BASE_1 {
    struct {
        unsigned int DEST_BASE_256B                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COHER_DEST_BASE_2 {
    struct {
        unsigned int DEST_BASE_256B                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COHER_DEST_BASE_3 {
    struct {
        unsigned int DEST_BASE_256B                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COHER_DEST_BASE_HI_0 {
    struct {
        unsigned int DEST_BASE_HI_256B                                            :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COHER_DEST_BASE_HI_1 {
    struct {
        unsigned int DEST_BASE_HI_256B                                            :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COHER_DEST_BASE_HI_2 {
    struct {
        unsigned int DEST_BASE_HI_256B                                            :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COHER_DEST_BASE_HI_3 {
    struct {
        unsigned int DEST_BASE_HI_256B                                            :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_DDID_INDEX {
    struct {
        unsigned int INDEX                                                        : 11;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_DESTINATION_EN_SE0 {
    struct {
        unsigned int CU_EN                                                        : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_DESTINATION_EN_SE1 {
    struct {
        unsigned int CU_EN                                                        : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_DESTINATION_EN_SE2 {
    struct {
        unsigned int CU_EN                                                        : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_DESTINATION_EN_SE3 {
    struct {
        unsigned int CU_EN                                                        : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_DIM_X {
    struct {
        unsigned int SIZE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_DIM_Y {
    struct {
        unsigned int SIZE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_DIM_Z {
    struct {
        unsigned int SIZE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_DISPATCH_END {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_DISPATCH_ID {
    struct {
        unsigned int DISPATCH_ID                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_DISPATCH_INITIATOR {
    struct {
        unsigned int COMPUTE_SHADER_EN                                            :  1;
        unsigned int PARTIAL_TG_EN                                                :  1;
        unsigned int FORCE_START_AT_000                                           :  1;
        unsigned int ORDERED_APPEND_ENBL                                          :  1;
        unsigned int ORDERED_APPEND_MODE                                          :  1;
        unsigned int USE_THREAD_DIMENSIONS                                        :  1;
        unsigned int ORDER_MODE                                                   :  1;
        unsigned int                                                              :  3;
        unsigned int SCALAR_L1_INV_VOL                                            :  1;
        unsigned int VECTOR_L1_INV_VOL                                            :  1;
        unsigned int RESERVED                                                     :  1;
        unsigned int                                                              :  1;
        unsigned int RESTORE                                                      :  1;
        unsigned int                                                              : 17;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 13;
        unsigned int TUNNEL_ENABLE                                                :  1;
        unsigned int                                                              :  1;
        unsigned int CS_W32_EN                                                    :  1;
        unsigned int                                                              : 16;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_DISPATCH_PKT_ADDR_HI {
    struct {
        unsigned int DATA                                                         :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_DISPATCH_PKT_ADDR_LO {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_DISPATCH_SCRATCH_BASE_HI {
    struct {
        unsigned int DATA                                                         :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_DISPATCH_SCRATCH_BASE_LO {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_DISPATCH_TUNNEL {
    struct {
        unsigned int OFF_DELAY                                                    : 10;
        unsigned int IMMEDIATE                                                    :  1;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_MISC_RESERVED {
    struct {
        unsigned int SEND_SEID                                                    :  2;
        unsigned int RESERVED2                                                    :  1;
        unsigned int RESERVED3                                                    :  1;
        unsigned int RESERVED4                                                    :  1;
        unsigned int WAVE_ID_BASE                                                 : 12;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_NOWHERE {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_NUM_THREAD_X {
    struct {
        unsigned int NUM_THREAD_FULL                                              : 16;
        unsigned int NUM_THREAD_PARTIAL                                           : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_NUM_THREAD_Y {
    struct {
        unsigned int NUM_THREAD_FULL                                              : 16;
        unsigned int NUM_THREAD_PARTIAL                                           : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_NUM_THREAD_Z {
    struct {
        unsigned int NUM_THREAD_FULL                                              : 16;
        unsigned int NUM_THREAD_PARTIAL                                           : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_PERFCOUNT_ENABLE {
    struct {
        unsigned int PERFCOUNT_ENABLE                                             :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_PGM_HI {
    struct {
        unsigned int DATA                                                         :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_PGM_LO {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_PGM_RSRC1 {
    struct {
        unsigned int VGPRS                                                        :  6;
        unsigned int SGPRS                                                        :  4;
        unsigned int PRIORITY                                                     :  2;
        unsigned int FLOAT_MODE                                                   :  8;
        unsigned int PRIV                                                         :  1;
        unsigned int DX10_CLAMP                                                   :  1;
        unsigned int DEBUG_MODE                                                   :  1;
        unsigned int IEEE_MODE                                                    :  1;
        unsigned int BULKY                                                        :  1;
        unsigned int CDBG_USER                                                    :  1;
        unsigned int FP16_OVFL                                                    :  1;
        unsigned int                                                              :  5;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 29;
        unsigned int WGP_MODE                                                     :  1;
        unsigned int MEM_ORDERED                                                  :  1;
        unsigned int FWD_PROGRESS                                                 :  1;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_PGM_RSRC2 {
    struct {
        unsigned int SCRATCH_EN                                                   :  1;
        unsigned int USER_SGPR                                                    :  5;
        unsigned int TRAP_PRESENT                                                 :  1;
        unsigned int TGID_X_EN                                                    :  1;
        unsigned int TGID_Y_EN                                                    :  1;
        unsigned int TGID_Z_EN                                                    :  1;
        unsigned int TG_SIZE_EN                                                   :  1;
        unsigned int TIDIG_COMP_CNT                                               :  2;
        unsigned int EXCP_EN_MSB                                                  :  2;
        unsigned int LDS_SIZE                                                     :  9;
        unsigned int EXCP_EN                                                      :  7;
        unsigned int                                                              :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 31;
        unsigned int SKIP_USGPR0                                                  :  1;
    } gfx09;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_PGM_RSRC3 {
    struct {
        unsigned int SHARED_VGPR_CNT                                              :  4;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_PIPELINESTAT_ENABLE {
    struct {
        unsigned int PIPELINESTAT_ENABLE                                          :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_PREF_PRI_ACCUM_0 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int COEFFICIENT_HIER_SELECT                                      :  3;
        unsigned int CONTRIBUTION_HIER_SELECT                                     :  3;
        unsigned int GROUP_UPDATE_EN                                              :  1;
        unsigned int RESERVED                                                     :  1;
        unsigned int COEFFICIENT                                                  :  8;
        unsigned int                                                              :  9;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_PREF_PRI_ACCUM_1 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int COEFFICIENT_HIER_SELECT                                      :  3;
        unsigned int CONTRIBUTION_HIER_SELECT                                     :  3;
        unsigned int GROUP_UPDATE_EN                                              :  1;
        unsigned int RESERVED                                                     :  1;
        unsigned int COEFFICIENT                                                  :  8;
        unsigned int                                                              :  9;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_PREF_PRI_ACCUM_2 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int COEFFICIENT_HIER_SELECT                                      :  3;
        unsigned int CONTRIBUTION_HIER_SELECT                                     :  3;
        unsigned int GROUP_UPDATE_EN                                              :  1;
        unsigned int RESERVED                                                     :  1;
        unsigned int COEFFICIENT                                                  :  8;
        unsigned int                                                              :  9;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_PREF_PRI_ACCUM_3 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int COEFFICIENT_HIER_SELECT                                      :  3;
        unsigned int CONTRIBUTION_HIER_SELECT                                     :  3;
        unsigned int GROUP_UPDATE_EN                                              :  1;
        unsigned int RESERVED                                                     :  1;
        unsigned int COEFFICIENT                                                  :  8;
        unsigned int                                                              :  9;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_RELAUNCH {
    struct {
        unsigned int PAYLOAD                                                      : 30;
        unsigned int IS_EVENT                                                     :  1;
        unsigned int IS_STATE                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_RELAUNCH2 {
    struct {
        unsigned int PAYLOAD                                                      : 30;
        unsigned int IS_EVENT                                                     :  1;
        unsigned int IS_STATE                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_REQ_CTRL {
    struct {
        unsigned int SOFT_GROUPING_EN                                             :  1;
        unsigned int NUMBER_OF_REQUESTS_PER_CU                                    :  4;
        unsigned int SOFT_GROUPING_ALLOCATION_TIMEOUT                             :  4;
        unsigned int HARD_LOCK_HYSTERESIS                                         :  1;
        unsigned int HARD_LOCK_LOW_THRESHOLD                                      :  5;
        unsigned int PRODUCER_REQUEST_LOCKOUT                                     :  1;
        unsigned int GLOBAL_SCANNING_EN                                           :  1;
        unsigned int ALLOCATION_RATE_THROTTLING_THRESHOLD                         :  3;
        unsigned int DEDICATED_PREALLOCATION_BUFFER_LIMIT                         :  7;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_RESOURCE_LIMITS {
    struct {
        unsigned int WAVES_PER_SH                                                 : 10;
        unsigned int                                                              :  2;
        unsigned int TG_PER_CU                                                    :  4;
        unsigned int LOCK_THRESHOLD                                               :  6;
        unsigned int SIMD_DEST_CNTL                                               :  1;
        unsigned int FORCE_SIMD_DIST                                              :  1;
        unsigned int CU_GROUP_COUNT                                               :  3;
        unsigned int                                                              :  5;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 27;
        unsigned int SIMD_DISABLE                                                 :  4;
        unsigned int                                                              :  1;
    } gfx09;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_RESTART_X {
    struct {
        unsigned int RESTART                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_RESTART_Y {
    struct {
        unsigned int RESTART                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_RESTART_Z {
    struct {
        unsigned int RESTART                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_SHADER_CHKSUM {
    struct {
        unsigned int CHECKSUM                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_START_X {
    struct {
        unsigned int START                                                        : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_START_Y {
    struct {
        unsigned int START                                                        : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_START_Z {
    struct {
        unsigned int START                                                        : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_STATIC_THREAD_MGMT_SE0 {
    struct {
        unsigned int SH0_CU_EN                                                    : 16;
        unsigned int SH1_CU_EN                                                    : 16;
    } gfx09;
    struct {
        unsigned int SA0_CU_EN                                                    : 16;
        unsigned int SA1_CU_EN                                                    : 16;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_STATIC_THREAD_MGMT_SE1 {
    struct {
        unsigned int SH0_CU_EN                                                    : 16;
        unsigned int SH1_CU_EN                                                    : 16;
    } gfx09;
    struct {
        unsigned int SA0_CU_EN                                                    : 16;
        unsigned int SA1_CU_EN                                                    : 16;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_STATIC_THREAD_MGMT_SE2 {
    struct {
        unsigned int SH0_CU_EN                                                    : 16;
        unsigned int SH1_CU_EN                                                    : 16;
    } gfx09;
    struct {
        unsigned int SA0_CU_EN                                                    : 16;
        unsigned int SA1_CU_EN                                                    : 16;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_STATIC_THREAD_MGMT_SE3 {
    struct {
        unsigned int SH0_CU_EN                                                    : 16;
        unsigned int SH1_CU_EN                                                    : 16;
    } gfx09;
    struct {
        unsigned int SA0_CU_EN                                                    : 16;
        unsigned int SA1_CU_EN                                                    : 16;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_THREADGROUP_ID {
    struct {
        unsigned int THREADGROUP_ID                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_THREAD_TRACE_ENABLE {
    struct {
        unsigned int THREAD_TRACE_ENABLE                                          :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_TMPRING_SIZE {
    struct {
        unsigned int WAVES                                                        : 12;
        unsigned int WAVESIZE                                                     : 13;
        unsigned int                                                              :  7;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_ACCUM_0 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_ACCUM_1 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_ACCUM_2 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_ACCUM_3 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_DATA_0 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_DATA_1 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_DATA_2 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_DATA_3 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_DATA_4 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_DATA_5 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_DATA_6 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_DATA_7 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_DATA_8 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_DATA_9 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_DATA_10 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_DATA_11 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_DATA_12 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_DATA_13 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_DATA_14 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_DATA_15 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_VMID {
    struct {
        unsigned int DATA                                                         :  4;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_WAVE_RESTORE_ADDR_HI {
    struct {
        unsigned int ADDR                                                         : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_WAVE_RESTORE_ADDR_LO {
    struct {
        unsigned int ADDR                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPC_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPC_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPC_PERFCOUNTER0_SELECT {
    struct {
        unsigned int                                                              : 20;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int CNTR_MODE1                                                   :  4;
        unsigned int CNTR_MODE0                                                   :  4;
    } bits, bitfields;
    struct {
        unsigned int CNTR_SEL0                                                    : 10;
        unsigned int CNTR_SEL1                                                    : 10;
        unsigned int                                                              : 12;
    } gfx09;
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int                                                              : 12;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPC_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int                                                              : 24;
        unsigned int CNTR_MODE3                                                   :  4;
        unsigned int CNTR_MODE2                                                   :  4;
    } bits, bitfields;
    struct {
        unsigned int CNTR_SEL2                                                    : 10;
        unsigned int CNTR_SEL3                                                    : 10;
        unsigned int                                                              : 12;
    } gfx09;
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              : 12;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPC_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPC_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPC_PERFCOUNTER1_SELECT {
    struct {
        unsigned int                                                              : 20;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int CNTR_MODE1                                                   :  4;
        unsigned int CNTR_MODE0                                                   :  4;
    } bits, bitfields;
    struct {
        unsigned int CNTR_SEL0                                                    : 10;
        unsigned int CNTR_SEL1                                                    : 10;
        unsigned int                                                              : 12;
    } gfx09;
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int                                                              : 12;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPF_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPF_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPF_PERFCOUNTER0_SELECT {
    struct {
        unsigned int                                                              : 20;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int CNTR_MODE1                                                   :  4;
        unsigned int CNTR_MODE0                                                   :  4;
    } bits, bitfields;
    struct {
        unsigned int CNTR_SEL0                                                    : 10;
        unsigned int CNTR_SEL1                                                    : 10;
        unsigned int                                                              : 12;
    } gfx09;
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int                                                              : 12;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPF_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int                                                              : 24;
        unsigned int CNTR_MODE3                                                   :  4;
        unsigned int CNTR_MODE2                                                   :  4;
    } bits, bitfields;
    struct {
        unsigned int CNTR_SEL2                                                    : 10;
        unsigned int CNTR_SEL3                                                    : 10;
        unsigned int                                                              : 12;
    } gfx09;
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              : 12;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPF_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPF_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPF_PERFCOUNTER1_SELECT {
    struct {
        unsigned int                                                              : 20;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int CNTR_MODE1                                                   :  4;
        unsigned int CNTR_MODE0                                                   :  4;
    } bits, bitfields;
    struct {
        unsigned int CNTR_SEL0                                                    : 10;
        unsigned int CNTR_SEL1                                                    : 10;
        unsigned int                                                              : 12;
    } gfx09;
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int                                                              : 12;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPG_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPG_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPG_PERFCOUNTER0_SELECT {
    struct {
        unsigned int                                                              : 20;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int CNTR_MODE1                                                   :  4;
        unsigned int CNTR_MODE0                                                   :  4;
    } bits, bitfields;
    struct {
        unsigned int CNTR_SEL0                                                    : 10;
        unsigned int CNTR_SEL1                                                    : 10;
        unsigned int                                                              : 12;
    } gfx09;
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int                                                              : 12;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPG_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int                                                              : 24;
        unsigned int CNTR_MODE3                                                   :  4;
        unsigned int CNTR_MODE2                                                   :  4;
    } bits, bitfields;
    struct {
        unsigned int CNTR_SEL2                                                    : 10;
        unsigned int CNTR_SEL3                                                    : 10;
        unsigned int                                                              : 12;
    } gfx09;
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              : 12;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPG_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPG_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPG_PERFCOUNTER1_SELECT {
    struct {
        unsigned int                                                              : 20;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int CNTR_MODE1                                                   :  4;
        unsigned int CNTR_MODE0                                                   :  4;
    } bits, bitfields;
    struct {
        unsigned int CNTR_SEL0                                                    : 10;
        unsigned int CNTR_SEL1                                                    : 10;
        unsigned int                                                              : 12;
    } gfx09;
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int                                                              : 12;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_APPEND_ADDR_HI {
    struct {
        unsigned int MEM_ADDR_HI                                                  : 16;
        unsigned int CS_PS_SEL                                                    :  1;
        unsigned int                                                              : 12;
        unsigned int COMMAND                                                      :  3;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 25;
        unsigned int CACHE_POLICY                                                 :  1;
        unsigned int                                                              :  6;
    } gfx09;
    struct {
        unsigned int                                                              : 25;
        unsigned int CACHE_POLICY                                                 :  2;
        unsigned int                                                              :  5;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_APPEND_ADDR_LO {
    struct {
        unsigned int                                                              :  2;
        unsigned int MEM_ADDR_LO                                                  : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_APPEND_CMD_ADDR_HI {
    struct {
        unsigned int ADDR_HI                                                      : 16;
        unsigned int RSVD                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_APPEND_CMD_ADDR_LO {
    struct {
        unsigned int RSVD                                                         :  2;
        unsigned int ADDR_LO                                                      : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_APPEND_DATA {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_APPEND_DATA_HI {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_APPEND_DATA_LO {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_APPEND_DDID_CNT {
    struct {
        unsigned int DATA                                                         :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_APPEND_LAST_CS_FENCE {
    struct {
        unsigned int LAST_FENCE                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_APPEND_LAST_CS_FENCE_HI {
    struct {
        unsigned int LAST_FENCE                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_APPEND_LAST_CS_FENCE_LO {
    struct {
        unsigned int LAST_FENCE                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_APPEND_LAST_PS_FENCE {
    struct {
        unsigned int LAST_FENCE                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_APPEND_LAST_PS_FENCE_HI {
    struct {
        unsigned int LAST_FENCE                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_APPEND_LAST_PS_FENCE_LO {
    struct {
        unsigned int LAST_FENCE                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_AQL_SMM_STATUS {
    struct {
        unsigned int AQL_QUEUE_SMM                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ATOMIC_PREOP_HI {
    struct {
        unsigned int ATOMIC_PREOP_HI                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ATOMIC_PREOP_LO {
    struct {
        unsigned int ATOMIC_PREOP_LO                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_BUSY_STAT {
    struct {
        unsigned int REG_BUS_FIFO_BUSY                                            :  1;
        unsigned int                                                              :  5;
        unsigned int COHER_CNT_NEQ_ZERO                                           :  1;
        unsigned int PFP_PARSING_PACKETS                                          :  1;
        unsigned int ME_PARSING_PACKETS                                           :  1;
        unsigned int RCIU_PFP_BUSY                                                :  1;
        unsigned int RCIU_ME_BUSY                                                 :  1;
        unsigned int                                                              :  1;
        unsigned int SEM_CMDFIFO_NOT_EMPTY                                        :  1;
        unsigned int SEM_FAILED_AND_HOLDING                                       :  1;
        unsigned int SEM_POLLING_FOR_PASS                                         :  1;
        unsigned int GFX_CONTEXT_BUSY                                             :  1;
        unsigned int                                                              :  1;
        unsigned int ME_PARSER_BUSY                                               :  1;
        unsigned int EOP_DONE_BUSY                                                :  1;
        unsigned int STRM_OUT_BUSY                                                :  1;
        unsigned int PIPE_STATS_BUSY                                              :  1;
        unsigned int RCIU_CE_BUSY                                                 :  1;
        unsigned int CE_PARSING_PACKETS                                           :  1;
        unsigned int                                                              :  9;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CEQ1_AVAIL {
    struct {
        unsigned int CEQ_CNT_RING                                                 : 11;
        unsigned int                                                              :  5;
        unsigned int CEQ_CNT_IB1                                                  : 11;
        unsigned int                                                              :  5;
    } gfx09;
    struct {
        unsigned int CEQ_CNT_RING                                                 : 12;
        unsigned int                                                              :  4;
        unsigned int CEQ_CNT_IB1                                                  : 12;
        unsigned int                                                              :  4;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CEQ2_AVAIL {
    struct {
        unsigned int CEQ_CNT_IB2                                                  : 11;
        unsigned int                                                              : 21;
    } gfx09;
    struct {
        unsigned int CEQ_CNT_IB2                                                  : 12;
        unsigned int                                                              :  4;
        unsigned int CEQ_CNT_DB                                                   : 12;
        unsigned int                                                              :  4;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_COMPARE_COUNT {
    struct {
        unsigned int COMPARE_COUNT                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_COMPLETION_STATUS {
    struct {
        unsigned int STATUS                                                       :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_COUNTER {
    struct {
        unsigned int CONST_ENGINE_COUNT                                           : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_CS_PARTITION_INDEX {
    struct {
        unsigned int CS1_INDEX                                                    : 17;
        unsigned int                                                              : 15;
    } gfx101Plus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_DB_BASE_HI {
    struct {
        unsigned int DB_BASE_HI                                                   : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_DB_BASE_LO {
    struct {
        unsigned int                                                              :  2;
        unsigned int DB_BASE_LO                                                   : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_DB_BUFSZ {
    struct {
        unsigned int DB_BUFSZ                                                     : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_DB_CMD_BUFSZ {
    struct {
        unsigned int DB_CMD_REQSZ                                                 : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_DE_COUNT {
    struct {
        unsigned int DRAW_ENGINE_COUNT                                            : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_DOORBELL_CONTROL {
    struct {
        unsigned int                                                              :  1;
        unsigned int DOORBELL_BIF_DROP                                            :  1;
        unsigned int DOORBELL_OFFSET                                              : 26;
        unsigned int                                                              :  2;
        unsigned int DOORBELL_EN                                                  :  1;
        unsigned int DOORBELL_HIT                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_HEADER_DUMP {
    struct {
        unsigned int CE_HEADER_DUMP                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_IB1_BASE_HI {
    struct {
        unsigned int IB1_BASE_HI                                                  : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_IB1_BASE_LO {
    struct {
        unsigned int                                                              :  2;
        unsigned int IB1_BASE_LO                                                  : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_IB1_BUFSZ {
    struct {
        unsigned int IB1_BUFSZ                                                    : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_IB1_CMD_BUFSZ {
    struct {
        unsigned int IB1_CMD_REQSZ                                                : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_IB1_OFFSET {
    struct {
        unsigned int IB1_OFFSET                                                   : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_IB2_BASE_HI {
    struct {
        unsigned int IB2_BASE_HI                                                  : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_IB2_BASE_LO {
    struct {
        unsigned int                                                              :  2;
        unsigned int IB2_BASE_LO                                                  : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_IB2_BUFSZ {
    struct {
        unsigned int IB2_BUFSZ                                                    : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_IB2_CMD_BUFSZ {
    struct {
        unsigned int IB2_CMD_REQSZ                                                : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_IB2_OFFSET {
    struct {
        unsigned int IB2_OFFSET                                                   : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_IC_BASE_CNTL {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int ADDRESS_CLAMP                                                :  1;
        unsigned int                                                              : 18;
        unsigned int EXE_DISABLE                                                  :  1;
        unsigned int CACHE_POLICY                                                 :  2;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_IC_BASE_HI {
    struct {
        unsigned int IC_BASE_HI                                                   : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_IC_BASE_LO {
    struct {
        unsigned int                                                              : 12;
        unsigned int IC_BASE_LO                                                   : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_IC_OP_CNTL {
    struct {
        unsigned int INVALIDATE_CACHE                                             :  1;
        unsigned int INVALIDATE_CACHE_COMPLETE                                    :  1;
        unsigned int                                                              :  2;
        unsigned int PRIME_ICACHE                                                 :  1;
        unsigned int ICACHE_PRIMED                                                :  1;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_INIT_BASE_HI {
    struct {
        unsigned int INIT_BASE_HI                                                 : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_INIT_BASE_LO {
    struct {
        unsigned int                                                              :  5;
        unsigned int INIT_BASE_LO                                                 : 27;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_INIT_BUFSZ {
    struct {
        unsigned int INIT_BUFSZ                                                   : 12;
        unsigned int                                                              : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_INIT_CMD_BUFSZ {
    struct {
        unsigned int INIT_CMD_REQSZ                                               : 12;
        unsigned int                                                              : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_INSTR_PNTR {
    struct {
        unsigned int INSTR_PNTR                                                   : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_INTR_ROUTINE_START {
    struct {
        unsigned int IR_START                                                     : 11;
        unsigned int                                                              : 21;
    } gfx09;
    struct {
        unsigned int IR_START                                                     : 20;
        unsigned int                                                              : 12;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_JT_STAT {
    struct {
        unsigned int JT_LOADED                                                    :  2;
        unsigned int                                                              : 14;
        unsigned int WR_MASK                                                      :  2;
        unsigned int                                                              : 14;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_METADATA_BASE_ADDR {
    struct {
        unsigned int ADDR_LO                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_METADATA_BASE_ADDR_HI {
    struct {
        unsigned int ADDR_HI                                                      : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_PRGRM_CNTR_START {
    struct {
        unsigned int IP_START                                                     : 11;
        unsigned int                                                              : 21;
    } gfx09;
    struct {
        unsigned int IP_START                                                     : 20;
        unsigned int                                                              : 12;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_RB_OFFSET {
    struct {
        unsigned int RB_OFFSET                                                    : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_ROQ_DB_STAT {
    struct {
        unsigned int CEQ_RPTR_DB                                                  : 12;
        unsigned int                                                              :  4;
        unsigned int CEQ_WPTR_DB                                                  : 12;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_ROQ_IB1_STAT {
    struct {
        unsigned int CEQ_RPTR_INDIRECT1                                           : 10;
        unsigned int                                                              :  6;
        unsigned int CEQ_WPTR_INDIRECT1                                           : 10;
        unsigned int                                                              :  6;
    } gfx09;
    struct {
        unsigned int CEQ_RPTR_INDIRECT1                                           : 12;
        unsigned int                                                              :  4;
        unsigned int CEQ_WPTR_INDIRECT1                                           : 12;
        unsigned int                                                              :  4;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_ROQ_IB2_STAT {
    struct {
        unsigned int CEQ_RPTR_INDIRECT2                                           : 10;
        unsigned int                                                              :  6;
        unsigned int CEQ_WPTR_INDIRECT2                                           : 10;
        unsigned int                                                              :  6;
    } gfx09;
    struct {
        unsigned int CEQ_RPTR_INDIRECT2                                           : 12;
        unsigned int                                                              :  4;
        unsigned int CEQ_WPTR_INDIRECT2                                           : 12;
        unsigned int                                                              :  4;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CE_ROQ_RB_STAT {
    struct {
        unsigned int CEQ_RPTR_PRIMARY                                             : 10;
        unsigned int                                                              :  6;
        unsigned int CEQ_WPTR_PRIMARY                                             : 10;
        unsigned int                                                              :  6;
    } gfx09;
    struct {
        unsigned int CEQ_RPTR_PRIMARY                                             : 12;
        unsigned int                                                              :  4;
        unsigned int CEQ_WPTR_PRIMARY                                             : 12;
        unsigned int                                                              :  4;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CMD_DATA {
    struct {
        unsigned int CMD_DATA                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CMD_INDEX {
    struct {
        unsigned int CMD_INDEX                                                    : 11;
        unsigned int                                                              :  1;
        unsigned int CMD_ME_SEL                                                   :  2;
        unsigned int                                                              :  2;
        unsigned int CMD_QUEUE_SEL                                                :  3;
        unsigned int                                                              : 13;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CNTX_STAT {
    struct {
        unsigned int ACTIVE_HP3D_CONTEXTS                                         :  8;
        unsigned int CURRENT_HP3D_CONTEXT                                         :  3;
        unsigned int                                                              :  9;
        unsigned int ACTIVE_GFX_CONTEXTS                                          :  8;
        unsigned int CURRENT_GFX_CONTEXT                                          :  3;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_COHER_BASE {
    struct {
        unsigned int COHER_BASE_256B                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_COHER_BASE_HI {
    struct {
        unsigned int COHER_BASE_HI_256B                                           :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_COHER_CNTL {
    struct {
        unsigned int                                                              :  3;
        unsigned int TC_NC_ACTION_ENA                                             :  1;
        unsigned int TC_WC_ACTION_ENA                                             :  1;
        unsigned int TC_INV_METADATA_ACTION_ENA                                   :  1;
        unsigned int                                                              :  9;
        unsigned int TCL1_VOL_ACTION_ENA                                          :  1;
        unsigned int                                                              :  2;
        unsigned int TC_WB_ACTION_ENA                                             :  1;
        unsigned int                                                              :  3;
        unsigned int TCL1_ACTION_ENA                                              :  1;
        unsigned int TC_ACTION_ENA                                                :  1;
        unsigned int                                                              :  1;
        unsigned int CB_ACTION_ENA                                                :  1;
        unsigned int DB_ACTION_ENA                                                :  1;
        unsigned int SH_KCACHE_ACTION_ENA                                         :  1;
        unsigned int SH_KCACHE_VOL_ACTION_ENA                                     :  1;
        unsigned int SH_ICACHE_ACTION_ENA                                         :  1;
        unsigned int SH_KCACHE_WB_ACTION_ENA                                      :  1;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_COHER_SIZE {
    struct {
        unsigned int COHER_SIZE_256B                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_COHER_SIZE_HI {
    struct {
        unsigned int COHER_SIZE_HI_256B                                           :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_COHER_START_DELAY {
    struct {
        unsigned int START_DELAY_COUNT                                            :  6;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_COHER_STATUS {
    struct {
        unsigned int                                                              : 24;
        unsigned int MEID                                                         :  2;
        unsigned int                                                              :  5;
        unsigned int STATUS                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CONTEXT_CNTL {
    struct {
        unsigned int                                                              :  4;
        unsigned int ME0PIPE0_MAX_PIPE_CNTX                                       :  3;
        unsigned int                                                              : 13;
        unsigned int ME0PIPE1_MAX_PIPE_CNTX                                       :  3;
        unsigned int                                                              :  9;
    } bits, bitfields;
    struct {
        unsigned int ME0PIPE0_MAX_WD_CNTX                                         :  3;
        unsigned int                                                              : 13;
        unsigned int ME0PIPE1_MAX_WD_CNTX                                         :  3;
        unsigned int                                                              : 13;
    } gfx09;
    struct {
        unsigned int ME0PIPE0_MAX_GE_CNTX                                         :  3;
        unsigned int                                                              : 13;
        unsigned int ME0PIPE1_MAX_GE_CNTX                                         :  3;
        unsigned int                                                              : 13;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_CSF_STAT {
    struct {
        unsigned int                                                              :  8;
        unsigned int BUFFER_REQUEST_COUNT                                         :  9;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DB_BASE_HI {
    struct {
        unsigned int DB_BASE_HI                                                   : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DB_BASE_LO {
    struct {
        unsigned int                                                              :  2;
        unsigned int DB_BASE_LO                                                   : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DB_BUFSZ {
    struct {
        unsigned int DB_BUFSZ                                                     : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DB_CMD_BUFSZ {
    struct {
        unsigned int DB_CMD_REQSZ                                                 : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DDID_BASE_ADDR_HI {
    struct {
        unsigned int BASE_ADDR_HI                                                 : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DDID_BASE_ADDR_LO {
    struct {
        unsigned int                                                              :  6;
        unsigned int BASE_ADDR_LO                                                 : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DDID_CNTL {
    struct {
        unsigned int THRESHOLD                                                    :  8;
        unsigned int                                                              :  8;
        unsigned int SIZE                                                         :  1;
        unsigned int                                                              :  3;
        unsigned int VMID                                                         :  4;
        unsigned int VMID_SEL                                                     :  1;
        unsigned int                                                              :  3;
        unsigned int POLICY                                                       :  2;
        unsigned int MODE                                                         :  1;
        unsigned int ENABLE                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DEBUG_2 {
    struct {
        unsigned int                                                              : 17;
        unsigned int DC_INTERLEAVE_WG_SIZE_CNST                                   : 10;
        unsigned int                                                              :  1;
        unsigned int DC_FORCE_CLK_EN                                              :  1;
        unsigned int DC_DISABLE_BROADCAST                                         :  1;
        unsigned int NOT_EOP_HW_DETECT_DISABLE                                    :  1;
        unsigned int PFP_DDID_HW_DETECT_DISABLE                                   :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 15;
        unsigned int CNTX_DONE_COPY_STATE_DISABLE                                 :  1;
        unsigned int NOP_DISCARD_DISABLE                                          :  1;
        unsigned int                                                              : 15;
    } gfx101Plus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DEVICE_ID {
    struct {
        unsigned int DEVICE_ID                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DE_CE_COUNT {
    struct {
        unsigned int CONST_ENGINE_COUNT                                           : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DE_DE_COUNT {
    struct {
        unsigned int DRAW_ENGINE_COUNT                                            : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DE_LAST_INVAL_COUNT {
    struct {
        unsigned int LAST_INVAL_COUNT                                             : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DFY_ADDR_HI {
    struct {
        unsigned int ADDR_HI                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DFY_ADDR_LO {
    struct {
        unsigned int                                                              :  5;
        unsigned int ADDR_LO                                                      : 27;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DFY_CMD {
    struct {
        unsigned int OFFSET                                                       :  9;
        unsigned int                                                              :  7;
        unsigned int SIZE                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DFY_CNTL {
    struct {
        unsigned int                                                              : 26;
        unsigned int TPI_SDP_SEL                                                  :  1;
        unsigned int                                                              :  1;
        unsigned int LFSR_RESET                                                   :  1;
        unsigned int MODE                                                         :  2;
        unsigned int ENABLE                                                       :  1;
    } bits, bitfields;
    struct {
        unsigned int POLICY                                                       :  1;
        unsigned int                                                              :  1;
        unsigned int MTYPE                                                        :  2;
        unsigned int                                                              : 28;
    } gfx09;
    struct {
        unsigned int                                                              :  8;
        unsigned int POLICY                                                       :  2;
        unsigned int VOL                                                          :  1;
        unsigned int                                                              :  1;
        unsigned int MTYPE                                                        :  3;
        unsigned int                                                              : 10;
        unsigned int REPEATER_FGCG_DISABLE                                        :  1;
        unsigned int                                                              :  6;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DFY_DATA_0 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DFY_DATA_1 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DFY_DATA_2 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DFY_DATA_3 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DFY_DATA_4 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DFY_DATA_5 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DFY_DATA_6 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DFY_DATA_7 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DFY_DATA_8 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DFY_DATA_9 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DFY_DATA_10 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DFY_DATA_11 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DFY_DATA_12 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DFY_DATA_13 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DFY_DATA_14 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DFY_DATA_15 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DFY_STAT {
    struct {
        unsigned int BURST_COUNT                                                  : 16;
        unsigned int TAGS_PENDING                                                 : 11;
        unsigned int                                                              :  4;
        unsigned int BUSY                                                         :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DISPATCH_INDR_ADDR {
    struct {
        unsigned int ADDR_LO                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DISPATCH_INDR_ADDR_HI {
    struct {
        unsigned int ADDR_HI                                                      : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_CNTL {
    struct {
        unsigned int UTCL1_FAULT_CONTROL                                          :  1;
        unsigned int                                                              :  3;
        unsigned int MIN_AVAILSZ                                                  :  2;
        unsigned int                                                              : 22;
        unsigned int PIO_FIFO_EMPTY                                               :  1;
        unsigned int PIO_FIFO_FULL                                                :  1;
        unsigned int PIO_COUNT                                                    :  2;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 16;
        unsigned int BUFFER_DEPTH                                                 :  4;
        unsigned int                                                              : 12;
    } gfx09;
    struct {
        unsigned int                                                              :  1;
        unsigned int WATCH_CONTROL                                                :  1;
        unsigned int                                                              : 14;
        unsigned int BUFFER_DEPTH                                                 :  9;
        unsigned int                                                              :  7;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_ME_CMD_ADDR_HI {
    struct {
        unsigned int ADDR_HI                                                      : 16;
        unsigned int RSVD                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_ME_CMD_ADDR_LO {
    struct {
        unsigned int RSVD                                                         :  2;
        unsigned int ADDR_LO                                                      : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_ME_COMMAND {
    struct {
        unsigned int BYTE_COUNT                                                   : 26;
        unsigned int SAS                                                          :  1;
        unsigned int DAS                                                          :  1;
        unsigned int SAIC                                                         :  1;
        unsigned int DAIC                                                         :  1;
        unsigned int RAW_WAIT                                                     :  1;
        unsigned int DIS_WC                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_ME_CONTROL {
    struct {
        unsigned int                                                              : 10;
        unsigned int MEMLOG_CLEAR                                                 :  1;
        unsigned int                                                              :  9;
        unsigned int DST_SELECT                                                   :  2;
        unsigned int                                                              :  7;
        unsigned int SRC_SELECT                                                   :  2;
        unsigned int                                                              :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 13;
        unsigned int SRC_CACHE_POLICY                                             :  1;
        unsigned int                                                              : 11;
        unsigned int DST_CACHE_POLICY                                             :  1;
        unsigned int                                                              :  6;
    } gfx09;
    struct {
        unsigned int                                                              : 13;
        unsigned int SRC_CACHE_POLICY                                             :  2;
        unsigned int SRC_VOLATLE                                                  :  1;
        unsigned int                                                              :  9;
        unsigned int DST_CACHE_POLICY                                             :  2;
        unsigned int DST_VOLATLE                                                  :  1;
        unsigned int                                                              :  4;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_ME_DST_ADDR {
    struct {
        unsigned int DST_ADDR                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_ME_DST_ADDR_HI {
    struct {
        unsigned int DST_ADDR_HI                                                  : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_ME_SRC_ADDR {
    struct {
        unsigned int SRC_ADDR                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_ME_SRC_ADDR_HI {
    struct {
        unsigned int SRC_ADDR_HI                                                  : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_PFP_CMD_ADDR_HI {
    struct {
        unsigned int ADDR_HI                                                      : 16;
        unsigned int RSVD                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_PFP_CMD_ADDR_LO {
    struct {
        unsigned int RSVD                                                         :  2;
        unsigned int ADDR_LO                                                      : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_PFP_COMMAND {
    struct {
        unsigned int BYTE_COUNT                                                   : 26;
        unsigned int SAS                                                          :  1;
        unsigned int DAS                                                          :  1;
        unsigned int SAIC                                                         :  1;
        unsigned int DAIC                                                         :  1;
        unsigned int RAW_WAIT                                                     :  1;
        unsigned int DIS_WC                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_PFP_CONTROL {
    struct {
        unsigned int                                                              : 10;
        unsigned int MEMLOG_CLEAR                                                 :  1;
        unsigned int                                                              :  9;
        unsigned int DST_SELECT                                                   :  2;
        unsigned int                                                              :  7;
        unsigned int SRC_SELECT                                                   :  2;
        unsigned int                                                              :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 13;
        unsigned int SRC_CACHE_POLICY                                             :  1;
        unsigned int                                                              : 11;
        unsigned int DST_CACHE_POLICY                                             :  1;
        unsigned int                                                              :  6;
    } gfx09;
    struct {
        unsigned int                                                              : 13;
        unsigned int SRC_CACHE_POLICY                                             :  2;
        unsigned int SRC_VOLATLE                                                  :  1;
        unsigned int                                                              :  9;
        unsigned int DST_CACHE_POLICY                                             :  2;
        unsigned int DST_VOLATLE                                                  :  1;
        unsigned int                                                              :  4;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_PFP_DST_ADDR {
    struct {
        unsigned int DST_ADDR                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_PFP_DST_ADDR_HI {
    struct {
        unsigned int DST_ADDR_HI                                                  : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_PFP_SRC_ADDR {
    struct {
        unsigned int SRC_ADDR                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_PFP_SRC_ADDR_HI {
    struct {
        unsigned int SRC_ADDR_HI                                                  : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_PIO_COMMAND {
    struct {
        unsigned int BYTE_COUNT                                                   : 26;
        unsigned int SAS                                                          :  1;
        unsigned int DAS                                                          :  1;
        unsigned int SAIC                                                         :  1;
        unsigned int DAIC                                                         :  1;
        unsigned int RAW_WAIT                                                     :  1;
        unsigned int DIS_WC                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_READ_TAGS {
    struct {
        unsigned int DMA_READ_TAG                                                 : 26;
        unsigned int                                                              :  2;
        unsigned int DMA_READ_TAG_VALID                                           :  1;
        unsigned int                                                              :  3;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_WATCH0_ADDR_HI {
    struct {
        unsigned int ADDR_HI                                                      : 16;
        unsigned int RSVD                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_WATCH0_ADDR_LO {
    struct {
        unsigned int RSVD                                                         :  7;
        unsigned int ADDR_LO                                                      : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_WATCH0_CNTL {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int RSVD1                                                        :  4;
        unsigned int WATCH_READS                                                  :  1;
        unsigned int WATCH_WRITES                                                 :  1;
        unsigned int ANY_VMID                                                     :  1;
        unsigned int RSVD2                                                        : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_WATCH0_MASK {
    struct {
        unsigned int RSVD                                                         :  7;
        unsigned int MASK                                                         : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_WATCH1_ADDR_HI {
    struct {
        unsigned int ADDR_HI                                                      : 16;
        unsigned int RSVD                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_WATCH1_ADDR_LO {
    struct {
        unsigned int RSVD                                                         :  7;
        unsigned int ADDR_LO                                                      : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_WATCH1_CNTL {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int RSVD1                                                        :  4;
        unsigned int WATCH_READS                                                  :  1;
        unsigned int WATCH_WRITES                                                 :  1;
        unsigned int ANY_VMID                                                     :  1;
        unsigned int RSVD2                                                        : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_WATCH1_MASK {
    struct {
        unsigned int RSVD                                                         :  7;
        unsigned int MASK                                                         : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_WATCH2_ADDR_HI {
    struct {
        unsigned int ADDR_HI                                                      : 16;
        unsigned int RSVD                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_WATCH2_ADDR_LO {
    struct {
        unsigned int RSVD                                                         :  7;
        unsigned int ADDR_LO                                                      : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_WATCH2_CNTL {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int RSVD1                                                        :  4;
        unsigned int WATCH_READS                                                  :  1;
        unsigned int WATCH_WRITES                                                 :  1;
        unsigned int ANY_VMID                                                     :  1;
        unsigned int RSVD2                                                        : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_WATCH2_MASK {
    struct {
        unsigned int RSVD                                                         :  7;
        unsigned int MASK                                                         : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_WATCH3_ADDR_HI {
    struct {
        unsigned int ADDR_HI                                                      : 16;
        unsigned int RSVD                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_WATCH3_ADDR_LO {
    struct {
        unsigned int RSVD                                                         :  7;
        unsigned int ADDR_LO                                                      : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_WATCH3_CNTL {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int RSVD1                                                        :  4;
        unsigned int WATCH_READS                                                  :  1;
        unsigned int WATCH_WRITES                                                 :  1;
        unsigned int ANY_VMID                                                     :  1;
        unsigned int RSVD2                                                        : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_WATCH3_MASK {
    struct {
        unsigned int RSVD                                                         :  7;
        unsigned int MASK                                                         : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_WATCH_STAT {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int                                                              :  4;
        unsigned int CLIENT_ID                                                    :  3;
        unsigned int                                                              :  1;
        unsigned int PIPE                                                         :  2;
        unsigned int                                                              :  2;
        unsigned int WATCH_ID                                                     :  2;
        unsigned int                                                              :  2;
        unsigned int RD_WR                                                        :  1;
        unsigned int                                                              : 10;
        unsigned int TRAP_FLAG                                                    :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_WATCH_STAT_ADDR_HI {
    struct {
        unsigned int ADDR_HI                                                      : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_WATCH_STAT_ADDR_LO {
    struct {
        unsigned int                                                              :  2;
        unsigned int ADDR_LO                                                      : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DRAW_INDX_INDR_ADDR {
    struct {
        unsigned int ADDR_LO                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DRAW_INDX_INDR_ADDR_HI {
    struct {
        unsigned int ADDR_HI                                                      : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DRAW_OBJECT {
    struct {
        unsigned int OBJECT                                                       : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DRAW_OBJECT_COUNTER {
    struct {
        unsigned int COUNT                                                        : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DRAW_WINDOW_CNTL {
    struct {
        unsigned int DISABLE_DRAW_WINDOW_LO_MAX                                   :  1;
        unsigned int DISABLE_DRAW_WINDOW_LO_MIN                                   :  1;
        unsigned int DISABLE_DRAW_WINDOW_HI                                       :  1;
        unsigned int                                                              :  5;
        unsigned int MODE                                                         :  1;
        unsigned int                                                              : 23;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DRAW_WINDOW_HI {
    struct {
        unsigned int WINDOW_HI                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DRAW_WINDOW_LO {
    struct {
        unsigned int MIN                                                          : 16;
        unsigned int MAX                                                          : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DRAW_WINDOW_MASK_HI {
    struct {
        unsigned int WINDOW_MASK_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ECC_DMA_FIRST_OCCURRENCE {
    struct {
        unsigned int INTERFACE                                                    :  2;
        unsigned int                                                              :  2;
        unsigned int CLIENT                                                       :  4;
        unsigned int ME                                                           :  2;
        unsigned int PIPE                                                         :  2;
        unsigned int QUEUE                                                        :  3;
        unsigned int                                                              :  1;
        unsigned int VMID                                                         :  4;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ECC_FIRSTOCCURRENCE {
    struct {
        unsigned int INTERFACE                                                    :  2;
        unsigned int                                                              :  2;
        unsigned int CLIENT                                                       :  4;
        unsigned int ME                                                           :  2;
        unsigned int PIPE                                                         :  2;
        unsigned int QUEUE                                                        :  3;
        unsigned int                                                              :  1;
        unsigned int VMID                                                         :  4;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ECC_FIRSTOCCURRENCE_RING0 {
    struct {
        unsigned int OBSOLETE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ECC_FIRSTOCCURRENCE_RING1 {
    struct {
        unsigned int OBSOLETE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ECC_FIRSTOCCURRENCE_RING2 {
    struct {
        unsigned int OBSOLETE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_EDC_FUE_CNTL {
    struct {
        unsigned int CP_FUE_MASK                                                  :  1;
        unsigned int SPI_FUE_MASK                                                 :  1;
        unsigned int GDS_FUE_MASK                                                 :  1;
        unsigned int TC_RLC_FUE_MASK                                              :  1;
        unsigned int TC_CPG_FUE_MASK                                              :  1;
        unsigned int TCA_FUE_MASK                                                 :  1;
        unsigned int TCC_FUE_MASK                                                 :  1;
        unsigned int UTCL2_FUE_MASK                                               :  1;
        unsigned int                                                              :  8;
        unsigned int CP_FUE_FLAG                                                  :  1;
        unsigned int SPI_FUE_FLAG                                                 :  1;
        unsigned int GDS_FUE_FLAG                                                 :  1;
        unsigned int TC_RLC_FUE_FLAG                                              :  1;
        unsigned int TC_CPG_FUE_FLAG                                              :  1;
        unsigned int TCA_FUE_FLAG                                                 :  1;
        unsigned int TCC_FUE_FLAG                                                 :  1;
        unsigned int UTCL2_FUE_FLAG                                               :  1;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_EOPQ_WAIT_TIME {
    struct {
        unsigned int WAIT_TIME                                                    : 10;
        unsigned int SCALE_COUNT                                                  :  8;
        unsigned int                                                              : 14;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_EOP_DONE_ADDR_HI {
    struct {
        unsigned int ADDR_HI                                                      : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_EOP_DONE_ADDR_LO {
    struct {
        unsigned int                                                              :  2;
        unsigned int ADDR_LO                                                      : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_EOP_DONE_CNTX_ID {
    struct {
        unsigned int CNTX_ID                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_EOP_DONE_DATA_CNTL {
    struct {
        unsigned int                                                              : 16;
        unsigned int DST_SEL                                                      :  2;
        unsigned int                                                              :  6;
        unsigned int INT_SEL                                                      :  3;
        unsigned int                                                              :  2;
        unsigned int DATA_SEL                                                     :  3;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_EOP_DONE_DATA_HI {
    struct {
        unsigned int DATA_HI                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_EOP_DONE_DATA_LO {
    struct {
        unsigned int DATA_LO                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_EOP_DONE_DOORBELL {
    struct {
        unsigned int                                                              :  2;
        unsigned int DOORBELL_OFFSET                                              : 26;
        unsigned int                                                              :  4;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_EOP_DONE_EVENT_CNTL {
    struct {
        unsigned int                                                              : 28;
        unsigned int EXECUTE                                                      :  1;
        unsigned int                                                              :  3;
    } bits, bitfields;
    struct {
        unsigned int WBINV_TC_OP                                                  :  7;
        unsigned int                                                              :  5;
        unsigned int WBINV_ACTION_ENA                                             :  6;
        unsigned int                                                              :  7;
        unsigned int CACHE_POLICY                                                 :  1;
        unsigned int                                                              :  6;
    } gfx09;
    struct {
        unsigned int                                                              : 12;
        unsigned int GCR_CNTL                                                     : 12;
        unsigned int                                                              :  1;
        unsigned int CACHE_POLICY                                                 :  2;
        unsigned int EOP_VOLATILE                                                 :  1;
        unsigned int                                                              :  4;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_EOP_LAST_FENCE_HI {
    struct {
        unsigned int LAST_FENCE_HI                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_EOP_LAST_FENCE_LO {
    struct {
        unsigned int LAST_FENCE_LO                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_FATAL_ERROR {
    struct {
        unsigned int CPF_FATAL_ERROR                                              :  1;
        unsigned int CPG_FATAL_ERROR                                              :  1;
        unsigned int GFX_HALT_PROC                                                :  1;
        unsigned int DIS_CPG_FATAL_ERROR                                          :  1;
        unsigned int CPG_TAG_FATAL_ERROR_EN                                       :  1;
        unsigned int                                                              : 27;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_FETCHER_SOURCE {
    struct {
        unsigned int ME_SRC                                                       :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_DDID_DELTA_RPT_COUNT {
    struct {
        unsigned int COUNT                                                        :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_DDID_INFLIGHT_COUNT {
    struct {
        unsigned int COUNT                                                        : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_DDID_RPTR {
    struct {
        unsigned int COUNT                                                        : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_DDID_WPTR {
    struct {
        unsigned int COUNT                                                        : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_ERROR {
    struct {
        unsigned int EDC_ERROR_ID                                                 :  4;
        unsigned int SUA_ERROR                                                    :  1;
        unsigned int                                                              :  2;
        unsigned int SEM_UTCL1_ERROR                                              :  1;
        unsigned int QU_STRM_UTCL1_ERROR                                          :  1;
        unsigned int QU_EOP_UTCL1_ERROR                                           :  1;
        unsigned int QU_PIPE_UTCL1_ERROR                                          :  1;
        unsigned int QU_READ_UTCL1_ERROR                                          :  1;
        unsigned int SYNC_MEMRD_UTCL1_ERROR                                       :  1;
        unsigned int SYNC_MEMWR_UTCL1_ERROR                                       :  1;
        unsigned int SHADOW_UTCL1_ERROR                                           :  1;
        unsigned int APPEND_UTCL1_ERROR                                           :  1;
        unsigned int CE_DMA_UTCL1_ERROR                                           :  1;
        unsigned int PFP_VGTDMA_UTCL1_ERROR                                       :  1;
        unsigned int DMA_SRC_UTCL1_ERROR                                          :  1;
        unsigned int DMA_DST_UTCL1_ERROR                                          :  1;
        unsigned int PFP_TC_UTCL1_ERROR                                           :  1;
        unsigned int ME_TC_UTCL1_ERROR                                            :  1;
        unsigned int CE_TC_UTCL1_ERROR                                            :  1;
        unsigned int PRT_LOD_UTCL1_ERROR                                          :  1;
        unsigned int RDPTR_RPT_UTCL1_ERROR                                        :  1;
        unsigned int RB_FETCHER_UTCL1_ERROR                                       :  1;
        unsigned int I1_FETCHER_UTCL1_ERROR                                       :  1;
        unsigned int I2_FETCHER_UTCL1_ERROR                                       :  1;
        unsigned int C1_FETCHER_UTCL1_ERROR                                       :  1;
        unsigned int C2_FETCHER_UTCL1_ERROR                                       :  1;
        unsigned int ST_FETCHER_UTCL1_ERROR                                       :  1;
        unsigned int CE_INIT_UTCL1_ERROR                                          :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  5;
        unsigned int RSVD1_ERROR                                                  :  1;
        unsigned int RSVD2_ERROR                                                  :  1;
        unsigned int                                                              : 25;
    } gfx09;
    struct {
        unsigned int                                                              :  5;
        unsigned int CE_DATA_FETCHER_UTCL1_ERROR                                  :  1;
        unsigned int DATA_FETCHER_UTCL1_ERROR                                     :  1;
        unsigned int                                                              : 25;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_HPD_CONTROL0 {
    struct {
        unsigned int SUSPEND_ENABLE                                               :  1;
        unsigned int                                                              :  3;
        unsigned int PIPE_HOLDING                                                 :  1;
        unsigned int                                                              : 27;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_HPD_OSPRE_FENCE_ADDR_HI {
    struct {
        unsigned int ADDR_HI                                                      : 16;
        unsigned int RSVD                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_HPD_OSPRE_FENCE_ADDR_LO {
    struct {
        unsigned int                                                              :  2;
        unsigned int ADDR_LO                                                      : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_HPD_OSPRE_FENCE_DATA_HI {
    struct {
        unsigned int DATA_HI                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_HPD_OSPRE_FENCE_DATA_LO {
    struct {
        unsigned int DATA_LO                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_HPD_STATUS0 {
    struct {
        unsigned int QUEUE_STATE                                                  :  5;
        unsigned int MAPPED_QUEUE                                                 :  3;
        unsigned int QUEUE_AVAILABLE                                              :  8;
        unsigned int                                                              :  4;
        unsigned int FORCE_QUEUE_STATE                                            :  5;
        unsigned int                                                              :  3;
        unsigned int SUSPEND_REQ                                                  :  1;
        unsigned int                                                              :  2;
        unsigned int FORCE_QUEUE                                                  :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 16;
        unsigned int FORCE_MAPPED_QUEUE                                           :  3;
        unsigned int                                                              : 10;
        unsigned int ENABLE_OVERIDE_QUEUEID                                       :  1;
        unsigned int OVERIDE_QUEUEID                                              :  1;
        unsigned int                                                              :  1;
    } gfx101Plus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_HQD_ACTIVE {
    struct {
        unsigned int ACTIVE                                                       :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_HQD_BASE {
    struct {
        unsigned int RB_BASE                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_HQD_BASE_HI {
    struct {
        unsigned int RB_BASE_HI                                                   :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_HQD_CE_BASE {
    struct {
        unsigned int RB_BASE                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_HQD_CE_BASE_HI {
    struct {
        unsigned int RB_BASE_HI                                                   :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_HQD_CE_CNTL {
    struct {
        unsigned int RB_BUFSZ                                                     :  6;
        unsigned int                                                              :  2;
        unsigned int RB_BLKSZ                                                     :  6;
        unsigned int                                                              :  6;
        unsigned int MIN_AVAILSZ                                                  :  2;
        unsigned int MIN_IB_AVAILSZ                                               :  2;
        unsigned int CACHE_POLICY                                                 :  2;
        unsigned int RB_VOLATILE                                                  :  1;
        unsigned int RB_NO_UPDATE                                                 :  1;
        unsigned int RB_EXE                                                       :  1;
        unsigned int                                                              :  2;
        unsigned int RB_RPTR_WR_ENA                                               :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 16;
        unsigned int BUF_SWAP                                                     :  2;
        unsigned int                                                              : 14;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_HQD_CE_CSMD_RPTR {
    struct {
        unsigned int RB_RPTR                                                      : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_HQD_CE_OFFSET {
    struct {
        unsigned int RB_OFFSET                                                    : 20;
        unsigned int                                                              : 11;
        unsigned int DISABLE_RB_OFFSET                                            :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_HQD_CE_RPTR {
    struct {
        unsigned int RB_RPTR                                                      : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_HQD_CE_RPTR_ADDR {
    struct {
        unsigned int                                                              :  2;
        unsigned int RB_RPTR_ADDR                                                 : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_HQD_CE_RPTR_ADDR_HI {
    struct {
        unsigned int RB_RPTR_ADDR_HI                                              : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_HQD_CE_RPTR_WR {
    struct {
        unsigned int RB_RPTR_WR                                                   : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_HQD_CE_WPTR {
    struct {
        unsigned int RB_WPTR                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_HQD_CE_WPTR_HI {
    struct {
        unsigned int RB_WPTR                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_HQD_CE_WPTR_POLL_ADDR_HI {
    struct {
        unsigned int RB_WPTR_POLL_ADDR_HI                                         : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_HQD_CE_WPTR_POLL_ADDR_LO {
    struct {
        unsigned int                                                              :  2;
        unsigned int RB_WPTR_POLL_ADDR_LO                                         : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_HQD_CNTL {
    struct {
        unsigned int RB_BUFSZ                                                     :  6;
        unsigned int                                                              :  2;
        unsigned int RB_BLKSZ                                                     :  6;
        unsigned int                                                              :  2;
        unsigned int BUF_SWAP                                                     :  2;
        unsigned int                                                              :  2;
        unsigned int MIN_AVAILSZ                                                  :  2;
        unsigned int MIN_IB_AVAILSZ                                               :  2;
        unsigned int CACHE_POLICY                                                 :  2;
        unsigned int RB_VOLATILE                                                  :  1;
        unsigned int RB_NO_UPDATE                                                 :  1;
        unsigned int RB_EXE                                                       :  1;
        unsigned int                                                              :  2;
        unsigned int RB_RPTR_WR_ENA                                               :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 29;
        unsigned int KMD_QUEUE                                                    :  1;
        unsigned int CE_HQD_NEQ_RB_HQD                                            :  1;
        unsigned int                                                              :  1;
    } gfx101Plus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_HQD_CSMD_RPTR {
    struct {
        unsigned int RB_RPTR                                                      : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_HQD_DEQUEUE_REQUEST {
    struct {
        unsigned int DEQUEUE_REQ                                                  :  1;
        unsigned int                                                              :  3;
        unsigned int IQ_REQ_PEND                                                  :  1;
        unsigned int                                                              :  4;
        unsigned int IQ_REQ_PEND_EN                                               :  1;
        unsigned int DEQUEUE_REQ_EN                                               :  1;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_HQD_HQ_CONTROL0 {
    struct {
        unsigned int COMMAND                                                      :  4;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_HQD_HQ_STATUS0 {
    struct {
        unsigned int DEQUEUE_STATUS                                               :  1;
        unsigned int                                                              :  3;
        unsigned int OS_PREEMPT_STATUS                                            :  2;
        unsigned int PREEMPT_ACK                                                  :  1;
        unsigned int                                                              : 23;
        unsigned int QUEUE_IDLE                                                   :  1;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_HQD_MAPPED {
    struct {
        unsigned int MAPPED                                                       :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_HQD_OFFSET {
    struct {
        unsigned int RB_OFFSET                                                    : 20;
        unsigned int                                                              : 11;
        unsigned int DISABLE_RB_OFFSET                                            :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_HQD_QUANTUM {
    struct {
        unsigned int QUANTUM_EN                                                   :  1;
        unsigned int                                                              :  2;
        unsigned int QUANTUM_SCALE                                                :  2;
        unsigned int                                                              :  3;
        unsigned int QUANTUM_DURATION                                             :  8;
        unsigned int                                                              : 15;
        unsigned int QUANTUM_ACTIVE                                               :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_HQD_QUEUE_PRIORITY {
    struct {
        unsigned int PRIORITY_LEVEL                                               :  4;
        unsigned int                                                              : 28;
    } gfx101Plus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_HQD_QUE_MGR_CONTROL {
    struct {
        unsigned int CONTROL                                                      : 24;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_HQD_RPTR {
    struct {
        unsigned int RB_RPTR                                                      : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_HQD_RPTR_ADDR {
    struct {
        unsigned int                                                              :  2;
        unsigned int RB_RPTR_ADDR                                                 : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_HQD_RPTR_ADDR_HI {
    struct {
        unsigned int RB_RPTR_ADDR_HI                                              : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_HQD_VMID {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_HQD_WPTR {
    struct {
        unsigned int RB_WPTR                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_HQD_WPTR_HI {
    struct {
        unsigned int RB_WPTR                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_INDEX_MUTEX {
    struct {
        unsigned int REQUEST                                                      :  1;
        unsigned int CLIENTID                                                     :  3;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_MQD_BASE_ADDR {
    struct {
        unsigned int                                                              :  2;
        unsigned int BASE_ADDR                                                    : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_MQD_BASE_ADDR_HI {
    struct {
        unsigned int BASE_ADDR_HI                                                 : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 28;
        unsigned int APP_VMID                                                     :  4;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_MQD_CONTROL {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int                                                              :  4;
        unsigned int PRIV_STATE                                                   :  1;
        unsigned int                                                              : 14;
        unsigned int EXE_DISABLE                                                  :  1;
        unsigned int                                                              :  8;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 24;
        unsigned int CACHE_POLICY                                                 :  1;
        unsigned int                                                              :  7;
    } gfx09;
    struct {
        unsigned int                                                              : 12;
        unsigned int PROCESSING_MQD                                               :  1;
        unsigned int PROCESSING_MQD_EN                                            :  1;
        unsigned int                                                              : 10;
        unsigned int CACHE_POLICY                                                 :  2;
        unsigned int                                                              :  6;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_QUEUE_INDEX {
    struct {
        unsigned int QUEUE_ACCESS                                                 :  1;
        unsigned int                                                              :  3;
        unsigned int PIPE_ID                                                      :  2;
        unsigned int                                                              :  2;
        unsigned int QUEUE_ID                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GRBM_FREE_COUNT {
    struct {
        unsigned int FREE_COUNT                                                   :  6;
        unsigned int                                                              :  2;
        unsigned int FREE_COUNT_GDS                                               :  6;
        unsigned int                                                              :  2;
        unsigned int FREE_COUNT_PFP                                               :  6;
        unsigned int                                                              : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HPD_MES_ROQ_OFFSETS {
    struct {
        unsigned int IQ_OFFSET                                                    :  3;
        unsigned int                                                              :  5;
        unsigned int PQ_OFFSET                                                    :  6;
        unsigned int                                                              :  2;
        unsigned int IB_OFFSET                                                    :  7;
        unsigned int                                                              :  9;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HPD_ROQ_OFFSETS {
    struct {
        unsigned int IQ_OFFSET                                                    :  3;
        unsigned int                                                              :  5;
        unsigned int PQ_OFFSET                                                    :  6;
        unsigned int                                                              : 18;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 16;
        unsigned int IB_OFFSET                                                    :  6;
        unsigned int                                                              : 10;
    } gfx09;
    struct {
        unsigned int                                                              : 16;
        unsigned int IB_OFFSET                                                    :  7;
        unsigned int                                                              :  9;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HPD_STATUS0 {
    struct {
        unsigned int QUEUE_STATE                                                  :  5;
        unsigned int MAPPED_QUEUE                                                 :  3;
        unsigned int QUEUE_AVAILABLE                                              :  8;
        unsigned int FETCHING_MQD                                                 :  1;
        unsigned int PEND_TXFER_SIZE_PQIB                                         :  1;
        unsigned int PEND_TXFER_SIZE_IQ                                           :  1;
        unsigned int                                                              :  1;
        unsigned int FORCE_QUEUE_STATE                                            :  5;
        unsigned int                                                              :  6;
        unsigned int FORCE_QUEUE                                                  :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 27;
        unsigned int MASTER_QUEUE_IDLE_DIS                                        :  1;
        unsigned int ENABLE_OFFLOAD_CHECK                                         :  2;
        unsigned int FREEZE_QUEUE_STATE                                           :  1;
        unsigned int                                                              :  1;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HPD_UTCL1_CNTL {
    struct {
        unsigned int SELECT                                                       :  4;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HPD_UTCL1_ERROR {
    struct {
        unsigned int ADDR_HI                                                      : 16;
        unsigned int TYPE                                                         :  1;
        unsigned int                                                              :  3;
        unsigned int VMID                                                         :  4;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HPD_UTCL1_ERROR_ADDR {
    struct {
        unsigned int                                                              : 12;
        unsigned int ADDR                                                         : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_ACTIVE {
    struct {
        unsigned int ACTIVE                                                       :  1;
        unsigned int BUSY_GATE                                                    :  1;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_AQL_CONTROL {
    struct {
        unsigned int CONTROL0                                                     : 15;
        unsigned int CONTROL0_EN                                                  :  1;
        unsigned int CONTROL1                                                     : 15;
        unsigned int CONTROL1_EN                                                  :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_ATOMIC0_PREOP_HI {
    struct {
        unsigned int ATOMIC0_PREOP_HI                                             : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_ATOMIC0_PREOP_LO {
    struct {
        unsigned int ATOMIC0_PREOP_LO                                             : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_ATOMIC1_PREOP_HI {
    struct {
        unsigned int ATOMIC1_PREOP_HI                                             : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_ATOMIC1_PREOP_LO {
    struct {
        unsigned int ATOMIC1_PREOP_LO                                             : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_CNTL_STACK_OFFSET {
    struct {
        unsigned int                                                              :  2;
        unsigned int OFFSET                                                       : 13;
        unsigned int                                                              : 17;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_CNTL_STACK_SIZE {
    struct {
        unsigned int                                                              : 12;
        unsigned int SIZE                                                         :  3;
        unsigned int                                                              : 17;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_CTX_SAVE_BASE_ADDR_HI {
    struct {
        unsigned int ADDR_HI                                                      : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_CTX_SAVE_BASE_ADDR_LO {
    struct {
        unsigned int                                                              : 12;
        unsigned int ADDR                                                         : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_CTX_SAVE_CONTROL {
    struct {
        unsigned int                                                              : 23;
        unsigned int EXE_DISABLE                                                  :  1;
        unsigned int                                                              :  8;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  3;
        unsigned int POLICY                                                       :  1;
        unsigned int                                                              : 28;
    } gfx09;
    struct {
        unsigned int                                                              :  3;
        unsigned int POLICY                                                       :  2;
        unsigned int                                                              : 27;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_CTX_SAVE_SIZE {
    struct {
        unsigned int                                                              : 12;
        unsigned int SIZE                                                         : 13;
        unsigned int                                                              :  7;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_DDID_DELTA_RPT_COUNT {
    struct {
        unsigned int COUNT                                                        :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_DDID_INFLIGHT_COUNT {
    struct {
        unsigned int COUNT                                                        : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_DDID_RPTR {
    struct {
        unsigned int RPTR                                                         : 11;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_DDID_WPTR {
    struct {
        unsigned int WPTR                                                         : 11;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_DEQUEUE_REQUEST {
    struct {
        unsigned int                                                              :  4;
        unsigned int IQ_REQ_PEND                                                  :  1;
        unsigned int                                                              :  3;
        unsigned int DEQUEUE_INT                                                  :  1;
        unsigned int IQ_REQ_PEND_EN                                               :  1;
        unsigned int DEQUEUE_REQ_EN                                               :  1;
        unsigned int                                                              : 21;
    } bits, bitfields;
    struct {
        unsigned int DEQUEUE_REQ                                                  :  3;
        unsigned int                                                              : 29;
    } gfx09;
    struct {
        unsigned int DEQUEUE_REQ                                                  :  4;
        unsigned int                                                              : 28;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_DEQUEUE_STATUS {
    struct {
        unsigned int DEQUEUE_STAT                                                 :  4;
        unsigned int SUSPEND_REQ_PEND                                             :  1;
        unsigned int                                                              :  4;
        unsigned int SUSPEND_REQ_PEND_EN                                          :  1;
        unsigned int DEQUEUE_STAT_EN                                              :  1;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_DMA_OFFLOAD {
    struct {
        unsigned int DMA_OFFLOAD                                                  :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_EOP_BASE_ADDR {
    struct {
        unsigned int BASE_ADDR                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_EOP_BASE_ADDR_HI {
    struct {
        unsigned int BASE_ADDR_HI                                                 :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_EOP_CONTROL {
    struct {
        unsigned int EOP_SIZE                                                     :  6;
        unsigned int                                                              :  2;
        unsigned int PROCESSING_EOP                                               :  1;
        unsigned int                                                              :  3;
        unsigned int PROCESS_EOP_EN                                               :  1;
        unsigned int PROCESSING_EOPIB                                             :  1;
        unsigned int PROCESS_EOPIB_EN                                             :  1;
        unsigned int                                                              :  6;
        unsigned int HALT_FETCHER                                                 :  1;
        unsigned int HALT_FETCHER_EN                                              :  1;
        unsigned int EXE_DISABLE                                                  :  1;
        unsigned int                                                              :  5;
        unsigned int SIG_SEM_RESULT                                               :  2;
        unsigned int PEND_SIG_SEM                                                 :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 24;
        unsigned int CACHE_POLICY                                                 :  1;
        unsigned int                                                              :  7;
    } gfx09;
    struct {
        unsigned int                                                              : 24;
        unsigned int CACHE_POLICY                                                 :  2;
        unsigned int EOP_VOLATILE                                                 :  1;
        unsigned int                                                              :  5;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_EOP_EVENTS {
    struct {
        unsigned int EVENT_COUNT                                                  : 12;
        unsigned int                                                              :  4;
        unsigned int CS_PARTIAL_FLUSH_PEND                                        :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_EOP_RPTR {
    struct {
        unsigned int RPTR                                                         : 13;
        unsigned int                                                              : 15;
        unsigned int RESET_FETCHER                                                :  1;
        unsigned int DEQUEUE_PEND                                                 :  1;
        unsigned int RPTR_EQ_CSMD_WPTR                                            :  1;
        unsigned int INIT_FETCHER                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_EOP_WPTR {
    struct {
        unsigned int WPTR                                                         : 13;
        unsigned int                                                              :  2;
        unsigned int EOP_EMPTY                                                    :  1;
        unsigned int EOP_AVAIL                                                    : 13;
        unsigned int                                                              :  3;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_EOP_WPTR_MEM {
    struct {
        unsigned int WPTR                                                         : 13;
        unsigned int                                                              : 19;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_ERROR {
    struct {
        unsigned int EDC_ERROR_ID                                                 :  4;
        unsigned int SUA_ERROR                                                    :  1;
        unsigned int AQL_ERROR                                                    :  1;
        unsigned int                                                              :  2;
        unsigned int PQ_UTCL1_ERROR                                               :  1;
        unsigned int IB_UTCL1_ERROR                                               :  1;
        unsigned int EOP_UTCL1_ERROR                                              :  1;
        unsigned int IQ_UTCL1_ERROR                                               :  1;
        unsigned int RRPT_UTCL1_ERROR                                             :  1;
        unsigned int WPP_UTCL1_ERROR                                              :  1;
        unsigned int SEM_UTCL1_ERROR                                              :  1;
        unsigned int DMA_SRC_UTCL1_ERROR                                          :  1;
        unsigned int DMA_DST_UTCL1_ERROR                                          :  1;
        unsigned int SR_UTCL1_ERROR                                               :  1;
        unsigned int QU_UTCL1_ERROR                                               :  1;
        unsigned int TC_UTCL1_ERROR                                               :  1;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_GFX_CONTROL {
    struct {
        unsigned int MESSAGE                                                      :  4;
        unsigned int MISC                                                         : 11;
        unsigned int DB_UPDATED_MSG_EN                                            :  1;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_GFX_STATUS {
    struct {
        unsigned int STATUS                                                       : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_HQ_CONTROL0 {
    struct {
        unsigned int CONTROL                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_HQ_CONTROL1 {
    struct {
        unsigned int CONTROL                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_HQ_SCHEDULER0 {
    struct {
        unsigned int SCHEDULER                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_HQ_SCHEDULER1 {
    struct {
        unsigned int SCHEDULER                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_HQ_STATUS0 {
    struct {
        unsigned int DEQUEUE_STATUS                                               :  2;
        unsigned int DEQUEUE_RETRY_CNT                                            :  2;
        unsigned int RSV_6_4                                                      :  3;
        unsigned int SCRATCH_RAM_INIT                                             :  1;
        unsigned int TCL2_DIRTY                                                   :  1;
        unsigned int PG_ACTIVATED                                                 :  1;
        unsigned int RSVR_29_10                                                   : 20;
        unsigned int QUEUE_IDLE                                                   :  1;
        unsigned int DB_UPDATED_MSG_EN                                            :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_HQ_STATUS1 {
    struct {
        unsigned int STATUS                                                       : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_IB_BASE_ADDR {
    struct {
        unsigned int                                                              :  2;
        unsigned int IB_BASE_ADDR                                                 : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_IB_BASE_ADDR_HI {
    struct {
        unsigned int IB_BASE_ADDR_HI                                              : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_IB_CONTROL {
    struct {
        unsigned int IB_SIZE                                                      : 20;
        unsigned int MIN_IB_AVAIL_SIZE                                            :  2;
        unsigned int                                                              :  1;
        unsigned int IB_EXE_DISABLE                                               :  1;
        unsigned int                                                              :  7;
        unsigned int PROCESSING_IB                                                :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 24;
        unsigned int IB_CACHE_POLICY                                              :  1;
        unsigned int                                                              :  7;
    } gfx09;
    struct {
        unsigned int                                                              : 24;
        unsigned int IB_CACHE_POLICY                                              :  2;
        unsigned int IB_VOLATILE                                                  :  1;
        unsigned int                                                              :  5;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_IB_RPTR {
    struct {
        unsigned int CONSUMED_OFFSET                                              : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_IQ_RPTR {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_IQ_TIMER {
    struct {
        unsigned int WAIT_TIME                                                    :  8;
        unsigned int RETRY_TYPE                                                   :  3;
        unsigned int IMMEDIATE_EXPIRE                                             :  1;
        unsigned int INTERRUPT_TYPE                                               :  2;
        unsigned int CLOCK_COUNT                                                  :  2;
        unsigned int INTERRUPT_SIZE                                               :  6;
        unsigned int QUANTUM_TIMER                                                :  1;
        unsigned int EXE_DISABLE                                                  :  1;
        unsigned int                                                              :  4;
        unsigned int REARM_TIMER                                                  :  1;
        unsigned int PROCESS_IQ_EN                                                :  1;
        unsigned int PROCESSING_IQ                                                :  1;
        unsigned int ACTIVE                                                       :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 24;
        unsigned int CACHE_POLICY                                                 :  1;
        unsigned int QUEUE_TYPE                                                   :  1;
        unsigned int                                                              :  6;
    } gfx09;
    struct {
        unsigned int                                                              : 24;
        unsigned int CACHE_POLICY                                                 :  2;
        unsigned int IQ_VOLATILE                                                  :  1;
        unsigned int QUEUE_TYPE                                                   :  1;
        unsigned int                                                              :  4;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_MSG_TYPE {
    struct {
        unsigned int ACTION                                                       :  3;
        unsigned int                                                              :  1;
        unsigned int SAVE_STATE                                                   :  3;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_OFFLOAD {
    struct {
        unsigned int DMA_OFFLOAD                                                  :  1;
        unsigned int DMA_OFFLOAD_EN                                               :  1;
        unsigned int AQL_OFFLOAD                                                  :  1;
        unsigned int AQL_OFFLOAD_EN                                               :  1;
        unsigned int EOP_OFFLOAD                                                  :  1;
        unsigned int EOP_OFFLOAD_EN                                               :  1;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_PERSISTENT_STATE {
    struct {
        unsigned int PRELOAD_REQ                                                  :  1;
        unsigned int                                                              :  7;
        unsigned int PRELOAD_SIZE                                                 : 10;
        unsigned int                                                              :  3;
        unsigned int WPP_SWITCH_QOS_EN                                            :  1;
        unsigned int IQ_SWITCH_QOS_EN                                             :  1;
        unsigned int IB_SWITCH_QOS_EN                                             :  1;
        unsigned int EOP_SWITCH_QOS_EN                                            :  1;
        unsigned int PQ_SWITCH_QOS_EN                                             :  1;
        unsigned int TC_OFFLOAD_QOS_EN                                            :  1;
        unsigned int CACHE_FULL_PACKET_EN                                         :  1;
        unsigned int RESTORE_ACTIVE                                               :  1;
        unsigned int RELAUNCH_WAVES                                               :  1;
        unsigned int QSWITCH_MODE                                                 :  1;
        unsigned int DISP_ACTIVE                                                  :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  7;
        unsigned int SUSPEND_STATUS                                               :  1;
        unsigned int                                                              : 12;
        unsigned int WPP_CLAMP_EN                                                 :  1;
        unsigned int                                                              : 11;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_PIPE_PRIORITY {
    struct {
        unsigned int PIPE_PRIORITY                                                :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_PQ_BASE {
    struct {
        unsigned int ADDR                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_PQ_BASE_HI {
    struct {
        unsigned int ADDR_HI                                                      :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_PQ_CONTROL {
    struct {
        unsigned int QUEUE_SIZE                                                   :  6;
        unsigned int WPTR_CARRY                                                   :  1;
        unsigned int RPTR_CARRY                                                   :  1;
        unsigned int RPTR_BLOCK_SIZE                                              :  6;
        unsigned int QUEUE_FULL_EN                                                :  1;
        unsigned int PQ_EMPTY                                                     :  1;
        unsigned int                                                              :  4;
        unsigned int MIN_AVAIL_SIZE                                               :  2;
        unsigned int                                                              :  1;
        unsigned int EXE_DISABLE                                                  :  1;
        unsigned int                                                              :  3;
        unsigned int NO_UPDATE_RPTR                                               :  1;
        unsigned int UNORD_DISPATCH                                               :  1;
        unsigned int                                                              :  1;
        unsigned int PRIV_STATE                                                   :  1;
        unsigned int KMD_QUEUE                                                    :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 16;
        unsigned int WPP_CLAMP_EN                                                 :  1;
        unsigned int ENDIAN_SWAP                                                  :  2;
        unsigned int                                                              :  5;
        unsigned int CACHE_POLICY                                                 :  1;
        unsigned int SLOT_BASED_WPTR                                              :  2;
        unsigned int                                                              :  2;
        unsigned int ROQ_PQ_IB_FLIP                                               :  1;
        unsigned int                                                              :  2;
    } gfx09;
    struct {
        unsigned int                                                              : 18;
        unsigned int SLOT_BASED_WPTR                                              :  2;
        unsigned int                                                              :  4;
        unsigned int CACHE_POLICY                                                 :  2;
        unsigned int PQ_VOLATILE                                                  :  1;
        unsigned int                                                              :  2;
        unsigned int TUNNEL_DISPATCH                                              :  1;
        unsigned int                                                              :  2;
    } gfx10;
    struct {
        unsigned int                                                              : 16;
        unsigned int ENDIAN_SWAP                                                  :  2;
        unsigned int                                                              : 14;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_PQ_DOORBELL_CONTROL {
    struct {
        unsigned int DOORBELL_MODE                                                :  1;
        unsigned int DOORBELL_BIF_DROP                                            :  1;
        unsigned int DOORBELL_OFFSET                                              : 26;
        unsigned int DOORBELL_SOURCE                                              :  1;
        unsigned int DOORBELL_SCHD_HIT                                            :  1;
        unsigned int DOORBELL_EN                                                  :  1;
        unsigned int DOORBELL_HIT                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_PQ_RPTR {
    struct {
        unsigned int CONSUMED_OFFSET                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_PQ_RPTR_REPORT_ADDR {
    struct {
        unsigned int                                                              :  2;
        unsigned int RPTR_REPORT_ADDR                                             : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_PQ_RPTR_REPORT_ADDR_HI {
    struct {
        unsigned int RPTR_REPORT_ADDR_HI                                          : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_PQ_WPTR_HI {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_PQ_WPTR_LO {
    struct {
        unsigned int OFFSET                                                       : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_PQ_WPTR_POLL_ADDR {
    struct {
        unsigned int                                                              :  3;
        unsigned int WPTR_ADDR                                                    : 29;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_PQ_WPTR_POLL_ADDR_HI {
    struct {
        unsigned int WPTR_ADDR_HI                                                 : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_QUANTUM {
    struct {
        unsigned int QUANTUM_EN                                                   :  1;
        unsigned int                                                              :  3;
        unsigned int QUANTUM_SCALE                                                :  1;
        unsigned int                                                              :  3;
        unsigned int QUANTUM_DURATION                                             :  6;
        unsigned int                                                              : 17;
        unsigned int QUANTUM_ACTIVE                                               :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_QUEUE_PRIORITY {
    struct {
        unsigned int PRIORITY_LEVEL                                               :  4;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_SEMA_CMD {
    struct {
        unsigned int RETRY                                                        :  1;
        unsigned int RESULT                                                       :  2;
        unsigned int                                                              : 29;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  8;
        unsigned int POLLING_DIS                                                  :  1;
        unsigned int MESSAGE_EN                                                   :  1;
        unsigned int                                                              : 22;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_SUSPEND_CNTL_STACK_DW_CNT {
    struct {
        unsigned int CNT                                                          : 13;
        unsigned int                                                              : 19;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_SUSPEND_CNTL_STACK_OFFSET {
    struct {
        unsigned int                                                              :  2;
        unsigned int OFFSET                                                       : 13;
        unsigned int                                                              : 17;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_SUSPEND_WG_STATE_OFFSET {
    struct {
        unsigned int                                                              :  2;
        unsigned int OFFSET                                                       : 23;
        unsigned int                                                              :  7;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_VMID {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int                                                              :  4;
        unsigned int IB_VMID                                                      :  4;
        unsigned int                                                              :  4;
        unsigned int VQID                                                         : 10;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_WG_STATE_OFFSET {
    struct {
        unsigned int                                                              :  2;
        unsigned int OFFSET                                                       : 23;
        unsigned int                                                              :  7;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_IB1_BASE_HI {
    struct {
        unsigned int IB1_BASE_HI                                                  : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_IB1_BASE_LO {
    struct {
        unsigned int                                                              :  2;
        unsigned int IB1_BASE_LO                                                  : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_IB1_BUFSZ {
    struct {
        unsigned int IB1_BUFSZ                                                    : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_IB1_CMD_BUFSZ {
    struct {
        unsigned int IB1_CMD_REQSZ                                                : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_IB1_OFFSET {
    struct {
        unsigned int IB1_OFFSET                                                   : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_IB1_PREAMBLE_BEGIN {
    struct {
        unsigned int IB1_PREAMBLE_BEGIN                                           : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_IB1_PREAMBLE_END {
    struct {
        unsigned int IB1_PREAMBLE_END                                             : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_IB2_BASE_HI {
    struct {
        unsigned int IB2_BASE_HI                                                  : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_IB2_BASE_LO {
    struct {
        unsigned int                                                              :  2;
        unsigned int IB2_BASE_LO                                                  : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_IB2_BUFSZ {
    struct {
        unsigned int IB2_BUFSZ                                                    : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_IB2_CMD_BUFSZ {
    struct {
        unsigned int IB2_CMD_REQSZ                                                : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_IB2_OFFSET {
    struct {
        unsigned int IB2_OFFSET                                                   : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_IB2_PREAMBLE_BEGIN {
    struct {
        unsigned int IB2_PREAMBLE_BEGIN                                           : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_IB2_PREAMBLE_END {
    struct {
        unsigned int IB2_PREAMBLE_END                                             : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_INDEX_BASE_ADDR {
    struct {
        unsigned int ADDR_LO                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_INDEX_BASE_ADDR_HI {
    struct {
        unsigned int ADDR_HI                                                      : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_INDEX_TYPE {
    struct {
        unsigned int INDEX_TYPE                                                   :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_INT_CNTL {
    struct {
        unsigned int                                                              : 11;
        unsigned int CP_VM_DOORBELL_WR_INT_ENABLE                                 :  1;
        unsigned int                                                              :  2;
        unsigned int CP_ECC_ERROR_INT_ENABLE                                      :  1;
        unsigned int                                                              :  1;
        unsigned int GPF_INT_ENABLE                                               :  1;
        unsigned int WRM_POLL_TIMEOUT_INT_ENABLE                                  :  1;
        unsigned int CMP_BUSY_INT_ENABLE                                          :  1;
        unsigned int CNTX_BUSY_INT_ENABLE                                         :  1;
        unsigned int CNTX_EMPTY_INT_ENABLE                                        :  1;
        unsigned int GFX_IDLE_INT_ENABLE                                          :  1;
        unsigned int PRIV_INSTR_INT_ENABLE                                        :  1;
        unsigned int PRIV_REG_INT_ENABLE                                          :  1;
        unsigned int OPCODE_ERROR_INT_ENABLE                                      :  1;
        unsigned int                                                              :  1;
        unsigned int TIME_STAMP_INT_ENABLE                                        :  1;
        unsigned int RESERVED_BIT_ERROR_INT_ENABLE                                :  1;
        unsigned int                                                              :  1;
        unsigned int GENERIC2_INT_ENABLE                                          :  1;
        unsigned int GENERIC1_INT_ENABLE                                          :  1;
        unsigned int GENERIC0_INT_ENABLE                                          :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  8;
        unsigned int RESUME_INT_ENABLE                                            :  1;
        unsigned int SUSPEND_INT_ENABLE                                           :  1;
        unsigned int DMA_WATCH_INT_ENABLE                                         :  1;
        unsigned int                                                              : 21;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_INT_CNTL_RING0 {
    struct {
        unsigned int                                                              : 11;
        unsigned int CP_VM_DOORBELL_WR_INT_ENABLE                                 :  1;
        unsigned int                                                              :  2;
        unsigned int CP_ECC_ERROR_INT_ENABLE                                      :  1;
        unsigned int                                                              :  1;
        unsigned int GPF_INT_ENABLE                                               :  1;
        unsigned int WRM_POLL_TIMEOUT_INT_ENABLE                                  :  1;
        unsigned int CMP_BUSY_INT_ENABLE                                          :  1;
        unsigned int CNTX_BUSY_INT_ENABLE                                         :  1;
        unsigned int CNTX_EMPTY_INT_ENABLE                                        :  1;
        unsigned int GFX_IDLE_INT_ENABLE                                          :  1;
        unsigned int PRIV_INSTR_INT_ENABLE                                        :  1;
        unsigned int PRIV_REG_INT_ENABLE                                          :  1;
        unsigned int OPCODE_ERROR_INT_ENABLE                                      :  1;
        unsigned int                                                              :  1;
        unsigned int TIME_STAMP_INT_ENABLE                                        :  1;
        unsigned int RESERVED_BIT_ERROR_INT_ENABLE                                :  1;
        unsigned int                                                              :  1;
        unsigned int GENERIC2_INT_ENABLE                                          :  1;
        unsigned int GENERIC1_INT_ENABLE                                          :  1;
        unsigned int GENERIC0_INT_ENABLE                                          :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  8;
        unsigned int RESUME_INT_ENABLE                                            :  1;
        unsigned int SUSPEND_INT_ENABLE                                           :  1;
        unsigned int DMA_WATCH_INT_ENABLE                                         :  1;
        unsigned int                                                              : 21;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_INT_CNTL_RING1 {
    struct {
        unsigned int                                                              : 11;
        unsigned int CP_VM_DOORBELL_WR_INT_ENABLE                                 :  1;
        unsigned int                                                              :  2;
        unsigned int CP_ECC_ERROR_INT_ENABLE                                      :  1;
        unsigned int                                                              :  1;
        unsigned int GPF_INT_ENABLE                                               :  1;
        unsigned int WRM_POLL_TIMEOUT_INT_ENABLE                                  :  1;
        unsigned int CMP_BUSY_INT_ENABLE                                          :  1;
        unsigned int CNTX_BUSY_INT_ENABLE                                         :  1;
        unsigned int CNTX_EMPTY_INT_ENABLE                                        :  1;
        unsigned int GFX_IDLE_INT_ENABLE                                          :  1;
        unsigned int PRIV_INSTR_INT_ENABLE                                        :  1;
        unsigned int PRIV_REG_INT_ENABLE                                          :  1;
        unsigned int OPCODE_ERROR_INT_ENABLE                                      :  1;
        unsigned int                                                              :  1;
        unsigned int TIME_STAMP_INT_ENABLE                                        :  1;
        unsigned int RESERVED_BIT_ERROR_INT_ENABLE                                :  1;
        unsigned int                                                              :  1;
        unsigned int GENERIC2_INT_ENABLE                                          :  1;
        unsigned int GENERIC1_INT_ENABLE                                          :  1;
        unsigned int GENERIC0_INT_ENABLE                                          :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 10;
        unsigned int DMA_WATCH_INT_ENABLE                                         :  1;
        unsigned int                                                              : 21;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_INT_CNTL_RING2 {
    struct {
        unsigned int                                                              : 11;
        unsigned int CP_VM_DOORBELL_WR_INT_ENABLE                                 :  1;
        unsigned int                                                              :  2;
        unsigned int CP_ECC_ERROR_INT_ENABLE                                      :  1;
        unsigned int                                                              :  1;
        unsigned int GPF_INT_ENABLE                                               :  1;
        unsigned int WRM_POLL_TIMEOUT_INT_ENABLE                                  :  1;
        unsigned int CMP_BUSY_INT_ENABLE                                          :  1;
        unsigned int CNTX_BUSY_INT_ENABLE                                         :  1;
        unsigned int CNTX_EMPTY_INT_ENABLE                                        :  1;
        unsigned int GFX_IDLE_INT_ENABLE                                          :  1;
        unsigned int PRIV_INSTR_INT_ENABLE                                        :  1;
        unsigned int PRIV_REG_INT_ENABLE                                          :  1;
        unsigned int OPCODE_ERROR_INT_ENABLE                                      :  1;
        unsigned int                                                              :  1;
        unsigned int TIME_STAMP_INT_ENABLE                                        :  1;
        unsigned int RESERVED_BIT_ERROR_INT_ENABLE                                :  1;
        unsigned int                                                              :  1;
        unsigned int GENERIC2_INT_ENABLE                                          :  1;
        unsigned int GENERIC1_INT_ENABLE                                          :  1;
        unsigned int GENERIC0_INT_ENABLE                                          :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 10;
        unsigned int DMA_WATCH_INT_ENABLE                                         :  1;
        unsigned int                                                              : 21;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_INT_STATUS {
    struct {
        unsigned int                                                              : 11;
        unsigned int CP_VM_DOORBELL_WR_INT_STAT                                   :  1;
        unsigned int                                                              :  2;
        unsigned int CP_ECC_ERROR_INT_STAT                                        :  1;
        unsigned int                                                              :  1;
        unsigned int GPF_INT_STAT                                                 :  1;
        unsigned int WRM_POLL_TIMEOUT_INT_STAT                                    :  1;
        unsigned int CMP_BUSY_INT_STAT                                            :  1;
        unsigned int CNTX_BUSY_INT_STAT                                           :  1;
        unsigned int CNTX_EMPTY_INT_STAT                                          :  1;
        unsigned int GFX_IDLE_INT_STAT                                            :  1;
        unsigned int PRIV_INSTR_INT_STAT                                          :  1;
        unsigned int PRIV_REG_INT_STAT                                            :  1;
        unsigned int OPCODE_ERROR_INT_STAT                                        :  1;
        unsigned int                                                              :  1;
        unsigned int TIME_STAMP_INT_STAT                                          :  1;
        unsigned int RESERVED_BIT_ERROR_INT_STAT                                  :  1;
        unsigned int                                                              :  1;
        unsigned int GENERIC2_INT_STAT                                            :  1;
        unsigned int GENERIC1_INT_STAT                                            :  1;
        unsigned int GENERIC0_INT_STAT                                            :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  8;
        unsigned int RESUME_INT_STAT                                              :  1;
        unsigned int SUSPEND_INT_STAT                                             :  1;
        unsigned int DMA_WATCH_INT_STAT                                           :  1;
        unsigned int                                                              : 21;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_INT_STATUS_RING0 {
    struct {
        unsigned int                                                              : 11;
        unsigned int CP_VM_DOORBELL_WR_INT_STAT                                   :  1;
        unsigned int                                                              :  2;
        unsigned int CP_ECC_ERROR_INT_STAT                                        :  1;
        unsigned int                                                              :  1;
        unsigned int GPF_INT_STAT                                                 :  1;
        unsigned int WRM_POLL_TIMEOUT_INT_STAT                                    :  1;
        unsigned int CMP_BUSY_INT_STAT                                            :  1;
        unsigned int GCNTX_BUSY_INT_STAT                                          :  1;
        unsigned int CNTX_EMPTY_INT_STAT                                          :  1;
        unsigned int GFX_IDLE_INT_STAT                                            :  1;
        unsigned int PRIV_INSTR_INT_STAT                                          :  1;
        unsigned int PRIV_REG_INT_STAT                                            :  1;
        unsigned int OPCODE_ERROR_INT_STAT                                        :  1;
        unsigned int                                                              :  1;
        unsigned int TIME_STAMP_INT_STAT                                          :  1;
        unsigned int RESERVED_BIT_ERROR_INT_STAT                                  :  1;
        unsigned int                                                              :  1;
        unsigned int GENERIC2_INT_STAT                                            :  1;
        unsigned int GENERIC1_INT_STAT                                            :  1;
        unsigned int GENERIC0_INT_STAT                                            :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  8;
        unsigned int RESUME_INT_STAT                                              :  1;
        unsigned int SUSPEND_INT_STAT                                             :  1;
        unsigned int DMA_WATCH_INT_STAT                                           :  1;
        unsigned int                                                              : 21;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_INT_STATUS_RING1 {
    struct {
        unsigned int                                                              : 11;
        unsigned int CP_VM_DOORBELL_WR_INT_STAT                                   :  1;
        unsigned int                                                              :  2;
        unsigned int CP_ECC_ERROR_INT_STAT                                        :  1;
        unsigned int                                                              :  1;
        unsigned int GPF_INT_STAT                                                 :  1;
        unsigned int WRM_POLL_TIMEOUT_INT_STAT                                    :  1;
        unsigned int CMP_BUSY_INT_STAT                                            :  1;
        unsigned int CNTX_BUSY_INT_STAT                                           :  1;
        unsigned int CNTX_EMPTY_INT_STAT                                          :  1;
        unsigned int GFX_IDLE_INT_STAT                                            :  1;
        unsigned int PRIV_INSTR_INT_STAT                                          :  1;
        unsigned int PRIV_REG_INT_STAT                                            :  1;
        unsigned int OPCODE_ERROR_INT_STAT                                        :  1;
        unsigned int                                                              :  1;
        unsigned int TIME_STAMP_INT_STAT                                          :  1;
        unsigned int RESERVED_BIT_ERROR_INT_STAT                                  :  1;
        unsigned int                                                              :  1;
        unsigned int GENERIC2_INT_STAT                                            :  1;
        unsigned int GENERIC1_INT_STAT                                            :  1;
        unsigned int GENERIC0_INT_STAT                                            :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 10;
        unsigned int DMA_WATCH_INT_STAT                                           :  1;
        unsigned int                                                              : 21;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_INT_STATUS_RING2 {
    struct {
        unsigned int                                                              : 11;
        unsigned int CP_VM_DOORBELL_WR_INT_STAT                                   :  1;
        unsigned int                                                              :  2;
        unsigned int CP_ECC_ERROR_INT_STAT                                        :  1;
        unsigned int                                                              :  1;
        unsigned int GPF_INT_STAT                                                 :  1;
        unsigned int WRM_POLL_TIMEOUT_INT_STAT                                    :  1;
        unsigned int CMP_BUSY_INT_STAT                                            :  1;
        unsigned int CNTX_BUSY_INT_STAT                                           :  1;
        unsigned int CNTX_EMPTY_INT_STAT                                          :  1;
        unsigned int GFX_IDLE_INT_STAT                                            :  1;
        unsigned int PRIV_INSTR_INT_STAT                                          :  1;
        unsigned int PRIV_REG_INT_STAT                                            :  1;
        unsigned int OPCODE_ERROR_INT_STAT                                        :  1;
        unsigned int                                                              :  1;
        unsigned int TIME_STAMP_INT_STAT                                          :  1;
        unsigned int RESERVED_BIT_ERROR_INT_STAT                                  :  1;
        unsigned int                                                              :  1;
        unsigned int GENERIC2_INT_STAT                                            :  1;
        unsigned int GENERIC1_INT_STAT                                            :  1;
        unsigned int GENERIC0_INT_STAT                                            :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 10;
        unsigned int DMA_WATCH_INT_STAT                                           :  1;
        unsigned int                                                              : 21;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_INT_STAT_DEBUG {
    struct {
        unsigned int                                                              : 11;
        unsigned int CP_VM_DOORBELL_WR_INT_ASSERTED                               :  1;
        unsigned int                                                              :  2;
        unsigned int CP_ECC_ERROR_INT_ASSERTED                                    :  1;
        unsigned int                                                              :  1;
        unsigned int GPF_INT_ASSERTED                                             :  1;
        unsigned int WRM_POLL_TIMEOUT_INT_ASSERTED                                :  1;
        unsigned int CMP_BUSY_INT_ASSERTED                                        :  1;
        unsigned int CNTX_BUSY_INT_ASSERTED                                       :  1;
        unsigned int CNTX_EMPTY_INT_ASSERTED                                      :  1;
        unsigned int GFX_IDLE_INT_ASSERTED                                        :  1;
        unsigned int PRIV_INSTR_INT_ASSERTED                                      :  1;
        unsigned int PRIV_REG_INT_ASSERTED                                        :  1;
        unsigned int OPCODE_ERROR_INT_ASSERTED                                    :  1;
        unsigned int                                                              :  1;
        unsigned int TIME_STAMP_INT_ASSERTED                                      :  1;
        unsigned int RESERVED_BIT_ERROR_INT_ASSERTED                              :  1;
        unsigned int                                                              :  1;
        unsigned int GENERIC2_INT_ASSERTED                                        :  1;
        unsigned int GENERIC1_INT_ASSERTED                                        :  1;
        unsigned int GENERIC0_INT_ASSERTED                                        :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  8;
        unsigned int RESUME_INT_ASSERTED                                          :  1;
        unsigned int SUSPEND_INT_ASSERTED                                         :  1;
        unsigned int DMA_WATCH_INT_ASSERTED                                       :  1;
        unsigned int                                                              : 21;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_IQ_WAIT_TIME1 {
    struct {
        unsigned int IB_OFFLOAD                                                   :  8;
        unsigned int ATOMIC_OFFLOAD                                               :  8;
        unsigned int WRM_OFFLOAD                                                  :  8;
        unsigned int GWS                                                          :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_IQ_WAIT_TIME2 {
    struct {
        unsigned int QUE_SLEEP                                                    :  8;
        unsigned int SCH_WAVE                                                     :  8;
        unsigned int SEM_REARM                                                    :  8;
        unsigned int DEQ_RETRY                                                    :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_IQ_WAIT_TIME3 {
    struct {
        unsigned int SUSPEND_QUE                                                  :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MAX_CONTEXT {
    struct {
        unsigned int MAX_CONTEXT                                                  :  3;
        unsigned int                                                              : 29;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME0_PIPE0_PRIORITY {
    struct {
        unsigned int PRIORITY                                                     :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME0_PIPE0_VMID {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME0_PIPE1_PRIORITY {
    struct {
        unsigned int PRIORITY                                                     :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME0_PIPE1_VMID {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME0_PIPE2_PRIORITY {
    struct {
        unsigned int PRIORITY                                                     :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME0_PIPE_PRIORITY_CNTS {
    struct {
        unsigned int PRIORITY1_CNT                                                :  8;
        unsigned int PRIORITY2A_CNT                                               :  8;
        unsigned int PRIORITY2B_CNT                                               :  8;
        unsigned int PRIORITY3_CNT                                                :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME1_INT_STAT_DEBUG {
    struct {
        unsigned int                                                              : 12;
        unsigned int CMP_QUERY_STATUS_INT_ASSERTED                                :  1;
        unsigned int DEQUEUE_REQUEST_INT_ASSERTED                                 :  1;
        unsigned int CP_ECC_ERROR_INT_ASSERTED                                    :  1;
        unsigned int SUA_VIOLATION_INT_STATUS                                     :  1;
        unsigned int GPF_INT_ASSERTED                                             :  1;
        unsigned int WRM_POLL_TIMEOUT_INT_ASSERTED                                :  1;
        unsigned int                                                              :  5;
        unsigned int PRIV_REG_INT_ASSERTED                                        :  1;
        unsigned int OPCODE_ERROR_INT_ASSERTED                                    :  1;
        unsigned int                                                              :  1;
        unsigned int TIME_STAMP_INT_ASSERTED                                      :  1;
        unsigned int RESERVED_BIT_ERROR_INT_ASSERTED                              :  1;
        unsigned int                                                              :  1;
        unsigned int GENERIC2_INT_ASSERTED                                        :  1;
        unsigned int GENERIC1_INT_ASSERTED                                        :  1;
        unsigned int GENERIC0_INT_ASSERTED                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME1_PIPE0_INT_CNTL {
    struct {
        unsigned int                                                              : 12;
        unsigned int CMP_QUERY_STATUS_INT_ENABLE                                  :  1;
        unsigned int DEQUEUE_REQUEST_INT_ENABLE                                   :  1;
        unsigned int CP_ECC_ERROR_INT_ENABLE                                      :  1;
        unsigned int SUA_VIOLATION_INT_ENABLE                                     :  1;
        unsigned int GPF_INT_ENABLE                                               :  1;
        unsigned int WRM_POLL_TIMEOUT_INT_ENABLE                                  :  1;
        unsigned int                                                              :  5;
        unsigned int PRIV_REG_INT_ENABLE                                          :  1;
        unsigned int OPCODE_ERROR_INT_ENABLE                                      :  1;
        unsigned int                                                              :  1;
        unsigned int TIME_STAMP_INT_ENABLE                                        :  1;
        unsigned int RESERVED_BIT_ERROR_INT_ENABLE                                :  1;
        unsigned int                                                              :  1;
        unsigned int GENERIC2_INT_ENABLE                                          :  1;
        unsigned int GENERIC1_INT_ENABLE                                          :  1;
        unsigned int GENERIC0_INT_ENABLE                                          :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME1_PIPE0_INT_STATUS {
    struct {
        unsigned int                                                              : 12;
        unsigned int CMP_QUERY_STATUS_INT_STATUS                                  :  1;
        unsigned int DEQUEUE_REQUEST_INT_STATUS                                   :  1;
        unsigned int CP_ECC_ERROR_INT_STATUS                                      :  1;
        unsigned int SUA_VIOLATION_INT_STATUS                                     :  1;
        unsigned int GPF_INT_STATUS                                               :  1;
        unsigned int WRM_POLL_TIMEOUT_INT_STATUS                                  :  1;
        unsigned int                                                              :  5;
        unsigned int PRIV_REG_INT_STATUS                                          :  1;
        unsigned int OPCODE_ERROR_INT_STATUS                                      :  1;
        unsigned int                                                              :  1;
        unsigned int TIME_STAMP_INT_STATUS                                        :  1;
        unsigned int RESERVED_BIT_ERROR_INT_STATUS                                :  1;
        unsigned int                                                              :  1;
        unsigned int GENERIC2_INT_STATUS                                          :  1;
        unsigned int GENERIC1_INT_STATUS                                          :  1;
        unsigned int GENERIC0_INT_STATUS                                          :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME1_PIPE0_PRIORITY {
    struct {
        unsigned int PRIORITY                                                     :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME1_PIPE1_INT_CNTL {
    struct {
        unsigned int                                                              : 12;
        unsigned int CMP_QUERY_STATUS_INT_ENABLE                                  :  1;
        unsigned int DEQUEUE_REQUEST_INT_ENABLE                                   :  1;
        unsigned int CP_ECC_ERROR_INT_ENABLE                                      :  1;
        unsigned int SUA_VIOLATION_INT_ENABLE                                     :  1;
        unsigned int GPF_INT_ENABLE                                               :  1;
        unsigned int WRM_POLL_TIMEOUT_INT_ENABLE                                  :  1;
        unsigned int                                                              :  5;
        unsigned int PRIV_REG_INT_ENABLE                                          :  1;
        unsigned int OPCODE_ERROR_INT_ENABLE                                      :  1;
        unsigned int                                                              :  1;
        unsigned int TIME_STAMP_INT_ENABLE                                        :  1;
        unsigned int RESERVED_BIT_ERROR_INT_ENABLE                                :  1;
        unsigned int                                                              :  1;
        unsigned int GENERIC2_INT_ENABLE                                          :  1;
        unsigned int GENERIC1_INT_ENABLE                                          :  1;
        unsigned int GENERIC0_INT_ENABLE                                          :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME1_PIPE1_INT_STATUS {
    struct {
        unsigned int                                                              : 12;
        unsigned int CMP_QUERY_STATUS_INT_STATUS                                  :  1;
        unsigned int DEQUEUE_REQUEST_INT_STATUS                                   :  1;
        unsigned int CP_ECC_ERROR_INT_STATUS                                      :  1;
        unsigned int SUA_VIOLATION_INT_STATUS                                     :  1;
        unsigned int GPF_INT_STATUS                                               :  1;
        unsigned int WRM_POLL_TIMEOUT_INT_STATUS                                  :  1;
        unsigned int                                                              :  5;
        unsigned int PRIV_REG_INT_STATUS                                          :  1;
        unsigned int OPCODE_ERROR_INT_STATUS                                      :  1;
        unsigned int                                                              :  1;
        unsigned int TIME_STAMP_INT_STATUS                                        :  1;
        unsigned int RESERVED_BIT_ERROR_INT_STATUS                                :  1;
        unsigned int                                                              :  1;
        unsigned int GENERIC2_INT_STATUS                                          :  1;
        unsigned int GENERIC1_INT_STATUS                                          :  1;
        unsigned int GENERIC0_INT_STATUS                                          :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME1_PIPE1_PRIORITY {
    struct {
        unsigned int PRIORITY                                                     :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME1_PIPE2_INT_CNTL {
    struct {
        unsigned int                                                              : 12;
        unsigned int CMP_QUERY_STATUS_INT_ENABLE                                  :  1;
        unsigned int DEQUEUE_REQUEST_INT_ENABLE                                   :  1;
        unsigned int CP_ECC_ERROR_INT_ENABLE                                      :  1;
        unsigned int SUA_VIOLATION_INT_ENABLE                                     :  1;
        unsigned int GPF_INT_ENABLE                                               :  1;
        unsigned int WRM_POLL_TIMEOUT_INT_ENABLE                                  :  1;
        unsigned int                                                              :  5;
        unsigned int PRIV_REG_INT_ENABLE                                          :  1;
        unsigned int OPCODE_ERROR_INT_ENABLE                                      :  1;
        unsigned int                                                              :  1;
        unsigned int TIME_STAMP_INT_ENABLE                                        :  1;
        unsigned int RESERVED_BIT_ERROR_INT_ENABLE                                :  1;
        unsigned int                                                              :  1;
        unsigned int GENERIC2_INT_ENABLE                                          :  1;
        unsigned int GENERIC1_INT_ENABLE                                          :  1;
        unsigned int GENERIC0_INT_ENABLE                                          :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME1_PIPE2_INT_STATUS {
    struct {
        unsigned int                                                              : 12;
        unsigned int CMP_QUERY_STATUS_INT_STATUS                                  :  1;
        unsigned int DEQUEUE_REQUEST_INT_STATUS                                   :  1;
        unsigned int CP_ECC_ERROR_INT_STATUS                                      :  1;
        unsigned int SUA_VIOLATION_INT_STATUS                                     :  1;
        unsigned int GPF_INT_STATUS                                               :  1;
        unsigned int WRM_POLL_TIMEOUT_INT_STATUS                                  :  1;
        unsigned int                                                              :  5;
        unsigned int PRIV_REG_INT_STATUS                                          :  1;
        unsigned int OPCODE_ERROR_INT_STATUS                                      :  1;
        unsigned int                                                              :  1;
        unsigned int TIME_STAMP_INT_STATUS                                        :  1;
        unsigned int RESERVED_BIT_ERROR_INT_STATUS                                :  1;
        unsigned int                                                              :  1;
        unsigned int GENERIC2_INT_STATUS                                          :  1;
        unsigned int GENERIC1_INT_STATUS                                          :  1;
        unsigned int GENERIC0_INT_STATUS                                          :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME1_PIPE2_PRIORITY {
    struct {
        unsigned int PRIORITY                                                     :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME1_PIPE3_INT_CNTL {
    struct {
        unsigned int                                                              : 12;
        unsigned int CMP_QUERY_STATUS_INT_ENABLE                                  :  1;
        unsigned int DEQUEUE_REQUEST_INT_ENABLE                                   :  1;
        unsigned int CP_ECC_ERROR_INT_ENABLE                                      :  1;
        unsigned int SUA_VIOLATION_INT_ENABLE                                     :  1;
        unsigned int GPF_INT_ENABLE                                               :  1;
        unsigned int WRM_POLL_TIMEOUT_INT_ENABLE                                  :  1;
        unsigned int                                                              :  5;
        unsigned int PRIV_REG_INT_ENABLE                                          :  1;
        unsigned int OPCODE_ERROR_INT_ENABLE                                      :  1;
        unsigned int                                                              :  1;
        unsigned int TIME_STAMP_INT_ENABLE                                        :  1;
        unsigned int RESERVED_BIT_ERROR_INT_ENABLE                                :  1;
        unsigned int                                                              :  1;
        unsigned int GENERIC2_INT_ENABLE                                          :  1;
        unsigned int GENERIC1_INT_ENABLE                                          :  1;
        unsigned int GENERIC0_INT_ENABLE                                          :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME1_PIPE3_INT_STATUS {
    struct {
        unsigned int                                                              : 12;
        unsigned int CMP_QUERY_STATUS_INT_STATUS                                  :  1;
        unsigned int DEQUEUE_REQUEST_INT_STATUS                                   :  1;
        unsigned int CP_ECC_ERROR_INT_STATUS                                      :  1;
        unsigned int SUA_VIOLATION_INT_STATUS                                     :  1;
        unsigned int GPF_INT_STATUS                                               :  1;
        unsigned int WRM_POLL_TIMEOUT_INT_STATUS                                  :  1;
        unsigned int                                                              :  5;
        unsigned int PRIV_REG_INT_STATUS                                          :  1;
        unsigned int OPCODE_ERROR_INT_STATUS                                      :  1;
        unsigned int                                                              :  1;
        unsigned int TIME_STAMP_INT_STATUS                                        :  1;
        unsigned int RESERVED_BIT_ERROR_INT_STATUS                                :  1;
        unsigned int                                                              :  1;
        unsigned int GENERIC2_INT_STATUS                                          :  1;
        unsigned int GENERIC1_INT_STATUS                                          :  1;
        unsigned int GENERIC0_INT_STATUS                                          :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME1_PIPE3_PRIORITY {
    struct {
        unsigned int PRIORITY                                                     :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME1_PIPE_PRIORITY_CNTS {
    struct {
        unsigned int PRIORITY1_CNT                                                :  8;
        unsigned int PRIORITY2A_CNT                                               :  8;
        unsigned int PRIORITY2B_CNT                                               :  8;
        unsigned int PRIORITY3_CNT                                                :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME2_INT_STAT_DEBUG {
    struct {
        unsigned int                                                              : 12;
        unsigned int CMP_QUERY_STATUS_INT_ASSERTED                                :  1;
        unsigned int DEQUEUE_REQUEST_INT_ASSERTED                                 :  1;
        unsigned int CP_ECC_ERROR_INT_ASSERTED                                    :  1;
        unsigned int SUA_VIOLATION_INT_STATUS                                     :  1;
        unsigned int GPF_INT_ASSERTED                                             :  1;
        unsigned int WRM_POLL_TIMEOUT_INT_ASSERTED                                :  1;
        unsigned int                                                              :  5;
        unsigned int PRIV_REG_INT_ASSERTED                                        :  1;
        unsigned int OPCODE_ERROR_INT_ASSERTED                                    :  1;
        unsigned int                                                              :  1;
        unsigned int TIME_STAMP_INT_ASSERTED                                      :  1;
        unsigned int RESERVED_BIT_ERROR_INT_ASSERTED                              :  1;
        unsigned int                                                              :  1;
        unsigned int GENERIC2_INT_ASSERTED                                        :  1;
        unsigned int GENERIC1_INT_ASSERTED                                        :  1;
        unsigned int GENERIC0_INT_ASSERTED                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME2_PIPE0_INT_CNTL {
    struct {
        unsigned int                                                              : 12;
        unsigned int CMP_QUERY_STATUS_INT_ENABLE                                  :  1;
        unsigned int DEQUEUE_REQUEST_INT_ENABLE                                   :  1;
        unsigned int CP_ECC_ERROR_INT_ENABLE                                      :  1;
        unsigned int SUA_VIOLATION_INT_ENABLE                                     :  1;
        unsigned int GPF_INT_ENABLE                                               :  1;
        unsigned int WRM_POLL_TIMEOUT_INT_ENABLE                                  :  1;
        unsigned int                                                              :  5;
        unsigned int PRIV_REG_INT_ENABLE                                          :  1;
        unsigned int OPCODE_ERROR_INT_ENABLE                                      :  1;
        unsigned int                                                              :  1;
        unsigned int TIME_STAMP_INT_ENABLE                                        :  1;
        unsigned int RESERVED_BIT_ERROR_INT_ENABLE                                :  1;
        unsigned int                                                              :  1;
        unsigned int GENERIC2_INT_ENABLE                                          :  1;
        unsigned int GENERIC1_INT_ENABLE                                          :  1;
        unsigned int GENERIC0_INT_ENABLE                                          :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME2_PIPE0_INT_STATUS {
    struct {
        unsigned int                                                              : 12;
        unsigned int CMP_QUERY_STATUS_INT_STATUS                                  :  1;
        unsigned int DEQUEUE_REQUEST_INT_STATUS                                   :  1;
        unsigned int CP_ECC_ERROR_INT_STATUS                                      :  1;
        unsigned int SUA_VIOLATION_INT_STATUS                                     :  1;
        unsigned int GPF_INT_STATUS                                               :  1;
        unsigned int WRM_POLL_TIMEOUT_INT_STATUS                                  :  1;
        unsigned int                                                              :  5;
        unsigned int PRIV_REG_INT_STATUS                                          :  1;
        unsigned int OPCODE_ERROR_INT_STATUS                                      :  1;
        unsigned int                                                              :  1;
        unsigned int TIME_STAMP_INT_STATUS                                        :  1;
        unsigned int RESERVED_BIT_ERROR_INT_STATUS                                :  1;
        unsigned int                                                              :  1;
        unsigned int GENERIC2_INT_STATUS                                          :  1;
        unsigned int GENERIC1_INT_STATUS                                          :  1;
        unsigned int GENERIC0_INT_STATUS                                          :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME2_PIPE0_PRIORITY {
    struct {
        unsigned int PRIORITY                                                     :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME2_PIPE1_INT_CNTL {
    struct {
        unsigned int                                                              : 12;
        unsigned int CMP_QUERY_STATUS_INT_ENABLE                                  :  1;
        unsigned int DEQUEUE_REQUEST_INT_ENABLE                                   :  1;
        unsigned int CP_ECC_ERROR_INT_ENABLE                                      :  1;
        unsigned int SUA_VIOLATION_INT_ENABLE                                     :  1;
        unsigned int GPF_INT_ENABLE                                               :  1;
        unsigned int WRM_POLL_TIMEOUT_INT_ENABLE                                  :  1;
        unsigned int                                                              :  5;
        unsigned int PRIV_REG_INT_ENABLE                                          :  1;
        unsigned int OPCODE_ERROR_INT_ENABLE                                      :  1;
        unsigned int                                                              :  1;
        unsigned int TIME_STAMP_INT_ENABLE                                        :  1;
        unsigned int RESERVED_BIT_ERROR_INT_ENABLE                                :  1;
        unsigned int                                                              :  1;
        unsigned int GENERIC2_INT_ENABLE                                          :  1;
        unsigned int GENERIC1_INT_ENABLE                                          :  1;
        unsigned int GENERIC0_INT_ENABLE                                          :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME2_PIPE1_INT_STATUS {
    struct {
        unsigned int                                                              : 12;
        unsigned int CMP_QUERY_STATUS_INT_STATUS                                  :  1;
        unsigned int DEQUEUE_REQUEST_INT_STATUS                                   :  1;
        unsigned int CP_ECC_ERROR_INT_STATUS                                      :  1;
        unsigned int SUA_VIOLATION_INT_STATUS                                     :  1;
        unsigned int GPF_INT_STATUS                                               :  1;
        unsigned int WRM_POLL_TIMEOUT_INT_STATUS                                  :  1;
        unsigned int                                                              :  5;
        unsigned int PRIV_REG_INT_STATUS                                          :  1;
        unsigned int OPCODE_ERROR_INT_STATUS                                      :  1;
        unsigned int                                                              :  1;
        unsigned int TIME_STAMP_INT_STATUS                                        :  1;
        unsigned int RESERVED_BIT_ERROR_INT_STATUS                                :  1;
        unsigned int                                                              :  1;
        unsigned int GENERIC2_INT_STATUS                                          :  1;
        unsigned int GENERIC1_INT_STATUS                                          :  1;
        unsigned int GENERIC0_INT_STATUS                                          :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME2_PIPE1_PRIORITY {
    struct {
        unsigned int PRIORITY                                                     :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME2_PIPE2_INT_CNTL {
    struct {
        unsigned int                                                              : 12;
        unsigned int CMP_QUERY_STATUS_INT_ENABLE                                  :  1;
        unsigned int DEQUEUE_REQUEST_INT_ENABLE                                   :  1;
        unsigned int CP_ECC_ERROR_INT_ENABLE                                      :  1;
        unsigned int SUA_VIOLATION_INT_ENABLE                                     :  1;
        unsigned int GPF_INT_ENABLE                                               :  1;
        unsigned int WRM_POLL_TIMEOUT_INT_ENABLE                                  :  1;
        unsigned int                                                              :  5;
        unsigned int PRIV_REG_INT_ENABLE                                          :  1;
        unsigned int OPCODE_ERROR_INT_ENABLE                                      :  1;
        unsigned int                                                              :  1;
        unsigned int TIME_STAMP_INT_ENABLE                                        :  1;
        unsigned int RESERVED_BIT_ERROR_INT_ENABLE                                :  1;
        unsigned int                                                              :  1;
        unsigned int GENERIC2_INT_ENABLE                                          :  1;
        unsigned int GENERIC1_INT_ENABLE                                          :  1;
        unsigned int GENERIC0_INT_ENABLE                                          :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME2_PIPE2_INT_STATUS {
    struct {
        unsigned int                                                              : 12;
        unsigned int CMP_QUERY_STATUS_INT_STATUS                                  :  1;
        unsigned int DEQUEUE_REQUEST_INT_STATUS                                   :  1;
        unsigned int CP_ECC_ERROR_INT_STATUS                                      :  1;
        unsigned int SUA_VIOLATION_INT_STATUS                                     :  1;
        unsigned int GPF_INT_STATUS                                               :  1;
        unsigned int WRM_POLL_TIMEOUT_INT_STATUS                                  :  1;
        unsigned int                                                              :  5;
        unsigned int PRIV_REG_INT_STATUS                                          :  1;
        unsigned int OPCODE_ERROR_INT_STATUS                                      :  1;
        unsigned int                                                              :  1;
        unsigned int TIME_STAMP_INT_STATUS                                        :  1;
        unsigned int RESERVED_BIT_ERROR_INT_STATUS                                :  1;
        unsigned int                                                              :  1;
        unsigned int GENERIC2_INT_STATUS                                          :  1;
        unsigned int GENERIC1_INT_STATUS                                          :  1;
        unsigned int GENERIC0_INT_STATUS                                          :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME2_PIPE2_PRIORITY {
    struct {
        unsigned int PRIORITY                                                     :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME2_PIPE3_INT_CNTL {
    struct {
        unsigned int                                                              : 12;
        unsigned int CMP_QUERY_STATUS_INT_ENABLE                                  :  1;
        unsigned int DEQUEUE_REQUEST_INT_ENABLE                                   :  1;
        unsigned int CP_ECC_ERROR_INT_ENABLE                                      :  1;
        unsigned int SUA_VIOLATION_INT_ENABLE                                     :  1;
        unsigned int GPF_INT_ENABLE                                               :  1;
        unsigned int WRM_POLL_TIMEOUT_INT_ENABLE                                  :  1;
        unsigned int                                                              :  5;
        unsigned int PRIV_REG_INT_ENABLE                                          :  1;
        unsigned int OPCODE_ERROR_INT_ENABLE                                      :  1;
        unsigned int                                                              :  1;
        unsigned int TIME_STAMP_INT_ENABLE                                        :  1;
        unsigned int RESERVED_BIT_ERROR_INT_ENABLE                                :  1;
        unsigned int                                                              :  1;
        unsigned int GENERIC2_INT_ENABLE                                          :  1;
        unsigned int GENERIC1_INT_ENABLE                                          :  1;
        unsigned int GENERIC0_INT_ENABLE                                          :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME2_PIPE3_INT_STATUS {
    struct {
        unsigned int                                                              : 12;
        unsigned int CMP_QUERY_STATUS_INT_STATUS                                  :  1;
        unsigned int DEQUEUE_REQUEST_INT_STATUS                                   :  1;
        unsigned int CP_ECC_ERROR_INT_STATUS                                      :  1;
        unsigned int SUA_VIOLATION_INT_STATUS                                     :  1;
        unsigned int GPF_INT_STATUS                                               :  1;
        unsigned int WRM_POLL_TIMEOUT_INT_STATUS                                  :  1;
        unsigned int                                                              :  5;
        unsigned int PRIV_REG_INT_STATUS                                          :  1;
        unsigned int OPCODE_ERROR_INT_STATUS                                      :  1;
        unsigned int                                                              :  1;
        unsigned int TIME_STAMP_INT_STATUS                                        :  1;
        unsigned int RESERVED_BIT_ERROR_INT_STATUS                                :  1;
        unsigned int                                                              :  1;
        unsigned int GENERIC2_INT_STATUS                                          :  1;
        unsigned int GENERIC1_INT_STATUS                                          :  1;
        unsigned int GENERIC0_INT_STATUS                                          :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME2_PIPE3_PRIORITY {
    struct {
        unsigned int PRIORITY                                                     :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME2_PIPE_PRIORITY_CNTS {
    struct {
        unsigned int PRIORITY1_CNT                                                :  8;
        unsigned int PRIORITY2A_CNT                                               :  8;
        unsigned int PRIORITY2B_CNT                                               :  8;
        unsigned int PRIORITY3_CNT                                                :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MEC1_F32_INTERRUPT {
    struct {
        unsigned int EDC_ROQ_FED_INT                                              :  1;
        unsigned int PRIV_REG_INT                                                 :  1;
        unsigned int RESERVED_BIT_ERR_INT                                         :  1;
        unsigned int EDC_TC_FED_INT                                               :  1;
        unsigned int EDC_GDS_FED_INT                                              :  1;
        unsigned int EDC_SCRATCH_FED_INT                                          :  1;
        unsigned int WAVE_RESTORE_INT                                             :  1;
        unsigned int SUA_VIOLATION_INT                                            :  1;
        unsigned int EDC_DMA_FED_INT                                              :  1;
        unsigned int IQ_TIMER_INT                                                 :  1;
        unsigned int GPF_INT_CPF                                                  :  1;
        unsigned int GPF_INT_DMA                                                  :  1;
        unsigned int GPF_INT_CPC                                                  :  1;
        unsigned int EDC_SR_MEM_FED_INT                                           :  1;
        unsigned int QUEUE_MESSAGE_INT                                            :  1;
        unsigned int FATAL_EDC_ERROR_INT                                          :  1;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MEC1_F32_INT_DIS {
    struct {
        unsigned int EDC_ROQ_FED_INT                                              :  1;
        unsigned int PRIV_REG_INT                                                 :  1;
        unsigned int RESERVED_BIT_ERR_INT                                         :  1;
        unsigned int EDC_TC_FED_INT                                               :  1;
        unsigned int EDC_GDS_FED_INT                                              :  1;
        unsigned int EDC_SCRATCH_FED_INT                                          :  1;
        unsigned int WAVE_RESTORE_INT                                             :  1;
        unsigned int SUA_VIOLATION_INT                                            :  1;
        unsigned int EDC_DMA_FED_INT                                              :  1;
        unsigned int IQ_TIMER_INT                                                 :  1;
        unsigned int GPF_INT_CPF                                                  :  1;
        unsigned int GPF_INT_DMA                                                  :  1;
        unsigned int GPF_INT_CPC                                                  :  1;
        unsigned int EDC_SR_MEM_FED_INT                                           :  1;
        unsigned int QUEUE_MESSAGE_INT                                            :  1;
        unsigned int FATAL_EDC_ERROR_INT                                          :  1;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MEC1_INSTR_PNTR {
    struct {
        unsigned int INSTR_PNTR                                                   : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MEC1_INTR_ROUTINE_START {
    struct {
        unsigned int IR_START                                                     : 16;
        unsigned int                                                              : 16;
    } gfx09;
    struct {
        unsigned int IR_START                                                     : 20;
        unsigned int                                                              : 12;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MEC1_PRGRM_CNTR_START {
    struct {
        unsigned int IP_START                                                     : 16;
        unsigned int                                                              : 16;
    } gfx09;
    struct {
        unsigned int IP_START                                                     : 20;
        unsigned int                                                              : 12;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MEC2_F32_INTERRUPT {
    struct {
        unsigned int EDC_ROQ_FED_INT                                              :  1;
        unsigned int PRIV_REG_INT                                                 :  1;
        unsigned int RESERVED_BIT_ERR_INT                                         :  1;
        unsigned int EDC_TC_FED_INT                                               :  1;
        unsigned int EDC_GDS_FED_INT                                              :  1;
        unsigned int EDC_SCRATCH_FED_INT                                          :  1;
        unsigned int WAVE_RESTORE_INT                                             :  1;
        unsigned int SUA_VIOLATION_INT                                            :  1;
        unsigned int EDC_DMA_FED_INT                                              :  1;
        unsigned int IQ_TIMER_INT                                                 :  1;
        unsigned int GPF_INT_CPF                                                  :  1;
        unsigned int GPF_INT_DMA                                                  :  1;
        unsigned int GPF_INT_CPC                                                  :  1;
        unsigned int EDC_SR_MEM_FED_INT                                           :  1;
        unsigned int QUEUE_MESSAGE_INT                                            :  1;
        unsigned int FATAL_EDC_ERROR_INT                                          :  1;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MEC2_F32_INT_DIS {
    struct {
        unsigned int EDC_ROQ_FED_INT                                              :  1;
        unsigned int PRIV_REG_INT                                                 :  1;
        unsigned int RESERVED_BIT_ERR_INT                                         :  1;
        unsigned int EDC_TC_FED_INT                                               :  1;
        unsigned int EDC_GDS_FED_INT                                              :  1;
        unsigned int EDC_SCRATCH_FED_INT                                          :  1;
        unsigned int WAVE_RESTORE_INT                                             :  1;
        unsigned int SUA_VIOLATION_INT                                            :  1;
        unsigned int EDC_DMA_FED_INT                                              :  1;
        unsigned int IQ_TIMER_INT                                                 :  1;
        unsigned int GPF_INT_CPF                                                  :  1;
        unsigned int GPF_INT_DMA                                                  :  1;
        unsigned int GPF_INT_CPC                                                  :  1;
        unsigned int EDC_SR_MEM_FED_INT                                           :  1;
        unsigned int QUEUE_MESSAGE_INT                                            :  1;
        unsigned int FATAL_EDC_ERROR_INT                                          :  1;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MEC2_INSTR_PNTR {
    struct {
        unsigned int INSTR_PNTR                                                   : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MEC2_INTR_ROUTINE_START {
    struct {
        unsigned int IR_START                                                     : 16;
        unsigned int                                                              : 16;
    } gfx09;
    struct {
        unsigned int IR_START                                                     : 20;
        unsigned int                                                              : 12;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MEC2_PRGRM_CNTR_START {
    struct {
        unsigned int IP_START                                                     : 16;
        unsigned int                                                              : 16;
    } gfx09;
    struct {
        unsigned int IP_START                                                     : 20;
        unsigned int                                                              : 12;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MEC_CNTL {
    struct {
        unsigned int                                                              : 16;
        unsigned int MEC_ME1_PIPE0_RESET                                          :  1;
        unsigned int MEC_ME1_PIPE1_RESET                                          :  1;
        unsigned int MEC_ME1_PIPE2_RESET                                          :  1;
        unsigned int MEC_ME1_PIPE3_RESET                                          :  1;
        unsigned int MEC_ME2_PIPE0_RESET                                          :  1;
        unsigned int MEC_ME2_PIPE1_RESET                                          :  1;
        unsigned int                                                              :  6;
        unsigned int MEC_ME2_HALT                                                 :  1;
        unsigned int MEC_ME2_STEP                                                 :  1;
        unsigned int MEC_ME1_HALT                                                 :  1;
        unsigned int MEC_ME1_STEP                                                 :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  4;
        unsigned int MEC_INVALIDATE_ICACHE                                        :  1;
        unsigned int                                                              : 27;
    } gfx09;
    struct {
        unsigned int                                                              : 22;
        unsigned int MEC_ME2_PIPE2_RESET                                          :  1;
        unsigned int MEC_ME2_PIPE3_RESET                                          :  1;
        unsigned int                                                              :  3;
        unsigned int MEC_INVALIDATE_ICACHE                                        :  1;
        unsigned int                                                              :  4;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MEC_DOORBELL_RANGE_LOWER {
    struct {
        unsigned int                                                              :  2;
        unsigned int DOORBELL_RANGE_LOWER                                         : 26;
        unsigned int                                                              :  4;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MEC_DOORBELL_RANGE_UPPER {
    struct {
        unsigned int                                                              :  2;
        unsigned int DOORBELL_RANGE_UPPER                                         : 26;
        unsigned int                                                              :  4;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MEC_JT_STAT {
    struct {
        unsigned int JT_LOADED                                                    :  8;
        unsigned int                                                              :  8;
        unsigned int WR_MASK                                                      :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MEC_ME1_HEADER_DUMP {
    struct {
        unsigned int HEADER_DUMP                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MEC_ME2_HEADER_DUMP {
    struct {
        unsigned int HEADER_DUMP                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MEM_SLP_CNTL {
    struct {
        unsigned int CP_MEM_LS_EN                                                 :  1;
        unsigned int CP_MEM_DS_EN                                                 :  1;
        unsigned int RESERVED                                                     :  5;
        unsigned int CP_LS_DS_BUSY_OVERRIDE                                       :  1;
        unsigned int CP_MEM_LS_ON_DELAY                                           :  8;
        unsigned int CP_MEM_LS_OFF_DELAY                                          :  8;
        unsigned int RESERVED1                                                    :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MEQ_AVAIL {
    struct {
        unsigned int MEQ_CNT                                                      : 10;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MEQ_STAT {
    struct {
        unsigned int MEQ_RPTR                                                     : 10;
        unsigned int                                                              :  6;
        unsigned int MEQ_WPTR                                                     : 10;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MEQ_STQ_THRESHOLD {
    struct {
        unsigned int STQ_START                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MEQ_THRESHOLDS {
    struct {
        unsigned int MEQ1_START                                                   :  8;
        unsigned int MEQ2_START                                                   :  8;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_CNTL {
    struct {
        unsigned int                                                              :  4;
        unsigned int MES_INVALIDATE_ICACHE                                        :  1;
        unsigned int                                                              : 11;
        unsigned int MES_PIPE0_RESET                                              :  1;
        unsigned int MES_PIPE1_RESET                                              :  1;
        unsigned int MES_PIPE2_RESET                                              :  1;
        unsigned int MES_PIPE3_RESET                                              :  1;
        unsigned int                                                              :  6;
        unsigned int MES_PIPE0_ACTIVE                                             :  1;
        unsigned int MES_PIPE1_ACTIVE                                             :  1;
        unsigned int MES_PIPE2_ACTIVE                                             :  1;
        unsigned int MES_PIPE3_ACTIVE                                             :  1;
        unsigned int MES_HALT                                                     :  1;
        unsigned int MES_STEP                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_DCSR {
    struct {
        unsigned int CSR                                                          : 32;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_DC_BASE_CNTL {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int                                                              : 20;
        unsigned int CACHE_POLICY                                                 :  2;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_DC_BASE_HI {
    struct {
        unsigned int DC_BASE_HI                                                   : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_DC_BASE_LO {
    struct {
        unsigned int                                                              : 16;
        unsigned int DC_BASE_LO                                                   : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_DC_OP_CNTL {
    struct {
        unsigned int INVALIDATE_DCACHE                                            :  1;
        unsigned int INVALIDATE_DCACHE_COMPLETE                                   :  1;
        unsigned int BYPASS_ALL                                                   :  1;
        unsigned int BYPASS_UNCACHED                                              :  1;
        unsigned int PRIME_DCACHE                                                 :  1;
        unsigned int DCACHE_PRIMED                                                :  1;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_DMCONTROL {
    struct {
        unsigned int CONTROL                                                      : 32;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_DMINFO {
    struct {
        unsigned int INFO                                                         : 32;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_DM_INDEX_ADDR {
    struct {
        unsigned int ADDR                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_DM_INDEX_DATA {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_DOORBELL_CONTROL1 {
    struct {
        unsigned int                                                              :  2;
        unsigned int DOORBELL_OFFSET                                              : 26;
        unsigned int                                                              :  2;
        unsigned int DOORBELL_EN                                                  :  1;
        unsigned int DOORBELL_HIT                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_DOORBELL_CONTROL2 {
    struct {
        unsigned int                                                              :  2;
        unsigned int DOORBELL_OFFSET                                              : 26;
        unsigned int                                                              :  2;
        unsigned int DOORBELL_EN                                                  :  1;
        unsigned int DOORBELL_HIT                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_DOORBELL_CONTROL3 {
    struct {
        unsigned int                                                              :  2;
        unsigned int DOORBELL_OFFSET                                              : 26;
        unsigned int                                                              :  2;
        unsigned int DOORBELL_EN                                                  :  1;
        unsigned int DOORBELL_HIT                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_DOORBELL_CONTROL4 {
    struct {
        unsigned int                                                              :  2;
        unsigned int DOORBELL_OFFSET                                              : 26;
        unsigned int                                                              :  2;
        unsigned int DOORBELL_EN                                                  :  1;
        unsigned int DOORBELL_HIT                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_DOORBELL_CONTROL5 {
    struct {
        unsigned int                                                              :  2;
        unsigned int DOORBELL_OFFSET                                              : 26;
        unsigned int                                                              :  2;
        unsigned int DOORBELL_EN                                                  :  1;
        unsigned int DOORBELL_HIT                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_DOORBELL_CONTROL6 {
    struct {
        unsigned int                                                              :  2;
        unsigned int DOORBELL_OFFSET                                              : 26;
        unsigned int                                                              :  2;
        unsigned int DOORBELL_EN                                                  :  1;
        unsigned int DOORBELL_HIT                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_DPC_HIGH {
    struct {
        unsigned int INSTR_PNTR                                                   : 32;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_DPC_LOW {
    struct {
        unsigned int INSTR_PNTR                                                   : 32;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_DSCRATCH_HIGH {
    struct {
        unsigned int DATA                                                         : 32;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_DSCRATCH_LOW {
    struct {
        unsigned int DATA                                                         : 32;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_GP0_HI {
    struct {
        unsigned int M_RET_ADDR                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_GP0_LO {
    struct {
        unsigned int PG_VIRT_HALTED                                               :  1;
        unsigned int DATA                                                         : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_GP1_HI {
    struct {
        unsigned int RD_WR_SELECT_HI                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_GP1_LO {
    struct {
        unsigned int RD_WR_SELECT_LO                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_GP2_HI {
    struct {
        unsigned int STACK_PNTR_HI                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_GP2_LO {
    struct {
        unsigned int STACK_PNTR_LO                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_GP3_HI {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_GP3_LO {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_GP4_HI {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_GP4_LO {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_GP5_HI {
    struct {
        unsigned int M_RET_ADDR                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_GP5_LO {
    struct {
        unsigned int PG_VIRT_HALTED                                               :  1;
        unsigned int DATA                                                         : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_GP6_HI {
    struct {
        unsigned int RD_WR_SELECT_HI                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_GP6_LO {
    struct {
        unsigned int RD_WR_SELECT_LO                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_GP7_HI {
    struct {
        unsigned int STACK_PNTR_HI                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_GP7_LO {
    struct {
        unsigned int STACK_PNTR_LO                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_GP8_HI {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_GP8_LO {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_GP9_HI {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_GP9_LO {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_HEADER_DUMP {
    struct {
        unsigned int HEADER_DUMP                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_IC_BASE_CNTL {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int                                                              : 19;
        unsigned int EXE_DISABLE                                                  :  1;
        unsigned int CACHE_POLICY                                                 :  2;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_IC_BASE_HI {
    struct {
        unsigned int IC_BASE_HI                                                   : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_IC_BASE_LO {
    struct {
        unsigned int                                                              : 12;
        unsigned int IC_BASE_LO                                                   : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_IC_OP_CNTL {
    struct {
        unsigned int INVALIDATE_CACHE                                             :  1;
        unsigned int                                                              :  3;
        unsigned int PRIME_ICACHE                                                 :  1;
        unsigned int ICACHE_PRIMED                                                :  1;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_INSTR_PNTR {
    struct {
        unsigned int INSTR_PNTR                                                   : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_INTERRUPT {
    struct {
        unsigned int MES_INT                                                      : 16;
        unsigned int PENDING_INTERRUPT                                            : 16;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_INTR_ROUTINE_START {
    struct {
        unsigned int IR_START                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_LOCAL_APERTURE {
    struct {
        unsigned int APERTURE                                                     :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_LOCAL_BASE0_HI {
    struct {
        unsigned int BASE0_HI                                                     : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_LOCAL_BASE0_LO {
    struct {
        unsigned int                                                              : 16;
        unsigned int BASE0_LO                                                     : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_LOCAL_MASK0_HI {
    struct {
        unsigned int MASK0_HI                                                     : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_LOCAL_MASK0_LO {
    struct {
        unsigned int                                                              : 16;
        unsigned int MASK0_LO                                                     : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MARCHID_HI {
    struct {
        unsigned int MARCHID_HI                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MARCHID_LO {
    struct {
        unsigned int MARCHID_LO                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MBADADDR_HI {
    struct {
        unsigned int ADDR_HI                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MBADADDR_LO {
    struct {
        unsigned int ADDR_LO                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MCAUSE_HI {
    struct {
        unsigned int CAUSE_HI                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MCAUSE_LO {
    struct {
        unsigned int CAUSE_LO                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MCYCLE_HI {
    struct {
        unsigned int CYCLE_HI                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MCYCLE_LO {
    struct {
        unsigned int CYCLE_LO                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MDBASE_HI {
    struct {
        unsigned int BASE_HI                                                      : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MDBASE_LO {
    struct {
        unsigned int                                                              : 16;
        unsigned int BASE_LO                                                      : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MDBOUND_HI {
    struct {
        unsigned int BOUND_HI                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MDBOUND_LO {
    struct {
        unsigned int BOUND_LO                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MEPC_HI {
    struct {
        unsigned int MEPC_HI                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MEPC_LO {
    struct {
        unsigned int MEPC_LO                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MHARTID_HI {
    struct {
        unsigned int MHARTID_HI                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MHARTID_LO {
    struct {
        unsigned int MHARTID_LO                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MIBASE_HI {
    struct {
        unsigned int IC_BASE_HI                                                   : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MIBASE_LO {
    struct {
        unsigned int                                                              : 12;
        unsigned int IC_BASE_LO                                                   : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MIBOUND_HI {
    struct {
        unsigned int BOUND_HI                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MIBOUND_LO {
    struct {
        unsigned int BOUND_LO                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MIE_HI {
    struct {
        unsigned int MES_INT                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MIE_LO {
    struct {
        unsigned int MES_INT                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MIMPID_HI {
    struct {
        unsigned int MIMPID_HI                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MIMPID_LO {
    struct {
        unsigned int MIMPID_LO                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MINSTRET_HI {
    struct {
        unsigned int INSTRET_HI                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MINSTRET_LO {
    struct {
        unsigned int INSTRET_LO                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MIP_HI {
    struct {
        unsigned int MIP_HI                                                       : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MIP_LO {
    struct {
        unsigned int MIP_LO                                                       : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MISA_HI {
    struct {
        unsigned int MISA_HI                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MISA_LO {
    struct {
        unsigned int MISA_LO                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MSCRATCH_HI {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MSCRATCH_LO {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MSTATUS_HI {
    struct {
        unsigned int STATUS_HI                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MSTATUS_LO {
    struct {
        unsigned int STATUS_LO                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MTIMECMP_HI {
    struct {
        unsigned int TIME_HI                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MTIMECMP_LO {
    struct {
        unsigned int TIME_LO                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MTIME_HI {
    struct {
        unsigned int TIME_HI                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MTIME_LO {
    struct {
        unsigned int TIME_LO                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MTVEC_HI {
    struct {
        unsigned int ADDR_LO                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MTVEC_LO {
    struct {
        unsigned int ADDR_LO                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MVENDORID_HI {
    struct {
        unsigned int MVENDORID_HI                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_MVENDORID_LO {
    struct {
        unsigned int MVENDORID_LO                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_PERFCOUNT_CNTL {
    struct {
        unsigned int EVENT_SEL                                                    :  3;
        unsigned int                                                              : 29;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_PIPE0_PRIORITY {
    struct {
        unsigned int PRIORITY                                                     :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_PIPE1_PRIORITY {
    struct {
        unsigned int PRIORITY                                                     :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_PIPE2_PRIORITY {
    struct {
        unsigned int PRIORITY                                                     :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_PIPE3_PRIORITY {
    struct {
        unsigned int PRIORITY                                                     :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_PIPE_PRIORITY_CNTS {
    struct {
        unsigned int PRIORITY1_CNT                                                :  8;
        unsigned int PRIORITY2A_CNT                                               :  8;
        unsigned int PRIORITY2B_CNT                                               :  8;
        unsigned int PRIORITY3_CNT                                                :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_PRGRM_CNTR_START {
    struct {
        unsigned int IP_START                                                     : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_PROCESS_QUANTUM_PIPE0 {
    struct {
        unsigned int QUANTUM_DURATION                                             : 28;
        unsigned int TIMER_EXPIRED                                                :  1;
        unsigned int QUANTUM_SCALE                                                :  2;
        unsigned int QUANTUM_EN                                                   :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_PROCESS_QUANTUM_PIPE1 {
    struct {
        unsigned int QUANTUM_DURATION                                             : 28;
        unsigned int TIMER_EXPIRED                                                :  1;
        unsigned int QUANTUM_SCALE                                                :  2;
        unsigned int QUANTUM_EN                                                   :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_SCRATCH_DATA {
    struct {
        unsigned int SCRATCH_DATA                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_SCRATCH_INDEX {
    struct {
        unsigned int SCRATCH_INDEX                                                :  9;
        unsigned int                                                              : 22;
        unsigned int SCRATCH_INDEX_64BIT_MODE                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_SETHALTNOTIFICATION {
    struct {
        unsigned int SETHALT                                                      : 32;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_TDATA1_HIGH {
    struct {
        unsigned int DATA                                                         : 32;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_TDATA1_LOW {
    struct {
        unsigned int DATA                                                         : 32;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_TDATA2_HIGH {
    struct {
        unsigned int DATA                                                         : 32;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_TDATA2_LOW {
    struct {
        unsigned int DATA                                                         : 32;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_TDATA3_HIH {
    struct {
        unsigned int DATA                                                         : 32;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_TDATA3_LOW {
    struct {
        unsigned int DATA                                                         : 32;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_TSELCT_HIGH {
    struct {
        unsigned int TSELECT                                                      : 32;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_TSELCT_LOW {
    struct {
        unsigned int TSELECT                                                      : 32;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME_ATOMIC_PREOP_HI {
    struct {
        unsigned int ATOMIC_PREOP_HI                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME_ATOMIC_PREOP_LO {
    struct {
        unsigned int ATOMIC_PREOP_LO                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME_CNTL {
    struct {
        unsigned int                                                              :  4;
        unsigned int CE_INVALIDATE_ICACHE                                         :  1;
        unsigned int                                                              :  1;
        unsigned int PFP_INVALIDATE_ICACHE                                        :  1;
        unsigned int                                                              :  1;
        unsigned int ME_INVALIDATE_ICACHE                                         :  1;
        unsigned int                                                              :  7;
        unsigned int CE_PIPE0_RESET                                               :  1;
        unsigned int CE_PIPE1_RESET                                               :  1;
        unsigned int PFP_PIPE0_RESET                                              :  1;
        unsigned int PFP_PIPE1_RESET                                              :  1;
        unsigned int ME_PIPE0_RESET                                               :  1;
        unsigned int ME_PIPE1_RESET                                               :  1;
        unsigned int                                                              :  2;
        unsigned int CE_HALT                                                      :  1;
        unsigned int CE_STEP                                                      :  1;
        unsigned int PFP_HALT                                                     :  1;
        unsigned int PFP_STEP                                                     :  1;
        unsigned int ME_HALT                                                      :  1;
        unsigned int ME_STEP                                                      :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME_COHER_BASE {
    struct {
        unsigned int COHER_BASE_256B                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME_COHER_BASE_HI {
    struct {
        unsigned int COHER_BASE_HI_256B                                           :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME_COHER_CNTL {
    struct {
        unsigned int DEST_BASE_0_ENA                                              :  1;
        unsigned int DEST_BASE_1_ENA                                              :  1;
        unsigned int                                                              :  4;
        unsigned int CB0_DEST_BASE_ENA                                            :  1;
        unsigned int CB1_DEST_BASE_ENA                                            :  1;
        unsigned int CB2_DEST_BASE_ENA                                            :  1;
        unsigned int CB3_DEST_BASE_ENA                                            :  1;
        unsigned int CB4_DEST_BASE_ENA                                            :  1;
        unsigned int CB5_DEST_BASE_ENA                                            :  1;
        unsigned int CB6_DEST_BASE_ENA                                            :  1;
        unsigned int CB7_DEST_BASE_ENA                                            :  1;
        unsigned int DB_DEST_BASE_ENA                                             :  1;
        unsigned int                                                              :  4;
        unsigned int DEST_BASE_2_ENA                                              :  1;
        unsigned int                                                              :  1;
        unsigned int DEST_BASE_3_ENA                                              :  1;
        unsigned int                                                              : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME_COHER_SIZE {
    struct {
        unsigned int COHER_SIZE_256B                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME_COHER_SIZE_HI {
    struct {
        unsigned int COHER_SIZE_HI_256B                                           :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME_COHER_STATUS {
    struct {
        unsigned int MATCHING_GFX_CNTX                                            :  8;
        unsigned int                                                              : 23;
        unsigned int STATUS                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME_HEADER_DUMP {
    struct {
        unsigned int ME_HEADER_DUMP                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME_IC_BASE_CNTL {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int ADDRESS_CLAMP                                                :  1;
        unsigned int                                                              : 18;
        unsigned int EXE_DISABLE                                                  :  1;
        unsigned int CACHE_POLICY                                                 :  2;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME_IC_BASE_HI {
    struct {
        unsigned int IC_BASE_HI                                                   : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME_IC_BASE_LO {
    struct {
        unsigned int                                                              : 12;
        unsigned int IC_BASE_LO                                                   : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME_IC_OP_CNTL {
    struct {
        unsigned int INVALIDATE_CACHE                                             :  1;
        unsigned int INVALIDATE_CACHE_COMPLETE                                    :  1;
        unsigned int                                                              :  2;
        unsigned int PRIME_ICACHE                                                 :  1;
        unsigned int ICACHE_PRIMED                                                :  1;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME_INSTR_PNTR {
    struct {
        unsigned int INSTR_PNTR                                                   : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME_INTR_ROUTINE_START {
    struct {
        unsigned int IR_START                                                     : 12;
        unsigned int                                                              : 20;
    } gfx09;
    struct {
        unsigned int IR_START                                                     : 20;
        unsigned int                                                              : 12;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME_MC_RADDR_HI {
    struct {
        unsigned int ME_MC_RADDR_HI                                               : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 22;
        unsigned int CACHE_POLICY                                                 :  1;
        unsigned int                                                              :  9;
    } gfx09;
    struct {
        unsigned int                                                              : 22;
        unsigned int CACHE_POLICY                                                 :  2;
        unsigned int                                                              :  8;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME_MC_RADDR_LO {
    struct {
        unsigned int                                                              :  2;
        unsigned int ME_MC_RADDR_LO                                               : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME_MC_WADDR_HI {
    struct {
        unsigned int ME_MC_WADDR_HI                                               : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 22;
        unsigned int CACHE_POLICY                                                 :  1;
        unsigned int                                                              :  9;
    } gfx09;
    struct {
        unsigned int                                                              : 22;
        unsigned int CACHE_POLICY                                                 :  2;
        unsigned int                                                              :  8;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME_MC_WADDR_LO {
    struct {
        unsigned int                                                              :  2;
        unsigned int ME_MC_WADDR_LO                                               : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME_MC_WDATA_HI {
    struct {
        unsigned int ME_MC_WDATA_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME_MC_WDATA_LO {
    struct {
        unsigned int ME_MC_WDATA_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME_PREEMPTION {
    struct {
        unsigned int OBSOLETE                                                     :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME_PRGRM_CNTR_START {
    struct {
        unsigned int IP_START                                                     : 12;
        unsigned int                                                              : 20;
    } gfx09;
    struct {
        unsigned int IP_START                                                     : 20;
        unsigned int                                                              : 12;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME_RAM_DATA {
    struct {
        unsigned int ME_RAM_DATA                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME_RAM_RADDR {
    struct {
        unsigned int ME_RAM_RADDR                                                 : 13;
        unsigned int                                                              : 19;
    } gfx09;
    struct {
        unsigned int ME_RAM_RADDR                                                 : 20;
        unsigned int                                                              : 12;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME_RAM_WADDR {
    struct {
        unsigned int ME_RAM_WADDR                                                 : 13;
        unsigned int                                                              : 19;
    } gfx09;
    struct {
        unsigned int ME_RAM_WADDR                                                 : 21;
        unsigned int                                                              : 11;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MQD_BASE_ADDR {
    struct {
        unsigned int                                                              :  2;
        unsigned int BASE_ADDR                                                    : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MQD_BASE_ADDR_HI {
    struct {
        unsigned int BASE_ADDR_HI                                                 : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MQD_CONTROL {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int                                                              :  4;
        unsigned int PRIV_STATE                                                   :  1;
        unsigned int                                                              :  3;
        unsigned int PROCESSING_MQD                                               :  1;
        unsigned int PROCESSING_MQD_EN                                            :  1;
        unsigned int                                                              :  9;
        unsigned int EXE_DISABLE                                                  :  1;
        unsigned int                                                              :  8;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 24;
        unsigned int CACHE_POLICY                                                 :  1;
        unsigned int                                                              :  7;
    } gfx09;
    struct {
        unsigned int                                                              : 24;
        unsigned int CACHE_POLICY                                                 :  2;
        unsigned int MQD_VOLATILE                                                 :  1;
        unsigned int                                                              :  5;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_NUM_PRIM_NEEDED_COUNT0_HI {
    struct {
        unsigned int NUM_PRIM_NEEDED_CNT0_HI                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_NUM_PRIM_NEEDED_COUNT0_LO {
    struct {
        unsigned int NUM_PRIM_NEEDED_CNT0_LO                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_NUM_PRIM_NEEDED_COUNT1_HI {
    struct {
        unsigned int NUM_PRIM_NEEDED_CNT1_HI                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_NUM_PRIM_NEEDED_COUNT1_LO {
    struct {
        unsigned int NUM_PRIM_NEEDED_CNT1_LO                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_NUM_PRIM_NEEDED_COUNT2_HI {
    struct {
        unsigned int NUM_PRIM_NEEDED_CNT2_HI                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_NUM_PRIM_NEEDED_COUNT2_LO {
    struct {
        unsigned int NUM_PRIM_NEEDED_CNT2_LO                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_NUM_PRIM_NEEDED_COUNT3_HI {
    struct {
        unsigned int NUM_PRIM_NEEDED_CNT3_HI                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_NUM_PRIM_NEEDED_COUNT3_LO {
    struct {
        unsigned int NUM_PRIM_NEEDED_CNT3_LO                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_NUM_PRIM_WRITTEN_COUNT0_HI {
    struct {
        unsigned int NUM_PRIM_WRITTEN_CNT0_HI                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_NUM_PRIM_WRITTEN_COUNT0_LO {
    struct {
        unsigned int NUM_PRIM_WRITTEN_CNT0_LO                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_NUM_PRIM_WRITTEN_COUNT1_HI {
    struct {
        unsigned int NUM_PRIM_WRITTEN_CNT1_HI                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_NUM_PRIM_WRITTEN_COUNT1_LO {
    struct {
        unsigned int NUM_PRIM_WRITTEN_CNT1_LO                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_NUM_PRIM_WRITTEN_COUNT2_HI {
    struct {
        unsigned int NUM_PRIM_WRITTEN_CNT2_HI                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_NUM_PRIM_WRITTEN_COUNT2_LO {
    struct {
        unsigned int NUM_PRIM_WRITTEN_CNT2_LO                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_NUM_PRIM_WRITTEN_COUNT3_HI {
    struct {
        unsigned int NUM_PRIM_WRITTEN_CNT3_HI                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_NUM_PRIM_WRITTEN_COUNT3_LO {
    struct {
        unsigned int NUM_PRIM_WRITTEN_CNT3_LO                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PA_CINVOC_COUNT_HI {
    struct {
        unsigned int CINVOC_COUNT_HI                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PA_CINVOC_COUNT_LO {
    struct {
        unsigned int CINVOC_COUNT_LO                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PA_CPRIM_COUNT_HI {
    struct {
        unsigned int CPRIM_COUNT_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PA_CPRIM_COUNT_LO {
    struct {
        unsigned int CPRIM_COUNT_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PERFMON_CNTL {
    struct {
        unsigned int PERFMON_STATE                                                :  4;
        unsigned int SPM_PERFMON_STATE                                            :  4;
        unsigned int PERFMON_ENABLE_MODE                                          :  2;
        unsigned int PERFMON_SAMPLE_ENABLE                                        :  1;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PERFMON_CNTX_CNTL {
    struct {
        unsigned int                                                              : 31;
        unsigned int PERFMON_ENABLE                                               :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PFP_ATOMIC_PREOP_HI {
    struct {
        unsigned int ATOMIC_PREOP_HI                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PFP_ATOMIC_PREOP_LO {
    struct {
        unsigned int ATOMIC_PREOP_LO                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PFP_COMPLETION_STATUS {
    struct {
        unsigned int STATUS                                                       :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PFP_F32_INTERRUPT {
    struct {
        unsigned int                                                              :  1;
        unsigned int PRIV_REG_INT                                                 :  1;
        unsigned int RESERVED_BIT_ERR_INT                                         :  1;
        unsigned int                                                              : 29;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PFP_HEADER_DUMP {
    struct {
        unsigned int PFP_HEADER_DUMP                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PFP_IB_CONTROL {
    struct {
        unsigned int IB_EN                                                        :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PFP_IC_BASE_CNTL {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int ADDRESS_CLAMP                                                :  1;
        unsigned int                                                              : 18;
        unsigned int EXE_DISABLE                                                  :  1;
        unsigned int CACHE_POLICY                                                 :  2;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PFP_IC_BASE_HI {
    struct {
        unsigned int IC_BASE_HI                                                   : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PFP_IC_BASE_LO {
    struct {
        unsigned int                                                              : 12;
        unsigned int IC_BASE_LO                                                   : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PFP_IC_OP_CNTL {
    struct {
        unsigned int INVALIDATE_CACHE                                             :  1;
        unsigned int INVALIDATE_CACHE_COMPLETE                                    :  1;
        unsigned int                                                              :  2;
        unsigned int PRIME_ICACHE                                                 :  1;
        unsigned int ICACHE_PRIMED                                                :  1;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PFP_INSTR_PNTR {
    struct {
        unsigned int INSTR_PNTR                                                   : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PFP_INTR_ROUTINE_START {
    struct {
        unsigned int IR_START                                                     : 13;
        unsigned int                                                              : 19;
    } gfx09;
    struct {
        unsigned int IR_START                                                     : 20;
        unsigned int                                                              : 12;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PFP_JT_STAT {
    struct {
        unsigned int JT_LOADED                                                    :  2;
        unsigned int                                                              : 14;
        unsigned int WR_MASK                                                      :  2;
        unsigned int                                                              : 14;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PFP_LOAD_CONTROL {
    struct {
        unsigned int CONFIG_REG_EN                                                :  1;
        unsigned int CNTX_REG_EN                                                  :  1;
        unsigned int                                                              : 14;
        unsigned int SH_GFX_REG_EN                                                :  1;
        unsigned int                                                              :  7;
        unsigned int SH_CS_REG_EN                                                 :  1;
        unsigned int                                                              :  7;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PFP_METADATA_BASE_ADDR {
    struct {
        unsigned int ADDR_LO                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PFP_METADATA_BASE_ADDR_HI {
    struct {
        unsigned int ADDR_HI                                                      : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PFP_PRGRM_CNTR_START {
    struct {
        unsigned int IP_START                                                     : 13;
        unsigned int                                                              : 19;
    } gfx09;
    struct {
        unsigned int IP_START                                                     : 20;
        unsigned int                                                              : 12;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PIPEID {
    struct {
        unsigned int PIPE_ID                                                      :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PIPE_STATS_ADDR_HI {
    struct {
        unsigned int PIPE_STATS_ADDR_HI                                           : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PIPE_STATS_ADDR_LO {
    struct {
        unsigned int                                                              :  2;
        unsigned int PIPE_STATS_ADDR_LO                                           : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PIPE_STATS_CONTROL {
    struct {
        unsigned int                                                              : 25;
        unsigned int CACHE_POLICY                                                 :  1;
        unsigned int                                                              :  6;
    } gfx09;
    struct {
        unsigned int                                                              : 25;
        unsigned int CACHE_POLICY                                                 :  2;
        unsigned int                                                              :  5;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PIPE_STATS_DOORBELL {
    struct {
        unsigned int                                                              :  2;
        unsigned int DOORBELL_OFFSET                                              : 26;
        unsigned int                                                              :  4;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PQ_STATUS {
    struct {
        unsigned int DOORBELL_UPDATED                                             :  1;
        unsigned int DOORBELL_ENABLE                                              :  1;
        unsigned int                                                              : 30;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  2;
        unsigned int DOORBELL_UPDATED_EN                                          :  1;
        unsigned int                                                              : 29;
    } gfx10;
    struct {
        unsigned int                                                              :  3;
        unsigned int DOORBELL_UPDATED_MODE                                        :  1;
        unsigned int                                                              : 28;
    } gfx101Plus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PQ_WPTR_POLL_CNTL {
    struct {
        unsigned int PERIOD                                                       :  8;
        unsigned int                                                              : 21;
        unsigned int DISABLE_PEND_REQ_ONE_SHOT                                    :  1;
        unsigned int POLL_ACTIVE                                                  :  1;
        unsigned int EN                                                           :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PQ_WPTR_POLL_CNTL1 {
    struct {
        unsigned int QUEUE_MASK                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PRED_NOT_VISIBLE {
    struct {
        unsigned int NOT_VISIBLE                                                  :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PROCESS_QUANTUM {
    struct {
        unsigned int QUANTUM_DURATION                                             : 28;
        unsigned int TIMER_EXPIRED                                                :  1;
        unsigned int QUANTUM_SCALE                                                :  2;
        unsigned int QUANTUM_EN                                                   :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PRT_LOD_STATS_CNTL0 {
    struct {
        unsigned int BU_SIZE                                                      : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PRT_LOD_STATS_CNTL1 {
    struct {
        unsigned int BASE_LO                                                      : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PRT_LOD_STATS_CNTL2 {
    struct {
        unsigned int BASE_HI                                                      : 10;
        unsigned int                                                              : 22;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PRT_LOD_STATS_CNTL3 {
    struct {
        unsigned int                                                              :  2;
        unsigned int INTERVAL                                                     :  8;
        unsigned int RESET_CNT                                                    :  8;
        unsigned int RESET_FORCE                                                  :  1;
        unsigned int REPORT_AND_RESET                                             :  1;
        unsigned int                                                              :  3;
        unsigned int MC_VMID                                                      :  4;
        unsigned int                                                              :  5;
    } most;
    struct {
        unsigned int                                                              : 28;
        unsigned int CACHE_POLICY                                                 :  1;
        unsigned int                                                              :  3;
    } gfx09_0;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PWR_CNTL {
    struct {
        unsigned int GFX_CLK_HALT_ME0_PIPE0                                       :  1;
        unsigned int GFX_CLK_HALT_ME0_PIPE1                                       :  1;
        unsigned int                                                              :  6;
        unsigned int CMP_CLK_HALT_ME1_PIPE0                                       :  1;
        unsigned int CMP_CLK_HALT_ME1_PIPE1                                       :  1;
        unsigned int CMP_CLK_HALT_ME1_PIPE2                                       :  1;
        unsigned int CMP_CLK_HALT_ME1_PIPE3                                       :  1;
        unsigned int                                                              :  4;
        unsigned int CMP_CLK_HALT_ME2_PIPE0                                       :  1;
        unsigned int CMP_CLK_HALT_ME2_PIPE1                                       :  1;
        unsigned int CMP_CLK_HALT_ME2_PIPE2                                       :  1;
        unsigned int CMP_CLK_HALT_ME2_PIPE3                                       :  1;
        unsigned int                                                              : 12;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 20;
        unsigned int CMP_CLK_HALT_ME3_PIPE0                                       :  1;
        unsigned int CMP_CLK_HALT_ME3_PIPE1                                       :  1;
        unsigned int CMP_CLK_HALT_ME3_PIPE2                                       :  1;
        unsigned int CMP_CLK_HALT_ME3_PIPE3                                       :  1;
        unsigned int                                                              :  8;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_QUEUE_THRESHOLDS {
    struct {
        unsigned int ROQ_IB1_START                                                :  6;
        unsigned int                                                              :  2;
        unsigned int ROQ_IB2_START                                                :  6;
        unsigned int                                                              : 18;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB0_ACTIVE {
    struct {
        unsigned int ACTIVE                                                       :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB0_BASE {
    struct {
        unsigned int RB_BASE                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB0_BASE_HI {
    struct {
        unsigned int RB_BASE_HI                                                   :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB0_BUFSZ_MASK {
    struct {
        unsigned int DATA                                                         : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB0_CNTL {
    struct {
        unsigned int RB_BUFSZ                                                     :  6;
        unsigned int                                                              :  2;
        unsigned int RB_BLKSZ                                                     :  6;
        unsigned int                                                              :  6;
        unsigned int MIN_AVAILSZ                                                  :  2;
        unsigned int MIN_IB_AVAILSZ                                               :  2;
        unsigned int                                                              :  3;
        unsigned int RB_NO_UPDATE                                                 :  1;
        unsigned int                                                              :  3;
        unsigned int RB_RPTR_WR_ENA                                               :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 17;
        unsigned int BUF_SWAP                                                     :  2;
        unsigned int                                                              :  5;
        unsigned int CACHE_POLICY                                                 :  1;
        unsigned int                                                              :  7;
    } gfx09;
    struct {
        unsigned int                                                              : 24;
        unsigned int CACHE_POLICY                                                 :  2;
        unsigned int RB_VOLATILE                                                  :  1;
        unsigned int                                                              :  1;
        unsigned int RB_EXE                                                       :  1;
        unsigned int                                                              :  3;
    } gfx10;
    struct {
        unsigned int                                                              : 30;
        unsigned int CE_HQD_NEQ_RB_HQD                                            :  1;
        unsigned int                                                              :  1;
    } gfx101Plus;
    struct {
        unsigned int                                                              : 16;
        unsigned int BUF_SWAP                                                     :  2;
        unsigned int                                                              : 14;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB0_RPTR {
    struct {
        unsigned int RB_RPTR                                                      : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB0_RPTR_ADDR {
    struct {
        unsigned int                                                              :  2;
        unsigned int RB_RPTR_ADDR                                                 : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB0_RPTR_ADDR_HI {
    struct {
        unsigned int RB_RPTR_ADDR_HI                                              : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB0_WPTR {
    struct {
        unsigned int RB_WPTR                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB0_WPTR_HI {
    struct {
        unsigned int RB_WPTR                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB1_ACTIVE {
    struct {
        unsigned int ACTIVE                                                       :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB1_BASE {
    struct {
        unsigned int RB_BASE                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB1_BASE_HI {
    struct {
        unsigned int RB_BASE_HI                                                   :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB1_BUFSZ_MASK {
    struct {
        unsigned int DATA                                                         : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB1_CNTL {
    struct {
        unsigned int RB_BUFSZ                                                     :  6;
        unsigned int                                                              :  2;
        unsigned int RB_BLKSZ                                                     :  6;
        unsigned int                                                              :  6;
        unsigned int MIN_AVAILSZ                                                  :  2;
        unsigned int MIN_IB_AVAILSZ                                               :  2;
        unsigned int                                                              :  3;
        unsigned int RB_NO_UPDATE                                                 :  1;
        unsigned int                                                              :  3;
        unsigned int RB_RPTR_WR_ENA                                               :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 24;
        unsigned int CACHE_POLICY                                                 :  1;
        unsigned int                                                              :  7;
    } gfx09;
    struct {
        unsigned int                                                              : 24;
        unsigned int CACHE_POLICY                                                 :  2;
        unsigned int RB_VOLATILE                                                  :  1;
        unsigned int                                                              :  1;
        unsigned int RB_EXE                                                       :  1;
        unsigned int                                                              :  3;
    } gfx10;
    struct {
        unsigned int                                                              : 29;
        unsigned int KMD_QUEUE                                                    :  1;
        unsigned int CE_HQD_NEQ_RB_HQD                                            :  1;
        unsigned int                                                              :  1;
    } gfx101Plus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB1_RPTR {
    struct {
        unsigned int RB_RPTR                                                      : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB1_RPTR_ADDR {
    struct {
        unsigned int                                                              :  2;
        unsigned int RB_RPTR_ADDR                                                 : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB1_RPTR_ADDR_HI {
    struct {
        unsigned int RB_RPTR_ADDR_HI                                              : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB1_WPTR {
    struct {
        unsigned int RB_WPTR                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB1_WPTR_HI {
    struct {
        unsigned int RB_WPTR                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB2_BASE {
    struct {
        unsigned int RB_BASE                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB2_CNTL {
    struct {
        unsigned int RB_BUFSZ                                                     :  6;
        unsigned int                                                              :  2;
        unsigned int RB_BLKSZ                                                     :  6;
        unsigned int                                                              :  6;
        unsigned int MIN_AVAILSZ                                                  :  2;
        unsigned int MIN_IB_AVAILSZ                                               :  2;
        unsigned int                                                              :  3;
        unsigned int RB_NO_UPDATE                                                 :  1;
        unsigned int                                                              :  3;
        unsigned int RB_RPTR_WR_ENA                                               :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 24;
        unsigned int CACHE_POLICY                                                 :  1;
        unsigned int                                                              :  7;
    } gfx09;
    struct {
        unsigned int                                                              : 24;
        unsigned int CACHE_POLICY                                                 :  2;
        unsigned int RB_VOLATILE                                                  :  1;
        unsigned int                                                              :  1;
        unsigned int RB_EXE                                                       :  1;
        unsigned int                                                              :  3;
    } gfx10;
    struct {
        unsigned int                                                              : 29;
        unsigned int KMD_QUEUE                                                    :  1;
        unsigned int CE_HQD_NEQ_RB_HQD                                            :  1;
        unsigned int                                                              :  1;
    } gfx101Plus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB2_RPTR {
    struct {
        unsigned int RB_RPTR                                                      : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB2_RPTR_ADDR {
    struct {
        unsigned int                                                              :  2;
        unsigned int RB_RPTR_ADDR                                                 : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB2_RPTR_ADDR_HI {
    struct {
        unsigned int RB_RPTR_ADDR_HI                                              : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB2_WPTR {
    struct {
        unsigned int RB_WPTR                                                      : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB_ACTIVE {
    struct {
        unsigned int ACTIVE                                                       :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB_BASE {
    struct {
        unsigned int RB_BASE                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB_BUFSZ_MASK {
    struct {
        unsigned int DATA                                                         : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB_CNTL {
    struct {
        unsigned int RB_BUFSZ                                                     :  6;
        unsigned int                                                              :  2;
        unsigned int RB_BLKSZ                                                     :  6;
        unsigned int                                                              :  6;
        unsigned int MIN_AVAILSZ                                                  :  2;
        unsigned int MIN_IB_AVAILSZ                                               :  2;
        unsigned int                                                              :  3;
        unsigned int RB_NO_UPDATE                                                 :  1;
        unsigned int                                                              :  3;
        unsigned int RB_RPTR_WR_ENA                                               :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 24;
        unsigned int CACHE_POLICY                                                 :  1;
        unsigned int                                                              :  7;
    } gfx09;
    struct {
        unsigned int                                                              : 24;
        unsigned int CACHE_POLICY                                                 :  2;
        unsigned int RB_VOLATILE                                                  :  1;
        unsigned int                                                              :  1;
        unsigned int RB_EXE                                                       :  1;
        unsigned int                                                              :  3;
    } gfx10;
    struct {
        unsigned int                                                              : 29;
        unsigned int KMD_QUEUE                                                    :  1;
        unsigned int CE_HQD_NEQ_RB_HQD                                            :  1;
        unsigned int                                                              :  1;
    } gfx101Plus;
    struct {
        unsigned int                                                              : 16;
        unsigned int BUF_SWAP                                                     :  2;
        unsigned int                                                              : 14;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB_DOORBELL_CLEAR {
    struct {
        unsigned int MAPPED_QUEUE                                                 :  3;
        unsigned int                                                              :  5;
        unsigned int MAPPED_QUE_DOORBELL_EN_CLEAR                                 :  1;
        unsigned int MAPPED_QUE_DOORBELL_HIT_CLEAR                                :  1;
        unsigned int MASTER_DOORBELL_EN_CLEAR                                     :  1;
        unsigned int MASTER_DOORBELL_HIT_CLEAR                                    :  1;
        unsigned int QUEUES_DOORBELL_EN_CLEAR                                     :  1;
        unsigned int QUEUES_DOORBELL_HIT_CLEAR                                    :  1;
        unsigned int                                                              : 18;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB_DOORBELL_CONTROL {
    struct {
        unsigned int                                                              :  1;
        unsigned int DOORBELL_BIF_DROP                                            :  1;
        unsigned int DOORBELL_OFFSET                                              : 26;
        unsigned int                                                              :  2;
        unsigned int DOORBELL_EN                                                  :  1;
        unsigned int DOORBELL_HIT                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB_DOORBELL_CONTROL_SCH_0 {
    struct {
        unsigned int                                                              :  2;
        unsigned int DOORBELL_OFFSET                                              : 26;
        unsigned int                                                              :  2;
        unsigned int DOORBELL_EN                                                  :  1;
        unsigned int DOORBELL_HIT                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB_DOORBELL_CONTROL_SCH_1 {
    struct {
        unsigned int                                                              :  2;
        unsigned int DOORBELL_OFFSET                                              : 26;
        unsigned int                                                              :  2;
        unsigned int DOORBELL_EN                                                  :  1;
        unsigned int DOORBELL_HIT                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB_DOORBELL_CONTROL_SCH_2 {
    struct {
        unsigned int                                                              :  2;
        unsigned int DOORBELL_OFFSET                                              : 26;
        unsigned int                                                              :  2;
        unsigned int DOORBELL_EN                                                  :  1;
        unsigned int DOORBELL_HIT                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB_DOORBELL_CONTROL_SCH_3 {
    struct {
        unsigned int                                                              :  2;
        unsigned int DOORBELL_OFFSET                                              : 26;
        unsigned int                                                              :  2;
        unsigned int DOORBELL_EN                                                  :  1;
        unsigned int DOORBELL_HIT                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB_DOORBELL_CONTROL_SCH_4 {
    struct {
        unsigned int                                                              :  2;
        unsigned int DOORBELL_OFFSET                                              : 26;
        unsigned int                                                              :  2;
        unsigned int DOORBELL_EN                                                  :  1;
        unsigned int DOORBELL_HIT                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB_DOORBELL_CONTROL_SCH_5 {
    struct {
        unsigned int                                                              :  2;
        unsigned int DOORBELL_OFFSET                                              : 26;
        unsigned int                                                              :  2;
        unsigned int DOORBELL_EN                                                  :  1;
        unsigned int DOORBELL_HIT                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB_DOORBELL_CONTROL_SCH_6 {
    struct {
        unsigned int                                                              :  2;
        unsigned int DOORBELL_OFFSET                                              : 26;
        unsigned int                                                              :  2;
        unsigned int DOORBELL_EN                                                  :  1;
        unsigned int DOORBELL_HIT                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB_DOORBELL_CONTROL_SCH_7 {
    struct {
        unsigned int                                                              :  2;
        unsigned int DOORBELL_OFFSET                                              : 26;
        unsigned int                                                              :  2;
        unsigned int DOORBELL_EN                                                  :  1;
        unsigned int DOORBELL_HIT                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB_DOORBELL_RANGE_LOWER {
    struct {
        unsigned int                                                              :  2;
        unsigned int DOORBELL_RANGE_LOWER                                         : 26;
        unsigned int                                                              :  4;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB_DOORBELL_RANGE_UPPER {
    struct {
        unsigned int                                                              :  2;
        unsigned int DOORBELL_RANGE_UPPER                                         : 26;
        unsigned int                                                              :  4;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB_OFFSET {
    struct {
        unsigned int RB_OFFSET                                                    : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB_RPTR {
    struct {
        unsigned int RB_RPTR                                                      : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB_RPTR_ADDR {
    struct {
        unsigned int                                                              :  2;
        unsigned int RB_RPTR_ADDR                                                 : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB_RPTR_ADDR_HI {
    struct {
        unsigned int RB_RPTR_ADDR_HI                                              : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB_RPTR_WR {
    struct {
        unsigned int RB_RPTR_WR                                                   : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB_STATUS {
    struct {
        unsigned int DOORBELL_UPDATED                                             :  1;
        unsigned int DOORBELL_ENABLE                                              :  1;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB_VMID {
    struct {
        unsigned int RB0_VMID                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int RB1_VMID                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int RB2_VMID                                                     :  4;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB_WPTR {
    struct {
        unsigned int RB_WPTR                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB_WPTR_DELAY {
    struct {
        unsigned int PRE_WRITE_TIMER                                              : 28;
        unsigned int PRE_WRITE_LIMIT                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB_WPTR_HI {
    struct {
        unsigned int RB_WPTR                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB_WPTR_POLL_ADDR_HI {
    struct {
        unsigned int RB_WPTR_POLL_ADDR_HI                                         : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB_WPTR_POLL_ADDR_LO {
    struct {
        unsigned int                                                              :  2;
        unsigned int RB_WPTR_POLL_ADDR_LO                                         : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RB_WPTR_POLL_CNTL {
    struct {
        unsigned int POLL_FREQUENCY                                               : 16;
        unsigned int IDLE_POLL_COUNT                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RING0_PRIORITY {
    struct {
        unsigned int PRIORITY                                                     :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RING1_PRIORITY {
    struct {
        unsigned int PRIORITY                                                     :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RING2_PRIORITY {
    struct {
        unsigned int PRIORITY                                                     :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RINGID {
    struct {
        unsigned int RINGID                                                       :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_RING_PRIORITY_CNTS {
    struct {
        unsigned int PRIORITY1_CNT                                                :  8;
        unsigned int PRIORITY2A_CNT                                               :  8;
        unsigned int PRIORITY2B_CNT                                               :  8;
        unsigned int PRIORITY3_CNT                                                :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ROQ1_THRESHOLDS {
    struct {
        unsigned int RB1_START                                                    :  8;
        unsigned int RB2_START                                                    :  8;
        unsigned int R0_IB1_START                                                 :  8;
        unsigned int R1_IB1_START                                                 :  8;
    } gfx09;
    struct {
        unsigned int RB1_START                                                    : 10;
        unsigned int R0_IB1_START                                                 : 10;
        unsigned int R1_IB1_START                                                 : 10;
        unsigned int                                                              :  2;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ROQ2_AVAIL {
    struct {
        unsigned int ROQ_CNT_IB2                                                  : 11;
        unsigned int                                                              : 21;
    } gfx09;
    struct {
        unsigned int ROQ_CNT_IB2                                                  : 12;
        unsigned int                                                              :  4;
        unsigned int ROQ_CNT_DB                                                   : 12;
        unsigned int                                                              :  4;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ROQ2_THRESHOLDS {
    struct {
        unsigned int R2_IB1_START                                                 :  8;
        unsigned int R0_IB2_START                                                 :  8;
        unsigned int R1_IB2_START                                                 :  8;
        unsigned int R2_IB2_START                                                 :  8;
    } gfx09;
    struct {
        unsigned int R0_IB2_START                                                 : 10;
        unsigned int R1_IB2_START                                                 : 10;
        unsigned int                                                              : 12;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ROQ3_THRESHOLDS {
    struct {
        unsigned int R0_DB_START                                                  : 10;
        unsigned int R1_DB_START                                                  : 10;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ROQ_AVAIL {
    struct {
        unsigned int ROQ_CNT_RING                                                 : 11;
        unsigned int                                                              :  5;
        unsigned int ROQ_CNT_IB1                                                  : 11;
        unsigned int                                                              :  5;
    } gfx09;
    struct {
        unsigned int ROQ_CNT_RING                                                 : 12;
        unsigned int                                                              :  4;
        unsigned int ROQ_CNT_IB1                                                  : 12;
        unsigned int                                                              :  4;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ROQ_DB_STAT {
    struct {
        unsigned int ROQ_RPTR_DB                                                  : 12;
        unsigned int                                                              :  4;
        unsigned int ROQ_WPTR_DB                                                  : 12;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ROQ_IB1_STAT {
    struct {
        unsigned int ROQ_RPTR_INDIRECT1                                           : 10;
        unsigned int                                                              :  6;
        unsigned int ROQ_WPTR_INDIRECT1                                           : 10;
        unsigned int                                                              :  6;
    } gfx09;
    struct {
        unsigned int ROQ_RPTR_INDIRECT1                                           : 12;
        unsigned int                                                              :  4;
        unsigned int ROQ_WPTR_INDIRECT1                                           : 12;
        unsigned int                                                              :  4;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ROQ_IB2_STAT {
    struct {
        unsigned int ROQ_RPTR_INDIRECT2                                           : 10;
        unsigned int                                                              :  6;
        unsigned int ROQ_WPTR_INDIRECT2                                           : 10;
        unsigned int                                                              :  6;
    } gfx09;
    struct {
        unsigned int ROQ_RPTR_INDIRECT2                                           : 12;
        unsigned int                                                              :  4;
        unsigned int ROQ_WPTR_INDIRECT2                                           : 12;
        unsigned int                                                              :  4;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ROQ_RB_STAT {
    struct {
        unsigned int ROQ_RPTR_PRIMARY                                             : 10;
        unsigned int                                                              :  6;
        unsigned int ROQ_WPTR_PRIMARY                                             : 10;
        unsigned int                                                              :  6;
    } gfx09;
    struct {
        unsigned int ROQ_RPTR_PRIMARY                                             : 12;
        unsigned int                                                              :  4;
        unsigned int ROQ_WPTR_PRIMARY                                             : 12;
        unsigned int                                                              :  4;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ROQ_THRESHOLDS {
    struct {
        unsigned int IB1_START                                                    :  8;
        unsigned int IB2_START                                                    :  8;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_SAMPLE_STATUS {
    struct {
        unsigned int Z_PASS_ACITVE                                                :  1;
        unsigned int STREAMOUT_ACTIVE                                             :  1;
        unsigned int PIPELINE_ACTIVE                                              :  1;
        unsigned int STIPPLE_ACTIVE                                               :  1;
        unsigned int VGT_BUFFERS_ACTIVE                                           :  1;
        unsigned int                                                              :  1;
        unsigned int DRAW_INDIRECT_ACTIVE                                         :  1;
        unsigned int DISP_INDIRECT_ACTIVE                                         :  1;
        unsigned int                                                              : 24;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  5;
        unsigned int SCREEN_EXT_ACTIVE                                            :  1;
        unsigned int                                                              : 26;
    } core;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_SCRATCH_DATA {
    struct {
        unsigned int SCRATCH_DATA                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_SCRATCH_INDEX {
    struct {
        unsigned int SCRATCH_INDEX                                                :  8;
        unsigned int                                                              : 24;
    } most;
    struct {
        unsigned int                                                              : 31;
        unsigned int SCRATCH_INDEX_64BIT_MODE                                     :  1;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_SC_PSINVOC_COUNT0_HI {
    struct {
        unsigned int PSINVOC_COUNT0_HI                                            : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_SC_PSINVOC_COUNT0_LO {
    struct {
        unsigned int PSINVOC_COUNT0_LO                                            : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_SC_PSINVOC_COUNT1_HI {
    struct {
        unsigned int OBSOLETE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_SC_PSINVOC_COUNT1_LO {
    struct {
        unsigned int OBSOLETE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_SD_CNTL {
    struct {
        unsigned int CPF_EN                                                       :  1;
        unsigned int CPG_EN                                                       :  1;
        unsigned int CPC_EN                                                       :  1;
        unsigned int RLC_EN                                                       :  1;
        unsigned int SPI_EN                                                       :  1;
        unsigned int                                                              :  4;
        unsigned int EA_EN                                                        :  1;
        unsigned int                                                              : 22;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  8;
        unsigned int RMI_EN                                                       :  1;
        unsigned int                                                              : 23;
    } most;
    struct {
        unsigned int                                                              :  5;
        unsigned int WD_EN                                                        :  1;
        unsigned int IA_EN                                                        :  1;
        unsigned int PA_EN                                                        :  1;
        unsigned int                                                              : 24;
    } gfx09;
    struct {
        unsigned int                                                              :  5;
        unsigned int GE_EN                                                        :  1;
        unsigned int UTCL1_EN                                                     :  1;
        unsigned int                                                              :  3;
        unsigned int SDMA_EN                                                      :  1;
        unsigned int                                                              : 20;
        unsigned int SD_VMIDVEC_OVERRIDE                                          :  1;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_SEM_DOORBELL {
    struct {
        unsigned int                                                              :  2;
        unsigned int DOORBELL_OFFSET                                              : 26;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_SEM_WAIT_TIMER {
    struct {
        unsigned int SEM_WAIT_TIMER                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_SIG_SEM_ADDR_HI {
    struct {
        unsigned int SEM_ADDR_HI                                                  : 16;
        unsigned int SEM_USE_MAILBOX                                              :  1;
        unsigned int                                                              :  3;
        unsigned int SEM_SIGNAL_TYPE                                              :  1;
        unsigned int                                                              :  3;
        unsigned int SEM_CLIENT_CODE                                              :  2;
        unsigned int                                                              :  3;
        unsigned int SEM_SELECT                                                   :  3;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_SIG_SEM_ADDR_LO {
    struct {
        unsigned int                                                              :  3;
        unsigned int SEM_ADDR_LO                                                  : 29;
    } bits, bitfields;
    struct {
        unsigned int SEM_ADDR_SWAP                                                :  2;
        unsigned int                                                              : 30;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_SOFT_RESET_CNTL {
    struct {
        unsigned int CMP_ONLY_SOFT_RESET                                          :  1;
        unsigned int GFX_ONLY_SOFT_RESET                                          :  1;
        unsigned int CMP_HQD_REG_RESET                                            :  1;
        unsigned int CMP_INTR_REG_RESET                                           :  1;
        unsigned int CMP_HQD_QUEUE_DOORBELL_RESET                                 :  1;
        unsigned int GFX_RB_DOORBELL_RESET                                        :  1;
        unsigned int GFX_INTR_REG_RESET                                           :  1;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_STALLED_STAT1 {
    struct {
        unsigned int RBIU_TO_DMA_NOT_RDY_TO_RCV                                   :  1;
        unsigned int                                                              :  9;
        unsigned int ME_HAS_ACTIVE_CE_BUFFER_FLAG                                 :  1;
        unsigned int ME_HAS_ACTIVE_DE_BUFFER_FLAG                                 :  1;
        unsigned int ME_STALLED_ON_TC_WR_CONFIRM                                  :  1;
        unsigned int ME_STALLED_ON_ATOMIC_RTN_DATA                                :  1;
        unsigned int ME_WAITING_ON_TC_READ_DATA                                   :  1;
        unsigned int ME_WAITING_ON_REG_READ_DATA                                  :  1;
        unsigned int                                                              :  7;
        unsigned int RCIU_WAITING_ON_GDS_FREE                                     :  1;
        unsigned int RCIU_WAITING_ON_GRBM_FREE                                    :  1;
        unsigned int RCIU_WAITING_ON_VGT_FREE                                     :  1;
        unsigned int RCIU_STALLED_ON_ME_READ                                      :  1;
        unsigned int RCIU_STALLED_ON_DMA_READ                                     :  1;
        unsigned int RCIU_STALLED_ON_APPEND_READ                                  :  1;
        unsigned int RCIU_HALTED_BY_REG_VIOLATION                                 :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  2;
        unsigned int RBIU_TO_SEM_NOT_RDY_TO_RCV                                   :  1;
        unsigned int                                                              :  1;
        unsigned int RBIU_TO_MEMWR_NOT_RDY_TO_RCV                                 :  1;
        unsigned int                                                              : 27;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_STALLED_STAT2 {
    struct {
        unsigned int PFP_TO_CSF_NOT_RDY_TO_RCV                                    :  1;
        unsigned int PFP_TO_MEQ_NOT_RDY_TO_RCV                                    :  1;
        unsigned int PFP_TO_RCIU_NOT_RDY_TO_RCV                                   :  1;
        unsigned int                                                              :  1;
        unsigned int PFP_TO_VGT_WRITES_PENDING                                    :  1;
        unsigned int PFP_RCIU_READ_PENDING                                        :  1;
        unsigned int                                                              :  2;
        unsigned int PFP_WAITING_ON_BUFFER_DATA                                   :  1;
        unsigned int ME_WAIT_ON_CE_COUNTER                                        :  1;
        unsigned int ME_WAIT_ON_AVAIL_BUFFER                                      :  1;
        unsigned int GFX_CNTX_NOT_AVAIL_TO_ME                                     :  1;
        unsigned int ME_RCIU_NOT_RDY_TO_RCV                                       :  1;
        unsigned int ME_TO_CONST_NOT_RDY_TO_RCV                                   :  1;
        unsigned int ME_WAITING_DATA_FROM_PFP                                     :  1;
        unsigned int ME_WAITING_ON_PARTIAL_FLUSH                                  :  1;
        unsigned int MEQ_TO_ME_NOT_RDY_TO_RCV                                     :  1;
        unsigned int STQ_TO_ME_NOT_RDY_TO_RCV                                     :  1;
        unsigned int ME_WAITING_DATA_FROM_STQ                                     :  1;
        unsigned int PFP_STALLED_ON_TC_WR_CONFIRM                                 :  1;
        unsigned int PFP_STALLED_ON_ATOMIC_RTN_DATA                               :  1;
        unsigned int                                                              :  2;
        unsigned int STRMO_WR_OF_PRIM_DATA_PENDING                                :  1;
        unsigned int PIPE_STATS_WR_DATA_PENDING                                   :  1;
        unsigned int APPEND_RDY_WAIT_ON_CS_DONE                                   :  1;
        unsigned int APPEND_RDY_WAIT_ON_PS_DONE                                   :  1;
        unsigned int APPEND_WAIT_ON_WR_CONFIRM                                    :  1;
        unsigned int APPEND_ACTIVE_PARTITION                                      :  1;
        unsigned int APPEND_WAITING_TO_SEND_MEMWRITE                              :  1;
        unsigned int SURF_SYNC_NEEDS_IDLE_CNTXS                                   :  1;
        unsigned int SURF_SYNC_NEEDS_ALL_CLEAN                                    :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 21;
        unsigned int EOPD_FIFO_NEEDS_SC_EOP_DONE                                  :  1;
        unsigned int EOPD_FIFO_NEEDS_WR_CONFIRM                                   :  1;
        unsigned int                                                              :  9;
    } most;
    struct {
        unsigned int                                                              :  6;
        unsigned int PFP_TO_MEQ_DDID_NOT_RDY_TO_RCV                               :  1;
        unsigned int                                                              : 25;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_STALLED_STAT3 {
    struct {
        unsigned int CE_TO_CSF_NOT_RDY_TO_RCV                                     :  1;
        unsigned int CE_TO_RAM_INIT_FETCHER_NOT_RDY_TO_RCV                        :  1;
        unsigned int CE_WAITING_ON_DATA_FROM_RAM_INIT_FETCHER                     :  1;
        unsigned int CE_TO_RAM_INIT_NOT_RDY                                       :  1;
        unsigned int CE_TO_RAM_DUMP_NOT_RDY                                       :  1;
        unsigned int CE_TO_RAM_WRITE_NOT_RDY                                      :  1;
        unsigned int CE_TO_INC_FIFO_NOT_RDY_TO_RCV                                :  1;
        unsigned int CE_TO_WR_FIFO_NOT_RDY_TO_RCV                                 :  1;
        unsigned int                                                              :  2;
        unsigned int CE_WAITING_ON_BUFFER_DATA                                    :  1;
        unsigned int CE_WAITING_ON_CE_BUFFER_FLAG                                 :  1;
        unsigned int CE_WAITING_ON_DE_COUNTER                                     :  1;
        unsigned int CE_WAITING_ON_DE_COUNTER_UNDERFLOW                           :  1;
        unsigned int TCIU_WAITING_ON_FREE                                         :  1;
        unsigned int TCIU_WAITING_ON_TAGS                                         :  1;
        unsigned int CE_STALLED_ON_TC_WR_CONFIRM                                  :  1;
        unsigned int CE_STALLED_ON_ATOMIC_RTN_DATA                                :  1;
        unsigned int UTCL2IU_WAITING_ON_FREE                                      :  1;
        unsigned int UTCL2IU_WAITING_ON_TAGS                                      :  1;
        unsigned int UTCL1_WAITING_ON_TRANS                                       :  1;
        unsigned int                                                              : 11;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 21;
        unsigned int GCRIU_WAITING_ON_FREE                                        :  1;
        unsigned int                                                              : 10;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_STAT {
    struct {
        unsigned int                                                              :  9;
        unsigned int ROQ_RING_BUSY                                                :  1;
        unsigned int ROQ_INDIRECT1_BUSY                                           :  1;
        unsigned int ROQ_INDIRECT2_BUSY                                           :  1;
        unsigned int ROQ_STATE_BUSY                                               :  1;
        unsigned int DC_BUSY                                                      :  1;
        unsigned int UTCL2IU_BUSY                                                 :  1;
        unsigned int PFP_BUSY                                                     :  1;
        unsigned int MEQ_BUSY                                                     :  1;
        unsigned int ME_BUSY                                                      :  1;
        unsigned int QUERY_BUSY                                                   :  1;
        unsigned int SEMAPHORE_BUSY                                               :  1;
        unsigned int INTERRUPT_BUSY                                               :  1;
        unsigned int SURFACE_SYNC_BUSY                                            :  1;
        unsigned int DMA_BUSY                                                     :  1;
        unsigned int RCIU_BUSY                                                    :  1;
        unsigned int SCRATCH_RAM_BUSY                                             :  1;
        unsigned int                                                              :  1;
        unsigned int CE_BUSY                                                      :  1;
        unsigned int TCIU_BUSY                                                    :  1;
        unsigned int ROQ_CE_RING_BUSY                                             :  1;
        unsigned int ROQ_CE_INDIRECT1_BUSY                                        :  1;
        unsigned int ROQ_CE_INDIRECT2_BUSY                                        :  1;
        unsigned int CP_BUSY                                                      :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  5;
        unsigned int ROQ_DB_BUSY                                                  :  1;
        unsigned int ROQ_CE_DB_BUSY                                               :  1;
        unsigned int                                                              : 18;
        unsigned int GCRIU_BUSY                                                   :  1;
        unsigned int                                                              :  6;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_STQ_AVAIL {
    struct {
        unsigned int STQ_CNT                                                      :  9;
        unsigned int                                                              : 23;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_STQ_STAT {
    struct {
        unsigned int STQ_RPTR                                                     : 10;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_STQ_THRESHOLDS {
    struct {
        unsigned int STQ0_START                                                   :  8;
        unsigned int STQ1_START                                                   :  8;
        unsigned int STQ2_START                                                   :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_STQ_WR_STAT {
    struct {
        unsigned int STQ_WPTR                                                     : 10;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_STREAM_OUT_ADDR_HI {
    struct {
        unsigned int STREAM_OUT_ADDR_HI                                           : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_STREAM_OUT_ADDR_LO {
    struct {
        unsigned int                                                              :  2;
        unsigned int STREAM_OUT_ADDR_LO                                           : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_STREAM_OUT_CONTROL {
    struct {
        unsigned int                                                              : 25;
        unsigned int CACHE_POLICY                                                 :  1;
        unsigned int                                                              :  6;
    } gfx09;
    struct {
        unsigned int                                                              : 25;
        unsigned int CACHE_POLICY                                                 :  2;
        unsigned int                                                              :  5;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_STREAM_OUT_DOORBELL {
    struct {
        unsigned int                                                              :  2;
        unsigned int DOORBELL_OFFSET                                              : 26;
        unsigned int                                                              :  4;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_STRMOUT_CNTL {
    struct {
        unsigned int OFFSET_UPDATE_DONE                                           :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ST_BASE_HI {
    struct {
        unsigned int ST_BASE_HI                                                   : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ST_BASE_LO {
    struct {
        unsigned int                                                              :  2;
        unsigned int ST_BASE_LO                                                   : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ST_BUFSZ {
    struct {
        unsigned int ST_BUFSZ                                                     : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ST_CMD_BUFSZ {
    struct {
        unsigned int ST_CMD_REQSZ                                                 : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_SUSPEND_CNTL {
    struct {
        unsigned int SUSPEND_MODE                                                 :  1;
        unsigned int SUSPEND_ENABLE                                               :  1;
        unsigned int RESUME_LOCK                                                  :  1;
        unsigned int ACE_SUSPEND_ACTIVE                                           :  1;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_SUSPEND_RESUME_REQ {
    struct {
        unsigned int SUSPEND_REQ                                                  :  1;
        unsigned int RESUME_REQ                                                   :  1;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_VGT_CSINVOC_COUNT_HI {
    struct {
        unsigned int CSINVOC_COUNT_HI                                             : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_VGT_CSINVOC_COUNT_LO {
    struct {
        unsigned int CSINVOC_COUNT_LO                                             : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_VGT_DSINVOC_COUNT_HI {
    struct {
        unsigned int DSINVOC_COUNT_HI                                             : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_VGT_DSINVOC_COUNT_LO {
    struct {
        unsigned int DSINVOC_COUNT_LO                                             : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_VGT_GSINVOC_COUNT_HI {
    struct {
        unsigned int GSINVOC_COUNT_HI                                             : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_VGT_GSINVOC_COUNT_LO {
    struct {
        unsigned int GSINVOC_COUNT_LO                                             : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_VGT_GSPRIM_COUNT_HI {
    struct {
        unsigned int GSPRIM_COUNT_HI                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_VGT_GSPRIM_COUNT_LO {
    struct {
        unsigned int GSPRIM_COUNT_LO                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_VGT_HSINVOC_COUNT_HI {
    struct {
        unsigned int HSINVOC_COUNT_HI                                             : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_VGT_HSINVOC_COUNT_LO {
    struct {
        unsigned int HSINVOC_COUNT_LO                                             : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_VGT_IAPRIM_COUNT_HI {
    struct {
        unsigned int IAPRIM_COUNT_HI                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_VGT_IAPRIM_COUNT_LO {
    struct {
        unsigned int IAPRIM_COUNT_LO                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_VGT_IAVERT_COUNT_HI {
    struct {
        unsigned int IAVERT_COUNT_HI                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_VGT_IAVERT_COUNT_LO {
    struct {
        unsigned int IAVERT_COUNT_LO                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_VGT_VSINVOC_COUNT_HI {
    struct {
        unsigned int VSINVOC_COUNT_HI                                             : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_VGT_VSINVOC_COUNT_LO {
    struct {
        unsigned int VSINVOC_COUNT_LO                                             : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_VIRT_STATUS {
    struct {
        unsigned int VIRT_STATUS                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_VMID {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_VMID_PREEMPT {
    struct {
        unsigned int PREEMPT_REQUEST                                              : 16;
        unsigned int VIRT_COMMAND                                                 :  4;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_VMID_RESET {
    struct {
        unsigned int RESET_REQUEST                                                : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 16;
        unsigned int PIPE0_QUEUES                                                 :  8;
        unsigned int PIPE1_QUEUES                                                 :  8;
    } gfx101Plus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_VMID_STATUS {
    struct {
        unsigned int PREEMPT_DE_STATUS                                            : 16;
        unsigned int PREEMPT_CE_STATUS                                            : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_WAIT_REG_MEM_TIMEOUT {
    struct {
        unsigned int WAIT_REG_MEM_TIMEOUT                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_WAIT_SEM_ADDR_HI {
    struct {
        unsigned int SEM_ADDR_HI                                                  : 16;
        unsigned int SEM_USE_MAILBOX                                              :  1;
        unsigned int                                                              :  3;
        unsigned int SEM_SIGNAL_TYPE                                              :  1;
        unsigned int                                                              :  3;
        unsigned int SEM_CLIENT_CODE                                              :  2;
        unsigned int                                                              :  3;
        unsigned int SEM_SELECT                                                   :  3;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_WAIT_SEM_ADDR_LO {
    struct {
        unsigned int                                                              :  3;
        unsigned int SEM_ADDR_LO                                                  : 29;
    } bits, bitfields;
    struct {
        unsigned int SEM_ADDR_SWAP                                                :  2;
        unsigned int                                                              : 30;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CSPRIV_CONNECT {
    struct {
        unsigned int DOORBELL_OFFSET                                              : 16;
        unsigned int                                                              :  5;
        unsigned int QUEUE_ID                                                     :  3;
        unsigned int RELAUNCH_WAVES                                               :  1;
        unsigned int QSWITCH_MODE                                                 :  1;
        unsigned int VMID                                                         :  4;
        unsigned int                                                              :  2;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 18;
        unsigned int TUNNEL_DISP                                                  :  1;
        unsigned int SAME_SIMD_RESUME                                             :  1;
        unsigned int                                                              : 10;
        unsigned int TRACKER_BIT                                                  :  1;
        unsigned int                                                              :  1;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CS_COPY_STATE {
    struct {
        unsigned int SRC_STATE_ID                                                 :  3;
        unsigned int                                                              : 29;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_ALPHA_TO_MASK {
    struct {
        unsigned int ALPHA_TO_MASK_ENABLE                                         :  1;
        unsigned int                                                              :  7;
        unsigned int ALPHA_TO_MASK_OFFSET0                                        :  2;
        unsigned int ALPHA_TO_MASK_OFFSET1                                        :  2;
        unsigned int ALPHA_TO_MASK_OFFSET2                                        :  2;
        unsigned int ALPHA_TO_MASK_OFFSET3                                        :  2;
        unsigned int OFFSET_ROUND                                                 :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_CGTT_CLK_CTRL_0 {
    struct {
        unsigned int ON_DELAY                                                     :  4;
        unsigned int OFF_HYSTERESIS                                               :  8;
        unsigned int RESERVED                                                     :  4;
        unsigned int SOFT_STALL_OVERRIDE7                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE6                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE5                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE4                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE3                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE2                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE1                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE0                                         :  1;
        unsigned int SOFT_OVERRIDE7                                               :  1;
        unsigned int SOFT_OVERRIDE6                                               :  1;
        unsigned int SOFT_OVERRIDE5                                               :  1;
        unsigned int SOFT_OVERRIDE4                                               :  1;
        unsigned int SOFT_OVERRIDE3                                               :  1;
        unsigned int SOFT_OVERRIDE2                                               :  1;
        unsigned int SOFT_OVERRIDE1                                               :  1;
        unsigned int SOFT_OVERRIDE0                                               :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_COUNT_CONTROL {
    struct {
        unsigned int ZPASS_INCREMENT_DISABLE                                      :  1;
        unsigned int PERFECT_ZPASS_COUNTS                                         :  1;
        unsigned int                                                              :  2;
        unsigned int SAMPLE_RATE                                                  :  3;
        unsigned int                                                              :  1;
        unsigned int ZPASS_ENABLE                                                 :  4;
        unsigned int ZFAIL_ENABLE                                                 :  4;
        unsigned int SFAIL_ENABLE                                                 :  4;
        unsigned int DBFAIL_ENABLE                                                :  4;
        unsigned int SLICE_EVEN_ENABLE                                            :  4;
        unsigned int SLICE_ODD_ENABLE                                             :  4;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  2;
        unsigned int DISABLE_CONSERVATIVE_ZPASS_COUNTS                            :  1;
        unsigned int ENHANCED_CONSERVATIVE_ZPASS_COUNTS                           :  1;
        unsigned int                                                              : 28;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_CREDIT_LIMIT {
    struct {
        unsigned int DB_SC_TILE_CREDITS                                           :  5;
        unsigned int DB_SC_QUAD_CREDITS                                           :  5;
        unsigned int DB_CB_LQUAD_CREDITS                                          :  3;
        unsigned int                                                              : 11;
        unsigned int DB_CB_TILE_CREDITS                                           :  7;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_DEBUG {
    struct {
        unsigned int DEBUG_STENCIL_COMPRESS_DISABLE                               :  1;
        unsigned int DEBUG_DEPTH_COMPRESS_DISABLE                                 :  1;
        unsigned int FETCH_FULL_Z_TILE                                            :  1;
        unsigned int FETCH_FULL_STENCIL_TILE                                      :  1;
        unsigned int FORCE_Z_MODE                                                 :  2;
        unsigned int DEBUG_FORCE_DEPTH_READ                                       :  1;
        unsigned int DEBUG_FORCE_STENCIL_READ                                     :  1;
        unsigned int DEBUG_FORCE_HIZ_ENABLE                                       :  2;
        unsigned int DEBUG_FORCE_HIS_ENABLE0                                      :  2;
        unsigned int DEBUG_FORCE_HIS_ENABLE1                                      :  2;
        unsigned int DEBUG_FAST_Z_DISABLE                                         :  1;
        unsigned int DEBUG_FAST_STENCIL_DISABLE                                   :  1;
        unsigned int DEBUG_NOOP_CULL_DISABLE                                      :  1;
        unsigned int DISABLE_SUMM_SQUADS                                          :  1;
        unsigned int DEPTH_CACHE_FORCE_MISS                                       :  1;
        unsigned int DEBUG_FORCE_FULL_Z_RANGE                                     :  2;
        unsigned int NEVER_FREE_Z_ONLY                                            :  1;
        unsigned int ZPASS_COUNTS_LOOK_AT_PIPE_STAT_EVENTS                        :  1;
        unsigned int DISABLE_VPORT_ZPLANE_OPTIMIZATION                            :  1;
        unsigned int DECOMPRESS_AFTER_N_ZPLANES                                   :  4;
        unsigned int ONE_FREE_IN_FLIGHT                                           :  1;
        unsigned int FORCE_MISS_IF_NOT_INFLIGHT                                   :  1;
        unsigned int DISABLE_DEPTH_SURFACE_SYNC                                   :  1;
        unsigned int DISABLE_HTILE_SURFACE_SYNC                                   :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_DEBUG2 {
    struct {
        unsigned int ALLOW_COMPZ_BYTE_MASKING                                     :  1;
        unsigned int DISABLE_TC_ZRANGE_L0_CACHE                                   :  1;
        unsigned int DISABLE_TC_MASK_L0_CACHE                                     :  1;
        unsigned int DTR_ROUND_ROBIN_ARB                                          :  1;
        unsigned int DTR_PREZ_STALLS_FOR_ETF_ROOM                                 :  1;
        unsigned int DISABLE_PREZL_FIFO_STALL                                     :  1;
        unsigned int DISABLE_PREZL_FIFO_STALL_REZ                                 :  1;
        unsigned int ENABLE_VIEWPORT_STALL_ON_ALL                                 :  1;
        unsigned int OPTIMIZE_HIZ_MATCHES_FB_DISABLE                              :  1;
        unsigned int CLK_OFF_DELAY                                                :  5;
        unsigned int                                                              :  3;
        unsigned int DISABLE_NULL_EOT_FORWARDING                                  :  1;
        unsigned int DISABLE_DTT_DATA_FORWARDING                                  :  1;
        unsigned int DISABLE_QUAD_COHERENCY_STALL                                 :  1;
        unsigned int                                                              :  8;
        unsigned int ENABLE_PREZ_OF_REZ_SUMM                                      :  1;
        unsigned int DISABLE_PREZL_VIEWPORT_STALL                                 :  1;
        unsigned int DISABLE_SINGLE_STENCIL_QUAD_SUMM                             :  1;
        unsigned int DISABLE_WRITE_STALL_ON_RDWR_CONFLICT                         :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 14;
        unsigned int DISABLE_TILE_COVERED_FOR_PS_ITER                             :  1;
        unsigned int ENABLE_SUBTILE_GROUPING                                      :  1;
        unsigned int RESERVED                                                     :  1;
        unsigned int                                                              : 15;
    } gfx09;
    struct {
        unsigned int                                                              : 26;
        unsigned int                                                              :  1;
        unsigned int                                                              :  1;
        unsigned int                                                              :  4;
    } gfx09_1x;
    struct {
        unsigned int                                                              : 15;
        unsigned int FULL_TILE_CACHE_EVICT_ON_HALF_FULL                           :  1;
        unsigned int DISABLE_HTILE_PAIRED_PIPES                                   :  1;
        unsigned int                                                              :  7;
        unsigned int FORCE_ITERATE_256                                            :  2;
        unsigned int                                                              :  6;
    } gfx10;
    struct {
        unsigned int                                                              : 14;
        unsigned int DISABLE_TILE_COVERED_FOR_PS_ITER                             :  1;
        unsigned int                                                              : 11;
        unsigned int                                                              :  1;
        unsigned int                                                              :  1;
        unsigned int                                                              :  4;
    } gfx101;
    struct {
        unsigned int                                                              : 20;
        unsigned int FULL_TILE_WAVE_BREAK_MODE                                    :  2;
        unsigned int DUAL_PIPE_REZ_STALL_MANUAL_CONTROL                           :  1;
        unsigned int DUAL_PIPE_REZ_STALL_SELECT_NEW                               :  1;
        unsigned int                                                              :  8;
    } gfx10Core;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_DEBUG3 {
    struct {
        unsigned int DISABLE_CLEAR_ZRANGE_CORRECTION                              :  1;
        unsigned int                                                              :  1;
        unsigned int FORCE_DB_IS_GOOD                                             :  1;
        unsigned int DISABLE_TL_SSO_NULL_SUPPRESSION                              :  1;
        unsigned int DISABLE_HIZ_ON_VPORT_CLAMP                                   :  1;
        unsigned int EQAA_INTERPOLATE_COMP_Z                                      :  1;
        unsigned int EQAA_INTERPOLATE_SRC_Z                                       :  1;
        unsigned int DISABLE_TCP_CAM_BYPASS                                       :  1;
        unsigned int DISABLE_ZCMP_DIRTY_SUPPRESSION                               :  1;
        unsigned int DISABLE_REDUNDANT_PLANE_FLUSHES_OPT                          :  1;
        unsigned int DISABLE_RECOMP_TO_1ZPLANE_WITHOUT_FASTOP                     :  1;
        unsigned int ENABLE_INCOHERENT_EQAA_READS                                 :  1;
        unsigned int DISABLE_OP_Z_DATA_FORWARDING                                 :  1;
        unsigned int DISABLE_OP_DF_BYPASS                                         :  1;
        unsigned int DISABLE_OP_DF_WRITE_COMBINE                                  :  1;
        unsigned int DISABLE_OP_DF_DIRECT_FEEDBACK                                :  1;
        unsigned int                                                              :  1;
        unsigned int SLOW_PREZ_TO_A2M_OMASK_RATE                                  :  1;
        unsigned int DISABLE_OP_S_DATA_FORWARDING                                 :  1;
        unsigned int DISABLE_TC_UPDATE_WRITE_COMBINE                              :  1;
        unsigned int DISABLE_HZ_TC_WRITE_COMBINE                                  :  1;
        unsigned int ENABLE_RECOMP_ZDIRTY_SUPPRESSION_OPT                         :  1;
        unsigned int ENABLE_TC_MA_ROUND_ROBIN_ARB                                 :  1;
        unsigned int DISABLE_RAM_READ_SUPPRESION_ON_FWD                           :  1;
        unsigned int DISABLE_EQAA_A2M_PERF_OPT                                    :  1;
        unsigned int DISABLE_DI_DT_STALL                                          :  1;
        unsigned int ENABLE_DB_PROCESS_RESET                                      :  1;
        unsigned int DISABLE_OVERRASTERIZATION_FIX                                :  1;
        unsigned int DONT_INSERT_CONTEXT_SUSPEND                                  :  1;
        unsigned int                                                              :  3;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  1;
        unsigned int ROUND_ZRANGE_CORRECTION                                      :  1;
        unsigned int                                                              : 14;
        unsigned int ALLOW_RF2P_RW_COLLISION                                      :  1;
        unsigned int                                                              : 12;
        unsigned int DONT_DELETE_CONTEXT_SUSPEND                                  :  1;
        unsigned int DISABLE_4XAA_2P_DELAYED_WRITE                                :  1;
        unsigned int DISABLE_4XAA_2P_INTERLEAVED_PMASK                            :  1;
    } gfx09;
    struct {
        unsigned int                                                              : 16;
        unsigned int DISABLE_SLOCS_PER_CTXT_MATCH                                 :  1;
        unsigned int                                                              : 12;
        unsigned int DELETE_CONTEXT_SUSPEND                                       :  1;
        unsigned int DISABLE_TS_WRITE_L0                                          :  1;
        unsigned int DISABLE_MULTIDTAG_FL_PANIC_REQUIREMENT                       :  1;
    } gfx10;
    struct {
        unsigned int                                                              :  1;
        unsigned int DISABLE_RELOAD_CONTEXT_DRAW_DATA                             :  1;
        unsigned int                                                              : 30;
    } gfx101Plus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_DEBUG4 {
    struct {
        unsigned int DISABLE_QC_Z_MASK_SUMMATION                                  :  1;
        unsigned int DISABLE_QC_STENCIL_MASK_SUMMATION                            :  1;
        unsigned int DISABLE_RESUMM_TO_SINGLE_STENCIL                             :  1;
        unsigned int DISABLE_PREZ_POSTZ_DTILE_CONFLICT_STALL                      :  1;
        unsigned int                                                              :  4;
        unsigned int DFSM_CONVERT_PASSTHROUGH_TO_BYPASS                           :  1;
        unsigned int DISABLE_UNMAPPED_Z_INDICATOR                                 :  1;
        unsigned int DISABLE_UNMAPPED_S_INDICATOR                                 :  1;
        unsigned int DISABLE_UNMAPPED_H_INDICATOR                                 :  1;
        unsigned int                                                              :  1;
        unsigned int DISABLE_DTT_FAST_HTILENACK_LOOKUP                            :  1;
        unsigned int DISABLE_RESCHECK_MEMCOHER_OPTIMIZATION                       :  1;
        unsigned int                                                              : 17;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 19;
        unsigned int DB_EXTRA_DEBUG4                                              : 11;
        unsigned int                                                              :  2;
    } most;
    struct {
        unsigned int                                                              :  4;
        unsigned int DISABLE_4XAA_2P_ZD_HOLDOFF                                   :  1;
        unsigned int ENABLE_A2M_DQUAD_OPTIMIZATION                                :  1;
        unsigned int ENABLE_DBCB_SLOW_FORMAT_COLLAPSE                             :  1;
        unsigned int ALWAYS_ON_RMI_CLK_EN                                         :  1;
        unsigned int                                                              :  4;
        unsigned int DISABLE_SEPARATE_DFSM_CLK                                    :  1;
        unsigned int                                                              :  2;
        unsigned int DISABLE_TS_WRITE_L0                                          :  1;
        unsigned int DISABLE_DYNAMIC_RAM_LIGHT_SLEEP_MODE                         :  1;
        unsigned int DISABLE_HIZ_Q1_TS_COLLISION_DETECT                           :  1;
        unsigned int DISABLE_HIZ_Q2_TS_COLLISION_DETECT                           :  1;
        unsigned int                                                              : 13;
    } gfx09;
    struct {
        unsigned int                                                              : 19;
        unsigned int DB_EXTRA_DEBUG4                                              : 13;
    } gfx09_0;
    struct {
        unsigned int                                                              : 31;
        unsigned int FULL_TILE_CACHE_EVICT_ON_HALF_FULL                           :  1;
    } gfx09_1x;
    struct {
        unsigned int                                                              :  4;
        unsigned int DISABLE_SEPARATE_OP_PIPE_CLK                                 :  1;
        unsigned int                                                              :  1;
        unsigned int ALWAYS_ON_RMI_CLK_EN                                         :  1;
        unsigned int ENABLE_DBCB_SLOW_FORMAT_COLLAPSE                             :  1;
        unsigned int                                                              :  4;
        unsigned int ENABLE_A2M_DQUAD_OPTIMIZATION                                :  1;
        unsigned int                                                              :  2;
        unsigned int DISABLE_DYNAMIC_RAM_LIGHT_SLEEP_MODE                         :  1;
        unsigned int DISABLE_HIZ_TS_COLLISION_DETECT                              :  1;
        unsigned int DISABLE_LAST_OF_BURST_ON_ACCUM_ALL_EOT                       :  1;
        unsigned int DISABLE_LAST_OF_BURST_ON_FLUSH_CHUNK0_ALL_DONE               :  1;
        unsigned int ENABLE_CZ_OVERFLOW_TESTMODE                                  :  1;
        unsigned int DISABLE_LATEZ_NO_EXPORT_POWER_SAVING                         :  1;
        unsigned int DISABLE_MCC_BURST_FIFO                                       :  1;
        unsigned int DISABLE_MCC_BURST_FIFO_CONFLICT                              :  1;
        unsigned int DISABLE_WR_MEM_BURST_FLF_CONSECUTIVE_CHECK                   :  1;
        unsigned int WR_MEM_BURST_CTL                                             :  3;
        unsigned int DISABLE_WR_MEM_BURST_POOLING                                 :  1;
        unsigned int DISABLE_RD_MEM_BURST                                         :  1;
        unsigned int LATE_ACK_SCOREBOARD_NEW                                      :  1;
        unsigned int LATE_ACK_SCOREBOARD_MULTIPLE_SLOT                            :  1;
        unsigned int                                                              :  1;
    } gfx10;
    struct {
        unsigned int                                                              :  5;
        unsigned int DISABLE_SEPARATE_DFSM_CLK                                    :  1;
        unsigned int                                                              : 25;
        unsigned int LATE_ACK_PSD_EOP_GFX9_METHOD                                 :  1;
    } gfx10Core;
    struct {
        unsigned int                                                              : 19;
        unsigned int DB_EXTRA_DEBUG4                                              : 12;
        unsigned int                                                              :  1;
    } rv2x;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_DEPTH_BOUNDS_MAX {
    struct {
        unsigned int MAX                                                          : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_DEPTH_BOUNDS_MIN {
    struct {
        unsigned int MIN                                                          : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_DEPTH_CLEAR {
    struct {
        unsigned int DEPTH_CLEAR                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_DEPTH_CONTROL {
    struct {
        unsigned int STENCIL_ENABLE                                               :  1;
        unsigned int Z_ENABLE                                                     :  1;
        unsigned int Z_WRITE_ENABLE                                               :  1;
        unsigned int DEPTH_BOUNDS_ENABLE                                          :  1;
        unsigned int ZFUNC                                                        :  3;
        unsigned int BACKFACE_ENABLE                                              :  1;
        unsigned int STENCILFUNC                                                  :  3;
        unsigned int                                                              :  9;
        unsigned int STENCILFUNC_BF                                               :  3;
        unsigned int                                                              :  7;
        unsigned int ENABLE_COLOR_WRITES_ON_DEPTH_FAIL                            :  1;
        unsigned int DISABLE_COLOR_WRITES_ON_DEPTH_PASS                           :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_DEPTH_SIZE {
    struct {
        unsigned int X_MAX                                                        : 14;
        unsigned int                                                              :  2;
        unsigned int Y_MAX                                                        : 14;
        unsigned int                                                              :  2;
    } gfx09;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_DEPTH_SIZE_XY {
    struct {
        unsigned int X_MAX                                                        : 14;
        unsigned int                                                              :  2;
        unsigned int Y_MAX                                                        : 14;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_DEPTH_VIEW {
    struct {
        unsigned int SLICE_START                                                  : 11;
        unsigned int                                                              :  2;
        unsigned int SLICE_MAX                                                    : 11;
        unsigned int Z_READ_ONLY                                                  :  1;
        unsigned int STENCIL_READ_ONLY                                            :  1;
        unsigned int MIPID                                                        :  4;
        unsigned int                                                              :  2;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 11;
        unsigned int SLICE_START_HI                                               :  2;
        unsigned int                                                              : 17;
        unsigned int SLICE_MAX_HI                                                 :  2;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_DFSM_CONFIG {
    struct {
        unsigned int BYPASS_DFSM                                                  :  1;
        unsigned int DISABLE_PUNCHOUT                                             :  1;
        unsigned int DISABLE_POPS                                                 :  1;
        unsigned int FORCE_FLUSH                                                  :  1;
        unsigned int                                                              : 28;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  8;
        unsigned int MIDDLE_PIPE_MAX_DEPTH                                        :  7;
        unsigned int                                                              : 17;
    } gfx09;
    struct {
        unsigned int                                                              :  4;
        unsigned int SQUAD_WATERMARK                                              : 10;
        unsigned int                                                              : 10;
        unsigned int OUTPUT_WATCHDOG                                              :  8;
    } gfx101;
    struct {
        unsigned int                                                              : 16;
        unsigned int CAM_WATERMARK                                                :  8;
        unsigned int                                                              :  8;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_DFSM_CONTROL {
    struct {
        unsigned int PUNCHOUT_MODE                                                :  2;
        unsigned int POPS_DRAIN_PS_ON_OVERLAP                                     :  1;
        unsigned int DISALLOW_OVERFLOW                                            :  1;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_DFSM_FLUSH_AUX_EVENT {
    struct {
        unsigned int EVENT_A                                                      :  8;
        unsigned int EVENT_B                                                      :  8;
        unsigned int EVENT_C                                                      :  8;
        unsigned int EVENT_D                                                      :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_DFSM_FLUSH_ENABLE {
    struct {
        unsigned int                                                              : 24;
        unsigned int AUX_FORCE_PASSTHRU                                           :  4;
        unsigned int AUX_EVENTS                                                   :  4;
    } bits, bitfields;
    struct {
        unsigned int PRIMARY_EVENTS                                               : 10;
        unsigned int                                                              : 22;
    } gfx09;
    struct {
        unsigned int PRIMARY_EVENTS                                               : 11;
        unsigned int                                                              : 21;
    } gfx10Core;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_DFSM_PRIMS_IN_FLIGHT {
    struct {
        unsigned int HIGH_WATERMARK                                               : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 16;
        unsigned int HARD_LIMIT                                                   : 16;
    } gfx09;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_DFSM_TILES_IN_FLIGHT {
    struct {
        unsigned int HIGH_WATERMARK                                               : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 16;
        unsigned int HARD_LIMIT                                                   : 16;
    } gfx09;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_DFSM_WATCHDOG {
    struct {
        unsigned int TIMER_TARGET                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_DFSM_WATERMARK {
    struct {
        unsigned int DFSM_HIGH_WATERMARK                                          : 16;
        unsigned int POPS_HIGH_WATERMARK                                          : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_EQAA {
    struct {
        unsigned int MAX_ANCHOR_SAMPLES                                           :  3;
        unsigned int                                                              :  1;
        unsigned int PS_ITER_SAMPLES                                              :  3;
        unsigned int                                                              :  1;
        unsigned int MASK_EXPORT_NUM_SAMPLES                                      :  3;
        unsigned int                                                              :  1;
        unsigned int ALPHA_TO_MASK_NUM_SAMPLES                                    :  3;
        unsigned int                                                              :  1;
        unsigned int HIGH_QUALITY_INTERSECTIONS                                   :  1;
        unsigned int INCOHERENT_EQAA_READS                                        :  1;
        unsigned int INTERPOLATE_COMP_Z                                           :  1;
        unsigned int INTERPOLATE_SRC_Z                                            :  1;
        unsigned int STATIC_ANCHOR_ASSOCIATIONS                                   :  1;
        unsigned int ALPHA_TO_MASK_EQAA_DISABLE                                   :  1;
        unsigned int                                                              :  2;
        unsigned int OVERRASTERIZATION_AMOUNT                                     :  3;
        unsigned int ENABLE_POSTZ_OVERRASTERIZATION                               :  1;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_EQUAD_STUTTER_CONTROL {
    struct {
        unsigned int THRESHOLD                                                    :  8;
        unsigned int                                                              :  8;
        unsigned int TIMEOUT                                                      :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_ETILE_STUTTER_CONTROL {
    struct {
        unsigned int THRESHOLD                                                    :  8;
        unsigned int                                                              :  8;
        unsigned int TIMEOUT                                                      :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_EXCEPTION_CONTROL {
    struct {
        unsigned int EARLY_Z_PANIC_DISABLE                                        :  1;
        unsigned int LATE_Z_PANIC_DISABLE                                         :  1;
        unsigned int RE_Z_PANIC_DISABLE                                           :  1;
        unsigned int                                                              : 29;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  3;
        unsigned int AUTO_FLUSH_HTILE                                             :  1;
        unsigned int AUTO_FLUSH_QUAD                                              :  1;
        unsigned int                                                              :  3;
        unsigned int FORCE_SUMMARIZE                                              :  4;
        unsigned int                                                              : 12;
        unsigned int DTAG_WATERMARK                                               :  7;
        unsigned int                                                              :  1;
    } gfx10;
    struct {
        unsigned int                                                              :  5;
        unsigned int EXTRA_BITS_GROUP_A                                           :  3;
        unsigned int                                                              : 24;
    } gfx101;
    struct {
        unsigned int                                                              : 12;
        unsigned int EXTRA_BITS_GROUP_B                                           : 12;
        unsigned int                                                              :  8;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_FGCG_INTERFACES_CLK_CTRL {
    struct {
        unsigned int DB_SC_QUAD_OVERRIDE                                          :  1;
        unsigned int DB_CB_TILE_OVERRIDE                                          :  1;
        unsigned int DB_CB_LQUAD_OVERRIDE                                         :  1;
        unsigned int DB_RMI_RDREQ_OVERRIDE                                        :  1;
        unsigned int DB_RMI_WRREQ_OVERRIDE                                        :  1;
        unsigned int                                                              : 27;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  5;
        unsigned int DB_SC_TILE_OVERRIDE                                          :  1;
        unsigned int DB_CB_RMIRET_OVERRIDE                                        :  1;
        unsigned int                                                              : 25;
    } gfx101Plus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_FGCG_SRAMS_CLK_CTRL {
    struct {
        unsigned int OVERRIDE0                                                    :  1;
        unsigned int OVERRIDE1                                                    :  1;
        unsigned int OVERRIDE2                                                    :  1;
        unsigned int OVERRIDE3                                                    :  1;
        unsigned int OVERRIDE4                                                    :  1;
        unsigned int OVERRIDE5                                                    :  1;
        unsigned int OVERRIDE6                                                    :  1;
        unsigned int OVERRIDE7                                                    :  1;
        unsigned int OVERRIDE8                                                    :  1;
        unsigned int OVERRIDE9                                                    :  1;
        unsigned int OVERRIDE10                                                   :  1;
        unsigned int OVERRIDE11                                                   :  1;
        unsigned int OVERRIDE12                                                   :  1;
        unsigned int OVERRIDE13                                                   :  1;
        unsigned int OVERRIDE14                                                   :  1;
        unsigned int OVERRIDE15                                                   :  1;
        unsigned int OVERRIDE16                                                   :  1;
        unsigned int OVERRIDE17                                                   :  1;
        unsigned int OVERRIDE18                                                   :  1;
        unsigned int OVERRIDE19                                                   :  1;
        unsigned int OVERRIDE20                                                   :  1;
        unsigned int OVERRIDE21                                                   :  1;
        unsigned int OVERRIDE22                                                   :  1;
        unsigned int OVERRIDE23                                                   :  1;
        unsigned int OVERRIDE24                                                   :  1;
        unsigned int OVERRIDE25                                                   :  1;
        unsigned int OVERRIDE26                                                   :  1;
        unsigned int                                                              :  5;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_FIFO_DEPTH1 {
    struct {
        unsigned int DB_RMI_RDREQ_CREDITS                                         :  5;
        unsigned int DB_RMI_WRREQ_CREDITS                                         :  5;
        unsigned int MCC_DEPTH                                                    :  6;
        unsigned int QC_DEPTH                                                     :  5;
        unsigned int LTILE_PROBE_FIFO_DEPTH                                       :  8;
        unsigned int                                                              :  3;
    } gfx09;
    struct {
        unsigned int MI_RDREQ_FIFO_DEPTH                                          :  8;
        unsigned int MI_WRREQ_FIFO_DEPTH                                          :  8;
        unsigned int MCC_DEPTH                                                    :  8;
        unsigned int QC_DEPTH                                                     :  8;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_FIFO_DEPTH2 {
    struct {
        unsigned int EQUAD_FIFO_DEPTH                                             :  8;
        unsigned int                                                              : 17;
        unsigned int LTILE_OP_FIFO_DEPTH                                          :  7;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  8;
        unsigned int ETILE_OP_FIFO_DEPTH                                          :  7;
        unsigned int LQUAD_FIFO_DEPTH                                             : 10;
        unsigned int                                                              :  7;
    } gfx09;
    struct {
        unsigned int                                                              :  8;
        unsigned int ETILE_OP_FIFO_DEPTH                                          :  8;
        unsigned int LQUAD_FIFO_DEPTH                                             :  9;
        unsigned int                                                              :  7;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_FIFO_DEPTH3 {
    struct {
        unsigned int LTILE_PROBE_FIFO_DEPTH                                       :  8;
        unsigned int                                                              : 16;
        unsigned int QUAD_READ_REQS                                               :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_FREE_CACHELINES {
    struct {
        unsigned int FREE_DTILE_DEPTH                                             :  7;
        unsigned int FREE_PLANE_DEPTH                                             :  7;
        unsigned int FREE_Z_DEPTH                                                 :  6;
        unsigned int FREE_HTILE_DEPTH                                             :  4;
        unsigned int QUAD_READ_REQS                                               :  8;
    } gfx09;
    struct {
        unsigned int FREE_DTILE_DEPTH                                             :  8;
        unsigned int FREE_PLANE_DEPTH                                             :  8;
        unsigned int FREE_Z_DEPTH                                                 :  8;
        unsigned int FREE_HTILE_DEPTH                                             :  8;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_HTILE_DATA_BASE {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_HTILE_DATA_BASE_HI {
    struct {
        unsigned int BASE_HI                                                      :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_HTILE_SURFACE {
    struct {
        unsigned int                                                              :  1;
        unsigned int FULL_CACHE                                                   :  1;
        unsigned int                                                              : 14;
        unsigned int DST_OUTSIDE_ZERO_TO_ONE                                      :  1;
        unsigned int                                                              :  1;
        unsigned int PIPE_ALIGNED                                                 :  1;
        unsigned int                                                              : 13;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  2;
        unsigned int HTILE_USES_PRELOAD_WIN                                       :  1;
        unsigned int PRELOAD                                                      :  1;
        unsigned int PREFETCH_WIDTH                                               :  6;
        unsigned int PREFETCH_HEIGHT                                              :  6;
        unsigned int                                                              : 16;
    } most;
    struct {
        unsigned int                                                              : 19;
        unsigned int RB_ALIGNED                                                   :  1;
        unsigned int                                                              : 12;
    } gfx09;
    struct {
        unsigned int RESERVED_FIELD_1                                             :  1;
        unsigned int                                                              :  1;
        unsigned int RESERVED_FIELD_2                                             :  1;
        unsigned int RESERVED_FIELD_3                                             :  1;
        unsigned int RESERVED_FIELD_4                                             :  6;
        unsigned int RESERVED_FIELD_5                                             :  6;
        unsigned int                                                              :  1;
        unsigned int RESERVED_FIELD_6                                             :  1;
        unsigned int                                                              : 14;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_LAST_OF_BURST_CONFIG {
    struct {
        unsigned int MAXBURST                                                     :  8;
        unsigned int TIMEOUT                                                      :  3;
        unsigned int                                                              : 19;
        unsigned int DISABLE_RD_BURST                                             :  1;
        unsigned int LEGACY_LOB_INSERT_EN                                         :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 11;
        unsigned int DBCB_LOB_SWITCH_TIMEOUT                                      :  7;
        unsigned int ENABLE_FG_DEFAULT_TIMEOUT                                    :  1;
        unsigned int DISABLE_MCC_BURST_COUNT_RESET_ON_LOB                         :  1;
        unsigned int DISABLE_FLQ_LOB_EVERY_256B                                   :  1;
        unsigned int DISABLE_ZCACHE_FL_OP_EVEN_ARB                                :  1;
        unsigned int DISABLE_MCC_BURST_FORCE_FLUSH_BEFORE_FIFO                    :  1;
        unsigned int ENABLE_TIMEOUT_DKG_LOB_GEN                                   :  1;
        unsigned int ENABLE_TIMEOUT_LPF_LOB_GEN                                   :  1;
        unsigned int ENABLE_TIMEOUT_CB_LOB_GEN                                    :  1;
        unsigned int ENABLE_TIMEOUT_FL_BURST                                      :  1;
        unsigned int ENABLE_TIMEOUT_FG_LOB_FWDR                                   :  1;
        unsigned int ENABLE_TIMEOUT_RD_BA_ACCUM                                   :  1;
        unsigned int BYPASS_SORT_RD_BA                                            :  1;
        unsigned int                                                              :  2;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_LQUAD_STUTTER_CONTROL {
    struct {
        unsigned int THRESHOLD                                                    :  8;
        unsigned int                                                              :  8;
        unsigned int TIMEOUT                                                      :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_LTILE_STUTTER_CONTROL {
    struct {
        unsigned int THRESHOLD                                                    :  8;
        unsigned int                                                              :  8;
        unsigned int TIMEOUT                                                      :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_MEM_ARB_WATERMARKS {
    struct {
        unsigned int CLIENT0_WATERMARK                                            :  3;
        unsigned int                                                              :  5;
        unsigned int CLIENT1_WATERMARK                                            :  3;
        unsigned int                                                              :  5;
        unsigned int CLIENT2_WATERMARK                                            :  3;
        unsigned int                                                              :  5;
        unsigned int CLIENT3_WATERMARK                                            :  3;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_OCCLUSION_COUNT0_HI {
    struct {
        unsigned int COUNT_HI                                                     : 31;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_OCCLUSION_COUNT0_LOW {
    struct {
        unsigned int COUNT_LOW                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_OCCLUSION_COUNT1_HI {
    struct {
        unsigned int COUNT_HI                                                     : 31;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_OCCLUSION_COUNT1_LOW {
    struct {
        unsigned int COUNT_LOW                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_OCCLUSION_COUNT2_HI {
    struct {
        unsigned int COUNT_HI                                                     : 31;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_OCCLUSION_COUNT2_LOW {
    struct {
        unsigned int COUNT_LOW                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_OCCLUSION_COUNT3_HI {
    struct {
        unsigned int COUNT_HI                                                     : 31;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_OCCLUSION_COUNT3_LOW {
    struct {
        unsigned int COUNT_LOW                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_PERFCOUNTER1_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_PRELOAD_CONTROL {
    struct {
        unsigned int START_X                                                      :  8;
        unsigned int START_Y                                                      :  8;
        unsigned int MAX_X                                                        :  8;
        unsigned int MAX_Y                                                        :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_READ_DEBUG_3 {
    struct {
        unsigned int DEBUG_DATA                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_READ_DEBUG_4 {
    struct {
        unsigned int DEBUG_DATA                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_READ_DEBUG_5 {
    struct {
        unsigned int DEBUG_DATA                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_READ_DEBUG_6 {
    struct {
        unsigned int DEBUG_DATA                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_READ_DEBUG_7 {
    struct {
        unsigned int DEBUG_DATA                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_READ_DEBUG_8 {
    struct {
        unsigned int DEBUG_DATA                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_READ_DEBUG_9 {
    struct {
        unsigned int DEBUG_DATA                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_READ_DEBUG_A {
    struct {
        unsigned int DEBUG_DATA                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_READ_DEBUG_B {
    struct {
        unsigned int DEBUG_DATA                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_READ_DEBUG_C {
    struct {
        unsigned int DEBUG_DATA                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_READ_DEBUG_D {
    struct {
        unsigned int DEBUG_DATA                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_READ_DEBUG_E {
    struct {
        unsigned int DEBUG_DATA                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_READ_DEBUG_F {
    struct {
        unsigned int DEBUG_DATA                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_RENDER_CONTROL {
    struct {
        unsigned int DEPTH_CLEAR_ENABLE                                           :  1;
        unsigned int STENCIL_CLEAR_ENABLE                                         :  1;
        unsigned int DEPTH_COPY                                                   :  1;
        unsigned int STENCIL_COPY                                                 :  1;
        unsigned int RESUMMARIZE_ENABLE                                           :  1;
        unsigned int STENCIL_COMPRESS_DISABLE                                     :  1;
        unsigned int DEPTH_COMPRESS_DISABLE                                       :  1;
        unsigned int COPY_CENTROID                                                :  1;
        unsigned int COPY_SAMPLE                                                  :  4;
        unsigned int DECOMPRESS_ENABLE                                            :  1;
        unsigned int                                                              : 19;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_RENDER_OVERRIDE {
    struct {
        unsigned int FORCE_HIZ_ENABLE                                             :  2;
        unsigned int FORCE_HIS_ENABLE0                                            :  2;
        unsigned int FORCE_HIS_ENABLE1                                            :  2;
        unsigned int FORCE_SHADER_Z_ORDER                                         :  1;
        unsigned int FAST_Z_DISABLE                                               :  1;
        unsigned int FAST_STENCIL_DISABLE                                         :  1;
        unsigned int NOOP_CULL_DISABLE                                            :  1;
        unsigned int FORCE_COLOR_KILL                                             :  1;
        unsigned int FORCE_Z_READ                                                 :  1;
        unsigned int FORCE_STENCIL_READ                                           :  1;
        unsigned int FORCE_FULL_Z_RANGE                                           :  2;
        unsigned int FORCE_QC_SMASK_CONFLICT                                      :  1;
        unsigned int DISABLE_VIEWPORT_CLAMP                                       :  1;
        unsigned int IGNORE_SC_ZRANGE                                             :  1;
        unsigned int DISABLE_FULLY_COVERED                                        :  1;
        unsigned int FORCE_Z_LIMIT_SUMM                                           :  2;
        unsigned int MAX_TILES_IN_DTT                                             :  5;
        unsigned int DISABLE_TILE_RATE_TILES                                      :  1;
        unsigned int FORCE_Z_DIRTY                                                :  1;
        unsigned int FORCE_STENCIL_DIRTY                                          :  1;
        unsigned int FORCE_Z_VALID                                                :  1;
        unsigned int FORCE_STENCIL_VALID                                          :  1;
        unsigned int PRESERVE_COMPRESSION                                         :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_RENDER_OVERRIDE2 {
    struct {
        unsigned int PARTIAL_SQUAD_LAUNCH_CONTROL                                 :  2;
        unsigned int PARTIAL_SQUAD_LAUNCH_COUNTDOWN                               :  3;
        unsigned int DISABLE_ZMASK_EXPCLEAR_OPTIMIZATION                          :  1;
        unsigned int DISABLE_SMEM_EXPCLEAR_OPTIMIZATION                           :  1;
        unsigned int DISABLE_COLOR_ON_VALIDATION                                  :  1;
        unsigned int DECOMPRESS_Z_ON_FLUSH                                        :  1;
        unsigned int DISABLE_REG_SNOOP                                            :  1;
        unsigned int DEPTH_BOUNDS_HIER_DEPTH_DISABLE                              :  1;
        unsigned int SEPARATE_HIZS_FUNC_ENABLE                                    :  1;
        unsigned int HIZ_ZFUNC                                                    :  3;
        unsigned int HIS_SFUNC_FF                                                 :  3;
        unsigned int HIS_SFUNC_BF                                                 :  3;
        unsigned int PRESERVE_ZRANGE                                              :  1;
        unsigned int PRESERVE_SRESULTS                                            :  1;
        unsigned int DISABLE_FAST_PASS                                            :  1;
        unsigned int                                                              :  1;
        unsigned int ALLOW_PARTIAL_RES_HIER_KILL                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_RESERVED_REG_1 {
    struct {
        unsigned int FIELD_1                                                      : 11;
        unsigned int FIELD_2                                                      : 11;
        unsigned int                                                              : 10;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_RESERVED_REG_2 {
    struct {
        unsigned int FIELD_1                                                      :  4;
        unsigned int FIELD_2                                                      :  4;
        unsigned int FIELD_3                                                      :  5;
        unsigned int FIELD_4                                                      :  2;
        unsigned int FIELD_5                                                      :  2;
        unsigned int FIELD_6                                                      :  2;
        unsigned int FIELD_7                                                      :  2;
        unsigned int                                                              :  7;
        unsigned int FIELD_8                                                      :  4;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_RESERVED_REG_3 {
    struct {
        unsigned int FIELD_1                                                      : 22;
        unsigned int                                                              : 10;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_RING_CONTROL {
    struct {
        unsigned int COUNTER_CONTROL                                              :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_RMI_BC_GL2_CACHE_CONTROL {
    struct {
        unsigned int Z_WR_POLICY                                                  :  2;
        unsigned int S_WR_POLICY                                                  :  2;
        unsigned int HTILE_WR_POLICY                                              :  2;
        unsigned int ZPCPSD_WR_POLICY                                             :  2;
        unsigned int                                                              :  8;
        unsigned int Z_RD_POLICY                                                  :  2;
        unsigned int S_RD_POLICY                                                  :  2;
        unsigned int HTILE_RD_POLICY                                              :  2;
        unsigned int                                                              :  9;
        unsigned int VOL                                                          :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_RMI_CACHE_POLICY {
    struct {
        unsigned int Z_RD                                                         :  1;
        unsigned int S_RD                                                         :  1;
        unsigned int HTILE_RD                                                     :  1;
        unsigned int                                                              :  5;
        unsigned int Z_WR                                                         :  1;
        unsigned int S_WR                                                         :  1;
        unsigned int HTILE_WR                                                     :  1;
        unsigned int ZPCPSD_WR                                                    :  1;
        unsigned int                                                              :  4;
        unsigned int CC_RD                                                        :  1;
        unsigned int FMASK_RD                                                     :  1;
        unsigned int CMASK_RD                                                     :  1;
        unsigned int DCC_RD                                                       :  1;
        unsigned int                                                              :  4;
        unsigned int CC_WR                                                        :  1;
        unsigned int FMASK_WR                                                     :  1;
        unsigned int CMASK_WR                                                     :  1;
        unsigned int DCC_WR                                                       :  1;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_RMI_L2_CACHE_CONTROL {
    struct {
        unsigned int Z_WR_POLICY                                                  :  2;
        unsigned int S_WR_POLICY                                                  :  2;
        unsigned int HTILE_WR_POLICY                                              :  2;
        unsigned int ZPCPSD_WR_POLICY                                             :  2;
        unsigned int                                                              :  8;
        unsigned int Z_RD_POLICY                                                  :  2;
        unsigned int S_RD_POLICY                                                  :  2;
        unsigned int HTILE_RD_POLICY                                              :  2;
        unsigned int                                                              : 10;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 24;
        unsigned int Z_BIG_PAGE                                                   :  1;
        unsigned int S_BIG_PAGE                                                   :  1;
        unsigned int                                                              :  6;
    } gfx101Plus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_SHADER_CONTROL {
    struct {
        unsigned int Z_EXPORT_ENABLE                                              :  1;
        unsigned int STENCIL_TEST_VAL_EXPORT_ENABLE                               :  1;
        unsigned int STENCIL_OP_VAL_EXPORT_ENABLE                                 :  1;
        unsigned int                                                              :  1;
        unsigned int Z_ORDER                                                      :  2;
        unsigned int KILL_ENABLE                                                  :  1;
        unsigned int COVERAGE_TO_MASK_ENABLE                                      :  1;
        unsigned int MASK_EXPORT_ENABLE                                           :  1;
        unsigned int EXEC_ON_HIER_FAIL                                            :  1;
        unsigned int EXEC_ON_NOOP                                                 :  1;
        unsigned int ALPHA_TO_MASK_DISABLE                                        :  1;
        unsigned int DEPTH_BEFORE_SHADER                                          :  1;
        unsigned int CONSERVATIVE_Z_EXPORT                                        :  2;
        unsigned int DUAL_QUAD_DISABLE                                            :  1;
        unsigned int PRIMITIVE_ORDERED_PIXEL_SHADER                               :  1;
        unsigned int EXEC_IF_OVERLAPPED                                           :  1;
        unsigned int                                                              :  2;
        unsigned int POPS_OVERLAP_NUM_SAMPLES                                     :  3;
        unsigned int                                                              :  9;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 23;
        unsigned int PRE_SHADER_DEPTH_COVERAGE_ENABLE                             :  1;
        unsigned int                                                              :  8;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_SRESULTS_COMPARE_STATE0 {
    struct {
        unsigned int COMPAREFUNC0                                                 :  3;
        unsigned int                                                              :  1;
        unsigned int COMPAREVALUE0                                                :  8;
        unsigned int COMPAREMASK0                                                 :  8;
        unsigned int                                                              :  4;
        unsigned int ENABLE0                                                      :  1;
        unsigned int                                                              :  7;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_SRESULTS_COMPARE_STATE1 {
    struct {
        unsigned int COMPAREFUNC1                                                 :  3;
        unsigned int                                                              :  1;
        unsigned int COMPAREVALUE1                                                :  8;
        unsigned int COMPAREMASK1                                                 :  8;
        unsigned int                                                              :  4;
        unsigned int ENABLE1                                                      :  1;
        unsigned int                                                              :  7;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_STENCILREFMASK {
    struct {
        unsigned int STENCILTESTVAL                                               :  8;
        unsigned int STENCILMASK                                                  :  8;
        unsigned int STENCILWRITEMASK                                             :  8;
        unsigned int STENCILOPVAL                                                 :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_STENCILREFMASK_BF {
    struct {
        unsigned int STENCILTESTVAL_BF                                            :  8;
        unsigned int STENCILMASK_BF                                               :  8;
        unsigned int STENCILWRITEMASK_BF                                          :  8;
        unsigned int STENCILOPVAL_BF                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_STENCIL_CLEAR {
    struct {
        unsigned int CLEAR                                                        :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_STENCIL_CONTROL {
    struct {
        unsigned int STENCILFAIL                                                  :  4;
        unsigned int STENCILZPASS                                                 :  4;
        unsigned int STENCILZFAIL                                                 :  4;
        unsigned int STENCILFAIL_BF                                               :  4;
        unsigned int STENCILZPASS_BF                                              :  4;
        unsigned int STENCILZFAIL_BF                                              :  4;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_STENCIL_INFO {
    struct {
        unsigned int FORMAT                                                       :  1;
        unsigned int                                                              :  3;
        unsigned int SW_MODE                                                      :  5;
        unsigned int                                                              :  3;
        unsigned int PARTIALLY_RESIDENT                                           :  1;
        unsigned int                                                              : 14;
        unsigned int ALLOW_EXPCLEAR                                               :  1;
        unsigned int                                                              :  1;
        unsigned int TILE_STENCIL_DISABLE                                         :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 13;
        unsigned int FAULT_BEHAVIOR                                               :  2;
        unsigned int ITERATE_FLUSH                                                :  1;
        unsigned int                                                              : 14;
        unsigned int CLEAR_DISALLOWED                                             :  1;
        unsigned int                                                              :  1;
    } gfx09;
    struct {
        unsigned int                                                              :  9;
        unsigned int FAULT_BEHAVIOR                                               :  2;
        unsigned int ITERATE_FLUSH                                                :  1;
        unsigned int                                                              :  8;
        unsigned int ITERATE_256                                                  :  1;
        unsigned int                                                              : 11;
    } gfx10;
    struct {
        unsigned int                                                              : 13;
        unsigned int RESERVED_FIELD_1                                             :  3;
        unsigned int                                                              : 16;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_STENCIL_INFO2 {
    struct {
        unsigned int EPITCH                                                       : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_STENCIL_READ_BASE {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_STENCIL_READ_BASE_HI {
    struct {
        unsigned int BASE_HI                                                      :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_STENCIL_WRITE_BASE {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_STENCIL_WRITE_BASE_HI {
    struct {
        unsigned int BASE_HI                                                      :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_SUBTILE_CONTROL {
    struct {
        unsigned int MSAA1_X                                                      :  2;
        unsigned int MSAA1_Y                                                      :  2;
        unsigned int MSAA2_X                                                      :  2;
        unsigned int MSAA2_Y                                                      :  2;
        unsigned int MSAA4_X                                                      :  2;
        unsigned int MSAA4_Y                                                      :  2;
        unsigned int MSAA8_X                                                      :  2;
        unsigned int MSAA8_Y                                                      :  2;
        unsigned int MSAA16_X                                                     :  2;
        unsigned int MSAA16_Y                                                     :  2;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_WATERMARKS {
    struct {
        unsigned int DEPTH_FREE                                                   :  5;
        unsigned int DEPTH_FLUSH                                                  :  6;
        unsigned int FORCE_SUMMARIZE                                              :  4;
        unsigned int DEPTH_PENDING_FREE                                           :  5;
        unsigned int DEPTH_CACHELINE_FREE                                         :  8;
        unsigned int                                                              :  2;
        unsigned int AUTO_FLUSH_HTILE                                             :  1;
        unsigned int AUTO_FLUSH_QUAD                                              :  1;
    } gfx09;
    struct {
        unsigned int DEPTH_FREE                                                   :  8;
        unsigned int DEPTH_FLUSH                                                  :  8;
        unsigned int DEPTH_PENDING_FREE                                           :  8;
        unsigned int DEPTH_CACHELINE_FREE                                         :  8;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_ZPASS_COUNT_HI {
    struct {
        unsigned int COUNT_HI                                                     : 31;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_ZPASS_COUNT_LOW {
    struct {
        unsigned int COUNT_LOW                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_Z_INFO {
    struct {
        unsigned int FORMAT                                                       :  2;
        unsigned int NUM_SAMPLES                                                  :  2;
        unsigned int SW_MODE                                                      :  5;
        unsigned int                                                              :  3;
        unsigned int PARTIALLY_RESIDENT                                           :  1;
        unsigned int                                                              :  3;
        unsigned int MAXMIP                                                       :  4;
        unsigned int                                                              :  3;
        unsigned int DECOMPRESS_ON_N_ZPLANES                                      :  4;
        unsigned int ALLOW_EXPCLEAR                                               :  1;
        unsigned int READ_SIZE                                                    :  1;
        unsigned int TILE_SURFACE_ENABLE                                          :  1;
        unsigned int                                                              :  1;
        unsigned int ZRANGE_PRECISION                                             :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 13;
        unsigned int FAULT_BEHAVIOR                                               :  2;
        unsigned int ITERATE_FLUSH                                                :  1;
        unsigned int                                                              : 14;
        unsigned int CLEAR_DISALLOWED                                             :  1;
        unsigned int                                                              :  1;
    } gfx09;
    struct {
        unsigned int                                                              :  9;
        unsigned int FAULT_BEHAVIOR                                               :  2;
        unsigned int ITERATE_FLUSH                                                :  1;
        unsigned int                                                              :  8;
        unsigned int ITERATE_256                                                  :  1;
        unsigned int                                                              : 11;
    } gfx10;
    struct {
        unsigned int                                                              : 13;
        unsigned int RESERVED_FIELD_1                                             :  3;
        unsigned int                                                              : 16;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_Z_INFO2 {
    struct {
        unsigned int EPITCH                                                       : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_Z_READ_BASE {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_Z_READ_BASE_HI {
    struct {
        unsigned int BASE_HI                                                      :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_Z_WRITE_BASE {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_Z_WRITE_BASE_HI {
    struct {
        unsigned int BASE_HI                                                      :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DEBUG_DATA {
    struct {
        unsigned int DEBUG_DATA                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DEBUG_INDEX {
    struct {
        unsigned int DEBUG_INDEX                                                  : 18;
        unsigned int                                                              : 14;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union FORCE_SYMCLK_DISABLE {
    struct {
        unsigned int FORCE_SYMCLKA_DISABLE                                        :  1;
        unsigned int FORCE_SYMCLKB_DISABLE                                        :  1;
        unsigned int FORCE_SYMCLKC_DISABLE                                        :  1;
        unsigned int FORCE_SYMCLKD_DISABLE                                        :  1;
        unsigned int FORCE_SYMCLKE_DISABLE                                        :  1;
        unsigned int FORCE_SYMCLKF_DISABLE                                        :  1;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_ADDR_CONFIG {
    struct {
        unsigned int NUM_PIPES                                                    :  3;
        unsigned int PIPE_INTERLEAVE_SIZE                                         :  3;
        unsigned int MAX_COMPRESSED_FRAGS                                         :  2;
        unsigned int                                                              : 11;
        unsigned int NUM_SHADER_ENGINES                                           :  2;
        unsigned int                                                              :  5;
        unsigned int NUM_RB_PER_SE                                                :  2;
        unsigned int                                                              :  4;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  8;
        unsigned int BANK_INTERLEAVE_SIZE                                         :  3;
        unsigned int                                                              :  1;
        unsigned int NUM_BANKS                                                    :  3;
        unsigned int                                                              :  1;
        unsigned int SHADER_ENGINE_TILE_SIZE                                      :  3;
        unsigned int                                                              :  2;
        unsigned int NUM_GPUS                                                     :  3;
        unsigned int MULTI_GPU_TILE_SIZE                                          :  2;
        unsigned int                                                              :  2;
        unsigned int ROW_SIZE                                                     :  2;
        unsigned int NUM_LOWER_PIPES                                              :  1;
        unsigned int SE_ENABLE                                                    :  1;
    } gfx09;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_ADDR_CONFIG_READ {
    struct {
        unsigned int NUM_PIPES                                                    :  3;
        unsigned int PIPE_INTERLEAVE_SIZE                                         :  3;
        unsigned int MAX_COMPRESSED_FRAGS                                         :  2;
        unsigned int                                                              : 11;
        unsigned int NUM_SHADER_ENGINES                                           :  2;
        unsigned int                                                              :  5;
        unsigned int NUM_RB_PER_SE                                                :  2;
        unsigned int                                                              :  4;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  8;
        unsigned int BANK_INTERLEAVE_SIZE                                         :  3;
        unsigned int                                                              :  1;
        unsigned int NUM_BANKS                                                    :  3;
        unsigned int                                                              :  1;
        unsigned int SHADER_ENGINE_TILE_SIZE                                      :  3;
        unsigned int                                                              :  2;
        unsigned int NUM_GPUS                                                     :  3;
        unsigned int MULTI_GPU_TILE_SIZE                                          :  2;
        unsigned int                                                              :  2;
        unsigned int ROW_SIZE                                                     :  2;
        unsigned int NUM_LOWER_PIPES                                              :  1;
        unsigned int SE_ENABLE                                                    :  1;
    } gfx09;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_BACKEND_MAP {
    struct {
        unsigned int BACKEND_MAP                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_EDC_MODE {
    struct {
        unsigned int                                                              : 15;
        unsigned int FORCE_SEC_ON_DED                                             :  1;
        unsigned int COUNT_FED_OUT                                                :  1;
        unsigned int GATE_FUE                                                     :  1;
        unsigned int                                                              :  2;
        unsigned int DED_MODE                                                     :  2;
        unsigned int                                                              :  7;
        unsigned int PROP_FED                                                     :  1;
        unsigned int                                                              :  1;
        unsigned int BYPASS                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_GPU_ID {
    struct {
        unsigned int GPU_ID                                                       :  4;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_MACROTILE_MODE0 {
    struct {
        unsigned int BANK_WIDTH                                                   :  2;
        unsigned int BANK_HEIGHT                                                  :  2;
        unsigned int MACRO_TILE_ASPECT                                            :  2;
        unsigned int NUM_BANKS                                                    :  2;
        unsigned int                                                              : 24;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_MACROTILE_MODE1 {
    struct {
        unsigned int BANK_WIDTH                                                   :  2;
        unsigned int BANK_HEIGHT                                                  :  2;
        unsigned int MACRO_TILE_ASPECT                                            :  2;
        unsigned int NUM_BANKS                                                    :  2;
        unsigned int                                                              : 24;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_MACROTILE_MODE2 {
    struct {
        unsigned int BANK_WIDTH                                                   :  2;
        unsigned int BANK_HEIGHT                                                  :  2;
        unsigned int MACRO_TILE_ASPECT                                            :  2;
        unsigned int NUM_BANKS                                                    :  2;
        unsigned int                                                              : 24;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_MACROTILE_MODE3 {
    struct {
        unsigned int BANK_WIDTH                                                   :  2;
        unsigned int BANK_HEIGHT                                                  :  2;
        unsigned int MACRO_TILE_ASPECT                                            :  2;
        unsigned int NUM_BANKS                                                    :  2;
        unsigned int                                                              : 24;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_MACROTILE_MODE4 {
    struct {
        unsigned int BANK_WIDTH                                                   :  2;
        unsigned int BANK_HEIGHT                                                  :  2;
        unsigned int MACRO_TILE_ASPECT                                            :  2;
        unsigned int NUM_BANKS                                                    :  2;
        unsigned int                                                              : 24;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_MACROTILE_MODE5 {
    struct {
        unsigned int BANK_WIDTH                                                   :  2;
        unsigned int BANK_HEIGHT                                                  :  2;
        unsigned int MACRO_TILE_ASPECT                                            :  2;
        unsigned int NUM_BANKS                                                    :  2;
        unsigned int                                                              : 24;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_MACROTILE_MODE6 {
    struct {
        unsigned int BANK_WIDTH                                                   :  2;
        unsigned int BANK_HEIGHT                                                  :  2;
        unsigned int MACRO_TILE_ASPECT                                            :  2;
        unsigned int NUM_BANKS                                                    :  2;
        unsigned int                                                              : 24;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_MACROTILE_MODE7 {
    struct {
        unsigned int BANK_WIDTH                                                   :  2;
        unsigned int BANK_HEIGHT                                                  :  2;
        unsigned int MACRO_TILE_ASPECT                                            :  2;
        unsigned int NUM_BANKS                                                    :  2;
        unsigned int                                                              : 24;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_MACROTILE_MODE8 {
    struct {
        unsigned int BANK_WIDTH                                                   :  2;
        unsigned int BANK_HEIGHT                                                  :  2;
        unsigned int MACRO_TILE_ASPECT                                            :  2;
        unsigned int NUM_BANKS                                                    :  2;
        unsigned int                                                              : 24;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_MACROTILE_MODE9 {
    struct {
        unsigned int BANK_WIDTH                                                   :  2;
        unsigned int BANK_HEIGHT                                                  :  2;
        unsigned int MACRO_TILE_ASPECT                                            :  2;
        unsigned int NUM_BANKS                                                    :  2;
        unsigned int                                                              : 24;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_MACROTILE_MODE10 {
    struct {
        unsigned int BANK_WIDTH                                                   :  2;
        unsigned int BANK_HEIGHT                                                  :  2;
        unsigned int MACRO_TILE_ASPECT                                            :  2;
        unsigned int NUM_BANKS                                                    :  2;
        unsigned int                                                              : 24;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_MACROTILE_MODE11 {
    struct {
        unsigned int BANK_WIDTH                                                   :  2;
        unsigned int BANK_HEIGHT                                                  :  2;
        unsigned int MACRO_TILE_ASPECT                                            :  2;
        unsigned int NUM_BANKS                                                    :  2;
        unsigned int                                                              : 24;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_MACROTILE_MODE12 {
    struct {
        unsigned int BANK_WIDTH                                                   :  2;
        unsigned int BANK_HEIGHT                                                  :  2;
        unsigned int MACRO_TILE_ASPECT                                            :  2;
        unsigned int NUM_BANKS                                                    :  2;
        unsigned int                                                              : 24;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_MACROTILE_MODE13 {
    struct {
        unsigned int BANK_WIDTH                                                   :  2;
        unsigned int BANK_HEIGHT                                                  :  2;
        unsigned int MACRO_TILE_ASPECT                                            :  2;
        unsigned int NUM_BANKS                                                    :  2;
        unsigned int                                                              : 24;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_MACROTILE_MODE14 {
    struct {
        unsigned int BANK_WIDTH                                                   :  2;
        unsigned int BANK_HEIGHT                                                  :  2;
        unsigned int MACRO_TILE_ASPECT                                            :  2;
        unsigned int NUM_BANKS                                                    :  2;
        unsigned int                                                              : 24;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_MACROTILE_MODE15 {
    struct {
        unsigned int BANK_WIDTH                                                   :  2;
        unsigned int BANK_HEIGHT                                                  :  2;
        unsigned int MACRO_TILE_ASPECT                                            :  2;
        unsigned int NUM_BANKS                                                    :  2;
        unsigned int                                                              : 24;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_TILE_MODE0 {
    struct {
        unsigned int                                                              :  2;
        unsigned int ARRAY_MODE                                                   :  4;
        unsigned int PIPE_CONFIG                                                  :  5;
        unsigned int TILE_SPLIT                                                   :  3;
        unsigned int                                                              :  8;
        unsigned int MICRO_TILE_MODE_NEW                                          :  3;
        unsigned int SAMPLE_SPLIT                                                 :  2;
        unsigned int                                                              :  5;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_TILE_MODE1 {
    struct {
        unsigned int                                                              :  2;
        unsigned int ARRAY_MODE                                                   :  4;
        unsigned int PIPE_CONFIG                                                  :  5;
        unsigned int TILE_SPLIT                                                   :  3;
        unsigned int                                                              :  8;
        unsigned int MICRO_TILE_MODE_NEW                                          :  3;
        unsigned int SAMPLE_SPLIT                                                 :  2;
        unsigned int                                                              :  5;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_TILE_MODE2 {
    struct {
        unsigned int                                                              :  2;
        unsigned int ARRAY_MODE                                                   :  4;
        unsigned int PIPE_CONFIG                                                  :  5;
        unsigned int TILE_SPLIT                                                   :  3;
        unsigned int                                                              :  8;
        unsigned int MICRO_TILE_MODE_NEW                                          :  3;
        unsigned int SAMPLE_SPLIT                                                 :  2;
        unsigned int                                                              :  5;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_TILE_MODE3 {
    struct {
        unsigned int                                                              :  2;
        unsigned int ARRAY_MODE                                                   :  4;
        unsigned int PIPE_CONFIG                                                  :  5;
        unsigned int TILE_SPLIT                                                   :  3;
        unsigned int                                                              :  8;
        unsigned int MICRO_TILE_MODE_NEW                                          :  3;
        unsigned int SAMPLE_SPLIT                                                 :  2;
        unsigned int                                                              :  5;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_TILE_MODE4 {
    struct {
        unsigned int                                                              :  2;
        unsigned int ARRAY_MODE                                                   :  4;
        unsigned int PIPE_CONFIG                                                  :  5;
        unsigned int TILE_SPLIT                                                   :  3;
        unsigned int                                                              :  8;
        unsigned int MICRO_TILE_MODE_NEW                                          :  3;
        unsigned int SAMPLE_SPLIT                                                 :  2;
        unsigned int                                                              :  5;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_TILE_MODE5 {
    struct {
        unsigned int                                                              :  2;
        unsigned int ARRAY_MODE                                                   :  4;
        unsigned int PIPE_CONFIG                                                  :  5;
        unsigned int TILE_SPLIT                                                   :  3;
        unsigned int                                                              :  8;
        unsigned int MICRO_TILE_MODE_NEW                                          :  3;
        unsigned int SAMPLE_SPLIT                                                 :  2;
        unsigned int                                                              :  5;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_TILE_MODE6 {
    struct {
        unsigned int                                                              :  2;
        unsigned int ARRAY_MODE                                                   :  4;
        unsigned int PIPE_CONFIG                                                  :  5;
        unsigned int TILE_SPLIT                                                   :  3;
        unsigned int                                                              :  8;
        unsigned int MICRO_TILE_MODE_NEW                                          :  3;
        unsigned int SAMPLE_SPLIT                                                 :  2;
        unsigned int                                                              :  5;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_TILE_MODE7 {
    struct {
        unsigned int                                                              :  2;
        unsigned int ARRAY_MODE                                                   :  4;
        unsigned int PIPE_CONFIG                                                  :  5;
        unsigned int TILE_SPLIT                                                   :  3;
        unsigned int                                                              :  8;
        unsigned int MICRO_TILE_MODE_NEW                                          :  3;
        unsigned int SAMPLE_SPLIT                                                 :  2;
        unsigned int                                                              :  5;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_TILE_MODE8 {
    struct {
        unsigned int                                                              :  2;
        unsigned int ARRAY_MODE                                                   :  4;
        unsigned int PIPE_CONFIG                                                  :  5;
        unsigned int TILE_SPLIT                                                   :  3;
        unsigned int                                                              :  8;
        unsigned int MICRO_TILE_MODE_NEW                                          :  3;
        unsigned int SAMPLE_SPLIT                                                 :  2;
        unsigned int                                                              :  5;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_TILE_MODE9 {
    struct {
        unsigned int                                                              :  2;
        unsigned int ARRAY_MODE                                                   :  4;
        unsigned int PIPE_CONFIG                                                  :  5;
        unsigned int TILE_SPLIT                                                   :  3;
        unsigned int                                                              :  8;
        unsigned int MICRO_TILE_MODE_NEW                                          :  3;
        unsigned int SAMPLE_SPLIT                                                 :  2;
        unsigned int                                                              :  5;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_TILE_MODE10 {
    struct {
        unsigned int                                                              :  2;
        unsigned int ARRAY_MODE                                                   :  4;
        unsigned int PIPE_CONFIG                                                  :  5;
        unsigned int TILE_SPLIT                                                   :  3;
        unsigned int                                                              :  8;
        unsigned int MICRO_TILE_MODE_NEW                                          :  3;
        unsigned int SAMPLE_SPLIT                                                 :  2;
        unsigned int                                                              :  5;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_TILE_MODE11 {
    struct {
        unsigned int                                                              :  2;
        unsigned int ARRAY_MODE                                                   :  4;
        unsigned int PIPE_CONFIG                                                  :  5;
        unsigned int TILE_SPLIT                                                   :  3;
        unsigned int                                                              :  8;
        unsigned int MICRO_TILE_MODE_NEW                                          :  3;
        unsigned int SAMPLE_SPLIT                                                 :  2;
        unsigned int                                                              :  5;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_TILE_MODE12 {
    struct {
        unsigned int                                                              :  2;
        unsigned int ARRAY_MODE                                                   :  4;
        unsigned int PIPE_CONFIG                                                  :  5;
        unsigned int TILE_SPLIT                                                   :  3;
        unsigned int                                                              :  8;
        unsigned int MICRO_TILE_MODE_NEW                                          :  3;
        unsigned int SAMPLE_SPLIT                                                 :  2;
        unsigned int                                                              :  5;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_TILE_MODE13 {
    struct {
        unsigned int                                                              :  2;
        unsigned int ARRAY_MODE                                                   :  4;
        unsigned int PIPE_CONFIG                                                  :  5;
        unsigned int TILE_SPLIT                                                   :  3;
        unsigned int                                                              :  8;
        unsigned int MICRO_TILE_MODE_NEW                                          :  3;
        unsigned int SAMPLE_SPLIT                                                 :  2;
        unsigned int                                                              :  5;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_TILE_MODE14 {
    struct {
        unsigned int                                                              :  2;
        unsigned int ARRAY_MODE                                                   :  4;
        unsigned int PIPE_CONFIG                                                  :  5;
        unsigned int TILE_SPLIT                                                   :  3;
        unsigned int                                                              :  8;
        unsigned int MICRO_TILE_MODE_NEW                                          :  3;
        unsigned int SAMPLE_SPLIT                                                 :  2;
        unsigned int                                                              :  5;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_TILE_MODE15 {
    struct {
        unsigned int                                                              :  2;
        unsigned int ARRAY_MODE                                                   :  4;
        unsigned int PIPE_CONFIG                                                  :  5;
        unsigned int TILE_SPLIT                                                   :  3;
        unsigned int                                                              :  8;
        unsigned int MICRO_TILE_MODE_NEW                                          :  3;
        unsigned int SAMPLE_SPLIT                                                 :  2;
        unsigned int                                                              :  5;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_TILE_MODE16 {
    struct {
        unsigned int                                                              :  2;
        unsigned int ARRAY_MODE                                                   :  4;
        unsigned int PIPE_CONFIG                                                  :  5;
        unsigned int TILE_SPLIT                                                   :  3;
        unsigned int                                                              :  8;
        unsigned int MICRO_TILE_MODE_NEW                                          :  3;
        unsigned int SAMPLE_SPLIT                                                 :  2;
        unsigned int                                                              :  5;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_TILE_MODE17 {
    struct {
        unsigned int                                                              :  2;
        unsigned int ARRAY_MODE                                                   :  4;
        unsigned int PIPE_CONFIG                                                  :  5;
        unsigned int TILE_SPLIT                                                   :  3;
        unsigned int                                                              :  8;
        unsigned int MICRO_TILE_MODE_NEW                                          :  3;
        unsigned int SAMPLE_SPLIT                                                 :  2;
        unsigned int                                                              :  5;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_TILE_MODE18 {
    struct {
        unsigned int                                                              :  2;
        unsigned int ARRAY_MODE                                                   :  4;
        unsigned int PIPE_CONFIG                                                  :  5;
        unsigned int TILE_SPLIT                                                   :  3;
        unsigned int                                                              :  8;
        unsigned int MICRO_TILE_MODE_NEW                                          :  3;
        unsigned int SAMPLE_SPLIT                                                 :  2;
        unsigned int                                                              :  5;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_TILE_MODE19 {
    struct {
        unsigned int                                                              :  2;
        unsigned int ARRAY_MODE                                                   :  4;
        unsigned int PIPE_CONFIG                                                  :  5;
        unsigned int TILE_SPLIT                                                   :  3;
        unsigned int                                                              :  8;
        unsigned int MICRO_TILE_MODE_NEW                                          :  3;
        unsigned int SAMPLE_SPLIT                                                 :  2;
        unsigned int                                                              :  5;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_TILE_MODE20 {
    struct {
        unsigned int                                                              :  2;
        unsigned int ARRAY_MODE                                                   :  4;
        unsigned int PIPE_CONFIG                                                  :  5;
        unsigned int TILE_SPLIT                                                   :  3;
        unsigned int                                                              :  8;
        unsigned int MICRO_TILE_MODE_NEW                                          :  3;
        unsigned int SAMPLE_SPLIT                                                 :  2;
        unsigned int                                                              :  5;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_TILE_MODE21 {
    struct {
        unsigned int                                                              :  2;
        unsigned int ARRAY_MODE                                                   :  4;
        unsigned int PIPE_CONFIG                                                  :  5;
        unsigned int TILE_SPLIT                                                   :  3;
        unsigned int                                                              :  8;
        unsigned int MICRO_TILE_MODE_NEW                                          :  3;
        unsigned int SAMPLE_SPLIT                                                 :  2;
        unsigned int                                                              :  5;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_TILE_MODE22 {
    struct {
        unsigned int                                                              :  2;
        unsigned int ARRAY_MODE                                                   :  4;
        unsigned int PIPE_CONFIG                                                  :  5;
        unsigned int TILE_SPLIT                                                   :  3;
        unsigned int                                                              :  8;
        unsigned int MICRO_TILE_MODE_NEW                                          :  3;
        unsigned int SAMPLE_SPLIT                                                 :  2;
        unsigned int                                                              :  5;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_TILE_MODE23 {
    struct {
        unsigned int                                                              :  2;
        unsigned int ARRAY_MODE                                                   :  4;
        unsigned int PIPE_CONFIG                                                  :  5;
        unsigned int TILE_SPLIT                                                   :  3;
        unsigned int                                                              :  8;
        unsigned int MICRO_TILE_MODE_NEW                                          :  3;
        unsigned int SAMPLE_SPLIT                                                 :  2;
        unsigned int                                                              :  5;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_TILE_MODE24 {
    struct {
        unsigned int                                                              :  2;
        unsigned int ARRAY_MODE                                                   :  4;
        unsigned int PIPE_CONFIG                                                  :  5;
        unsigned int TILE_SPLIT                                                   :  3;
        unsigned int                                                              :  8;
        unsigned int MICRO_TILE_MODE_NEW                                          :  3;
        unsigned int SAMPLE_SPLIT                                                 :  2;
        unsigned int                                                              :  5;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_TILE_MODE25 {
    struct {
        unsigned int                                                              :  2;
        unsigned int ARRAY_MODE                                                   :  4;
        unsigned int PIPE_CONFIG                                                  :  5;
        unsigned int TILE_SPLIT                                                   :  3;
        unsigned int                                                              :  8;
        unsigned int MICRO_TILE_MODE_NEW                                          :  3;
        unsigned int SAMPLE_SPLIT                                                 :  2;
        unsigned int                                                              :  5;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_TILE_MODE26 {
    struct {
        unsigned int                                                              :  2;
        unsigned int ARRAY_MODE                                                   :  4;
        unsigned int PIPE_CONFIG                                                  :  5;
        unsigned int TILE_SPLIT                                                   :  3;
        unsigned int                                                              :  8;
        unsigned int MICRO_TILE_MODE_NEW                                          :  3;
        unsigned int SAMPLE_SPLIT                                                 :  2;
        unsigned int                                                              :  5;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_TILE_MODE27 {
    struct {
        unsigned int                                                              :  2;
        unsigned int ARRAY_MODE                                                   :  4;
        unsigned int PIPE_CONFIG                                                  :  5;
        unsigned int TILE_SPLIT                                                   :  3;
        unsigned int                                                              :  8;
        unsigned int MICRO_TILE_MODE_NEW                                          :  3;
        unsigned int SAMPLE_SPLIT                                                 :  2;
        unsigned int                                                              :  5;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_TILE_MODE28 {
    struct {
        unsigned int                                                              :  2;
        unsigned int ARRAY_MODE                                                   :  4;
        unsigned int PIPE_CONFIG                                                  :  5;
        unsigned int TILE_SPLIT                                                   :  3;
        unsigned int                                                              :  8;
        unsigned int MICRO_TILE_MODE_NEW                                          :  3;
        unsigned int SAMPLE_SPLIT                                                 :  2;
        unsigned int                                                              :  5;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_TILE_MODE29 {
    struct {
        unsigned int                                                              :  2;
        unsigned int ARRAY_MODE                                                   :  4;
        unsigned int PIPE_CONFIG                                                  :  5;
        unsigned int TILE_SPLIT                                                   :  3;
        unsigned int                                                              :  8;
        unsigned int MICRO_TILE_MODE_NEW                                          :  3;
        unsigned int SAMPLE_SPLIT                                                 :  2;
        unsigned int                                                              :  5;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_TILE_MODE30 {
    struct {
        unsigned int                                                              :  2;
        unsigned int ARRAY_MODE                                                   :  4;
        unsigned int PIPE_CONFIG                                                  :  5;
        unsigned int TILE_SPLIT                                                   :  3;
        unsigned int                                                              :  8;
        unsigned int MICRO_TILE_MODE_NEW                                          :  3;
        unsigned int SAMPLE_SPLIT                                                 :  2;
        unsigned int                                                              :  5;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_TILE_MODE31 {
    struct {
        unsigned int                                                              :  2;
        unsigned int ARRAY_MODE                                                   :  4;
        unsigned int PIPE_CONFIG                                                  :  5;
        unsigned int TILE_SPLIT                                                   :  3;
        unsigned int                                                              :  8;
        unsigned int MICRO_TILE_MODE_NEW                                          :  3;
        unsigned int SAMPLE_SPLIT                                                 :  2;
        unsigned int                                                              :  5;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCEA_PERFCOUNTER0_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCEA_PERFCOUNTER1_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCEA_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCEA_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCEA_PERFCOUNTER2_MODE {
    struct {
        unsigned int COMPARE_MODE0                                                :  2;
        unsigned int COMPARE_MODE1                                                :  2;
        unsigned int COMPARE_MODE2                                                :  2;
        unsigned int COMPARE_MODE3                                                :  2;
        unsigned int COMPARE_VALUE0                                               :  4;
        unsigned int COMPARE_VALUE1                                               :  4;
        unsigned int COMPARE_VALUE2                                               :  4;
        unsigned int COMPARE_VALUE3                                               :  4;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCEA_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCEA_PERFCOUNTER2_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCEA_PERFCOUNTER_HI {
    struct {
        unsigned int COUNTER_HI                                                   : 16;
        unsigned int COMPARE_VALUE                                                : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCEA_PERFCOUNTER_LO {
    struct {
        unsigned int COUNTER_LO                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCEA_PERFCOUNTER_RSLT_CNTL {
    struct {
        unsigned int PERF_COUNTER_SELECT                                          :  4;
        unsigned int                                                              :  4;
        unsigned int START_TRIGGER                                                :  8;
        unsigned int STOP_TRIGGER                                                 :  8;
        unsigned int ENABLE_ANY                                                   :  1;
        unsigned int CLEAR_ALL                                                    :  1;
        unsigned int STOP_ALL_ON_SATURATE                                         :  1;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_MEM_POWER_LS {
    struct {
        unsigned int LS_SETUP                                                     :  6;
        unsigned int LS_HOLD                                                      :  6;
        unsigned int                                                              : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_SHARED_ACTIVE_FCN_ID {
    struct {
        unsigned int VFID                                                         :  5;
        unsigned int                                                              : 26;
        unsigned int VF                                                           :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_SHARED_VIRT_RESET_REQ {
    struct {
        unsigned int VF                                                           : 31;
        unsigned int PF                                                           :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_SHARED_VIRT_RESET_REQ2 {
    struct {
        unsigned int VF                                                           :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_AGP_BASE {
    struct {
        unsigned int AGP_BASE                                                     : 24;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_AGP_BOT {
    struct {
        unsigned int AGP_BOT                                                      : 24;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_AGP_TOP {
    struct {
        unsigned int AGP_TOP                                                      : 24;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_APT_CNTL {
    struct {
        unsigned int FORCE_MTYPE_UC                                               :  1;
        unsigned int DIRECT_SYSTEM_EN                                             :  1;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_CACHEABLE_DRAM_ADDRESS_END {
    struct {
        unsigned int ADDRESS                                                      : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_CACHEABLE_DRAM_ADDRESS_START {
    struct {
        unsigned int ADDRESS                                                      : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_FB_LOCATION_BASE {
    struct {
        unsigned int FB_BASE                                                      : 24;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_FB_LOCATION_TOP {
    struct {
        unsigned int FB_TOP                                                       : 24;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_FB_OFFSET {
    struct {
        unsigned int FB_OFFSET                                                    : 24;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_FB_SIZE_OFFSET_VF0 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_FB_SIZE_OFFSET_VF1 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_FB_SIZE_OFFSET_VF2 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_FB_SIZE_OFFSET_VF3 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_FB_SIZE_OFFSET_VF4 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_FB_SIZE_OFFSET_VF5 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_FB_SIZE_OFFSET_VF6 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_FB_SIZE_OFFSET_VF7 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_FB_SIZE_OFFSET_VF8 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_FB_SIZE_OFFSET_VF9 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_FB_SIZE_OFFSET_VF10 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_FB_SIZE_OFFSET_VF11 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_FB_SIZE_OFFSET_VF12 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_FB_SIZE_OFFSET_VF13 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_FB_SIZE_OFFSET_VF14 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_FB_SIZE_OFFSET_VF15 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_FB_SIZE_OFFSET_VF16 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_FB_SIZE_OFFSET_VF17 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_FB_SIZE_OFFSET_VF18 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_FB_SIZE_OFFSET_VF19 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_FB_SIZE_OFFSET_VF20 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_FB_SIZE_OFFSET_VF21 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_FB_SIZE_OFFSET_VF22 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_FB_SIZE_OFFSET_VF23 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_FB_SIZE_OFFSET_VF24 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_FB_SIZE_OFFSET_VF25 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_FB_SIZE_OFFSET_VF26 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_FB_SIZE_OFFSET_VF27 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_FB_SIZE_OFFSET_VF28 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_FB_SIZE_OFFSET_VF29 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_FB_SIZE_OFFSET_VF30 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_FB_SIZE_OFFSET_VF31 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_L2_PERFCOUNTER0_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_L2_PERFCOUNTER1_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_L2_PERFCOUNTER2_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_L2_PERFCOUNTER3_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_L2_PERFCOUNTER4_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_L2_PERFCOUNTER5_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_L2_PERFCOUNTER6_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_L2_PERFCOUNTER7_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_L2_PERFCOUNTER_HI {
    struct {
        unsigned int COUNTER_HI                                                   : 16;
        unsigned int COMPARE_VALUE                                                : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_L2_PERFCOUNTER_LO {
    struct {
        unsigned int COUNTER_LO                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_L2_PERFCOUNTER_RSLT_CNTL {
    struct {
        unsigned int PERF_COUNTER_SELECT                                          :  4;
        unsigned int                                                              :  4;
        unsigned int START_TRIGGER                                                :  8;
        unsigned int STOP_TRIGGER                                                 :  8;
        unsigned int ENABLE_ANY                                                   :  1;
        unsigned int CLEAR_ALL                                                    :  1;
        unsigned int STOP_ALL_ON_SATURATE                                         :  1;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_LOCAL_HBM_ADDRESS_END {
    struct {
        unsigned int ADDRESS                                                      : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_LOCAL_HBM_ADDRESS_LOCK_CNTL {
    struct {
        unsigned int LOCK                                                         :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_LOCAL_HBM_ADDRESS_START {
    struct {
        unsigned int ADDRESS                                                      : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_MARC_BASE_HI_0 {
    struct {
        unsigned int MARC_BASE_HI_0                                               : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_MARC_BASE_HI_1 {
    struct {
        unsigned int MARC_BASE_HI_1                                               : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_MARC_BASE_HI_2 {
    struct {
        unsigned int MARC_BASE_HI_2                                               : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_MARC_BASE_HI_3 {
    struct {
        unsigned int MARC_BASE_HI_3                                               : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_MARC_BASE_LO_0 {
    struct {
        unsigned int                                                              : 12;
        unsigned int MARC_BASE_LO_0                                               : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_MARC_BASE_LO_1 {
    struct {
        unsigned int                                                              : 12;
        unsigned int MARC_BASE_LO_1                                               : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_MARC_BASE_LO_2 {
    struct {
        unsigned int                                                              : 12;
        unsigned int MARC_BASE_LO_2                                               : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_MARC_BASE_LO_3 {
    struct {
        unsigned int                                                              : 12;
        unsigned int MARC_BASE_LO_3                                               : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_MARC_LEN_HI_0 {
    struct {
        unsigned int MARC_LEN_HI_0                                                : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_MARC_LEN_HI_1 {
    struct {
        unsigned int MARC_LEN_HI_1                                                : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_MARC_LEN_HI_2 {
    struct {
        unsigned int MARC_LEN_HI_2                                                : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_MARC_LEN_HI_3 {
    struct {
        unsigned int MARC_LEN_HI_3                                                : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_MARC_LEN_LO_0 {
    struct {
        unsigned int                                                              : 12;
        unsigned int MARC_LEN_LO_0                                                : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_MARC_LEN_LO_1 {
    struct {
        unsigned int                                                              : 12;
        unsigned int MARC_LEN_LO_1                                                : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_MARC_LEN_LO_2 {
    struct {
        unsigned int                                                              : 12;
        unsigned int MARC_LEN_LO_2                                                : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_MARC_LEN_LO_3 {
    struct {
        unsigned int                                                              : 12;
        unsigned int MARC_LEN_LO_3                                                : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_MARC_RELOC_HI_0 {
    struct {
        unsigned int MARC_RELOC_HI_0                                              : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_MARC_RELOC_HI_1 {
    struct {
        unsigned int MARC_RELOC_HI_1                                              : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_MARC_RELOC_HI_2 {
    struct {
        unsigned int MARC_RELOC_HI_2                                              : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_MARC_RELOC_HI_3 {
    struct {
        unsigned int MARC_RELOC_HI_3                                              : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_MARC_RELOC_LO_0 {
    struct {
        unsigned int MARC_ENABLE_0                                                :  1;
        unsigned int MARC_READONLY_0                                              :  1;
        unsigned int                                                              : 10;
        unsigned int MARC_RELOC_LO_0                                              : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_MARC_RELOC_LO_1 {
    struct {
        unsigned int MARC_ENABLE_1                                                :  1;
        unsigned int MARC_READONLY_1                                              :  1;
        unsigned int                                                              : 10;
        unsigned int MARC_RELOC_LO_1                                              : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_MARC_RELOC_LO_2 {
    struct {
        unsigned int MARC_ENABLE_2                                                :  1;
        unsigned int MARC_READONLY_2                                              :  1;
        unsigned int                                                              : 10;
        unsigned int MARC_RELOC_LO_2                                              : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_MARC_RELOC_LO_3 {
    struct {
        unsigned int MARC_ENABLE_3                                                :  1;
        unsigned int MARC_READONLY_3                                              :  1;
        unsigned int                                                              : 10;
        unsigned int MARC_RELOC_LO_3                                              : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_MX_L1_TLB_CNTL {
    struct {
        unsigned int ENABLE_L1_TLB                                                :  1;
        unsigned int                                                              :  2;
        unsigned int SYSTEM_ACCESS_MODE                                           :  2;
        unsigned int SYSTEM_APERTURE_UNMAPPED_ACCESS                              :  1;
        unsigned int ENABLE_ADVANCED_DRIVER_MODEL                                 :  1;
        unsigned int ECO_BITS                                                     :  4;
        unsigned int MTYPE                                                        :  3;
        unsigned int                                                              : 18;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_NB_LOWER_TOP_OF_DRAM2 {
    struct {
        unsigned int ENABLE                                                       :  1;
        unsigned int                                                              : 22;
        unsigned int LOWER_TOM2                                                   :  9;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_NB_MMIOBASE {
    struct {
        unsigned int MMIOBASE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_NB_MMIOLIMIT {
    struct {
        unsigned int MMIOLIMIT                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_NB_PCI_ARB {
    struct {
        unsigned int                                                              :  3;
        unsigned int VGA_HOLE                                                     :  1;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_NB_PCI_CTRL {
    struct {
        unsigned int                                                              : 23;
        unsigned int MMIOENABLE                                                   :  1;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_NB_TOP_OF_DRAM_SLOT1 {
    struct {
        unsigned int                                                              : 23;
        unsigned int TOP_OF_DRAM                                                  :  9;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_NB_UPPER_TOP_OF_DRAM2 {
    struct {
        unsigned int UPPER_TOM2                                                   : 12;
        unsigned int                                                              : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_STEERING {
    struct {
        unsigned int DEFAULT_STEERING                                             :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_SYSTEM_APERTURE_DEFAULT_ADDR_LSB {
    struct {
        unsigned int PHYSICAL_PAGE_NUMBER_LSB                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_SYSTEM_APERTURE_DEFAULT_ADDR_MSB {
    struct {
        unsigned int PHYSICAL_PAGE_NUMBER_MSB                                     :  4;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_SYSTEM_APERTURE_HIGH_ADDR {
    struct {
        unsigned int LOGICAL_ADDR                                                 : 30;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_SYSTEM_APERTURE_LOW_ADDR {
    struct {
        unsigned int LOGICAL_ADDR                                                 : 30;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCRD_SA_TARGETS_DISABLE {
    struct {
        unsigned int GCRD_TARGETS_DISABLE                                         : 19;
        unsigned int                                                              : 13;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCR_CGTT_SCLK_CTRL {
    struct {
        unsigned int ON_DELAY                                                     :  4;
        unsigned int OFF_HYSTERESIS                                               :  8;
        unsigned int                                                              :  4;
        unsigned int SOFT_STALL_OVERRIDE7                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE6                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE5                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE4                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE3                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE2                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE1                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE0                                         :  1;
        unsigned int                                                              :  1;
        unsigned int SOFT_OVERRIDE6                                               :  1;
        unsigned int SOFT_OVERRIDE5                                               :  1;
        unsigned int SOFT_OVERRIDE4                                               :  1;
        unsigned int SOFT_OVERRIDE3                                               :  1;
        unsigned int SOFT_OVERRIDE2                                               :  1;
        unsigned int SOFT_OVERRIDE1                                               :  1;
        unsigned int SOFT_OVERRIDE0                                               :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCR_CMD_STATUS {
    struct {
        unsigned int GCR_CONTROL                                                  : 19;
        unsigned int                                                              :  1;
        unsigned int GCR_SRC                                                      :  3;
        unsigned int GCR_TLB_SHOOTDOWN                                            :  1;
        unsigned int GCR_TLB_SHOOTDOWN_VMID                                       :  4;
        unsigned int UTCL2_NACK_STATUS                                            :  2;
        unsigned int GCR_SEQ_OP_ERROR                                             :  1;
        unsigned int UTCL2_NACK_ERROR                                             :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCR_GENERAL_CNTL {
    struct {
        unsigned int FORCE_4K_L2_RESP                                             :  1;
        unsigned int REDUCE_HALF_MAIN_WQ                                          :  1;
        unsigned int REDUCE_HALF_PHY_WQ                                           :  1;
        unsigned int FORCE_INV_ALL                                                :  1;
        unsigned int HI_PRIORITY_CNTL                                             :  2;
        unsigned int HI_PRIORITY_DISABLE                                          :  1;
        unsigned int BIG_PAGE_FILTER_DISABLE                                      :  1;
        unsigned int PERF_CNTR_ENABLE                                             :  1;
        unsigned int FORCE_SINGLE_WQ                                              :  1;
        unsigned int UTCL2_REQ_PERM                                               :  3;
        unsigned int TARGET_MGCG_CLKEN_DIS                                        :  1;
        unsigned int MIXED_RANGE_MODE_DIS                                         :  1;
        unsigned int                                                              :  5;
        unsigned int CLIENT_ID                                                    :  9;
        unsigned int                                                              :  3;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 15;
        unsigned int ENABLE_16K_UTCL2_REQ                                         :  1;
        unsigned int                                                              : 16;
    } gfx101Plus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCR_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCR_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCR_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              :  1;
        unsigned int PERF_SEL1                                                    :  9;
        unsigned int                                                              :  1;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCR_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    :  9;
        unsigned int                                                              :  1;
        unsigned int PERF_SEL3                                                    :  9;
        unsigned int                                                              :  5;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCR_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCR_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCR_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 15;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int CNTL_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCR_PIO_CNTL {
    struct {
        unsigned int GCR_DATA_INDEX                                               :  2;
        unsigned int GCR_REG_DONE                                                 :  1;
        unsigned int GCR_REG_RESET                                                :  1;
        unsigned int                                                              : 12;
        unsigned int GCR_PIO_RSP_TAG                                              :  8;
        unsigned int                                                              :  6;
        unsigned int GCR_PIO_RSP_DONE                                             :  1;
        unsigned int GCR_READY                                                    :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCR_PIO_DATA {
    struct {
        unsigned int GCR_DATA                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCR_SPARE {
    struct {
        unsigned int                                                              :  1;
        unsigned int SPARE_BIT_1                                                  :  1;
        unsigned int SPARE_BIT_2                                                  :  1;
        unsigned int SPARE_BIT_3                                                  :  1;
        unsigned int SPARE_BIT_4                                                  :  1;
        unsigned int SPARE_BIT_5                                                  :  1;
        unsigned int SPARE_BIT_6                                                  :  1;
        unsigned int SPARE_BIT_7                                                  :  1;
        unsigned int SPARE_BIT_8_0                                                :  8;
        unsigned int SPARE_BIT_31_16                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCR_TARGET_DISABLE {
    struct {
        unsigned int DISABLE_SA0_PHY                                              :  1;
        unsigned int DISABLE_SA0_VIRT                                             :  1;
        unsigned int DISABLE_SA1_PHY                                              :  1;
        unsigned int DISABLE_SA1_VIRT                                             :  1;
        unsigned int DISABLE_SA2_PHY                                              :  1;
        unsigned int DISABLE_SA2_VIRT                                             :  1;
        unsigned int DISABLE_SA3_PHY                                              :  1;
        unsigned int DISABLE_SA3_VIRT                                             :  1;
        unsigned int DISABLE_GL2A0_PHY                                            :  1;
        unsigned int DISABLE_GL2A1_PHY                                            :  1;
        unsigned int DISABLE_GL2A2_PHY                                            :  1;
        unsigned int DISABLE_GL2A3_PHY                                            :  1;
        unsigned int                                                              : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCUTCL2_CGTT_CLK_CTRL {
    struct {
        unsigned int ON_DELAY                                                     :  4;
        unsigned int OFF_HYSTERESIS                                               :  8;
        unsigned int SOFT_OVERRIDE_EXTRA                                          :  3;
        unsigned int MGLS_OVERRIDE                                                :  1;
        unsigned int SOFT_STALL_OVERRIDE                                          :  8;
        unsigned int SOFT_OVERRIDE                                                :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCVML2_PERFCOUNTER2_0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCVML2_PERFCOUNTER2_0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCVML2_PERFCOUNTER2_0_MODE {
    struct {
        unsigned int COMPARE_MODE0                                                :  2;
        unsigned int COMPARE_MODE1                                                :  2;
        unsigned int COMPARE_MODE2                                                :  2;
        unsigned int COMPARE_MODE3                                                :  2;
        unsigned int COMPARE_VALUE0                                               :  4;
        unsigned int COMPARE_VALUE1                                               :  4;
        unsigned int COMPARE_VALUE2                                               :  4;
        unsigned int COMPARE_VALUE3                                               :  4;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCVML2_PERFCOUNTER2_0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCVML2_PERFCOUNTER2_0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCVML2_PERFCOUNTER2_1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCVML2_PERFCOUNTER2_1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCVML2_PERFCOUNTER2_1_MODE {
    struct {
        unsigned int COMPARE_MODE0                                                :  2;
        unsigned int COMPARE_MODE1                                                :  2;
        unsigned int COMPARE_MODE2                                                :  2;
        unsigned int COMPARE_MODE3                                                :  2;
        unsigned int COMPARE_VALUE0                                               :  4;
        unsigned int COMPARE_VALUE1                                               :  4;
        unsigned int COMPARE_VALUE2                                               :  4;
        unsigned int COMPARE_VALUE3                                               :  4;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCVML2_PERFCOUNTER2_1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCVML2_PERFCOUNTER2_1_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCVML2_WALKER_MACRO_THROTTLE_FETCH_LIMIT {
    struct {
        unsigned int                                                              :  1;
        unsigned int LIMIT                                                        : 15;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCVML2_WALKER_MACRO_THROTTLE_TIME {
    struct {
        unsigned int TIME                                                         : 24;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCVML2_WALKER_MICRO_THROTTLE_FETCH_LIMIT {
    struct {
        unsigned int                                                              :  1;
        unsigned int LIMIT                                                        : 15;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCVML2_WALKER_MICRO_THROTTLE_TIME {
    struct {
        unsigned int TIME                                                         : 24;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_ATC_L2_PERFCOUNTER0_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_ATC_L2_PERFCOUNTER1_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_ATC_L2_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_ATC_L2_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_ATC_L2_PERFCOUNTER2_MODE {
    struct {
        unsigned int COMPARE_MODE0                                                :  2;
        unsigned int COMPARE_MODE1                                                :  2;
        unsigned int COMPARE_MODE2                                                :  2;
        unsigned int COMPARE_MODE3                                                :  2;
        unsigned int COMPARE_VALUE0                                               :  4;
        unsigned int COMPARE_VALUE1                                               :  4;
        unsigned int COMPARE_VALUE2                                               :  4;
        unsigned int COMPARE_VALUE3                                               :  4;
        unsigned int                                                              :  8;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_ATC_L2_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_ATC_L2_PERFCOUNTER2_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_ATC_L2_PERFCOUNTER_HI {
    struct {
        unsigned int COUNTER_HI                                                   : 16;
        unsigned int COMPARE_VALUE                                                : 16;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_ATC_L2_PERFCOUNTER_LO {
    struct {
        unsigned int COUNTER_LO                                                   : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_ATC_L2_PERFCOUNTER_RSLT_CNTL {
    struct {
        unsigned int PERF_COUNTER_SELECT                                          :  4;
        unsigned int                                                              :  4;
        unsigned int START_TRIGGER                                                :  8;
        unsigned int STOP_TRIGGER                                                 :  8;
        unsigned int ENABLE_ANY                                                   :  1;
        unsigned int CLEAR_ALL                                                    :  1;
        unsigned int STOP_ALL_ON_SATURATE                                         :  1;
        unsigned int                                                              :  5;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_BCI0 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_BCI1 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_CB0 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_CB1 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_CB2 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_CB3 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_CBR0 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_CBR1 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_CBR2 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_CBR3 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_CP0 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_CP1 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_CP2 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_CU0 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_CU1 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_CU2 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_CU3 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_CU4 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_CU5 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_CU6 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_CU7 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_CU8 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_CU9 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_CU10 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_CU11 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_CU12 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_CU13 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_CU14 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_CU15 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_DB0 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_DB1 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_DB2 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_DB3 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_DBR0 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_DBR1 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_DBR2 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_DBR3 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_EA0 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_EA1 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_EA2 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_EA3 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_EA4 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_EA5 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_GDS0 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_GDS1 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_GDS2 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_GDS3 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_IA0 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_LDS0 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_LDS1 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_LDS2 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_LDS3 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_PA0 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_PA1 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_PC0 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_PG0 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_RMI0 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_SC0 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_SPI0 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_SPI1 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_SPI2 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_SPI3 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_SPI4 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_SPI5 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_SQ0_LOWER {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_SQ0_UPPER {
    struct {
        unsigned int                                                              :  8;
        unsigned int UNUSED_0                                                     : 24;
    } gfx09_0;
    struct {
        unsigned int ACCUMULATOR_39_32                                            :  8;
        unsigned int UNUSED_0                                                     : 24;
    } gfx101;
    struct {
        unsigned int ACCUMULATOR_39_32                                            :  8;
        unsigned int                                                              : 24;
    } rv1x;
    struct {
        unsigned int ACCUMULATOR_39_32                                            :  8;
        unsigned int                                                              : 24;
    } rv2x;
    struct {
        unsigned int ACCUMULATOR_39_32                                            :  8;
        unsigned int                                                              : 24;
    } vega;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_SQ1_LOWER {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_SQ1_UPPER {
    struct {
        unsigned int                                                              :  8;
        unsigned int UNUSED_0                                                     : 24;
    } gfx09_0;
    struct {
        unsigned int ACCUMULATOR_39_32                                            :  8;
        unsigned int UNUSED_0                                                     : 24;
    } gfx101;
    struct {
        unsigned int ACCUMULATOR_39_32                                            :  8;
        unsigned int                                                              : 24;
    } rv1x;
    struct {
        unsigned int ACCUMULATOR_39_32                                            :  8;
        unsigned int                                                              : 24;
    } rv2x;
    struct {
        unsigned int ACCUMULATOR_39_32                                            :  8;
        unsigned int                                                              : 24;
    } vega;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_SQ2_LOWER {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_SQ2_UPPER {
    struct {
        unsigned int                                                              :  8;
        unsigned int UNUSED_0                                                     : 24;
    } gfx09_0;
    struct {
        unsigned int ACCUMULATOR_39_32                                            :  8;
        unsigned int UNUSED_0                                                     : 24;
    } gfx101;
    struct {
        unsigned int ACCUMULATOR_39_32                                            :  8;
        unsigned int                                                              : 24;
    } rv1x;
    struct {
        unsigned int ACCUMULATOR_39_32                                            :  8;
        unsigned int                                                              : 24;
    } rv2x;
    struct {
        unsigned int ACCUMULATOR_39_32                                            :  8;
        unsigned int                                                              : 24;
    } vega;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_SQ3_LOWER {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_SQ3_UPPER {
    struct {
        unsigned int                                                              :  8;
        unsigned int UNUSED_0                                                     : 24;
    } gfx09_0;
    struct {
        unsigned int ACCUMULATOR_39_32                                            :  8;
        unsigned int UNUSED_0                                                     : 24;
    } gfx101;
    struct {
        unsigned int ACCUMULATOR_39_32                                            :  8;
        unsigned int                                                              : 24;
    } rv1x;
    struct {
        unsigned int ACCUMULATOR_39_32                                            :  8;
        unsigned int                                                              : 24;
    } rv2x;
    struct {
        unsigned int ACCUMULATOR_39_32                                            :  8;
        unsigned int                                                              : 24;
    } vega;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_SQ4_LOWER {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_SQ4_UPPER {
    struct {
        unsigned int                                                              :  8;
        unsigned int UNUSED_0                                                     : 24;
    } gfx09_0;
    struct {
        unsigned int ACCUMULATOR_39_32                                            :  8;
        unsigned int UNUSED_0                                                     : 24;
    } gfx101;
    struct {
        unsigned int ACCUMULATOR_39_32                                            :  8;
        unsigned int                                                              : 24;
    } rv1x;
    struct {
        unsigned int ACCUMULATOR_39_32                                            :  8;
        unsigned int                                                              : 24;
    } rv2x;
    struct {
        unsigned int ACCUMULATOR_39_32                                            :  8;
        unsigned int                                                              : 24;
    } vega;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_SQ5_LOWER {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_SQ5_UPPER {
    struct {
        unsigned int                                                              :  8;
        unsigned int UNUSED_0                                                     : 24;
    } gfx09_0;
    struct {
        unsigned int ACCUMULATOR_39_32                                            :  8;
        unsigned int UNUSED_0                                                     : 24;
    } gfx101;
    struct {
        unsigned int ACCUMULATOR_39_32                                            :  8;
        unsigned int                                                              : 24;
    } rv1x;
    struct {
        unsigned int ACCUMULATOR_39_32                                            :  8;
        unsigned int                                                              : 24;
    } rv2x;
    struct {
        unsigned int ACCUMULATOR_39_32                                            :  8;
        unsigned int                                                              : 24;
    } vega;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_SQ6_LOWER {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_SQ6_UPPER {
    struct {
        unsigned int                                                              :  8;
        unsigned int UNUSED_0                                                     : 24;
    } gfx09_0;
    struct {
        unsigned int ACCUMULATOR_39_32                                            :  8;
        unsigned int UNUSED_0                                                     : 24;
    } gfx101;
    struct {
        unsigned int ACCUMULATOR_39_32                                            :  8;
        unsigned int                                                              : 24;
    } rv1x;
    struct {
        unsigned int ACCUMULATOR_39_32                                            :  8;
        unsigned int                                                              : 24;
    } rv2x;
    struct {
        unsigned int ACCUMULATOR_39_32                                            :  8;
        unsigned int                                                              : 24;
    } vega;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_SQ7_LOWER {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_SQ7_UPPER {
    struct {
        unsigned int                                                              :  8;
        unsigned int UNUSED_0                                                     : 24;
    } gfx09_0;
    struct {
        unsigned int ACCUMULATOR_39_32                                            :  8;
        unsigned int UNUSED_0                                                     : 24;
    } gfx101;
    struct {
        unsigned int ACCUMULATOR_39_32                                            :  8;
        unsigned int                                                              : 24;
    } rv1x;
    struct {
        unsigned int ACCUMULATOR_39_32                                            :  8;
        unsigned int                                                              : 24;
    } rv2x;
    struct {
        unsigned int ACCUMULATOR_39_32                                            :  8;
        unsigned int                                                              : 24;
    } vega;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_SQ8_LOWER {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_SQ8_UPPER {
    struct {
        unsigned int                                                              :  8;
        unsigned int UNUSED_0                                                     : 24;
    } gfx09_0;
    struct {
        unsigned int ACCUMULATOR_39_32                                            :  8;
        unsigned int UNUSED_0                                                     : 24;
    } gfx101;
    struct {
        unsigned int ACCUMULATOR_39_32                                            :  8;
        unsigned int                                                              : 24;
    } rv1x;
    struct {
        unsigned int ACCUMULATOR_39_32                                            :  8;
        unsigned int                                                              : 24;
    } rv2x;
    struct {
        unsigned int ACCUMULATOR_39_32                                            :  8;
        unsigned int                                                              : 24;
    } vega;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_SX0 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_SXRB0 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_SXRB1 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_TA0 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_TCC0 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_TCC1 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_TCC2 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_TCC3 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_TCC4 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_TCP0 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_TCP1 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_TCP2 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_TCP3 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_TCP4 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_TD0 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_TD1 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_TD2 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_TD3 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_TD4 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_TD5 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_UTCL2_ATCL20 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_UTCL2_ATCL21 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_UTCL2_ATCL22 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_UTCL2_ATCL23 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_UTCL2_ATCL24 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_UTCL2_ROUTER0 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_UTCL2_ROUTER1 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_UTCL2_ROUTER2 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_UTCL2_ROUTER3 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_UTCL2_ROUTER4 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_UTCL2_ROUTER5 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_UTCL2_ROUTER6 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_UTCL2_ROUTER7 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_UTCL2_ROUTER8 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_UTCL2_ROUTER9 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_UTCL2_VML20 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_UTCL2_VML21 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_UTCL2_VML22 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_UTCL2_VML23 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_UTCL2_VML24 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_UTCL2_WALKER0 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_UTCL2_WALKER1 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_UTCL2_WALKER2 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_UTCL2_WALKER3 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_UTCL2_WALKER4 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_VGT0 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_VGT1 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_VGT2 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_ACC_WD0 {
    struct {
        unsigned int ACCUMULATOR_31_0                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_AGGR_LOWER {
    struct {
        unsigned int AGGR_31_0                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_AGGR_UPPER {
    struct {
        unsigned int AGGR_63_32                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_CGTT_CLK_CTRL {
    struct {
        unsigned int ON_DELAY                                                     :  4;
        unsigned int OFF_HYSTERESIS                                               :  8;
        unsigned int                                                              : 18;
        unsigned int SOFT_OVERRIDE_DYN                                            :  1;
        unsigned int SOFT_OVERRIDE_REG                                            :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_CNTL {
    struct {
        unsigned int CAC_ENABLE                                                   :  1;
        unsigned int                                                              : 30;
        unsigned int UNUSED_0                                                     :  1;
    } gfx09_0;
    struct {
        unsigned int CAC_FORCE_DISABLE                                            :  1;
        unsigned int CAC_THRESHOLD                                                : 16;
        unsigned int UNUSED_0                                                     : 15;
    } gfx101;
    struct {
        unsigned int                                                              :  1;
        unsigned int CAC_THRESHOLD                                                : 16;
        unsigned int CAC_BLOCK_ID                                                 :  6;
        unsigned int CAC_SIGNAL_ID                                                :  8;
        unsigned int                                                              :  1;
    } rv1x;
    struct {
        unsigned int CAC_ENABLE                                                   :  1;
        unsigned int CAC_THRESHOLD                                                : 16;
        unsigned int CAC_BLOCK_ID                                                 :  6;
        unsigned int CAC_SIGNAL_ID                                                :  8;
        unsigned int                                                              :  1;
    } rv2x;
    struct {
        unsigned int                                                              :  1;
        unsigned int CAC_THRESHOLD                                                : 16;
        unsigned int CAC_BLOCK_ID                                                 :  6;
        unsigned int CAC_SIGNAL_ID                                                :  8;
        unsigned int                                                              :  1;
    } vega;
    struct {
        unsigned int CAC_FORCE_DISABLE                                            :  1;
        unsigned int                                                              : 31;
    } vg12;
    struct {
        unsigned int CAC_FORCE_DISABLE                                            :  1;
        unsigned int                                                              : 31;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_CTRL_1 {
    struct {
        unsigned int CAC_WINDOW                                                   : 24;
        unsigned int TDP_WINDOW                                                   :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_CTRL_2 {
    struct {
        unsigned int CAC_ENABLE                                                   :  1;
        unsigned int CAC_SOFT_CTRL_ENABLE                                         :  1;
        unsigned int                                                              : 30;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  2;
        unsigned int UNUSED_0                                                     : 30;
    } gfx09_0;
    struct {
        unsigned int                                                              :  2;
        unsigned int GC_LCAC_ENABLE                                               :  1;
        unsigned int SE_LCAC_ENABLE                                               :  1;
        unsigned int                                                              : 28;
    } gfx09_1xPlus;
    struct {
        unsigned int                                                              :  4;
        unsigned int GC_CAC_INDEX_AUTO_INCR_EN                                    :  1;
        unsigned int                                                              : 27;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_INDEX_AUTO_INCR_EN {
    struct {
        unsigned int GC_CAC_INDEX_AUTO_INCR_EN                                    :  1;
        unsigned int                                                              : 31;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_IND_DATA {
    struct {
        unsigned int GC_CAC_IND_DATA                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_IND_INDEX {
    struct {
        unsigned int GC_CAC_IND_ADDR                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_OVRD_BCI {
    struct {
        unsigned int OVRRD_SELECT                                                 :  2;
        unsigned int OVRRD_VALUE                                                  :  2;
        unsigned int                                                              : 28;
    } most;
    struct {
        unsigned int OVRRD_SELECT                                                 :  1;
        unsigned int OVRRD_VALUE                                                  :  1;
        unsigned int                                                              : 30;
    } rv2x;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_OVRD_CB {
    struct {
        unsigned int OVRRD_SELECT                                                 :  4;
        unsigned int OVRRD_VALUE                                                  :  4;
        unsigned int                                                              : 24;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_OVRD_CBR {
    struct {
        unsigned int OVRRD_SELECT                                                 :  4;
        unsigned int OVRRD_VALUE                                                  :  4;
        unsigned int                                                              : 24;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_OVRD_CP {
    struct {
        unsigned int OVRRD_SELECT                                                 :  3;
        unsigned int OVRRD_VALUE                                                  :  3;
        unsigned int                                                              : 26;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_OVRD_CU {
    struct {
        unsigned int OVRRD_SELECT                                                 :  1;
        unsigned int OVRRD_VALUE                                                  :  1;
        unsigned int                                                              : 30;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_OVRD_DB {
    struct {
        unsigned int OVRRD_SELECT                                                 :  4;
        unsigned int OVRRD_VALUE                                                  :  4;
        unsigned int                                                              : 24;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_OVRD_DBR {
    struct {
        unsigned int OVRRD_SELECT                                                 :  4;
        unsigned int OVRRD_VALUE                                                  :  4;
        unsigned int                                                              : 24;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_OVRD_EA {
    struct {
        unsigned int OVRRD_SELECT                                                 :  6;
        unsigned int OVRRD_VALUE                                                  :  6;
        unsigned int                                                              : 20;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_OVRD_GDS {
    struct {
        unsigned int OVRRD_SELECT                                                 :  4;
        unsigned int OVRRD_VALUE                                                  :  4;
        unsigned int                                                              : 24;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_OVRD_IA {
    struct {
        unsigned int OVRRD_SELECT                                                 :  1;
        unsigned int OVRRD_VALUE                                                  :  1;
        unsigned int                                                              : 30;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_OVRD_LDS {
    struct {
        unsigned int OVRRD_SELECT                                                 :  4;
        unsigned int OVRRD_VALUE                                                  :  4;
        unsigned int                                                              : 24;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_OVRD_PA {
    struct {
        unsigned int OVRRD_SELECT                                                 :  2;
        unsigned int OVRRD_VALUE                                                  :  2;
        unsigned int                                                              : 28;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_OVRD_PC {
    struct {
        unsigned int OVRRD_SELECT                                                 :  1;
        unsigned int OVRRD_VALUE                                                  :  1;
        unsigned int                                                              : 30;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_OVRD_PG {
    struct {
        unsigned int OVRRD_SELECT                                                 : 16;
        unsigned int OVRRD_VALUE                                                  : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_OVRD_RMI {
    struct {
        unsigned int OVRRD_SELECT                                                 :  1;
        unsigned int OVRRD_VALUE                                                  :  1;
        unsigned int                                                              : 30;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_OVRD_SC {
    struct {
        unsigned int OVRRD_SELECT                                                 :  1;
        unsigned int OVRRD_VALUE                                                  :  1;
        unsigned int                                                              : 30;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_OVRD_SPI {
    struct {
        unsigned int OVRRD_SELECT                                                 :  6;
        unsigned int OVRRD_VALUE                                                  :  6;
        unsigned int                                                              : 20;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_OVRD_SQ {
    struct {
        unsigned int OVRRD_SELECT                                                 :  9;
        unsigned int OVRRD_VALUE                                                  :  9;
        unsigned int                                                              : 14;
    } most;
    struct {
        unsigned int OVRRD_SELECT                                                 :  6;
        unsigned int OVRRD_VALUE                                                  :  6;
        unsigned int                                                              : 20;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_OVRD_SX {
    struct {
        unsigned int OVRRD_SELECT                                                 :  1;
        unsigned int OVRRD_VALUE                                                  :  1;
        unsigned int                                                              : 30;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_OVRD_SXRB {
    struct {
        unsigned int OVRRD_SELECT                                                 :  1;
        unsigned int OVRRD_VALUE                                                  :  1;
        unsigned int                                                              : 30;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_OVRD_TA {
    struct {
        unsigned int OVRRD_SELECT                                                 :  1;
        unsigned int OVRRD_VALUE                                                  :  1;
        unsigned int                                                              : 30;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_OVRD_TCC {
    struct {
        unsigned int OVRRD_SELECT                                                 :  5;
        unsigned int OVRRD_VALUE                                                  :  5;
        unsigned int                                                              : 22;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_OVRD_TCP {
    struct {
        unsigned int OVRRD_SELECT                                                 :  5;
        unsigned int OVRRD_VALUE                                                  :  5;
        unsigned int                                                              : 22;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_OVRD_TD {
    struct {
        unsigned int OVRRD_SELECT                                                 :  6;
        unsigned int OVRRD_VALUE                                                  :  6;
        unsigned int                                                              : 20;
    } most;
    struct {
        unsigned int OVRRD_SELECT                                                 : 10;
        unsigned int OVRRD_VALUE                                                  : 10;
        unsigned int                                                              : 12;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_OVRD_UTCL2_ATCL2 {
    struct {
        unsigned int OVRRD_SELECT                                                 :  5;
        unsigned int OVRRD_VALUE                                                  :  5;
        unsigned int                                                              : 22;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_OVRD_UTCL2_ROUTER {
    struct {
        unsigned int OVRRD_SELECT                                                 : 10;
        unsigned int OVRRD_VALUE                                                  : 10;
        unsigned int                                                              : 12;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_OVRD_UTCL2_VML2 {
    struct {
        unsigned int OVRRD_SELECT                                                 :  5;
        unsigned int OVRRD_VALUE                                                  :  5;
        unsigned int                                                              : 22;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_OVRD_UTCL2_WALKER {
    struct {
        unsigned int OVRRD_SELECT                                                 :  5;
        unsigned int OVRRD_VALUE                                                  :  5;
        unsigned int                                                              : 22;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_OVRD_VGT {
    struct {
        unsigned int OVRRD_SELECT                                                 :  3;
        unsigned int OVRRD_VALUE                                                  :  3;
        unsigned int                                                              : 26;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_OVRD_WD {
    struct {
        unsigned int OVRRD_SELECT                                                 :  1;
        unsigned int OVRRD_VALUE                                                  :  1;
        unsigned int                                                              : 30;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_OVR_SEL {
    struct {
        unsigned int CAC_OVR_SEL                                                  : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_OVR_VAL {
    struct {
        unsigned int CAC_OVR_VAL                                                  : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_PG_AGGR_LOWER {
    struct {
        unsigned int LKG_AGGR_31_0                                                : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_PG_AGGR_UPPER {
    struct {
        unsigned int LKG_AGGR_63_32                                               : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_SOFT_CTRL {
    struct {
        unsigned int SOFT_SNAP                                                    :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  1;
        unsigned int UNUSED                                                       : 31;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_BCI_0 {
    struct {
        unsigned int WEIGHT_BCI_SIG0                                              : 16;
        unsigned int WEIGHT_BCI_SIG1                                              : 16;
    } gfx101;
    struct {
        unsigned int WEIGHT_BCI_SIG0                                              : 16;
        unsigned int WEIGHT_BCI_SIG1                                              : 16;
    } rv1x;
    struct {
        unsigned int WEIGHT_BCI_SIG0                                              : 16;
        unsigned int                                                              : 16;
    } rv2x;
    struct {
        unsigned int WEIGHT_BCI_SIG0                                              : 16;
        unsigned int WEIGHT_BCI_SIG1                                              : 16;
    } vega;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_CBR_0 {
    struct {
        unsigned int WEIGHT_CBR_SIG0                                              : 16;
        unsigned int WEIGHT_CBR_SIG1                                              : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_CBR_1 {
    struct {
        unsigned int WEIGHT_CBR_SIG2                                              : 16;
        unsigned int WEIGHT_CBR_SIG3                                              : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_CB_0 {
    struct {
        unsigned int WEIGHT_CB_SIG0                                               : 16;
        unsigned int WEIGHT_CB_SIG1                                               : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_CB_1 {
    struct {
        unsigned int WEIGHT_CB_SIG2                                               : 16;
        unsigned int WEIGHT_CB_SIG3                                               : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_CP_0 {
    struct {
        unsigned int WEIGHT_CP_SIG0                                               : 16;
        unsigned int WEIGHT_CP_SIG1                                               : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_CP_1 {
    struct {
        unsigned int                                                              : 16;
        unsigned int UNUSED_0                                                     : 16;
    } gfx09_0;
    struct {
        unsigned int WEIGHT_CP_SIG2                                               : 16;
        unsigned int UNUSED_0                                                     : 16;
    } gfx101;
    struct {
        unsigned int WEIGHT_CP_SIG2                                               : 16;
        unsigned int                                                              : 16;
    } rv1x;
    struct {
        unsigned int WEIGHT_CP_SIG2                                               : 16;
        unsigned int                                                              : 16;
    } rv2x;
    struct {
        unsigned int WEIGHT_CP_SIG2                                               : 16;
        unsigned int                                                              : 16;
    } vega;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_CU_0 {
    struct {
        unsigned int                                                              : 16;
        unsigned int WEIGHT_CU_SIG1                                               : 16;
    } gfx09_0;
    struct {
        unsigned int WEIGHT_CU_SIG0                                               : 16;
        unsigned int UNUSED_0                                                     : 16;
    } gfx101;
    struct {
        unsigned int WEIGHT_CU_SIG0                                               : 16;
        unsigned int                                                              : 16;
    } rv1x;
    struct {
        unsigned int WEIGHT_CU_SIG0                                               : 16;
        unsigned int WEIGHT_CU_SIG1                                               : 16;
    } rv2x;
    struct {
        unsigned int WEIGHT_CU_SIG0                                               : 16;
        unsigned int                                                              : 16;
    } vega;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_CU_1 {
    struct {
        unsigned int WEIGHT_CU_SIG2                                               : 16;
        unsigned int WEIGHT_CU_SIG3                                               : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_CU_2 {
    struct {
        unsigned int WEIGHT_CU_SIG4                                               : 16;
        unsigned int WEIGHT_CU_SIG5                                               : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_CU_3 {
    struct {
        unsigned int WEIGHT_CU_SIG6                                               : 16;
        unsigned int WEIGHT_CU_SIG7                                               : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_CU_4 {
    struct {
        unsigned int WEIGHT_CU_SIG8                                               : 16;
        unsigned int WEIGHT_CU_SIG9                                               : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_CU_5 {
    struct {
        unsigned int WEIGHT_CU_SIG10                                              : 16;
        unsigned int WEIGHT_CU_SIG11                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_CU_6 {
    struct {
        unsigned int WEIGHT_CU_SIG12                                              : 16;
        unsigned int WEIGHT_CU_SIG13                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_CU_7 {
    struct {
        unsigned int WEIGHT_CU_SIG14                                              : 16;
        unsigned int WEIGHT_CU_SIG15                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_DBR_0 {
    struct {
        unsigned int WEIGHT_DBR_SIG0                                              : 16;
        unsigned int WEIGHT_DBR_SIG1                                              : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_DBR_1 {
    struct {
        unsigned int WEIGHT_DBR_SIG2                                              : 16;
        unsigned int WEIGHT_DBR_SIG3                                              : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_DB_0 {
    struct {
        unsigned int WEIGHT_DB_SIG0                                               : 16;
        unsigned int WEIGHT_DB_SIG1                                               : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_DB_1 {
    struct {
        unsigned int WEIGHT_DB_SIG2                                               : 16;
        unsigned int WEIGHT_DB_SIG3                                               : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_EA_0 {
    struct {
        unsigned int WEIGHT_EA_SIG0                                               : 16;
        unsigned int WEIGHT_EA_SIG1                                               : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_EA_1 {
    struct {
        unsigned int WEIGHT_EA_SIG2                                               : 16;
        unsigned int WEIGHT_EA_SIG3                                               : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_EA_2 {
    struct {
        unsigned int WEIGHT_EA_SIG4                                               : 16;
        unsigned int WEIGHT_EA_SIG5                                               : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_IA_0 {
    struct {
        unsigned int WEIGHT_IA_SIG0                                               : 16;
        unsigned int                                                              : 16;
    } most;
    struct {
        unsigned int                                                              : 16;
        unsigned int UNUSED_0                                                     : 16;
    } gfx09_0;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_LDS_0 {
    struct {
        unsigned int WEIGHT_LDS_SIG0                                              : 16;
        unsigned int WEIGHT_LDS_SIG1                                              : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_LDS_1 {
    struct {
        unsigned int WEIGHT_LDS_SIG2                                              : 16;
        unsigned int WEIGHT_LDS_SIG3                                              : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_PA_0 {
    struct {
        unsigned int WEIGHT_PA_SIG0                                               : 16;
        unsigned int WEIGHT_PA_SIG1                                               : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_PC_0 {
    struct {
        unsigned int                                                              : 16;
        unsigned int UNUSED_0                                                     : 16;
    } gfx09_0;
    struct {
        unsigned int WEIGHT_PC_SIG0                                               : 16;
        unsigned int UNUSED_0                                                     : 16;
    } gfx101;
    struct {
        unsigned int WEIGHT_PC_SIG0                                               : 16;
        unsigned int                                                              : 16;
    } rv1x;
    struct {
        unsigned int WEIGHT_PC_SIG0                                               : 16;
        unsigned int                                                              : 16;
    } rv2x;
    struct {
        unsigned int WEIGHT_PC_SIG0                                               : 16;
        unsigned int                                                              : 16;
    } vega;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_PG_0 {
    struct {
        unsigned int WEIGHT_PG_SIG0                                               : 16;
        unsigned int                                                              : 16;
    } most;
    struct {
        unsigned int                                                              : 16;
        unsigned int unused                                                       : 16;
    } rv1x;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_RMI_0 {
    struct {
        unsigned int WEIGHT_RMI_SIG0                                              : 16;
        unsigned int                                                              : 16;
    } most;
    struct {
        unsigned int                                                              : 16;
        unsigned int UNUSED                                                       : 16;
    } gfx09_0;
    struct {
        unsigned int                                                              : 16;
        unsigned int UNUSED_0                                                     : 16;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_SC_0 {
    struct {
        unsigned int                                                              : 16;
        unsigned int UNUSED_0                                                     : 16;
    } gfx09_0;
    struct {
        unsigned int WEIGHT_SC_SIG0                                               : 16;
        unsigned int UNUSED_0                                                     : 16;
    } gfx101;
    struct {
        unsigned int WEIGHT_SC_SIG0                                               : 16;
        unsigned int                                                              : 16;
    } rv1x;
    struct {
        unsigned int WEIGHT_SC_SIG0                                               : 16;
        unsigned int                                                              : 16;
    } rv2x;
    struct {
        unsigned int WEIGHT_SC_SIG0                                               : 16;
        unsigned int                                                              : 16;
    } vega;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_SPI_0 {
    struct {
        unsigned int WEIGHT_SPI_SIG0                                              : 16;
        unsigned int WEIGHT_SPI_SIG1                                              : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_SPI_1 {
    struct {
        unsigned int WEIGHT_SPI_SIG2                                              : 16;
        unsigned int WEIGHT_SPI_SIG3                                              : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_SPI_2 {
    struct {
        unsigned int WEIGHT_SPI_SIG4                                              : 16;
        unsigned int WEIGHT_SPI_SIG5                                              : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_SQ_0 {
    struct {
        unsigned int WEIGHT_SQ_SIG0                                               : 16;
        unsigned int WEIGHT_SQ_SIG1                                               : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_SQ_1 {
    struct {
        unsigned int WEIGHT_SQ_SIG2                                               : 16;
        unsigned int WEIGHT_SQ_SIG3                                               : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_SQ_2 {
    struct {
        unsigned int WEIGHT_SQ_SIG4                                               : 16;
        unsigned int WEIGHT_SQ_SIG5                                               : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_SQ_3 {
    struct {
        unsigned int WEIGHT_SQ_SIG6                                               : 16;
        unsigned int WEIGHT_SQ_SIG7                                               : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_SQ_4 {
    struct {
        unsigned int WEIGHT_SQ_SIG8                                               : 16;
        unsigned int                                                              : 16;
    } most;
    struct {
        unsigned int                                                              : 16;
        unsigned int UNUSED_0                                                     : 16;
    } gfx09_0;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_SXRB_0 {
    struct {
        unsigned int                                                              : 16;
        unsigned int WEIGHT_SXRB_SIG1                                             : 16;
    } gfx09_0;
    struct {
        unsigned int WEIGHT_SXRB_SIG0                                             : 16;
        unsigned int UNUSED_0                                                     : 16;
    } gfx101;
    struct {
        unsigned int WEIGHT_SXRB_SIG0                                             : 16;
        unsigned int                                                              : 16;
    } rv1x;
    struct {
        unsigned int WEIGHT_SXRB_SIG0                                             : 16;
        unsigned int WEIGHT_SXRB_SIG1                                             : 16;
    } rv2x;
    struct {
        unsigned int WEIGHT_SXRB_SIG0                                             : 16;
        unsigned int                                                              : 16;
    } vega;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_SX_0 {
    struct {
        unsigned int                                                              : 16;
        unsigned int UNUSED_0                                                     : 16;
    } gfx09_0;
    struct {
        unsigned int WEIGHT_SX_SIG0                                               : 16;
        unsigned int UNUSED_0                                                     : 16;
    } gfx101;
    struct {
        unsigned int WEIGHT_SX_SIG0                                               : 16;
        unsigned int                                                              : 16;
    } rv1x;
    struct {
        unsigned int WEIGHT_SX_SIG0                                               : 16;
        unsigned int                                                              : 16;
    } rv2x;
    struct {
        unsigned int WEIGHT_SX_SIG0                                               : 16;
        unsigned int                                                              : 16;
    } vega;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_TA_0 {
    struct {
        unsigned int                                                              : 16;
        unsigned int UNUSED_0                                                     : 16;
    } gfx09_0;
    struct {
        unsigned int WEIGHT_TA_SIG0                                               : 16;
        unsigned int UNUSED_0                                                     : 16;
    } gfx101;
    struct {
        unsigned int WEIGHT_TA_SIG0                                               : 16;
        unsigned int                                                              : 16;
    } rv1x;
    struct {
        unsigned int WEIGHT_TA_SIG0                                               : 16;
        unsigned int                                                              : 16;
    } rv2x;
    struct {
        unsigned int WEIGHT_TA_SIG0                                               : 16;
        unsigned int                                                              : 16;
    } vega;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_TD_0 {
    struct {
        unsigned int WEIGHT_TD_SIG0                                               : 16;
        unsigned int WEIGHT_TD_SIG1                                               : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_TD_1 {
    struct {
        unsigned int WEIGHT_TD_SIG2                                               : 16;
        unsigned int WEIGHT_TD_SIG3                                               : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_TD_2 {
    struct {
        unsigned int WEIGHT_TD_SIG4                                               : 16;
        unsigned int WEIGHT_TD_SIG5                                               : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_UTCL2_ATCL2_0 {
    struct {
        unsigned int WEIGHT_UTCL2_ATCL2_SIG0                                      : 16;
        unsigned int WEIGHT_UTCL2_ATCL2_SIG1                                      : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_UTCL2_ATCL2_1 {
    struct {
        unsigned int WEIGHT_UTCL2_ATCL2_SIG2                                      : 16;
        unsigned int WEIGHT_UTCL2_ATCL2_SIG3                                      : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_UTCL2_ATCL2_2 {
    struct {
        unsigned int                                                              : 16;
        unsigned int WEIGHT_UTCL2_ATCL2_SIG5                                      : 16;
    } gfx09_0;
    struct {
        unsigned int WEIGHT_UTCL2_ATCL2_SIG4                                      : 16;
        unsigned int UNUSED_0                                                     : 16;
    } gfx101;
    struct {
        unsigned int WEIGHT_UTCL2_ATCL2_SIG4                                      : 16;
        unsigned int                                                              : 16;
    } rv1x;
    struct {
        unsigned int WEIGHT_UTCL2_ATCL2_SIG4                                      : 16;
        unsigned int WEIGHT_UTCL2_ATCL2_SIG5                                      : 16;
    } rv2x;
    struct {
        unsigned int WEIGHT_UTCL2_ATCL2_SIG4                                      : 16;
        unsigned int                                                              : 16;
    } vega;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_UTCL2_ROUTER_0 {
    struct {
        unsigned int WEIGHT_UTCL2_ROUTER_SIG0                                     : 16;
        unsigned int WEIGHT_UTCL2_ROUTER_SIG1                                     : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_UTCL2_ROUTER_1 {
    struct {
        unsigned int WEIGHT_UTCL2_ROUTER_SIG2                                     : 16;
        unsigned int WEIGHT_UTCL2_ROUTER_SIG3                                     : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_UTCL2_ROUTER_2 {
    struct {
        unsigned int WEIGHT_UTCL2_ROUTER_SIG4                                     : 16;
        unsigned int WEIGHT_UTCL2_ROUTER_SIG5                                     : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_UTCL2_ROUTER_3 {
    struct {
        unsigned int WEIGHT_UTCL2_ROUTER_SIG6                                     : 16;
        unsigned int WEIGHT_UTCL2_ROUTER_SIG7                                     : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_UTCL2_ROUTER_4 {
    struct {
        unsigned int WEIGHT_UTCL2_ROUTER_SIG8                                     : 16;
        unsigned int WEIGHT_UTCL2_ROUTER_SIG9                                     : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_UTCL2_VML2_0 {
    struct {
        unsigned int WEIGHT_UTCL2_VML2_SIG0                                       : 16;
        unsigned int WEIGHT_UTCL2_VML2_SIG1                                       : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_UTCL2_VML2_1 {
    struct {
        unsigned int WEIGHT_UTCL2_VML2_SIG2                                       : 16;
        unsigned int WEIGHT_UTCL2_VML2_SIG3                                       : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_UTCL2_VML2_2 {
    struct {
        unsigned int                                                              : 16;
        unsigned int WEIGHT_UTCL2_VML2_SIG5                                       : 16;
    } gfx09_0;
    struct {
        unsigned int WEIGHT_UTCL2_VML2_SIG4                                       : 16;
        unsigned int UNUSED_0                                                     : 16;
    } gfx101;
    struct {
        unsigned int WEIGHT_UTCL2_VML2_SIG4                                       : 16;
        unsigned int                                                              : 16;
    } rv1x;
    struct {
        unsigned int WEIGHT_UTCL2_VML2_SIG4                                       : 16;
        unsigned int WEIGHT_UTCL2_VML2_SIG5                                       : 16;
    } rv2x;
    struct {
        unsigned int WEIGHT_UTCL2_VML2_SIG4                                       : 16;
        unsigned int                                                              : 16;
    } vega;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_UTCL2_WALKER_0 {
    struct {
        unsigned int WEIGHT_UTCL2_WALKER_SIG0                                     : 16;
        unsigned int WEIGHT_UTCL2_WALKER_SIG1                                     : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_UTCL2_WALKER_1 {
    struct {
        unsigned int WEIGHT_UTCL2_WALKER_SIG2                                     : 16;
        unsigned int WEIGHT_UTCL2_WALKER_SIG3                                     : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_UTCL2_WALKER_2 {
    struct {
        unsigned int                                                              : 16;
        unsigned int WEIGHT_UTCL2_WALKER_SIG5                                     : 16;
    } gfx09_0;
    struct {
        unsigned int WEIGHT_UTCL2_WALKER_SIG4                                     : 16;
        unsigned int UNUSED_0                                                     : 16;
    } gfx101;
    struct {
        unsigned int WEIGHT_UTCL2_WALKER_SIG4                                     : 16;
        unsigned int                                                              : 16;
    } rv1x;
    struct {
        unsigned int WEIGHT_UTCL2_WALKER_SIG4                                     : 16;
        unsigned int WEIGHT_UTCL2_WALKER_SIG5                                     : 16;
    } rv2x;
    struct {
        unsigned int WEIGHT_UTCL2_WALKER_SIG4                                     : 16;
        unsigned int                                                              : 16;
    } vega;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_VGT_0 {
    struct {
        unsigned int WEIGHT_VGT_SIG0                                              : 16;
        unsigned int WEIGHT_VGT_SIG1                                              : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_VGT_1 {
    struct {
        unsigned int WEIGHT_VGT_SIG2                                              : 16;
        unsigned int                                                              : 16;
    } most;
    struct {
        unsigned int                                                              : 16;
        unsigned int UNUSED_0                                                     : 16;
    } gfx09_0;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_CAC_WEIGHT_WD_0 {
    struct {
        unsigned int WEIGHT_WD_SIG0                                               : 16;
        unsigned int                                                              : 16;
    } most;
    struct {
        unsigned int                                                              : 16;
        unsigned int UNUSED_0                                                     : 16;
    } gfx09_0;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_DIDT_CTRL0 {
    struct {
        unsigned int DIDT_CTRL_EN                                                 :  1;
        unsigned int PHASE_OFFSET                                                 :  2;
        unsigned int DIDT_SW_RST                                                  :  1;
        unsigned int DIDT_CLK_EN_OVERRIDE                                         :  1;
        unsigned int DIDT_TRIGGER_THROTTLE_LOWBIT                                 :  4;
        unsigned int                                                              : 23;
    } most;
    struct {
        unsigned int                                                              :  9;
        unsigned int DIDT_PSM_SELECTION                                           :  1;
        unsigned int                                                              : 22;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_DIDT_CTRL1 {
    struct {
        unsigned int MIN_POWER                                                    : 16;
        unsigned int MAX_POWER                                                    : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_DIDT_CTRL2 {
    struct {
        unsigned int MAX_POWER_DELTA                                              : 14;
        unsigned int                                                              :  2;
        unsigned int SHORT_TERM_INTERVAL_SIZE                                     : 10;
        unsigned int                                                              :  1;
        unsigned int LONG_TERM_INTERVAL_RATIO                                     :  4;
        unsigned int                                                              :  1;
    } gfx09;
    struct {
        unsigned int                                                              : 14;
        unsigned int UNUSED_0                                                     :  2;
        unsigned int                                                              : 10;
        unsigned int UNUSED_1                                                     :  1;
        unsigned int                                                              :  4;
        unsigned int UNUSED_2                                                     :  1;
    } gfx09_0;
    struct {
        unsigned int MAX_POWER_DELTA                                              : 14;
        unsigned int UNUSED_0                                                     :  2;
        unsigned int SHORT_TERM_INTERVAL_SIZE                                     : 10;
        unsigned int UNUSED_1                                                     :  1;
        unsigned int LONG_TERM_INTERVAL_RATIO                                     :  4;
        unsigned int UNUSED_2                                                     :  1;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_DIDT_DROOP_CTRL {
    struct {
        unsigned int DIDT_DROOP_LEVEL_EN                                          :  1;
        unsigned int DIDT_DROOP_THRESHOLD                                         : 14;
        unsigned int DIDT_DROOP_LEVEL_INDEX                                       :  4;
        unsigned int DIDT_LEVEL_SEL                                               :  1;
        unsigned int                                                              : 11;
        unsigned int DIDT_DROOP_LEVEL_OVERFLOW                                    :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_DIDT_DROOP_CTRL1 {
    struct {
        unsigned int DIDT_DROOP_LEVEL_RELEASE_EN                                  :  1;
        unsigned int DIDT_DROOP_DELTA_THRESHOLD                                   : 14;
        unsigned int                                                              : 17;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_DIDT_WEIGHT {
    struct {
        unsigned int SQ_WEIGHT                                                    :  8;
        unsigned int DB_WEIGHT                                                    :  8;
        unsigned int TD_WEIGHT                                                    :  8;
        unsigned int TCP_WEIGHT                                                   :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_DIDT_WEIGHT_1 {
    struct {
        unsigned int DBR_WEIGHT                                                   :  8;
        unsigned int                                                              : 24;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_EDC_CTRL {
    struct {
        unsigned int EDC_EN                                                       :  1;
        unsigned int EDC_SW_RST                                                   :  1;
        unsigned int EDC_CLK_EN_OVERRIDE                                          :  1;
        unsigned int EDC_FORCE_STALL                                              :  1;
        unsigned int EDC_TRIGGER_THROTTLE_LOWBIT                                  :  5;
        unsigned int EDC_ALLOW_WRITE_PWRDELTA                                     :  1;
        unsigned int                                                              : 22;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 11;
        unsigned int GC_EDC_ONLY_MODE                                             :  1;
        unsigned int EDC_THROTTLE_PATTERN_BIT_NUMS                                :  4;
        unsigned int PCC_THROTTLE_PATTERN_BIT_NUMS                                :  4;
        unsigned int RELEASE_STEP_INTERVAL                                        : 10;
        unsigned int                                                              :  2;
    } most;
    struct {
        unsigned int                                                              : 10;
        unsigned int UNUSED_0                                                     : 22;
    } gfx09_0;
    struct {
        unsigned int                                                              : 10;
        unsigned int EDC_THROTTLE_PATTERN_BIT_NUMS                                :  4;
        unsigned int EDC_LEVEL_SEL                                                :  1;
        unsigned int                                                              : 17;
    } gfx10;
    struct {
        unsigned int                                                              : 10;
        unsigned int EDC_PSM_SELECTION                                            :  1;
        unsigned int                                                              : 21;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_EDC_DROOP_CTRL {
    struct {
        unsigned int EDC_DROOP_LEVEL_EN                                           :  1;
        unsigned int EDC_DROOP_THRESHOLD                                          : 14;
        unsigned int EDC_DROOP_LEVEL_INDEX                                        :  5;
        unsigned int AVG_PSM_SEL                                                  :  1;
        unsigned int EDC_LEVEL_SEL                                                :  1;
        unsigned int                                                              : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_EDC_OVERFLOW {
    struct {
        unsigned int EDC_ROLLING_POWER_DELTA_OVERFLOW                             :  1;
        unsigned int EDC_THROTTLE_LEVEL_OVERFLOW_COUNTER                          : 16;
        unsigned int                                                              : 15;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 18;
        unsigned int PSM_COUNTER                                                  : 14;
    } most;
    struct {
        unsigned int                                                              : 17;
        unsigned int EDC_DROOP_LEVEL_OVERFLOW                                     :  1;
        unsigned int                                                              : 14;
    } gfx09;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_EDC_PERF_COUNTER {
    struct {
        unsigned int EDC_PERF_COUNTER                                             : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_EDC_ROLLING_POWER_DELTA {
    struct {
        unsigned int EDC_ROLLING_POWER_DELTA                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_EDC_STATUS {
    struct {
        unsigned int EDC_THROTTLE_LEVEL                                           :  3;
        unsigned int                                                              : 29;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 26;
        unsigned int THROTTLE_PATTERN_INDEX                                       :  6;
    } most;
    struct {
        unsigned int                                                              :  3;
        unsigned int EDC_ROLLING_DROOP_DELTA                                      : 23;
        unsigned int                                                              :  6;
    } gfx09;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_EDC_THRESHOLD {
    struct {
        unsigned int EDC_THRESHOLD                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_IH_COOKIE_0_PTR {
    struct {
        unsigned int ADDR                                                         : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_THROTTLE_CTRL {
    struct {
        unsigned int THROTTLE_CTRL_SW_RST                                         :  1;
        unsigned int GC_EDC_STALL_EN                                              :  1;
        unsigned int PWRBRK_STALL_EN                                              :  1;
        unsigned int PWRBRK_POLARITY_CNTL                                         :  1;
        unsigned int PCC_STALL_EN                                                 :  1;
        unsigned int PATTERN_MODE                                                 :  1;
        unsigned int GC_EDC_ONLY_MODE                                             :  1;
        unsigned int GC_EDC_OVERRIDE                                              :  1;
        unsigned int PCC_OVERRIDE                                                 :  1;
        unsigned int PWRBRK_OVERRIDE                                              :  1;
        unsigned int GC_EDC_PERF_COUNTER_EN                                       :  1;
        unsigned int PCC_PERF_COUNTER_EN                                          :  1;
        unsigned int PWRBRK_PERF_COUNTER_EN                                       :  1;
        unsigned int RELEASE_STEP_INTERVAL                                        : 10;
        unsigned int FIXED_PATTERN_PERF_COUNTER_EN                                :  1;
        unsigned int                                                              :  5;
        unsigned int LUT_HW_UPDATE                                                :  1;
        unsigned int                                                              :  2;
    } gfx10;
    struct {
        unsigned int                                                              : 30;
        unsigned int THROTTLE_CTRL_CLK_EN_OVERRIDE                                :  1;
        unsigned int                                                              :  1;
    } gfx101Plus;
    struct {
        unsigned int THROTTLE_CTRL_SW_RST                                         :  1;
        unsigned int GC_EDC_STALL_EN                                              :  1;
        unsigned int PCC_STALL_EN                                                 :  1;
        unsigned int PATTERN_MODE                                                 :  1;
        unsigned int GC_EDC_OVERRIDE                                              :  3;
        unsigned int PCC_OVERRIDE                                                 :  1;
        unsigned int GC_EDC_PERF_COUNTER_EN                                       :  1;
        unsigned int PCC_PERF_COUNTER_EN                                          :  1;
        unsigned int PCC_THROT_INCR_STEP_INTERVAL                                 : 10;
        unsigned int PCC_FIXED_PATTERN_MIN                                        :  5;
        unsigned int PCC_FIXED_PATTERN_MAX                                        :  5;
        unsigned int INST_THROT_INCR                                              :  1;
        unsigned int INST_THROT_DECR                                              :  1;
    } vg12;
    struct {
        unsigned int THROTTLE_CTRL_SW_RST                                         :  1;
        unsigned int GC_EDC_STALL_EN                                              :  1;
        unsigned int PCC_STALL_EN                                                 :  1;
        unsigned int PATTERN_MODE                                                 :  1;
        unsigned int GC_EDC_OVERRIDE                                              :  3;
        unsigned int PCC_OVERRIDE                                                 :  1;
        unsigned int GC_EDC_PERF_COUNTER_EN                                       :  1;
        unsigned int PCC_PERF_COUNTER_EN                                          :  1;
        unsigned int PCC_THROT_INCR_STEP_INTERVAL                                 : 10;
        unsigned int PCC_FIXED_PATTERN_MIN                                        :  5;
        unsigned int PCC_FIXED_PATTERN_MAX                                        :  5;
        unsigned int INST_THROT_INCR                                              :  1;
        unsigned int INST_THROT_DECR                                              :  1;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_THROTTLE_CTRL1 {
    struct {
        unsigned int PCC_FP_PROGRAM_STEP_EN                                       :  1;
        unsigned int PCC_PROGRAM_MIN_STEP                                         :  4;
        unsigned int PCC_PROGRAM_MAX_STEP                                         :  5;
        unsigned int PCC_PROGRAM_UPWARDS_STEP_SIZE                                :  3;
        unsigned int PWRBRK_FP_PROGRAM_STEP_EN                                    :  1;
        unsigned int PWRBRK_PROGRAM_MIN_STEP                                      :  4;
        unsigned int PWRBRK_PROGRAM_MAX_STEP                                      :  5;
        unsigned int PWRBRK_PROGRAM_UPWARDS_STEP_SIZE                             :  3;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_THROTTLE_STATUS {
    struct {
        unsigned int FSM_STATE                                                    :  4;
        unsigned int PATTERN_INDEX                                                :  6;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_USER_PRIM_CONFIG {
    struct {
        unsigned int                                                              : 16;
        unsigned int INACTIVE_IA                                                  :  2;
        unsigned int                                                              :  6;
        unsigned int INACTIVE_VGT_PA                                              :  4;
        unsigned int                                                              :  4;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_USER_RB_BACKEND_DISABLE {
    struct {
        unsigned int                                                              : 16;
        unsigned int BACKEND_DISABLE                                              :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_USER_RB_REDUNDANCY {
    struct {
        unsigned int                                                              :  8;
        unsigned int FAILED_RB0                                                   :  4;
        unsigned int EN_REDUNDANCY0                                               :  1;
        unsigned int                                                              :  3;
        unsigned int FAILED_RB1                                                   :  4;
        unsigned int EN_REDUNDANCY1                                               :  1;
        unsigned int                                                              : 11;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_USER_RMI_REDUNDANCY {
    struct {
        unsigned int                                                              :  1;
        unsigned int REPAIR_EN_IN_0                                               :  1;
        unsigned int REPAIR_EN_IN_1                                               :  1;
        unsigned int REPAIR_RMI_OVERRIDE                                          :  1;
        unsigned int REPAIR_ID_SWAP                                               :  1;
        unsigned int                                                              : 27;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_USER_SHADER_ARRAY_CONFIG {
    struct {
        unsigned int                                                              : 16;
        unsigned int INACTIVE_CUS                                                 : 16;
    } gfx09;
    struct {
        unsigned int                                                              : 16;
        unsigned int INACTIVE_WGPS                                                : 16;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_USER_SHADER_RATE_CONFIG {
    struct {
        unsigned int                                                              :  1;
        unsigned int DPFP_RATE                                                    :  2;
        unsigned int SQC_BALANCE_DISABLE                                          :  1;
        unsigned int                                                              : 28;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  4;
        unsigned int HALF_LDS                                                     :  1;
        unsigned int                                                              : 27;
    } gfx09;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GDS_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GDS_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GDS_PERFCOUNTER0_SELECT {
    struct {
        unsigned int                                                              : 20;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  8;
    } bits, bitfields;
    struct {
        unsigned int PERFCOUNTER_SELECT                                           : 10;
        unsigned int PERFCOUNTER_SELECT1                                          : 10;
        unsigned int                                                              : 12;
    } gfx09_0;
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } gfx09_1xPlus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GDS_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERFCOUNTER_SELECT2                                          : 10;
        unsigned int PERFCOUNTER_SELECT3                                          : 10;
        unsigned int                                                              : 12;
    } gfx09_0;
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } gfx09_1xPlus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GDS_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GDS_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GDS_PERFCOUNTER1_SELECT {
    struct {
        unsigned int                                                              : 20;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  8;
    } bits, bitfields;
    struct {
        unsigned int PERFCOUNTER_SELECT                                           : 10;
        unsigned int PERFCOUNTER_SELECT1                                          : 10;
        unsigned int                                                              : 12;
    } gfx09_0;
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } gfx09_1xPlus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GDS_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GDS_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GDS_PERFCOUNTER2_SELECT {
    struct {
        unsigned int                                                              : 20;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  8;
    } bits, bitfields;
    struct {
        unsigned int PERFCOUNTER_SELECT                                           : 10;
        unsigned int PERFCOUNTER_SELECT1                                          : 10;
        unsigned int                                                              : 12;
    } gfx09_0;
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } gfx09_1xPlus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GDS_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GDS_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GDS_PERFCOUNTER3_SELECT {
    struct {
        unsigned int                                                              : 20;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  8;
    } bits, bitfields;
    struct {
        unsigned int PERFCOUNTER_SELECT                                           : 10;
        unsigned int PERFCOUNTER_SELECT1                                          : 10;
        unsigned int                                                              : 12;
    } gfx09_0;
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } gfx09_1xPlus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_CNTL {
    struct {
        unsigned int PRIM_GRP_SIZE                                                :  9;
        unsigned int VERT_GRP_SIZE                                                :  9;
        unsigned int BREAK_WAVE_AT_EOI                                            :  1;
        unsigned int PACKET_TO_ONE_PA                                             :  1;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_DMA_FIRST_INDEX {
    struct {
        unsigned int FIRST_INDEX                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_FAST_CLKS {
    struct {
        unsigned int HYSTERESIS                                                   : 30;
        unsigned int LOCK                                                         :  1;
        unsigned int FORCE_FAST_CLK                                               :  1;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_INDX_OFFSET {
    struct {
        unsigned int INDX_OFFSET                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_MAX_OUTPUT_PER_SUBGROUP {
    struct {
        unsigned int MAX_VERTS_PER_SUBGROUP                                       : 10;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_MAX_VTX_INDX {
    struct {
        unsigned int MAX_INDX                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_MIN_VTX_INDX {
    struct {
        unsigned int MIN_INDX                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_MULTI_PRIM_IB_RESET_EN {
    struct {
        unsigned int RESET_EN                                                     :  1;
        unsigned int MATCH_ALL_BITS                                               :  1;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_NGG_SUBGRP_CNTL {
    struct {
        unsigned int PRIM_AMP_FACTOR                                              :  9;
        unsigned int THDS_PER_SUBGRP                                              :  9;
        unsigned int                                                              : 14;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PC_ALLOC {
    struct {
        unsigned int OVERSUB_EN                                                   :  1;
        unsigned int NUM_PC_LINES                                                 : 10;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PC_CNTL {
    struct {
        unsigned int PC_SIZE                                                      : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 16;
        unsigned int EN_GEN_0_1_LATE_ALLOC                                        :  1;
        unsigned int                                                              : 15;
    } gfx10Core;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL0                                                    : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE0                                                   :  4;
        unsigned int PERF_MODE1                                                   :  4;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL0                                                    : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE0                                                   :  4;
        unsigned int PERF_MODE1                                                   :  4;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER1_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL0                                                    : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE0                                                   :  4;
        unsigned int PERF_MODE1                                                   :  4;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER2_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL0                                                    : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE0                                                   :  4;
        unsigned int PERF_MODE1                                                   :  4;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER3_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER4_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER4_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER4_SELECT {
    struct {
        unsigned int PERF_SEL0                                                    : 10;
        unsigned int                                                              : 18;
        unsigned int PERF_MODE                                                    :  4;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER5_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER5_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER5_SELECT {
    struct {
        unsigned int PERF_SEL0                                                    : 10;
        unsigned int                                                              : 18;
        unsigned int PERF_MODE                                                    :  4;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER6_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER6_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER6_SELECT {
    struct {
        unsigned int PERF_SEL0                                                    : 10;
        unsigned int                                                              : 18;
        unsigned int PERF_MODE                                                    :  4;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER7_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER7_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER7_SELECT {
    struct {
        unsigned int PERF_SEL0                                                    : 10;
        unsigned int                                                              : 18;
        unsigned int PERF_MODE                                                    :  4;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER8_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER8_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER8_SELECT {
    struct {
        unsigned int PERF_SEL0                                                    : 10;
        unsigned int                                                              : 18;
        unsigned int PERF_MODE                                                    :  4;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER9_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER9_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER9_SELECT {
    struct {
        unsigned int PERF_SEL0                                                    : 10;
        unsigned int                                                              : 18;
        unsigned int PERF_MODE                                                    :  4;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER10_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER10_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER10_SELECT {
    struct {
        unsigned int PERF_SEL0                                                    : 10;
        unsigned int                                                              : 18;
        unsigned int PERF_MODE                                                    :  4;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER11_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER11_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PERFCOUNTER11_SELECT {
    struct {
        unsigned int PERF_SEL0                                                    : 10;
        unsigned int                                                              : 18;
        unsigned int PERF_MODE                                                    :  4;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PRIV_CONTROL {
    struct {
        unsigned int DISCARD_LEGACY                                               :  1;
        unsigned int CLAMP_PRIMGRP_SIZE                                           :  9;
        unsigned int RESET_ON_PIPELINE_CHANGE                                     :  1;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_STATUS {
    struct {
        unsigned int PERFCOUNTER_STATUS                                           :  1;
        unsigned int THREAD_TRACE_STATUS                                          :  1;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_STEREO_CNTL {
    struct {
        unsigned int RT_SLICE                                                     :  3;
        unsigned int VIEWPORT                                                     :  4;
        unsigned int                                                              :  1;
        unsigned int EN_STEREO                                                    :  1;
        unsigned int                                                              : 23;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_USER_VGPR1 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_USER_VGPR2 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_USER_VGPR3 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_USER_VGPR_EN {
    struct {
        unsigned int EN_USER_VGPR1                                                :  1;
        unsigned int EN_USER_VGPR2                                                :  1;
        unsigned int EN_USER_VGPR3                                                :  1;
        unsigned int                                                              : 29;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GFX_COPY_STATE {
    struct {
        unsigned int SRC_STATE_ID                                                 :  3;
        unsigned int                                                              : 29;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GFX_GAP_PWROK {
    struct {
        unsigned int gfx_gap_pwrok                                                :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GFX_PIPE_CONTROL {
    struct {
        unsigned int HYSTERESIS_CNT                                               : 13;
        unsigned int RESERVED                                                     :  3;
        unsigned int CONTEXT_SUSPEND_EN                                           :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GFX_PIPE_PRIORITY {
    struct {
        unsigned int HP_PIPE_SELECT                                               :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1A_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1A_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1A_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1A_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1A_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1A_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1A_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1A_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1A_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1A_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1A_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1A_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1A_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1C_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1C_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1C_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1C_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1C_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1C_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1C_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1C_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1C_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1C_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1C_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1C_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1C_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1_ARB_CTRL {
    struct {
        unsigned int NUM_MEM_PIPES                                                :  3;
        unsigned int                                                              : 29;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1_ARB_STATUS {
    struct {
        unsigned int REQ_ARB_BUSY                                                 :  1;
        unsigned int RET_ARB_BUSY                                                 :  1;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1_DRAM_BURST_CTRL {
    struct {
        unsigned int MAX_DRAM_BURST                                               :  3;
        unsigned int BURST_DISABLE                                                :  1;
        unsigned int                                                              : 28;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  4;
        unsigned int GATHER_64B_MEMORY_BURST_DISABLE                              :  1;
        unsigned int GATHER_64B_IO_BURST_DISABLE                                  :  1;
        unsigned int                                                              : 26;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1_DRAM_BURST_MASK {
    struct {
        unsigned int DRAM_BURST_ADDR_MASK                                         :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1_PIPE_STEER {
    struct {
        unsigned int PIPE0                                                        :  2;
        unsigned int PIPE1                                                        :  2;
        unsigned int PIPE2                                                        :  2;
        unsigned int PIPE3                                                        :  2;
        unsigned int                                                              : 24;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  8;
        unsigned int PIPE4                                                        :  2;
        unsigned int PIPE5                                                        :  2;
        unsigned int PIPE6                                                        :  2;
        unsigned int PIPE7                                                        :  2;
        unsigned int PIPE8                                                        :  2;
        unsigned int PIPE9                                                        :  2;
        unsigned int PIPE10                                                       :  2;
        unsigned int PIPE11                                                       :  2;
        unsigned int PIPE12                                                       :  2;
        unsigned int PIPE13                                                       :  2;
        unsigned int PIPE14                                                       :  2;
        unsigned int PIPE15                                                       :  2;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2A_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2A_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2A_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2A_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2A_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2A_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2A_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2A_PERFCOUNTER1_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2A_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2A_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2A_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2A_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2A_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2A_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2C_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2C_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2C_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2C_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2C_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2C_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2C_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2C_PERFCOUNTER1_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2C_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2C_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2C_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2C_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2C_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2C_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2_PIPE_STEER_0 {
    struct {
        unsigned int PIPE_0_TO_CHAN_IN_Q0                                         :  3;
        unsigned int                                                              :  1;
        unsigned int PIPE_1_TO_CHAN_IN_Q0                                         :  3;
        unsigned int                                                              :  1;
        unsigned int PIPE_2_TO_CHAN_IN_Q0                                         :  3;
        unsigned int                                                              :  1;
        unsigned int PIPE_3_TO_CHAN_IN_Q0                                         :  3;
        unsigned int                                                              :  1;
        unsigned int PIPE_0_TO_CHAN_IN_Q1                                         :  3;
        unsigned int                                                              :  1;
        unsigned int PIPE_1_TO_CHAN_IN_Q1                                         :  3;
        unsigned int                                                              :  1;
        unsigned int PIPE_2_TO_CHAN_IN_Q1                                         :  3;
        unsigned int                                                              :  1;
        unsigned int PIPE_3_TO_CHAN_IN_Q1                                         :  3;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2_PIPE_STEER_1 {
    struct {
        unsigned int PIPE_0_TO_CHAN_IN_Q2                                         :  3;
        unsigned int                                                              :  1;
        unsigned int PIPE_1_TO_CHAN_IN_Q2                                         :  3;
        unsigned int                                                              :  1;
        unsigned int PIPE_2_TO_CHAN_IN_Q2                                         :  3;
        unsigned int                                                              :  1;
        unsigned int PIPE_3_TO_CHAN_IN_Q2                                         :  3;
        unsigned int                                                              :  1;
        unsigned int PIPE_0_TO_CHAN_IN_Q3                                         :  3;
        unsigned int                                                              :  1;
        unsigned int PIPE_1_TO_CHAN_IN_Q3                                         :  3;
        unsigned int                                                              :  1;
        unsigned int PIPE_2_TO_CHAN_IN_Q3                                         :  3;
        unsigned int                                                              :  1;
        unsigned int PIPE_3_TO_CHAN_IN_Q3                                         :  3;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_CAM_DATA {
    struct {
        unsigned int CAM_ADDR                                                     : 16;
        unsigned int CAM_REMAPADDR                                                : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_CAM_DATA_UPPER {
    struct {
        unsigned int CAM_ADDR                                                     :  2;
        unsigned int                                                              : 14;
        unsigned int CAM_REMAPADDR                                                :  2;
        unsigned int                                                              : 14;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_CAM_INDEX {
    struct {
        unsigned int CAM_INDEX                                                    :  3;
        unsigned int                                                              : 29;
    } gfx09;
    struct {
        unsigned int CAM_INDEX                                                    :  4;
        unsigned int                                                              : 28;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_CGTT_CLK_CNTL {
    struct {
        unsigned int ON_DELAY                                                     :  4;
        unsigned int OFF_HYSTERESIS                                               :  8;
        unsigned int                                                              :  4;
        unsigned int SOFT_STALL_OVERRIDE7                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE6                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE5                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE4                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE3                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE2                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE1                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE0                                         :  1;
        unsigned int                                                              :  6;
        unsigned int SOFT_OVERRIDE_DYN                                            :  1;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_CHICKEN_BITS {
    struct {
        unsigned int DISABLE_CP_VMID_RESET_REQ                                    :  1;
        unsigned int                                                              : 31;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_CHIP_REVISION {
    struct {
        unsigned int CHIP_REVISION                                                :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_CNTL {
    struct {
        unsigned int READ_TIMEOUT                                                 :  8;
        unsigned int                                                              : 23;
        unsigned int REPORT_LAST_RDERR                                            :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_DEBUG_DATA {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_DSM_BYPASS {
    struct {
        unsigned int BYPASS_BITS                                                  :  2;
        unsigned int BYPASS_EN                                                    :  1;
        unsigned int                                                              : 29;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_FENCE_RANGE0 {
    struct {
        unsigned int START                                                        : 16;
        unsigned int END                                                          : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_FENCE_RANGE1 {
    struct {
        unsigned int START                                                        : 16;
        unsigned int END                                                          : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_GFX_CLKEN_CNTL {
    struct {
        unsigned int PREFIX_DELAY_CNT                                             :  4;
        unsigned int                                                              :  4;
        unsigned int POST_DELAY_CNT                                               :  5;
        unsigned int                                                              : 19;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_GFX_CNTL {
    struct {
        unsigned int PIPEID                                                       :  2;
        unsigned int MEID                                                         :  2;
        unsigned int VMID                                                         :  4;
        unsigned int QUEUEID                                                      :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_GFX_CNTL_SR_DATA {
    struct {
        unsigned int PIPEID                                                       :  2;
        unsigned int MEID                                                         :  2;
        unsigned int VMID                                                         :  4;
        unsigned int QUEUEID                                                      :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_GFX_CNTL_SR_SELECT {
    struct {
        unsigned int INDEX                                                        :  3;
        unsigned int                                                              : 29;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_GFX_INDEX {
    struct {
        unsigned int INSTANCE_INDEX                                               :  8;
        unsigned int                                                              :  8;
        unsigned int SE_INDEX                                                     :  8;
        unsigned int                                                              :  6;
        unsigned int INSTANCE_BROADCAST_WRITES                                    :  1;
        unsigned int SE_BROADCAST_WRITES                                          :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  8;
        unsigned int SH_INDEX                                                     :  8;
        unsigned int                                                              : 13;
        unsigned int SH_BROADCAST_WRITES                                          :  1;
        unsigned int                                                              :  2;
    } gfx09;
    struct {
        unsigned int                                                              :  8;
        unsigned int SA_INDEX                                                     :  8;
        unsigned int                                                              : 13;
        unsigned int SA_BROADCAST_WRITES                                          :  1;
        unsigned int                                                              :  2;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_GFX_INDEX_SR_DATA {
    struct {
        unsigned int INSTANCE_INDEX                                               :  8;
        unsigned int                                                              :  8;
        unsigned int SE_INDEX                                                     :  8;
        unsigned int                                                              :  6;
        unsigned int INSTANCE_BROADCAST_WRITES                                    :  1;
        unsigned int SE_BROADCAST_WRITES                                          :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  8;
        unsigned int SH_INDEX                                                     :  8;
        unsigned int                                                              : 13;
        unsigned int SH_BROADCAST_WRITES                                          :  1;
        unsigned int                                                              :  2;
    } gfx09;
    struct {
        unsigned int                                                              :  8;
        unsigned int SA_INDEX                                                     :  8;
        unsigned int                                                              : 13;
        unsigned int SA_BROADCAST_WRITES                                          :  1;
        unsigned int                                                              :  2;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_GFX_INDEX_SR_SELECT {
    struct {
        unsigned int INDEX                                                        :  3;
        unsigned int                                                              : 29;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_HYP_CAM_DATA {
    struct {
        unsigned int CAM_ADDR                                                     : 16;
        unsigned int CAM_REMAPADDR                                                : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_HYP_CAM_DATA_UPPER {
    struct {
        unsigned int CAM_ADDR                                                     :  2;
        unsigned int                                                              : 14;
        unsigned int CAM_REMAPADDR                                                :  2;
        unsigned int                                                              : 14;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_HYP_CAM_INDEX {
    struct {
        unsigned int CAM_INDEX                                                    :  3;
        unsigned int                                                              : 29;
    } gfx09;
    struct {
        unsigned int CAM_INDEX                                                    :  4;
        unsigned int                                                              : 28;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_IH_CREDIT {
    struct {
        unsigned int CREDIT_VALUE                                                 :  2;
        unsigned int                                                              : 14;
        unsigned int IH_CLIENT_ID                                                 :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_INT_CNTL {
    struct {
        unsigned int RDERR_INT_ENABLE                                             :  1;
        unsigned int                                                              : 18;
        unsigned int GUI_IDLE_INT_ENABLE                                          :  1;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_IOV_ERROR {
    struct {
        unsigned int                                                              :  2;
        unsigned int IOV_ADDR                                                     : 18;
        unsigned int IOV_VFID                                                     :  6;
        unsigned int IOV_VF                                                       :  1;
        unsigned int IOV_OP                                                       :  1;
        unsigned int                                                              :  3;
        unsigned int IOV_ERROR                                                    :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_IOV_READ_ERROR {
    struct {
        unsigned int                                                              :  2;
        unsigned int IOV_ADDR                                                     : 18;
        unsigned int IOV_VFID                                                     :  6;
        unsigned int IOV_VF                                                       :  1;
        unsigned int IOV_OP                                                       :  1;
        unsigned int                                                              :  3;
        unsigned int IOV_ERROR                                                    :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_NOWHERE {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  6;
        unsigned int                                                              :  4;
        unsigned int DB_CLEAN_USER_DEFINED_MASK                                   :  1;
        unsigned int CB_CLEAN_USER_DEFINED_MASK                                   :  1;
        unsigned int                                                              :  1;
        unsigned int TA_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int SX_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int                                                              :  1;
        unsigned int SPI_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int SC_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int PA_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int GRBM_BUSY_USER_DEFINED_MASK                                  :  1;
        unsigned int DB_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int CB_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int CP_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int                                                              :  1;
        unsigned int GDS_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int BCI_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int RLC_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int                                                              :  2;
        unsigned int UTCL2_BUSY_USER_DEFINED_MASK                                 :  1;
        unsigned int EA_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int RMI_BUSY_USER_DEFINED_MASK                                   :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 12;
        unsigned int VGT_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int                                                              : 10;
        unsigned int IA_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int                                                              :  3;
        unsigned int TC_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int WD_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int                                                              :  3;
    } gfx09;
    struct {
        unsigned int                                                              : 27;
        unsigned int TCP_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int GE_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int                                                              :  3;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_PERFCOUNTER0_SELECT_HI {
    struct {
        unsigned int                                                              :  1;
        unsigned int UTCL1_BUSY_USER_DEFINED_MASK                                 :  1;
        unsigned int GL2CC_BUSY_USER_DEFINED_MASK                                 :  1;
        unsigned int SDMA_BUSY_USER_DEFINED_MASK                                  :  1;
        unsigned int CH_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int PH_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int                                                              :  2;
        unsigned int GL1CC_BUSY_USER_DEFINED_MASK                                 :  1;
        unsigned int                                                              : 23;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  6;
        unsigned int PMM_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int GUS_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int                                                              : 24;
    } gfx10Core;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  6;
        unsigned int                                                              :  4;
        unsigned int DB_CLEAN_USER_DEFINED_MASK                                   :  1;
        unsigned int CB_CLEAN_USER_DEFINED_MASK                                   :  1;
        unsigned int                                                              :  1;
        unsigned int TA_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int SX_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int                                                              :  1;
        unsigned int SPI_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int SC_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int PA_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int GRBM_BUSY_USER_DEFINED_MASK                                  :  1;
        unsigned int DB_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int CB_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int CP_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int                                                              :  1;
        unsigned int GDS_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int BCI_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int RLC_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int                                                              :  2;
        unsigned int UTCL2_BUSY_USER_DEFINED_MASK                                 :  1;
        unsigned int EA_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int RMI_BUSY_USER_DEFINED_MASK                                   :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 12;
        unsigned int VGT_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int                                                              : 10;
        unsigned int IA_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int                                                              :  3;
        unsigned int TC_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int WD_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int                                                              :  3;
    } gfx09;
    struct {
        unsigned int                                                              : 27;
        unsigned int TCP_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int GE_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int                                                              :  3;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_PERFCOUNTER1_SELECT_HI {
    struct {
        unsigned int                                                              :  1;
        unsigned int UTCL1_BUSY_USER_DEFINED_MASK                                 :  1;
        unsigned int GL2CC_BUSY_USER_DEFINED_MASK                                 :  1;
        unsigned int SDMA_BUSY_USER_DEFINED_MASK                                  :  1;
        unsigned int CH_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int PH_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int                                                              :  2;
        unsigned int GL1CC_BUSY_USER_DEFINED_MASK                                 :  1;
        unsigned int                                                              : 23;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  6;
        unsigned int PMM_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int GUS_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int                                                              : 24;
    } gfx10Core;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_PM_CNTL {
    struct {
        unsigned int PM_READY                                                     :  1;
        unsigned int                                                              : 15;
        unsigned int PM_START                                                     :  1;
        unsigned int                                                              : 15;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_PWR_CNTL {
    struct {
        unsigned int ALL_REQ_TYPE                                                 :  2;
        unsigned int GFX_REQ_TYPE                                                 :  2;
        unsigned int ALL_RSP_TYPE                                                 :  2;
        unsigned int GFX_RSP_TYPE                                                 :  2;
        unsigned int                                                              :  6;
        unsigned int GFX_REQ_EN                                                   :  1;
        unsigned int ALL_REQ_EN                                                   :  1;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_PWR_CNTL2 {
    struct {
        unsigned int                                                              : 16;
        unsigned int PWR_REQUEST_HALT                                             :  1;
        unsigned int                                                              :  3;
        unsigned int PWR_GFX3D_REQUEST_HALT                                       :  1;
        unsigned int                                                              : 11;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_READ_ERROR {
    struct {
        unsigned int                                                              :  2;
        unsigned int READ_ADDRESS                                                 : 16;
        unsigned int                                                              :  2;
        unsigned int READ_PIPEID                                                  :  2;
        unsigned int READ_MEID                                                    :  2;
        unsigned int                                                              :  7;
        unsigned int READ_ERROR                                                   :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_READ_ERROR2 {
    struct {
        unsigned int                                                              : 17;
        unsigned int READ_REQUESTER_RSMU                                          :  1;
        unsigned int READ_REQUESTER_RLC                                           :  1;
        unsigned int READ_REQUESTER_GDS_DMA                                       :  1;
        unsigned int READ_REQUESTER_ME0PIPE0_CF                                   :  1;
        unsigned int READ_REQUESTER_ME0PIPE0_PF                                   :  1;
        unsigned int READ_REQUESTER_ME0PIPE1_CF                                   :  1;
        unsigned int READ_REQUESTER_ME0PIPE1_PF                                   :  1;
        unsigned int READ_REQUESTER_ME1PIPE0                                      :  1;
        unsigned int READ_REQUESTER_ME1PIPE1                                      :  1;
        unsigned int READ_REQUESTER_ME1PIPE2                                      :  1;
        unsigned int READ_REQUESTER_ME1PIPE3                                      :  1;
        unsigned int READ_REQUESTER_ME2PIPE0                                      :  1;
        unsigned int READ_REQUESTER_ME2PIPE1                                      :  1;
        unsigned int READ_REQUESTER_ME2PIPE2                                      :  1;
        unsigned int READ_REQUESTER_ME2PIPE3                                      :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 16;
        unsigned int READ_REQUESTER_CPF                                           :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_SCRATCH_REG0 {
    struct {
        unsigned int SCRATCH_REG0                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_SCRATCH_REG1 {
    struct {
        unsigned int SCRATCH_REG1                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_SCRATCH_REG2 {
    struct {
        unsigned int SCRATCH_REG2                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_SCRATCH_REG3 {
    struct {
        unsigned int SCRATCH_REG3                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_SCRATCH_REG4 {
    struct {
        unsigned int SCRATCH_REG4                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_SCRATCH_REG5 {
    struct {
        unsigned int SCRATCH_REG5                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_SCRATCH_REG6 {
    struct {
        unsigned int SCRATCH_REG6                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_SCRATCH_REG7 {
    struct {
        unsigned int SCRATCH_REG7                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_SE0_PERFCOUNTER_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_SE0_PERFCOUNTER_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_SE0_PERFCOUNTER_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  6;
        unsigned int                                                              :  4;
        unsigned int DB_CLEAN_USER_DEFINED_MASK                                   :  1;
        unsigned int CB_CLEAN_USER_DEFINED_MASK                                   :  1;
        unsigned int TA_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int SX_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int                                                              :  1;
        unsigned int SPI_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int SC_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int DB_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int CB_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int                                                              :  1;
        unsigned int PA_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int BCI_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int RMI_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int                                                              :  9;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 19;
        unsigned int VGT_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int                                                              : 12;
    } gfx09;
    struct {
        unsigned int                                                              : 23;
        unsigned int UTCL1_BUSY_USER_DEFINED_MASK                                 :  1;
        unsigned int TCP_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int GL1CC_BUSY_USER_DEFINED_MASK                                 :  1;
        unsigned int                                                              :  6;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_SE1_PERFCOUNTER_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_SE1_PERFCOUNTER_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_SE1_PERFCOUNTER_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  6;
        unsigned int                                                              :  4;
        unsigned int DB_CLEAN_USER_DEFINED_MASK                                   :  1;
        unsigned int CB_CLEAN_USER_DEFINED_MASK                                   :  1;
        unsigned int TA_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int SX_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int                                                              :  1;
        unsigned int SPI_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int SC_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int DB_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int CB_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int                                                              :  1;
        unsigned int PA_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int BCI_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int RMI_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int                                                              :  9;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 19;
        unsigned int VGT_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int                                                              : 12;
    } gfx09;
    struct {
        unsigned int                                                              : 23;
        unsigned int UTCL1_BUSY_USER_DEFINED_MASK                                 :  1;
        unsigned int TCP_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int GL1CC_BUSY_USER_DEFINED_MASK                                 :  1;
        unsigned int                                                              :  6;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_SE2_PERFCOUNTER_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_SE2_PERFCOUNTER_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_SE2_PERFCOUNTER_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  6;
        unsigned int                                                              :  4;
        unsigned int DB_CLEAN_USER_DEFINED_MASK                                   :  1;
        unsigned int CB_CLEAN_USER_DEFINED_MASK                                   :  1;
        unsigned int TA_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int SX_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int                                                              :  1;
        unsigned int SPI_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int SC_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int DB_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int CB_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int                                                              :  1;
        unsigned int PA_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int BCI_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int RMI_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int                                                              :  9;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 19;
        unsigned int VGT_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int                                                              : 12;
    } gfx09;
    struct {
        unsigned int                                                              : 23;
        unsigned int UTCL1_BUSY_USER_DEFINED_MASK                                 :  1;
        unsigned int TCP_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int GL1CC_BUSY_USER_DEFINED_MASK                                 :  1;
        unsigned int                                                              :  6;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_SE3_PERFCOUNTER_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_SE3_PERFCOUNTER_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_SE3_PERFCOUNTER_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  6;
        unsigned int                                                              :  4;
        unsigned int DB_CLEAN_USER_DEFINED_MASK                                   :  1;
        unsigned int CB_CLEAN_USER_DEFINED_MASK                                   :  1;
        unsigned int TA_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int SX_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int                                                              :  1;
        unsigned int SPI_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int SC_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int DB_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int CB_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int                                                              :  1;
        unsigned int PA_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int BCI_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int RMI_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int                                                              :  9;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 19;
        unsigned int VGT_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int                                                              : 12;
    } gfx09;
    struct {
        unsigned int                                                              : 23;
        unsigned int UTCL1_BUSY_USER_DEFINED_MASK                                 :  1;
        unsigned int TCP_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int GL1CC_BUSY_USER_DEFINED_MASK                                 :  1;
        unsigned int                                                              :  6;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_SKEW_CNTL {
    struct {
        unsigned int SKEW_TOP_THRESHOLD                                           :  6;
        unsigned int SKEW_COUNT                                                   :  6;
        unsigned int                                                              : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_SOFT_RESET {
    struct {
        unsigned int SOFT_RESET_CP                                                :  1;
        unsigned int                                                              :  1;
        unsigned int SOFT_RESET_RLC                                               :  1;
        unsigned int                                                              : 13;
        unsigned int SOFT_RESET_GFX                                               :  1;
        unsigned int SOFT_RESET_CPF                                               :  1;
        unsigned int SOFT_RESET_CPC                                               :  1;
        unsigned int SOFT_RESET_CPG                                               :  1;
        unsigned int SOFT_RESET_CAC                                               :  1;
        unsigned int SOFT_RESET_CPAXI                                             :  1;
        unsigned int SOFT_RESET_EA                                                :  1;
        unsigned int                                                              :  9;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 23;
        unsigned int SOFT_RESET_SDMA0                                             :  1;
        unsigned int SOFT_RESET_SDMA1                                             :  1;
        unsigned int                                                              :  7;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_STATUS {
    struct {
        unsigned int ME0PIPE0_CMDFIFO_AVAIL                                       :  4;
        unsigned int                                                              :  1;
        unsigned int RSMU_RQ_PENDING                                              :  1;
        unsigned int                                                              :  1;
        unsigned int ME0PIPE0_CF_RQ_PENDING                                       :  1;
        unsigned int ME0PIPE0_PF_RQ_PENDING                                       :  1;
        unsigned int GDS_DMA_RQ_PENDING                                           :  1;
        unsigned int                                                              :  2;
        unsigned int DB_CLEAN                                                     :  1;
        unsigned int CB_CLEAN                                                     :  1;
        unsigned int TA_BUSY                                                      :  1;
        unsigned int GDS_BUSY                                                     :  1;
        unsigned int                                                              :  4;
        unsigned int SX_BUSY                                                      :  1;
        unsigned int                                                              :  1;
        unsigned int SPI_BUSY                                                     :  1;
        unsigned int BCI_BUSY                                                     :  1;
        unsigned int SC_BUSY                                                      :  1;
        unsigned int PA_BUSY                                                      :  1;
        unsigned int DB_BUSY                                                      :  1;
        unsigned int                                                              :  1;
        unsigned int CP_COHERENCY_BUSY                                            :  1;
        unsigned int CP_BUSY                                                      :  1;
        unsigned int CB_BUSY                                                      :  1;
        unsigned int GUI_ACTIVE                                                   :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 16;
        unsigned int WD_BUSY_NO_DMA                                               :  1;
        unsigned int VGT_BUSY                                                     :  1;
        unsigned int IA_BUSY_NO_DMA                                               :  1;
        unsigned int IA_BUSY                                                      :  1;
        unsigned int                                                              :  1;
        unsigned int WD_BUSY                                                      :  1;
        unsigned int                                                              : 10;
    } gfx09;
    struct {
        unsigned int                                                              : 16;
        unsigned int GE_BUSY_NO_DMA                                               :  1;
        unsigned int                                                              :  4;
        unsigned int GE_BUSY                                                      :  1;
        unsigned int                                                              : 10;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_STATUS2 {
    struct {
        unsigned int ME0PIPE1_CMDFIFO_AVAIL                                       :  4;
        unsigned int ME0PIPE1_CF_RQ_PENDING                                       :  1;
        unsigned int ME0PIPE1_PF_RQ_PENDING                                       :  1;
        unsigned int ME1PIPE0_RQ_PENDING                                          :  1;
        unsigned int ME1PIPE1_RQ_PENDING                                          :  1;
        unsigned int ME1PIPE2_RQ_PENDING                                          :  1;
        unsigned int ME1PIPE3_RQ_PENDING                                          :  1;
        unsigned int ME2PIPE0_RQ_PENDING                                          :  1;
        unsigned int ME2PIPE1_RQ_PENDING                                          :  1;
        unsigned int ME2PIPE2_RQ_PENDING                                          :  1;
        unsigned int ME2PIPE3_RQ_PENDING                                          :  1;
        unsigned int RLC_RQ_PENDING                                               :  1;
        unsigned int UTCL2_BUSY                                                   :  1;
        unsigned int EA_BUSY                                                      :  1;
        unsigned int RMI_BUSY                                                     :  1;
        unsigned int UTCL2_RQ_PENDING                                             :  1;
        unsigned int                                                              :  1;
        unsigned int EA_LINK_BUSY                                                 :  1;
        unsigned int                                                              :  7;
        unsigned int CPF_BUSY                                                     :  1;
        unsigned int CPC_BUSY                                                     :  1;
        unsigned int CPG_BUSY                                                     :  1;
        unsigned int CPAXI_BUSY                                                   :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 19;
        unsigned int CPF_RQ_PENDING                                               :  1;
        unsigned int                                                              :  4;
        unsigned int RLC_BUSY                                                     :  1;
        unsigned int                                                              :  7;
    } most;
    struct {
        unsigned int                                                              : 25;
        unsigned int TC_BUSY                                                      :  1;
        unsigned int TCC_CC_RESIDENT                                              :  1;
        unsigned int                                                              :  5;
    } gfx09;
    struct {
        unsigned int                                                              : 21;
        unsigned int SDMA_BUSY                                                    :  1;
        unsigned int SDMA0_RQ_PENDING                                             :  1;
        unsigned int SDMA1_RQ_PENDING                                             :  1;
        unsigned int                                                              :  8;
    } gfx10;
    struct {
        unsigned int                                                              : 25;
        unsigned int TCP_BUSY                                                     :  1;
        unsigned int                                                              :  6;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_STATUS3 {
    struct {
        unsigned int                                                              :  5;
        unsigned int GRBM_RLC_INTR_CREDIT_PENDING                                 :  1;
        unsigned int GRBM_UTCL2_INTR_CREDIT_PENDING                               :  1;
        unsigned int GRBM_CPF_INTR_CREDIT_PENDING                                 :  1;
        unsigned int MESPIPE0_RQ_PENDING                                          :  1;
        unsigned int MESPIPE1_RQ_PENDING                                          :  1;
        unsigned int MESPIPE2_RQ_PENDING                                          :  1;
        unsigned int MESPIPE3_RQ_PENDING                                          :  1;
        unsigned int                                                              :  1;
        unsigned int PH_BUSY                                                      :  1;
        unsigned int CH_BUSY                                                      :  1;
        unsigned int GL2CC_BUSY                                                   :  1;
        unsigned int GL1CC_BUSY                                                   :  1;
        unsigned int                                                              : 13;
        unsigned int UTCL1_BUSY                                                   :  1;
        unsigned int                                                              :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 28;
        unsigned int GUS_LINK_BUSY                                                :  1;
        unsigned int GUS_BUSY                                                     :  1;
        unsigned int                                                              :  1;
        unsigned int PMM_BUSY                                                     :  1;
    } gfx10Core;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_STATUS_SE0 {
    struct {
        unsigned int                                                              :  1;
        unsigned int DB_CLEAN                                                     :  1;
        unsigned int CB_CLEAN                                                     :  1;
        unsigned int                                                              : 18;
        unsigned int RMI_BUSY                                                     :  1;
        unsigned int BCI_BUSY                                                     :  1;
        unsigned int                                                              :  1;
        unsigned int PA_BUSY                                                      :  1;
        unsigned int TA_BUSY                                                      :  1;
        unsigned int SX_BUSY                                                      :  1;
        unsigned int SPI_BUSY                                                     :  1;
        unsigned int                                                              :  1;
        unsigned int SC_BUSY                                                      :  1;
        unsigned int DB_BUSY                                                      :  1;
        unsigned int CB_BUSY                                                      :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 23;
        unsigned int VGT_BUSY                                                     :  1;
        unsigned int                                                              :  8;
    } gfx09;
    struct {
        unsigned int                                                              :  3;
        unsigned int UTCL1_BUSY                                                   :  1;
        unsigned int TCP_BUSY                                                     :  1;
        unsigned int GL1CC_BUSY                                                   :  1;
        unsigned int                                                              : 26;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_STATUS_SE1 {
    struct {
        unsigned int                                                              :  1;
        unsigned int DB_CLEAN                                                     :  1;
        unsigned int CB_CLEAN                                                     :  1;
        unsigned int                                                              : 18;
        unsigned int RMI_BUSY                                                     :  1;
        unsigned int BCI_BUSY                                                     :  1;
        unsigned int                                                              :  1;
        unsigned int PA_BUSY                                                      :  1;
        unsigned int TA_BUSY                                                      :  1;
        unsigned int SX_BUSY                                                      :  1;
        unsigned int SPI_BUSY                                                     :  1;
        unsigned int                                                              :  1;
        unsigned int SC_BUSY                                                      :  1;
        unsigned int DB_BUSY                                                      :  1;
        unsigned int CB_BUSY                                                      :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 23;
        unsigned int VGT_BUSY                                                     :  1;
        unsigned int                                                              :  8;
    } gfx09;
    struct {
        unsigned int                                                              :  3;
        unsigned int UTCL1_BUSY                                                   :  1;
        unsigned int TCP_BUSY                                                     :  1;
        unsigned int GL1CC_BUSY                                                   :  1;
        unsigned int                                                              : 26;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_STATUS_SE2 {
    struct {
        unsigned int                                                              :  1;
        unsigned int DB_CLEAN                                                     :  1;
        unsigned int CB_CLEAN                                                     :  1;
        unsigned int                                                              : 18;
        unsigned int RMI_BUSY                                                     :  1;
        unsigned int BCI_BUSY                                                     :  1;
        unsigned int                                                              :  1;
        unsigned int PA_BUSY                                                      :  1;
        unsigned int TA_BUSY                                                      :  1;
        unsigned int SX_BUSY                                                      :  1;
        unsigned int SPI_BUSY                                                     :  1;
        unsigned int                                                              :  1;
        unsigned int SC_BUSY                                                      :  1;
        unsigned int DB_BUSY                                                      :  1;
        unsigned int CB_BUSY                                                      :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 23;
        unsigned int VGT_BUSY                                                     :  1;
        unsigned int                                                              :  8;
    } gfx09;
    struct {
        unsigned int                                                              :  3;
        unsigned int UTCL1_BUSY                                                   :  1;
        unsigned int TCP_BUSY                                                     :  1;
        unsigned int GL1CC_BUSY                                                   :  1;
        unsigned int                                                              : 26;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_STATUS_SE3 {
    struct {
        unsigned int                                                              :  1;
        unsigned int DB_CLEAN                                                     :  1;
        unsigned int CB_CLEAN                                                     :  1;
        unsigned int                                                              : 18;
        unsigned int RMI_BUSY                                                     :  1;
        unsigned int BCI_BUSY                                                     :  1;
        unsigned int                                                              :  1;
        unsigned int PA_BUSY                                                      :  1;
        unsigned int TA_BUSY                                                      :  1;
        unsigned int SX_BUSY                                                      :  1;
        unsigned int SPI_BUSY                                                     :  1;
        unsigned int                                                              :  1;
        unsigned int SC_BUSY                                                      :  1;
        unsigned int DB_BUSY                                                      :  1;
        unsigned int CB_BUSY                                                      :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 23;
        unsigned int VGT_BUSY                                                     :  1;
        unsigned int                                                              :  8;
    } gfx09;
    struct {
        unsigned int                                                              :  3;
        unsigned int UTCL1_BUSY                                                   :  1;
        unsigned int TCP_BUSY                                                     :  1;
        unsigned int GL1CC_BUSY                                                   :  1;
        unsigned int                                                              : 26;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_TRAP_ADDR {
    struct {
        unsigned int DATA                                                         : 18;
        unsigned int                                                              : 14;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_TRAP_ADDR_MSK {
    struct {
        unsigned int DATA                                                         : 18;
        unsigned int                                                              : 14;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_TRAP_OP {
    struct {
        unsigned int RW                                                           :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_TRAP_WD {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_TRAP_WD_MSK {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_UTCL2_INVAL_RANGE_END {
    struct {
        unsigned int DATA                                                         : 18;
        unsigned int                                                              : 14;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_UTCL2_INVAL_RANGE_START {
    struct {
        unsigned int DATA                                                         : 18;
        unsigned int                                                              : 14;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_WAIT_IDLE_CLOCKS {
    struct {
        unsigned int WAIT_IDLE_CLOCKS                                             :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_WRITE_ERROR {
    struct {
        unsigned int WRITE_REQUESTER_RLC                                          :  1;
        unsigned int WRITE_REQUESTER_RSMU                                         :  1;
        unsigned int WRITE_SSRCID                                                 :  3;
        unsigned int                                                              :  7;
        unsigned int WRITE_VF                                                     :  1;
        unsigned int WRITE_VMID                                                   :  4;
        unsigned int                                                              :  3;
        unsigned int WRITE_PIPEID                                                 :  2;
        unsigned int WRITE_MEID                                                   :  2;
        unsigned int                                                              :  7;
        unsigned int WRITE_ERROR                                                  :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  5;
        unsigned int WRITE_VFID                                                   :  4;
        unsigned int                                                              : 23;
    } gfx09;
    struct {
        unsigned int                                                              :  5;
        unsigned int WRITE_VFID                                                   :  6;
        unsigned int                                                              : 21;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GSL_SOURCE_SELECT {
    struct {
        unsigned int GSL0_READY_SOURCE_SEL                                        :  3;
        unsigned int                                                              :  1;
        unsigned int GSL1_READY_SOURCE_SEL                                        :  3;
        unsigned int                                                              :  1;
        unsigned int GSL2_READY_SOURCE_SEL                                        :  3;
        unsigned int                                                              :  5;
        unsigned int GSL_TIMING_SYNC_SEL                                          :  3;
        unsigned int                                                              : 13;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GUS_PERFCOUNTER0_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GUS_PERFCOUNTER1_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GUS_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GUS_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GUS_PERFCOUNTER2_MODE {
    struct {
        unsigned int COMPARE_MODE0                                                :  2;
        unsigned int COMPARE_MODE1                                                :  2;
        unsigned int COMPARE_MODE2                                                :  2;
        unsigned int COMPARE_MODE3                                                :  2;
        unsigned int COMPARE_VALUE0                                               :  4;
        unsigned int COMPARE_VALUE1                                               :  4;
        unsigned int COMPARE_VALUE2                                               :  4;
        unsigned int COMPARE_VALUE3                                               :  4;
        unsigned int                                                              :  8;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GUS_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GUS_PERFCOUNTER2_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GUS_PERFCOUNTER_HI {
    struct {
        unsigned int COUNTER_HI                                                   : 16;
        unsigned int COMPARE_VALUE                                                : 16;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GUS_PERFCOUNTER_LO {
    struct {
        unsigned int COUNTER_LO                                                   : 32;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GUS_PERFCOUNTER_RSLT_CNTL {
    struct {
        unsigned int PERF_COUNTER_SELECT                                          :  4;
        unsigned int                                                              :  4;
        unsigned int START_TRIGGER                                                :  8;
        unsigned int STOP_TRIGGER                                                 :  8;
        unsigned int ENABLE_ANY                                                   :  1;
        unsigned int CLEAR_ALL                                                    :  1;
        unsigned int STOP_ALL_ON_SATURATE                                         :  1;
        unsigned int                                                              :  5;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union gws_DEBUG_DATA {
    struct {
        unsigned int                                                              :  3;
        unsigned int cur_reso_5to0                                                :  6;
        unsigned int                                                              :  6;
        unsigned int cur_reso_count                                               :  1;
        unsigned int credit_cnt                                                   :  1;
        unsigned int                                                              :  7;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union IA_CNTL_STATUS {
    struct {
        unsigned int IA_BUSY                                                      :  1;
        unsigned int IA_DMA_BUSY                                                  :  1;
        unsigned int IA_DMA_REQ_BUSY                                              :  1;
        unsigned int IA_GRP_BUSY                                                  :  1;
        unsigned int IA_ADC_BUSY                                                  :  1;
        unsigned int                                                              : 27;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union IA_DEBUG_DATA {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union IA_DEBUG_REG0 {
    struct {
        unsigned int                                                              :  2;
        unsigned int ia_busy                                                      :  1;
        unsigned int                                                              :  1;
        unsigned int SPARE0                                                       :  1;
        unsigned int                                                              :  4;
        unsigned int SPARE1                                                       :  1;
        unsigned int                                                              :  4;
        unsigned int SPARE2                                                       : 10;
        unsigned int                                                              :  2;
        unsigned int SPARE3                                                       :  1;
        unsigned int SPARE4                                                       :  1;
        unsigned int sclk_reg_vld                                                 :  1;
        unsigned int sclk_core_vld                                                :  1;
        unsigned int SPARE5                                                       :  1;
        unsigned int                                                              :  1;
    } rv1x;
    struct {
        unsigned int                                                              :  2;
        unsigned int ia_busy                                                      :  1;
        unsigned int                                                              :  1;
        unsigned int RESERVED0                                                    :  1;
        unsigned int                                                              :  2;
        unsigned int tcarb_busy                                                   :  1;
        unsigned int                                                              :  1;
        unsigned int RESERVED1                                                    :  1;
        unsigned int                                                              :  2;
        unsigned int dma_grp_pipe1_valid                                          :  1;
        unsigned int grp_dma_pipe1_read                                           :  1;
        unsigned int RESERVED2                                                    : 10;
        unsigned int                                                              :  2;
        unsigned int RESERVED3                                                    :  1;
        unsigned int RESERVED4                                                    :  1;
        unsigned int sclk_reg_vld                                                 :  1;
        unsigned int sclk_core_vld                                                :  1;
        unsigned int RESERVED5                                                    :  1;
        unsigned int RESERVED6                                                    :  1;
    } rv2x;
    struct {
        unsigned int                                                              :  2;
        unsigned int ia_busy                                                      :  1;
        unsigned int                                                              :  1;
        unsigned int SPARE0                                                       :  1;
        unsigned int                                                              :  4;
        unsigned int SPARE1                                                       :  1;
        unsigned int                                                              :  4;
        unsigned int SPARE2                                                       : 10;
        unsigned int                                                              :  2;
        unsigned int SPARE3                                                       :  1;
        unsigned int SPARE4                                                       :  1;
        unsigned int sclk_reg_vld                                                 :  1;
        unsigned int sclk_core_vld                                                :  1;
        unsigned int SPARE5                                                       :  1;
        unsigned int                                                              :  1;
    } vega;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union IA_DEBUG_REG1 {
    struct {
        unsigned int                                                              : 15;
        unsigned int dma_data_fifo_full                                           :  1;
        unsigned int                                                              : 16;
    } most;
    struct {
        unsigned int                                                              :  7;
        unsigned int RESERVED0                                                    :  1;
        unsigned int                                                              : 24;
    } rv2x;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union IA_DEBUG_REG9 {
    struct {
        unsigned int                                                              : 14;
        unsigned int SPARE0                                                       :  1;
        unsigned int SPARE1                                                       :  1;
        unsigned int                                                              : 16;
    } most;
    struct {
        unsigned int                                                              : 14;
        unsigned int RESERVED0                                                    :  1;
        unsigned int RESERVED1                                                    :  1;
        unsigned int                                                              :  4;
        unsigned int prim_counter_q_11_0                                          : 12;
    } rv2x;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union IA_DEBUG_REG10 {
    struct {
        unsigned int RESERVED0                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union IA_DEBUG_REG11 {
    struct {
        unsigned int RESERVED0                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union IA_DEBUG_REG12 {
    struct {
        unsigned int iUTCL2_IA_ret_tag                                            :  7;
        unsigned int iUTCL2_IA_ret_nack                                           :  2;
        unsigned int iUTCL2_IA_ret_mtype                                          :  2;
        unsigned int iUTCL2_IA_ret_memlog                                         :  1;
        unsigned int iUTCL2_IA_ret_no_pte                                         :  1;
        unsigned int iUTCL2_IA_ret_pte_tmz                                        :  1;
        unsigned int iUTCL2_IA_ret_spa                                            :  1;
        unsigned int iUTCL2_IA_ret_io_steer                                       :  1;
        unsigned int iUTCL2_IA_ret_fragment_size                                  :  6;
        unsigned int iUTCL2_IA_ret_perms_granted                                  :  3;
        unsigned int iUTCL2_IA_ret_perms_requested                                :  3;
        unsigned int iUTCL2_IA_ret_snoop                                          :  1;
        unsigned int iUTCL2_IA_ret_valid                                          :  1;
        unsigned int iUTCL2_IA_ret_physical_address_1_0                           :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union IA_DEBUG_REG13 {
    struct {
        unsigned int                                                              : 30;
        unsigned int iUTCL2_IA_ret_physical_address_33_2                          :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union IA_DEBUG_REG14 {
    struct {
        unsigned int iUTCL2_IA_ret_physical_address_35_34                         :  2;
        unsigned int IA_UTCL2_req_data                                            : 16;
        unsigned int qIA_UTCL2_req_free                                           :  1;
        unsigned int IA_UTCL2_req_send                                            :  1;
        unsigned int qUtcL2XferCycle_eq_0                                         :  1;
        unsigned int RESERVED0                                                    : 11;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union IA_DEBUG_REG15 {
    struct {
        unsigned int IAUtcL2iuPostArbFifo_Empty                                   :  1;
        unsigned int IAUtcL2iuPostArbFifo_Full                                    :  1;
        unsigned int IAUtcL2Cnt                                                   :  6;
        unsigned int IAUtcL2OutCnt                                                :  6;
        unsigned int qUtcL2XferCycle                                              :  3;
        unsigned int iUTCL10_UTCL2IU_req_rts                                      :  1;
        unsigned int XferId                                                       :  3;
        unsigned int qXferId                                                      :  3;
        unsigned int NewCycleXfer                                                 :  1;
        unsigned int RESERVED0                                                    :  7;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union IA_DEBUG_REG16 {
    struct {
        unsigned int outstanding_tc_accesses_8_0                                  :  9;
        unsigned int req0_valid_hold2                                             :  1;
        unsigned int req0_valid_hold1                                             :  1;
        unsigned int utcl2iu_busy                                                 :  1;
        unsigned int utcl10_busy                                                  :  1;
        unsigned int busy                                                         :  1;
        unsigned int UTCL2IU_UTCL1_ret_nack                                       :  2;
        unsigned int UTCL2IU_UTCL1_ret_valid                                      :  1;
        unsigned int TC_IA_rdret_valid                                            :  1;
        unsigned int cpf_ia_invreq_clean_out                                      :  1;
        unsigned int cpf_ia_invreq_flushtype_in                                   :  1;
        unsigned int cpf_ia_invreq_start_in                                       :  1;
        unsigned int IA_UTCL2_inv_ack                                             :  1;
        unsigned int UTCL2_IA_inv_valid                                           :  1;
        unsigned int UTCL2_IA_ret_xnack                                           :  2;
        unsigned int UTCL2_IA_ret_valid                                           :  1;
        unsigned int IA_UTCL2_req_free                                            :  1;
        unsigned int IA_UTCL2_req_send                                            :  1;
        unsigned int UTCL1_CLIENT_ret0_ready                                      :  1;
        unsigned int UTCL1_CLIENT_ret0_valid                                      :  1;
        unsigned int CLIENT_UTCL1_req0_ready                                      :  1;
        unsigned int CLIENT_UTCL1_req0_valid                                      :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union IA_ENHANCE {
    struct {
        unsigned int MISC                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union IA_MULTI_VGT_PARAM {
    struct {
        unsigned int PRIMGROUP_SIZE                                               : 16;
        unsigned int PARTIAL_VS_WAVE_ON                                           :  1;
        unsigned int SWITCH_ON_EOP                                                :  1;
        unsigned int PARTIAL_ES_WAVE_ON                                           :  1;
        unsigned int SWITCH_ON_EOI                                                :  1;
        unsigned int WD_SWITCH_ON_EOP                                             :  1;
        unsigned int                                                              : 11;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 21;
        unsigned int EN_INST_OPT_BASIC                                            :  1;
        unsigned int EN_INST_OPT_ADV                                              :  1;
        unsigned int HW_USE_ONLY                                                  :  1;
        unsigned int                                                              :  8;
    } gfx09;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union IA_MULTI_VGT_PARAM_BC {
    struct {
        unsigned int RESERVED                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union IA_MULTI_VGT_PARAM_PIPED {
    struct {
        unsigned int PRIMGROUP_SIZE                                               : 16;
        unsigned int PARTIAL_VS_WAVE_ON                                           :  1;
        unsigned int SWITCH_ON_EOP                                                :  1;
        unsigned int PARTIAL_ES_WAVE_ON                                           :  1;
        unsigned int SWITCH_ON_EOI                                                :  1;
        unsigned int WD_SWITCH_ON_EOP                                             :  1;
        unsigned int EN_INST_OPT_BASIC                                            :  1;
        unsigned int EN_INST_OPT_ADV                                              :  1;
        unsigned int HW_USE_ONLY                                                  :  1;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union IA_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union IA_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union IA_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union IA_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union IA_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union IA_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union IA_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int                                                              : 20;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union IA_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union IA_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union IA_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int                                                              : 20;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union IA_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union IA_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union IA_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int                                                              : 20;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union IA_UTCL1_CNTL {
    struct {
        unsigned int XNACK_REDO_TIMER_CNT                                         : 20;
        unsigned int                                                              :  3;
        unsigned int VMID_RESET_MODE                                              :  1;
        unsigned int DROP_MODE                                                    :  1;
        unsigned int BYPASS                                                       :  1;
        unsigned int INVALIDATE                                                   :  1;
        unsigned int FRAG_LIMIT_MODE                                              :  1;
        unsigned int FORCE_SNOOP                                                  :  1;
        unsigned int                                                              :  3;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 29;
        unsigned int FORCE_SD_VMID_DIRTY                                          :  1;
        unsigned int                                                              :  2;
    } gfx09;
    struct {
        unsigned int                                                              : 29;
        unsigned int MTYPE_OVERRIDE                                               :  1;
        unsigned int                                                              :  2;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union IA_UTCL1_STATUS {
    struct {
        unsigned int FAULT_DETECTED                                               :  1;
        unsigned int RETRY_DETECTED                                               :  1;
        unsigned int PRT_DETECTED                                                 :  1;
        unsigned int                                                              :  5;
        unsigned int FAULT_UTCL1ID                                                :  6;
        unsigned int                                                              :  2;
        unsigned int RETRY_UTCL1ID                                                :  6;
        unsigned int                                                              :  2;
        unsigned int PRT_UTCL1ID                                                  :  6;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union IA_UTCL1_STATUS_2 {
    struct {
        unsigned int IA_BUSY                                                      :  1;
        unsigned int IA_DMA_BUSY                                                  :  1;
        unsigned int IA_DMA_REQ_BUSY                                              :  1;
        unsigned int IA_GRP_BUSY                                                  :  1;
        unsigned int IA_ADC_BUSY                                                  :  1;
        unsigned int FAULT_DETECTED                                               :  1;
        unsigned int RETRY_DETECTED                                               :  1;
        unsigned int PRT_DETECTED                                                 :  1;
        unsigned int FAULT_UTCL1ID                                                :  6;
        unsigned int                                                              :  2;
        unsigned int RETRY_UTCL1ID                                                :  6;
        unsigned int                                                              :  2;
        unsigned int PRT_UTCL1ID                                                  :  6;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union LDS_CONFIG {
    struct {
        unsigned int ADDR_OUT_OF_RANGE_REPORTING                                  :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  2;
        unsigned int DISABLE_RAM_CLOCK_GATING                                     :  1;
        unsigned int                                                              : 29;
    } gfx09_1x;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_MEM_POWER_LS {
    struct {
        unsigned int LS_SETUP                                                     :  6;
        unsigned int LS_HOLD                                                      :  6;
        unsigned int                                                              : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_SHARED_ACTIVE_FCN_ID {
    struct {
        unsigned int VFID                                                         :  4;
        unsigned int                                                              : 27;
        unsigned int VF                                                           :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_SHARED_VIRT_RESET_REQ {
    struct {
        unsigned int VF                                                           : 16;
        unsigned int                                                              : 15;
        unsigned int PF                                                           :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_AGP_BASE {
    struct {
        unsigned int AGP_BASE                                                     : 24;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_AGP_BOT {
    struct {
        unsigned int AGP_BOT                                                      : 24;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_AGP_TOP {
    struct {
        unsigned int AGP_TOP                                                      : 24;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_APT_CNTL {
    struct {
        unsigned int FORCE_MTYPE_UC                                               :  1;
        unsigned int DIRECT_SYSTEM_EN                                             :  1;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_CACHEABLE_DRAM_ADDRESS_END {
    struct {
        unsigned int ADDRESS                                                      : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_CACHEABLE_DRAM_ADDRESS_START {
    struct {
        unsigned int ADDRESS                                                      : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_FB_LOCATION_BASE {
    struct {
        unsigned int FB_BASE                                                      : 24;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_FB_LOCATION_TOP {
    struct {
        unsigned int FB_TOP                                                       : 24;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_FB_OFFSET {
    struct {
        unsigned int FB_OFFSET                                                    : 24;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_FB_SIZE_OFFSET_VF0 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_FB_SIZE_OFFSET_VF1 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_FB_SIZE_OFFSET_VF2 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_FB_SIZE_OFFSET_VF3 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_FB_SIZE_OFFSET_VF4 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_FB_SIZE_OFFSET_VF5 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_FB_SIZE_OFFSET_VF6 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_FB_SIZE_OFFSET_VF7 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_FB_SIZE_OFFSET_VF8 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_FB_SIZE_OFFSET_VF9 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_FB_SIZE_OFFSET_VF10 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_FB_SIZE_OFFSET_VF11 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_FB_SIZE_OFFSET_VF12 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_FB_SIZE_OFFSET_VF13 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_FB_SIZE_OFFSET_VF14 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_FB_SIZE_OFFSET_VF15 {
    struct {
        unsigned int VF_FB_SIZE                                                   : 16;
        unsigned int VF_FB_OFFSET                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_L2_PERFCOUNTER0_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_L2_PERFCOUNTER1_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_L2_PERFCOUNTER2_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_L2_PERFCOUNTER3_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_L2_PERFCOUNTER4_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_L2_PERFCOUNTER5_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_L2_PERFCOUNTER6_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_L2_PERFCOUNTER7_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_L2_PERFCOUNTER_HI {
    struct {
        unsigned int COUNTER_HI                                                   : 16;
        unsigned int COMPARE_VALUE                                                : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_L2_PERFCOUNTER_LO {
    struct {
        unsigned int COUNTER_LO                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_L2_PERFCOUNTER_RSLT_CNTL {
    struct {
        unsigned int PERF_COUNTER_SELECT                                          :  4;
        unsigned int                                                              :  4;
        unsigned int START_TRIGGER                                                :  8;
        unsigned int STOP_TRIGGER                                                 :  8;
        unsigned int ENABLE_ANY                                                   :  1;
        unsigned int CLEAR_ALL                                                    :  1;
        unsigned int STOP_ALL_ON_SATURATE                                         :  1;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_LOCAL_HBM_ADDRESS_END {
    struct {
        unsigned int ADDRESS                                                      : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_LOCAL_HBM_ADDRESS_LOCK_CNTL {
    struct {
        unsigned int LOCK                                                         :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_LOCAL_HBM_ADDRESS_START {
    struct {
        unsigned int ADDRESS                                                      : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_MARC_BASE_HI_0 {
    struct {
        unsigned int MARC_BASE_HI_0                                               : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_MARC_BASE_HI_1 {
    struct {
        unsigned int MARC_BASE_HI_1                                               : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_MARC_BASE_HI_2 {
    struct {
        unsigned int MARC_BASE_HI_2                                               : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_MARC_BASE_HI_3 {
    struct {
        unsigned int MARC_BASE_HI_3                                               : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_MARC_BASE_LO_0 {
    struct {
        unsigned int                                                              : 12;
        unsigned int MARC_BASE_LO_0                                               : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_MARC_BASE_LO_1 {
    struct {
        unsigned int                                                              : 12;
        unsigned int MARC_BASE_LO_1                                               : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_MARC_BASE_LO_2 {
    struct {
        unsigned int                                                              : 12;
        unsigned int MARC_BASE_LO_2                                               : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_MARC_BASE_LO_3 {
    struct {
        unsigned int                                                              : 12;
        unsigned int MARC_BASE_LO_3                                               : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_MARC_LEN_HI_0 {
    struct {
        unsigned int MARC_LEN_HI_0                                                : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_MARC_LEN_HI_1 {
    struct {
        unsigned int MARC_LEN_HI_1                                                : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_MARC_LEN_HI_2 {
    struct {
        unsigned int MARC_LEN_HI_2                                                : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_MARC_LEN_HI_3 {
    struct {
        unsigned int MARC_LEN_HI_3                                                : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_MARC_LEN_LO_0 {
    struct {
        unsigned int                                                              : 12;
        unsigned int MARC_LEN_LO_0                                                : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_MARC_LEN_LO_1 {
    struct {
        unsigned int                                                              : 12;
        unsigned int MARC_LEN_LO_1                                                : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_MARC_LEN_LO_2 {
    struct {
        unsigned int                                                              : 12;
        unsigned int MARC_LEN_LO_2                                                : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_MARC_LEN_LO_3 {
    struct {
        unsigned int                                                              : 12;
        unsigned int MARC_LEN_LO_3                                                : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_MARC_RELOC_HI_0 {
    struct {
        unsigned int MARC_RELOC_HI_0                                              : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_MARC_RELOC_HI_1 {
    struct {
        unsigned int MARC_RELOC_HI_1                                              : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_MARC_RELOC_HI_2 {
    struct {
        unsigned int MARC_RELOC_HI_2                                              : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_MARC_RELOC_HI_3 {
    struct {
        unsigned int MARC_RELOC_HI_3                                              : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_MARC_RELOC_LO_0 {
    struct {
        unsigned int MARC_ENABLE_0                                                :  1;
        unsigned int MARC_READONLY_0                                              :  1;
        unsigned int                                                              : 10;
        unsigned int MARC_RELOC_LO_0                                              : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_MARC_RELOC_LO_1 {
    struct {
        unsigned int MARC_ENABLE_1                                                :  1;
        unsigned int MARC_READONLY_1                                              :  1;
        unsigned int                                                              : 10;
        unsigned int MARC_RELOC_LO_1                                              : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_MARC_RELOC_LO_2 {
    struct {
        unsigned int MARC_ENABLE_2                                                :  1;
        unsigned int MARC_READONLY_2                                              :  1;
        unsigned int                                                              : 10;
        unsigned int MARC_RELOC_LO_2                                              : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_MARC_RELOC_LO_3 {
    struct {
        unsigned int MARC_ENABLE_3                                                :  1;
        unsigned int MARC_READONLY_3                                              :  1;
        unsigned int                                                              : 10;
        unsigned int MARC_RELOC_LO_3                                              : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_MX_L1_TLB_CNTL {
    struct {
        unsigned int ENABLE_L1_TLB                                                :  1;
        unsigned int                                                              :  2;
        unsigned int SYSTEM_ACCESS_MODE                                           :  2;
        unsigned int SYSTEM_APERTURE_UNMAPPED_ACCESS                              :  1;
        unsigned int ENABLE_ADVANCED_DRIVER_MODEL                                 :  1;
        unsigned int ECO_BITS                                                     :  4;
        unsigned int MTYPE                                                        :  2;
        unsigned int ATC_EN                                                       :  1;
        unsigned int                                                              : 18;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_NB_LOWER_TOP_OF_DRAM2 {
    struct {
        unsigned int ENABLE                                                       :  1;
        unsigned int                                                              : 22;
        unsigned int LOWER_TOM2                                                   :  9;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_NB_MMIOBASE {
    struct {
        unsigned int MMIOBASE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_NB_MMIOLIMIT {
    struct {
        unsigned int MMIOLIMIT                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_NB_PCI_ARB {
    struct {
        unsigned int                                                              :  3;
        unsigned int VGA_HOLE                                                     :  1;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_NB_PCI_CTRL {
    struct {
        unsigned int                                                              : 23;
        unsigned int MMIOENABLE                                                   :  1;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_NB_TOP_OF_DRAM_SLOT1 {
    struct {
        unsigned int                                                              : 23;
        unsigned int TOP_OF_DRAM                                                  :  9;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_NB_UPPER_TOP_OF_DRAM2 {
    struct {
        unsigned int UPPER_TOM2                                                   : 12;
        unsigned int                                                              : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_STEERING {
    struct {
        unsigned int DEFAULT_STEERING                                             :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_SYSTEM_APERTURE_DEFAULT_ADDR_LSB {
    struct {
        unsigned int PHYSICAL_PAGE_NUMBER_LSB                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_SYSTEM_APERTURE_DEFAULT_ADDR_MSB {
    struct {
        unsigned int PHYSICAL_PAGE_NUMBER_MSB                                     :  4;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_SYSTEM_APERTURE_HIGH_ADDR {
    struct {
        unsigned int LOGICAL_ADDR                                                 : 30;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_SYSTEM_APERTURE_LOW_ADDR {
    struct {
        unsigned int LOGICAL_ADDR                                                 : 30;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_XGMI_GPUIOV_ENABLE {
    struct {
        unsigned int ENABLE_VF0                                                   :  1;
        unsigned int ENABLE_VF1                                                   :  1;
        unsigned int ENABLE_VF2                                                   :  1;
        unsigned int ENABLE_VF3                                                   :  1;
        unsigned int ENABLE_VF4                                                   :  1;
        unsigned int ENABLE_VF5                                                   :  1;
        unsigned int ENABLE_VF6                                                   :  1;
        unsigned int ENABLE_VF7                                                   :  1;
        unsigned int ENABLE_VF8                                                   :  1;
        unsigned int ENABLE_VF9                                                   :  1;
        unsigned int ENABLE_VF10                                                  :  1;
        unsigned int ENABLE_VF11                                                  :  1;
        unsigned int ENABLE_VF12                                                  :  1;
        unsigned int ENABLE_VF13                                                  :  1;
        unsigned int ENABLE_VF14                                                  :  1;
        unsigned int ENABLE_VF15                                                  :  1;
        unsigned int                                                              : 15;
        unsigned int ENABLE_PF                                                    :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_XGMI_LFB_CNTL {
    struct {
        unsigned int PF_LFB_REGION                                                :  3;
        unsigned int                                                              : 29;
    } most;
    struct {
        unsigned int                                                              :  3;
        unsigned int PF_MAX_REGION                                                :  3;
        unsigned int                                                              : 26;
    } vg12;
    struct {
        unsigned int                                                              :  4;
        unsigned int PF_MAX_REGION                                                :  3;
        unsigned int                                                              : 25;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MC_VM_XGMI_LFB_SIZE {
    struct {
        unsigned int PF_LFB_SIZE                                                  : 16;
        unsigned int                                                              : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union oa_wc0_DEBUG_DATA {
    struct {
        unsigned int                                                              :  1;
        unsigned int credit_cnt                                                   :  1;
        unsigned int                                                              :  6;
        unsigned int req                                                          : 11;
        unsigned int Reserved0                                                    : 13;
    } gfx09_0;
    struct {
        unsigned int                                                              :  1;
        unsigned int credit_cnt                                                   :  1;
        unsigned int                                                              :  6;
        unsigned int req                                                          : 11;
        unsigned int Reserved0                                                    : 13;
    } rv2x;
    struct {
        unsigned int                                                              :  1;
        unsigned int credit_cnt                                                   :  1;
        unsigned int                                                              :  6;
        unsigned int req                                                          : 11;
        unsigned int Reserved0                                                    : 13;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union oa_wc1_DEBUG_DATA {
    struct {
        unsigned int pipe0_busy                                                   :  1;
        unsigned int pipe1_busy                                                   :  1;
        unsigned int pipe2_busy                                                   :  1;
        unsigned int pipe3_busy                                                   :  1;
        unsigned int pipe4_busy                                                   :  1;
        unsigned int pipe5_busy                                                   :  1;
        unsigned int pipe6_busy                                                   :  1;
        unsigned int pipe7_busy                                                   :  1;
        unsigned int pipe8_busy                                                   :  1;
        unsigned int pipe9_busy                                                   :  1;
        unsigned int Pipe10_busy                                                  :  1;
        unsigned int pipe0_busy0                                                  :  1;
        unsigned int pipe0_busy1                                                  :  1;
        unsigned int pipe0_busy2                                                  :  1;
        unsigned int pipe0_busy3                                                  :  1;
        unsigned int pipe0_busy4                                                  :  1;
        unsigned int pipe0_busy5                                                  :  1;
        unsigned int pipe0_busy6                                                  :  1;
        unsigned int pipe0_busy7                                                  :  1;
        unsigned int Reserved0                                                    : 13;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union ord_app_DEBUG_DATA {
    struct {
        unsigned int fifo_busy                                                    :  1;
        unsigned int ord_busy                                                     :  1;
        unsigned int                                                              :  1;
        unsigned int Reserved0                                                    :  1;
        unsigned int sh0_cmd_fifo_empty                                           :  1;
        unsigned int sh1_cmd_fifo_empty                                           :  1;
        unsigned int sh2_cmd_fifo_empty                                           :  1;
        unsigned int sh3_cmd_fifo_empty                                           :  1;
        unsigned int sh0_data_fifo_empty                                          :  1;
        unsigned int sh1_data_fifo_empty                                          :  1;
        unsigned int sh2_data_fifo_empty                                          :  1;
        unsigned int sh3_data_fifo_empty                                          :  1;
        unsigned int Reserved1                                                    : 20;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_CLIP_CNTL {
    struct {
        unsigned int UCP_ENA_0                                                    :  1;
        unsigned int UCP_ENA_1                                                    :  1;
        unsigned int UCP_ENA_2                                                    :  1;
        unsigned int UCP_ENA_3                                                    :  1;
        unsigned int UCP_ENA_4                                                    :  1;
        unsigned int UCP_ENA_5                                                    :  1;
        unsigned int                                                              :  7;
        unsigned int PS_UCP_Y_SCALE_NEG                                           :  1;
        unsigned int PS_UCP_MODE                                                  :  2;
        unsigned int CLIP_DISABLE                                                 :  1;
        unsigned int UCP_CULL_ONLY_ENA                                            :  1;
        unsigned int BOUNDARY_EDGE_FLAG_ENA                                       :  1;
        unsigned int DX_CLIP_SPACE_DEF                                            :  1;
        unsigned int DIS_CLIP_ERR_DETECT                                          :  1;
        unsigned int VTX_KILL_OR                                                  :  1;
        unsigned int DX_RASTERIZATION_KILL                                        :  1;
        unsigned int                                                              :  1;
        unsigned int DX_LINEAR_ATTR_CLIP_ENA                                      :  1;
        unsigned int VTE_VPORT_PROVOKE_DISABLE                                    :  1;
        unsigned int ZCLIP_NEAR_DISABLE                                           :  1;
        unsigned int ZCLIP_FAR_DISABLE                                            :  1;
        unsigned int                                                              :  4;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 28;
        unsigned int ZCLIP_PROG_NEAR_ENA                                          :  1;
        unsigned int                                                              :  3;
    } gfx09_1xPlus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_CNTL_STATUS {
    struct {
        unsigned int UTC_FAULT_DETECTED                                           :  1;
        unsigned int UTC_RETRY_DETECTED                                           :  1;
        unsigned int UTC_PRT_DETECTED                                             :  1;
        unsigned int                                                              : 29;
    } most;
    struct {
        unsigned int                                                              : 31;
        unsigned int CL_BUSY                                                      :  1;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_ENHANCE {
    struct {
        unsigned int CLIP_VTX_REORDER_ENA                                         :  1;
        unsigned int NUM_CLIP_SEQ                                                 :  2;
        unsigned int CLIPPED_PRIM_SEQ_STALL                                       :  1;
        unsigned int VE_NAN_PROC_DISABLE                                          :  1;
        unsigned int XTRA_DEBUG_REG_SEL                                           :  1;
        unsigned int IGNORE_PIPELINE_RESET                                        :  1;
        unsigned int KILL_INNER_EDGE_FLAGS                                        :  1;
        unsigned int NGG_PA_TO_ALL_SC                                             :  1;
        unsigned int TC_LATENCY_TIME_STAMP_RESOLUTION                             :  2;
        unsigned int NGG_BYPASS_PRIM_FILTER                                       :  1;
        unsigned int NGG_SIDEBAND_MEMORY_DEPTH                                    :  2;
        unsigned int NGG_PRIM_INDICES_FIFO_DEPTH                                  :  3;
        unsigned int                                                              : 13;
        unsigned int ECO_SPARE1                                                   :  1;
        unsigned int ECO_SPARE0                                                   :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 28;
        unsigned int ECO_SPARE3                                                   :  1;
        unsigned int ECO_SPARE2                                                   :  1;
        unsigned int                                                              :  2;
    } most;
    struct {
        unsigned int                                                              : 17;
        unsigned int PROG_NEAR_CLIP_PLANE_ENABLE                                  :  1;
        unsigned int OUTPUT_SWITCH_TO_LEGACY_EVENT                                :  1;
        unsigned int NO_SWITCH_TO_LEGACY_AFTER_VMID_RESET                         :  1;
        unsigned int POLY_INNER_EDGE_FLAG_DISABLE                                 :  1;
        unsigned int TC_REQUEST_PERF_CNTR_ENABLE                                  :  1;
        unsigned int                                                              : 10;
    } gfx09_1xPlus;
    struct {
        unsigned int                                                              : 22;
        unsigned int DISABLE_PA_PH_INTF_FINE_CLOCK_GATE                           :  1;
        unsigned int                                                              :  9;
    } gfx10;
    struct {
        unsigned int                                                              : 23;
        unsigned int                                                              :  1;
        unsigned int                                                              :  8;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_GB_HORZ_CLIP_ADJ {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_GB_HORZ_DISC_ADJ {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_GB_VERT_CLIP_ADJ {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_GB_VERT_DISC_ADJ {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_NANINF_CNTL {
    struct {
        unsigned int VTE_XY_INF_DISCARD                                           :  1;
        unsigned int VTE_Z_INF_DISCARD                                            :  1;
        unsigned int VTE_W_INF_DISCARD                                            :  1;
        unsigned int VTE_0XNANINF_IS_0                                            :  1;
        unsigned int VTE_XY_NAN_RETAIN                                            :  1;
        unsigned int VTE_Z_NAN_RETAIN                                             :  1;
        unsigned int VTE_W_NAN_RETAIN                                             :  1;
        unsigned int VTE_W_RECIP_NAN_IS_0                                         :  1;
        unsigned int VS_XY_NAN_TO_INF                                             :  1;
        unsigned int VS_XY_INF_RETAIN                                             :  1;
        unsigned int VS_Z_NAN_TO_INF                                              :  1;
        unsigned int VS_Z_INF_RETAIN                                              :  1;
        unsigned int VS_W_NAN_TO_INF                                              :  1;
        unsigned int VS_W_INF_RETAIN                                              :  1;
        unsigned int VS_CLIP_DIST_INF_DISCARD                                     :  1;
        unsigned int                                                              :  5;
        unsigned int VTE_NO_OUTPUT_NEG_0                                          :  1;
        unsigned int                                                              : 11;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_NGG_CNTL {
    struct {
        unsigned int VERTEX_REUSE_OFF                                             :  1;
        unsigned int INDEX_BUF_EDGE_FLAG_ENA                                      :  1;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_POINT_CULL_RAD {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_POINT_SIZE {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_POINT_X_RAD {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_POINT_Y_RAD {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_PROG_NEAR_CLIP_Z {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_RESET_DEBUG {
    struct {
        unsigned int CL_TRIV_DISC_DISABLE                                         :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_0_W {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_0_X {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_0_Y {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_0_Z {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_1_W {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_1_X {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_1_Y {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_1_Z {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_2_W {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_2_X {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_2_Y {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_2_Z {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_3_W {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_3_X {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_3_Y {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_3_Z {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_4_W {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_4_X {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_4_Y {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_4_Z {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_5_W {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_5_X {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_5_Y {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_5_Z {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XOFFSET {
    struct {
        unsigned int VPORT_XOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XOFFSET_1 {
    struct {
        unsigned int VPORT_XOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XOFFSET_2 {
    struct {
        unsigned int VPORT_XOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XOFFSET_3 {
    struct {
        unsigned int VPORT_XOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XOFFSET_4 {
    struct {
        unsigned int VPORT_XOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XOFFSET_5 {
    struct {
        unsigned int VPORT_XOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XOFFSET_6 {
    struct {
        unsigned int VPORT_XOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XOFFSET_7 {
    struct {
        unsigned int VPORT_XOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XOFFSET_8 {
    struct {
        unsigned int VPORT_XOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XOFFSET_9 {
    struct {
        unsigned int VPORT_XOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XOFFSET_10 {
    struct {
        unsigned int VPORT_XOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XOFFSET_11 {
    struct {
        unsigned int VPORT_XOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XOFFSET_12 {
    struct {
        unsigned int VPORT_XOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XOFFSET_13 {
    struct {
        unsigned int VPORT_XOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XOFFSET_14 {
    struct {
        unsigned int VPORT_XOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XOFFSET_15 {
    struct {
        unsigned int VPORT_XOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XSCALE {
    struct {
        unsigned int VPORT_XSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XSCALE_1 {
    struct {
        unsigned int VPORT_XSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XSCALE_2 {
    struct {
        unsigned int VPORT_XSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XSCALE_3 {
    struct {
        unsigned int VPORT_XSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XSCALE_4 {
    struct {
        unsigned int VPORT_XSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XSCALE_5 {
    struct {
        unsigned int VPORT_XSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XSCALE_6 {
    struct {
        unsigned int VPORT_XSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XSCALE_7 {
    struct {
        unsigned int VPORT_XSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XSCALE_8 {
    struct {
        unsigned int VPORT_XSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XSCALE_9 {
    struct {
        unsigned int VPORT_XSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XSCALE_10 {
    struct {
        unsigned int VPORT_XSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XSCALE_11 {
    struct {
        unsigned int VPORT_XSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XSCALE_12 {
    struct {
        unsigned int VPORT_XSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XSCALE_13 {
    struct {
        unsigned int VPORT_XSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XSCALE_14 {
    struct {
        unsigned int VPORT_XSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XSCALE_15 {
    struct {
        unsigned int VPORT_XSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YOFFSET {
    struct {
        unsigned int VPORT_YOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YOFFSET_1 {
    struct {
        unsigned int VPORT_YOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YOFFSET_2 {
    struct {
        unsigned int VPORT_YOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YOFFSET_3 {
    struct {
        unsigned int VPORT_YOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YOFFSET_4 {
    struct {
        unsigned int VPORT_YOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YOFFSET_5 {
    struct {
        unsigned int VPORT_YOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YOFFSET_6 {
    struct {
        unsigned int VPORT_YOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YOFFSET_7 {
    struct {
        unsigned int VPORT_YOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YOFFSET_8 {
    struct {
        unsigned int VPORT_YOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YOFFSET_9 {
    struct {
        unsigned int VPORT_YOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YOFFSET_10 {
    struct {
        unsigned int VPORT_YOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YOFFSET_11 {
    struct {
        unsigned int VPORT_YOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YOFFSET_12 {
    struct {
        unsigned int VPORT_YOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YOFFSET_13 {
    struct {
        unsigned int VPORT_YOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YOFFSET_14 {
    struct {
        unsigned int VPORT_YOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YOFFSET_15 {
    struct {
        unsigned int VPORT_YOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YSCALE {
    struct {
        unsigned int VPORT_YSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YSCALE_1 {
    struct {
        unsigned int VPORT_YSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YSCALE_2 {
    struct {
        unsigned int VPORT_YSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YSCALE_3 {
    struct {
        unsigned int VPORT_YSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YSCALE_4 {
    struct {
        unsigned int VPORT_YSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YSCALE_5 {
    struct {
        unsigned int VPORT_YSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YSCALE_6 {
    struct {
        unsigned int VPORT_YSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YSCALE_7 {
    struct {
        unsigned int VPORT_YSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YSCALE_8 {
    struct {
        unsigned int VPORT_YSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YSCALE_9 {
    struct {
        unsigned int VPORT_YSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YSCALE_10 {
    struct {
        unsigned int VPORT_YSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YSCALE_11 {
    struct {
        unsigned int VPORT_YSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YSCALE_12 {
    struct {
        unsigned int VPORT_YSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YSCALE_13 {
    struct {
        unsigned int VPORT_YSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YSCALE_14 {
    struct {
        unsigned int VPORT_YSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YSCALE_15 {
    struct {
        unsigned int VPORT_YSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZOFFSET {
    struct {
        unsigned int VPORT_ZOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZOFFSET_1 {
    struct {
        unsigned int VPORT_ZOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZOFFSET_2 {
    struct {
        unsigned int VPORT_ZOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZOFFSET_3 {
    struct {
        unsigned int VPORT_ZOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZOFFSET_4 {
    struct {
        unsigned int VPORT_ZOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZOFFSET_5 {
    struct {
        unsigned int VPORT_ZOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZOFFSET_6 {
    struct {
        unsigned int VPORT_ZOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZOFFSET_7 {
    struct {
        unsigned int VPORT_ZOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZOFFSET_8 {
    struct {
        unsigned int VPORT_ZOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZOFFSET_9 {
    struct {
        unsigned int VPORT_ZOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZOFFSET_10 {
    struct {
        unsigned int VPORT_ZOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZOFFSET_11 {
    struct {
        unsigned int VPORT_ZOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZOFFSET_12 {
    struct {
        unsigned int VPORT_ZOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZOFFSET_13 {
    struct {
        unsigned int VPORT_ZOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZOFFSET_14 {
    struct {
        unsigned int VPORT_ZOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZOFFSET_15 {
    struct {
        unsigned int VPORT_ZOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZSCALE {
    struct {
        unsigned int VPORT_ZSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZSCALE_1 {
    struct {
        unsigned int VPORT_ZSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZSCALE_2 {
    struct {
        unsigned int VPORT_ZSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZSCALE_3 {
    struct {
        unsigned int VPORT_ZSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZSCALE_4 {
    struct {
        unsigned int VPORT_ZSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZSCALE_5 {
    struct {
        unsigned int VPORT_ZSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZSCALE_6 {
    struct {
        unsigned int VPORT_ZSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZSCALE_7 {
    struct {
        unsigned int VPORT_ZSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZSCALE_8 {
    struct {
        unsigned int VPORT_ZSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZSCALE_9 {
    struct {
        unsigned int VPORT_ZSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZSCALE_10 {
    struct {
        unsigned int VPORT_ZSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZSCALE_11 {
    struct {
        unsigned int VPORT_ZSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZSCALE_12 {
    struct {
        unsigned int VPORT_ZSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZSCALE_13 {
    struct {
        unsigned int VPORT_ZSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZSCALE_14 {
    struct {
        unsigned int VPORT_ZSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZSCALE_15 {
    struct {
        unsigned int VPORT_ZSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VS_OUT_CNTL {
    struct {
        unsigned int CLIP_DIST_ENA_0                                              :  1;
        unsigned int CLIP_DIST_ENA_1                                              :  1;
        unsigned int CLIP_DIST_ENA_2                                              :  1;
        unsigned int CLIP_DIST_ENA_3                                              :  1;
        unsigned int CLIP_DIST_ENA_4                                              :  1;
        unsigned int CLIP_DIST_ENA_5                                              :  1;
        unsigned int CLIP_DIST_ENA_6                                              :  1;
        unsigned int CLIP_DIST_ENA_7                                              :  1;
        unsigned int CULL_DIST_ENA_0                                              :  1;
        unsigned int CULL_DIST_ENA_1                                              :  1;
        unsigned int CULL_DIST_ENA_2                                              :  1;
        unsigned int CULL_DIST_ENA_3                                              :  1;
        unsigned int CULL_DIST_ENA_4                                              :  1;
        unsigned int CULL_DIST_ENA_5                                              :  1;
        unsigned int CULL_DIST_ENA_6                                              :  1;
        unsigned int CULL_DIST_ENA_7                                              :  1;
        unsigned int USE_VTX_POINT_SIZE                                           :  1;
        unsigned int USE_VTX_EDGE_FLAG                                            :  1;
        unsigned int USE_VTX_RENDER_TARGET_INDX                                   :  1;
        unsigned int USE_VTX_VIEWPORT_INDX                                        :  1;
        unsigned int USE_VTX_KILL_FLAG                                            :  1;
        unsigned int VS_OUT_MISC_VEC_ENA                                          :  1;
        unsigned int VS_OUT_CCDIST0_VEC_ENA                                       :  1;
        unsigned int VS_OUT_CCDIST1_VEC_ENA                                       :  1;
        unsigned int VS_OUT_MISC_SIDE_BUS_ENA                                     :  1;
        unsigned int USE_VTX_GS_CUT_FLAG                                          :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 26;
        unsigned int USE_VTX_LINE_WIDTH                                           :  1;
        unsigned int                                                              :  1;
        unsigned int                                                              :  4;
    } gfx09;
    struct {
        unsigned int                                                              : 27;
        unsigned int USE_VTX_LINE_WIDTH                                           :  1;
        unsigned int                                                              :  4;
    } gfx10;
    struct {
        unsigned int                                                              : 26;
        unsigned int                                                              :  1;
        unsigned int                                                              :  5;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VTE_CNTL {
    struct {
        unsigned int VPORT_X_SCALE_ENA                                            :  1;
        unsigned int VPORT_X_OFFSET_ENA                                           :  1;
        unsigned int VPORT_Y_SCALE_ENA                                            :  1;
        unsigned int VPORT_Y_OFFSET_ENA                                           :  1;
        unsigned int VPORT_Z_SCALE_ENA                                            :  1;
        unsigned int VPORT_Z_OFFSET_ENA                                           :  1;
        unsigned int                                                              :  2;
        unsigned int VTX_XY_FMT                                                   :  1;
        unsigned int VTX_Z_FMT                                                    :  1;
        unsigned int VTX_W0_FMT                                                   :  1;
        unsigned int PERFCOUNTER_REF                                              :  1;
        unsigned int                                                              : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG00_0 {
    struct {
        unsigned int clip_ga_bc_fifo_write                                        :  1;
        unsigned int su_clip_baryc_free                                           :  2;
        unsigned int clip_to_ga_fifo_write                                        :  1;
        unsigned int clip_to_ga_fifo_full                                         :  1;
        unsigned int primic_to_clprim_fifo_empty                                  :  1;
        unsigned int primic_to_clprim_fifo_full                                   :  1;
        unsigned int clip_to_outsm_fifo_empty                                     :  1;
        unsigned int clip_to_outsm_fifo_full                                      :  1;
        unsigned int vgt_to_clipp_fifo_empty                                      :  1;
        unsigned int vgt_to_clipp_fifo_full                                       :  1;
        unsigned int vgt_to_clips_fifo_empty                                      :  1;
        unsigned int vgt_to_clips_fifo_full                                       :  1;
        unsigned int clipcode_fifo_fifo_empty                                     :  1;
        unsigned int clipcode_fifo_full                                           :  1;
        unsigned int vte_out_clip_fifo_fifo_empty                                 :  1;
        unsigned int vte_out_clip_fifo_fifo_full                                  :  1;
        unsigned int vte_out_orig_fifo_fifo_empty                                 :  1;
        unsigned int vte_out_orig_fifo_fifo_full                                  :  1;
        unsigned int ccgen_to_clipcc_fifo_empty                                   :  1;
        unsigned int ccgen_to_clipcc_fifo_full                                    :  1;
        unsigned int clip_to_outsm_fifo_write                                     :  1;
        unsigned int vte_out_orig_fifo_fifo_write                                 :  1;
        unsigned int vgt_to_clipp_fifo_write                                      :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG00_1 {
    struct {
        unsigned int vertex_fifo_entriesavailable                                 :  4;
        unsigned int statevar_bits_vs_out_ccdist1_vec_ena                         :  1;
        unsigned int statevar_bits_vs_out_ccdist0_vec_ena                         :  1;
        unsigned int available_positions                                          :  7;
        unsigned int current_state                                                :  2;
        unsigned int vertex_fifo_empty                                            :  1;
        unsigned int vertex_fifo_full                                             :  1;
        unsigned int sx0_receive_fifo_empty                                       :  1;
        unsigned int sx0_receive_fifo_full                                        :  1;
        unsigned int vgt_to_ccgen_fifo_empty                                      :  1;
        unsigned int vgt_to_ccgen_fifo_full                                       :  1;
        unsigned int ccgen_to_clipcc_fifo_full                                    :  1;
        unsigned int sx0_receive_fifo_write                                       :  1;
        unsigned int ccgen_to_clipcc_write                                        :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG01_0 {
    struct {
        unsigned int clip_extra_bc_valid                                          :  3;
        unsigned int                                                              :  2;
        unsigned int clip_vert_vte_valid                                          :  1;
        unsigned int clip_to_outsm_vertex_deallocate                              :  3;
        unsigned int clip_to_outsm_deallocate_slot                                :  3;
        unsigned int clip_to_outsm_null_primitive                                 :  1;
        unsigned int vte_positions_vte_clip_vte_naninf_kill_2                     :  1;
        unsigned int vte_positions_vte_clip_vte_naninf_kill_1                     :  1;
        unsigned int vte_positions_vte_clip_vte_naninf_kill_0                     :  1;
        unsigned int vte_out_clip_rd_extra_bc_valid                               :  1;
        unsigned int vte_out_clip_rd_vte_naninf_kill                              :  1;
        unsigned int vte_out_clip_rd_vertex_store_indx                            :  2;
        unsigned int clip_ga_bc_fifo_write                                        :  1;
        unsigned int clip_to_ga_fifo_write                                        :  1;
        unsigned int vte_out_clip_fifo_fifo_advanceread                           :  1;
        unsigned int vte_out_clip_fifo_fifo_empty                                 :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG01_1 {
    struct {
        unsigned int ALWAYS_ZERO                                                  :  8;
        unsigned int clip_extra_bc_valid                                          :  3;
        unsigned int clip_vert_vte_valid                                          :  3;
        unsigned int clip_to_outsm_vertex_deallocate                              :  2;
        unsigned int vte_out_clip_rd_extra_bc_valid                               :  1;
        unsigned int vte_out_clip_rd_vte_naninf_kill                              :  1;
        unsigned int vte_out_clip_rd_vertex_store_indx                            :  2;
        unsigned int clip_ga_bc_fifo_write                                        :  1;
        unsigned int clip_to_ga_fifo_write                                        :  1;
        unsigned int vte_out_clip_fifo_fifo_advanceread                           :  1;
        unsigned int vte_out_clip_fifo_fifo_empty                                 :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG02_0 {
    struct {
        unsigned int clip_to_outsm_vertex_store_indx_2                            :  4;
        unsigned int clip_to_outsm_vertex_store_indx_1                            :  4;
        unsigned int clip_to_outsm_vertex_store_indx_0                            :  4;
        unsigned int clip_to_clipga_extra_bc_coords                               :  1;
        unsigned int clip_to_clipga_vte_naninf_kill                               :  1;
        unsigned int clip_to_outsm_end_of_packet                                  :  1;
        unsigned int clip_to_outsm_first_prim_of_slot                             :  1;
        unsigned int clip_to_outsm_clipped_prim                                   :  1;
        unsigned int clip_to_outsm_null_primitive                                 :  1;
        unsigned int clip_ga_bc_fifo_full                                         :  1;
        unsigned int clip_to_ga_fifo_full                                         :  1;
        unsigned int clip_ga_bc_fifo_write                                        :  1;
        unsigned int clip_to_ga_fifo_write                                        :  1;
        unsigned int                                                              :  8;
        unsigned int clip_to_outsm_fifo_advanceread                               :  1;
        unsigned int clip_to_outsm_fifo_empty                                     :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG02_1 {
    struct {
        unsigned int clip_extra_bc_valid                                          :  3;
        unsigned int clip_vert_vte_valid                                          :  3;
        unsigned int clip_to_outsm_clip_seq_indx                                  :  2;
        unsigned int clip_to_outsm_vertex_store_indx_2                            :  4;
        unsigned int clip_to_outsm_vertex_store_indx_1                            :  4;
        unsigned int clip_to_outsm_clipped_prim                                   :  1;
        unsigned int clip_to_outsm_null_primitive                                 :  1;
        unsigned int clip_ga_bc_fifo_full                                         :  1;
        unsigned int clip_to_ga_fifo_full                                         :  1;
        unsigned int clip_ga_bc_fifo_write                                        :  1;
        unsigned int clip_to_ga_fifo_write                                        :  1;
        unsigned int clip_to_outsm_fifo_advanceread                               :  1;
        unsigned int clip_to_outsm_fifo_empty                                     :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG03_0 {
    struct {
        unsigned int clipsm0_clprim_to_clip_clip_code_or                          :  6;
        unsigned int clipsm0_clprim_to_clip_event_id                              :  6;
        unsigned int clipsm0_clprim_to_clip_state_var_indx                        :  3;
        unsigned int clipsm0_clprim_to_clip_clip_primitive                        :  1;
        unsigned int clipsm0_clprim_to_clip_deallocate_slot                       :  3;
        unsigned int clipsm0_clprim_to_clip_first_prim_of_slot                    :  1;
        unsigned int clipsm0_clprim_to_clip_end_of_packet                         :  1;
        unsigned int clipsm0_clprim_to_clip_event                                 :  1;
        unsigned int clipsm0_clprim_to_clip_null_primitive                        :  1;
        unsigned int clipsm0_clprim_to_clip_prim_valid                            :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG03_1 {
    struct {
        unsigned int clipsm0_clprim_to_clip_clip_code_or                          : 14;
        unsigned int clipsm0_clprim_to_clip_event_id                              :  2;
        unsigned int clipsm0_clprim_to_clip_deallocate_slot                       :  3;
        unsigned int clipsm0_clprim_to_clip_first_prim_of_slot                    :  1;
        unsigned int clipsm0_clprim_to_clip_end_of_packet                         :  1;
        unsigned int clipsm0_clprim_to_clip_event                                 :  1;
        unsigned int clipsm0_clprim_to_clip_null_primitive                        :  1;
        unsigned int clipsm0_clprim_to_clip_prim_valid                            :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG04_0 {
    struct {
        unsigned int clipsm0_clprim_to_clip_param_cache_indx_0                    :  3;
        unsigned int clipsm0_clprim_to_clip_vertex_store_indx_2                   :  6;
        unsigned int clipsm0_clprim_to_clip_vertex_store_indx_1                   :  6;
        unsigned int clipsm0_clprim_to_clip_vertex_store_indx_0                   :  6;
        unsigned int clipsm0_clprim_to_clip_event                                 :  1;
        unsigned int clipsm0_clprim_to_clip_null_primitive                        :  1;
        unsigned int clipsm0_clprim_to_clip_prim_valid                            :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG04_1 {
    struct {
        unsigned int clipsm0_clprim_to_clip_param_cache_indx_0                    : 11;
        unsigned int clipsm0_clprim_to_clip_vertex_store_indx_2                   :  5;
        unsigned int clipsm0_clprim_to_clip_vertex_store_indx_0                   :  5;
        unsigned int clipsm0_clprim_to_clip_event                                 :  1;
        unsigned int clipsm0_clprim_to_clip_null_primitive                        :  1;
        unsigned int clipsm0_clprim_to_clip_prim_valid                            :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG05_0 {
    struct {
        unsigned int clipsm1_clprim_to_clip_clip_code_or                          :  6;
        unsigned int clipsm1_clprim_to_clip_event_id                              :  6;
        unsigned int clipsm1_clprim_to_clip_state_var_indx                        :  3;
        unsigned int clipsm1_clprim_to_clip_clip_primitive                        :  1;
        unsigned int clipsm1_clprim_to_clip_deallocate_slot                       :  3;
        unsigned int clipsm1_clprim_to_clip_first_prim_of_slot                    :  1;
        unsigned int clipsm1_clprim_to_clip_end_of_packet                         :  1;
        unsigned int clipsm1_clprim_to_clip_event                                 :  1;
        unsigned int clipsm1_clprim_to_clip_null_primitive                        :  1;
        unsigned int clipsm1_clprim_to_clip_prim_valid                            :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG05_1 {
    struct {
        unsigned int clipsm1_clprim_to_clip_clip_code_or                          : 14;
        unsigned int clipsm1_clprim_to_clip_event_id                              :  2;
        unsigned int clipsm1_clprim_to_clip_deallocate_slot                       :  3;
        unsigned int clipsm1_clprim_to_clip_first_prim_of_slot                    :  1;
        unsigned int clipsm1_clprim_to_clip_end_of_packet                         :  1;
        unsigned int clipsm1_clprim_to_clip_event                                 :  1;
        unsigned int clipsm1_clprim_to_clip_null_primitive                        :  1;
        unsigned int clipsm1_clprim_to_clip_prim_valid                            :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG06_0 {
    struct {
        unsigned int clipsm1_clprim_to_clip_param_cache_indx_0                    :  3;
        unsigned int clipsm1_clprim_to_clip_vertex_store_indx_2                   :  6;
        unsigned int clipsm1_clprim_to_clip_vertex_store_indx_1                   :  6;
        unsigned int clipsm1_clprim_to_clip_vertex_store_indx_0                   :  6;
        unsigned int clipsm1_clprim_to_clip_event                                 :  1;
        unsigned int clipsm1_clprim_to_clip_null_primitive                        :  1;
        unsigned int clipsm1_clprim_to_clip_prim_valid                            :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG06_1 {
    struct {
        unsigned int clipsm1_clprim_to_clip_param_cache_indx_0                    : 11;
        unsigned int clipsm1_clprim_to_clip_vertex_store_indx_2                   :  5;
        unsigned int clipsm1_clprim_to_clip_vertex_store_indx_0                   :  5;
        unsigned int clipsm1_clprim_to_clip_event                                 :  1;
        unsigned int clipsm1_clprim_to_clip_null_primitive                        :  1;
        unsigned int clipsm1_clprim_to_clip_prim_valid                            :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG07_0 {
    struct {
        unsigned int clipsm2_clprim_to_clip_clip_code_or                          :  6;
        unsigned int clipsm2_clprim_to_clip_event_id                              :  6;
        unsigned int clipsm2_clprim_to_clip_state_var_indx                        :  3;
        unsigned int clipsm2_clprim_to_clip_clip_primitive                        :  1;
        unsigned int clipsm2_clprim_to_clip_deallocate_slot                       :  3;
        unsigned int clipsm2_clprim_to_clip_first_prim_of_slot                    :  1;
        unsigned int clipsm2_clprim_to_clip_end_of_packet                         :  1;
        unsigned int clipsm2_clprim_to_clip_event                                 :  1;
        unsigned int clipsm2_clprim_to_clip_null_primitive                        :  1;
        unsigned int clipsm2_clprim_to_clip_prim_valid                            :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG07_1 {
    struct {
        unsigned int clipsm2_clprim_to_clip_clip_code_or                          : 14;
        unsigned int clipsm2_clprim_to_clip_event_id                              :  2;
        unsigned int clipsm2_clprim_to_clip_deallocate_slot                       :  3;
        unsigned int clipsm2_clprim_to_clip_first_prim_of_slot                    :  1;
        unsigned int clipsm2_clprim_to_clip_end_of_packet                         :  1;
        unsigned int clipsm2_clprim_to_clip_event                                 :  1;
        unsigned int clipsm2_clprim_to_clip_null_primitive                        :  1;
        unsigned int clipsm2_clprim_to_clip_prim_valid                            :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG08_0 {
    struct {
        unsigned int clipsm2_clprim_to_clip_param_cache_indx_0                    :  3;
        unsigned int clipsm2_clprim_to_clip_vertex_store_indx_2                   :  6;
        unsigned int clipsm2_clprim_to_clip_vertex_store_indx_1                   :  6;
        unsigned int clipsm2_clprim_to_clip_vertex_store_indx_0                   :  6;
        unsigned int clipsm2_clprim_to_clip_event                                 :  1;
        unsigned int clipsm2_clprim_to_clip_null_primitive                        :  1;
        unsigned int clipsm2_clprim_to_clip_prim_valid                            :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG08_1 {
    struct {
        unsigned int clipsm2_clprim_to_clip_param_cache_indx_0                    : 11;
        unsigned int clipsm2_clprim_to_clip_vertex_store_indx_2                   :  5;
        unsigned int clipsm2_clprim_to_clip_vertex_store_indx_0                   :  5;
        unsigned int clipsm2_clprim_to_clip_event                                 :  1;
        unsigned int clipsm2_clprim_to_clip_null_primitive                        :  1;
        unsigned int clipsm2_clprim_to_clip_prim_valid                            :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG09_0 {
    struct {
        unsigned int clipsm3_clprim_to_clip_clip_code_or                          :  6;
        unsigned int clipsm3_clprim_to_clip_event_id                              :  6;
        unsigned int clipsm3_clprim_to_clip_state_var_indx                        :  3;
        unsigned int clipsm3_clprim_to_clip_clip_primitive                        :  1;
        unsigned int clipsm3_clprim_to_clip_deallocate_slot                       :  3;
        unsigned int clipsm3_clprim_to_clip_first_prim_of_slot                    :  1;
        unsigned int clipsm3_clprim_to_clip_end_of_packet                         :  1;
        unsigned int clipsm3_clprim_to_clip_event                                 :  1;
        unsigned int clipsm3_clprim_to_clip_null_primitive                        :  1;
        unsigned int clipsm3_clprim_to_clip_prim_valid                            :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG09_1 {
    struct {
        unsigned int clipsm3_clprim_to_clip_clip_code_or                          : 14;
        unsigned int clipsm3_clprim_to_clip_event_id                              :  2;
        unsigned int clipsm3_clprim_to_clip_deallocate_slot                       :  3;
        unsigned int clipsm3_clprim_to_clip_first_prim_of_slot                    :  1;
        unsigned int clipsm3_clprim_to_clip_end_of_packet                         :  1;
        unsigned int clipsm3_clprim_to_clip_event                                 :  1;
        unsigned int clipsm3_clprim_to_clip_null_primitive                        :  1;
        unsigned int clipsm3_clprim_to_clip_prim_valid                            :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG10_0 {
    struct {
        unsigned int clipsm3_clprim_to_clip_param_cache_indx_0                    :  3;
        unsigned int clipsm3_clprim_to_clip_vertex_store_indx_2                   :  6;
        unsigned int clipsm3_clprim_to_clip_vertex_store_indx_1                   :  6;
        unsigned int clipsm3_clprim_to_clip_vertex_store_indx_0                   :  6;
        unsigned int clipsm3_clprim_to_clip_event                                 :  1;
        unsigned int clipsm3_clprim_to_clip_null_primitive                        :  1;
        unsigned int clipsm3_clprim_to_clip_prim_valid                            :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG10_1 {
    struct {
        unsigned int clipsm3_clprim_to_clip_param_cache_indx_0                    : 11;
        unsigned int clipsm3_clprim_to_clip_vertex_store_indx_2                   :  5;
        unsigned int clipsm3_clprim_to_clip_vertex_store_indx_0                   :  5;
        unsigned int clipsm3_clprim_to_clip_event                                 :  1;
        unsigned int clipsm3_clprim_to_clip_null_primitive                        :  1;
        unsigned int                                                              :  9;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG11_0 {
    struct {
        unsigned int clipsm3_clip_to_clipga_clip_to_outsm_cnt                     :  4;
        unsigned int clipsm2_clip_to_clipga_clip_to_outsm_cnt                     :  4;
        unsigned int clipsm1_clip_to_clipga_clip_to_outsm_cnt                     :  4;
        unsigned int clipsm0_clip_to_clipga_clip_to_outsm_cnt                     :  4;
        unsigned int clipsm3_clip_to_clipga_prim_valid                            :  1;
        unsigned int clipsm2_clip_to_clipga_prim_valid                            :  1;
        unsigned int clipsm1_clip_to_clipga_prim_valid                            :  1;
        unsigned int clipsm0_clip_to_clipga_prim_valid                            :  1;
        unsigned int clipsm3_inc_clip_to_clipga_clip_to_outsm_cnt                 :  1;
        unsigned int clipsm2_inc_clip_to_clipga_clip_to_outsm_cnt                 :  1;
        unsigned int clipsm1_inc_clip_to_clipga_clip_to_outsm_cnt                 :  1;
        unsigned int clipsm0_inc_clip_to_clipga_clip_to_outsm_cnt                 :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG11_1 {
    struct {
        unsigned int clipsm3_clip_to_clipga_event                                 :  1;
        unsigned int clipsm2_clip_to_clipga_event                                 :  1;
        unsigned int clipsm1_clip_to_clipga_event                                 :  1;
        unsigned int clipsm0_clip_to_clipga_event                                 :  1;
        unsigned int clipsm3_clip_to_clipga_clip_primitive                        :  1;
        unsigned int clipsm2_clip_to_clipga_clip_primitive                        :  1;
        unsigned int clipsm1_clip_to_clipga_clip_primitive                        :  1;
        unsigned int clipsm0_clip_to_clipga_clip_primitive                        :  1;
        unsigned int clipsm3_clip_to_clipga_clip_to_outsm_cnt                     :  4;
        unsigned int clipsm2_clip_to_clipga_clip_to_outsm_cnt                     :  4;
        unsigned int clipsm3_clip_to_clipga_prim_valid                            :  1;
        unsigned int clipsm2_clip_to_clipga_prim_valid                            :  1;
        unsigned int clipsm1_clip_to_clipga_prim_valid                            :  1;
        unsigned int clipsm0_clip_to_clipga_prim_valid                            :  1;
        unsigned int clipsm3_inc_clip_to_clipga_clip_to_outsm_cnt                 :  1;
        unsigned int clipsm2_inc_clip_to_clipga_clip_to_outsm_cnt                 :  1;
        unsigned int clipsm1_inc_clip_to_clipga_clip_to_outsm_cnt                 :  1;
        unsigned int clipsm0_inc_clip_to_clipga_clip_to_outsm_cnt                 :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG12_0 {
    struct {
        unsigned int clip_priority_available_vte_out_clip                         :  5;
        unsigned int clip_priority_available_clip_verts                           :  5;
        unsigned int clip_priority_seq_indx_out                                   :  2;
        unsigned int clip_priority_seq_indx_vert                                  :  2;
        unsigned int clip_priority_seq_indx_load                                  :  2;
        unsigned int clipsm3_clprim_to_clip_clip_primitive                        :  1;
        unsigned int clipsm3_clprim_to_clip_prim_valid                            :  1;
        unsigned int clipsm2_clprim_to_clip_clip_primitive                        :  1;
        unsigned int clipsm2_clprim_to_clip_prim_valid                            :  1;
        unsigned int clipsm1_clprim_to_clip_clip_primitive                        :  1;
        unsigned int clipsm1_clprim_to_clip_prim_valid                            :  1;
        unsigned int clipsm0_clprim_to_clip_clip_primitive                        :  1;
        unsigned int clipsm0_clprim_to_clip_prim_valid                            :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG12_1 {
    struct {
        unsigned int ALWAYS_ZERO                                                  :  8;
        unsigned int clip_priority_available_vte_out_clip                         :  5;
        unsigned int clip_priority_available_clip_verts                           :  3;
        unsigned int clipsm3_clprim_to_clip_clip_primitive                        :  1;
        unsigned int clipsm3_clprim_to_clip_prim_valid                            :  1;
        unsigned int clipsm2_clprim_to_clip_clip_primitive                        :  1;
        unsigned int clipsm2_clprim_to_clip_prim_valid                            :  1;
        unsigned int clipsm1_clprim_to_clip_clip_primitive                        :  1;
        unsigned int clipsm1_clprim_to_clip_prim_valid                            :  1;
        unsigned int clipsm0_clprim_to_clip_clip_primitive                        :  1;
        unsigned int clipsm0_clprim_to_clip_prim_valid                            :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG13_0 {
    struct {
        unsigned int vertval_bits_vertex_cc_next_valid                            :  4;
        unsigned int clipcc_vertex_store_indx                                     :  2;
        unsigned int vte_out_orig_fifo_fifo_empty                                 :  1;
        unsigned int clipcode_fifo_fifo_empty                                     :  1;
        unsigned int ccgen_to_clipcc_fifo_empty                                   :  1;
        unsigned int clip_priority_seq_indx_out_cnt                               :  4;
        unsigned int outsm_clr_rd_orig_vertices                                   :  2;
        unsigned int outsm_clr_rd_clipsm_wait                                     :  1;
        unsigned int outsm_clr_fifo_contents                                      :  5;
        unsigned int outsm_clr_fifo_full                                          :  1;
        unsigned int outsm_clr_fifo_advanceread                                   :  1;
        unsigned int outsm_clr_fifo_write                                         :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG13_1 {
    struct {
        unsigned int clprim_in_back_state_var_indx                                :  3;
        unsigned int point_clip_candidate                                         :  1;
        unsigned int prim_nan_kill                                                :  1;
        unsigned int clprim_clip_primitive                                        :  1;
        unsigned int clprim_cull_primitive                                        :  1;
        unsigned int prim_back_valid                                              :  1;
        unsigned int vertval_bits_vertex_cc_next_valid                            :  4;
        unsigned int clipcc_vertex_store_indx                                     :  2;
        unsigned int vte_out_orig_fifo_fifo_empty                                 :  1;
        unsigned int clipcode_fifo_fifo_empty                                     :  1;
        unsigned int outsm_clr_fifo_contents                                      :  5;
        unsigned int outsm_clr_fifo_full                                          :  1;
        unsigned int outsm_clr_fifo_advanceread                                   :  1;
        unsigned int outsm_clr_fifo_write                                         :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG14_0 {
    struct {
        unsigned int clprim_in_back_vertex_store_indx_1                           :  4;
        unsigned int clprim_in_back_vertex_store_indx_0                           :  6;
        unsigned int outputclprimtoclip_null_primitive                            :  1;
        unsigned int clprim_in_back_end_of_packet                                 :  1;
        unsigned int clprim_in_back_first_prim_of_slot                            :  1;
        unsigned int clprim_in_back_deallocate_slot                               :  3;
        unsigned int clprim_in_back_event_id                                      :  6;
        unsigned int clprim_in_back_event                                         :  1;
        unsigned int prim_back_valid                                              :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG14_1 {
    struct {
        unsigned int clprim_in_back_vertex_store_indx_2                           :  6;
        unsigned int clprim_in_back_vertex_store_indx_1                           :  6;
        unsigned int clprim_in_back_vertex_store_indx_0                           :  4;
        unsigned int clprim_in_back_event_id                                      :  6;
        unsigned int clprim_in_back_event                                         :  1;
        unsigned int prim_back_valid                                              :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG15_0 {
    struct {
        unsigned int vertval_bits_vertex_vertex_store_msb                         :  8;
        unsigned int primic_to_clprim_fifo_vertex_store_indx_2                    :  5;
        unsigned int primic_to_clprim_fifo_vertex_store_indx_1                    :  5;
        unsigned int primic_to_clprim_fifo_vertex_store_indx_0                    :  5;
        unsigned int primic_to_clprim_valid                                       :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG15_1 {
    struct {
        unsigned int vertval_bits_vertex_vertex_store_msb                         : 16;
        unsigned int primic_to_clprim_fifo_vertex_store_indx_1                    :  2;
        unsigned int primic_to_clprim_fifo_vertex_store_indx_0                    :  5;
        unsigned int primic_to_clprim_valid                                       :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG16_0 {
    struct {
        unsigned int sm0_clip_vert_cnt                                            :  5;
        unsigned int sm0_vertex_clip_cnt                                          :  5;
        unsigned int sm0_inv_to_clip_data_valid_1                                 :  1;
        unsigned int sm0_inv_to_clip_data_valid_0                                 :  1;
        unsigned int sm0_current_state                                            :  7;
        unsigned int sm0_clip_to_clipga_clip_to_outsm_cnt_eq0                     :  1;
        unsigned int sm0_clip_to_outsm_fifo_full                                  :  1;
        unsigned int sm0_highest_priority_seq                                     :  1;
        unsigned int sm0_outputcliptoga_0                                         :  1;
        unsigned int sm0_clprim_to_clip_prim_valid                                :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG16_1 {
    struct {
        unsigned int sm0_prim_end_state                                           :  7;
        unsigned int sm0_ps_expand                                                :  1;
        unsigned int sm0_clip_vert_cnt                                            :  5;
        unsigned int sm0_vertex_clip_cnt                                          :  3;
        unsigned int sm0_current_state                                            :  3;
        unsigned int sm0_clip_to_clipga_clip_to_outsm_cnt_eq0                     :  1;
        unsigned int sm0_clip_to_outsm_fifo_full                                  :  1;
        unsigned int sm0_highest_priority_seq                                     :  1;
        unsigned int sm0_outputcliptoga_0                                         :  1;
        unsigned int sm0_clprim_to_clip_prim_valid                                :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG17_0 {
    struct {
        unsigned int sm1_clip_vert_cnt                                            :  5;
        unsigned int sm1_vertex_clip_cnt                                          :  5;
        unsigned int sm1_inv_to_clip_data_valid_1                                 :  1;
        unsigned int sm1_inv_to_clip_data_valid_0                                 :  1;
        unsigned int sm1_current_state                                            :  7;
        unsigned int sm1_clip_to_clipga_clip_to_outsm_cnt_eq0                     :  1;
        unsigned int sm1_clip_to_outsm_fifo_full                                  :  1;
        unsigned int sm1_highest_priority_seq                                     :  1;
        unsigned int sm1_outputcliptoga_0                                         :  1;
        unsigned int sm1_clprim_to_clip_prim_valid                                :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG17_1 {
    struct {
        unsigned int sm1_prim_end_state                                           :  7;
        unsigned int sm1_ps_expand                                                :  1;
        unsigned int sm1_clip_vert_cnt                                            :  5;
        unsigned int sm1_vertex_clip_cnt                                          :  3;
        unsigned int sm1_current_state                                            :  3;
        unsigned int sm1_clip_to_clipga_clip_to_outsm_cnt_eq0                     :  1;
        unsigned int sm1_clip_to_outsm_fifo_full                                  :  1;
        unsigned int sm1_highest_priority_seq                                     :  1;
        unsigned int sm1_outputcliptoga_0                                         :  1;
        unsigned int sm1_clprim_to_clip_prim_valid                                :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG18_0 {
    struct {
        unsigned int sm2_clip_vert_cnt                                            :  5;
        unsigned int sm2_vertex_clip_cnt                                          :  5;
        unsigned int sm2_inv_to_clip_data_valid_1                                 :  1;
        unsigned int sm2_inv_to_clip_data_valid_0                                 :  1;
        unsigned int sm2_current_state                                            :  7;
        unsigned int sm2_clip_to_clipga_clip_to_outsm_cnt_eq0                     :  1;
        unsigned int sm2_clip_to_outsm_fifo_full                                  :  1;
        unsigned int sm2_highest_priority_seq                                     :  1;
        unsigned int sm2_outputcliptoga_0                                         :  1;
        unsigned int sm2_clprim_to_clip_prim_valid                                :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG18_1 {
    struct {
        unsigned int sm2_prim_end_state                                           :  7;
        unsigned int sm2_ps_expand                                                :  1;
        unsigned int sm2_clip_vert_cnt                                            :  5;
        unsigned int sm2_vertex_clip_cnt                                          :  3;
        unsigned int sm2_current_state                                            :  3;
        unsigned int sm2_clip_to_clipga_clip_to_outsm_cnt_eq0                     :  1;
        unsigned int sm2_clip_to_outsm_fifo_full                                  :  1;
        unsigned int sm2_highest_priority_seq                                     :  1;
        unsigned int sm2_outputcliptoga_0                                         :  1;
        unsigned int sm2_clprim_to_clip_prim_valid                                :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG19_0 {
    struct {
        unsigned int sm3_clip_vert_cnt                                            :  5;
        unsigned int sm3_vertex_clip_cnt                                          :  5;
        unsigned int sm3_inv_to_clip_data_valid_1                                 :  1;
        unsigned int sm3_inv_to_clip_data_valid_0                                 :  1;
        unsigned int sm3_current_state                                            :  7;
        unsigned int sm3_clip_to_clipga_clip_to_outsm_cnt_eq0                     :  1;
        unsigned int sm3_clip_to_outsm_fifo_full                                  :  1;
        unsigned int sm3_highest_priority_seq                                     :  1;
        unsigned int sm3_outputcliptoga_0                                         :  1;
        unsigned int sm3_clprim_to_clip_prim_valid                                :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG19_1 {
    struct {
        unsigned int sm3_prim_end_state                                           :  7;
        unsigned int sm3_ps_expand                                                :  1;
        unsigned int sm3_clip_vert_cnt                                            :  5;
        unsigned int sm3_vertex_clip_cnt                                          :  3;
        unsigned int sm3_current_state                                            :  3;
        unsigned int sm3_clip_to_clipga_clip_to_outsm_cnt_eq0                     :  1;
        unsigned int sm3_clip_to_outsm_fifo_full                                  :  1;
        unsigned int sm3_highest_priority_seq                                     :  1;
        unsigned int sm3_outputcliptoga_0                                         :  1;
        unsigned int sm3_clprim_to_clip_prim_valid                                :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG20_0 {
    struct {
        unsigned int point_address                                                :  1;
        unsigned int sx_pending_rd_state_var_indx                                 :  3;
        unsigned int sx_pending_rd_req_mask                                       :  4;
        unsigned int sx_pending_rd_pci                                            : 10;
        unsigned int sx_pending_rd_aux_sel                                        :  2;
        unsigned int sx_pending_rd_sp_id                                          :  2;
        unsigned int sx_pending_rd_aux_inc                                        :  1;
        unsigned int sx_pending_rd_advance                                        :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG20_1 {
    struct {
        unsigned int position_address                                             :  6;
        unsigned int point_address                                                :  3;
        unsigned int sx_pending_rd_state_var_indx                                 :  3;
        unsigned int sx_pending_rd_req_mask                                       :  4;
        unsigned int sx_pending_rd_pci                                            :  2;
        unsigned int sx_pending_rd_aux_sel                                        :  2;
        unsigned int sx_pending_rd_sp_id                                          :  2;
        unsigned int sx_pending_rd_aux_inc                                        :  1;
        unsigned int sx_pending_rd_advance                                        :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG21_0 {
    struct {
        unsigned int sx_receive_indx                                              :  2;
        unsigned int sx_pending_fifo_contents                                     :  5;
        unsigned int statevar_bits_vs_out_misc_vec_ena                            :  1;
        unsigned int statevar_bits_disable_sp                                     :  4;
        unsigned int aux_sel                                                      :  2;
        unsigned int sx_to_pa_empty_1                                             :  1;
        unsigned int sx_to_pa_empty_0                                             :  1;
        unsigned int pasx_req_cnt_1                                               :  4;
        unsigned int pasx_req_cnt_0                                               :  4;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG21_1 {
    struct {
        unsigned int available_positions                                          :  7;
        unsigned int sx_receive_indx                                              :  3;
        unsigned int sx_pending_fifo_contents                                     :  5;
        unsigned int statevar_bits_vs_out_misc_vec_ena                            :  1;
        unsigned int pasx_req_cnt_1                                               :  4;
        unsigned int pasx_req_cnt_0                                               :  4;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG22_0 {
    struct {
        unsigned int su_aux                                                       :  1;
        unsigned int sx_request_indx                                              :  6;
        unsigned int req_active_verts_loaded                                      :  1;
        unsigned int req_active_verts                                             :  7;
        unsigned int vgt_to_ccgen_state_var_indx                                  :  3;
        unsigned int vgt_to_ccgen_active_verts                                    :  6;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG22_1 {
    struct {
        unsigned int param_cache_base                                             :  7;
        unsigned int su_aux                                                       :  2;
        unsigned int sx_request_indx                                              :  6;
        unsigned int req_active_verts_loaded                                      :  1;
        unsigned int vgt_to_ccgen_state_var_indx                                  :  2;
        unsigned int vgt_to_ccgen_active_verts                                    :  6;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG23_0 {
    struct {
        unsigned int su_baryc_cntl_state                                          :  2;
        unsigned int su_cntl_state                                                :  4;
        unsigned int ALWAYS_ZERO                                                  :  2;
        unsigned int pmode_state                                                  :  6;
        unsigned int ge_stallb                                                    :  1;
        unsigned int geom_enable                                                  :  1;
        unsigned int su_clip_baryc_free                                           :  2;
        unsigned int su_clip_rtr                                                  :  1;
        unsigned int pfifo_busy                                                   :  1;
        unsigned int su_cntl_busy                                                 :  1;
        unsigned int geom_busy                                                    :  1;
        unsigned int event_id_gated_1to0                                          :  2;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG24_0 {
    struct {
        unsigned int event_id_gated_5to2                                          :  4;
        unsigned int event_gated                                                  :  1;
        unsigned int pmode_prim_gated                                             :  1;
        unsigned int su_dyn_sclk_vld                                              :  1;
        unsigned int cl_dyn_sclk_vld                                              :  1;
        unsigned int y_sort0_gated_23_8                                           : 16;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG25_0 {
    struct {
        unsigned int x_sort0_gated_23_8                                           : 16;
        unsigned int y_sort1_gated_15_8                                           :  8;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG26_0 {
    struct {
        unsigned int y_sort1_gated_23_16                                          :  8;
        unsigned int x_sort1_gated_23_8                                           : 16;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG27_0 {
    struct {
        unsigned int y_sort2_gated_23_8                                           : 16;
        unsigned int x_sort2_gated_15_8                                           :  8;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG28_0 {
    struct {
        unsigned int x_sort2_gated_23to16                                         :  8;
        unsigned int attr_indx_sort0_gated                                        : 14;
        unsigned int null_prim_gated                                              :  1;
        unsigned int backfacing_gated                                             :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG29_0 {
    struct {
        unsigned int st_indx_gated                                                :  3;
        unsigned int clipped_gated                                                :  1;
        unsigned int dealloc_slot_gated                                           :  3;
        unsigned int xmajor_gated                                                 :  1;
        unsigned int diamond_rule_gated                                           :  2;
        unsigned int type_gated                                                   :  3;
        unsigned int fpov_gated                                                   :  2;
        unsigned int eop_gated                                                    :  1;
        unsigned int attr_indx_sort2_gated_7to0                                   :  8;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG30_0 {
    struct {
        unsigned int attr_indx_sort2_gated_13to8                                  :  6;
        unsigned int attr_indx_sort1_gated                                        : 14;
        unsigned int provoking_vtx_gated                                          :  2;
        unsigned int valid_prim_gated                                             :  1;
        unsigned int pa_reg_sclk_vld                                              :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG31_0 {
    struct {
        unsigned int VGT_PA_clipv_send_q                                          :  1;
        unsigned int VGT_PA_clips_send_q                                          :  1;
        unsigned int VGT_PA_clipp_send_q                                          :  1;
        unsigned int PA0_SC0_send_q                                               :  1;
        unsigned int PA0_SC1_send_q                                               :  1;
        unsigned int PA_PA_pascDataOut_send0_q                                    :  1;
        unsigned int PA_PA_pascDataOut_send1_q                                    :  1;
        unsigned int vte_busy                                                     :  1;
        unsigned int clipper_busy                                                 :  1;
        unsigned int su_busy                                                      :  1;
        unsigned int su_busy_debug_hold_busy                                      :  1;
        unsigned int su_busy_debug_valid_prim_gated                               :  1;
        unsigned int su_busy_debug_su_cntl_busy                                   :  1;
        unsigned int su_busy_debug_geom_busy                                      :  1;
        unsigned int su_busy_debug_pfifo_busy                                     :  1;
        unsigned int vte_busy_debug_veu_busy                                      :  1;
        unsigned int vte_busy_debug_ib_busy                                       :  1;
        unsigned int clipper_busy_debug_outsm_clr_fifo_not_empty                  :  1;
        unsigned int clipper_busy_debug_clipsm3_clprim_to_clip_prim_valid         :  1;
        unsigned int clipper_busy_debug_clipsm2_clprim_to_clip_prim_valid         :  1;
        unsigned int clipper_busy_debug_clipsm1_clprim_to_clip_prim_valid         :  1;
        unsigned int clipper_busy_debug_clipsm0_clprim_to_clip_prim_valid         :  1;
        unsigned int clipper_busy_debug_primic_to_clprim_valid                    :  1;
        unsigned int clipper_busy_debug_clipsm_clip_to_outsm_fifo_write           :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG32_0 {
    struct {
        unsigned int clipper_busy_debug_clipsm3_current_state_not_empty           :  1;
        unsigned int clipper_busy_debug_clipsm2_current_state_not_empty           :  1;
        unsigned int clipper_busy_debug_clipsm1_current_state_not_empty           :  1;
        unsigned int clipper_busy_debug_clipsm0_current_state_not_empty           :  1;
        unsigned int clipper_busy_debug_clip_to_ga_bc_busy                        :  1;
        unsigned int clipper_busy_debug_clip_to_ga_fifo_write                     :  1;
        unsigned int clipper_busy_debug_prim_back_valid                           :  1;
        unsigned int clipper_busy_debug_next_prim_back_valid                      :  1;
        unsigned int clipper_busy_debug_primic_to_clprim_fifo_not_empty           :  1;
        unsigned int clipper_busy_debug_primic_to_clprim_fifo_write               :  1;
        unsigned int clipper_busy_debug_sx_pending_fifo_not_empty                 :  1;
        unsigned int clipper_busy_debug_clip_to_outsm_fifo_busy                   :  1;
        unsigned int clipper_busy_debug_clip_to_outsm_fifo_write                  :  1;
        unsigned int clipper_busy_debug_clipcode_fifo_fifo_busy                   :  1;
        unsigned int clipper_busy_debug_clipcode_fifo_fifo_write                  :  1;
        unsigned int clipper_busy_debug_vte_out_clip_fifo_fifo_busy               :  1;
        unsigned int clipper_busy_debug_vte_out_clip_fifo_fifo_write              :  1;
        unsigned int clipper_busy_debug_vte_out_orig_fifo_fifo_busy               :  1;
        unsigned int clipper_busy_debug_vte_out_orig_fifo_fifo_write              :  1;
        unsigned int clipper_busy_debug_ccgen_to_clipcc_fifo_busy                 :  1;
        unsigned int clipper_busy_debug_ccgen_to_clipcc_fifo_write                :  1;
        unsigned int clipper_busy_debug_vgt_to_clipp_fifo_busy                    :  1;
        unsigned int clipper_busy_debug_vgt_to_clipp_fifo_write                   :  1;
        unsigned int clipper_busy_debug_vgt_to_clips_fifo_busy                    :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG33_0 {
    struct {
        unsigned int clipper_busy_debug_vgt_to_clips_fifo_write                   :  1;
        unsigned int                                                              :  3;
        unsigned int PA_SE3SC_freeze                                              :  1;
        unsigned int Reserved0                                                    : 27;
    } gfx09_0;
    struct {
        unsigned int clipper_busy_debug_vgt_to_clips_fifo_write                   :  1;
        unsigned int                                                              :  3;
        unsigned int PA_SE3SC_freeze                                              :  1;
        unsigned int Reserved0                                                    : 27;
    } rv2x;
    struct {
        unsigned int clipper_busy_debug_vgt_to_clips_fifo_write                   :  1;
        unsigned int                                                              :  3;
        unsigned int PA_SE3SC_freeze                                              :  1;
        unsigned int Reserved0                                                    : 27;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG34_0 {
    struct {
        unsigned int ngg_sideband_utc_driver_tag                                  :  8;
        unsigned int ngg_sideband_utc_driver_client_send_reg_rtr_out              :  1;
        unsigned int ngg_sideband_utc_driver_client_send_reg_rts_out              :  1;
        unsigned int ngg_sideband_utc_driver_client_send_reg_rts_in               :  1;
        unsigned int ngg_sideband_utc_driver_client_send_reg_rtr_in               :  1;
        unsigned int ngg_sideband_utc_driver_client_send_reg_busy                 :  1;
        unsigned int ngg_sideband_utc_driver_outcl1_send_valid                    :  1;
        unsigned int ngg_sideband_utc_driver_iutcl1_send_ready                    :  1;
        unsigned int ngg_sideband_utc_driver_iclient_send_valid                   :  1;
        unsigned int ngg_sideband_utc_driver_oclient_send_ready                   :  1;
        unsigned int ngg_sideband_utc_driver_outcl1_send_vmid                     :  4;
        unsigned int ngg_sideband_utc_driver_outcl1_send_type                     :  2;
        unsigned int Reserved0                                                    :  9;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG34_1 {
    struct {
        unsigned int ngg_utcl1_debug00                                            : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG35_0 {
    struct {
        unsigned int ngg_index_utc_driver_tag                                     :  8;
        unsigned int ngg_index_utc_driver_client_send_fifo_read                   :  1;
        unsigned int ngg_index_utc_driver_client_send_fifo_empty                  :  1;
        unsigned int ngg_index_utc_driver_client_send_fifo_write                  :  1;
        unsigned int ngg_index_utc_driver_client_send_fifo_full                   :  1;
        unsigned int ngg_index_utc_driver_client_send_fifo_busy                   :  1;
        unsigned int ngg_index_utc_driver_outcl1_send_valid                       :  1;
        unsigned int ngg_index_utc_driver_iutcl1_send_ready                       :  1;
        unsigned int ngg_index_utc_driver_iclient_send_valid                      :  1;
        unsigned int ngg_index_utc_driver_oclient_send_ready                      :  1;
        unsigned int ngg_index_utc_driver_outcl1_send_vmid                        :  4;
        unsigned int ngg_index_utc_driver_outcl1_send_type                        :  2;
        unsigned int Reserved0                                                    :  9;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG35_1 {
    struct {
        unsigned int ngg_utcl1_debug01                                            : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG36_0 {
    struct {
        unsigned int ngg_position_utc_driver_tag                                  :  8;
        unsigned int ngg_position_utc_driver_client_send_fifo_read                :  1;
        unsigned int ngg_position_utc_driver_client_send_fifo_empty               :  1;
        unsigned int ngg_position_utc_driver_client_send_fifo_write               :  1;
        unsigned int ngg_position_utc_driver_client_send_fifo_full                :  1;
        unsigned int ngg_position_utc_driver_client_send_fifo_busy                :  1;
        unsigned int ngg_position_utc_driver_outcl1_send_valid                    :  1;
        unsigned int ngg_position_utc_driver_iutcl1_send_ready                    :  1;
        unsigned int ngg_position_utc_driver_iclient_send_valid                   :  1;
        unsigned int ngg_position_utc_driver_oclient_send_ready                   :  1;
        unsigned int ngg_position_utc_driver_outcl1_send_vmid                     :  4;
        unsigned int ngg_position_utc_driver_outcl1_send_type                     :  2;
        unsigned int Reserved0                                                    :  9;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG36_1 {
    struct {
        unsigned int ngg_utcl1_debug02                                            : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG37_0 {
    struct {
        unsigned int ngg_debug_sideband_utc_receiver_busy                         :  1;
        unsigned int ngg_debug_sideband_utc_receiver_output_reg_rtr_out           :  1;
        unsigned int ngg_debug_sideband_utc_receiver_output_reg_rts_out           :  1;
        unsigned int ngg_debug_sideband_utc_receiver_output_reg_rtr_in            :  1;
        unsigned int ngg_debug_sideband_utc_receiver_output_reg_rts_in            :  1;
        unsigned int ngg_debug_sideband_utc_receiver_output_reg_dataout_drop      :  1;
        unsigned int ngg_debug_sideband_utc_receiver_output_reg_dataout_perf_cntr_en :  1;
        unsigned int ngg_debug_sideband_utc_receiver_output_reg_dataout_space     :  1;
        unsigned int ngg_debug_sideband_utc_receiver_output_reg_dataout_rd_tmz_encr :  1;
        unsigned int ngg_debug_sideband_utc_receiver_output_reg_dataout_io        :  1;
        unsigned int ngg_debug_sideband_utc_receiver_output_reg_dataout_snoop     :  1;
        unsigned int ngg_debug_sideband_utc_receiver_output_reg_dataout_mtype     :  3;
        unsigned int ngg_debug_sideband_utc_receiver_output_reg_dataout_vmid      :  4;
        unsigned int ngg_debug_sideband_utc_receiver_output_reg_dataout_tag       :  8;
        unsigned int Reserved0                                                    :  6;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG37_1 {
    struct {
        unsigned int ngg_utcl1_debug03                                            : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG38_0 {
    struct {
        unsigned int ngg_debug_index_utc_receiver_busy                            :  1;
        unsigned int ngg_debug_index_utc_receiver_output_reg_rtr_out              :  1;
        unsigned int ngg_debug_index_utc_receiver_output_reg_rts_out              :  1;
        unsigned int ngg_debug_index_utc_receiver_output_reg_rtr_in               :  1;
        unsigned int ngg_debug_index_utc_receiver_output_reg_rts_in               :  1;
        unsigned int ngg_debug_index_utc_receiver_output_reg_dataout_drop         :  1;
        unsigned int ngg_debug_index_utc_receiver_output_reg_dataout_perf_cntr_en :  1;
        unsigned int ngg_debug_index_utc_receiver_output_reg_dataout_space        :  1;
        unsigned int ngg_debug_index_utc_receiver_output_reg_dataout_rd_tmz_encr  :  1;
        unsigned int ngg_debug_index_utc_receiver_output_reg_dataout_io           :  1;
        unsigned int ngg_debug_index_utc_receiver_output_reg_dataout_snoop        :  1;
        unsigned int ngg_debug_index_utc_receiver_output_reg_dataout_mtype        :  3;
        unsigned int ngg_debug_index_utc_receiver_output_reg_dataout_vmid         :  4;
        unsigned int ngg_debug_index_utc_receiver_output_reg_dataout_tag          :  8;
        unsigned int Reserved0                                                    :  6;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG38_1 {
    struct {
        unsigned int ngg_utcl1_debug04                                            : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG39_0 {
    struct {
        unsigned int ngg_debug_position_utc_receiver_busy                         :  1;
        unsigned int ngg_debug_position_utc_receiver_output_reg_rtr_out           :  1;
        unsigned int ngg_debug_position_utc_receiver_output_reg_rts_out           :  1;
        unsigned int ngg_debug_position_utc_receiver_output_reg_rtr_in            :  1;
        unsigned int ngg_debug_position_utc_receiver_output_reg_rts_in            :  1;
        unsigned int ngg_debug_position_utc_receiver_output_reg_dataout_drop      :  1;
        unsigned int ngg_debug_position_utc_receiver_output_reg_dataout_perf_cntr_en :  1;
        unsigned int ngg_debug_position_utc_receiver_output_reg_dataout_space     :  1;
        unsigned int ngg_debug_position_utc_receiver_output_reg_dataout_rd_tmz_encr :  1;
        unsigned int ngg_debug_position_utc_receiver_output_reg_dataout_io        :  1;
        unsigned int ngg_debug_position_utc_receiver_output_reg_dataout_snoop     :  1;
        unsigned int ngg_debug_position_utc_receiver_output_reg_dataout_mtype     :  3;
        unsigned int ngg_debug_position_utc_receiver_output_reg_dataout_vmid      :  4;
        unsigned int ngg_debug_position_utc_receiver_output_reg_dataout_tag       :  8;
        unsigned int Reserved0                                                    :  6;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG39_1 {
    struct {
        unsigned int ngg_utcl1_debug05                                            : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG40_0 {
    struct {
        unsigned int ngg_debug_tc_request_arbiter_current_client_count            :  2;
        unsigned int                                                              : 20;
        unsigned int ngg_debug_tc_request_arbiter_istream0_client_tag             :  8;
        unsigned int ngg_debug_tc_request_arbiter_ostream0_client_rtr             :  1;
        unsigned int ngg_debug_tc_request_arbiter_istream0_client_rts             :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG40_1 {
    struct {
        unsigned int ngg_utcl1_debug06                                            : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG41_0 {
    struct {
        unsigned int ngg_debug_tc_if_credit_count_overflow                        :  1;
        unsigned int ngg_debug_tc_if_credit_count_underflow                       :  1;
        unsigned int ngg_debug_tc_if_credits                                      :  5;
        unsigned int ngg_debug_tc_if_obusy                                        :  1;
        unsigned int ngg_debug_tc_if_ooutstanding_tc_requests_exist               :  1;
        unsigned int ngg_debug_tc_if_irdret_vld                                   :  1;
        unsigned int ngg_debug_tc_if_irdreq_free                                  :  1;
        unsigned int ngg_debug_tc_if_ordreq_send                                  :  1;
        unsigned int ngg_debug_tc_if_ordreq_clken                                 :  1;
        unsigned int ngg_debug_tc_if_ortr_in                                      :  1;
        unsigned int ngg_debug_tc_if_irts_in                                      :  1;
        unsigned int ngg_debug_tc_if_reg_valid1                                   :  1;
        unsigned int ngg_debug_tc_if_reg_valid0                                   :  1;
        unsigned int ngg_debug_tc_if_data_rd_ptr                                  :  1;
        unsigned int ngg_debug_tc_if_data_wr_ptr                                  :  1;
        unsigned int ngg_debug_tc_if_outstanding_tc_requests                      : 10;
        unsigned int Reserved0                                                    :  3;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG41_1 {
    struct {
        unsigned int ngg_utcl1_debug07                                            : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG42_0 {
    struct {
        unsigned int ngg_debug_sideband_sideband_tokens_not_max_value             :  1;
        unsigned int ngg_debug_sideband_pending_subgroup_count_not_zero           :  1;
        unsigned int ngg_debug_sideband_wait_counter_busy                         :  1;
        unsigned int ngg_debug_sideband_current_state                             :  3;
        unsigned int ngg_debug_sideband_sideband_tokens_underflow                 :  1;
        unsigned int ngg_debug_sideband_sideband_tokens_overflow                  :  1;
        unsigned int ngg_debug_sideband_current_dword_pointer                     :  4;
        unsigned int ngg_debug_sideband_pending_subgroup_count_range              :  5;
        unsigned int ngg_debug_sideband_valid_dword_count                         :  5;
        unsigned int ngg_debug_sideband_itc_return_empty                          :  1;
        unsigned int                                                              :  1;
        unsigned int ngg_debug_sideband_current_sideband_event_bit_set            :  1;
        unsigned int ngg_debug_sideband_have_available_sideband_tokens            :  1;
        unsigned int Reserved0                                                    :  6;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG42_1 {
    struct {
        unsigned int ngg_utcl1_debug08                                            : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG43_0 {
    struct {
        unsigned int ngg_debug_sideband_wdif_ovmid                                :  4;
        unsigned int ngg_debug_sideband_wdif_vmid_fifo_busy                       :  1;
        unsigned int ngg_debug_sideband_wdif_vmid_fifo_empty                      :  1;
        unsigned int ngg_debug_sideband_wdif_vmid_fifo_full                       :  1;
        unsigned int ngg_debug_sideband_wdif_sideband_pop_bit_fifo_empty          :  1;
        unsigned int ngg_debug_sideband_wdif_sideband_pop_bit_fifo_busy           :  1;
        unsigned int ngg_debug_sideband_wdif_sideband_pop_bit_fifo_full           :  1;
        unsigned int ngg_debug_sideband_wdif_current_state                        :  1;
        unsigned int ngg_debug_sideband_wdif_osideband_active                     :  1;
        unsigned int Reserved0                                                    : 20;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG43_1 {
    struct {
        unsigned int ngg_utcl1_debug09                                            : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG44_0 {
    struct {
        unsigned int ngg_debug_sideband_memory_contents1_underflow                :  1;
        unsigned int ngg_debug_sideband_memory_contents1_overflow                 :  1;
        unsigned int ngg_debug_sideband_memory_contents0_underflow                :  1;
        unsigned int ngg_debug_sideband_memory_contents0_overflow                 :  1;
        unsigned int ngg_debug_sideband_memory_contents0                          :  8;
        unsigned int ngg_debug_sideband_memory_contents1                          :  8;
        unsigned int Reserved0                                                    : 12;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG44_1 {
    struct {
        unsigned int ngg_utcl1_debug10                                            : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG45_0 {
    struct {
        unsigned int ngg_debug_index_index_tokens_underflow                       :  1;
        unsigned int ngg_debug_index_index_tokens_overflow                        :  1;
        unsigned int ngg_debug_index_index_tokens                                 :  6;
        unsigned int ngg_debug_index_prim_indices_fifo_busy                       :  1;
        unsigned int ngg_debug_index_index_receive_fifo_busy                      :  1;
        unsigned int ngg_debug_index_current_receive_state                        :  1;
        unsigned int ngg_debug_index_current_fetch_state                          :  1;
        unsigned int ngg_debug_index_sideband_data_context_id_bits                :  3;
        unsigned int ngg_debug_index_sideband_data_eop_bit                        :  1;
        unsigned int ngg_debug_index_sideband_data_event_bit                      :  1;
        unsigned int ngg_debug_index_final_receive_dword_of_subgroup              :  1;
        unsigned int ngg_debug_index_final_receive_cacheline_this_subgroup        :  1;
        unsigned int ngg_debug_index_current_receive_first_dword_pointer          :  4;
        unsigned int ngg_debug_index_current_receive_finalt_dword_pointer         :  4;
        unsigned int ngg_debug_index_index_receive_fifo_empty                     :  1;
        unsigned int ngg_debug_index_index_receive_fifo_full                      :  1;
        unsigned int ngg_debug_index_have_available_index_tokens                  :  1;
        unsigned int Reserved0                                                    :  2;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG45_1 {
    struct {
        unsigned int ngg_utcl1_debug11                                            : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG46_0 {
    struct {
        unsigned int ngg_debug_position_request_position_tokens_underflow         :  1;
        unsigned int ngg_debug_position_request_position_tokens_overflow          :  1;
        unsigned int ngg_debug_position_request_have_available_position_tokens    :  1;
        unsigned int ngg_debug_position_request_reuse_busy                        :  1;
        unsigned int ngg_debug_position_request_prev_select_end_cacheline_address :  1;
        unsigned int ngg_debug_position_request_prev_pos_cacheline_valid          :  1;
        unsigned int ngg_debug_position_request_active_pos_req_fosg               :  1;
        unsigned int ngg_debug_position_request_reuse_miss_count                  :  2;
        unsigned int ngg_debug_position_request_reuse_final_prim                  :  1;
        unsigned int ngg_debug_position_request_reuse_first_prim                  :  1;
        unsigned int ngg_debug_position_request_reuse_null_prim                   :  1;
        unsigned int ngg_debug_position_request_reuse_prim_valid                  :  1;
        unsigned int                                                              :  1;
        unsigned int ngg_debug_position_request_packer_sideband_valid             :  1;
        unsigned int ngg_debug_position_request_current_state                     :  2;
        unsigned int ngg_debug_position_request_ipa_to_wd_dealloc_index_full      :  1;
        unsigned int ngg_debug_position_request_iposreq_to_posrtn_s_fifo_full     :  1;
        unsigned int ngg_debug_position_request_iposreq_to_posrtn_v_fifo_full     :  1;
        unsigned int ngg_debug_position_request_ifetch_to_primic_s_fifo_full      :  1;
        unsigned int ngg_debug_position_request_ifetch_to_primic_p_fifo_full      :  1;
        unsigned int ngg_debug_position_request_another_pos_fetch                 :  1;
        unsigned int ngg_debug_position_request_end_prev_diff                     :  1;
        unsigned int ngg_debug_position_request_start_prev_diff                   :  1;
        unsigned int ngg_debug_position_request_end_start_diff                    :  1;
        unsigned int Reserved0                                                    :  6;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG46_1 {
    struct {
        unsigned int ngg_utcl1_debug12                                            : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG47_0 {
    struct {
        unsigned int ngg_debug_position_reuse_sideband_fifo_full                  :  1;
        unsigned int ngg_debug_position_reuse_sideband_fifo_empty                 :  1;
        unsigned int ngg_debug_position_reuse_sideband_fifo_busy                  :  1;
        unsigned int ngg_debug_position_reuse_prim_fifo_full                      :  1;
        unsigned int ngg_debug_position_reuse_output_prim_empty                   :  1;
        unsigned int ngg_debug_position_reuse_prim_fifo_busy                      :  1;
        unsigned int ngg_debug_position_reuse_current_state                       :  1;
        unsigned int ngg_debug_position_reuse_iinput_sideband_valid               :  1;
        unsigned int ngg_debug_position_reuse_input_sideband_dword_stream_id      :  2;
        unsigned int ngg_debug_position_reuse_input_sideband_dword_eop_bit        :  1;
        unsigned int ngg_debug_position_reuse_input_sideband_dword_event_id       :  6;
        unsigned int ngg_debug_position_reuse_input_sideband_dword_event_second_cycle :  1;
        unsigned int ngg_debug_position_reuse_input_sideband_dword_event_bit      :  1;
        unsigned int ngg_debug_position_reuse_input_sideband_dword_state_id       :  3;
        unsigned int Reserved0                                                    : 10;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG47_1 {
    struct {
        unsigned int ngg_utcl1_debug13                                            : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG48_0 {
    struct {
        unsigned int ngg_debug_position_return_ifetch_to_sxif_fifo_full           :  1;
        unsigned int ngg_debug_position_return_ipa_to_wd_dealloc_position_full    :  1;
        unsigned int ngg_debug_position_return_itc_return_empty                   :  1;
        unsigned int ngg_debug_position_return_oposreq_to_posrtn_v_fifo_full      :  1;
        unsigned int ngg_debug_position_return_oposreq_to_posrtn_s_fifo_full      :  1;
        unsigned int ngg_debug_position_return_posreq_to_posrtn_v_fifo_busy       :  1;
        unsigned int ngg_debug_position_return_posreq_to_posrtn_s_fifo_busy       :  1;
        unsigned int ngg_debug_position_return_posreq_to_posrtn_v_fifo_empty      :  1;
        unsigned int ngg_debug_position_return_posreq_to_posrtn_s_fifo_empty      :  1;
        unsigned int ngg_debug_position_return_active_pos_return_state            :  2;
        unsigned int ngg_debug_position_return_cache_pos_index                    :  2;
        unsigned int ngg_debug_position_return_active_pos_return_first_vert       :  1;
        unsigned int ngg_debug_position_return_active_pos_return_cache_pos_index  :  2;
        unsigned int Reserved0                                                    : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG48_1 {
    struct {
        unsigned int ngg_utcl1_debug14                                            : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG49_0 {
    struct {
        unsigned int ngg_debug_tc_return_if_memreg0_busy                          :  1;
        unsigned int ngg_debug_tc_return_if_oreturn_empty_stream0                 :  1;
        unsigned int ngg_debug_tc_return_if_return_data_read_reg_full_stream0     :  1;
        unsigned int ngg_debug_tc_return_if_memreg1_busy                          :  1;
        unsigned int ngg_debug_tc_return_if_oreturn_empty_stream1                 :  1;
        unsigned int ngg_debug_tc_return_if_return_data_read_reg_full_stream1     :  1;
        unsigned int ngg_debug_tc_return_if_memreg2_busy                          :  1;
        unsigned int ngg_debug_tc_return_if_oreturn_empty_stream2                 :  1;
        unsigned int ngg_debug_tc_return_if_return_data_read_reg_full_stream2     :  1;
        unsigned int ngg_debug_tc_return_if_crawler_busy                          :  1;
        unsigned int Reserved0                                                    : 22;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG49_1 {
    struct {
        unsigned int ngg_utcl1_debug15                                            : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG50_0 {
    struct {
        unsigned int ngg_debug_return_crawler_encountered_error_writing_to_busy_location_stream0 :  1;
        unsigned int ngg_debug_return_crawler_encountered_error_writing_to_busy_location_stream1 :  1;
        unsigned int ngg_debug_return_crawler_encountered_error_writing_to_busy_location_stream2 :  1;
        unsigned int ngg_debug_return_crawler_memory_data_valid_stream0_count     :  6;
        unsigned int ngg_debug_return_crawler_memory_data_valid_stream1_count     :  6;
        unsigned int ngg_debug_return_crawler_memory_data_valid_stream2_count     :  8;
        unsigned int ngg_debug_return_crawler_obusy                               :  1;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG50_1 {
    struct {
        unsigned int ngg_utcl1_debug16                                            : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG51_0 {
    struct {
        unsigned int ngg_debug_wd_dealloc_pa_to_wd_dealloc_fifo_read              :  1;
        unsigned int ngg_debug_wd_dealloc_pa_to_wd_dealloc_fifo_empty             :  1;
        unsigned int ngg_debug_wd_dealloc_opa_to_wd_dealloc_position_full         :  1;
        unsigned int ngg_debug_wd_dealloc_opa_to_wd_dealloc_index_full            :  1;
        unsigned int ngg_debug_wd_dealloc_opa_to_wd_dealloc_fifo_busy             :  1;
        unsigned int ngg_debug_wd_dealloc_opa_to_wd_dealloc_amount                : 11;
        unsigned int ngg_debug_wd_dealloc_opa_to_wd_dealloc_state_var_index       :  3;
        unsigned int ngg_debug_wd_dealloc_opa_to_wd_dealloc_type                  :  1;
        unsigned int Reserved0                                                    : 12;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG52_0 {
    struct {
        unsigned int ngg_debug_shootdown_control_iutcl1_busy                      :  1;
        unsigned int ngg_debug_shootdown_control_iposition_requester_idle         :  1;
        unsigned int ngg_debug_shootdown_control_iindex_requester_idle            :  1;
        unsigned int ngg_debug_shootdown_control_isideband_requester_idle         :  1;
        unsigned int ngg_debug_shootdown_control_iposition_utcl1_return_busy      :  1;
        unsigned int ngg_debug_shootdown_control_iindex_utcl1_return_busy         :  1;
        unsigned int ngg_debug_shootdown_control_isideband_utcl1_return_busy      :  1;
        unsigned int ngg_debug_shootdown_control_iposition_utcl1_send_busy        :  1;
        unsigned int ngg_debug_shootdown_control_iindex_utcl1_send_busy           :  1;
        unsigned int ngg_debug_shootdown_control_isideband_utcl1_send_busy        :  1;
        unsigned int ngg_debug_shootdown_control_ioutstanding_tc_requests_exist   :  1;
        unsigned int ngg_debug_shootdown_control_all_clean                        :  1;
        unsigned int ngg_debug_shootdown_control_vmid_count                       :  4;
        unsigned int ngg_debug_shootdown_control_oclean                           :  1;
        unsigned int ngg_debug_shootdown_control_current_state                    :  3;
        unsigned int Reserved0                                                    : 12;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG53_0 {
    struct {
        unsigned int ngg_debug_fatal_events_status_fatal_tc_credit_count_overflow :  1;
        unsigned int ngg_debug_fatal_events_status_fatal_tc_credit_count_underflow :  1;
        unsigned int ngg_debug_fatal_events_status_fatal_sideband_tokens_overflow :  1;
        unsigned int ngg_debug_fatal_events_status_fatal_sideband_tokens_underflow :  1;
        unsigned int ngg_debug_fatal_events_status_fatal_sbmem_contents0_overflow :  1;
        unsigned int ngg_debug_fatal_events_status_fatal_sbmem_contents0_underflow :  1;
        unsigned int ngg_debug_fatal_events_status_fatal_sbmem_contents1_overflow :  1;
        unsigned int ngg_debug_fatal_events_status_fatal_sbmem_contents1_underflow :  1;
        unsigned int ngg_debug_fatal_events_status_fatal_index_tokens_overflow    :  1;
        unsigned int ngg_debug_fatal_events_status_fatal_index_tokens_underflow   :  1;
        unsigned int ngg_debug_fatal_events_status_fatal_position_tokens_overflow :  1;
        unsigned int ngg_debug_fatal_events_status_fatal_position_tokens_underflow :  1;
        unsigned int ngg_debug_fatal_events_status_fatal_writing_to_busy_location_stream0 :  1;
        unsigned int ngg_debug_fatal_events_status_fatal_writing_to_busy_location_stream1 :  1;
        unsigned int ngg_debug_fatal_events_status_fatal_writing_to_busy_location_stream2 :  1;
        unsigned int Reserved0                                                    : 17;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG54_0 {
    struct {
        unsigned int ngg_debug_fifo_full_status_fifo_sideband_pop_bit_fifo_full   :  1;
        unsigned int ngg_debug_fifo_full_status_fifo_vmid_fifo_full               :  1;
        unsigned int ngg_debug_fifo_full_status_fifo_sbmem_full                   :  1;
        unsigned int ngg_debug_fifo_full_status_fifo_index_receive_fifo_full      :  1;
        unsigned int ngg_debug_fifo_full_status_fifo_prim_indices_fifo_full       :  1;
        unsigned int ngg_debug_fifo_full_status_fifo_prim_fifo_full               :  1;
        unsigned int ngg_debug_fifo_full_status_fifo_sideband_fifo_full           :  1;
        unsigned int ngg_debug_fifo_full_status_fifo_posreq_to_posrtn_v_fifo_full :  1;
        unsigned int ngg_debug_fifo_full_status_fifo_posreq_to_posrtn_s_fifo_full :  1;
        unsigned int ngg_debug_fifo_full_status_fifo_return_data_full_stream0     :  1;
        unsigned int ngg_debug_fifo_full_status_fifo_return_data_full_stream1     :  1;
        unsigned int ngg_debug_fifo_full_status_fifo_return_data_full_stream2     :  1;
        unsigned int ngg_debug_fifo_full_status_fifo_pa_to_wd_dealloc_index_full  :  1;
        unsigned int ngg_debug_fifo_full_status_fifo_pa_to_wd_dealloc_position_full :  1;
        unsigned int Reserved0                                                    : 18;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_DEBUG55_0 {
    struct {
        unsigned int ngg_debug_fifo_empty_status_fifo_sideband_pop_bit_fifo_empty :  1;
        unsigned int ngg_debug_fifo_empty_status_fifo_vmid_fifo_empty             :  1;
        unsigned int ngg_debug_fifo_empty_status_fifo_sbmem_empty                 :  1;
        unsigned int ngg_debug_fifo_empty_status_fifo_index_receive_fifo_empty    :  1;
        unsigned int ngg_debug_fifo_empty_status_fifo_prim_indices_fifo_empty     :  1;
        unsigned int ngg_debug_fifo_empty_status_fifo_prim_fifo_empty             :  1;
        unsigned int ngg_debug_fifo_empty_status_fifo_sideband_fifo_empty         :  1;
        unsigned int ngg_debug_fifo_empty_status_fifo_posreq_to_posrtn_v_fifo_empty :  1;
        unsigned int ngg_debug_fifo_empty_status_fifo_posreq_to_posrtn_s_fifo_empty :  1;
        unsigned int ngg_debug_fifo_empty_status_fifo_return_data_empty_stream0   :  1;
        unsigned int ngg_debug_fifo_empty_status_fifo_return_data_empty_stream1   :  1;
        unsigned int ngg_debug_fifo_empty_status_fifo_return_data_empty_stream2   :  1;
        unsigned int ngg_debug_fifo_empty_status_fifo_pa_to_wd_dealloc_fifo_empty :  1;
        unsigned int Reserved0                                                    : 19;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_ENHANCE {
    struct {
        unsigned int ECO_SPARE0                                                   :  1;
        unsigned int ECO_SPARE1                                                   :  1;
        unsigned int ECO_SPARE2                                                   :  1;
        unsigned int ECO_SPARE3                                                   :  1;
        unsigned int DISABLE_PH_SC_INTF_FINE_CLOCK_GATE                           :  1;
        unsigned int DISABLE_FOPKT                                                :  1;
        unsigned int DISABLE_FOPKT_SCAN_POST_RESET                                :  1;
        unsigned int DISABLE_PH_SC_INTF_CLKEN_CLOCK_GATE                          :  1;
        unsigned int                                                              :  1;
        unsigned int DISABLE_PH_PERF_REG_FGCG                                     :  1;
        unsigned int ENABLE_PH_INTF_CLKEN_STRETCH                                 :  3;
        unsigned int                                                              : 19;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_INTERFACE_FIFO_SIZE {
    struct {
        unsigned int PA_PH_IF_FIFO_SIZE                                           : 10;
        unsigned int                                                              :  6;
        unsigned int PH_SC_IF_FIFO_SIZE                                           :  6;
        unsigned int                                                              : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER1_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER2_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER3_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER4_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER4_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER4_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER5_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER5_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER5_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER6_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER6_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER6_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER7_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER7_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER7_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_AA_CONFIG {
    struct {
        unsigned int MSAA_NUM_SAMPLES                                             :  3;
        unsigned int                                                              :  1;
        unsigned int AA_MASK_CENTROID_DTMN                                        :  1;
        unsigned int                                                              :  8;
        unsigned int MAX_SAMPLE_DIST                                              :  4;
        unsigned int                                                              :  3;
        unsigned int MSAA_EXPOSED_SAMPLES                                         :  3;
        unsigned int                                                              :  1;
        unsigned int DETAIL_TO_EXPOSED_MODE                                       :  2;
        unsigned int COVERAGE_TO_SHADER_SELECT                                    :  2;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_AA_MASK_X0Y0_X1Y0 {
    struct {
        unsigned int AA_MASK_X0Y0                                                 : 16;
        unsigned int AA_MASK_X1Y0                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_AA_MASK_X0Y1_X1Y1 {
    struct {
        unsigned int AA_MASK_X0Y1                                                 : 16;
        unsigned int AA_MASK_X1Y1                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_AA_SAMPLE_LOCS_PIXEL_X0Y0_0 {
    struct {
        unsigned int S0_X                                                         :  4;
        unsigned int S0_Y                                                         :  4;
        unsigned int S1_X                                                         :  4;
        unsigned int S1_Y                                                         :  4;
        unsigned int S2_X                                                         :  4;
        unsigned int S2_Y                                                         :  4;
        unsigned int S3_X                                                         :  4;
        unsigned int S3_Y                                                         :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_AA_SAMPLE_LOCS_PIXEL_X0Y0_1 {
    struct {
        unsigned int S4_X                                                         :  4;
        unsigned int S4_Y                                                         :  4;
        unsigned int S5_X                                                         :  4;
        unsigned int S5_Y                                                         :  4;
        unsigned int S6_X                                                         :  4;
        unsigned int S6_Y                                                         :  4;
        unsigned int S7_X                                                         :  4;
        unsigned int S7_Y                                                         :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_AA_SAMPLE_LOCS_PIXEL_X0Y0_2 {
    struct {
        unsigned int S8_X                                                         :  4;
        unsigned int S8_Y                                                         :  4;
        unsigned int S9_X                                                         :  4;
        unsigned int S9_Y                                                         :  4;
        unsigned int S10_X                                                        :  4;
        unsigned int S10_Y                                                        :  4;
        unsigned int S11_X                                                        :  4;
        unsigned int S11_Y                                                        :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_AA_SAMPLE_LOCS_PIXEL_X0Y0_3 {
    struct {
        unsigned int S12_X                                                        :  4;
        unsigned int S12_Y                                                        :  4;
        unsigned int S13_X                                                        :  4;
        unsigned int S13_Y                                                        :  4;
        unsigned int S14_X                                                        :  4;
        unsigned int S14_Y                                                        :  4;
        unsigned int S15_X                                                        :  4;
        unsigned int S15_Y                                                        :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_AA_SAMPLE_LOCS_PIXEL_X0Y1_0 {
    struct {
        unsigned int S0_X                                                         :  4;
        unsigned int S0_Y                                                         :  4;
        unsigned int S1_X                                                         :  4;
        unsigned int S1_Y                                                         :  4;
        unsigned int S2_X                                                         :  4;
        unsigned int S2_Y                                                         :  4;
        unsigned int S3_X                                                         :  4;
        unsigned int S3_Y                                                         :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_AA_SAMPLE_LOCS_PIXEL_X0Y1_1 {
    struct {
        unsigned int S4_X                                                         :  4;
        unsigned int S4_Y                                                         :  4;
        unsigned int S5_X                                                         :  4;
        unsigned int S5_Y                                                         :  4;
        unsigned int S6_X                                                         :  4;
        unsigned int S6_Y                                                         :  4;
        unsigned int S7_X                                                         :  4;
        unsigned int S7_Y                                                         :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_AA_SAMPLE_LOCS_PIXEL_X0Y1_2 {
    struct {
        unsigned int S8_X                                                         :  4;
        unsigned int S8_Y                                                         :  4;
        unsigned int S9_X                                                         :  4;
        unsigned int S9_Y                                                         :  4;
        unsigned int S10_X                                                        :  4;
        unsigned int S10_Y                                                        :  4;
        unsigned int S11_X                                                        :  4;
        unsigned int S11_Y                                                        :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_AA_SAMPLE_LOCS_PIXEL_X0Y1_3 {
    struct {
        unsigned int S12_X                                                        :  4;
        unsigned int S12_Y                                                        :  4;
        unsigned int S13_X                                                        :  4;
        unsigned int S13_Y                                                        :  4;
        unsigned int S14_X                                                        :  4;
        unsigned int S14_Y                                                        :  4;
        unsigned int S15_X                                                        :  4;
        unsigned int S15_Y                                                        :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_AA_SAMPLE_LOCS_PIXEL_X1Y0_0 {
    struct {
        unsigned int S0_X                                                         :  4;
        unsigned int S0_Y                                                         :  4;
        unsigned int S1_X                                                         :  4;
        unsigned int S1_Y                                                         :  4;
        unsigned int S2_X                                                         :  4;
        unsigned int S2_Y                                                         :  4;
        unsigned int S3_X                                                         :  4;
        unsigned int S3_Y                                                         :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_AA_SAMPLE_LOCS_PIXEL_X1Y0_1 {
    struct {
        unsigned int S4_X                                                         :  4;
        unsigned int S4_Y                                                         :  4;
        unsigned int S5_X                                                         :  4;
        unsigned int S5_Y                                                         :  4;
        unsigned int S6_X                                                         :  4;
        unsigned int S6_Y                                                         :  4;
        unsigned int S7_X                                                         :  4;
        unsigned int S7_Y                                                         :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_AA_SAMPLE_LOCS_PIXEL_X1Y0_2 {
    struct {
        unsigned int S8_X                                                         :  4;
        unsigned int S8_Y                                                         :  4;
        unsigned int S9_X                                                         :  4;
        unsigned int S9_Y                                                         :  4;
        unsigned int S10_X                                                        :  4;
        unsigned int S10_Y                                                        :  4;
        unsigned int S11_X                                                        :  4;
        unsigned int S11_Y                                                        :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_AA_SAMPLE_LOCS_PIXEL_X1Y0_3 {
    struct {
        unsigned int S12_X                                                        :  4;
        unsigned int S12_Y                                                        :  4;
        unsigned int S13_X                                                        :  4;
        unsigned int S13_Y                                                        :  4;
        unsigned int S14_X                                                        :  4;
        unsigned int S14_Y                                                        :  4;
        unsigned int S15_X                                                        :  4;
        unsigned int S15_Y                                                        :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_AA_SAMPLE_LOCS_PIXEL_X1Y1_0 {
    struct {
        unsigned int S0_X                                                         :  4;
        unsigned int S0_Y                                                         :  4;
        unsigned int S1_X                                                         :  4;
        unsigned int S1_Y                                                         :  4;
        unsigned int S2_X                                                         :  4;
        unsigned int S2_Y                                                         :  4;
        unsigned int S3_X                                                         :  4;
        unsigned int S3_Y                                                         :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_AA_SAMPLE_LOCS_PIXEL_X1Y1_1 {
    struct {
        unsigned int S4_X                                                         :  4;
        unsigned int S4_Y                                                         :  4;
        unsigned int S5_X                                                         :  4;
        unsigned int S5_Y                                                         :  4;
        unsigned int S6_X                                                         :  4;
        unsigned int S6_Y                                                         :  4;
        unsigned int S7_X                                                         :  4;
        unsigned int S7_Y                                                         :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_AA_SAMPLE_LOCS_PIXEL_X1Y1_2 {
    struct {
        unsigned int S8_X                                                         :  4;
        unsigned int S8_Y                                                         :  4;
        unsigned int S9_X                                                         :  4;
        unsigned int S9_Y                                                         :  4;
        unsigned int S10_X                                                        :  4;
        unsigned int S10_Y                                                        :  4;
        unsigned int S11_X                                                        :  4;
        unsigned int S11_Y                                                        :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_AA_SAMPLE_LOCS_PIXEL_X1Y1_3 {
    struct {
        unsigned int S12_X                                                        :  4;
        unsigned int S12_Y                                                        :  4;
        unsigned int S13_X                                                        :  4;
        unsigned int S13_Y                                                        :  4;
        unsigned int S14_X                                                        :  4;
        unsigned int S14_Y                                                        :  4;
        unsigned int S15_X                                                        :  4;
        unsigned int S15_Y                                                        :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_BC_WAVE_BREAK {
    struct {
        unsigned int MAX_DEALLOCS_IN_WAVE                                         : 11;
        unsigned int                                                              :  5;
        unsigned int MAX_FPOVS_IN_WAVE                                            :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_BINNER_CNTL_0 {
    struct {
        unsigned int BINNING_MODE                                                 :  2;
        unsigned int BIN_SIZE_X                                                   :  1;
        unsigned int BIN_SIZE_Y                                                   :  1;
        unsigned int BIN_SIZE_X_EXTEND                                            :  3;
        unsigned int BIN_SIZE_Y_EXTEND                                            :  3;
        unsigned int CONTEXT_STATES_PER_BIN                                       :  3;
        unsigned int PERSISTENT_STATES_PER_BIN                                    :  5;
        unsigned int DISABLE_START_OF_PRIM                                        :  1;
        unsigned int FPOVS_PER_BATCH                                              :  8;
        unsigned int OPTIMAL_BIN_SELECTION                                        :  1;
        unsigned int                                                              :  4;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 28;
        unsigned int FLUSH_ON_BINNING_TRANSITION                                  :  1;
        unsigned int                                                              :  3;
    } gfx09_1xPlus;
    struct {
        unsigned int                                                              : 29;
        unsigned int BIN_MAPPING_MODE                                             :  2;
        unsigned int                                                              :  1;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_BINNER_CNTL_1 {
    struct {
        unsigned int MAX_ALLOC_COUNT                                              : 16;
        unsigned int MAX_PRIM_PER_BATCH                                           : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_BINNER_CNTL_OVERRIDE {
    struct {
        unsigned int BINNING_MODE                                                 :  2;
        unsigned int                                                              :  8;
        unsigned int CONTEXT_STATES_PER_BIN                                       :  3;
        unsigned int PERSISTENT_STATES_PER_BIN                                    :  5;
        unsigned int                                                              :  1;
        unsigned int FPOVS_PER_BATCH                                              :  8;
        unsigned int DIRECT_OVERRIDE_MODE                                         :  1;
        unsigned int OVERRIDE                                                     :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_BINNER_EVENT_CNTL_0 {
    struct {
        unsigned int RESERVED_0                                                   :  2;
        unsigned int SAMPLE_STREAMOUTSTATS1                                       :  2;
        unsigned int SAMPLE_STREAMOUTSTATS2                                       :  2;
        unsigned int SAMPLE_STREAMOUTSTATS3                                       :  2;
        unsigned int CACHE_FLUSH_TS                                               :  2;
        unsigned int CONTEXT_DONE                                                 :  2;
        unsigned int CACHE_FLUSH                                                  :  2;
        unsigned int CS_PARTIAL_FLUSH                                             :  2;
        unsigned int VGT_STREAMOUT_SYNC                                           :  2;
        unsigned int RESERVED_9                                                   :  2;
        unsigned int VGT_STREAMOUT_RESET                                          :  2;
        unsigned int END_OF_PIPE_INCR_DE                                          :  2;
        unsigned int END_OF_PIPE_IB_END                                           :  2;
        unsigned int RST_PIX_CNT                                                  :  2;
        unsigned int BREAK_BATCH                                                  :  2;
        unsigned int VS_PARTIAL_FLUSH                                             :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_BINNER_EVENT_CNTL_1 {
    struct {
        unsigned int PS_PARTIAL_FLUSH                                             :  2;
        unsigned int FLUSH_HS_OUTPUT                                              :  2;
        unsigned int FLUSH_DFSM                                                   :  2;
        unsigned int RESET_TO_LOWEST_VGT                                          :  2;
        unsigned int CACHE_FLUSH_AND_INV_TS_EVENT                                 :  2;
        unsigned int ZPASS_DONE                                                   :  2;
        unsigned int CACHE_FLUSH_AND_INV_EVENT                                    :  2;
        unsigned int PERFCOUNTER_START                                            :  2;
        unsigned int PERFCOUNTER_STOP                                             :  2;
        unsigned int PIPELINESTAT_START                                           :  2;
        unsigned int PIPELINESTAT_STOP                                            :  2;
        unsigned int PERFCOUNTER_SAMPLE                                           :  2;
        unsigned int FLUSH_ES_OUTPUT                                              :  2;
        unsigned int                                                              :  2;
        unsigned int SAMPLE_PIPELINESTAT                                          :  2;
        unsigned int SO_VGTSTREAMOUT_FLUSH                                        :  2;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 26;
        unsigned int FLUSH_GS_OUTPUT                                              :  2;
        unsigned int                                                              :  4;
    } gfx09;
    struct {
        unsigned int                                                              : 26;
        unsigned int BIN_CONF_OVERRIDE_CHECK                                      :  2;
        unsigned int                                                              :  4;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_BINNER_EVENT_CNTL_2 {
    struct {
        unsigned int SAMPLE_STREAMOUTSTATS                                        :  2;
        unsigned int RESET_VTX_CNT                                                :  2;
        unsigned int BLOCK_CONTEXT_DONE                                           :  2;
        unsigned int                                                              :  2;
        unsigned int VGT_FLUSH                                                    :  2;
        unsigned int TGID_ROLLOVER                                                :  2;
        unsigned int SQ_NON_EVENT                                                 :  2;
        unsigned int SC_SEND_DB_VPZ                                               :  2;
        unsigned int BOTTOM_OF_PIPE_TS                                            :  2;
        unsigned int                                                              :  2;
        unsigned int DB_CACHE_FLUSH_AND_INV                                       :  2;
        unsigned int FLUSH_AND_INV_DB_DATA_TS                                     :  2;
        unsigned int FLUSH_AND_INV_DB_META                                        :  2;
        unsigned int FLUSH_AND_INV_CB_DATA_TS                                     :  2;
        unsigned int FLUSH_AND_INV_CB_META                                        :  2;
        unsigned int CS_DONE                                                      :  2;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  6;
        unsigned int CS_CONTEXT_DONE                                              :  2;
        unsigned int                                                              : 10;
        unsigned int FLUSH_SX_TS                                                  :  2;
        unsigned int                                                              : 12;
    } gfx09;
    struct {
        unsigned int                                                              :  6;
        unsigned int RESERVED_35                                                  :  2;
        unsigned int                                                              : 10;
        unsigned int RESERVED_41                                                  :  2;
        unsigned int                                                              : 12;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_BINNER_EVENT_CNTL_3 {
    struct {
        unsigned int PS_DONE                                                      :  2;
        unsigned int FLUSH_AND_INV_CB_PIXEL_DATA                                  :  2;
        unsigned int                                                              :  2;
        unsigned int THREAD_TRACE_START                                           :  2;
        unsigned int THREAD_TRACE_STOP                                            :  2;
        unsigned int THREAD_TRACE_MARKER                                          :  2;
        unsigned int                                                              :  2;
        unsigned int THREAD_TRACE_FINISH                                          :  2;
        unsigned int PIXEL_PIPE_STAT_CONTROL                                      :  2;
        unsigned int PIXEL_PIPE_STAT_DUMP                                         :  2;
        unsigned int PIXEL_PIPE_STAT_RESET                                        :  2;
        unsigned int CONTEXT_SUSPEND                                              :  2;
        unsigned int OFFCHIP_HS_DEALLOC                                           :  2;
        unsigned int ENABLE_NGG_PIPELINE                                          :  2;
        unsigned int ENABLE_LEGACY_PIPELINE                                       :  2;
        unsigned int                                                              :  2;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  4;
        unsigned int SX_CB_RAT_ACK_REQUEST                                        :  2;
        unsigned int                                                              :  6;
        unsigned int THREAD_TRACE_FLUSH                                           :  2;
        unsigned int                                                              : 16;
        unsigned int RESERVED_63                                                  :  2;
    } gfx09;
    struct {
        unsigned int                                                              :  4;
        unsigned int RESERVED_50                                                  :  2;
        unsigned int                                                              :  6;
        unsigned int THREAD_TRACE_DRAW                                            :  2;
        unsigned int                                                              : 16;
        unsigned int DRAW_DONE                                                    :  2;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_BINNER_PERF_CNTL_0 {
    struct {
        unsigned int BIN_HIST_NUM_PRIMS_THRESHOLD                                 : 10;
        unsigned int BATCH_HIST_NUM_PRIMS_THRESHOLD                               : 10;
        unsigned int BIN_HIST_NUM_CONTEXT_THRESHOLD                               :  3;
        unsigned int BATCH_HIST_NUM_CONTEXT_THRESHOLD                             :  3;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_BINNER_PERF_CNTL_1 {
    struct {
        unsigned int BIN_HIST_NUM_PERSISTENT_STATE_THRESHOLD                      :  5;
        unsigned int BATCH_HIST_NUM_PERSISTENT_STATE_THRESHOLD                    :  5;
        unsigned int BATCH_HIST_NUM_TRIV_REJECTED_PRIMS_THRESHOLD                 : 16;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_BINNER_PERF_CNTL_2 {
    struct {
        unsigned int BATCH_HIST_NUM_ROWS_PER_PRIM_THRESHOLD                       : 11;
        unsigned int BATCH_HIST_NUM_COLUMNS_PER_ROW_THRESHOLD                     : 11;
        unsigned int                                                              : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_BINNER_PERF_CNTL_3 {
    struct {
        unsigned int BATCH_HIST_NUM_PS_WAVE_BREAKS_THRESHOLD                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_BINNER_TIMEOUT_COUNTER {
    struct {
        unsigned int THRESHOLD                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_CENTROID_PRIORITY_0 {
    struct {
        unsigned int DISTANCE_0                                                   :  4;
        unsigned int DISTANCE_1                                                   :  4;
        unsigned int DISTANCE_2                                                   :  4;
        unsigned int DISTANCE_3                                                   :  4;
        unsigned int DISTANCE_4                                                   :  4;
        unsigned int DISTANCE_5                                                   :  4;
        unsigned int DISTANCE_6                                                   :  4;
        unsigned int DISTANCE_7                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_CENTROID_PRIORITY_1 {
    struct {
        unsigned int DISTANCE_8                                                   :  4;
        unsigned int DISTANCE_9                                                   :  4;
        unsigned int DISTANCE_10                                                  :  4;
        unsigned int DISTANCE_11                                                  :  4;
        unsigned int DISTANCE_12                                                  :  4;
        unsigned int DISTANCE_13                                                  :  4;
        unsigned int DISTANCE_14                                                  :  4;
        unsigned int DISTANCE_15                                                  :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_CLIPRECT_0_BR {
    struct {
        unsigned int BR_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int BR_Y                                                         : 15;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_CLIPRECT_0_TL {
    struct {
        unsigned int TL_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int TL_Y                                                         : 15;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_CLIPRECT_1_BR {
    struct {
        unsigned int BR_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int BR_Y                                                         : 15;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_CLIPRECT_1_TL {
    struct {
        unsigned int TL_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int TL_Y                                                         : 15;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_CLIPRECT_2_BR {
    struct {
        unsigned int BR_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int BR_Y                                                         : 15;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_CLIPRECT_2_TL {
    struct {
        unsigned int TL_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int TL_Y                                                         : 15;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_CLIPRECT_3_BR {
    struct {
        unsigned int BR_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int BR_Y                                                         : 15;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_CLIPRECT_3_TL {
    struct {
        unsigned int TL_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int TL_Y                                                         : 15;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_CLIPRECT_RULE {
    struct {
        unsigned int CLIP_RULE                                                    : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_CONSERVATIVE_RASTERIZATION_CNTL {
    struct {
        unsigned int OVER_RAST_ENABLE                                             :  1;
        unsigned int OVER_RAST_SAMPLE_SELECT                                      :  4;
        unsigned int UNDER_RAST_ENABLE                                            :  1;
        unsigned int UNDER_RAST_SAMPLE_SELECT                                     :  4;
        unsigned int PBB_UNCERTAINTY_REGION_ENABLE                                :  1;
        unsigned int ZMM_TRI_EXTENT                                               :  1;
        unsigned int ZMM_TRI_OFFSET                                               :  1;
        unsigned int OVERRIDE_OVER_RAST_INNER_TO_NORMAL                           :  1;
        unsigned int OVERRIDE_UNDER_RAST_INNER_TO_NORMAL                          :  1;
        unsigned int DEGENERATE_OVERRIDE_INNER_TO_NORMAL_DISABLE                  :  1;
        unsigned int UNCERTAINTY_REGION_MODE                                      :  2;
        unsigned int OUTER_UNCERTAINTY_EDGERULE_OVERRIDE                          :  1;
        unsigned int INNER_UNCERTAINTY_EDGERULE_OVERRIDE                          :  1;
        unsigned int NULL_SQUAD_AA_MASK_ENABLE                                    :  1;
        unsigned int COVERAGE_AA_MASK_ENABLE                                      :  1;
        unsigned int PREZ_AA_MASK_ENABLE                                          :  1;
        unsigned int POSTZ_AA_MASK_ENABLE                                         :  1;
        unsigned int CENTROID_SAMPLE_OVERRIDE                                     :  1;
        unsigned int                                                              :  7;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 25;
        unsigned int UNCERTAINTY_REGION_MULT                                      :  2;
        unsigned int UNCERTAINTY_REGION_PBB_MULT                                  :  2;
        unsigned int                                                              :  3;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_DEBUG_DATA {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_DSM_CNTL {
    struct {
        unsigned int FORCE_EOV_REZ_0                                              :  1;
        unsigned int FORCE_EOV_REZ_1                                              :  1;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_EDGERULE {
    struct {
        unsigned int ER_TRI                                                       :  4;
        unsigned int ER_POINT                                                     :  4;
        unsigned int ER_RECT                                                      :  4;
        unsigned int ER_LINE_LR                                                   :  6;
        unsigned int ER_LINE_RL                                                   :  6;
        unsigned int ER_LINE_TB                                                   :  4;
        unsigned int ER_LINE_BT                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_ENHANCE {
    struct {
        unsigned int ENABLE_PA_SC_OUT_OF_ORDER                                    :  1;
        unsigned int DISABLE_SC_DB_TILE_FIX                                       :  1;
        unsigned int DISABLE_AA_MASK_FULL_FIX                                     :  1;
        unsigned int ENABLE_1XMSAA_SAMPLE_LOCATIONS                               :  1;
        unsigned int ENABLE_1XMSAA_SAMPLE_LOC_CENTROID                            :  1;
        unsigned int DISABLE_SCISSOR_FIX                                          :  1;
        unsigned int SEND_UNLIT_STILES_TO_PACKER                                  :  1;
        unsigned int DISABLE_DUALGRAD_PERF_OPTIMIZATION                           :  1;
        unsigned int DISABLE_SC_PROCESS_RESET_PRIM                                :  1;
        unsigned int DISABLE_SC_PROCESS_RESET_SUPERTILE                           :  1;
        unsigned int DISABLE_SC_PROCESS_RESET_TILE                                :  1;
        unsigned int DISABLE_PA_SC_GUIDANCE                                       :  1;
        unsigned int DISABLE_EOV_ALL_CTRL_ONLY_COMBINATIONS                       :  1;
        unsigned int ENABLE_MULTICYCLE_BUBBLE_FREEZE                              :  1;
        unsigned int DISABLE_OUT_OF_ORDER_PA_SC_GUIDANCE                          :  1;
        unsigned int ENABLE_OUT_OF_ORDER_POLY_MODE                                :  1;
        unsigned int DISABLE_OUT_OF_ORDER_EOP_SYNC_NULL_PRIMS_LAST                :  1;
        unsigned int DISABLE_OUT_OF_ORDER_THRESHOLD_SWITCHING                     :  1;
        unsigned int ENABLE_OUT_OF_ORDER_THRESHOLD_SWITCH_AT_EOPG_ONLY            :  1;
        unsigned int DISABLE_OUT_OF_ORDER_DESIRED_FIFO_EMPTY_SWITCHING            :  1;
        unsigned int DISABLE_OUT_OF_ORDER_SELECTED_FIFO_EMPTY_SWITCHING           :  1;
        unsigned int DISABLE_OUT_OF_ORDER_EMPTY_SWITCHING_HYSTERYSIS              :  1;
        unsigned int ENABLE_OUT_OF_ORDER_DESIRED_FIFO_IS_NEXT_FEID                :  1;
        unsigned int DISABLE_OOO_NO_EOPG_SKEW_DESIRED_FIFO_IS_CURRENT_FIFO        :  1;
        unsigned int OOO_DISABLE_EOP_ON_FIRST_LIVE_PRIM_HIT                       :  1;
        unsigned int OOO_DISABLE_EOPG_SKEW_THRESHOLD_SWITCHING                    :  1;
        unsigned int DISABLE_EOP_LINE_STIPPLE_RESET                               :  1;
        unsigned int DISABLE_VPZ_EOP_LINE_STIPPLE_RESET                           :  1;
        unsigned int IOO_DISABLE_SCAN_UNSELECTED_FIFOS_FOR_DUAL_GFX_RING_CHANGE   :  1;
        unsigned int OOO_USE_ABSOLUTE_FIFO_COUNT_IN_THRESHOLD_SWITCHING           :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_ENHANCE_1 {
    struct {
        unsigned int REALIGN_DQUADS_OVERRIDE_ENABLE                               :  1;
        unsigned int REALIGN_DQUADS_OVERRIDE                                      :  2;
        unsigned int DISABLE_SC_BINNING                                           :  1;
        unsigned int BYPASS_PBB                                                   :  1;
        unsigned int ECO_SPARE0                                                   :  1;
        unsigned int ECO_SPARE1                                                   :  1;
        unsigned int ECO_SPARE2                                                   :  1;
        unsigned int ECO_SPARE3                                                   :  1;
        unsigned int DISABLE_SC_PROCESS_RESET_PBB                                 :  1;
        unsigned int DISABLE_PBB_SCISSOR_OPT                                      :  1;
        unsigned int                                                              :  3;
        unsigned int DISABLE_SC_DB_TILE_INTF_FINE_CLOCK_GATE                      :  1;
        unsigned int DISABLE_SC_PIPELINE_RESET_LEGACY_MODE_TRANSITION             :  1;
        unsigned int DISABLE_PACKER_ODC_ENHANCE                                   :  1;
        unsigned int                                                              :  1;
        unsigned int OPTIMAL_BIN_SELECTION                                        :  1;
        unsigned int                                                              :  1;
        unsigned int DISABLE_PBB_CLK_OPTIMIZATION                                 :  1;
        unsigned int DISABLE_PBB_SCISSOR_CLK_OPTIMIZATION                         :  1;
        unsigned int DISABLE_PBB_BINNING_CLK_OPTIMIZATION                         :  1;
        unsigned int                                                              :  9;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 11;
        unsigned int ENABLE_DFSM_FLUSH_EVENT_TO_FLUSH_POPS_CAM                    :  1;
        unsigned int                                                              :  7;
        unsigned int DISABLE_FORCE_SOP_ALL_EVENTS                                 :  1;
        unsigned int                                                              : 12;
    } core;
    struct {
        unsigned int                                                              : 12;
        unsigned int DEBUG_PIXEL_PICKER_XY_UNPACK                                 :  1;
        unsigned int DISABLE_PACKER_GRAD_FDCE_ENHANCE                             :  1;
        unsigned int                                                              :  3;
        unsigned int ALLOW_SCALE_LINE_WIDTH_PAD_WITH_BINNING                      :  1;
        unsigned int                                                              : 14;
    } gfx09;
    struct {
        unsigned int                                                              : 23;
        unsigned int RSVD                                                         :  9;
    } gfx09_0;
    struct {
        unsigned int                                                              : 31;
        unsigned int RSVD                                                         :  1;
    } gfx09_1x;
    struct {
        unsigned int                                                              : 23;
        unsigned int DISABLE_INTF_CG                                              :  1;
        unsigned int IOO_DISABLE_EOP_ON_FIRST_LIVE_PRIM_HIT                       :  1;
        unsigned int DISABLE_SHADER_PROFILING_FOR_POWER                           :  1;
        unsigned int FLUSH_ON_BINNING_TRANSITION                                  :  1;
        unsigned int DISABLE_QUAD_PROC_FDCE_ENHANCE                               :  1;
        unsigned int DISABLE_SC_PS_PA_ARBITER_FIX                                 :  1;
        unsigned int DISABLE_SC_PS_PA_ARBITER_FIX_1                               :  1;
        unsigned int PASS_VPZ_EVENT_TO_SPI                                        :  1;
        unsigned int                                                              :  1;
    } gfx09_1xPlus;
    struct {
        unsigned int                                                              : 13;
        unsigned int DEBUG_PIXEL_PICKER_COUNT_PIXELS                              :  1;
        unsigned int                                                              : 18;
    } gfx10;
    struct {
        unsigned int                                                              : 12;
        unsigned int DEBUG_PIXEL_PICKER_XY_UNPACK                                 :  1;
        unsigned int                                                              :  4;
        unsigned int ALLOW_SCALE_LINE_WIDTH_PAD_WITH_BINNING                      :  1;
        unsigned int                                                              : 14;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_ENHANCE_2 {
    struct {
        unsigned int                                                              :  4;
        unsigned int ENABLE_LPOV_WAVE_BREAK                                       :  1;
        unsigned int ENABLE_FPOV_WAVE_BREAK                                       :  1;
        unsigned int                                                              :  1;
        unsigned int ENABLE_SC_SEND_DB_VPZ_FOR_EN_PRIM_PAYLOAD                    :  1;
        unsigned int DISABLE_BREAK_BATCH_ON_GFX_PIPE_SWITCH                       :  1;
        unsigned int DISABLE_FULL_TILE_WAVE_BREAK                                 :  1;
        unsigned int ENABLE_VPZ_INJECTION_BEFORE_NULL_PRIMS                       :  1;
        unsigned int PBB_TIMEOUT_THRESHOLD_MODE                                   :  1;
        unsigned int DISABLE_PACKER_GRAD_FDCE_ENHANCE                             :  1;
        unsigned int DISABLE_SC_SPI_INTF_EARLY_WAKEUP                             :  1;
        unsigned int DISABLE_SC_BCI_INTF_EARLY_WAKEUP                             :  1;
        unsigned int DISABLE_EXPOSED_GT_DETAIL_RATE_TILE_COV_ADJ                  :  1;
        unsigned int PBB_WARP_CLK_MAIN_CLK_WAKEUP                                 :  1;
        unsigned int PBB_MAIN_CLK_REG_BUSY_WAKEUP                                 :  1;
        unsigned int DISABLE_BREAK_BATCH_ON_GFX_PIPELINE_RESET                    :  1;
        unsigned int                                                              :  2;
        unsigned int DISABLE_SC_DBR_DATAPATH_FGCG                                 :  1;
        unsigned int                                                              :  1;
        unsigned int PROCESS_RESET_FORCE_STILE_MASK_TO_ZERO                       :  1;
        unsigned int DISABLE_PBB_EOP_INSERTION_FOR_MIXED_BINNING_AND_IMMEDIATE    :  1;
        unsigned int                                                              :  7;
    } gfx10;
    struct {
        unsigned int                                                              : 25;
        unsigned int DISABLE_DFSM_FLUSH                                           :  1;
        unsigned int BREAK_WHEN_ONE_NULL_PRIM_BATCH                               :  1;
        unsigned int NULL_PRIM_BREAK_BATCH_LIMIT                                  :  3;
        unsigned int                                                              :  2;
    } gfx101Plus;
    struct {
        unsigned int                                                              : 20;
        unsigned int ENABLE_BLOCKING_WRITES_OF_GEN2_REG                           :  1;
        unsigned int                                                              : 11;
    } gfx10Core;
    struct {
        unsigned int ECO_SPARE0                                                   :  1;
        unsigned int ECO_SPARE1                                                   :  1;
        unsigned int ECO_SPARE2                                                   :  1;
        unsigned int ECO_SPARE3                                                   :  1;
        unsigned int                                                              : 26;
        unsigned int RSVD                                                         :  2;
    } nv10;
    struct {
        unsigned int RESERVED_0                                                   :  1;
        unsigned int RESERVED_1                                                   :  1;
        unsigned int RESERVED_2                                                   :  1;
        unsigned int RESERVED_3                                                   :  1;
        unsigned int RESERVED_4                                                   :  1;
        unsigned int RESERVED_5                                                   :  1;
        unsigned int                                                              :  2;
        unsigned int RSVD                                                         : 24;
    } vg12;
    struct {
        unsigned int RESERVED_0                                                   :  1;
        unsigned int RESERVED_1                                                   :  1;
        unsigned int RESERVED_2                                                   :  1;
        unsigned int RESERVED_3                                                   :  1;
        unsigned int RESERVED_4                                                   :  1;
        unsigned int RESERVED_5                                                   :  1;
        unsigned int                                                              :  2;
        unsigned int RSVD                                                         : 24;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_ENHANCE_INTERNAL {

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_FIFO_DEPTH_CNTL {
    struct {
        unsigned int DEPTH                                                        : 10;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_FIFO_SIZE {
    struct {
        unsigned int SC_FRONTEND_PRIM_FIFO_SIZE                                   :  6;
        unsigned int SC_BACKEND_PRIM_FIFO_SIZE                                    :  9;
        unsigned int SC_HIZ_TILE_FIFO_SIZE                                        :  6;
        unsigned int SC_EARLYZ_TILE_FIFO_SIZE                                     : 11;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_FORCE_EOV_MAX_CNTS {
    struct {
        unsigned int FORCE_EOV_MAX_CLK_CNT                                        : 16;
        unsigned int FORCE_EOV_MAX_REZ_CNT                                        : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_GENERIC_SCISSOR_BR {
    struct {
        unsigned int BR_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int BR_Y                                                         : 15;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_GENERIC_SCISSOR_TL {
    struct {
        unsigned int TL_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int TL_Y                                                         : 15;
        unsigned int WINDOW_OFFSET_DISABLE                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_HP3D_TRAP_SCREEN_COUNT {
    struct {
        unsigned int COUNT                                                        : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_HP3D_TRAP_SCREEN_H {
    struct {
        unsigned int X_COORD                                                      : 14;
        unsigned int                                                              : 18;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_HP3D_TRAP_SCREEN_HV_EN {
    struct {
        unsigned int ENABLE_HV_PRE_SHADER                                         :  1;
        unsigned int FORCE_PRE_SHADER_ALL_PIXELS                                  :  1;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_HP3D_TRAP_SCREEN_HV_LOCK {
    struct {
        unsigned int DISABLE_NON_PRIV_WRITES                                      :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_HP3D_TRAP_SCREEN_OCCURRENCE {
    struct {
        unsigned int COUNT                                                        : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_HP3D_TRAP_SCREEN_V {
    struct {
        unsigned int Y_COORD                                                      : 14;
        unsigned int                                                              : 18;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_IF_FIFO_SIZE {
    struct {
        unsigned int SC_DB_TILE_IF_FIFO_SIZE                                      :  6;
        unsigned int SC_DB_QUAD_IF_FIFO_SIZE                                      :  6;
        unsigned int SC_SPI_IF_FIFO_SIZE                                          :  6;
        unsigned int SC_BCI_IF_FIFO_SIZE                                          :  6;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_LINE_CNTL {
    struct {
        unsigned int                                                              :  9;
        unsigned int EXPAND_LINE_WIDTH                                            :  1;
        unsigned int LAST_PIXEL                                                   :  1;
        unsigned int PERPENDICULAR_ENDCAP_ENA                                     :  1;
        unsigned int DX10_DIAMOND_TEST_ENA                                        :  1;
        unsigned int                                                              : 19;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 13;
        unsigned int EXTRA_DX_DY_PRECISION                                        :  1;
        unsigned int                                                              : 18;
    } gfx09_1xPlus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_LINE_STIPPLE {
    struct {
        unsigned int LINE_PATTERN                                                 : 16;
        unsigned int REPEAT_COUNT                                                 :  8;
        unsigned int                                                              :  4;
        unsigned int PATTERN_BIT_ORDER                                            :  1;
        unsigned int AUTO_RESET_CNTL                                              :  2;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_LINE_STIPPLE_STATE {
    struct {
        unsigned int CURRENT_PTR                                                  :  4;
        unsigned int                                                              :  4;
        unsigned int CURRENT_COUNT                                                :  8;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_MODE_CNTL_0 {
    struct {
        unsigned int MSAA_ENABLE                                                  :  1;
        unsigned int VPORT_SCISSOR_ENABLE                                         :  1;
        unsigned int LINE_STIPPLE_ENABLE                                          :  1;
        unsigned int SEND_UNLIT_STILES_TO_PKR                                     :  1;
        unsigned int                                                              :  2;
        unsigned int COARSE_TILE_STARTS_ON_EVEN_RB                                :  1;
        unsigned int                                                              : 25;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  4;
        unsigned int SCALE_LINE_WIDTH_PAD                                         :  1;
        unsigned int                                                              : 27;
    } most;
    struct {
        unsigned int                                                              :  5;
        unsigned int ALTERNATE_RBS_PER_TILE                                       :  1;
        unsigned int                                                              : 26;
    } core;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_MODE_CNTL_1 {
    struct {
        unsigned int WALK_SIZE                                                    :  1;
        unsigned int WALK_ALIGNMENT                                               :  1;
        unsigned int WALK_ALIGN8_PRIM_FITS_ST                                     :  1;
        unsigned int WALK_FENCE_ENABLE                                            :  1;
        unsigned int WALK_FENCE_SIZE                                              :  3;
        unsigned int SUPERTILE_WALK_ORDER_ENABLE                                  :  1;
        unsigned int TILE_WALK_ORDER_ENABLE                                       :  1;
        unsigned int TILE_COVER_DISABLE                                           :  1;
        unsigned int TILE_COVER_NO_SCISSOR                                        :  1;
        unsigned int ZMM_LINE_EXTENT                                              :  1;
        unsigned int ZMM_LINE_OFFSET                                              :  1;
        unsigned int ZMM_RECT_EXTENT                                              :  1;
        unsigned int KILL_PIX_POST_HI_Z                                           :  1;
        unsigned int KILL_PIX_POST_DETAIL_MASK                                    :  1;
        unsigned int PS_ITER_SAMPLE                                               :  1;
        unsigned int MULTI_SHADER_ENGINE_PRIM_DISCARD_ENABLE                      :  1;
        unsigned int MULTI_GPU_SUPERTILE_ENABLE                                   :  1;
        unsigned int GPU_ID_OVERRIDE_ENABLE                                       :  1;
        unsigned int GPU_ID_OVERRIDE                                              :  4;
        unsigned int MULTI_GPU_PRIM_DISCARD_ENABLE                                :  1;
        unsigned int FORCE_EOV_CNTDWN_ENABLE                                      :  1;
        unsigned int FORCE_EOV_REZ_ENABLE                                         :  1;
        unsigned int OUT_OF_ORDER_PRIMITIVE_ENABLE                                :  1;
        unsigned int OUT_OF_ORDER_WATER_MARK                                      :  3;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_NGG_MODE_CNTL {
    struct {
        unsigned int MAX_DEALLOCS_IN_WAVE                                         : 11;
        unsigned int                                                              : 21;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 16;
        unsigned int MAX_FPOVS_IN_WAVE                                            :  8;
        unsigned int                                                              :  8;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_P3D_TRAP_SCREEN_COUNT {
    struct {
        unsigned int COUNT                                                        : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_P3D_TRAP_SCREEN_H {
    struct {
        unsigned int X_COORD                                                      : 14;
        unsigned int                                                              : 18;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_P3D_TRAP_SCREEN_HV_EN {
    struct {
        unsigned int ENABLE_HV_PRE_SHADER                                         :  1;
        unsigned int FORCE_PRE_SHADER_ALL_PIXELS                                  :  1;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_P3D_TRAP_SCREEN_HV_LOCK {
    struct {
        unsigned int DISABLE_NON_PRIV_WRITES                                      :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_P3D_TRAP_SCREEN_OCCURRENCE {
    struct {
        unsigned int COUNT                                                        : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_P3D_TRAP_SCREEN_V {
    struct {
        unsigned int Y_COORD                                                      : 14;
        unsigned int                                                              : 18;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PBB_OVERRIDE_FLAG {
    struct {
        unsigned int OVERRIDE                                                     :  1;
        unsigned int PIPE_ID                                                      :  1;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  8;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 24;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } gfx09_1xPlus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              : 12;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 24;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } gfx09_1xPlus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER4_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER4_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER4_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER5_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER5_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER5_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER6_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER6_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER6_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER7_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER7_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER7_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PKR_WAVE_TABLE_CNTL {
    struct {
        unsigned int SIZE                                                         :  6;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_RASTER_CONFIG {
    struct {
        unsigned int RB_MAP_PKR0                                                  :  2;
        unsigned int RB_MAP_PKR1                                                  :  2;
        unsigned int RB_XSEL2                                                     :  2;
        unsigned int RB_XSEL                                                      :  1;
        unsigned int RB_YSEL                                                      :  1;
        unsigned int PKR_MAP                                                      :  2;
        unsigned int PKR_XSEL                                                     :  2;
        unsigned int PKR_YSEL                                                     :  2;
        unsigned int PKR_XSEL2                                                    :  2;
        unsigned int SC_MAP                                                       :  2;
        unsigned int SC_XSEL                                                      :  2;
        unsigned int SC_YSEL                                                      :  2;
        unsigned int                                                              :  2;
        unsigned int SE_MAP                                                       :  2;
        unsigned int                                                              :  6;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 26;
        unsigned int SE_XSEL                                                      :  3;
        unsigned int SE_YSEL                                                      :  3;
    } gfx09;
    struct {
        unsigned int                                                              : 26;
        unsigned int SE_XSEL                                                      :  2;
        unsigned int SE_YSEL                                                      :  2;
        unsigned int                                                              :  2;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_RASTER_CONFIG_1 {
    struct {
        unsigned int SE_PAIR_MAP                                                  :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  2;
        unsigned int SE_PAIR_XSEL                                                 :  3;
        unsigned int SE_PAIR_YSEL                                                 :  3;
        unsigned int                                                              : 24;
    } gfx09;
    struct {
        unsigned int                                                              :  2;
        unsigned int SE_PAIR_XSEL                                                 :  2;
        unsigned int SE_PAIR_YSEL                                                 :  2;
        unsigned int                                                              : 26;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_SCREEN_SCISSOR_BR {
    struct {
        unsigned int BR_X                                                         : 16;
        unsigned int BR_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_SCREEN_SCISSOR_TL {
    struct {
        unsigned int TL_X                                                         : 16;
        unsigned int TL_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_SHADER_CONTROL {
    struct {
        unsigned int REALIGN_DQUADS_AFTER_N_WAVES                                 :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  2;
        unsigned int LOAD_COLLISION_WAVEID                                        :  1;
        unsigned int LOAD_INTRAWAVE_COLLISION                                     :  1;
        unsigned int                                                              : 28;
    } core;
    struct {
        unsigned int                                                              :  5;
        unsigned int WAVE_BREAK_REGION_SIZE                                       :  2;
        unsigned int                                                              : 25;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_TILE_STEERING_CREST_OVERRIDE {
    struct {
        unsigned int ONE_RB_MODE_ENABLE                                           :  1;
        unsigned int SE_SELECT                                                    :  2;
        unsigned int                                                              :  2;
        unsigned int RB_SELECT                                                    :  2;
        unsigned int                                                              : 25;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  8;
        unsigned int SA_SELECT                                                    :  3;
        unsigned int                                                              : 20;
        unsigned int FORCE_TILE_STEERING_OVERRIDE_USE                             :  1;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_TILE_STEERING_OVERRIDE {
    struct {
        unsigned int ENABLE                                                       :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  1;
        unsigned int NUM_SE                                                       :  2;
        unsigned int                                                              :  2;
        unsigned int NUM_RB_PER_SE                                                :  2;
        unsigned int                                                              : 25;
    } core;
    struct {
        unsigned int                                                              : 12;
        unsigned int NUM_SC                                                       :  2;
        unsigned int                                                              :  2;
        unsigned int NUM_RB_PER_SC                                                :  2;
        unsigned int                                                              : 14;
    } gfx10;
    struct {
        unsigned int                                                              : 20;
        unsigned int NUM_PACKER_PER_SC                                            :  1;
        unsigned int                                                              : 11;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_TRAP_SCREEN_COUNT {
    struct {
        unsigned int COUNT                                                        : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_TRAP_SCREEN_H {
    struct {
        unsigned int X_COORD                                                      : 14;
        unsigned int                                                              : 18;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_TRAP_SCREEN_HV_EN {
    struct {
        unsigned int ENABLE_HV_PRE_SHADER                                         :  1;
        unsigned int FORCE_PRE_SHADER_ALL_PIXELS                                  :  1;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_TRAP_SCREEN_HV_LOCK {
    struct {
        unsigned int DISABLE_NON_PRIV_WRITES                                      :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_TRAP_SCREEN_OCCURRENCE {
    struct {
        unsigned int COUNT                                                        : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_TRAP_SCREEN_V {
    struct {
        unsigned int Y_COORD                                                      : 14;
        unsigned int                                                              : 18;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_0_BR {
    struct {
        unsigned int BR_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int BR_Y                                                         : 15;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_0_TL {
    struct {
        unsigned int TL_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int TL_Y                                                         : 15;
        unsigned int WINDOW_OFFSET_DISABLE                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_1_BR {
    struct {
        unsigned int BR_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int BR_Y                                                         : 15;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_1_TL {
    struct {
        unsigned int TL_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int TL_Y                                                         : 15;
        unsigned int WINDOW_OFFSET_DISABLE                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_2_BR {
    struct {
        unsigned int BR_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int BR_Y                                                         : 15;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_2_TL {
    struct {
        unsigned int TL_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int TL_Y                                                         : 15;
        unsigned int WINDOW_OFFSET_DISABLE                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_3_BR {
    struct {
        unsigned int BR_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int BR_Y                                                         : 15;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_3_TL {
    struct {
        unsigned int TL_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int TL_Y                                                         : 15;
        unsigned int WINDOW_OFFSET_DISABLE                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_4_BR {
    struct {
        unsigned int BR_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int BR_Y                                                         : 15;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_4_TL {
    struct {
        unsigned int TL_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int TL_Y                                                         : 15;
        unsigned int WINDOW_OFFSET_DISABLE                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_5_BR {
    struct {
        unsigned int BR_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int BR_Y                                                         : 15;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_5_TL {
    struct {
        unsigned int TL_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int TL_Y                                                         : 15;
        unsigned int WINDOW_OFFSET_DISABLE                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_6_BR {
    struct {
        unsigned int BR_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int BR_Y                                                         : 15;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_6_TL {
    struct {
        unsigned int TL_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int TL_Y                                                         : 15;
        unsigned int WINDOW_OFFSET_DISABLE                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_7_BR {
    struct {
        unsigned int BR_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int BR_Y                                                         : 15;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_7_TL {
    struct {
        unsigned int TL_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int TL_Y                                                         : 15;
        unsigned int WINDOW_OFFSET_DISABLE                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_8_BR {
    struct {
        unsigned int BR_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int BR_Y                                                         : 15;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_8_TL {
    struct {
        unsigned int TL_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int TL_Y                                                         : 15;
        unsigned int WINDOW_OFFSET_DISABLE                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_9_BR {
    struct {
        unsigned int BR_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int BR_Y                                                         : 15;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_9_TL {
    struct {
        unsigned int TL_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int TL_Y                                                         : 15;
        unsigned int WINDOW_OFFSET_DISABLE                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_10_BR {
    struct {
        unsigned int BR_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int BR_Y                                                         : 15;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_10_TL {
    struct {
        unsigned int TL_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int TL_Y                                                         : 15;
        unsigned int WINDOW_OFFSET_DISABLE                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_11_BR {
    struct {
        unsigned int BR_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int BR_Y                                                         : 15;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_11_TL {
    struct {
        unsigned int TL_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int TL_Y                                                         : 15;
        unsigned int WINDOW_OFFSET_DISABLE                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_12_BR {
    struct {
        unsigned int BR_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int BR_Y                                                         : 15;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_12_TL {
    struct {
        unsigned int TL_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int TL_Y                                                         : 15;
        unsigned int WINDOW_OFFSET_DISABLE                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_13_BR {
    struct {
        unsigned int BR_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int BR_Y                                                         : 15;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_13_TL {
    struct {
        unsigned int TL_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int TL_Y                                                         : 15;
        unsigned int WINDOW_OFFSET_DISABLE                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_14_BR {
    struct {
        unsigned int BR_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int BR_Y                                                         : 15;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_14_TL {
    struct {
        unsigned int TL_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int TL_Y                                                         : 15;
        unsigned int WINDOW_OFFSET_DISABLE                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_15_BR {
    struct {
        unsigned int BR_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int BR_Y                                                         : 15;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_15_TL {
    struct {
        unsigned int TL_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int TL_Y                                                         : 15;
        unsigned int WINDOW_OFFSET_DISABLE                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMAX_0 {
    struct {
        unsigned int VPORT_ZMAX                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMAX_1 {
    struct {
        unsigned int VPORT_ZMAX                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMAX_2 {
    struct {
        unsigned int VPORT_ZMAX                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMAX_3 {
    struct {
        unsigned int VPORT_ZMAX                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMAX_4 {
    struct {
        unsigned int VPORT_ZMAX                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMAX_5 {
    struct {
        unsigned int VPORT_ZMAX                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMAX_6 {
    struct {
        unsigned int VPORT_ZMAX                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMAX_7 {
    struct {
        unsigned int VPORT_ZMAX                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMAX_8 {
    struct {
        unsigned int VPORT_ZMAX                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMAX_9 {
    struct {
        unsigned int VPORT_ZMAX                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMAX_10 {
    struct {
        unsigned int VPORT_ZMAX                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMAX_11 {
    struct {
        unsigned int VPORT_ZMAX                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMAX_12 {
    struct {
        unsigned int VPORT_ZMAX                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMAX_13 {
    struct {
        unsigned int VPORT_ZMAX                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMAX_14 {
    struct {
        unsigned int VPORT_ZMAX                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMAX_15 {
    struct {
        unsigned int VPORT_ZMAX                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMIN_0 {
    struct {
        unsigned int VPORT_ZMIN                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMIN_1 {
    struct {
        unsigned int VPORT_ZMIN                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMIN_2 {
    struct {
        unsigned int VPORT_ZMIN                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMIN_3 {
    struct {
        unsigned int VPORT_ZMIN                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMIN_4 {
    struct {
        unsigned int VPORT_ZMIN                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMIN_5 {
    struct {
        unsigned int VPORT_ZMIN                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMIN_6 {
    struct {
        unsigned int VPORT_ZMIN                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMIN_7 {
    struct {
        unsigned int VPORT_ZMIN                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMIN_8 {
    struct {
        unsigned int VPORT_ZMIN                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMIN_9 {
    struct {
        unsigned int VPORT_ZMIN                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMIN_10 {
    struct {
        unsigned int VPORT_ZMIN                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMIN_11 {
    struct {
        unsigned int VPORT_ZMIN                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMIN_12 {
    struct {
        unsigned int VPORT_ZMIN                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMIN_13 {
    struct {
        unsigned int VPORT_ZMIN                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMIN_14 {
    struct {
        unsigned int VPORT_ZMIN                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMIN_15 {
    struct {
        unsigned int VPORT_ZMIN                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_WINDOW_OFFSET {
    struct {
        unsigned int WINDOW_X_OFFSET                                              : 16;
        unsigned int WINDOW_Y_OFFSET                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_WINDOW_SCISSOR_BR {
    struct {
        unsigned int BR_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int BR_Y                                                         : 15;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_WINDOW_SCISSOR_TL {
    struct {
        unsigned int TL_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int TL_Y                                                         : 15;
        unsigned int WINDOW_OFFSET_DISABLE                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SIDEBAND_REQUEST_DELAYS {
    struct {
        unsigned int RETRY_DELAY                                                  : 16;
        unsigned int INITIAL_DELAY                                                : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_STATE_STEREO_X {
    struct {
        unsigned int STEREO_X_OFFSET                                              : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_STEREO_CNTL {
    struct {
        unsigned int                                                              :  1;
        unsigned int STEREO_MODE                                                  :  4;
        unsigned int RT_SLICE_MODE                                                :  3;
        unsigned int RT_SLICE_OFFSET                                              :  4;
        unsigned int                                                              :  4;
        unsigned int VP_ID_MODE                                                   :  3;
        unsigned int VP_ID_OFFSET                                                 :  4;
        unsigned int                                                              :  9;
    } gfx10;
    struct {
        unsigned int EN_STEREO                                                    :  1;
        unsigned int STEREO_MODE                                                  :  4;
        unsigned int RT_SLICE_MODE                                                :  3;
        unsigned int RT_SLICE_OFFSET                                              :  2;
        unsigned int VP_ID_MODE                                                   :  3;
        unsigned int VP_ID_OFFSET                                                 :  4;
        unsigned int                                                              : 15;
    } vg12;
    struct {
        unsigned int EN_STEREO                                                    :  1;
        unsigned int STEREO_MODE                                                  :  4;
        unsigned int RT_SLICE_MODE                                                :  3;
        unsigned int RT_SLICE_OFFSET                                              :  2;
        unsigned int VP_ID_MODE                                                   :  3;
        unsigned int VP_ID_OFFSET                                                 :  4;
        unsigned int                                                              : 15;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_CNTL_STATUS {
    struct {
        unsigned int                                                              : 31;
        unsigned int SU_BUSY                                                      :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_DEBUG_DATA {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_HARDWARE_SCREEN_OFFSET {
    struct {
        unsigned int HW_SCREEN_OFFSET_X                                           :  9;
        unsigned int                                                              :  7;
        unsigned int HW_SCREEN_OFFSET_Y                                           :  9;
        unsigned int                                                              :  7;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_LINE_CNTL {
    struct {
        unsigned int WIDTH                                                        : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_LINE_STIPPLE_CNTL {
    struct {
        unsigned int LINE_STIPPLE_RESET                                           :  2;
        unsigned int EXPAND_FULL_LENGTH                                           :  1;
        unsigned int FRACTIONAL_ACCUM                                             :  1;
        unsigned int DIAMOND_ADJUST                                               :  1;
        unsigned int                                                              : 27;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_LINE_STIPPLE_SCALE {
    struct {
        unsigned int LINE_STIPPLE_SCALE                                           : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_LINE_STIPPLE_VALUE {
    struct {
        unsigned int LINE_STIPPLE_VALUE                                           : 24;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_OVER_RASTERIZATION_CNTL {
    struct {
        unsigned int DISCARD_0_AREA_TRIANGLES                                     :  1;
        unsigned int DISCARD_0_AREA_LINES                                         :  1;
        unsigned int DISCARD_0_AREA_POINTS                                        :  1;
        unsigned int DISCARD_0_AREA_RECTANGLES                                    :  1;
        unsigned int USE_PROVOKING_ZW                                             :  1;
        unsigned int                                                              : 27;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 16;
        unsigned int                                                              : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  8;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 24;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } gfx09_1xPlus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              : 12;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 24;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } gfx09_1xPlus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 16;
        unsigned int                                                              : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  8;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 24;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } gfx09_1xPlus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_PERFCOUNTER1_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              : 12;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 24;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } gfx09_1xPlus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 16;
        unsigned int                                                              : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  8;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 28;
        unsigned int PERF_MODE                                                    :  4;
    } gfx09_1xPlus;
    struct {
        unsigned int                                                              : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int                                                              :  4;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_PERFCOUNTER2_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 16;
        unsigned int                                                              : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  8;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 28;
        unsigned int PERF_MODE                                                    :  4;
    } gfx09_1xPlus;
    struct {
        unsigned int                                                              : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int                                                              :  4;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_PERFCOUNTER3_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_POINT_MINMAX {
    struct {
        unsigned int MIN_SIZE                                                     : 16;
        unsigned int MAX_SIZE                                                     : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_POINT_SIZE {
    struct {
        unsigned int HEIGHT                                                       : 16;
        unsigned int WIDTH                                                        : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_POLY_OFFSET_BACK_OFFSET {
    struct {
        unsigned int OFFSET                                                       : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_POLY_OFFSET_BACK_SCALE {
    struct {
        unsigned int SCALE                                                        : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_POLY_OFFSET_CLAMP {
    struct {
        unsigned int CLAMP                                                        : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_POLY_OFFSET_DB_FMT_CNTL {
    struct {
        unsigned int POLY_OFFSET_NEG_NUM_DB_BITS                                  :  8;
        unsigned int POLY_OFFSET_DB_IS_FLOAT_FMT                                  :  1;
        unsigned int                                                              : 23;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_POLY_OFFSET_FRONT_OFFSET {
    struct {
        unsigned int OFFSET                                                       : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_POLY_OFFSET_FRONT_SCALE {
    struct {
        unsigned int SCALE                                                        : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_PRIM_FILTER_CNTL {
    struct {
        unsigned int TRIANGLE_FILTER_DISABLE                                      :  1;
        unsigned int LINE_FILTER_DISABLE                                          :  1;
        unsigned int POINT_FILTER_DISABLE                                         :  1;
        unsigned int RECTANGLE_FILTER_DISABLE                                     :  1;
        unsigned int TRIANGLE_EXPAND_ENA                                          :  1;
        unsigned int LINE_EXPAND_ENA                                              :  1;
        unsigned int POINT_EXPAND_ENA                                             :  1;
        unsigned int RECTANGLE_EXPAND_ENA                                         :  1;
        unsigned int PRIM_EXPAND_CONSTANT                                         :  8;
        unsigned int                                                              : 14;
        unsigned int XMAX_RIGHT_EXCLUSION                                         :  1;
        unsigned int YMAX_BOTTOM_EXCLUSION                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_SC_MODE_CNTL {
    struct {
        unsigned int CULL_FRONT                                                   :  1;
        unsigned int CULL_BACK                                                    :  1;
        unsigned int FACE                                                         :  1;
        unsigned int POLY_MODE                                                    :  2;
        unsigned int POLYMODE_FRONT_PTYPE                                         :  3;
        unsigned int POLYMODE_BACK_PTYPE                                          :  3;
        unsigned int POLY_OFFSET_FRONT_ENABLE                                     :  1;
        unsigned int POLY_OFFSET_BACK_ENABLE                                      :  1;
        unsigned int POLY_OFFSET_PARA_ENABLE                                      :  1;
        unsigned int                                                              :  2;
        unsigned int VTX_WINDOW_OFFSET_ENABLE                                     :  1;
        unsigned int                                                              :  2;
        unsigned int PROVOKING_VTX_LAST                                           :  1;
        unsigned int PERSP_CORR_DIS                                               :  1;
        unsigned int MULTI_PRIM_IB_ENA                                            :  1;
        unsigned int RIGHT_TRIANGLE_ALTERNATE_GRADIENT_REF                        :  1;
        unsigned int NEW_QUAD_DECOMPOSITION                                       :  1;
        unsigned int                                                              :  8;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 24;
        unsigned int KEEP_TOGETHER_ENABLE                                         :  1;
        unsigned int                                                              :  7;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_SMALL_PRIM_FILTER_CNTL {
    struct {
        unsigned int SMALL_PRIM_FILTER_ENABLE                                     :  1;
        unsigned int TRIANGLE_FILTER_DISABLE                                      :  1;
        unsigned int LINE_FILTER_DISABLE                                          :  1;
        unsigned int POINT_FILTER_DISABLE                                         :  1;
        unsigned int RECTANGLE_FILTER_DISABLE                                     :  1;
        unsigned int                                                              : 27;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  6;
        unsigned int SC_1XMSAA_COMPATIBLE_DISABLE                                 :  1;
        unsigned int                                                              : 25;
    } gfx09_1xPlus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_VTX_CNTL {
    struct {
        unsigned int PIX_CENTER                                                   :  1;
        unsigned int ROUND_MODE                                                   :  2;
        unsigned int QUANT_MODE                                                   :  3;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_UTCL1_CNTL1 {
    struct {
        unsigned int FORCE_4K_L2_RESP                                             :  1;
        unsigned int GPUVM_64K_DEFAULT                                            :  1;
        unsigned int GPUVM_PERM_MODE                                              :  1;
        unsigned int RESP_MODE                                                    :  2;
        unsigned int RESP_FAULT_MODE                                              :  2;
        unsigned int CLIENTID                                                     :  9;
        unsigned int SPARE                                                        :  1;
        unsigned int ENABLE_PUSH_LFIFO                                            :  1;
        unsigned int ENABLE_LFIFO_PRI_ARB                                         :  1;
        unsigned int REG_INV_VMID                                                 :  4;
        unsigned int REG_INV_ALL_VMID                                             :  1;
        unsigned int REG_INV_TOGGLE                                               :  1;
        unsigned int INVALIDATE_ALL_VMID                                          :  1;
        unsigned int FORCE_MISS                                                   :  1;
        unsigned int FORCE_IN_ORDER                                               :  1;
        unsigned int REDUCE_FIFO_DEPTH_BY_2                                       :  2;
        unsigned int REDUCE_CACHE_SIZE_BY_2                                       :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_UTCL1_CNTL2 {
    struct {
        unsigned int SPARE1                                                       :  8;
        unsigned int SPARE2                                                       :  1;
        unsigned int MTYPE_OVRD_DIS                                               :  1;
        unsigned int LINE_VALID                                                   :  1;
        unsigned int SPARE3                                                       :  1;
        unsigned int GPUVM_INV_MODE                                               :  1;
        unsigned int ENABLE_SHOOTDOWN_OPT                                         :  1;
        unsigned int FORCE_SNOOP                                                  :  1;
        unsigned int FORCE_GPUVM_INV_ACK                                          :  1;
        unsigned int SPARE4                                                       :  2;
        unsigned int ENABLE_PERF_EVENT_RD_WR                                      :  1;
        unsigned int PERF_EVENT_RD_WR                                             :  1;
        unsigned int ENABLE_PERF_EVENT_VMID                                       :  1;
        unsigned int PERF_EVENT_VMID                                              :  4;
        unsigned int SPARE5                                                       :  1;
        unsigned int FORCE_FRAG_2M_TO_64K                                         :  1;
        unsigned int RESERVED                                                     :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PMM_GENERAL_CNTL {
    struct {
        unsigned int PMM_MODE                                                     :  1;
        unsigned int PMM_DISABLE                                                  :  1;
        unsigned int PMM_ALOG_IH_IDLE                                             :  1;
        unsigned int                                                              : 29;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union port_a_addr {
    struct {
        unsigned int Index                                                        :  8;
        unsigned int Reserved                                                     : 23;
        unsigned int ReadEnable                                                   :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union port_a_data_hi {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union port_a_data_lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union port_b_addr {
    struct {
        unsigned int Index                                                        :  8;
        unsigned int Reserved                                                     : 23;
        unsigned int ReadEnable                                                   :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union port_b_data_hi {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union port_b_data_lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union port_c_addr {
    struct {
        unsigned int Index                                                        :  8;
        unsigned int Reserved                                                     : 23;
        unsigned int ReadEnable                                                   :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union port_c_data_hi {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union port_c_data_lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union port_d_addr {
    struct {
        unsigned int Index                                                        :  8;
        unsigned int Reserved                                                     : 23;
        unsigned int ReadEnable                                                   :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union port_d_data_hi {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union port_d_data_lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PWROK_REFCLK_GAP_CYCLES {
    struct {
        unsigned int Pwrok_PreAssertion_clkgap_cycles                             :  8;
        unsigned int Pwrok_PostAssertion_clkgap_cycles                            :  8;
        unsigned int                                                              : 16;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PWR_OVERRIDE {
    struct {
        unsigned int VDDCR_CPU_OVERRIDE                                           :  1;
        unsigned int                                                              : 31;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union rbuf_DEBUG_DATA {
    struct {
        unsigned int                                                              :  2;
        unsigned int pixel_addr_mask                                              : 15;
        unsigned int                                                              :  7;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERFCOUNTER_SELECT                                           :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERFCOUNTER_SELECT                                           :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_PERFMON_CLK_CNTL {
    struct {
        unsigned int PERFMON_CLOCK_STATE                                          :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_PERFMON_CNTL {
    struct {
        unsigned int PERFMON_STATE                                                :  3;
        unsigned int                                                              :  7;
        unsigned int PERFMON_SAMPLE_ENABLE                                        :  1;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_ACCUM_CTRL {
    struct {
        unsigned int StrobeResetPerfMonitors                                      :  1;
        unsigned int StrobeStartAccumulation                                      :  1;
        unsigned int StrobeRearmAccum                                             :  1;
        unsigned int                                                              : 29;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  3;
        unsigned int StrobeSpmDoneInt                                             :  1;
        unsigned int StrobeAccumDoneInt                                           :  1;
        unsigned int StrobeResetAccum                                             :  1;
        unsigned int StrobeStartSpm                                               :  4;
        unsigned int RESERVED                                                     : 22;
    } nv10;
    struct {
        unsigned int                                                              :  3;
        unsigned int StrobeSpmDoneInt                                             :  1;
        unsigned int StrobeAccumDoneInt                                           :  1;
        unsigned int StrobeResetAccum                                             :  1;
        unsigned int StrobeStartSpm                                               :  4;
        unsigned int RESERVED                                                     : 22;
    } rv2x;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_ACCUM_CTRLRAM_ADDR {
    struct {
        unsigned int addr                                                         :  9;
        unsigned int RESERVED                                                     : 23;
    } nv10;
    struct {
        unsigned int addr                                                         :  8;
        unsigned int RESERVED                                                     : 24;
    } rv2x;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_ACCUM_CTRLRAM_DATA {
    struct {
        unsigned int data                                                         :  8;
        unsigned int RESERVED                                                     : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_ACCUM_DATARAM_ADDR {
    struct {
        unsigned int addr                                                         :  7;
        unsigned int RESERVED                                                     : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_ACCUM_DATARAM_DATA {
    struct {
        unsigned int data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_ACCUM_DATARAM_WRCOUNT {
    struct {
        unsigned int DataRamWrCount                                               : 19;
        unsigned int RESERVED                                                     : 13;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_ACCUM_MODE {
    struct {
        unsigned int EnableAccum                                                  :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  1;
        unsigned int AutoAccumEn                                                  :  1;
        unsigned int AutoSpmEn                                                    :  1;
        unsigned int Globals_LoadOverride                                         :  1;
        unsigned int SE0_LoadOverride                                             :  1;
        unsigned int SE1_LoadOverride                                             :  1;
        unsigned int AutoResetPerfmonDisable                                      :  1;
        unsigned int RESERVED                                                     : 25;
    } nv10;
    struct {
        unsigned int                                                              :  1;
        unsigned int AutoAccumEn                                                  :  1;
        unsigned int AutoSpmEn                                                    :  1;
        unsigned int Globals_LoadOverride                                         :  1;
        unsigned int SE0_LoadOverride                                             :  1;
        unsigned int AutoResetPerfmonDisable                                      :  1;
        unsigned int RESERVED                                                     : 26;
    } rv2x;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_ACCUM_SAMPLES_REQUESTED {
    struct {
        unsigned int SamplesRequested                                             :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  8;
        unsigned int RESERVED                                                     : 24;
    } nv10;
    struct {
        unsigned int                                                              :  8;
        unsigned int RESERVED                                                     : 24;
    } rv2x;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_ACCUM_STATUS {
    struct {
        unsigned int NumbSamplesCompleted                                         :  8;
        unsigned int AccumDone                                                    :  1;
        unsigned int SpmDone                                                      :  1;
        unsigned int AccumOverflow                                                :  1;
        unsigned int AccumArmed                                                   :  1;
        unsigned int SequenceInProgress                                           :  1;
        unsigned int FinalSequenceInProgress                                      :  1;
        unsigned int AllFifosEmpty                                                :  1;
        unsigned int FSMIsIdle                                                    :  1;
        unsigned int                                                              : 16;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 16;
        unsigned int RESERVED                                                     : 16;
    } nv10;
    struct {
        unsigned int                                                              : 16;
        unsigned int RESERVED                                                     : 16;
    } rv2x;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_ACCUM_THRESHOLD {
    struct {
        unsigned int Threshold                                                    : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 16;
        unsigned int RESERVED                                                     : 16;
    } nv10;
    struct {
        unsigned int                                                              : 16;
        unsigned int RESERVED                                                     : 16;
    } rv2x;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_CBR0_PERFMON_SAMPLE_DELAY {
    struct {
        unsigned int PERFMON_SAMPLE_DELAY                                         :  8;
        unsigned int RESERVED                                                     : 24;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_CBR1_PERFMON_SAMPLE_DELAY {
    struct {
        unsigned int PERFMON_SAMPLE_DELAY                                         :  8;
        unsigned int RESERVED                                                     : 24;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_CB_PERFMON_SAMPLE_DELAY {
    struct {
        unsigned int PERFMON_SAMPLE_DELAY                                         :  8;
        unsigned int RESERVED                                                     : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_CPC_PERFMON_SAMPLE_DELAY {
    struct {
        unsigned int PERFMON_SAMPLE_DELAY                                         :  8;
        unsigned int RESERVED                                                     : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_CPF_PERFMON_SAMPLE_DELAY {
    struct {
        unsigned int PERFMON_SAMPLE_DELAY                                         :  8;
        unsigned int RESERVED                                                     : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_CPG_PERFMON_SAMPLE_DELAY {
    struct {
        unsigned int PERFMON_SAMPLE_DELAY                                         :  8;
        unsigned int RESERVED                                                     : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_DBR0_PERFMON_SAMPLE_DELAY {
    struct {
        unsigned int PERFMON_SAMPLE_DELAY                                         :  8;
        unsigned int RESERVED                                                     : 24;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_DBR1_PERFMON_SAMPLE_DELAY {
    struct {
        unsigned int PERFMON_SAMPLE_DELAY                                         :  8;
        unsigned int RESERVED                                                     : 24;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_DB_PERFMON_SAMPLE_DELAY {
    struct {
        unsigned int PERFMON_SAMPLE_DELAY                                         :  8;
        unsigned int RESERVED                                                     : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_DEBUG {
    struct {
        unsigned int DATA                                                         : 32;
    } gfx09;
    struct {
        unsigned int DATA0                                                        :  1;
        unsigned int DATA1                                                        :  1;
        unsigned int DATA2                                                        :  1;
        unsigned int DATA3                                                        :  1;
        unsigned int DATA4                                                        :  1;
        unsigned int DATA5                                                        :  1;
        unsigned int DATA6                                                        :  1;
        unsigned int DATA7                                                        :  1;
        unsigned int DATA8                                                        :  1;
        unsigned int DATA9                                                        :  1;
        unsigned int DATA10                                                       :  1;
        unsigned int DATA11                                                       :  1;
        unsigned int DATA12                                                       :  1;
        unsigned int DATA13                                                       :  1;
        unsigned int DATA14                                                       :  1;
        unsigned int DATA15                                                       :  1;
        unsigned int DATA16                                                       :  1;
        unsigned int DATA17                                                       :  1;
        unsigned int DATA18                                                       :  1;
        unsigned int DATA19                                                       :  1;
        unsigned int DATA20                                                       :  1;
        unsigned int DATA21                                                       :  1;
        unsigned int DATA22                                                       :  1;
        unsigned int DATA23                                                       :  1;
        unsigned int DATA24                                                       :  1;
        unsigned int DATA25                                                       :  1;
        unsigned int DATA26                                                       :  1;
        unsigned int DATA27                                                       :  1;
        unsigned int DATA28                                                       :  1;
        unsigned int DATA29                                                       :  1;
        unsigned int DATA30                                                       :  1;
        unsigned int DATA31                                                       :  1;
    } gfx101Plus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_DEBUG_SELECT {
    struct {
        unsigned int SELECT                                                       :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  8;
        unsigned int RESERVED                                                     :  7;
        unsigned int                                                              : 17;
    } most;
    struct {
        unsigned int                                                              :  8;
        unsigned int RESERVED                                                     :  4;
        unsigned int RLC_MC_FIFO_CNTL                                             :  1;
        unsigned int RLC_DBG_FIFO_CNTL                                            :  1;
        unsigned int RLC_SPM_DEBUG_MODE_DUAL                                      :  1;
        unsigned int                                                              : 17;
    } raven;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_DESER_START_SKEW {
    struct {
        unsigned int DESER_START_SKEW                                             :  7;
        unsigned int RESERVED                                                     : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_GDS_PERFMON_SAMPLE_DELAY {
    struct {
        unsigned int PERFMON_SAMPLE_DELAY                                         :  8;
        unsigned int RESERVED                                                     : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_GLB_SAMPLEDELAY_IND_ADDR {
    struct {
        unsigned int GLB_SAMPLEDELAY_INDEX                                        : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_GLB_SAMPLEDELAY_IND_DATA {
    struct {
        unsigned int data                                                         :  7;
        unsigned int RESERVED                                                     : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_GLOBALS_MUXSEL_SKEW {
    struct {
        unsigned int GLOBALS_MUXSEL_SKEW                                          :  7;
        unsigned int RESERVED                                                     : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_GLOBALS_SAMPLE_SKEW {
    struct {
        unsigned int GLOBALS_SAMPLE_SKEW                                          :  7;
        unsigned int RESERVED                                                     : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_GLOBAL_MUXSEL_ADDR {
    struct {
        unsigned int PERFMON_SEL_ADDR                                             : 32;
    } gfx09;
    struct {
        unsigned int PERFMON_SEL_ADDR                                             :  8;
        unsigned int RESERVED                                                     : 24;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_GLOBAL_MUXSEL_DATA {
    struct {
        unsigned int PERFMON_SEL_DATA                                             : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_IA_PERFMON_SAMPLE_DELAY {
    struct {
        unsigned int PERFMON_SAMPLE_DELAY                                         :  8;
        unsigned int RESERVED                                                     : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_INT_CNTL {
    struct {
        unsigned int RLC_SPM_INT_CNTL                                             :  1;
        unsigned int RESERVED                                                     : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_INT_INFO_1 {
    struct {
        unsigned int INTERRUPT_INFO_1                                             : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_INT_INFO_2 {
    struct {
        unsigned int INTERRUPT_INFO_2                                             : 16;
        unsigned int INTERRUPT_ID                                                 :  8;
        unsigned int RESERVED                                                     :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_INT_STATUS {
    struct {
        unsigned int RLC_SPM_INT_STATUS                                           :  1;
        unsigned int RESERVED                                                     : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_MC_CNTL {
    struct {
        unsigned int RLC_SPM_VMID                                                 :  4;
        unsigned int                                                              : 28;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  4;
        unsigned int RLC_SPM_POLICY                                               :  1;
        unsigned int RLC_SPM_PERF_CNTR                                            :  1;
        unsigned int RLC_SPM_FED                                                  :  1;
        unsigned int RLC_SPM_MTYPE_OVER                                           :  1;
        unsigned int RLC_SPM_MTYPE                                                :  2;
        unsigned int RESERVED                                                     : 22;
    } gfx09;
    struct {
        unsigned int                                                              :  4;
        unsigned int RLC_SPM_POLICY                                               :  2;
        unsigned int RLC_SPM_PERF_CNTR                                            :  1;
        unsigned int RLC_SPM_FED                                                  :  1;
        unsigned int RLC_SPM_MTYPE_OVER                                           :  1;
        unsigned int RLC_SPM_MTYPE                                                :  3;
        unsigned int                                                              :  3;
        unsigned int RLC_SPM_NOFILL                                               :  1;
        unsigned int                                                              : 16;
    } gfx10;
    struct {
        unsigned int                                                              : 16;
        unsigned int RESERVED                                                     : 16;
    } gfx101;
    struct {
        unsigned int                                                              : 12;
        unsigned int RLC_SPM_BC                                                   :  1;
        unsigned int                                                              :  1;
        unsigned int RLC_SPM_VOL                                                  :  1;
        unsigned int                                                              : 17;
    } gfx10Core;
    struct {
        unsigned int                                                              : 13;
        unsigned int RESERVED_2                                                   :  1;
        unsigned int                                                              : 18;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_PA_PERFMON_SAMPLE_DELAY {
    struct {
        unsigned int PERFMON_SAMPLE_DELAY                                         :  8;
        unsigned int RESERVED                                                     : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_PERFMON_CNTL {
    struct {
        unsigned int                                                              : 12;
        unsigned int PERFMON_RING_MODE                                            :  2;
        unsigned int RESERVED                                                     :  2;
        unsigned int PERFMON_SAMPLE_INTERVAL                                      : 16;
    } bits, bitfields;
    struct {
        unsigned int RESERVED1                                                    : 12;
        unsigned int                                                              : 20;
    } most;
    struct {
        unsigned int RLC_DBG_STRM_EN                                              :  1;
        unsigned int RLC_DBG_STRM_EXT_SAMPLE                                      :  1;
        unsigned int RESERVED1                                                    : 10;
        unsigned int                                                              : 20;
    } raven;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_PERFMON_GLB_SEGMENT_SIZE {
    struct {
        unsigned int PERFMON_SEGMENT_SIZE                                         :  8;
        unsigned int GLOBAL_NUM_LINE                                              :  8;
        unsigned int RESERVED                                                     : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_PERFMON_RING_BASE_HI {
    struct {
        unsigned int RING_BASE_HI                                                 : 16;
        unsigned int RESERVED                                                     : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_PERFMON_RING_BASE_LO {
    struct {
        unsigned int RING_BASE_LO                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_PERFMON_RING_SIZE {
    struct {
        unsigned int RING_BASE_SIZE                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_PERFMON_SAMPLE_DELAY_MAX {
    struct {
        unsigned int PERFMON_MAX_SAMPLE_DELAY                                     :  8;
        unsigned int RESERVED                                                     : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_PERFMON_SE3TO0_SEGMENT_SIZE {
    struct {
        unsigned int SE0_NUM_LINE                                                 :  8;
        unsigned int SE1_NUM_LINE                                                 :  8;
        unsigned int SE2_NUM_LINE                                                 :  8;
        unsigned int SE3_NUM_LINE                                                 :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_PERFMON_SEGMENT_SIZE {
    struct {
        unsigned int PERFMON_SEGMENT_SIZE                                         :  8;
        unsigned int RESERVED1                                                    :  3;
        unsigned int GLOBAL_NUM_LINE                                              :  5;
        unsigned int SE0_NUM_LINE                                                 :  5;
        unsigned int SE1_NUM_LINE                                                 :  5;
        unsigned int SE2_NUM_LINE                                                 :  5;
        unsigned int RESERVED                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_RING_RDPTR {
    struct {
        unsigned int PERFMON_RING_RDPTR                                           : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_RING_WRPTR {
    struct {
        unsigned int RESERVED                                                     :  5;
        unsigned int PERFMON_RING_WRPTR                                           : 27;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_RMI_PERFMON_SAMPLE_DELAY {
    struct {
        unsigned int PERFMON_SAMPLE_DELAY                                         :  8;
        unsigned int RESERVED                                                     : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_SAMPLE_CNT {
    struct {
        unsigned int COUNT                                                        : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_SC_PERFMON_SAMPLE_DELAY {
    struct {
        unsigned int PERFMON_SAMPLE_DELAY                                         :  8;
        unsigned int RESERVED                                                     : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_SEGMENT_THRESHOLD {
    struct {
        unsigned int NUM_SEGMENT_THRESHOLD                                        : 32;
    } gfx09;
    struct {
        unsigned int NUM_SEGMENT_THRESHOLD                                        :  8;
        unsigned int RESERVED                                                     : 24;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_SE_MUXSEL_ADDR {
    struct {
        unsigned int PERFMON_SEL_ADDR                                             : 32;
    } gfx09;
    struct {
        unsigned int PERFMON_SEL_ADDR                                             :  9;
        unsigned int RESERVED                                                     : 23;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_SE_MUXSEL_DATA {
    struct {
        unsigned int PERFMON_SEL_DATA                                             : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_SE_MUXSEL_SKEW {
    struct {
        unsigned int SE_MUXSEL_SKEW                                               :  7;
        unsigned int RESERVED                                                     : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_SE_SAMPLEDELAY_IND_ADDR {
    struct {
        unsigned int SE_SAMPLEDELAY_INDEX                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_SE_SAMPLEDELAY_IND_DATA {
    struct {
        unsigned int data                                                         :  7;
        unsigned int RESERVED                                                     : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_SE_SAMPLE_SKEW {
    struct {
        unsigned int SE_SAMPLE_SKEW                                               :  7;
        unsigned int RESERVED                                                     : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_SPI_PERFMON_SAMPLE_DELAY {
    struct {
        unsigned int PERFMON_SAMPLE_DELAY                                         :  8;
        unsigned int RESERVED                                                     : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_SQG_PERFMON_SAMPLE_DELAY {
    struct {
        unsigned int PERFMON_SAMPLE_DELAY                                         :  8;
        unsigned int RESERVED                                                     : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_SX_PERFMON_SAMPLE_DELAY {
    struct {
        unsigned int PERFMON_SAMPLE_DELAY                                         :  8;
        unsigned int RESERVED                                                     : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_TA_PERFMON_SAMPLE_DELAY {
    struct {
        unsigned int PERFMON_SAMPLE_DELAY                                         :  8;
        unsigned int RESERVED                                                     : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_TCA_PERFMON_SAMPLE_DELAY {
    struct {
        unsigned int PERFMON_SAMPLE_DELAY                                         :  8;
        unsigned int RESERVED                                                     : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_TCC_PERFMON_SAMPLE_DELAY {
    struct {
        unsigned int PERFMON_SAMPLE_DELAY                                         :  8;
        unsigned int RESERVED                                                     : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_TCP_PERFMON_SAMPLE_DELAY {
    struct {
        unsigned int PERFMON_SAMPLE_DELAY                                         :  8;
        unsigned int RESERVED                                                     : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_TD_PERFMON_SAMPLE_DELAY {
    struct {
        unsigned int PERFMON_SAMPLE_DELAY                                         :  8;
        unsigned int RESERVED                                                     : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_THREAD_TRACE_CTRL {
    struct {
        unsigned int THREAD_TRACE_INT_EN                                          :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_UTCL1_CNTL {
    struct {
        unsigned int XNACK_REDO_TIMER_CNT                                         : 20;
        unsigned int                                                              :  4;
        unsigned int DROP_MODE                                                    :  1;
        unsigned int BYPASS                                                       :  1;
        unsigned int INVALIDATE                                                   :  1;
        unsigned int FRAG_LIMIT_MODE                                              :  1;
        unsigned int FORCE_SNOOP                                                  :  1;
        unsigned int                                                              :  1;
        unsigned int RESERVED                                                     :  2;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 29;
        unsigned int FORCE_SD_VMID_DIRTY                                          :  1;
        unsigned int                                                              :  2;
    } gfx09;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_UTCL1_ERROR_1 {
    struct {
        unsigned int Translated_ReqError                                          :  2;
        unsigned int Translated_ReqErrorVmid                                      :  4;
        unsigned int Translated_ReqErrorAddr_MSB                                  :  4;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_UTCL1_ERROR_2 {
    struct {
        unsigned int Translated_ReqErrorAddr_LSB                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_VGT_PERFMON_SAMPLE_DELAY {
    struct {
        unsigned int PERFMON_SAMPLE_DELAY                                         :  8;
        unsigned int RESERVED                                                     : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_VIRT_CTRL {
    struct {
        unsigned int PauseSpmSamplingRequest                                      :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_VIRT_STATUS {
    struct {
        unsigned int SpmSamplingPaused                                            :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_CGTT_SCLK_CTRL {
    struct {
        unsigned int ON_DELAY                                                     :  4;
        unsigned int OFF_HYSTERESIS                                               :  8;
        unsigned int                                                              :  4;
        unsigned int SOFT_STALL_OVERRIDE7                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE6                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE5                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE4                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE3                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE2                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE1                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE0                                         :  1;
        unsigned int                                                              :  1;
        unsigned int SOFT_OVERRIDE6                                               :  1;
        unsigned int SOFT_OVERRIDE5                                               :  1;
        unsigned int SOFT_OVERRIDE4                                               :  1;
        unsigned int SOFT_OVERRIDE3                                               :  1;
        unsigned int SOFT_OVERRIDE2                                               :  1;
        unsigned int SOFT_OVERRIDE1                                               :  1;
        unsigned int SOFT_OVERRIDE0                                               :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_CLOCK_CNTRL {
    struct {
        unsigned int DYN_CLK_RB0_BUSY_MASK                                        :  5;
        unsigned int DYN_CLK_CMN_BUSY_MASK                                        :  5;
        unsigned int DYN_CLK_RB0_WAKEUP_MASK                                      :  5;
        unsigned int DYN_CLK_CMN_WAKEUP_MASK                                      :  5;
        unsigned int                                                              : 12;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 20;
        unsigned int DYN_CLK_RB1_BUSY_MASK                                        :  5;
        unsigned int DYN_CLK_RB1_WAKEUP_MASK                                      :  5;
        unsigned int                                                              :  2;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_DEMUX_CNTL {
    struct {
        unsigned int DEMUX_ARB0_STALL                                             :  1;
        unsigned int DEMUX_ARB0_BREAK_LOB_ON_IDLEIN                               :  1;
        unsigned int                                                              :  2;
        unsigned int DEMUX_ARB0_STALL_TIMER_OVERRIDE                              :  2;
        unsigned int DEMUX_ARB0_STALL_TIMER_START_VALUE                           :  8;
        unsigned int DEMUX_ARB0_MODE                                              :  2;
        unsigned int                                                              : 16;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 16;
        unsigned int DEMUX_ARB1_STALL                                             :  1;
        unsigned int DEMUX_ARB1_BREAK_LOB_ON_IDLEIN                               :  1;
        unsigned int                                                              :  2;
        unsigned int DEMUX_ARB1_STALL_TIMER_OVERRIDE                              :  2;
        unsigned int DEMUX_ARB1_STALL_TIMER_START_VALUE                           :  8;
        unsigned int DEMUX_ARB1_MODE                                              :  2;
    } most;
    struct {
        unsigned int                                                              :  2;
        unsigned int DEMUX_ARB0_MODE_OVERRIDE_EN                                  :  1;
        unsigned int                                                              : 15;
        unsigned int DEMUX_ARB1_MODE_OVERRIDE_EN                                  :  1;
        unsigned int                                                              : 13;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_GENERAL_CNTL {
    struct {
        unsigned int BURST_DISABLE                                                :  1;
        unsigned int VMID_BYPASS_ENABLE                                           : 16;
        unsigned int XBAR_MUX_CONFIG                                              :  2;
        unsigned int RB0_HARVEST_EN                                               :  1;
        unsigned int                                                              :  1;
        unsigned int LOOPBACK_DIS_BY_REQ_TYPE                                     :  4;
        unsigned int XBAR_MUX_CONFIG_UPDATE                                       :  1;
        unsigned int SKID_FIFO_0_OVERFLOW_ERROR_MASK                              :  1;
        unsigned int SKID_FIFO_0_UNDERFLOW_ERROR_MASK                             :  1;
        unsigned int                                                              :  2;
        unsigned int SKID_FIFO_FREESPACE_IS_ZERO_ERROR_MASK                       :  1;
        unsigned int                                                              :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 28;
        unsigned int SKID_FIFO_1_OVERFLOW_ERROR_MASK                              :  1;
        unsigned int SKID_FIFO_1_UNDERFLOW_ERROR_MASK                             :  1;
        unsigned int                                                              :  2;
    } gfx10;
    struct {
        unsigned int                                                              : 20;
        unsigned int RB1_HARVEST_EN                                               :  1;
        unsigned int                                                              : 11;
    } gfx101;
    struct {
        unsigned int                                                              : 20;
        unsigned int RB1_HARVEST_EN                                               :  1;
        unsigned int                                                              :  7;
        unsigned int SKID_FIFO_1_OVERFLOW_ERROR_MASK                              :  1;
        unsigned int SKID_FIFO_1_UNDERFLOW_ERROR_MASK                             :  1;
        unsigned int                                                              :  2;
    } rv1x;
    struct {
        unsigned int                                                              : 20;
        unsigned int RB1_HARVEST_EN                                               :  1;
        unsigned int                                                              :  7;
        unsigned int SKID_FIFO_1_OVERFLOW_ERROR_MASK                              :  1;
        unsigned int SKID_FIFO_1_UNDERFLOW_ERROR_MASK                             :  1;
        unsigned int                                                              :  2;
    } vega;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_GENERAL_CNTL1 {
    struct {
        unsigned int EARLY_WRACK_ENABLE_PER_MTYPE                                 :  4;
        unsigned int TCIW0_64B_RD_STALL_MODE                                      :  2;
        unsigned int                                                              :  2;
        unsigned int EARLY_WRACK_DISABLE_FOR_LOOPBACK                             :  1;
        unsigned int                                                              : 23;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  9;
        unsigned int POLICY_OVERRIDE_VALUE                                        :  1;
        unsigned int POLICY_OVERRIDE                                              :  1;
        unsigned int UTCL1_PROBE0_RR_ARB_BURST_HINT_EN                            :  1;
        unsigned int                                                              : 20;
    } gfx09;
    struct {
        unsigned int                                                              :  6;
        unsigned int TCIW1_64B_RD_STALL_MODE                                      :  2;
        unsigned int                                                              :  1;
        unsigned int POLICY_OVERRIDE_VALUE                                        :  2;
        unsigned int POLICY_OVERRIDE                                              :  1;
        unsigned int UTCL1_PROBE0_RR_ARB_BURST_HINT_EN                            :  1;
        unsigned int UTCL1_PROBE1_RR_ARB_BURST_HINT_EN                            :  1;
        unsigned int ARBITER_ADDRESS_CHANGE_ENABLE                                :  1;
        unsigned int LAST_OF_BURST_INSERTION_DISABLE                              :  1;
        unsigned int                                                              : 16;
    } gfx10;
    struct {
        unsigned int                                                              :  6;
        unsigned int TCIW1_64B_RD_STALL_MODE                                      :  2;
        unsigned int                                                              :  4;
        unsigned int UTCL1_PROBE1_RR_ARB_BURST_HINT_EN                            :  1;
        unsigned int                                                              : 19;
    } rv1x;
    struct {
        unsigned int                                                              :  6;
        unsigned int TCIW1_64B_RD_STALL_MODE                                      :  2;
        unsigned int                                                              :  4;
        unsigned int UTCL1_PROBE1_RR_ARB_BURST_HINT_EN                            :  1;
        unsigned int                                                              : 19;
    } vega;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_GENERAL_STATUS {
    struct {
        unsigned int GENERAL_RMI_ERRORS_COMBINED                                  :  1;
        unsigned int SKID_FIFO_0_OVERFLOW_ERROR                                   :  1;
        unsigned int SKID_FIFO_0_UNDERFLOW_ERROR                                  :  1;
        unsigned int                                                              :  2;
        unsigned int RMI_XBAR_BUSY                                                :  1;
        unsigned int RMI_UTCL1_BUSY                                               :  1;
        unsigned int RMI_SCOREBOARD_BUSY                                          :  1;
        unsigned int TCIW0_PRT_FIFO_BUSY                                          :  1;
        unsigned int TCIW_FRMTR0_BUSY                                             :  1;
        unsigned int TCIW_RTN_FRMTR0_BUSY                                         :  1;
        unsigned int WRREQ_CONSUMER_FIFO_0_BUSY                                   :  1;
        unsigned int RDREQ_CONSUMER_FIFO_0_BUSY                                   :  1;
        unsigned int                                                              :  6;
        unsigned int UTC_PROBE0_BUSY                                              :  1;
        unsigned int RMI_XNACK_BUSY                                               :  1;
        unsigned int XNACK_FIFO_NUM_USED                                          :  8;
        unsigned int XNACK_FIFO_EMPTY                                             :  1;
        unsigned int XNACK_FIFO_FULL                                              :  1;
        unsigned int SKID_FIFO_FREESPACE_IS_ZERO_ERROR                            :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  3;
        unsigned int SKID_FIFO_1_OVERFLOW_ERROR                                   :  1;
        unsigned int SKID_FIFO_1_UNDERFLOW_ERROR                                  :  1;
        unsigned int                                                              :  8;
        unsigned int TCIW1_PRT_FIFO_BUSY                                          :  1;
        unsigned int TCIW_FRMTR1_BUSY                                             :  1;
        unsigned int TCIW_RTN_FRMTR1_BUSY                                         :  1;
        unsigned int                                                              :  2;
        unsigned int UTC_PROBE1_BUSY                                              :  1;
        unsigned int                                                              : 13;
    } gfx10;
    struct {
        unsigned int                                                              : 16;
        unsigned int WRREQ_CONSUMER_FIFO_1_BUSY                                   :  1;
        unsigned int RDREQ_CONSUMER_FIFO_1_BUSY                                   :  1;
        unsigned int                                                              : 14;
    } gfx101;
    struct {
        unsigned int                                                              :  3;
        unsigned int SKID_FIFO_1_OVERFLOW_ERROR                                   :  1;
        unsigned int SKID_FIFO_1_UNDERFLOW_ERROR                                  :  1;
        unsigned int                                                              :  8;
        unsigned int TCIW1_PRT_FIFO_BUSY                                          :  1;
        unsigned int TCIW_FRMTR1_BUSY                                             :  1;
        unsigned int TCIW_RTN_FRMTR1_BUSY                                         :  1;
        unsigned int WRREQ_CONSUMER_FIFO_1_BUSY                                   :  1;
        unsigned int RDREQ_CONSUMER_FIFO_1_BUSY                                   :  1;
        unsigned int UTC_PROBE1_BUSY                                              :  1;
        unsigned int                                                              : 13;
    } rv1x;
    struct {
        unsigned int                                                              :  3;
        unsigned int SKID_FIFO_1_OVERFLOW_ERROR                                   :  1;
        unsigned int SKID_FIFO_1_UNDERFLOW_ERROR                                  :  1;
        unsigned int                                                              :  8;
        unsigned int TCIW1_PRT_FIFO_BUSY                                          :  1;
        unsigned int TCIW_FRMTR1_BUSY                                             :  1;
        unsigned int TCIW_RTN_FRMTR1_BUSY                                         :  1;
        unsigned int WRREQ_CONSUMER_FIFO_1_BUSY                                   :  1;
        unsigned int RDREQ_CONSUMER_FIFO_1_BUSY                                   :  1;
        unsigned int UTC_PROBE1_BUSY                                              :  1;
        unsigned int                                                              : 13;
    } vega;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              :  1;
        unsigned int PERF_SEL1                                                    :  9;
        unsigned int                                                              :  1;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    :  9;
        unsigned int                                                              :  1;
        unsigned int PERF_SEL3                                                    :  9;
        unsigned int                                                              :  5;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 19;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              :  1;
        unsigned int PERF_SEL1                                                    :  9;
        unsigned int                                                              :  1;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_PERFCOUNTER2_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    :  9;
        unsigned int                                                              :  1;
        unsigned int PERF_SEL3                                                    :  9;
        unsigned int                                                              :  5;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 19;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_PERF_COUNTER_CNTL {
    struct {
        unsigned int TRANS_BASED_PERF_EN_SEL                                      :  2;
        unsigned int EVENT_BASED_PERF_EN_SEL                                      :  2;
        unsigned int TC_PERF_EN_SEL                                               :  2;
        unsigned int PERF_EVENT_WINDOW_MASK0                                      :  2;
        unsigned int                                                              :  2;
        unsigned int PERF_COUNTER_CID                                             :  4;
        unsigned int PERF_COUNTER_VMID                                            :  5;
        unsigned int PERF_COUNTER_BURST_LENGTH_THRESHOLD                          :  6;
        unsigned int PERF_SOFT_RESET                                              :  1;
        unsigned int PERF_CNTR_SPM_SEL                                            :  1;
        unsigned int                                                              :  5;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  8;
        unsigned int PERF_EVENT_WINDOW_MASK1                                      :  2;
        unsigned int                                                              : 22;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_PROBE_POP_LOGIC_CNTL {
    struct {
        unsigned int EXT_LAT_FIFO_0_MAX_DEPTH                                     :  7;
        unsigned int XLAT_COMBINE0_DIS                                            :  1;
        unsigned int REDUCE_MAX_XLAT_CHAIN_SIZE_BY_2                              :  2;
        unsigned int                                                              : 22;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 10;
        unsigned int EXT_LAT_FIFO_1_MAX_DEPTH                                     :  7;
        unsigned int XLAT_COMBINE1_DIS                                            :  1;
        unsigned int                                                              : 14;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_RB_GLX_CID_MAP {
    struct {
        unsigned int CB_COLOR_MAP                                                 :  4;
        unsigned int CB_FMASK_MAP                                                 :  4;
        unsigned int CB_CMASK_MAP                                                 :  4;
        unsigned int CB_DCC_MAP                                                   :  4;
        unsigned int DB_Z_MAP                                                     :  4;
        unsigned int DB_S_MAP                                                     :  4;
        unsigned int DB_TILE_MAP                                                  :  4;
        unsigned int DB_ZPCPSD_MAP                                                :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_SCOREBOARD_CNTL {
    struct {
        unsigned int COMPLETE_RB0_FLUSH                                           :  1;
        unsigned int REQ_IN_RE_EN_AFTER_FLUSH_RB0                                 :  1;
        unsigned int                                                              :  3;
        unsigned int VMID_INVAL_FLUSH_TYPE_OVERRIDE_EN                            :  1;
        unsigned int VMID_INVAL_FLUSH_TYPE_OVERRIDE_VALUE                         :  1;
        unsigned int TIME_STAMP_FLUSH_RB0                                         :  1;
        unsigned int FORCE_VMID_INVAL_DONE_EN                                     :  1;
        unsigned int FORCE_VMID_INVAL_DONE_TIMER_START_VALUE                      : 12;
        unsigned int                                                              : 11;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  2;
        unsigned int COMPLETE_RB1_FLUSH                                           :  1;
        unsigned int REQ_IN_RE_EN_AFTER_FLUSH_RB1                                 :  1;
        unsigned int TIME_STAMP_FLUSH_RB1                                         :  1;
        unsigned int                                                              : 27;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_SCOREBOARD_STATUS0 {
    struct {
        unsigned int CURRENT_SESSION_ID                                           :  1;
        unsigned int CP_VMID_INV_IN_PROG                                          :  1;
        unsigned int CP_VMID_INV_REQ_VMID                                         : 16;
        unsigned int CP_VMID_INV_UTC_DONE                                         :  1;
        unsigned int CP_VMID_INV_DONE                                             :  1;
        unsigned int CP_VMID_INV_FLUSH_TYPE                                       :  1;
        unsigned int FORCE_VMID_INV_DONE                                          :  1;
        unsigned int                                                              : 10;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 22;
        unsigned int COUNTER_SELECT                                               :  5;
        unsigned int                                                              :  5;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_SCOREBOARD_STATUS1 {
    struct {
        unsigned int RUNNING_CNT_RB0                                              : 12;
        unsigned int RUNNING_CNT_UNDERFLOW_RB0                                    :  1;
        unsigned int RUNNING_CNT_OVERFLOW_RB0                                     :  1;
        unsigned int MULTI_VMID_INVAL_FROM_CP_DETECTED                            :  1;
        unsigned int                                                              : 15;
        unsigned int COM_FLUSH_IN_PROG_RB0                                        :  1;
        unsigned int                                                              :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 15;
        unsigned int RUNNING_CNT_RB1                                              : 12;
        unsigned int RUNNING_CNT_UNDERFLOW_RB1                                    :  1;
        unsigned int RUNNING_CNT_OVERFLOW_RB1                                     :  1;
        unsigned int COM_FLUSH_IN_PROG_RB1                                        :  1;
        unsigned int                                                              :  2;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_SCOREBOARD_STATUS2 {
    struct {
        unsigned int SNAPSHOT_CNT_RB0                                             : 12;
        unsigned int SNAPSHOT_CNT_UNDERFLOW_RB0                                   :  1;
        unsigned int                                                              : 14;
        unsigned int COM_FLUSH_DONE_RB0                                           :  1;
        unsigned int TIME_STAMP_FLUSH_IN_PROG_RB0                                 :  1;
        unsigned int                                                              :  1;
        unsigned int TIME_STAMP_FLUSH_DONE_RB0                                    :  1;
        unsigned int                                                              :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 13;
        unsigned int SNAPSHOT_CNT_RB1                                             : 12;
        unsigned int SNAPSHOT_CNT_UNDERFLOW_RB1                                   :  1;
        unsigned int COM_FLUSH_DONE_RB1                                           :  1;
        unsigned int                                                              :  2;
        unsigned int TIME_STAMP_FLUSH_IN_PROG_RB1                                 :  1;
        unsigned int                                                              :  1;
        unsigned int TIME_STAMP_FLUSH_DONE_RB1                                    :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_SPARE {
    struct {
        unsigned int RMI_ARBITER_STALL_TIMER_ENABLED_ALLOW_STREAMING              :  1;
        unsigned int                                                              :  6;
        unsigned int SPARE_BIT_7                                                  :  1;
        unsigned int                                                              : 24;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  1;
        unsigned int SPARE_BIT_1                                                  :  1;
        unsigned int SPARE_BIT_2                                                  :  1;
        unsigned int SPARE_BIT_3                                                  :  1;
        unsigned int SPARE_BIT_4                                                  :  1;
        unsigned int SPARE_BIT_5                                                  :  1;
        unsigned int SPARE_BIT_6                                                  :  1;
        unsigned int                                                              :  1;
        unsigned int SPARE_BIT_8_0                                                :  8;
        unsigned int SPARE_BIT_16_0                                               : 16;
    } gfx09;
    struct {
        unsigned int                                                              :  1;
        unsigned int RMI_2_GL1_128B_READ_DISABLE                                  :  1;
        unsigned int RMI_2_GL1_REPEATER_FGCG_DISABLE                              :  1;
        unsigned int                                                              : 13;
        unsigned int ARBITER_ADDRESS_MASK                                         : 16;
    } gfx10;
    struct {
        unsigned int                                                              :  3;
        unsigned int RMI_2_RB_REPEATER_FGCG_DISABLE                               :  1;
        unsigned int EARLY_WRITE_ACK_ENABLE_C_RW_NOA_RESOLVE_DIS                  :  1;
        unsigned int RMI_REORDER_BYPASS_CHANNEL_DIS                               :  1;
        unsigned int                                                              :  2;
        unsigned int NOFILL_RMI_CID_CC                                            :  1;
        unsigned int NOFILL_RMI_CID_FC                                            :  1;
        unsigned int NOFILL_RMI_CID_CM                                            :  1;
        unsigned int NOFILL_RMI_CID_DC                                            :  1;
        unsigned int NOFILL_RMI_CID_Z                                             :  1;
        unsigned int NOFILL_RMI_CID_S                                             :  1;
        unsigned int NOFILL_RMI_CID_TILE                                          :  1;
        unsigned int SPARE_BIT_15_0                                               :  1;
        unsigned int                                                              : 16;
    } gfx101Plus;
    struct {
        unsigned int                                                              :  6;
        unsigned int SPARE_BIT_6                                                  :  1;
        unsigned int                                                              : 25;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_SPARE_1 {
    struct {
        unsigned int                                                              :  1;
        unsigned int SPARE_BIT_9                                                  :  1;
        unsigned int SPARE_BIT_10                                                 :  1;
        unsigned int SPARE_BIT_11                                                 :  1;
        unsigned int SPARE_BIT_12                                                 :  1;
        unsigned int SPARE_BIT_13                                                 :  1;
        unsigned int SPARE_BIT_14                                                 :  1;
        unsigned int SPARE_BIT_15                                                 :  1;
        unsigned int                                                              :  8;
        unsigned int SPARE_BIT_16_1                                               : 16;
    } bits, bitfields;
    struct {
        unsigned int SPARE_BIT_8                                                  :  1;
        unsigned int                                                              :  7;
        unsigned int SPARE_BIT_8_1                                                :  8;
        unsigned int                                                              : 16;
    } gfx09;
    struct {
        unsigned int SPARE_BIT_8                                                  :  1;
        unsigned int                                                              : 31;
    } gfx101;
    struct {
        unsigned int                                                              :  8;
        unsigned int RMI_REORDER_DIS_BY_CID                                       :  8;
        unsigned int                                                              : 16;
    } gfx101Plus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_SPARE_2 {
    struct {
        unsigned int                                                              : 16;
        unsigned int SPARE_BIT_8_2                                                :  8;
        unsigned int SPARE_BIT_8_3                                                :  8;
    } bits, bitfields;
    struct {
        unsigned int SPARE_BIT_16                                                 :  1;
        unsigned int SPARE_BIT_17                                                 :  1;
        unsigned int SPARE_BIT_18                                                 :  1;
        unsigned int SPARE_BIT_19                                                 :  1;
        unsigned int SPARE_BIT_20                                                 :  1;
        unsigned int SPARE_BIT_21                                                 :  1;
        unsigned int SPARE_BIT_22                                                 :  1;
        unsigned int SPARE_BIT_23                                                 :  1;
        unsigned int SPARE_BIT_4_0                                                :  4;
        unsigned int SPARE_BIT_4_1                                                :  4;
        unsigned int                                                              : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_SUBBLOCK_STATUS0 {
    struct {
        unsigned int UTC_EXT_LAT_HID_FIFO_NUM_USED_PROBE0                         :  7;
        unsigned int UTC_EXT_LAT_HID_FIFO_FULL_PROBE0                             :  1;
        unsigned int UTC_EXT_LAT_HID_FIFO_EMPTY_PROBE0                            :  1;
        unsigned int                                                              :  9;
        unsigned int TCIW0_INFLIGHT_CNT                                           : 10;
        unsigned int                                                              :  4;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  9;
        unsigned int UTC_EXT_LAT_HID_FIFO_NUM_USED_PROBE1                         :  7;
        unsigned int UTC_EXT_LAT_HID_FIFO_FULL_PROBE1                             :  1;
        unsigned int UTC_EXT_LAT_HID_FIFO_EMPTY_PROBE1                            :  1;
        unsigned int                                                              : 14;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_SUBBLOCK_STATUS1 {
    struct {
        unsigned int SKID_FIFO_0_FREE_SPACE                                       : 10;
        unsigned int                                                              : 22;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 10;
        unsigned int SKID_FIFO_1_FREE_SPACE                                       : 10;
        unsigned int TCIW1_INFLIGHT_CNT                                           : 10;
        unsigned int                                                              :  2;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_SUBBLOCK_STATUS2 {
    struct {
        unsigned int PRT_FIFO_0_NUM_USED                                          :  9;
        unsigned int                                                              : 23;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  9;
        unsigned int PRT_FIFO_1_NUM_USED                                          :  9;
        unsigned int                                                              : 14;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_SUBBLOCK_STATUS3 {
    struct {
        unsigned int SKID_FIFO_0_FREE_SPACE_TOTAL                                 : 10;
        unsigned int                                                              : 22;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 10;
        unsigned int SKID_FIFO_1_FREE_SPACE_TOTAL                                 : 10;
        unsigned int                                                              : 12;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_TCIW_FORMATTER0_CNTL {
    struct {
        unsigned int WR_COMBINE0_DIS_OVERRIDE                                     :  1;
        unsigned int WR_COMBINE0_TIME_OUT_WINDOW                                  :  8;
        unsigned int TCIW0_MAX_ALLOWED_INFLIGHT_REQ                               : 10;
        unsigned int SKID_FIFO_0_FREE_SPACE_DELTA                                 :  8;
        unsigned int SKID_FIFO_0_FREE_SPACE_DELTA_UPDATE                          :  1;
        unsigned int TCIW0_REQ_SAFE_MODE                                          :  1;
        unsigned int RMI_IN0_REORDER_DIS                                          :  1;
        unsigned int WR_COMBINE0_DIS_AT_LAST_OF_BURST                             :  1;
        unsigned int ALL_FAULT_RET0_DATA                                          :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_TCIW_FORMATTER1_CNTL {
    struct {
        unsigned int WR_COMBINE1_DIS_OVERRIDE                                     :  1;
        unsigned int WR_COMBINE1_TIME_OUT_WINDOW                                  :  8;
        unsigned int TCIW1_MAX_ALLOWED_INFLIGHT_REQ                               : 10;
        unsigned int SKID_FIFO_1_FREE_SPACE_DELTA                                 :  8;
        unsigned int SKID_FIFO_1_FREE_SPACE_DELTA_UPDATE                          :  1;
        unsigned int TCIW1_REQ_SAFE_MODE                                          :  1;
        unsigned int RMI_IN1_REORDER_DIS                                          :  1;
        unsigned int WR_COMBINE1_DIS_AT_LAST_OF_BURST                             :  1;
        unsigned int ALL_FAULT_RET1_DATA                                          :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_UTCL1_CNTL1 {
    struct {
        unsigned int FORCE_4K_L2_RESP                                             :  1;
        unsigned int GPUVM_64K_DEF                                                :  1;
        unsigned int GPUVM_PERM_MODE                                              :  1;
        unsigned int RESP_MODE                                                    :  2;
        unsigned int RESP_FAULT_MODE                                              :  2;
        unsigned int CLIENTID                                                     :  9;
        unsigned int USERVM_DIS                                                   :  1;
        unsigned int ENABLE_PUSH_LFIFO                                            :  1;
        unsigned int ENABLE_LFIFO_PRI_ARB                                         :  1;
        unsigned int REG_INV_VMID                                                 :  4;
        unsigned int REG_INV_ALL_VMID                                             :  1;
        unsigned int REG_INV_TOGGLE                                               :  1;
        unsigned int CLIENT_INVALIDATE_ALL_VMID                                   :  1;
        unsigned int FORCE_MISS                                                   :  1;
        unsigned int FORCE_IN_ORDER                                               :  1;
        unsigned int REDUCE_FIFO_DEPTH_BY_2                                       :  2;
        unsigned int REDUCE_CACHE_SIZE_BY_2                                       :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_UTCL1_CNTL2 {
    struct {
        unsigned int UTC_SPARE                                                    :  8;
        unsigned int                                                              :  1;
        unsigned int MTYPE_OVRD_DIS                                               :  1;
        unsigned int LINE_VALID                                                   :  1;
        unsigned int DIS_EDC                                                      :  1;
        unsigned int GPUVM_INV_MODE                                               :  1;
        unsigned int SHOOTDOWN_OPT                                                :  1;
        unsigned int FORCE_SNOOP                                                  :  1;
        unsigned int FORCE_GPUVM_INV_ACK                                          :  1;
        unsigned int UTCL1_ARB_BURST_MODE                                         :  2;
        unsigned int UTCL1_ENABLE_PERF_EVENT_RD_WR                                :  1;
        unsigned int UTCL1_PERF_EVENT_RD_WR                                       :  1;
        unsigned int UTCL1_ENABLE_PERF_EVENT_VMID                                 :  1;
        unsigned int UTCL1_PERF_EVENT_VMID                                        :  4;
        unsigned int UTCL1_DIS_DUAL_L2_REQ                                        :  1;
        unsigned int UTCL1_FORCE_FRAG_2M_TO_64K                                   :  1;
        unsigned int                                                              :  5;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 27;
        unsigned int PERM_MODE_OVRD                                               :  1;
        unsigned int LINE_INVALIDATE_OPT                                          :  1;
        unsigned int GPUVM_16K_DEFAULT                                            :  1;
        unsigned int                                                              :  2;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_UTCL1_STATUS {
    struct {
        unsigned int FAULT_DETECTED                                               :  1;
        unsigned int RETRY_DETECTED                                               :  1;
        unsigned int PRT_DETECTED                                                 :  1;
        unsigned int                                                              : 29;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_UTC_XNACK_N_MISC_CNTL {
    struct {
        unsigned int MASTER_XNACK_TIMER_INC                                       :  8;
        unsigned int IND_XNACK_TIMER_START_VALUE                                  :  4;
        unsigned int UTCL1_PERM_MODE                                              :  1;
        unsigned int CP_VMID_RESET_REQUEST_DISABLE                                :  1;
        unsigned int                                                              : 18;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_XBAR_ARBITER_CONFIG {
    struct {
        unsigned int XBAR_ARB0_MODE                                               :  2;
        unsigned int XBAR_ARB0_BREAK_LOB_ON_WEIGHTEDRR                            :  1;
        unsigned int XBAR_ARB0_STALL                                              :  1;
        unsigned int XBAR_ARB0_BREAK_LOB_ON_IDLEIN                                :  1;
        unsigned int                                                              :  1;
        unsigned int XBAR_ARB0_STALL_TIMER_OVERRIDE                               :  2;
        unsigned int XBAR_ARB0_STALL_TIMER_START_VALUE                            :  8;
        unsigned int                                                              : 16;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 16;
        unsigned int XBAR_ARB1_MODE                                               :  2;
        unsigned int XBAR_ARB1_BREAK_LOB_ON_WEIGHTEDRR                            :  1;
        unsigned int XBAR_ARB1_STALL                                              :  1;
        unsigned int XBAR_ARB1_BREAK_LOB_ON_IDLEIN                                :  1;
        unsigned int                                                              :  1;
        unsigned int XBAR_ARB1_STALL_TIMER_OVERRIDE                               :  2;
        unsigned int XBAR_ARB1_STALL_TIMER_START_VALUE                            :  8;
    } most;
    struct {
        unsigned int                                                              :  5;
        unsigned int XBAR_ARB0_MODE_OVERRIDE_EN                                   :  1;
        unsigned int                                                              : 15;
        unsigned int XBAR_ARB1_MODE_OVERRIDE_EN                                   :  1;
        unsigned int                                                              : 10;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_XBAR_ARBITER_CONFIG_1 {
    struct {
        unsigned int XBAR_ARB_ROUND_ROBIN_WEIGHT_RB0_RD                           :  8;
        unsigned int XBAR_ARB_ROUND_ROBIN_WEIGHT_RB0_WR                           :  8;
        unsigned int                                                              : 16;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 16;
        unsigned int XBAR_ARB_ROUND_ROBIN_WEIGHT_RB1_RD                           :  8;
        unsigned int XBAR_ARB_ROUND_ROBIN_WEIGHT_RB1_WR                           :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_XBAR_CONFIG {
    struct {
        unsigned int XBAR_MUX_CONFIG_OVERRIDE                                     :  2;
        unsigned int XBAR_MUX_CONFIG_REQ_TYPE_OVERRIDE                            :  4;
        unsigned int XBAR_MUX_CONFIG_CB_DB_OVERRIDE                               :  1;
        unsigned int ARBITER_DIS                                                  :  1;
        unsigned int XBAR_EN_IN_REQ                                               :  4;
        unsigned int XBAR_EN_IN_REQ_OVERRIDE                                      :  1;
        unsigned int XBAR_EN_IN_RB0                                               :  1;
        unsigned int                                                              : 18;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 14;
        unsigned int XBAR_EN_IN_RB1                                               :  1;
        unsigned int                                                              : 17;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_XNACK_DEBUG {
    struct {
        unsigned int XNACK_PER_VMID                                               : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_ARB_CNTL {
    struct {
        unsigned int RD_SWITCH_NUM                                                :  8;
        unsigned int WR_SWITCH_NUM                                                :  8;
        unsigned int ATC_TR_SWITCH_NUM                                            :  8;
        unsigned int ARB_MODE                                                     :  1;
        unsigned int SWITCH_NUM_MODE                                              :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_ARB_CNTL2 {
    struct {
        unsigned int P2P_SWITCH_NUM                                               :  8;
        unsigned int ATOMIC_SWITCH_NUM                                            :  8;
        unsigned int ATC_PAGE_SWITCH_NUM                                          :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_ATS_CNTL {
    struct {
        unsigned int PAGE_MIN_LATENCY_ENABLE                                      :  1;
        unsigned int TR_MIN_LATENCY_ENABLE                                        :  1;
        unsigned int SWITCH_THRESHOLD                                             :  5;
        unsigned int TIME_SLICE                                                   :  8;
        unsigned int                                                              :  4;
        unsigned int ATCPAGE_SWITCH_NUM                                           :  4;
        unsigned int WR_AT                                                        :  2;
        unsigned int                                                              :  7;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 15;
        unsigned int ATCTR_SWITCH_NUM                                             :  4;
        unsigned int                                                              :  6;
        unsigned int INVAL_COM_CMD                                                :  6;
        unsigned int                                                              :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_ATS_CNTL2 {
    struct {
        unsigned int TRANS_CMD                                                    :  6;
        unsigned int PAGE_REQ_CMD                                                 :  6;
        unsigned int PAGE_ROUTING_CODE                                            :  3;
        unsigned int INVAL_COM_ROUTING_CODE                                       :  3;
        unsigned int VENDOR_ID                                                    :  2;
        unsigned int                                                              : 12;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_BIF_CNTL {
    struct {
        unsigned int VC0_SWITCH_NUM                                               :  8;
        unsigned int VC1_SWITCH_NUM                                               :  8;
        unsigned int                                                              : 16;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 16;
        unsigned int ARB_MODE                                                     :  1;
        unsigned int DRAIN_VC_NUM                                                 :  1;
        unsigned int SWITCH_ENABLE                                                :  1;
        unsigned int SWITCH_THRESHOLD                                             :  8;
        unsigned int PAGE_PRI_EN                                                  :  1;
        unsigned int TR_PRI_EN                                                    :  1;
        unsigned int VC0_CHAINED_OVERRIDE                                         :  1;
        unsigned int PARITY_CHECK_EN                                              :  1;
        unsigned int                                                              :  1;
    } most;
    struct {
        unsigned int                                                              : 31;
        unsigned int NBIF_DMA_ORIGCLKCTL_EN                                       :  1;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_BIF_CNTL2 {
    struct {
        unsigned int NBIF_HST_COMPCLKCTL_EN                                       :  1;
        unsigned int                                                              : 31;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_BLOCKLEVEL_CONF {
    struct {
        unsigned int XPB_BLOCKLEVEL_OVERRIDE                                      :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  2;
        unsigned int ATC_TR_BLOCKLEVEL                                            :  2;
        unsigned int ATC_PAGE_BLOCKLEVEL                                          :  2;
        unsigned int ATC_INV_BLOCKLEVEL                                           :  2;
        unsigned int IO_WR_BLOCKLEVEL_OVERRIDE                                    :  2;
        unsigned int IO_RD_BLOCKLEVEL_OVERRIDE                                    :  2;
        unsigned int ATOMIC_BLOCKLEVEL_OVERRIDE                                   :  2;
        unsigned int XPB_BLOCKLEVEL_OVERRIDE_EN                                   :  1;
        unsigned int IO_WR_BLOCKLEVEL_OVERRIDE_EN                                 :  1;
        unsigned int IO_RD_BLOCKLEVEL_OVERRIDE_EN                                 :  1;
        unsigned int ATOMIC_BLOCKLEVEL_OVERRIDE_EN                                :  1;
        unsigned int                                                              : 14;
    } nv10;
    struct {
        unsigned int                                                              :  2;
        unsigned int ATC_TR_BLOCKLEVEL                                            :  2;
        unsigned int ATC_PAGE_BLOCKLEVEL                                          :  2;
        unsigned int ATC_INV_BLOCKLEVEL                                           :  2;
        unsigned int IO_WR_BLOCKLEVEL_OVERRIDE                                    :  2;
        unsigned int IO_RD_BLOCKLEVEL_OVERRIDE                                    :  2;
        unsigned int ATOMIC_BLOCKLEVEL_OVERRIDE                                   :  2;
        unsigned int XPB_BLOCKLEVEL_OVERRIDE_EN                                   :  1;
        unsigned int IO_WR_BLOCKLEVEL_OVERRIDE_EN                                 :  1;
        unsigned int IO_RD_BLOCKLEVEL_OVERRIDE_EN                                 :  1;
        unsigned int ATOMIC_BLOCKLEVEL_OVERRIDE_EN                                :  1;
        unsigned int                                                              : 14;
    } rv1x;
    struct {
        unsigned int                                                              :  2;
        unsigned int ATC_TR_BLOCKLEVEL                                            :  2;
        unsigned int ATC_PAGE_BLOCKLEVEL                                          :  2;
        unsigned int ATC_INV_BLOCKLEVEL                                           :  2;
        unsigned int IO_WR_BLOCKLEVEL_OVERRIDE                                    :  2;
        unsigned int IO_RD_BLOCKLEVEL_OVERRIDE                                    :  2;
        unsigned int ATOMIC_BLOCKLEVEL_OVERRIDE                                   :  2;
        unsigned int XPB_BLOCKLEVEL_OVERRIDE_EN                                   :  1;
        unsigned int IO_WR_BLOCKLEVEL_OVERRIDE_EN                                 :  1;
        unsigned int IO_RD_BLOCKLEVEL_OVERRIDE_EN                                 :  1;
        unsigned int ATOMIC_BLOCKLEVEL_OVERRIDE_EN                                :  1;
        unsigned int                                                              : 14;
    } rv2x;
    struct {
        unsigned int                                                              :  2;
        unsigned int ATC_TR_BLOCKLEVEL                                            :  2;
        unsigned int ATC_PAGE_BLOCKLEVEL                                          :  2;
        unsigned int ATC_INV_BLOCKLEVEL                                           :  2;
        unsigned int IO_WR_BLOCKLEVEL_OVERRIDE                                    :  2;
        unsigned int IO_RD_BLOCKLEVEL_OVERRIDE                                    :  2;
        unsigned int ATOMIC_BLOCKLEVEL_OVERRIDE                                   :  2;
        unsigned int XPB_BLOCKLEVEL_OVERRIDE_EN                                   :  1;
        unsigned int IO_WR_BLOCKLEVEL_OVERRIDE_EN                                 :  1;
        unsigned int IO_RD_BLOCKLEVEL_OVERRIDE_EN                                 :  1;
        unsigned int ATOMIC_BLOCKLEVEL_OVERRIDE_EN                                :  1;
        unsigned int                                                              : 14;
    } vega;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_CID_QUEUE_EX {
    struct {
        unsigned int START                                                        :  1;
        unsigned int OFFSET                                                       :  8;
        unsigned int                                                              : 23;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_CID_QUEUE_EX_DATA {
    struct {
        unsigned int WRITE_ENTRIES                                                : 16;
        unsigned int READ_ENTRIES                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_CID_QUEUE_RD {
    struct {
        unsigned int CLIENT_ID_LOW                                                :  5;
        unsigned int CLIENT_ID_HIGH                                               :  6;
        unsigned int WRITE_QUEUE                                                  :  3;
        unsigned int READ_QUEUE                                                   :  3;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_CID_QUEUE_WR {
    struct {
        unsigned int CLIENT_ID_LOW                                                :  5;
        unsigned int CLIENT_ID_HIGH                                               :  6;
        unsigned int UPDATE_MODE                                                  :  1;
        unsigned int WRITE_QUEUE                                                  :  3;
        unsigned int READ_QUEUE                                                   :  3;
        unsigned int UPDATE                                                       :  1;
        unsigned int                                                              : 13;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_DEINTRLV_COMBINE_CNTL {
    struct {
        unsigned int WC_CHAINED_FLUSH_TIMER                                       :  4;
        unsigned int WC_CHAINED_BREAK_EN                                          :  1;
        unsigned int WC_HANDLE_CHECK_DISABLE                                      :  1;
        unsigned int                                                              : 26;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  6;
        unsigned int XPB_WRREQ_CRD                                                :  8;
        unsigned int                                                              : 18;
    } gfx101Plus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_DF_SDPPORT_CNTL {
    struct {
        unsigned int DF_REQ_CRD                                                   :  6;
        unsigned int DF_DATA_CRD                                                  :  6;
        unsigned int DF_HALT_THRESHOLD                                            :  4;
        unsigned int                                                              : 16;
    } gfx101Plus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_EA_QUEUE_WR {
    struct {
        unsigned int EA_NUMBER                                                    :  5;
        unsigned int WRITE_QUEUE                                                  :  3;
        unsigned int READ_QUEUE                                                   :  3;
        unsigned int UPDATE                                                       :  1;
        unsigned int                                                              : 20;
    } most;
    struct {
        unsigned int EA_NUMBER                                                    :  6;
        unsigned int WRITE_QUEUE                                                  :  3;
        unsigned int READ_QUEUE                                                   :  3;
        unsigned int UPDATE                                                       :  1;
        unsigned int                                                              : 19;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_EFF_CNTL {
    struct {
        unsigned int WR_LAZY_TIMER                                                :  8;
        unsigned int RD_LAZY_TIMER                                                :  8;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_PASSPW_CONF {
    struct {
        unsigned int XPB_PASSPW_OVERRIDE                                          :  1;
        unsigned int XPB_RSPPASSPW_OVERRIDE                                       :  1;
        unsigned int                                                              : 30;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  2;
        unsigned int ATC_TR_PASSPW_OVERRIDE                                       :  1;
        unsigned int ATC_PAGE_PASSPW_OVERRIDE                                     :  1;
        unsigned int WR_PASSPW_OVERRIDE                                           :  1;
        unsigned int RD_PASSPW_OVERRIDE                                           :  1;
        unsigned int WR_RSPPASSPW_OVERRIDE                                        :  1;
        unsigned int RD_RSPPASSPW_OVERRIDE                                        :  1;
        unsigned int ATC_RSPPASSPW_OVERRIDE                                       :  1;
        unsigned int ATOMIC_PASSPW_OVERRIDE                                       :  1;
        unsigned int ATOMIC_RSPPASSPW_OVERRIDE                                    :  1;
        unsigned int ATC_TR_PASSPW_OVERRIDE_EN                                    :  1;
        unsigned int ATC_PAGE_PASSPW_OVERRIDE_EN                                  :  1;
        unsigned int ATC_RSPPASSPW_OVERRIDE_EN                                    :  1;
        unsigned int WRRSP_PASSPW_OVERRIDE                                        :  1;
        unsigned int WRRSP_PASSPW_OVERRIDE_EN                                     :  1;
        unsigned int RDRSP_PASSPW_OVERRIDE                                        :  1;
        unsigned int RDRSP_PASSPW_OVERRIDE_EN                                     :  1;
        unsigned int                                                              : 14;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_PERFCOUNTER0_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_PERFCOUNTER1_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_PERFCOUNTER2_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_PERFCOUNTER3_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_PERFCOUNTER_HI {
    struct {
        unsigned int COUNTER_HI                                                   : 16;
        unsigned int COMPARE_VALUE                                                : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_PERFCOUNTER_LO {
    struct {
        unsigned int COUNTER_LO                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_PERFCOUNTER_RSLT_CNTL {
    struct {
        unsigned int PERF_COUNTER_SELECT                                          :  4;
        unsigned int                                                              :  4;
        unsigned int START_TRIGGER                                                :  8;
        unsigned int STOP_TRIGGER                                                 :  8;
        unsigned int ENABLE_ANY                                                   :  1;
        unsigned int CLEAR_ALL                                                    :  1;
        unsigned int STOP_ALL_ON_SATURATE                                         :  1;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_PERF_COUNTER_CNTL {
    struct {
        unsigned int PERF_COUNTER_SELECT                                          :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_RD_QUEUE_CNTL {
    struct {
        unsigned int ARB_MODE                                                     :  1;
        unsigned int Q4_SHARED                                                    :  1;
        unsigned int Q5_SHARED                                                    :  1;
        unsigned int Q4_UNITID_EA_MODE                                            :  1;
        unsigned int Q5_UNITID_EA_MODE                                            :  1;
        unsigned int Q4_PATTERN_LOW                                               :  5;
        unsigned int Q4_PATTERN_HIGH                                              :  6;
        unsigned int Q5_PATTERN_LOW                                               :  5;
        unsigned int Q5_PATTERN_HIGH                                              :  6;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_RD_QUEUE_CNTL2 {
    struct {
        unsigned int Q4_PATTERN_MASK_LOW                                          :  5;
        unsigned int Q4_PATTERN_MASK_HIGH                                         :  6;
        unsigned int Q5_PATTERN_MASK_LOW                                          :  5;
        unsigned int Q5_PATTERN_MASK_HIGH                                         :  6;
        unsigned int                                                              : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_RD_SWITCH_CNTL {
    struct {
        unsigned int QUEUE0_SWITCH_NUM                                            :  7;
        unsigned int QUEUE1_SWITCH_NUM                                            :  7;
        unsigned int QUEUE2_SWITCH_NUM                                            :  7;
        unsigned int QUEUE3_SWITCH_NUM                                            :  7;
        unsigned int SWITCH_NUM_MODE                                              :  1;
        unsigned int                                                              :  3;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 29;
        unsigned int WORKLOAD_ADJUST_EN                                           :  1;
        unsigned int WEIGHT_ADJUST_STEP                                           :  2;
    } gfx09_1xPlus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_SDPPORT_CNTL {
    struct {
        unsigned int NBIF_DMA_SELF_ACTIVATE                                       :  1;
        unsigned int NBIF_DMA_CFG_MODE                                            :  2;
        unsigned int NBIF_DMA_ENABLE_REISSUE_CREDIT                               :  1;
        unsigned int NBIF_DMA_ENABLE_SATURATE_COUNTER                             :  1;
        unsigned int NBIF_DMA_ENABLE_DISRUPT_FULLDIS                              :  1;
        unsigned int NBIF_DMA_HALT_THRESHOLD                                      :  4;
        unsigned int NBIF_HST_SELF_ACTIVATE                                       :  1;
        unsigned int NBIF_HST_CFG_MODE                                            :  2;
        unsigned int NBIF_HST_ENABLE_REISSUE_CREDIT                               :  1;
        unsigned int NBIF_HST_ENABLE_SATURATE_COUNTER                             :  1;
        unsigned int NBIF_HST_ENABLE_DISRUPT_FULLDIS                              :  1;
        unsigned int NBIF_HST_HALT_THRESHOLD                                      :  4;
        unsigned int NBIF_HST_PASSIVE_MODE                                        :  1;
        unsigned int NBIF_HST_QUICK_COMACK                                        :  1;
        unsigned int DF_SDPVDCI_RDRSPCKEN                                         :  1;
        unsigned int DF_SDPVDCI_RDRSPCKENRCV                                      :  1;
        unsigned int DF_SDPVDCI_RDRSPDATACKEN                                     :  1;
        unsigned int DF_SDPVDCI_RDRSPDATACKENRCV                                  :  1;
        unsigned int DF_SDPVDCI_WRRSPCKEN                                         :  1;
        unsigned int DF_SDPVDCI_WRRSPCKENRCV                                      :  1;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_SWITCH_CNTL2 {
    struct {
        unsigned int RD_QUEUE4_SWITCH_NUM                                         :  7;
        unsigned int RD_QUEUE5_SWITCH_NUM                                         :  7;
        unsigned int WR_QUEUE4_SWITCH_NUM                                         :  7;
        unsigned int WR_QUEUE5_SWITCH_NUM                                         :  7;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_TAG_CONF {
    struct {
        unsigned int RPB_ATS_TR                                                   :  8;
        unsigned int RPB_IO_WR                                                    :  8;
        unsigned int RPB_ATS_PR                                                   :  8;
        unsigned int                                                              :  8;
    } most;
    struct {
        unsigned int RPB_ATS_TR                                                   : 10;
        unsigned int RPB_IO_WR                                                    : 10;
        unsigned int RPB_ATS_PR                                                   : 10;
        unsigned int                                                              :  2;
    } nv10;
    struct {
        unsigned int RPB_ATS_TR                                                   : 10;
        unsigned int RPB_IO_WR                                                    : 10;
        unsigned int RPB_ATS_PR                                                   : 10;
        unsigned int                                                              :  2;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_VC_SWITCH_RDWR {
    struct {
        unsigned int MODE                                                         :  2;
        unsigned int NUM_RD                                                       :  8;
        unsigned int NUM_WR                                                       :  8;
        unsigned int                                                              : 14;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 18;
        unsigned int XPB_RDREQ_CRD                                                :  8;
        unsigned int                                                              :  6;
    } gfx101Plus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_WR_QUEUE_CNTL {
    struct {
        unsigned int ARB_MODE                                                     :  1;
        unsigned int Q4_SHARED                                                    :  1;
        unsigned int Q5_SHARED                                                    :  1;
        unsigned int Q4_UNITID_EA_MODE                                            :  1;
        unsigned int Q5_UNITID_EA_MODE                                            :  1;
        unsigned int Q4_PATTERN_LOW                                               :  5;
        unsigned int Q4_PATTERN_HIGH                                              :  6;
        unsigned int Q5_PATTERN_LOW                                               :  5;
        unsigned int Q5_PATTERN_HIGH                                              :  6;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_WR_QUEUE_CNTL2 {
    struct {
        unsigned int Q4_PATTERN_MASK_LOW                                          :  5;
        unsigned int Q4_PATTERN_MASK_HIGH                                         :  6;
        unsigned int Q5_PATTERN_MASK_LOW                                          :  5;
        unsigned int Q5_PATTERN_MASK_HIGH                                         :  6;
        unsigned int                                                              : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_WR_SWITCH_CNTL {
    struct {
        unsigned int QUEUE0_SWITCH_NUM                                            :  7;
        unsigned int QUEUE1_SWITCH_NUM                                            :  7;
        unsigned int QUEUE2_SWITCH_NUM                                            :  7;
        unsigned int QUEUE3_SWITCH_NUM                                            :  7;
        unsigned int SWITCH_NUM_MODE                                              :  1;
        unsigned int                                                              :  3;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 29;
        unsigned int WORKLOAD_ADJUST_EN                                           :  1;
        unsigned int WEIGHT_ADJUST_STEP                                           :  2;
    } gfx09_1xPlus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SCRATCH_ADDR {
    struct {
        unsigned int OBSOLETE_ADDR                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SCRATCH_REG0 {
    struct {
        unsigned int SCRATCH_REG0                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SCRATCH_REG1 {
    struct {
        unsigned int SCRATCH_REG1                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SCRATCH_REG2 {
    struct {
        unsigned int SCRATCH_REG2                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SCRATCH_REG3 {
    struct {
        unsigned int SCRATCH_REG3                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SCRATCH_REG4 {
    struct {
        unsigned int SCRATCH_REG4                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SCRATCH_REG5 {
    struct {
        unsigned int SCRATCH_REG5                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SCRATCH_REG6 {
    struct {
        unsigned int SCRATCH_REG6                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SCRATCH_REG7 {
    struct {
        unsigned int SCRATCH_REG7                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SCRATCH_REGISTER0 {
    struct {
        unsigned int ScratchPad0                                                  : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SCRATCH_REGISTER1 {
    struct {
        unsigned int ScratchPad1                                                  : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SCRATCH_REGISTER2 {
    struct {
        unsigned int ScratchPad2                                                  : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SCRATCH_REGISTER3 {
    struct {
        unsigned int ScratchPad3                                                  : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SCRATCH_REGISTER4 {
    struct {
        unsigned int ScratchPad4                                                  : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SCRATCH_REGISTER5 {
    struct {
        unsigned int ScratchPad5                                                  : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SCRATCH_REGISTER6 {
    struct {
        unsigned int ScratchPad6                                                  : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SCRATCH_REGISTER7 {
    struct {
        unsigned int ScratchPad7                                                  : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SCRATCH_UMSK {
    struct {
        unsigned int OBSOLETE_UMSK                                                :  8;
        unsigned int                                                              :  8;
        unsigned int OBSOLETE_SWAP                                                :  2;
        unsigned int                                                              : 14;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA0_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA0_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA0_PERFCOUNTER0_RESULT {
    struct {
        unsigned int PERF_COUNT                                                   : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA0_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA0_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA0_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA0_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA0_PERFCOUNTER1_RESULT {
    struct {
        unsigned int PERF_COUNT                                                   : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA0_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA0_PERFCOUNTER1_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA0_PERFCOUNTER_TAG_DELAY_RANGE {
    struct {
        unsigned int RANGE_LOW                                                    : 14;
        unsigned int RANGE_HIGH                                                   : 14;
        unsigned int SELECT_RW                                                    :  1;
        unsigned int                                                              :  3;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA0_PERFMON_CNTL {
    struct {
        unsigned int PERF_ENABLE0                                                 :  1;
        unsigned int PERF_CLEAR0                                                  :  1;
        unsigned int PERF_SEL0                                                    :  8;
        unsigned int PERF_ENABLE1                                                 :  1;
        unsigned int PERF_CLEAR1                                                  :  1;
        unsigned int PERF_SEL1                                                    :  8;
        unsigned int                                                              : 12;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA1_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } oss50;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA1_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } oss50;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA1_PERFCOUNTER0_RESULT {
    struct {
        unsigned int PERF_COUNT                                                   : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA1_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } oss50;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA1_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } oss50;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA1_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } oss50;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA1_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } oss50;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA1_PERFCOUNTER1_RESULT {
    struct {
        unsigned int PERF_COUNT                                                   : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA1_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } oss50;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA1_PERFCOUNTER1_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } oss50;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA1_PERFCOUNTER_TAG_DELAY_RANGE {
    struct {
        unsigned int RANGE_LOW                                                    : 14;
        unsigned int RANGE_HIGH                                                   : 14;
        unsigned int SELECT_RW                                                    :  1;
        unsigned int                                                              :  3;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA1_PERFMON_CNTL {
    struct {
        unsigned int PERF_ENABLE0                                                 :  1;
        unsigned int PERF_CLEAR0                                                  :  1;
        unsigned int PERF_SEL0                                                    :  8;
        unsigned int PERF_ENABLE1                                                 :  1;
        unsigned int PERF_CLEAR1                                                  :  1;
        unsigned int PERF_SEL1                                                    :  8;
        unsigned int                                                              : 12;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA_PGFSM_CONFIG {
    struct {
        unsigned int FSM_ADDR                                                     :  8;
        unsigned int POWER_DOWN                                                   :  1;
        unsigned int POWER_UP                                                     :  1;
        unsigned int P1_SELECT                                                    :  1;
        unsigned int P2_SELECT                                                    :  1;
        unsigned int WRITE                                                        :  1;
        unsigned int READ                                                         :  1;
        unsigned int                                                              : 13;
        unsigned int SRBM_OVERRIDE                                                :  1;
        unsigned int REG_ADDR                                                     :  4;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA_PGFSM_READ {
    struct {
        unsigned int VALUE                                                        : 24;
        unsigned int                                                              :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA_PGFSM_WRITE {
    struct {
        unsigned int VALUE                                                        : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA_POWER_GATING {
    struct {
        unsigned int SDMA0_POWER_OFF_CONDITION                                    :  1;
        unsigned int SDMA0_POWER_ON_CONDITION                                     :  1;
        unsigned int SDMA0_POWER_OFF_REQ                                          :  1;
        unsigned int SDMA0_POWER_ON_REQ                                           :  1;
        unsigned int PG_CNTL_STATUS                                               :  2;
        unsigned int                                                              : 26;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SH_MEM_BASES {
    struct {
        unsigned int PRIVATE_BASE                                                 : 16;
        unsigned int SHARED_BASE                                                  : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SH_MEM_CONFIG {
    struct {
        unsigned int ADDRESS_MODE                                                 :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  3;
        unsigned int ALIGNMENT_MODE                                               :  2;
        unsigned int                                                              :  7;
        unsigned int RETRY_DISABLE                                                :  1;
        unsigned int PRIVATE_NV                                                   :  1;
        unsigned int                                                              : 18;
    } gfx09;
    struct {
        unsigned int                                                              :  2;
        unsigned int ALIGNMENT_MODE                                               :  2;
        unsigned int DEFAULT_MTYPE                                                :  3;
        unsigned int                                                              :  7;
        unsigned int INITIAL_INST_PREFETCH                                        :  2;
        unsigned int                                                              :  2;
        unsigned int ICACHE_USE_GL1                                               :  1;
        unsigned int                                                              : 13;
    } gfx10;
    struct {
        unsigned int                                                              : 12;
        unsigned int RETRY_MODE                                                   :  2;
        unsigned int                                                              :  2;
        unsigned int NO_PREFETCH_ACROSS_PAGE                                      :  1;
        unsigned int ILLEGAL_INST_CHECK_DISABLE                                   :  1;
        unsigned int                                                              : 14;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SOCCLK_CGTT_BLK_CTRL_REG {
    struct {
        unsigned int SOCCLK_TURN_ON_DELAY                                         :  4;
        unsigned int SOCCLK_TURN_OFF_DELAY                                        :  8;
        unsigned int                                                              : 20;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SOC_GAP_PWROK {
    struct {
        unsigned int soc_gap_pwrok                                                :  1;
        unsigned int                                                              : 31;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPIS_DEBUG_READ {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_ARB_CNTL_0 {
    struct {
        unsigned int EXP_ARB_COL_WT                                               :  4;
        unsigned int EXP_ARB_POS_WT                                               :  4;
        unsigned int EXP_ARB_GDS_WT                                               :  4;
        unsigned int                                                              : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_ARB_CYCLES_0 {
    struct {
        unsigned int TS0_DURATION                                                 : 16;
        unsigned int TS1_DURATION                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_ARB_CYCLES_1 {
    struct {
        unsigned int TS2_DURATION                                                 : 16;
        unsigned int TS3_DURATION                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_ARB_PRIORITY {
    struct {
        unsigned int PIPE_ORDER_TS0                                               :  3;
        unsigned int PIPE_ORDER_TS1                                               :  3;
        unsigned int PIPE_ORDER_TS2                                               :  3;
        unsigned int PIPE_ORDER_TS3                                               :  3;
        unsigned int TS0_DUR_MULT                                                 :  2;
        unsigned int TS1_DUR_MULT                                                 :  2;
        unsigned int TS2_DUR_MULT                                                 :  2;
        unsigned int TS3_DUR_MULT                                                 :  2;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_BARYC_CNTL {
    struct {
        unsigned int PERSP_CENTER_CNTL                                            :  1;
        unsigned int                                                              :  3;
        unsigned int PERSP_CENTROID_CNTL                                          :  1;
        unsigned int                                                              :  3;
        unsigned int LINEAR_CENTER_CNTL                                           :  1;
        unsigned int                                                              :  3;
        unsigned int LINEAR_CENTROID_CNTL                                         :  1;
        unsigned int                                                              :  3;
        unsigned int POS_FLOAT_LOCATION                                           :  2;
        unsigned int                                                              :  2;
        unsigned int POS_FLOAT_ULC                                                :  1;
        unsigned int                                                              :  3;
        unsigned int FRONT_FACE_ALL_BITS                                          :  1;
        unsigned int                                                              :  7;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_clk_gate0 {
    struct {
        unsigned int read_ack_out                                                 :  1;
        unsigned int program_ack_out                                              :  1;
        unsigned int sm_busy_out                                                  :  1;
        unsigned int force_data_out                                               :  1;
        unsigned int data_out                                                     :  1;
        unsigned int valid_out                                                    :  1;
        unsigned int state_out                                                    :  1;
        unsigned int program_out                                                  :  1;
        unsigned int curr_sm_state                                                :  5;
        unsigned int off_cmd                                                      :  1;
        unsigned int all_clks_on_flag_in                                          :  1;
        unsigned int off_flag_in                                                  :  1;
        unsigned int read_flag_in                                                 :  1;
        unsigned int program_flag_in                                              :  1;
        unsigned int on_cmd                                                       :  1;
        unsigned int on_flag_in                                                   :  1;
        unsigned int force_override_in                                            :  1;
        unsigned int cgtt_reg_oclk_vld                                            :  1;
        unsigned int cgtt_dyn_oclk_vld                                            :  1;
        unsigned int Reserved0                                                    :  9;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_clk_gate1 {
    struct {
        unsigned int off_seq_cnt_eq0                                              :  1;
        unsigned int off_seq_cnt_decr                                             :  1;
        unsigned int off_seq_cnt_ld                                               :  1;
        unsigned int on_seq_cnt_eq0                                               :  1;
        unsigned int on_seq_cnt_decr                                              :  1;
        unsigned int on_seq_cnt_ld                                                :  1;
        unsigned int blk_row_cnt_last                                             :  1;
        unsigned int mxn_bit_cnt_last                                             :  1;
        unsigned int cu_cnt_out                                                   :  4;
        unsigned int blk_row_cnt_out                                              :  4;
        unsigned int blk_row_cnt_sel_out                                          :  1;
        unsigned int Reserved1                                                    :  1;
        unsigned int mxn_bit_reg_ld_out                                           :  1;
        unsigned int mxn_bit_reg_shift_out                                        :  1;
        unsigned int Reserved0                                                    : 12;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_clk_gate2 {
    struct {
        unsigned int on_monitor_cnt_10_0                                          : 11;
        unsigned int clkgate_all_on_out                                           :  1;
        unsigned int spi_active_in                                                :  1;
        unsigned int data_out                                                     :  1;
        unsigned int on_monitor_flag                                              :  1;
        unsigned int off_seq_decode                                               :  1;
        unsigned int on_seq_decode                                                :  1;
        unsigned int ctrl_ls_override                                             :  1;
        unsigned int rss_clkgate_en_combined                                      :  1;
        unsigned int rss_cnt_eq0                                                  :  1;
        unsigned int rss_cnt_ld                                                   :  1;
        unsigned int Reserved0                                                    : 11;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_clk_gate3 {
    struct {
        unsigned int rd_row_mux_sel                                               :  5;
        unsigned int rd_reg_loaded_dummy                                          :  1;
        unsigned int rdbus_data                                                   :  1;
        unsigned int rdbus_valid                                                  :  1;
        unsigned int rd_reg_loaded                                                : 10;
        unsigned int Reserved0                                                    : 14;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_COMPUTE_QUEUE_RESET {
    struct {
        unsigned int RESET                                                        :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_COMPUTE_WF_CTX_SAVE {
    struct {
        unsigned int INITIATE                                                     :  1;
        unsigned int GDS_INTERRUPT_EN                                             :  1;
        unsigned int DONE_INTERRUPT_EN                                            :  1;
        unsigned int                                                              : 27;
        unsigned int GDS_REQ_BUSY                                                 :  1;
        unsigned int SAVE_BUSY                                                    :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_CONFIG_CNTL {
    struct {
        unsigned int GPR_WRITE_PRIORITY                                           : 21;
        unsigned int EXP_PRIORITY_ORDER                                           :  3;
        unsigned int ENABLE_SQG_TOP_EVENTS                                        :  1;
        unsigned int ENABLE_SQG_BOP_EVENTS                                        :  1;
        unsigned int                                                              :  1;
        unsigned int TTRACE_STALL_ALL                                             :  1;
        unsigned int ALLOC_ARB_LRU_ENA                                            :  1;
        unsigned int EXP_ARB_LRU_ENA                                              :  1;
        unsigned int PS_PKR_PRIORITY_CNTL                                         :  2;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 26;
        unsigned int RSRC_MGMT_RESET                                              :  1;
        unsigned int                                                              :  5;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_CONFIG_CNTL_1 {
    struct {
        unsigned int VTX_DONE_DELAY                                               :  4;
        unsigned int INTERP_ONE_PRIM_PER_ROW                                      :  1;
        unsigned int                                                              :  2;
        unsigned int PC_LIMIT_STRICT                                              :  1;
        unsigned int CRC_SIMD_ID_WADDR_DISABLE                                    :  1;
        unsigned int LBPW_CU_CHK_MODE                                             :  1;
        unsigned int LBPW_CU_CHK_CNT                                              :  4;
        unsigned int CSC_PWR_SAVE_DISABLE                                         :  1;
        unsigned int CSG_PWR_SAVE_DISABLE                                         :  1;
        unsigned int                                                              : 16;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  5;
        unsigned int BATON_RESET_DISABLE                                          :  1;
        unsigned int PC_LIMIT_ENABLE                                              :  1;
        unsigned int                                                              :  9;
        unsigned int PC_LIMIT_SIZE                                                : 16;
    } gfx09;
    struct {
        unsigned int                                                              :  5;
        unsigned int PC_LIMIT_ENABLE                                              :  2;
        unsigned int                                                              :  9;
        unsigned int MAX_VTX_SYNC_CNT                                             :  5;
        unsigned int                                                              : 11;
    } gfx10;
    struct {
        unsigned int                                                              : 21;
        unsigned int EN_USER_ACCUM                                                :  1;
        unsigned int                                                              : 10;
    } gfx101Plus;
    struct {
        unsigned int                                                              : 22;
        unsigned int RESERVED                                                     : 10;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_CONFIG_CNTL_1_REMAP {
    struct {
        unsigned int RESERVED                                                     : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_CONFIG_CNTL_2 {
    struct {
        unsigned int CONTEXT_SAVE_WAIT_GDS_REQUEST_CYCLE_OVHD                     :  4;
        unsigned int CONTEXT_SAVE_WAIT_GDS_GRANT_CYCLE_OVHD                       :  4;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_CONFIG_CNTL_2_REMAP {
    struct {
        unsigned int RESERVED                                                     : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_CONFIG_CNTL_REMAP {
    struct {
        unsigned int RESERVED                                                     : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_CONFIG_PS_CU_EN {
    struct {
        unsigned int ENABLE                                                       :  1;
        unsigned int PKR0_CU_EN                                                   : 15;
        unsigned int PKR1_CU_EN                                                   : 16;
    } gfx09;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_CSQ_WF_ACTIVE_COUNT_0 {
    struct {
        unsigned int COUNT                                                        : 11;
        unsigned int                                                              :  5;
        unsigned int EVENTS                                                       : 11;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_CSQ_WF_ACTIVE_COUNT_1 {
    struct {
        unsigned int COUNT                                                        : 11;
        unsigned int                                                              :  5;
        unsigned int EVENTS                                                       : 11;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_CSQ_WF_ACTIVE_COUNT_2 {
    struct {
        unsigned int COUNT                                                        : 11;
        unsigned int                                                              :  5;
        unsigned int EVENTS                                                       : 11;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_CSQ_WF_ACTIVE_COUNT_3 {
    struct {
        unsigned int COUNT                                                        : 11;
        unsigned int                                                              :  5;
        unsigned int EVENTS                                                       : 11;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_CSQ_WF_ACTIVE_COUNT_4 {
    struct {
        unsigned int COUNT                                                        : 11;
        unsigned int                                                              :  5;
        unsigned int EVENTS                                                       : 11;
        unsigned int                                                              :  5;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_CSQ_WF_ACTIVE_COUNT_5 {
    struct {
        unsigned int COUNT                                                        : 11;
        unsigned int                                                              :  5;
        unsigned int EVENTS                                                       : 11;
        unsigned int                                                              :  5;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_CSQ_WF_ACTIVE_COUNT_6 {
    struct {
        unsigned int COUNT                                                        : 11;
        unsigned int                                                              :  5;
        unsigned int EVENTS                                                       : 11;
        unsigned int                                                              :  5;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_CSQ_WF_ACTIVE_COUNT_7 {
    struct {
        unsigned int COUNT                                                        : 11;
        unsigned int                                                              :  5;
        unsigned int EVENTS                                                       : 11;
        unsigned int                                                              :  5;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_CSQ_WF_ACTIVE_STATUS {
    struct {
        unsigned int ACTIVE                                                       : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_cs_ctl0 {
    struct {
        unsigned int Reserved3                                                    :  7;
        unsigned int remaining_threads0                                           :  7;
        unsigned int Reserved2                                                    :  1;
        unsigned int csc_ca_last_wave                                             :  1;
        unsigned int Reserved1                                                    :  1;
        unsigned int csc_ca_first_wave                                            :  1;
        unsigned int csdata_is_state                                              :  1;
        unsigned int csdata_is_event                                              :  1;
        unsigned int cpc_csdata_fifo_empty                                        :  1;
        unsigned int cpc_csdata_fifo_full                                         :  1;
        unsigned int Reserved0                                                    : 10;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_cs_ctl1 {
    struct {
        unsigned int crw_event_id                                                 :  6;
        unsigned int crw_event_valid                                              :  1;
        unsigned int stall_events                                                 :  1;
        unsigned int wave_done_crawler_id                                         : 10;
        unsigned int wave_done                                                    :  1;
        unsigned int crawler_full                                                 :  1;
        unsigned int crawler_empty                                                :  1;
        unsigned int event_count                                                  :  3;
        unsigned int Reserved0                                                    :  8;
    } gfx09_0;
    struct {
        unsigned int crw_event_id                                                 :  6;
        unsigned int crw_event_valid                                              :  1;
        unsigned int stall_events                                                 :  1;
        unsigned int wave_done_crawler_id                                         : 10;
        unsigned int wave_done                                                    :  1;
        unsigned int crawler_full                                                 :  1;
        unsigned int crawler_empty                                                :  1;
        unsigned int event_count                                                  :  3;
        unsigned int Reserved0                                                    :  8;
    } rv2x;
    struct {
        unsigned int crw_event_id                                                 :  6;
        unsigned int crw_event_valid                                              :  1;
        unsigned int stall_events                                                 :  1;
        unsigned int wave_done_crawler_id                                         : 10;
        unsigned int wave_done                                                    :  1;
        unsigned int crawler_full                                                 :  1;
        unsigned int crawler_empty                                                :  1;
        unsigned int event_count                                                  :  3;
        unsigned int Reserved0                                                    :  8;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_cs_ctl2 {
    struct {
        unsigned int send_to_shader_array_0                                       :  1;
        unsigned int wave_count                                                   :  6;
        unsigned int waves_sent0_d                                                :  6;
        unsigned int Reserved3                                                    :  1;
        unsigned int stall0                                                       :  1;
        unsigned int Reserved2                                                    :  1;
        unsigned int res_alloc_req0                                               :  1;
        unsigned int Reserved1                                                    :  3;
        unsigned int current_state0_q                                             :  3;
        unsigned int Reserved0                                                    :  9;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_cs_ctl_gfx0 {
    struct {
        unsigned int UNUSED_remaining_threads1                                    :  7;
        unsigned int remaining_threads0                                           :  7;
        unsigned int UNUSED_csc_ra1_last_wave                                     :  1;
        unsigned int csc_ra0_last_wave                                            :  1;
        unsigned int UNUSED_csc_ra1_first_wave                                    :  1;
        unsigned int csc_ra0_first_wave                                           :  1;
        unsigned int csdata_is_state                                              :  1;
        unsigned int csdata_is_event                                              :  1;
        unsigned int cpg_csdata_fifo_empty                                        :  1;
        unsigned int cpg_csdata_fifo_full                                         :  1;
        unsigned int Reserved0                                                    : 10;
    } gfx09_0;
    struct {
        unsigned int UNUSED_remaining_threads1                                    :  7;
        unsigned int remaining_threads0                                           :  7;
        unsigned int UNUSED_csc_ra1_last_wave                                     :  1;
        unsigned int csc_ra0_last_wave                                            :  1;
        unsigned int UNUSED_csc_ra1_first_wave                                    :  1;
        unsigned int csc_ra0_first_wave                                           :  1;
        unsigned int csdata_is_state                                              :  1;
        unsigned int csdata_is_event                                              :  1;
        unsigned int cpg_csdata_fifo_empty                                        :  1;
        unsigned int cpg_csdata_fifo_full                                         :  1;
        unsigned int Reserved0                                                    : 10;
    } rv2x;
    struct {
        unsigned int UNUSED_remaining_threads1                                    :  7;
        unsigned int remaining_threads0                                           :  7;
        unsigned int UNUSED_csc_ra1_last_wave                                     :  1;
        unsigned int csc_ra0_last_wave                                            :  1;
        unsigned int UNUSED_csc_ra1_first_wave                                    :  1;
        unsigned int csc_ra0_first_wave                                           :  1;
        unsigned int csdata_is_state                                              :  1;
        unsigned int csdata_is_event                                              :  1;
        unsigned int cpg_csdata_fifo_empty                                        :  1;
        unsigned int cpg_csdata_fifo_full                                         :  1;
        unsigned int Reserved0                                                    : 10;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_cs_ctl_gfx1 {
    struct {
        unsigned int crw_event_id                                                 :  6;
        unsigned int crw_event_valid                                              :  1;
        unsigned int stall_events                                                 :  1;
        unsigned int wave_done_crawler_id                                         : 10;
        unsigned int wave_done                                                    :  1;
        unsigned int crawler_full                                                 :  1;
        unsigned int crawler_empty                                                :  1;
        unsigned int event_count                                                  :  3;
        unsigned int Reserved0                                                    :  8;
    } gfx09_0;
    struct {
        unsigned int crw_event_id                                                 :  6;
        unsigned int crw_event_valid                                              :  1;
        unsigned int stall_events                                                 :  1;
        unsigned int wave_done_crawler_id                                         : 10;
        unsigned int wave_done                                                    :  1;
        unsigned int crawler_full                                                 :  1;
        unsigned int crawler_empty                                                :  1;
        unsigned int event_count                                                  :  3;
        unsigned int Reserved0                                                    :  8;
    } rv2x;
    struct {
        unsigned int crw_event_id                                                 :  6;
        unsigned int crw_event_valid                                              :  1;
        unsigned int stall_events                                                 :  1;
        unsigned int wave_done_crawler_id                                         : 10;
        unsigned int wave_done                                                    :  1;
        unsigned int crawler_full                                                 :  1;
        unsigned int crawler_empty                                                :  1;
        unsigned int event_count                                                  :  3;
        unsigned int Reserved0                                                    :  8;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_cs_ctl_gfx2 {
    struct {
        unsigned int send_to_shader_array_0                                       :  1;
        unsigned int wave_count                                                   :  6;
        unsigned int waves_sent0_d                                                :  6;
        unsigned int UNUSED_stall1                                                :  1;
        unsigned int stall0                                                       :  1;
        unsigned int UNUSED_res_alloc_req1                                        :  1;
        unsigned int res_alloc_req0                                               :  1;
        unsigned int UNUSED_current_state1_q                                      :  3;
        unsigned int current_state0_q                                             :  3;
        unsigned int Reserved0                                                    :  9;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_cs_wave_ctl {
    struct {
        unsigned int tr_slots_used                                                :  9;
        unsigned int loader_state_q                                               :  3;
        unsigned int halt_state_q                                                 :  3;
        unsigned int csdata_is_private                                            :  1;
        unsigned int csdata_is_tg                                                 :  1;
        unsigned int csc_ca_last_tg                                               :  1;
        unsigned int csc_ca_first_tg                                              :  1;
        unsigned int csc_ca_first_req_dispatch                                    :  1;
        unsigned int Reserved0                                                    : 12;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_cs_wave_gfx_ctl {
    struct {
        unsigned int tr_slots_used0                                               :  9;
        unsigned int loader_state_q                                               :  2;
        unsigned int Reserved2                                                    :  3;
        unsigned int csdata_is_private                                            :  1;
        unsigned int csdata_is_tg                                                 :  1;
        unsigned int Reserved1                                                    :  2;
        unsigned int csc_ra0_first_req_dispatch                                   :  1;
        unsigned int Reserved0                                                    : 13;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_DEBUG_CNTL_2 {
    struct {
        unsigned int SPI_SPARE                                                    : 32;
    } gfx09_1x;
    struct {
        unsigned int ECO_SPARE_0                                                  :  1;
        unsigned int ECO_SPARE_1                                                  :  1;
        unsigned int ECO_SPARE_2                                                  :  1;
        unsigned int ECO_SPARE_3                                                  :  1;
        unsigned int ECO_SPARE_4                                                  :  1;
        unsigned int ECO_SPARE_5                                                  :  1;
        unsigned int ECO_SPARE_6                                                  :  1;
        unsigned int ECO_SPARE_7                                                  :  1;
        unsigned int                                                              : 24;
    } gfx10;
    struct {
        unsigned int                                                              :  8;
        unsigned int DISABLE_INTRA_PRIM_CONFLICT                                  :  1;
        unsigned int                                                              : 23;
    } gfx101Plus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_DEBUG_READ {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_DSM_CNTL {
    struct {
        unsigned int SPI_SR_MEM_DSM_IRRITATOR_DATA                                :  2;
        unsigned int SPI_SR_MEM_ENABLE_SINGLE_WRITE                               :  1;
        unsigned int                                                              : 29;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  3;
        unsigned int UNUSED                                                       : 29;
    } gfx09;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_DSM_CNTL2 {
    struct {
        unsigned int SPI_SR_MEM_ENABLE_ERROR_INJECT                               :  2;
        unsigned int SPI_SR_MEM_SELECT_INJECT_DELAY                               :  1;
        unsigned int                                                              : 29;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  4;
        unsigned int SPI_SR_MEM_INJECT_DELAY                                      :  6;
        unsigned int UNUSED                                                       : 22;
    } gfx09;
    struct {
        unsigned int                                                              :  3;
        unsigned int SPI_SR_MEM_INJECT_DELAY                                      :  6;
        unsigned int                                                              : 23;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_EDC_CNT {
    struct {
        unsigned int SPI_SR_MEM_SED_COUNT                                         :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_es_wave_ctl0 {
    struct {
        unsigned int f_double_data                                                :  1;
        unsigned int f_gpr_ld_buf_sel                                             :  2;
        unsigned int wave_buffer_fifo_empty                                       :  1;
        unsigned int esc_gsc_group_fifo_empty                                     :  1;
        unsigned int es_gdbg_en_q                                                 :  1;
        unsigned int tr_fits                                                      :  1;
        unsigned int es_wave_cnt_lt_lim                                           :  1;
        unsigned int pipe_id                                                      :  2;
        unsigned int outstanding_waves                                            :  4;
        unsigned int outstanding_events                                           :  3;
        unsigned int gs_onchip                                                    :  2;
        unsigned int es_wave_offchip                                              :  1;
        unsigned int es_wave_first_subgrp                                         :  1;
        unsigned int esc_ra0_is_ds                                                :  1;
        unsigned int Reserved0                                                    : 10;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_es_wave_ctl1 {
    struct {
        unsigned int crawler_is_event                                             :  1;
        unsigned int stall_events                                                 :  1;
        unsigned int wave_done_crawler_id                                         : 10;
        unsigned int wave_done                                                    :  1;
        unsigned int crawler_full                                                 :  1;
        unsigned int crawler_empty                                                :  1;
        unsigned int crawler_rd                                                   :  9;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_es_wave_ctl2 {
    struct {
        unsigned int res_alloc_state                                              :  1;
        unsigned int esc_ra_rts                                                   :  1;
        unsigned int clocks_on                                                    :  1;
        unsigned int esc_ra_alloc_req1                                            :  1;
        unsigned int esc_ra_alloc_req0                                            :  1;
        unsigned int esc_ra_cu_id                                                 :  4;
        unsigned int hsc_group_cu_id                                              :  4;
        unsigned int hsc_group_fifo_empty                                         :  1;
        unsigned int es_wave_first_wave                                           :  1;
        unsigned int es_wave_stateid                                              :  3;
        unsigned int es_wave_is_event                                             :  1;
        unsigned int vgt_eswave_fifo_empty                                        :  1;
        unsigned int es_group_fifo_full                                           :  1;
        unsigned int Reserved0                                                    : 11;
    } gfx09_0;
    struct {
        unsigned int res_alloc_state                                              :  1;
        unsigned int esc_ra_rts                                                   :  1;
        unsigned int clocks_on                                                    :  1;
        unsigned int esc_ra_alloc_req1                                            :  1;
        unsigned int esc_ra_alloc_req0                                            :  1;
        unsigned int esc_ra_cu_id                                                 :  4;
        unsigned int hsc_group_cu_id                                              :  4;
        unsigned int hsc_group_fifo_empty                                         :  1;
        unsigned int es_wave_first_wave                                           :  1;
        unsigned int es_wave_stateid                                              :  3;
        unsigned int es_wave_is_event                                             :  1;
        unsigned int vgt_eswave_fifo_empty                                        :  1;
        unsigned int es_group_fifo_full                                           :  1;
        unsigned int Reserved0                                                    : 11;
    } rv2x;
    struct {
        unsigned int res_alloc_state                                              :  1;
        unsigned int esc_ra_rts                                                   :  1;
        unsigned int clocks_on                                                    :  1;
        unsigned int esc_ra_alloc_req1                                            :  1;
        unsigned int esc_ra_alloc_req0                                            :  1;
        unsigned int esc_ra_cu_id                                                 :  4;
        unsigned int hsc_group_cu_id                                              :  4;
        unsigned int hsc_group_fifo_empty                                         :  1;
        unsigned int es_wave_first_wave                                           :  1;
        unsigned int es_wave_stateid                                              :  3;
        unsigned int es_wave_is_event                                             :  1;
        unsigned int vgt_eswave_fifo_empty                                        :  1;
        unsigned int es_group_fifo_full                                           :  1;
        unsigned int Reserved0                                                    : 11;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_FEATURE_CTRL {
    struct {
        unsigned int CU_LOCKING_FAIRNESS_DISABLE                                  :  1;
        unsigned int                                                              :  1;
        unsigned int ALLOCATION_RATE_THROTTLE_THRESHOLD                           :  5;
        unsigned int ACTIVE_HARD_LOCK_LIMIT                                       :  5;
        unsigned int LR_IMBALANCE_THRESHOLD                                       :  6;
        unsigned int RA_PIPE_DEPTH_THRESHOLD_ALLOC_STALL_EN                       :  1;
        unsigned int BUS_ACTIVITY_THRESHOLD_ALLOC_STALL_EN                        :  1;
        unsigned int BUS_ACTIVITY_THRESHOLD                                       :  8;
        unsigned int TUNNELING_WAVE_LIMIT                                         :  4;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_GFX_CNTL {
    struct {
        unsigned int RESET_COUNTS                                                 :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_gfx_tmp_ring_mgr {
    struct {
        unsigned int trm_tr_slots_used                                            :  9;
        unsigned int trm_hs_lock                                                  :  1;
        unsigned int Reserved0                                                    : 22;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_gs_wave_ctl0 {
    struct {
        unsigned int f_double_data                                                :  1;
        unsigned int f_gpr_ld_buf_sel                                             :  2;
        unsigned int wave_buffer_fifo_empty                                       :  1;
        unsigned int gsc_vsc_group_fifo_empty                                     :  1;
        unsigned int gs_gdbg_en_q                                                 :  1;
        unsigned int tr_fits                                                      :  1;
        unsigned int gs_wave_cnt_lt_lim                                           :  1;
        unsigned int pipe_id                                                      :  2;
        unsigned int outstanding_waves                                            :  4;
        unsigned int outstanding_events                                           :  3;
        unsigned int Reserved0                                                    : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_gs_wave_ctl1 {
    struct {
        unsigned int crawler_rd                                                   :  7;
        unsigned int crawler_is_event                                             :  1;
        unsigned int stall_events                                                 :  1;
        unsigned int wave_done_crawler_id                                         : 10;
        unsigned int wave_done                                                    :  1;
        unsigned int crawler_full                                                 :  1;
        unsigned int crawler_empty                                                :  1;
        unsigned int Reserved0                                                    : 10;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_gs_wave_ctl2 {
    struct {
        unsigned int res_alloc_state                                              :  1;
        unsigned int clocks_on                                                    :  1;
        unsigned int gsc_ra_rts                                                   :  1;
        unsigned int UNUSED_gsc_ra_alloc_req1                                     :  1;
        unsigned int gsc_ra_alloc_req0                                            :  1;
        unsigned int gs_wave_stateid                                              :  3;
        unsigned int gs_wave_is_event                                             :  1;
        unsigned int vgt_gswave_fifo_empty                                        :  1;
        unsigned int gs_group_fifo_full                                           :  1;
        unsigned int gs_wave_first_subgrp                                         :  1;
        unsigned int gsc_ra0_cu_id                                                :  4;
        unsigned int esc_gsc_group_cu_id                                          :  4;
        unsigned int gs_onchip                                                    :  2;
        unsigned int Reserved0                                                    : 10;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_hs_wave_ctl0 {
    struct {
        unsigned int f_gpr_ld_line_sel                                            :  2;
        unsigned int wave_buffer_fifo_empty                                       :  1;
        unsigned int hsc_group_fifo_empty                                         :  1;
        unsigned int hs_gdbg_en_q                                                 :  1;
        unsigned int tr_fits                                                      :  1;
        unsigned int hs_wave_cnt_lt_lim                                           :  1;
        unsigned int pipe_id                                                      :  2;
        unsigned int outstanding_waves                                            :  4;
        unsigned int outstanding_events                                           :  3;
        unsigned int es_is_ds                                                     :  1;
        unsigned int Reserved0                                                    : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_hs_wave_ctl1 {
    struct {
        unsigned int crawler_rd                                                   :  7;
        unsigned int crawler_is_event                                             :  1;
        unsigned int stall_events                                                 :  1;
        unsigned int wave_done_crawler_id                                         : 10;
        unsigned int wave_done                                                    :  1;
        unsigned int crawler_full                                                 :  1;
        unsigned int crawler_empty                                                :  1;
        unsigned int Reserved0                                                    : 10;
    } gfx09_0;
    struct {
        unsigned int crawler_rd                                                   :  7;
        unsigned int crawler_is_event                                             :  1;
        unsigned int stall_events                                                 :  1;
        unsigned int wave_done_crawler_id                                         : 10;
        unsigned int wave_done                                                    :  1;
        unsigned int crawler_full                                                 :  1;
        unsigned int crawler_empty                                                :  1;
        unsigned int Reserved0                                                    : 10;
    } rv2x;
    struct {
        unsigned int crawler_rd                                                   :  7;
        unsigned int crawler_is_event                                             :  1;
        unsigned int stall_events                                                 :  1;
        unsigned int wave_done_crawler_id                                         : 10;
        unsigned int wave_done                                                    :  1;
        unsigned int crawler_full                                                 :  1;
        unsigned int crawler_empty                                                :  1;
        unsigned int Reserved0                                                    : 10;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_hs_wave_ctl2 {
    struct {
        unsigned int res_alloc_state                                              :  1;
        unsigned int hsc_ra_rts                                                   :  1;
        unsigned int clocks_on                                                    :  1;
        unsigned int UNUSED_hsc_ra_alloc_req1                                     :  1;
        unsigned int hsc_ra_alloc_req0                                            :  1;
        unsigned int hsc_ra_cu_id                                                 :  4;
        unsigned int lsc_hsc_group_cu_id                                          :  4;
        unsigned int lsc_hsc_group_fifo_empty                                     :  1;
        unsigned int hs_group_fifo_full                                           :  1;
        unsigned int hs_wave_first_wave                                           :  1;
        unsigned int hs_wave_stateid                                              :  3;
        unsigned int hs_wave_is_event                                             :  1;
        unsigned int vgt_hswave_fifo_empty                                        :  1;
        unsigned int Reserved0                                                    : 11;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_INTERP_CONTROL_0 {
    struct {
        unsigned int FLAT_SHADE_ENA                                               :  1;
        unsigned int PNT_SPRITE_ENA                                               :  1;
        unsigned int PNT_SPRITE_OVRD_X                                            :  3;
        unsigned int PNT_SPRITE_OVRD_Y                                            :  3;
        unsigned int PNT_SPRITE_OVRD_Z                                            :  3;
        unsigned int PNT_SPRITE_OVRD_W                                            :  3;
        unsigned int PNT_SPRITE_TOP_1                                             :  1;
        unsigned int                                                              : 17;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_LB_CTR_CTRL {
    struct {
        unsigned int LOAD                                                         :  1;
        unsigned int WAVES_SELECT                                                 :  2;
        unsigned int CLEAR_ON_READ                                                :  1;
        unsigned int RESET_COUNTS                                                 :  1;
        unsigned int                                                              : 27;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_LB_CU_MASK {
    struct {
        unsigned int CU_MASK                                                      : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_LB_DATA_PERCU_WAVE_CS {
    struct {
        unsigned int ACTIVE                                                       : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_LB_DATA_PERCU_WAVE_HSGS {
    struct {
        unsigned int CU_USED_HS                                                   : 16;
        unsigned int CU_USED_GS                                                   : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_LB_DATA_PERCU_WAVE_VSPS {
    struct {
        unsigned int CU_USED_VS                                                   : 16;
        unsigned int CU_USED_PS                                                   : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_LB_DATA_PERWGP_WAVE_CS {
    struct {
        unsigned int ACTIVE                                                       : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_LB_DATA_PERWGP_WAVE_HSGS {
    struct {
        unsigned int WGP_USED_HS                                                  : 16;
        unsigned int WGP_USED_GS                                                  : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_LB_DATA_PERWGP_WAVE_VSPS {
    struct {
        unsigned int WGP_USED_VS                                                  : 16;
        unsigned int WGP_USED_PS                                                  : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_LB_DATA_REG {
    struct {
        unsigned int CNT_DATA                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_LB_DATA_WAVES {
    struct {
        unsigned int COUNT0                                                       : 16;
        unsigned int COUNT1                                                       : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_LB_WGP_MASK {
    struct {
        unsigned int WGP_MASK                                                     : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_lds_wr_ctl0 {
    struct {
        unsigned int state_id                                                     :  3;
        unsigned int lds_in_fifo_empty                                            :  1;
        unsigned int lds_in_fifo_full                                             :  1;
        unsigned int lds_write_state                                              :  2;
        unsigned int even_debug_lds_valid_d                                       :  1;
        unsigned int even_debug_lds_valid_q1                                      :  1;
        unsigned int even_debug_lwc_pc_valid                                      :  1;
        unsigned int even_debug_lds_param_sent_cnt_q                              :  6;
        unsigned int even_debug_lds_pass_cnt_q                                    :  4;
        unsigned int even_debug_stall_odd                                         :  1;
        unsigned int Reserved0                                                    : 11;
    } gfx09_0;
    struct {
        unsigned int state_id                                                     :  3;
        unsigned int lds_in_fifo_empty                                            :  1;
        unsigned int lds_in_fifo_full                                             :  1;
        unsigned int lds_write_state                                              :  2;
        unsigned int even_debug_lds_valid_d                                       :  1;
        unsigned int even_debug_lds_valid_q1                                      :  1;
        unsigned int even_debug_lwc_pc_valid                                      :  1;
        unsigned int even_debug_lds_param_sent_cnt_q                              :  6;
        unsigned int even_debug_lds_pass_cnt_q                                    :  4;
        unsigned int even_debug_stall_odd                                         :  1;
        unsigned int Reserved0                                                    : 11;
    } rv2x;
    struct {
        unsigned int state_id                                                     :  3;
        unsigned int lds_in_fifo_empty                                            :  1;
        unsigned int lds_in_fifo_full                                             :  1;
        unsigned int lds_write_state                                              :  2;
        unsigned int even_debug                                                   : 17;
        unsigned int Reserved0                                                    :  8;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_lds_wr_ctl1 {
    struct {
        unsigned int odd_debug_lds_valid_d                                        :  1;
        unsigned int odd_debug_lds_valid_q1                                       :  1;
        unsigned int odd_debug_lwc_pc_valid                                       :  1;
        unsigned int odd_debug_lds_param_sent_cnt_q                               :  6;
        unsigned int odd_debug_lds_pass_cnt_q                                     :  4;
        unsigned int Reserved0                                                    : 19;
    } most;
    struct {
        unsigned int odd_debug                                                    : 13;
        unsigned int interp_cnt_debug                                             :  6;
        unsigned int ofc_mode_q1                                                  :  1;
        unsigned int lwc_pc_odd_valid                                             :  1;
        unsigned int lwc_pc_even_valid                                            :  1;
        unsigned int lwc_even_wave_done                                           :  1;
        unsigned int lwc_odd_wave_done                                            :  1;
        unsigned int Reserved0                                                    :  8;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_ls_wave_ctl0 {
    struct {
        unsigned int f_vsr_ld_buf_sel                                             :  2;
        unsigned int wave_buffer_fifo_empty                                       :  1;
        unsigned int lsc_hsc_group_fifo_empty                                     :  1;
        unsigned int ls_gdbg_en_q                                                 :  1;
        unsigned int tr_fits                                                      :  1;
        unsigned int ls_wave_cnt_lt_lim                                           :  1;
        unsigned int pipe_id                                                      :  2;
        unsigned int outstanding_waves                                            :  4;
        unsigned int outstanding_events                                           :  3;
        unsigned int Reserved0                                                    : 16;
    } gfx09_0;
    struct {
        unsigned int f_vsr_ld_buf_sel                                             :  2;
        unsigned int wave_buffer_fifo_empty                                       :  1;
        unsigned int lsc_hsc_group_fifo_empty                                     :  1;
        unsigned int ls_gdbg_en_q                                                 :  1;
        unsigned int tr_fits                                                      :  1;
        unsigned int ls_wave_cnt_lt_lim                                           :  1;
        unsigned int pipe_id                                                      :  2;
        unsigned int outstanding_waves                                            :  4;
        unsigned int outstanding_events                                           :  3;
        unsigned int Reserved0                                                    : 16;
    } rv2x;
    struct {
        unsigned int f_vsr_ld_buf_sel                                             :  2;
        unsigned int wave_buffer_fifo_empty                                       :  1;
        unsigned int lsc_hsc_group_fifo_empty                                     :  1;
        unsigned int ls_gdbg_en_q                                                 :  1;
        unsigned int tr_fits                                                      :  1;
        unsigned int ls_wave_cnt_lt_lim                                           :  1;
        unsigned int pipe_id                                                      :  2;
        unsigned int outstanding_waves                                            :  4;
        unsigned int outstanding_events                                           :  3;
        unsigned int Reserved0                                                    : 16;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_ls_wave_ctl1 {
    struct {
        unsigned int crawler_rd                                                   :  7;
        unsigned int crawler_is_event                                             :  1;
        unsigned int stall_events                                                 :  1;
        unsigned int wave_done_crawler_id                                         : 10;
        unsigned int wave_done                                                    :  1;
        unsigned int crawler_full                                                 :  1;
        unsigned int crawler_empty                                                :  1;
        unsigned int Reserved0                                                    : 10;
    } gfx09_0;
    struct {
        unsigned int crawler_rd                                                   :  7;
        unsigned int crawler_is_event                                             :  1;
        unsigned int stall_events                                                 :  1;
        unsigned int wave_done_crawler_id                                         : 10;
        unsigned int wave_done                                                    :  1;
        unsigned int crawler_full                                                 :  1;
        unsigned int crawler_empty                                                :  1;
        unsigned int Reserved0                                                    : 10;
    } rv2x;
    struct {
        unsigned int crawler_rd                                                   :  7;
        unsigned int crawler_is_event                                             :  1;
        unsigned int stall_events                                                 :  1;
        unsigned int wave_done_crawler_id                                         : 10;
        unsigned int wave_done                                                    :  1;
        unsigned int crawler_full                                                 :  1;
        unsigned int crawler_empty                                                :  1;
        unsigned int Reserved0                                                    : 10;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_ls_wave_ctl2 {
    struct {
        unsigned int res_alloc_state                                              :  2;
        unsigned int lsc_ra_rts                                                   :  1;
        unsigned int clocks_on                                                    :  1;
        unsigned int UNUSED_lsc_ra_alloc_req1                                     :  1;
        unsigned int lsc_ra_alloc_req0                                            :  1;
        unsigned int lsc_olm_alloc_req                                            :  1;
        unsigned int Reserved1                                                    :  4;
        unsigned int ls_group_fifo_full                                           :  1;
        unsigned int ls_wave_first_wave                                           :  1;
        unsigned int ls_wave_stateid                                              :  3;
        unsigned int ls_wave_is_event                                             :  1;
        unsigned int vgt_lswave_fifo_empty                                        :  1;
        unsigned int Reserved0                                                    : 14;
    } gfx09_0;
    struct {
        unsigned int res_alloc_state                                              :  2;
        unsigned int lsc_ra_rts                                                   :  1;
        unsigned int clocks_on                                                    :  1;
        unsigned int UNUSED_lsc_ra_alloc_req1                                     :  1;
        unsigned int lsc_ra_alloc_req0                                            :  1;
        unsigned int lsc_olm_alloc_req                                            :  1;
        unsigned int Reserved1                                                    :  4;
        unsigned int ls_group_fifo_full                                           :  1;
        unsigned int ls_wave_first_wave                                           :  1;
        unsigned int ls_wave_stateid                                              :  3;
        unsigned int ls_wave_is_event                                             :  1;
        unsigned int vgt_lswave_fifo_empty                                        :  1;
        unsigned int Reserved0                                                    : 14;
    } rv2x;
    struct {
        unsigned int res_alloc_state                                              :  2;
        unsigned int lsc_ra_rts                                                   :  1;
        unsigned int clocks_on                                                    :  1;
        unsigned int UNUSED_lsc_ra_alloc_req1                                     :  1;
        unsigned int lsc_ra_alloc_req0                                            :  1;
        unsigned int lsc_olm_alloc_req                                            :  1;
        unsigned int Reserved1                                                    :  4;
        unsigned int ls_group_fifo_full                                           :  1;
        unsigned int ls_wave_first_wave                                           :  1;
        unsigned int ls_wave_stateid                                              :  3;
        unsigned int ls_wave_is_event                                             :  1;
        unsigned int vgt_lswave_fifo_empty                                        :  1;
        unsigned int Reserved0                                                    : 14;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_offchip_lds_mgr0 {
    struct {
        unsigned int es_threadgroup_done_count                                    :  7;
        unsigned int Reserved2                                                    :  1;
        unsigned int lds_offchip_tail                                             :  7;
        unsigned int Reserved1                                                    :  1;
        unsigned int lds_offchip_head                                             :  7;
        unsigned int Reserved0                                                    :  9;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_offchip_lds_mgr1 {
    struct {
        unsigned int lds_offchip_full                                             :  1;
        unsigned int order_fifo_rd                                                :  1;
        unsigned int order_fifo_full                                              :  1;
        unsigned int order_fifo_empty                                             :  1;
        unsigned int esc_olm_ds_offchip_done                                      :  1;
        unsigned int vsc_olm_ds_offchip_done                                      :  1;
        unsigned int Reserved1                                                    :  2;
        unsigned int vs_threadgroup_done_count                                    :  7;
        unsigned int Reserved0                                                    : 17;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_P0_TRAP_SCREEN_GPR_MIN {
    struct {
        unsigned int VGPR_MIN                                                     :  6;
        unsigned int SGPR_MIN                                                     :  4;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_P0_TRAP_SCREEN_PSBA_HI {
    struct {
        unsigned int MEM_BASE                                                     :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_P0_TRAP_SCREEN_PSBA_LO {
    struct {
        unsigned int MEM_BASE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_P0_TRAP_SCREEN_PSMA_HI {
    struct {
        unsigned int MEM_BASE                                                     :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_P0_TRAP_SCREEN_PSMA_LO {
    struct {
        unsigned int MEM_BASE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_P1_TRAP_SCREEN_GPR_MIN {
    struct {
        unsigned int VGPR_MIN                                                     :  6;
        unsigned int SGPR_MIN                                                     :  4;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_P1_TRAP_SCREEN_PSBA_HI {
    struct {
        unsigned int MEM_BASE                                                     :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_P1_TRAP_SCREEN_PSBA_LO {
    struct {
        unsigned int MEM_BASE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_P1_TRAP_SCREEN_PSMA_HI {
    struct {
        unsigned int MEM_BASE                                                     :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_P1_TRAP_SCREEN_PSMA_LO {
    struct {
        unsigned int MEM_BASE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_pc_dealloc_ctl0 {
    struct {
        unsigned int decr_pc_dealloc_cnt                                          :  1;
        unsigned int pc_dealloc_min_cnt                                           : 10;
        unsigned int pc0_dealloc_cnt                                              : 10;
        unsigned int Reserved0                                                    : 11;
    } gfx09_0;
    struct {
        unsigned int decr_pc_dealloc_cnt                                          :  1;
        unsigned int pc_dealloc_min_cnt                                           : 10;
        unsigned int pc0_dealloc_cnt                                              : 10;
        unsigned int Reserved0                                                    : 11;
    } rv2x;
    struct {
        unsigned int decr_pc_dealloc_cnt                                          :  1;
        unsigned int pc_dealloc_min_cnt                                           : 10;
        unsigned int pc0_dealloc_cnt                                              : 10;
        unsigned int Reserved0                                                    : 11;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_pc_dealloc_ctl1 {
    struct {
        unsigned int pc1_dealloc_cnt                                              : 10;
        unsigned int pc_deallocIn0_cnt                                            : 10;
        unsigned int Reserved0                                                    : 12;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_pc_dealloc_ctl2 {
    struct {
        unsigned int pc_deallocIn1_cnt                                            : 10;
        unsigned int pc_deallocIn2_cnt                                            : 10;
        unsigned int Reserved0                                                    : 12;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_pc_dealloc_ctl3 {
    struct {
        unsigned int pc0_deallocOut0_cnt                                          : 10;
        unsigned int pc0_deallocOut1_cnt                                          : 10;
        unsigned int Reserved0                                                    : 12;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_pc_dealloc_ctl4 {
    struct {
        unsigned int pc0_deallocOut2_cnt                                          : 10;
        unsigned int pc1_deallocOut0_cnt                                          : 10;
        unsigned int Reserved0                                                    : 12;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_pc_dealloc_ctl5 {
    struct {
        unsigned int pc1_deallocOut1_cnt                                          : 10;
        unsigned int pc1_deallocOut2_cnt                                          : 10;
        unsigned int Reserved0                                                    : 12;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER1_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER2_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER3_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER4_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER4_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER4_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int                                                              : 24;
    } gfx09;
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 22;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER5_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER5_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER5_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int                                                              : 24;
    } gfx09;
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 22;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER_BINS {
    struct {
        unsigned int BIN0_MIN                                                     :  4;
        unsigned int BIN0_MAX                                                     :  4;
        unsigned int BIN1_MIN                                                     :  4;
        unsigned int BIN1_MAX                                                     :  4;
        unsigned int BIN2_MIN                                                     :  4;
        unsigned int BIN2_MAX                                                     :  4;
        unsigned int BIN3_MIN                                                     :  4;
        unsigned int BIN3_MAX                                                     :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PG_ENABLE_STATIC_CU_MASK {
    struct {
        unsigned int CU_MASK                                                      : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PG_ENABLE_STATIC_WGP_MASK {
    struct {
        unsigned int WGP_MASK                                                     : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PQEV_CTRL {
    struct {
        unsigned int SCAN_PERIOD                                                  : 10;
        unsigned int QUEUE_DURATION                                               :  6;
        unsigned int COMPUTE_PIPE_EN                                              :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_ps_ctl0_0 {
    struct {
        unsigned int new_vector1_save_q                                           :  1;
        unsigned int new_vector0_save_q                                           :  1;
        unsigned int vtx_sync_cnt_q                                               :  8;
        unsigned int vtx_sync_wrapped                                             :  1;
        unsigned int ose_vtx_sync_cnt_q                                           :  8;
        unsigned int psc_wr_line_q                                                :  3;
        unsigned int psc_gdbg_en_q                                                :  1;
        unsigned int Reserved0                                                    :  9;
    } most;
    struct {
        unsigned int new_vector0_save_q                                           :  6;
        unsigned int new_vector1_save_q                                           :  6;
        unsigned int new_vector2_save_q                                           :  6;
        unsigned int new_vector3_save_q                                           :  6;
        unsigned int sc_xfer_cycle                                                :  3;
        unsigned int psc_raw_perf_prim_cnt                                        :  5;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_ps_ctl0_1 {
    struct {
        unsigned int wd_end_of_wave                                               :  1;
        unsigned int wd_fifo_empty                                                :  1;
        unsigned int wd_fifo_full                                                 :  1;
        unsigned int ef_end_of_wave                                               :  1;
        unsigned int ef_state_id                                                  :  3;
        unsigned int ef_event_id                                                  :  6;
        unsigned int ef_event                                                     :  1;
        unsigned int ef_new1                                                      :  1;
        unsigned int ef_new0                                                      :  1;
        unsigned int ef_empty                                                     :  1;
        unsigned int ef_full                                                      :  1;
        unsigned int ef_new3                                                      :  1;
        unsigned int ef_new2                                                      :  1;
        unsigned int pipe_id                                                      :  2;
        unsigned int Reserved0                                                    : 10;
    } most;
    struct {
        unsigned int sc_dealloc0_save_q                                           :  8;
        unsigned int sc_dealloc1_save_q                                           :  8;
        unsigned int sc_dealloc2_save_q                                           :  8;
        unsigned int sc_dealloc3_save_q                                           :  8;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_ps_ctl0_2 {
    struct {
        unsigned int res_alloc_state                                              :  1;
        unsigned int psr_wave_cnt                                                 :  6;
        unsigned int psc_read_data                                                :  5;
        unsigned int pc_free_cnt                                                  :  4;
        unsigned int dealloc1_save_q                                              :  3;
        unsigned int dealloc0_save_q                                              :  3;
        unsigned int tr_fits                                                      :  1;
        unsigned int ps_wave_cnt_lt_lim                                           :  1;
        unsigned int Reserved0                                                    :  8;
    } most;
    struct {
        unsigned int ef_empty                                                     :  1;
        unsigned int ef_end_of_wave                                               :  1;
        unsigned int ef_state_id                                                  :  3;
        unsigned int ef_event                                                     :  1;
        unsigned int ef_event_id                                                  :  6;
        unsigned int ef_full                                                      :  1;
        unsigned int lds_done_cnt                                                 :  9;
        unsigned int psr_wave_cnt                                                 :  6;
        unsigned int RA_FSM_idle                                                  :  1;
        unsigned int ps_wave_cnt_lt_lim                                           :  1;
        unsigned int tr_fits                                                      :  1;
        unsigned int stall_events                                                 :  1;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_ps_ctl0_3 {
    struct {
        unsigned int new_vector2_save_q                                           :  1;
        unsigned int new_vector3_save_q                                           :  1;
        unsigned int ose1_vtx_sync_cnt_q                                          :  8;
        unsigned int ose2_vtx_sync_cnt_q                                          :  8;
        unsigned int dealloc2_save_q                                              :  3;
        unsigned int dealloc3_save_q                                              :  3;
        unsigned int Reserved0                                                    :  8;
    } gfx09_0;
    struct {
        unsigned int new_vector2_save_q                                           :  1;
        unsigned int new_vector3_save_q                                           :  1;
        unsigned int ose1_vtx_sync_cnt_q                                          :  8;
        unsigned int ose2_vtx_sync_cnt_q                                          :  8;
        unsigned int dealloc2_save_q                                              :  3;
        unsigned int dealloc3_save_q                                              :  3;
        unsigned int Reserved0                                                    :  8;
    } rv2x;
    struct {
        unsigned int ef_new0                                                      :  6;
        unsigned int ef_new1                                                      :  6;
        unsigned int ef_new3                                                      :  6;
        unsigned int ef_new2                                                      :  6;
        unsigned int pipe_id                                                      :  2;
        unsigned int psc_gdbg_en_q                                                :  1;
        unsigned int Reserved0                                                    :  5;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_ps_ctl0_4 {
    struct {
        unsigned int crawler_rd                                                   :  7;
        unsigned int crawler_is_event                                             :  1;
        unsigned int stall_events                                                 :  1;
        unsigned int SPIS_SPIM_wbcrw_crawler_id_q                                 :  8;
        unsigned int SPIS_SPIM_wbcrw_ps_pkr_id_q                                  :  1;
        unsigned int SPIS_SPIM_wbcrw_wave_type_q                                  :  3;
        unsigned int SPIS_SPIM_wbcrw_wave_done_q                                  :  1;
        unsigned int crawler_full                                                 :  1;
        unsigned int crawler_empty                                                :  1;
        unsigned int Reserved0                                                    :  8;
    } gfx09_0;
    struct {
        unsigned int crawler_rd                                                   :  7;
        unsigned int crawler_is_event                                             :  1;
        unsigned int stall_events                                                 :  1;
        unsigned int SPIS_SPIM_wbcrw_crawler_id_q                                 :  8;
        unsigned int SPIS_SPIM_wbcrw_ps_pkr_id_q                                  :  1;
        unsigned int SPIS_SPIM_wbcrw_wave_type_q                                  :  3;
        unsigned int SPIS_SPIM_wbcrw_wave_done_q                                  :  1;
        unsigned int crawler_full                                                 :  1;
        unsigned int crawler_empty                                                :  1;
        unsigned int Reserved0                                                    :  8;
    } rv2x;
    struct {
        unsigned int ose0_vtx_sync_cnt_q                                          : 10;
        unsigned int ose1_vtx_sync_cnt_q                                          : 10;
        unsigned int ose2_vtx_sync_cnt_q                                          : 10;
        unsigned int Reserved0                                                    :  2;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_ps_ctl1_0 {
    struct {
        unsigned int new_vector1_save_q                                           :  1;
        unsigned int new_vector0_save_q                                           :  1;
        unsigned int vtx_sync_cnt_q                                               :  8;
        unsigned int vtx_sync_wrapped                                             :  1;
        unsigned int ose_vtx_sync_cnt_q                                           :  8;
        unsigned int psc_wr_line_q                                                :  3;
        unsigned int psc_gdbg_en_q                                                :  1;
        unsigned int Reserved0                                                    :  9;
    } most;
    struct {
        unsigned int new_vector0_save_q                                           :  6;
        unsigned int new_vector1_save_q                                           :  6;
        unsigned int new_vector2_save_q                                           :  6;
        unsigned int new_vector3_save_q                                           :  6;
        unsigned int sc_xfer_cycle                                                :  3;
        unsigned int psc_raw_perf_prim_cnt                                        :  5;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_ps_ctl1_1 {
    struct {
        unsigned int wd_end_of_wave                                               :  1;
        unsigned int wd_fifo_empty                                                :  1;
        unsigned int wd_fifo_full                                                 :  1;
        unsigned int ef_end_of_wave                                               :  1;
        unsigned int ef_state_id                                                  :  3;
        unsigned int ef_event_id                                                  :  6;
        unsigned int ef_event                                                     :  1;
        unsigned int ef_new1                                                      :  1;
        unsigned int ef_new0                                                      :  1;
        unsigned int ef_empty                                                     :  1;
        unsigned int ef_full                                                      :  1;
        unsigned int ef_new3                                                      :  1;
        unsigned int ef_new2                                                      :  1;
        unsigned int pipe_id                                                      :  2;
        unsigned int Reserved0                                                    : 10;
    } most;
    struct {
        unsigned int sc_dealloc0_save_q                                           :  8;
        unsigned int sc_dealloc1_save_q                                           :  8;
        unsigned int sc_dealloc2_save_q                                           :  8;
        unsigned int sc_dealloc3_save_q                                           :  8;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_ps_ctl1_2 {
    struct {
        unsigned int res_alloc_state                                              :  1;
        unsigned int psr_wave_cnt                                                 :  6;
        unsigned int psc_read_data                                                :  5;
        unsigned int pc_free_cnt                                                  :  4;
        unsigned int dealloc1_save_q                                              :  3;
        unsigned int dealloc0_save_q                                              :  3;
        unsigned int tr_fits                                                      :  1;
        unsigned int ps_wave_cnt_lt_lim                                           :  1;
        unsigned int Reserved0                                                    :  8;
    } most;
    struct {
        unsigned int ef_empty                                                     :  1;
        unsigned int ef_end_of_wave                                               :  1;
        unsigned int ef_state_id                                                  :  3;
        unsigned int ef_event                                                     :  1;
        unsigned int ef_event_id                                                  :  6;
        unsigned int ef_full                                                      :  1;
        unsigned int lds_done_cnt                                                 :  9;
        unsigned int psr_wave_cnt                                                 :  6;
        unsigned int RA_FSM_idle                                                  :  1;
        unsigned int ps_wave_cnt_lt_lim                                           :  1;
        unsigned int tr_fits                                                      :  1;
        unsigned int stall_events                                                 :  1;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_ps_ctl1_3 {
    struct {
        unsigned int new_vector2_save_q                                           :  1;
        unsigned int new_vector3_save_q                                           :  1;
        unsigned int ose1_vtx_sync_cnt_q                                          :  8;
        unsigned int ose2_vtx_sync_cnt_q                                          :  8;
        unsigned int dealloc2_save_q                                              :  3;
        unsigned int dealloc3_save_q                                              :  3;
        unsigned int Reserved0                                                    :  8;
    } gfx09_0;
    struct {
        unsigned int new_vector2_save_q                                           :  1;
        unsigned int new_vector3_save_q                                           :  1;
        unsigned int ose1_vtx_sync_cnt_q                                          :  8;
        unsigned int ose2_vtx_sync_cnt_q                                          :  8;
        unsigned int dealloc2_save_q                                              :  3;
        unsigned int dealloc3_save_q                                              :  3;
        unsigned int Reserved0                                                    :  8;
    } rv2x;
    struct {
        unsigned int ef_new0                                                      :  6;
        unsigned int ef_new1                                                      :  6;
        unsigned int ef_new3                                                      :  6;
        unsigned int ef_new2                                                      :  6;
        unsigned int pipe_id                                                      :  2;
        unsigned int psc_gdbg_en_q                                                :  1;
        unsigned int Reserved0                                                    :  5;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_ps_ctl1_4 {
    struct {
        unsigned int crawler_rd                                                   :  7;
        unsigned int crawler_is_event                                             :  1;
        unsigned int stall_events                                                 :  1;
        unsigned int SPIS_SPIM_wbcrw_crawler_id_q_7_0                             :  8;
        unsigned int SPIS_SPIM_wbcrw_ps_pkr_id_q                                  :  1;
        unsigned int SPIS_SPIM_wbcrw_wave_type_q                                  :  3;
        unsigned int SPIS_SPIM_wbcrw_wave_done_q                                  :  1;
        unsigned int crawler_full                                                 :  1;
        unsigned int crawler_empty                                                :  1;
        unsigned int Reserved0                                                    :  8;
    } gfx09_0;
    struct {
        unsigned int crawler_rd                                                   :  7;
        unsigned int crawler_is_event                                             :  1;
        unsigned int stall_events                                                 :  1;
        unsigned int SPIS_SPIM_wbcrw_crawler_id_q_7_0                             :  8;
        unsigned int SPIS_SPIM_wbcrw_ps_pkr_id_q                                  :  1;
        unsigned int SPIS_SPIM_wbcrw_wave_type_q                                  :  3;
        unsigned int SPIS_SPIM_wbcrw_wave_done_q                                  :  1;
        unsigned int crawler_full                                                 :  1;
        unsigned int crawler_empty                                                :  1;
        unsigned int Reserved0                                                    :  8;
    } rv2x;
    struct {
        unsigned int ose0_vtx_sync_cnt_q                                          : 10;
        unsigned int ose1_vtx_sync_cnt_q                                          : 10;
        unsigned int ose2_vtx_sync_cnt_q                                          : 10;
        unsigned int Reserved0                                                    :  2;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_ps_ctl1_5 {
    struct {
        unsigned int vtx_sync_cnt_q                                               : 10;
        unsigned int vtx_sync_wrapped                                             :  1;
        unsigned int crawler_is_event                                             :  1;
        unsigned int crawler_full                                                 :  1;
        unsigned int crawler_empty                                                :  1;
        unsigned int crawler_rd                                                   :  7;
        unsigned int Reserved0                                                    :  8;
        unsigned int wd_end_of_wave                                               :  1;
        unsigned int wd_fifo_empty                                                :  1;
        unsigned int wd_fifo_full                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_ps_ctl1_6 {
    struct {
        unsigned int SPIS_SPIM_wbcrw_ps_pkr_id_q                                  :  1;
        unsigned int SPIS_SPIM_wbcrw_wave_type_q                                  :  3;
        unsigned int SPIS_SPIM_wbcrw_wave_done_q                                  :  1;
        unsigned int SPIS_SPIM_wbcrw_crawler_id_q                                 :  9;
        unsigned int Reserved0                                                    : 18;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_ADDR {
    struct {
        unsigned int PERSP_SAMPLE_ENA                                             :  1;
        unsigned int PERSP_CENTER_ENA                                             :  1;
        unsigned int PERSP_CENTROID_ENA                                           :  1;
        unsigned int PERSP_PULL_MODEL_ENA                                         :  1;
        unsigned int LINEAR_SAMPLE_ENA                                            :  1;
        unsigned int LINEAR_CENTER_ENA                                            :  1;
        unsigned int LINEAR_CENTROID_ENA                                          :  1;
        unsigned int LINE_STIPPLE_TEX_ENA                                         :  1;
        unsigned int POS_X_FLOAT_ENA                                              :  1;
        unsigned int POS_Y_FLOAT_ENA                                              :  1;
        unsigned int POS_Z_FLOAT_ENA                                              :  1;
        unsigned int POS_W_FLOAT_ENA                                              :  1;
        unsigned int FRONT_FACE_ENA                                               :  1;
        unsigned int ANCILLARY_ENA                                                :  1;
        unsigned int SAMPLE_COVERAGE_ENA                                          :  1;
        unsigned int POS_FIXED_PT_ENA                                             :  1;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_0 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int                                                              :  2;
        unsigned int CYL_WRAP                                                     :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_1 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int                                                              :  2;
        unsigned int CYL_WRAP                                                     :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_2 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int                                                              :  2;
        unsigned int CYL_WRAP                                                     :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_3 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int                                                              :  2;
        unsigned int CYL_WRAP                                                     :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_4 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int                                                              :  2;
        unsigned int CYL_WRAP                                                     :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_5 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int                                                              :  2;
        unsigned int CYL_WRAP                                                     :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_6 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int                                                              :  2;
        unsigned int CYL_WRAP                                                     :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_7 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int                                                              :  2;
        unsigned int CYL_WRAP                                                     :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_8 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int                                                              :  2;
        unsigned int CYL_WRAP                                                     :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_9 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int                                                              :  2;
        unsigned int CYL_WRAP                                                     :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_10 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int                                                              :  2;
        unsigned int CYL_WRAP                                                     :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_11 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int                                                              :  2;
        unsigned int CYL_WRAP                                                     :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_12 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int                                                              :  2;
        unsigned int CYL_WRAP                                                     :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_13 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int                                                              :  2;
        unsigned int CYL_WRAP                                                     :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_14 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int                                                              :  2;
        unsigned int CYL_WRAP                                                     :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_15 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int                                                              :  2;
        unsigned int CYL_WRAP                                                     :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_16 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int                                                              :  2;
        unsigned int CYL_WRAP                                                     :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_17 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int                                                              :  2;
        unsigned int CYL_WRAP                                                     :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_18 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int                                                              :  2;
        unsigned int CYL_WRAP                                                     :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_19 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int                                                              :  2;
        unsigned int CYL_WRAP                                                     :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_20 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int                                                              :  7;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int                                                              :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_21 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int                                                              :  7;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int                                                              :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_22 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int                                                              :  7;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int                                                              :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_23 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int                                                              :  7;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int                                                              :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_24 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int                                                              :  7;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int                                                              :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_25 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int                                                              :  7;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int                                                              :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_26 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int                                                              :  7;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int                                                              :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_27 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int                                                              :  7;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int                                                              :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_28 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int                                                              :  7;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int                                                              :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_29 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int                                                              :  7;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int                                                              :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_30 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int                                                              :  7;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int                                                              :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_31 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int                                                              :  7;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int                                                              :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_ENA {
    struct {
        unsigned int PERSP_SAMPLE_ENA                                             :  1;
        unsigned int PERSP_CENTER_ENA                                             :  1;
        unsigned int PERSP_CENTROID_ENA                                           :  1;
        unsigned int PERSP_PULL_MODEL_ENA                                         :  1;
        unsigned int LINEAR_SAMPLE_ENA                                            :  1;
        unsigned int LINEAR_CENTER_ENA                                            :  1;
        unsigned int LINEAR_CENTROID_ENA                                          :  1;
        unsigned int LINE_STIPPLE_TEX_ENA                                         :  1;
        unsigned int POS_X_FLOAT_ENA                                              :  1;
        unsigned int POS_Y_FLOAT_ENA                                              :  1;
        unsigned int POS_Z_FLOAT_ENA                                              :  1;
        unsigned int POS_W_FLOAT_ENA                                              :  1;
        unsigned int FRONT_FACE_ENA                                               :  1;
        unsigned int ANCILLARY_ENA                                                :  1;
        unsigned int SAMPLE_COVERAGE_ENA                                          :  1;
        unsigned int POS_FIXED_PT_ENA                                             :  1;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_IN_CONTROL {
    struct {
        unsigned int NUM_INTERP                                                   :  6;
        unsigned int PARAM_GEN                                                    :  1;
        unsigned int OFFCHIP_PARAM_EN                                             :  1;
        unsigned int LATE_PC_DEALLOC                                              :  1;
        unsigned int                                                              :  5;
        unsigned int BC_OPTIMIZE_DISABLE                                          :  1;
        unsigned int                                                              : 17;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 15;
        unsigned int PS_W32_EN                                                    :  1;
        unsigned int                                                              : 16;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_MAX_WAVE_ID {
    struct {
        unsigned int MAX_WAVE_ID                                                  : 12;
        unsigned int                                                              :  4;
        unsigned int MAX_COLLISION_WAVE_ID                                        : 10;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESET_DEBUG {
    struct {
        unsigned int DISABLE_GFX_RESET                                            :  1;
        unsigned int DISABLE_GFX_RESET_PER_VMID                                   :  1;
        unsigned int DISABLE_GFX_RESET_ALL_VMID                                   :  1;
        unsigned int DISABLE_GFX_RESET_RESOURCE                                   :  1;
        unsigned int DISABLE_GFX_RESET_PRIORITY                                   :  1;
        unsigned int                                                              : 27;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_resource_alloc0 {
    struct {
        unsigned int Reserved1                                                    :  1;
        unsigned int lds_updating_cu_simd_id_q                                    :  4;
        unsigned int sgpr_updating_cu_simd_id_q                                   :  6;
        unsigned int vgpr_updating_cu_simd_id_q                                   :  6;
        unsigned int allocating_cu_simd_q                                         :  6;
        unsigned int Reserved0                                                    :  9;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_resource_alloc1 {
    struct {
        unsigned int vgpr_dealloc_pointer_23_0                                    : 24;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_resource_alloc2 {
    struct {
        unsigned int vgpr_dealloc_pointer_47_24                                   : 24;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_resource_alloc3 {
    struct {
        unsigned int sgpr_dealloc_pointer_23_0                                    : 24;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_resource_alloc4 {
    struct {
        unsigned int sgpr_dealloc_pointer_47_24                                   : 24;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_resource_alloc5 {
    struct {
        unsigned int Reserved0                                                    : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_resource_alloc6 {
    struct {
        unsigned int sgpr_max_fits_cnt                                            :  7;
        unsigned int vgpr_max_fits_cnt                                            :  7;
        unsigned int dbg_cu_simd_id                                               :  6;
        unsigned int Reserved0                                                    : 12;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_resource_alloc7 {
    struct {
        unsigned int alloc_state_q                                                :  1;
        unsigned int ts_priority                                                  :  3;
        unsigned int dbg_lds_dealloc_pointer                                      : 16;
        unsigned int Reserved0                                                    : 12;
    } gfx09_0;
    struct {
        unsigned int alloc_state_q                                                :  1;
        unsigned int ts_priority                                                  :  3;
        unsigned int dbg_lds_dealloc_pointer                                      : 16;
        unsigned int Reserved0                                                    : 12;
    } rv2x;
    struct {
        unsigned int alloc_state_q                                                :  1;
        unsigned int ts_priority                                                  :  3;
        unsigned int dbg_lds_dealloc_pointer                                      : 16;
        unsigned int Reserved0                                                    : 12;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_resource_alloc8 {
    struct {
        unsigned int dbg_lock_mask                                                : 16;
        unsigned int Reserved0                                                    : 16;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_resource_alloc9 {
    struct {
        unsigned int wave_cnt_cu_simd                                             :  4;
        unsigned int vgpr_rsv_max_fits_cnt                                        :  7;
        unsigned int sgpr_rsv_max_fits_cnt                                        :  7;
        unsigned int Reserved0                                                    : 14;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_resource_alloc10 {
    struct {
        unsigned int barrier_cnt_per_cu                                           :  5;
        unsigned int lds_rsv_max_fits_cnt                                         :  8;
        unsigned int Reserved1                                                    :  1;
        unsigned int lds_max_fits_cnt                                             :  8;
        unsigned int Reserved0                                                    : 10;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_CU_0 {
    struct {
        unsigned int VGPR                                                         :  4;
        unsigned int SGPR                                                         :  4;
        unsigned int LDS                                                          :  4;
        unsigned int WAVES                                                        :  3;
        unsigned int BARRIERS                                                     :  4;
        unsigned int                                                              : 13;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_CU_1 {
    struct {
        unsigned int VGPR                                                         :  4;
        unsigned int SGPR                                                         :  4;
        unsigned int LDS                                                          :  4;
        unsigned int WAVES                                                        :  3;
        unsigned int BARRIERS                                                     :  4;
        unsigned int                                                              : 13;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_CU_2 {
    struct {
        unsigned int VGPR                                                         :  4;
        unsigned int SGPR                                                         :  4;
        unsigned int LDS                                                          :  4;
        unsigned int WAVES                                                        :  3;
        unsigned int BARRIERS                                                     :  4;
        unsigned int                                                              : 13;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_CU_3 {
    struct {
        unsigned int VGPR                                                         :  4;
        unsigned int SGPR                                                         :  4;
        unsigned int LDS                                                          :  4;
        unsigned int WAVES                                                        :  3;
        unsigned int BARRIERS                                                     :  4;
        unsigned int                                                              : 13;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_CU_4 {
    struct {
        unsigned int VGPR                                                         :  4;
        unsigned int SGPR                                                         :  4;
        unsigned int LDS                                                          :  4;
        unsigned int WAVES                                                        :  3;
        unsigned int BARRIERS                                                     :  4;
        unsigned int                                                              : 13;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_CU_5 {
    struct {
        unsigned int VGPR                                                         :  4;
        unsigned int SGPR                                                         :  4;
        unsigned int LDS                                                          :  4;
        unsigned int WAVES                                                        :  3;
        unsigned int BARRIERS                                                     :  4;
        unsigned int                                                              : 13;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_CU_6 {
    struct {
        unsigned int VGPR                                                         :  4;
        unsigned int SGPR                                                         :  4;
        unsigned int LDS                                                          :  4;
        unsigned int WAVES                                                        :  3;
        unsigned int BARRIERS                                                     :  4;
        unsigned int                                                              : 13;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_CU_7 {
    struct {
        unsigned int VGPR                                                         :  4;
        unsigned int SGPR                                                         :  4;
        unsigned int LDS                                                          :  4;
        unsigned int WAVES                                                        :  3;
        unsigned int BARRIERS                                                     :  4;
        unsigned int                                                              : 13;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_CU_8 {
    struct {
        unsigned int VGPR                                                         :  4;
        unsigned int SGPR                                                         :  4;
        unsigned int LDS                                                          :  4;
        unsigned int WAVES                                                        :  3;
        unsigned int BARRIERS                                                     :  4;
        unsigned int                                                              : 13;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_CU_9 {
    struct {
        unsigned int VGPR                                                         :  4;
        unsigned int SGPR                                                         :  4;
        unsigned int LDS                                                          :  4;
        unsigned int WAVES                                                        :  3;
        unsigned int BARRIERS                                                     :  4;
        unsigned int                                                              : 13;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_CU_10 {
    struct {
        unsigned int VGPR                                                         :  4;
        unsigned int SGPR                                                         :  4;
        unsigned int LDS                                                          :  4;
        unsigned int WAVES                                                        :  3;
        unsigned int BARRIERS                                                     :  4;
        unsigned int                                                              : 13;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_CU_11 {
    struct {
        unsigned int VGPR                                                         :  4;
        unsigned int SGPR                                                         :  4;
        unsigned int LDS                                                          :  4;
        unsigned int WAVES                                                        :  3;
        unsigned int BARRIERS                                                     :  4;
        unsigned int                                                              : 13;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_CU_12 {
    struct {
        unsigned int VGPR                                                         :  4;
        unsigned int SGPR                                                         :  4;
        unsigned int LDS                                                          :  4;
        unsigned int WAVES                                                        :  3;
        unsigned int BARRIERS                                                     :  4;
        unsigned int                                                              : 13;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_CU_13 {
    struct {
        unsigned int VGPR                                                         :  4;
        unsigned int SGPR                                                         :  4;
        unsigned int LDS                                                          :  4;
        unsigned int WAVES                                                        :  3;
        unsigned int BARRIERS                                                     :  4;
        unsigned int                                                              : 13;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_CU_14 {
    struct {
        unsigned int VGPR                                                         :  4;
        unsigned int SGPR                                                         :  4;
        unsigned int LDS                                                          :  4;
        unsigned int WAVES                                                        :  3;
        unsigned int BARRIERS                                                     :  4;
        unsigned int                                                              : 13;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_CU_15 {
    struct {
        unsigned int VGPR                                                         :  4;
        unsigned int SGPR                                                         :  4;
        unsigned int LDS                                                          :  4;
        unsigned int WAVES                                                        :  3;
        unsigned int BARRIERS                                                     :  4;
        unsigned int                                                              : 13;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_EN_CU_0 {
    struct {
        unsigned int EN                                                           :  1;
        unsigned int TYPE_MASK                                                    : 15;
        unsigned int QUEUE_MASK                                                   :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 24;
        unsigned int RESERVE_SPACE_ONLY                                           :  1;
        unsigned int                                                              :  7;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_EN_CU_1 {
    struct {
        unsigned int EN                                                           :  1;
        unsigned int TYPE_MASK                                                    : 15;
        unsigned int QUEUE_MASK                                                   :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 24;
        unsigned int RESERVE_SPACE_ONLY                                           :  1;
        unsigned int                                                              :  7;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_EN_CU_2 {
    struct {
        unsigned int EN                                                           :  1;
        unsigned int TYPE_MASK                                                    : 15;
        unsigned int QUEUE_MASK                                                   :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 24;
        unsigned int RESERVE_SPACE_ONLY                                           :  1;
        unsigned int                                                              :  7;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_EN_CU_3 {
    struct {
        unsigned int EN                                                           :  1;
        unsigned int TYPE_MASK                                                    : 15;
        unsigned int QUEUE_MASK                                                   :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 24;
        unsigned int RESERVE_SPACE_ONLY                                           :  1;
        unsigned int                                                              :  7;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_EN_CU_4 {
    struct {
        unsigned int EN                                                           :  1;
        unsigned int TYPE_MASK                                                    : 15;
        unsigned int QUEUE_MASK                                                   :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 24;
        unsigned int RESERVE_SPACE_ONLY                                           :  1;
        unsigned int                                                              :  7;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_EN_CU_5 {
    struct {
        unsigned int EN                                                           :  1;
        unsigned int TYPE_MASK                                                    : 15;
        unsigned int QUEUE_MASK                                                   :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 24;
        unsigned int RESERVE_SPACE_ONLY                                           :  1;
        unsigned int                                                              :  7;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_EN_CU_6 {
    struct {
        unsigned int EN                                                           :  1;
        unsigned int TYPE_MASK                                                    : 15;
        unsigned int QUEUE_MASK                                                   :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 24;
        unsigned int RESERVE_SPACE_ONLY                                           :  1;
        unsigned int                                                              :  7;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_EN_CU_7 {
    struct {
        unsigned int EN                                                           :  1;
        unsigned int TYPE_MASK                                                    : 15;
        unsigned int QUEUE_MASK                                                   :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 24;
        unsigned int RESERVE_SPACE_ONLY                                           :  1;
        unsigned int                                                              :  7;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_EN_CU_8 {
    struct {
        unsigned int EN                                                           :  1;
        unsigned int TYPE_MASK                                                    : 15;
        unsigned int QUEUE_MASK                                                   :  8;
        unsigned int RESERVE_SPACE_ONLY                                           :  1;
        unsigned int                                                              :  7;
    } gfx09;
    struct {
        unsigned int EN                                                           :  1;
        unsigned int TYPE_MASK                                                    : 15;
        unsigned int QUEUE_MASK                                                   :  8;
        unsigned int RESERVE_SPACE_ONLY                                           :  1;
        unsigned int                                                              :  7;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_EN_CU_9 {
    struct {
        unsigned int EN                                                           :  1;
        unsigned int TYPE_MASK                                                    : 15;
        unsigned int QUEUE_MASK                                                   :  8;
        unsigned int RESERVE_SPACE_ONLY                                           :  1;
        unsigned int                                                              :  7;
    } gfx09;
    struct {
        unsigned int EN                                                           :  1;
        unsigned int TYPE_MASK                                                    : 15;
        unsigned int QUEUE_MASK                                                   :  8;
        unsigned int RESERVE_SPACE_ONLY                                           :  1;
        unsigned int                                                              :  7;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_EN_CU_10 {
    struct {
        unsigned int EN                                                           :  1;
        unsigned int TYPE_MASK                                                    : 15;
        unsigned int QUEUE_MASK                                                   :  8;
        unsigned int RESERVE_SPACE_ONLY                                           :  1;
        unsigned int                                                              :  7;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_EN_CU_11 {
    struct {
        unsigned int EN                                                           :  1;
        unsigned int TYPE_MASK                                                    : 15;
        unsigned int QUEUE_MASK                                                   :  8;
        unsigned int RESERVE_SPACE_ONLY                                           :  1;
        unsigned int                                                              :  7;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_EN_CU_12 {
    struct {
        unsigned int EN                                                           :  1;
        unsigned int TYPE_MASK                                                    : 15;
        unsigned int QUEUE_MASK                                                   :  8;
        unsigned int RESERVE_SPACE_ONLY                                           :  1;
        unsigned int                                                              :  7;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_EN_CU_13 {
    struct {
        unsigned int EN                                                           :  1;
        unsigned int TYPE_MASK                                                    : 15;
        unsigned int QUEUE_MASK                                                   :  8;
        unsigned int RESERVE_SPACE_ONLY                                           :  1;
        unsigned int                                                              :  7;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_EN_CU_14 {
    struct {
        unsigned int EN                                                           :  1;
        unsigned int TYPE_MASK                                                    : 15;
        unsigned int QUEUE_MASK                                                   :  8;
        unsigned int RESERVE_SPACE_ONLY                                           :  1;
        unsigned int                                                              :  7;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_EN_CU_15 {
    struct {
        unsigned int EN                                                           :  1;
        unsigned int TYPE_MASK                                                    : 15;
        unsigned int QUEUE_MASK                                                   :  8;
        unsigned int RESERVE_SPACE_ONLY                                           :  1;
        unsigned int                                                              :  7;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_COL_FORMAT {
    struct {
        unsigned int COL0_EXPORT_FORMAT                                           :  4;
        unsigned int COL1_EXPORT_FORMAT                                           :  4;
        unsigned int COL2_EXPORT_FORMAT                                           :  4;
        unsigned int COL3_EXPORT_FORMAT                                           :  4;
        unsigned int COL4_EXPORT_FORMAT                                           :  4;
        unsigned int COL5_EXPORT_FORMAT                                           :  4;
        unsigned int COL6_EXPORT_FORMAT                                           :  4;
        unsigned int COL7_EXPORT_FORMAT                                           :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_IDX_FORMAT {
    struct {
        unsigned int IDX0_EXPORT_FORMAT                                           :  4;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_LATE_ALLOC_VS {
    struct {
        unsigned int LIMIT                                                        :  6;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_CHKSUM_GS {
    struct {
        unsigned int CHECKSUM                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_CHKSUM_HS {
    struct {
        unsigned int CHECKSUM                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_CHKSUM_PS {
    struct {
        unsigned int CHECKSUM                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_CHKSUM_VS {
    struct {
        unsigned int CHECKSUM                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_HI_ES {
    struct {
        unsigned int MEM_BASE                                                     :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_HI_ES_GS {
    struct {
        unsigned int MEM_BASE                                                     :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_HI_GS {
    struct {
        unsigned int MEM_BASE                                                     :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_HI_HS {
    struct {
        unsigned int MEM_BASE                                                     :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_HI_LS {
    struct {
        unsigned int MEM_BASE                                                     :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_HI_LS_HS {
    struct {
        unsigned int MEM_BASE                                                     :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_HI_PS {
    struct {
        unsigned int MEM_BASE                                                     :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_HI_VS {
    struct {
        unsigned int MEM_BASE                                                     :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_LO_ES {
    struct {
        unsigned int MEM_BASE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_LO_ES_GS {
    struct {
        unsigned int MEM_BASE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_LO_GS {
    struct {
        unsigned int MEM_BASE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_LO_HS {
    struct {
        unsigned int MEM_BASE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_LO_LS {
    struct {
        unsigned int MEM_BASE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_LO_LS_HS {
    struct {
        unsigned int MEM_BASE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_LO_PS {
    struct {
        unsigned int MEM_BASE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_LO_VS {
    struct {
        unsigned int MEM_BASE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC1_ES {
    struct {
        unsigned int VGPRS                                                        :  6;
        unsigned int SGPRS                                                        :  4;
        unsigned int PRIORITY                                                     :  2;
        unsigned int FLOAT_MODE                                                   :  8;
        unsigned int PRIV                                                         :  1;
        unsigned int DX10_CLAMP                                                   :  1;
        unsigned int DEBUG_MODE                                                   :  1;
        unsigned int IEEE_MODE                                                    :  1;
        unsigned int VGPR_COMP_CNT                                                :  2;
        unsigned int CU_GROUP_ENABLE                                              :  1;
        unsigned int                                                              :  3;
        unsigned int CDBG_USER                                                    :  1;
        unsigned int FP16_OVFL                                                    :  1;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC1_GS {
    struct {
        unsigned int VGPRS                                                        :  6;
        unsigned int SGPRS                                                        :  4;
        unsigned int PRIORITY                                                     :  2;
        unsigned int FLOAT_MODE                                                   :  8;
        unsigned int PRIV                                                         :  1;
        unsigned int DX10_CLAMP                                                   :  1;
        unsigned int DEBUG_MODE                                                   :  1;
        unsigned int IEEE_MODE                                                    :  1;
        unsigned int CU_GROUP_ENABLE                                              :  1;
        unsigned int                                                              :  3;
        unsigned int CDBG_USER                                                    :  1;
        unsigned int GS_VGPR_COMP_CNT                                             :  2;
        unsigned int FP16_OVFL                                                    :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 25;
        unsigned int MEM_ORDERED                                                  :  1;
        unsigned int FWD_PROGRESS                                                 :  1;
        unsigned int WGP_MODE                                                     :  1;
        unsigned int                                                              :  4;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC1_HS {
    struct {
        unsigned int VGPRS                                                        :  6;
        unsigned int SGPRS                                                        :  4;
        unsigned int PRIORITY                                                     :  2;
        unsigned int FLOAT_MODE                                                   :  8;
        unsigned int PRIV                                                         :  1;
        unsigned int DX10_CLAMP                                                   :  1;
        unsigned int DEBUG_MODE                                                   :  1;
        unsigned int IEEE_MODE                                                    :  1;
        unsigned int                                                              :  3;
        unsigned int CDBG_USER                                                    :  1;
        unsigned int LS_VGPR_COMP_CNT                                             :  2;
        unsigned int FP16_OVFL                                                    :  1;
        unsigned int                                                              :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 24;
        unsigned int MEM_ORDERED                                                  :  1;
        unsigned int FWD_PROGRESS                                                 :  1;
        unsigned int WGP_MODE                                                     :  1;
        unsigned int                                                              :  5;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC1_LS {
    struct {
        unsigned int VGPRS                                                        :  6;
        unsigned int SGPRS                                                        :  4;
        unsigned int PRIORITY                                                     :  2;
        unsigned int FLOAT_MODE                                                   :  8;
        unsigned int PRIV                                                         :  1;
        unsigned int DX10_CLAMP                                                   :  1;
        unsigned int DEBUG_MODE                                                   :  1;
        unsigned int IEEE_MODE                                                    :  1;
        unsigned int VGPR_COMP_CNT                                                :  2;
        unsigned int                                                              :  3;
        unsigned int CDBG_USER                                                    :  1;
        unsigned int FP16_OVFL                                                    :  1;
        unsigned int                                                              :  1;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC1_PS {
    struct {
        unsigned int VGPRS                                                        :  6;
        unsigned int SGPRS                                                        :  4;
        unsigned int PRIORITY                                                     :  2;
        unsigned int FLOAT_MODE                                                   :  8;
        unsigned int PRIV                                                         :  1;
        unsigned int DX10_CLAMP                                                   :  1;
        unsigned int DEBUG_MODE                                                   :  1;
        unsigned int IEEE_MODE                                                    :  1;
        unsigned int CU_GROUP_DISABLE                                             :  1;
        unsigned int                                                              :  3;
        unsigned int CDBG_USER                                                    :  1;
        unsigned int FP16_OVFL                                                    :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 25;
        unsigned int MEM_ORDERED                                                  :  1;
        unsigned int FWD_PROGRESS                                                 :  1;
        unsigned int                                                              :  5;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC1_VS {
    struct {
        unsigned int VGPRS                                                        :  6;
        unsigned int SGPRS                                                        :  4;
        unsigned int PRIORITY                                                     :  2;
        unsigned int FLOAT_MODE                                                   :  8;
        unsigned int PRIV                                                         :  1;
        unsigned int DX10_CLAMP                                                   :  1;
        unsigned int DEBUG_MODE                                                   :  1;
        unsigned int IEEE_MODE                                                    :  1;
        unsigned int VGPR_COMP_CNT                                                :  2;
        unsigned int CU_GROUP_ENABLE                                              :  1;
        unsigned int                                                              :  3;
        unsigned int CDBG_USER                                                    :  1;
        unsigned int FP16_OVFL                                                    :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 27;
        unsigned int MEM_ORDERED                                                  :  1;
        unsigned int FWD_PROGRESS                                                 :  1;
        unsigned int                                                              :  3;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC2_ES {
    struct {
        unsigned int SCRATCH_EN                                                   :  1;
        unsigned int USER_SGPR                                                    :  5;
        unsigned int TRAP_PRESENT                                                 :  1;
        unsigned int OC_LDS_EN                                                    :  1;
        unsigned int EXCP_EN                                                      :  9;
        unsigned int                                                              :  3;
        unsigned int LDS_SIZE                                                     :  9;
        unsigned int                                                              :  3;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC2_ES_GS {
    struct {
        unsigned int SCRATCH_EN                                                   :  1;
        unsigned int USER_SGPR                                                    :  5;
        unsigned int TRAP_PRESENT                                                 :  1;
        unsigned int OC_LDS_EN                                                    :  1;
        unsigned int EXCP_EN                                                      :  9;
        unsigned int                                                              :  3;
        unsigned int LDS_SIZE                                                     :  9;
        unsigned int                                                              :  3;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC2_ES_VS {
    struct {
        unsigned int SCRATCH_EN                                                   :  1;
        unsigned int USER_SGPR                                                    :  5;
        unsigned int TRAP_PRESENT                                                 :  1;
        unsigned int OC_LDS_EN                                                    :  1;
        unsigned int EXCP_EN                                                      :  9;
        unsigned int                                                              :  3;
        unsigned int LDS_SIZE                                                     :  9;
        unsigned int                                                              :  3;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC2_GS {
    struct {
        unsigned int SCRATCH_EN                                                   :  1;
        unsigned int USER_SGPR                                                    :  5;
        unsigned int TRAP_PRESENT                                                 :  1;
        unsigned int EXCP_EN                                                      :  9;
        unsigned int ES_VGPR_COMP_CNT                                             :  2;
        unsigned int OC_LDS_EN                                                    :  1;
        unsigned int LDS_SIZE                                                     :  8;
        unsigned int                                                              :  5;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 27;
        unsigned int SKIP_USGPR0                                                  :  1;
        unsigned int USER_SGPR_MSB                                                :  1;
        unsigned int                                                              :  3;
    } gfx09;
    struct {
        unsigned int                                                              : 27;
        unsigned int USER_SGPR_MSB                                                :  1;
        unsigned int SHARED_VGPR_CNT                                              :  4;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC2_GS_VS {
    struct {
        unsigned int SCRATCH_EN                                                   :  1;
        unsigned int USER_SGPR                                                    :  5;
        unsigned int TRAP_PRESENT                                                 :  1;
        unsigned int EXCP_EN                                                      :  9;
        unsigned int VGPR_COMP_CNT                                                :  2;
        unsigned int OC_LDS_EN                                                    :  1;
        unsigned int LDS_SIZE                                                     :  8;
        unsigned int SKIP_USGPR0                                                  :  1;
        unsigned int USER_SGPR_MSB                                                :  1;
        unsigned int                                                              :  3;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC2_HS {
    struct {
        unsigned int SCRATCH_EN                                                   :  1;
        unsigned int USER_SGPR                                                    :  5;
        unsigned int TRAP_PRESENT                                                 :  1;
        unsigned int                                                              : 25;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  7;
        unsigned int EXCP_EN                                                      :  9;
        unsigned int LDS_SIZE                                                     :  9;
        unsigned int                                                              :  2;
        unsigned int SKIP_USGPR0                                                  :  1;
        unsigned int USER_SGPR_MSB                                                :  1;
        unsigned int                                                              :  3;
    } gfx09;
    struct {
        unsigned int                                                              :  7;
        unsigned int OC_LDS_EN                                                    :  1;
        unsigned int TG_SIZE_EN                                                   :  1;
        unsigned int EXCP_EN                                                      :  9;
        unsigned int LDS_SIZE                                                     :  9;
        unsigned int USER_SGPR_MSB                                                :  1;
        unsigned int SHARED_VGPR_CNT                                              :  4;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC2_LS {
    struct {
        unsigned int SCRATCH_EN                                                   :  1;
        unsigned int USER_SGPR                                                    :  5;
        unsigned int TRAP_PRESENT                                                 :  1;
        unsigned int LDS_SIZE                                                     :  9;
        unsigned int EXCP_EN                                                      :  9;
        unsigned int                                                              :  7;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC2_LS_ES {
    struct {
        unsigned int SCRATCH_EN                                                   :  1;
        unsigned int USER_SGPR                                                    :  5;
        unsigned int TRAP_PRESENT                                                 :  1;
        unsigned int LDS_SIZE                                                     :  9;
        unsigned int EXCP_EN                                                      :  9;
        unsigned int                                                              :  7;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC2_LS_HS {
    struct {
        unsigned int SCRATCH_EN                                                   :  1;
        unsigned int USER_SGPR                                                    :  5;
        unsigned int TRAP_PRESENT                                                 :  1;
        unsigned int LDS_SIZE                                                     :  9;
        unsigned int EXCP_EN                                                      :  9;
        unsigned int                                                              :  7;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC2_LS_VS {
    struct {
        unsigned int SCRATCH_EN                                                   :  1;
        unsigned int USER_SGPR                                                    :  5;
        unsigned int TRAP_PRESENT                                                 :  1;
        unsigned int LDS_SIZE                                                     :  9;
        unsigned int EXCP_EN                                                      :  9;
        unsigned int                                                              :  7;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC2_PS {
    struct {
        unsigned int SCRATCH_EN                                                   :  1;
        unsigned int USER_SGPR                                                    :  5;
        unsigned int TRAP_PRESENT                                                 :  1;
        unsigned int WAVE_CNT_EN                                                  :  1;
        unsigned int EXTRA_LDS_SIZE                                               :  8;
        unsigned int EXCP_EN                                                      :  9;
        unsigned int LOAD_COLLISION_WAVEID                                        :  1;
        unsigned int LOAD_INTRAWAVE_COLLISION                                     :  1;
        unsigned int                                                              :  5;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 27;
        unsigned int SKIP_USGPR0                                                  :  1;
        unsigned int USER_SGPR_MSB                                                :  1;
        unsigned int                                                              :  3;
    } gfx09;
    struct {
        unsigned int                                                              : 27;
        unsigned int USER_SGPR_MSB                                                :  1;
        unsigned int SHARED_VGPR_CNT                                              :  4;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC2_VS {
    struct {
        unsigned int SCRATCH_EN                                                   :  1;
        unsigned int USER_SGPR                                                    :  5;
        unsigned int TRAP_PRESENT                                                 :  1;
        unsigned int OC_LDS_EN                                                    :  1;
        unsigned int SO_BASE0_EN                                                  :  1;
        unsigned int SO_BASE1_EN                                                  :  1;
        unsigned int SO_BASE2_EN                                                  :  1;
        unsigned int SO_BASE3_EN                                                  :  1;
        unsigned int SO_EN                                                        :  1;
        unsigned int EXCP_EN                                                      :  9;
        unsigned int PC_BASE_EN                                                   :  1;
        unsigned int                                                              :  9;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 24;
        unsigned int DISPATCH_DRAW_EN                                             :  1;
        unsigned int                                                              :  7;
    } most;
    struct {
        unsigned int                                                              : 27;
        unsigned int SKIP_USGPR0                                                  :  1;
        unsigned int USER_SGPR_MSB                                                :  1;
        unsigned int                                                              :  3;
    } gfx09;
    struct {
        unsigned int                                                              : 27;
        unsigned int USER_SGPR_MSB                                                :  1;
        unsigned int SHARED_VGPR_CNT                                              :  4;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC3_ES {
    struct {
        unsigned int CU_EN                                                        : 16;
        unsigned int WAVE_LIMIT                                                   :  6;
        unsigned int LOCK_LOW_THRESHOLD                                           :  4;
        unsigned int GROUP_FIFO_DEPTH                                             :  6;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC3_GS {
    struct {
        unsigned int CU_EN                                                        : 16;
        unsigned int WAVE_LIMIT                                                   :  6;
        unsigned int LOCK_LOW_THRESHOLD                                           :  4;
        unsigned int                                                              :  6;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 26;
        unsigned int SIMD_DISABLE                                                 :  4;
        unsigned int                                                              :  2;
    } gfx09;
    struct {
        unsigned int                                                              : 26;
        unsigned int GROUP_FIFO_DEPTH                                             :  6;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC3_HS {
    struct {
        unsigned int WAVE_LIMIT                                                   :  6;
        unsigned int LOCK_LOW_THRESHOLD                                           :  4;
        unsigned int                                                              :  6;
        unsigned int CU_EN                                                        : 16;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 10;
        unsigned int SIMD_DISABLE                                                 :  4;
        unsigned int                                                              : 18;
    } gfx09;
    struct {
        unsigned int                                                              : 10;
        unsigned int GROUP_FIFO_DEPTH                                             :  6;
        unsigned int                                                              : 16;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC3_LS {
    struct {
        unsigned int CU_EN                                                        : 16;
        unsigned int WAVE_LIMIT                                                   :  6;
        unsigned int LOCK_LOW_THRESHOLD                                           :  4;
        unsigned int GROUP_FIFO_DEPTH                                             :  6;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC3_PS {
    struct {
        unsigned int CU_EN                                                        : 16;
        unsigned int WAVE_LIMIT                                                   :  6;
        unsigned int LOCK_LOW_THRESHOLD                                           :  4;
        unsigned int                                                              :  6;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 26;
        unsigned int SIMD_DISABLE                                                 :  4;
        unsigned int                                                              :  2;
    } gfx09;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC3_VS {
    struct {
        unsigned int CU_EN                                                        : 16;
        unsigned int WAVE_LIMIT                                                   :  6;
        unsigned int LOCK_LOW_THRESHOLD                                           :  4;
        unsigned int                                                              :  6;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 26;
        unsigned int SIMD_DISABLE                                                 :  4;
        unsigned int                                                              :  2;
    } gfx09;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC4_GS {
    struct {
        unsigned int GROUP_FIFO_DEPTH                                             :  7;
        unsigned int SPI_SHADER_LATE_ALLOC_GS                                     :  7;
        unsigned int                                                              : 18;
    } gfx09;
    struct {
        unsigned int CU_EN                                                        : 16;
        unsigned int SPI_SHADER_LATE_ALLOC_GS                                     :  7;
        unsigned int                                                              :  9;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC4_HS {
    struct {
        unsigned int GROUP_FIFO_DEPTH                                             :  7;
        unsigned int                                                              : 25;
    } gfx09;
    struct {
        unsigned int CU_EN                                                        : 16;
        unsigned int                                                              : 16;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC4_PS {
    struct {
        unsigned int CU_EN                                                        : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC4_VS {
    struct {
        unsigned int CU_EN                                                        : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_POS_FORMAT {
    struct {
        unsigned int POS0_EXPORT_FORMAT                                           :  4;
        unsigned int POS1_EXPORT_FORMAT                                           :  4;
        unsigned int POS2_EXPORT_FORMAT                                           :  4;
        unsigned int POS3_EXPORT_FORMAT                                           :  4;
        unsigned int                                                              : 16;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 16;
        unsigned int POS4_EXPORT_FORMAT                                           :  4;
        unsigned int                                                              : 12;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PREF_PRI_ACCUM_ESGS_0 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int COEFFICIENT_HIER_SELECT                                      :  3;
        unsigned int CONTRIBUTION_HIER_SELECT                                     :  3;
        unsigned int GROUP_UPDATE_EN                                              :  1;
        unsigned int RESERVED                                                     :  1;
        unsigned int COEFFICIENT                                                  :  8;
        unsigned int                                                              :  9;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PREF_PRI_ACCUM_ESGS_1 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int COEFFICIENT_HIER_SELECT                                      :  3;
        unsigned int CONTRIBUTION_HIER_SELECT                                     :  3;
        unsigned int GROUP_UPDATE_EN                                              :  1;
        unsigned int RESERVED                                                     :  1;
        unsigned int COEFFICIENT                                                  :  8;
        unsigned int                                                              :  9;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PREF_PRI_ACCUM_ESGS_2 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int COEFFICIENT_HIER_SELECT                                      :  3;
        unsigned int CONTRIBUTION_HIER_SELECT                                     :  3;
        unsigned int GROUP_UPDATE_EN                                              :  1;
        unsigned int RESERVED                                                     :  1;
        unsigned int COEFFICIENT                                                  :  8;
        unsigned int                                                              :  9;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PREF_PRI_ACCUM_ESGS_3 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int COEFFICIENT_HIER_SELECT                                      :  3;
        unsigned int CONTRIBUTION_HIER_SELECT                                     :  3;
        unsigned int GROUP_UPDATE_EN                                              :  1;
        unsigned int RESERVED                                                     :  1;
        unsigned int COEFFICIENT                                                  :  8;
        unsigned int                                                              :  9;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PREF_PRI_ACCUM_LSHS_0 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int COEFFICIENT_HIER_SELECT                                      :  3;
        unsigned int CONTRIBUTION_HIER_SELECT                                     :  3;
        unsigned int GROUP_UPDATE_EN                                              :  1;
        unsigned int RESERVED                                                     :  1;
        unsigned int COEFFICIENT                                                  :  8;
        unsigned int                                                              :  9;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PREF_PRI_ACCUM_LSHS_1 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int COEFFICIENT_HIER_SELECT                                      :  3;
        unsigned int CONTRIBUTION_HIER_SELECT                                     :  3;
        unsigned int GROUP_UPDATE_EN                                              :  1;
        unsigned int RESERVED                                                     :  1;
        unsigned int COEFFICIENT                                                  :  8;
        unsigned int                                                              :  9;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PREF_PRI_ACCUM_LSHS_2 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int COEFFICIENT_HIER_SELECT                                      :  3;
        unsigned int CONTRIBUTION_HIER_SELECT                                     :  3;
        unsigned int GROUP_UPDATE_EN                                              :  1;
        unsigned int RESERVED                                                     :  1;
        unsigned int COEFFICIENT                                                  :  8;
        unsigned int                                                              :  9;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PREF_PRI_ACCUM_LSHS_3 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int COEFFICIENT_HIER_SELECT                                      :  3;
        unsigned int CONTRIBUTION_HIER_SELECT                                     :  3;
        unsigned int GROUP_UPDATE_EN                                              :  1;
        unsigned int RESERVED                                                     :  1;
        unsigned int COEFFICIENT                                                  :  8;
        unsigned int                                                              :  9;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PREF_PRI_ACCUM_PS_0 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int COEFFICIENT_HIER_SELECT                                      :  3;
        unsigned int CONTRIBUTION_HIER_SELECT                                     :  3;
        unsigned int GROUP_UPDATE_EN                                              :  1;
        unsigned int RESERVED                                                     :  1;
        unsigned int COEFFICIENT                                                  :  8;
        unsigned int                                                              :  9;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PREF_PRI_ACCUM_PS_1 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int COEFFICIENT_HIER_SELECT                                      :  3;
        unsigned int CONTRIBUTION_HIER_SELECT                                     :  3;
        unsigned int GROUP_UPDATE_EN                                              :  1;
        unsigned int RESERVED                                                     :  1;
        unsigned int COEFFICIENT                                                  :  8;
        unsigned int                                                              :  9;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PREF_PRI_ACCUM_PS_2 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int COEFFICIENT_HIER_SELECT                                      :  3;
        unsigned int CONTRIBUTION_HIER_SELECT                                     :  3;
        unsigned int GROUP_UPDATE_EN                                              :  1;
        unsigned int RESERVED                                                     :  1;
        unsigned int COEFFICIENT                                                  :  8;
        unsigned int                                                              :  9;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PREF_PRI_ACCUM_PS_3 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int COEFFICIENT_HIER_SELECT                                      :  3;
        unsigned int CONTRIBUTION_HIER_SELECT                                     :  3;
        unsigned int GROUP_UPDATE_EN                                              :  1;
        unsigned int RESERVED                                                     :  1;
        unsigned int COEFFICIENT                                                  :  8;
        unsigned int                                                              :  9;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PREF_PRI_ACCUM_VS_0 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int COEFFICIENT_HIER_SELECT                                      :  3;
        unsigned int CONTRIBUTION_HIER_SELECT                                     :  3;
        unsigned int GROUP_UPDATE_EN                                              :  1;
        unsigned int RESERVED                                                     :  1;
        unsigned int COEFFICIENT                                                  :  8;
        unsigned int                                                              :  9;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PREF_PRI_ACCUM_VS_1 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int COEFFICIENT_HIER_SELECT                                      :  3;
        unsigned int CONTRIBUTION_HIER_SELECT                                     :  3;
        unsigned int GROUP_UPDATE_EN                                              :  1;
        unsigned int RESERVED                                                     :  1;
        unsigned int COEFFICIENT                                                  :  8;
        unsigned int                                                              :  9;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PREF_PRI_ACCUM_VS_2 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int COEFFICIENT_HIER_SELECT                                      :  3;
        unsigned int CONTRIBUTION_HIER_SELECT                                     :  3;
        unsigned int GROUP_UPDATE_EN                                              :  1;
        unsigned int RESERVED                                                     :  1;
        unsigned int COEFFICIENT                                                  :  8;
        unsigned int                                                              :  9;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PREF_PRI_ACCUM_VS_3 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int COEFFICIENT_HIER_SELECT                                      :  3;
        unsigned int CONTRIBUTION_HIER_SELECT                                     :  3;
        unsigned int GROUP_UPDATE_EN                                              :  1;
        unsigned int RESERVED                                                     :  1;
        unsigned int COEFFICIENT                                                  :  8;
        unsigned int                                                              :  9;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PREF_PRI_CNTR_CTRL_ESGS {
    struct {
        unsigned int TOTAL_WAVE_COUNT_HIER_SELECT                                 :  3;
        unsigned int PER_TYPE_WAVE_COUNT_HIER_SELECT                              :  3;
        unsigned int GROUP_UPDATE_EN                                              :  1;
        unsigned int                                                              :  1;
        unsigned int TOTAL_WAVE_COUNT_COEFFICIENT                                 :  8;
        unsigned int PER_TYPE_WAVE_COUNT_COEFFICIENT                              :  8;
        unsigned int                                                              :  8;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PREF_PRI_CNTR_CTRL_LSHS {
    struct {
        unsigned int TOTAL_WAVE_COUNT_HIER_SELECT                                 :  3;
        unsigned int PER_TYPE_WAVE_COUNT_HIER_SELECT                              :  3;
        unsigned int GROUP_UPDATE_EN                                              :  1;
        unsigned int                                                              :  1;
        unsigned int TOTAL_WAVE_COUNT_COEFFICIENT                                 :  8;
        unsigned int PER_TYPE_WAVE_COUNT_COEFFICIENT                              :  8;
        unsigned int                                                              :  8;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PREF_PRI_CNTR_CTRL_PS {
    struct {
        unsigned int TOTAL_WAVE_COUNT_HIER_SELECT                                 :  3;
        unsigned int PER_TYPE_WAVE_COUNT_HIER_SELECT                              :  3;
        unsigned int GROUP_UPDATE_EN                                              :  1;
        unsigned int                                                              :  1;
        unsigned int TOTAL_WAVE_COUNT_COEFFICIENT                                 :  8;
        unsigned int PER_TYPE_WAVE_COUNT_COEFFICIENT                              :  8;
        unsigned int                                                              :  8;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PREF_PRI_CNTR_CTRL_VS {
    struct {
        unsigned int TOTAL_WAVE_COUNT_HIER_SELECT                                 :  3;
        unsigned int PER_TYPE_WAVE_COUNT_HIER_SELECT                              :  3;
        unsigned int GROUP_UPDATE_EN                                              :  1;
        unsigned int                                                              :  1;
        unsigned int TOTAL_WAVE_COUNT_COEFFICIENT                                 :  8;
        unsigned int PER_TYPE_WAVE_COUNT_COEFFICIENT                              :  8;
        unsigned int                                                              :  8;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_REQ_CTRL_ESGS {
    struct {
        unsigned int SOFT_GROUPING_EN                                             :  1;
        unsigned int NUMBER_OF_REQUESTS_PER_CU                                    :  4;
        unsigned int SOFT_GROUPING_ALLOCATION_TIMEOUT                             :  4;
        unsigned int HARD_LOCK_HYSTERESIS                                         :  1;
        unsigned int HARD_LOCK_LOW_THRESHOLD                                      :  5;
        unsigned int PRODUCER_REQUEST_LOCKOUT                                     :  1;
        unsigned int GLOBAL_SCANNING_EN                                           :  1;
        unsigned int ALLOCATION_RATE_THROTTLING_THRESHOLD                         :  3;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_REQ_CTRL_LSHS {
    struct {
        unsigned int SOFT_GROUPING_EN                                             :  1;
        unsigned int NUMBER_OF_REQUESTS_PER_CU                                    :  4;
        unsigned int SOFT_GROUPING_ALLOCATION_TIMEOUT                             :  4;
        unsigned int HARD_LOCK_HYSTERESIS                                         :  1;
        unsigned int HARD_LOCK_LOW_THRESHOLD                                      :  5;
        unsigned int PRODUCER_REQUEST_LOCKOUT                                     :  1;
        unsigned int GLOBAL_SCANNING_EN                                           :  1;
        unsigned int ALLOCATION_RATE_THROTTLING_THRESHOLD                         :  3;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_REQ_CTRL_PS {
    struct {
        unsigned int SOFT_GROUPING_EN                                             :  1;
        unsigned int NUMBER_OF_REQUESTS_PER_CU                                    :  4;
        unsigned int SOFT_GROUPING_ALLOCATION_TIMEOUT                             :  4;
        unsigned int HARD_LOCK_HYSTERESIS                                         :  1;
        unsigned int HARD_LOCK_LOW_THRESHOLD                                      :  5;
        unsigned int PRODUCER_REQUEST_LOCKOUT                                     :  1;
        unsigned int GLOBAL_SCANNING_EN                                           :  1;
        unsigned int ALLOCATION_RATE_THROTTLING_THRESHOLD                         :  3;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_REQ_CTRL_VS {
    struct {
        unsigned int SOFT_GROUPING_EN                                             :  1;
        unsigned int NUMBER_OF_REQUESTS_PER_CU                                    :  4;
        unsigned int SOFT_GROUPING_ALLOCATION_TIMEOUT                             :  4;
        unsigned int HARD_LOCK_HYSTERESIS                                         :  1;
        unsigned int HARD_LOCK_LOW_THRESHOLD                                      :  5;
        unsigned int PRODUCER_REQUEST_LOCKOUT                                     :  1;
        unsigned int GLOBAL_SCANNING_EN                                           :  1;
        unsigned int ALLOCATION_RATE_THROTTLING_THRESHOLD                         :  3;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_RSRC_LIMIT_CTRL {
    struct {
        unsigned int WAVES_PER_SIMD32                                             :  5;
        unsigned int VGPR_PER_SIMD32                                              :  7;
        unsigned int VGPR_WRAP_DISABLE                                            :  1;
        unsigned int BARRIER_LIMIT                                                :  6;
        unsigned int BARRIER_LIMIT_HIERARCHY_LEVEL                                :  1;
        unsigned int LDS_LIMIT                                                    :  8;
        unsigned int LDS_LIMIT_HIERARCHY_LEVEL                                    :  1;
        unsigned int                                                              :  2;
        unsigned int PERFORMANCE_LIMIT_ENABLE                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_ACCUM_ESGS_0 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_ACCUM_ESGS_1 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_ACCUM_ESGS_2 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_ACCUM_ESGS_3 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_ACCUM_LSHS_0 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_ACCUM_LSHS_1 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_ACCUM_LSHS_2 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_ACCUM_LSHS_3 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_ACCUM_PS_0 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_ACCUM_PS_1 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_ACCUM_PS_2 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_ACCUM_PS_3 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_ACCUM_VS_0 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_ACCUM_VS_1 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_ACCUM_VS_2 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_ACCUM_VS_3 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ADDR_HI_GS {
    struct {
        unsigned int MEM_BASE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ADDR_HI_HS {
    struct {
        unsigned int MEM_BASE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ADDR_LO_GS {
    struct {
        unsigned int MEM_BASE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ADDR_LO_HS {
    struct {
        unsigned int MEM_BASE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_COMMON_0 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_COMMON_1 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_COMMON_2 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_COMMON_3 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_COMMON_4 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_COMMON_5 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_COMMON_6 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_COMMON_7 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_COMMON_8 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_COMMON_9 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_COMMON_10 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_COMMON_11 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_COMMON_12 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_COMMON_13 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_COMMON_14 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_COMMON_15 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_COMMON_16 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_COMMON_17 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_COMMON_18 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_COMMON_19 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_COMMON_20 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_COMMON_21 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_COMMON_22 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_COMMON_23 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_COMMON_24 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_COMMON_25 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_COMMON_26 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_COMMON_27 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_COMMON_28 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_COMMON_29 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_COMMON_30 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_COMMON_31 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ES_0 {
    struct {
        unsigned int DATA                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ES_1 {
    struct {
        unsigned int DATA                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ES_2 {
    struct {
        unsigned int DATA                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ES_3 {
    struct {
        unsigned int DATA                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ES_4 {
    struct {
        unsigned int DATA                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ES_5 {
    struct {
        unsigned int DATA                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ES_6 {
    struct {
        unsigned int DATA                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ES_7 {
    struct {
        unsigned int DATA                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ES_8 {
    struct {
        unsigned int DATA                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ES_9 {
    struct {
        unsigned int DATA                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ES_10 {
    struct {
        unsigned int DATA                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ES_11 {
    struct {
        unsigned int DATA                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ES_12 {
    struct {
        unsigned int DATA                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ES_13 {
    struct {
        unsigned int DATA                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ES_14 {
    struct {
        unsigned int DATA                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ES_15 {
    struct {
        unsigned int DATA                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ES_16 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ES_17 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ES_18 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ES_19 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ES_20 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ES_21 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ES_22 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ES_23 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ES_24 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ES_25 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ES_26 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ES_27 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ES_28 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ES_29 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ES_30 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ES_31 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_0 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_1 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_2 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_3 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_4 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_5 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_6 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_7 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_8 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_9 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_10 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_11 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_12 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_13 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_14 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_15 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_16 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_17 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_18 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_19 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_20 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_21 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_22 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_23 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_24 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_25 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_26 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_27 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_28 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_29 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_30 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_31 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_0 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_1 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_2 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_3 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_4 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_5 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_6 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_7 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_8 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_9 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_10 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_11 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_12 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_13 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_14 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_15 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_16 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_17 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_18 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_19 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_20 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_21 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_22 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_23 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_24 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_25 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_26 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_27 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_28 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_29 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_30 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_31 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_LS_0 {
    struct {
        unsigned int DATA                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_LS_1 {
    struct {
        unsigned int DATA                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_LS_2 {
    struct {
        unsigned int DATA                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_LS_3 {
    struct {
        unsigned int DATA                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_LS_4 {
    struct {
        unsigned int DATA                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_LS_5 {
    struct {
        unsigned int DATA                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_LS_6 {
    struct {
        unsigned int DATA                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_LS_7 {
    struct {
        unsigned int DATA                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_LS_8 {
    struct {
        unsigned int DATA                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_LS_9 {
    struct {
        unsigned int DATA                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_LS_10 {
    struct {
        unsigned int DATA                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_LS_11 {
    struct {
        unsigned int DATA                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_LS_12 {
    struct {
        unsigned int DATA                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_LS_13 {
    struct {
        unsigned int DATA                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_LS_14 {
    struct {
        unsigned int DATA                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_LS_15 {
    struct {
        unsigned int DATA                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_LS_16 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_LS_17 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_LS_18 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_LS_19 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_LS_20 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_LS_21 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_LS_22 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_LS_23 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_LS_24 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_LS_25 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_LS_26 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_LS_27 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_LS_28 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_LS_29 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_LS_30 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_LS_31 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_0 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_1 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_2 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_3 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_4 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_5 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_6 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_7 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_8 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_9 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_10 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_11 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_12 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_13 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_14 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_15 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_16 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_17 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_18 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_19 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_20 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_21 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_22 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_23 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_24 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_25 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_26 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_27 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_28 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_29 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_30 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_31 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_VS_0 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_VS_1 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_VS_2 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_VS_3 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_VS_4 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_VS_5 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_VS_6 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_VS_7 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_VS_8 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_VS_9 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_VS_10 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_VS_11 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_VS_12 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_VS_13 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_VS_14 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_VS_15 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_VS_16 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_VS_17 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_VS_18 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_VS_19 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_VS_20 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_VS_21 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_VS_22 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_VS_23 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_VS_24 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_VS_25 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_VS_26 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_VS_27 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_VS_28 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_VS_29 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_VS_30 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_VS_31 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_Z_FORMAT {
    struct {
        unsigned int Z_EXPORT_FORMAT                                              :  4;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_START_PHASE {
    struct {
        unsigned int VGPR_START_PHASE                                             :  2;
        unsigned int SGPR_START_PHASE                                             :  2;
        unsigned int WAVE_START_PHASE                                             :  2;
        unsigned int                                                              : 26;
    } gfx09;
    struct {
        unsigned int PC_X_PHASE_SE0                                               :  2;
        unsigned int PC_X_PHASE_SE1                                               :  2;
        unsigned int PC_X_PHASE_SE2                                               :  2;
        unsigned int PC_X_PHASE_SE3                                               :  2;
        unsigned int                                                              : 24;
    } gfx101Plus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SX_EXPORT_BUFFER_SIZES {
    struct {
        unsigned int COLOR_BUFFER_SIZE                                            : 16;
        unsigned int POSITION_BUFFER_SIZE                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SX_SCOREBOARD_BUFFER_SIZES {
    struct {
        unsigned int COLOR_SCOREBOARD_SIZE                                        : 16;
        unsigned int POSITION_SCOREBOARD_SIZE                                     : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SYS_COMPUTE {
    struct {
        unsigned int PIPE                                                         :  8;
        unsigned int                                                              : 24;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SYS_WIF_CNTL {
    struct {
        unsigned int THRESHOLD                                                    :  8;
        unsigned int                                                              : 24;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_TMPRING_SIZE {
    struct {
        unsigned int WAVES                                                        : 12;
        unsigned int WAVESIZE                                                     : 13;
        unsigned int                                                              :  7;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_USER_ACCUM_VMID_CNTL {
    struct {
        unsigned int EN_USER_ACCUM                                                :  4;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_VS_OUT_CONFIG {
    struct {
        unsigned int                                                              :  1;
        unsigned int VS_EXPORT_COUNT                                              :  5;
        unsigned int VS_HALF_PACK                                                 :  1;
        unsigned int                                                              : 25;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  7;
        unsigned int NO_PC_EXPORT                                                 :  1;
        unsigned int                                                              : 24;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_vs_wave_ctl0 {
    struct {
        unsigned int f_double_data                                                :  1;
        unsigned int f_vs_vert_gs_on                                              :  1;
        unsigned int f_gpr_ld_line_sel                                            :  4;
        unsigned int wave_buffer_fifo_empty                                       :  1;
        unsigned int vs_wave_first_subgrp                                         :  1;
        unsigned int gsc_vsc_group_cu_id                                          :  4;
        unsigned int vs_gdbg_en_q                                                 :  1;
        unsigned int tr_fits                                                      :  1;
        unsigned int vs_wave_cnt_lt_lim                                           :  1;
        unsigned int pipe_id                                                      :  1;
        unsigned int late_alloc_lt_lim                                            :  1;
        unsigned int outstanding_waves                                            :  4;
        unsigned int outstanding_events                                           :  3;
        unsigned int Reserved0                                                    :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_vs_wave_ctl1 {
    struct {
        unsigned int crawler_is_event                                             :  1;
        unsigned int stall_events                                                 :  1;
        unsigned int wave_done_crawler_id                                         : 10;
        unsigned int wave_done                                                    :  1;
        unsigned int crawler_full                                                 :  1;
        unsigned int crawler_empty                                                :  1;
        unsigned int crawler_rd                                                   :  9;
        unsigned int Reserved0                                                    :  8;
    } gfx09_0;
    struct {
        unsigned int crawler_is_event                                             :  1;
        unsigned int stall_events                                                 :  1;
        unsigned int wave_done_crawler_id                                         : 10;
        unsigned int wave_done                                                    :  1;
        unsigned int crawler_full                                                 :  1;
        unsigned int crawler_empty                                                :  1;
        unsigned int crawler_rd                                                   :  9;
        unsigned int Reserved0                                                    :  8;
    } rv2x;
    struct {
        unsigned int crawler_is_event                                             :  1;
        unsigned int stall_events                                                 :  1;
        unsigned int wave_done_crawler_id                                         : 10;
        unsigned int wave_done                                                    :  1;
        unsigned int crawler_full                                                 :  1;
        unsigned int crawler_empty                                                :  1;
        unsigned int crawler_rd                                                   :  9;
        unsigned int Reserved0                                                    :  8;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union spi_vs_wave_ctl2 {
    struct {
        unsigned int res_alloc_state                                              :  2;
        unsigned int vsc_ra_rts                                                   :  1;
        unsigned int clocks_on                                                    :  1;
        unsigned int UNUSED_vsc_ra_alloc_req1                                     :  1;
        unsigned int vsc_ra_alloc_req0                                            :  1;
        unsigned int UNUSED_vsc_pc_posb1_alloc_req                                :  1;
        unsigned int vsc_pc_posb0_alloc_req                                       :  1;
        unsigned int vsc_ra_cu_id                                                 :  4;
        unsigned int hsc_group_cu_id                                              :  4;
        unsigned int hsc_group_fifo_empty                                         :  1;
        unsigned int vs_wave_first_wave                                           :  1;
        unsigned int vs_wave_stateid                                              :  3;
        unsigned int vs_wave_is_event                                             :  1;
        unsigned int vgt_vswave_fifo_empty                                        :  1;
        unsigned int gsc_vsc_group_fifo_empty                                     :  1;
        unsigned int Reserved0                                                    :  8;
    } gfx09_0;
    struct {
        unsigned int res_alloc_state                                              :  2;
        unsigned int vsc_ra_rts                                                   :  1;
        unsigned int clocks_on                                                    :  1;
        unsigned int UNUSED_vsc_ra_alloc_req1                                     :  1;
        unsigned int vsc_ra_alloc_req0                                            :  1;
        unsigned int UNUSED_vsc_pc_posb1_alloc_req                                :  1;
        unsigned int vsc_pc_posb0_alloc_req                                       :  1;
        unsigned int vsc_ra_cu_id                                                 :  4;
        unsigned int hsc_group_cu_id                                              :  4;
        unsigned int hsc_group_fifo_empty                                         :  1;
        unsigned int vs_wave_first_wave                                           :  1;
        unsigned int vs_wave_stateid                                              :  3;
        unsigned int vs_wave_is_event                                             :  1;
        unsigned int vgt_vswave_fifo_empty                                        :  1;
        unsigned int gsc_vsc_group_fifo_empty                                     :  1;
        unsigned int Reserved0                                                    :  8;
    } rv2x;
    struct {
        unsigned int res_alloc_state                                              :  2;
        unsigned int vsc_ra_rts                                                   :  1;
        unsigned int clocks_on                                                    :  1;
        unsigned int UNUSED_vsc_ra_alloc_req1                                     :  1;
        unsigned int vsc_ra_alloc_req0                                            :  1;
        unsigned int UNUSED_vsc_pc_posb1_alloc_req                                :  1;
        unsigned int vsc_pc_posb0_alloc_req                                       :  1;
        unsigned int vsc_ra_cu_id                                                 :  4;
        unsigned int hsc_group_cu_id                                              :  4;
        unsigned int hsc_group_fifo_empty                                         :  1;
        unsigned int vs_wave_first_wave                                           :  1;
        unsigned int vs_wave_stateid                                              :  3;
        unsigned int vs_wave_is_event                                             :  1;
        unsigned int vgt_vswave_fifo_empty                                        :  1;
        unsigned int gsc_vsc_group_fifo_empty                                     :  1;
        unsigned int Reserved0                                                    :  8;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WAVE_LIMIT_CNTL {
    struct {
        unsigned int PS_WAVE_GRAN                                                 :  2;
        unsigned int VS_WAVE_GRAN                                                 :  2;
        unsigned int GS_WAVE_GRAN                                                 :  2;
        unsigned int HS_WAVE_GRAN                                                 :  2;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WAVE_LIMIT_CNTL_REMAP {
    struct {
        unsigned int RESERVED                                                     : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WCL_PIPE_PERCENT_CS0 {
    struct {
        unsigned int VALUE                                                        :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WCL_PIPE_PERCENT_CS1 {
    struct {
        unsigned int VALUE                                                        :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WCL_PIPE_PERCENT_CS2 {
    struct {
        unsigned int VALUE                                                        :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WCL_PIPE_PERCENT_CS3 {
    struct {
        unsigned int VALUE                                                        :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WCL_PIPE_PERCENT_CS4 {
    struct {
        unsigned int VALUE                                                        :  7;
        unsigned int                                                              : 25;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WCL_PIPE_PERCENT_CS5 {
    struct {
        unsigned int VALUE                                                        :  7;
        unsigned int                                                              : 25;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WCL_PIPE_PERCENT_CS6 {
    struct {
        unsigned int VALUE                                                        :  7;
        unsigned int                                                              : 25;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WCL_PIPE_PERCENT_CS7 {
    struct {
        unsigned int VALUE                                                        :  7;
        unsigned int                                                              : 25;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WCL_PIPE_PERCENT_GFX {
    struct {
        unsigned int VALUE                                                        :  7;
        unsigned int                                                              :  5;
        unsigned int HS_GRP_VALUE                                                 :  5;
        unsigned int                                                              :  5;
        unsigned int GS_GRP_VALUE                                                 :  5;
        unsigned int                                                              :  5;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  7;
        unsigned int LS_GRP_VALUE                                                 :  5;
        unsigned int                                                              :  5;
        unsigned int ES_GRP_VALUE                                                 :  5;
        unsigned int                                                              : 10;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WCL_PIPE_PERCENT_HP3D {
    struct {
        unsigned int VALUE                                                        :  7;
        unsigned int                                                              :  5;
        unsigned int HS_GRP_VALUE                                                 :  5;
        unsigned int                                                              :  5;
        unsigned int GS_GRP_VALUE                                                 :  5;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_CNTL {
    struct {
        unsigned int SAMPLE_PERIOD                                                :  4;
        unsigned int EN                                                           :  1;
        unsigned int                                                              : 27;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_DEBUG {
    struct {
        unsigned int START_VALUE                                                  : 31;
        unsigned int OVERRIDE_EN                                                  :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_LIMIT_0 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int EN_WARN                                                      :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_LIMIT_1 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int EN_WARN                                                      :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_LIMIT_2 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int EN_WARN                                                      :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_LIMIT_3 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int EN_WARN                                                      :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_LIMIT_4 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int EN_WARN                                                      :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_LIMIT_5 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int EN_WARN                                                      :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_LIMIT_6 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int EN_WARN                                                      :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_LIMIT_7 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int EN_WARN                                                      :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_LIMIT_8 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int EN_WARN                                                      :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_LIMIT_9 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int EN_WARN                                                      :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_STATUS_0 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int INT_SENT                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_STATUS_1 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int INT_SENT                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_STATUS_2 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int INT_SENT                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_STATUS_3 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int INT_SENT                                                     :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_STATUS_4 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int INT_SENT                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_STATUS_5 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int INT_SENT                                                     :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_STATUS_6 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int INT_SENT                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_STATUS_7 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int INT_SENT                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_STATUS_8 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int INT_SENT                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_STATUS_9 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int INT_SENT                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_STATUS_10 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int INT_SENT                                                     :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_STATUS_11 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int INT_SENT                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_STATUS_12 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int INT_SENT                                                     :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_STATUS_13 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int INT_SENT                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_STATUS_14 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int INT_SENT                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_STATUS_15 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int INT_SENT                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_STATUS_16 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int INT_SENT                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_STATUS_17 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int INT_SENT                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_STATUS_18 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int INT_SENT                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_STATUS_19 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int INT_SENT                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_STATUS_20 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int INT_SENT                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SP_CONFIG {
    struct {
        unsigned int DEST_CACHE_EVICT_COUNTER                                     :  2;
        unsigned int ALU_BUSY_MGCG_OVERRIDE                                       :  1;
        unsigned int DISABLE_TRANS_COEXEC                                         :  1;
        unsigned int                                                              : 28;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  8;
        unsigned int UNUSED                                                       :  1;
        unsigned int                                                              : 23;
    } gfx101;
    struct {
        unsigned int                                                              :  4;
        unsigned int TRANS_MGCG_OVERRIDE                                          :  1;
        unsigned int CAC_COUNTER_OVERRIDE                                         :  1;
        unsigned int DPMACC_MGCG_OVERRIDE                                         :  1;
        unsigned int SMACC_MGCG_OVERRIDE                                          :  1;
        unsigned int                                                              : 24;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQC_CACHES {
    struct {
        unsigned int TARGET_INST                                                  :  1;
        unsigned int TARGET_DATA                                                  :  1;
        unsigned int INVALIDATE                                                   :  1;
        unsigned int                                                              : 13;
        unsigned int COMPLETE                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  3;
        unsigned int WRITEBACK                                                    :  1;
        unsigned int VOL                                                          :  1;
        unsigned int                                                              : 27;
    } most;
    struct {
        unsigned int                                                              : 17;
        unsigned int L2_WB_POLICY                                                 :  2;
        unsigned int                                                              : 13;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQC_CONFIG {
    struct {
        unsigned int INST_CACHE_SIZE                                              :  2;
        unsigned int DATA_CACHE_SIZE                                              :  2;
        unsigned int MISS_FIFO_DEPTH                                              :  2;
        unsigned int HIT_FIFO_DEPTH                                               :  1;
        unsigned int FORCE_ALWAYS_MISS                                            :  1;
        unsigned int FORCE_IN_ORDER                                               :  1;
        unsigned int                                                              :  1;
        unsigned int IDENTITY_HASH_SET                                            :  1;
        unsigned int PER_VMID_INV_DISABLE                                         :  1;
        unsigned int EVICT_LRU                                                    :  2;
        unsigned int FORCE_2_BANK                                                 :  1;
        unsigned int FORCE_1_BANK                                                 :  1;
        unsigned int LS_DISABLE_CLOCKS                                            :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  9;
        unsigned int IDENTITY_HASH_BANK                                           :  1;
        unsigned int                                                              : 22;
    } most;
    struct {
        unsigned int                                                              : 24;
        unsigned int INST_PRF_COUNT                                               :  2;
        unsigned int INST_PRF_FILTER_DIS                                          :  1;
        unsigned int                                                              :  5;
    } gfx09;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQC_DCACHE_UTCL0_CNTL1 {
    struct {
        unsigned int FORCE_4K_L2_RESP                                             :  1;
        unsigned int GPUVM_64K_DEF                                                :  1;
        unsigned int GPUVM_PERM_MODE                                              :  1;
        unsigned int RESP_MODE                                                    :  2;
        unsigned int RESP_FAULT_MODE                                              :  2;
        unsigned int CLIENTID                                                     :  9;
        unsigned int                                                              :  1;
        unsigned int ENABLE_PUSH_LFIFO                                            :  1;
        unsigned int ENABLE_LFIFO_PRI_ARB                                         :  1;
        unsigned int REG_INV_VMID                                                 :  4;
        unsigned int REG_INV_ALL_VMID                                             :  1;
        unsigned int REG_INV_TOGGLE                                               :  1;
        unsigned int CLIENT_INVALIDATE_ALL_VMID                                   :  1;
        unsigned int FORCE_MISS                                                   :  1;
        unsigned int FORCE_IN_ORDER                                               :  1;
        unsigned int REDUCE_FIFO_DEPTH_BY_2                                       :  2;
        unsigned int REDUCE_CACHE_SIZE_BY_2                                       :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQC_DCACHE_UTCL0_CNTL2 {
    struct {
        unsigned int SPARE                                                        :  8;
        unsigned int LFIFO_SCAN_DISABLE                                           :  1;
        unsigned int MTYPE_OVRD_DIS                                               :  1;
        unsigned int LINE_VALID                                                   :  1;
        unsigned int DIS_EDC                                                      :  1;
        unsigned int GPUVM_INV_MODE                                               :  1;
        unsigned int SHOOTDOWN_OPT                                                :  1;
        unsigned int FORCE_SNOOP                                                  :  1;
        unsigned int FORCE_GPUVM_INV_ACK                                          :  1;
        unsigned int ARB_BURST_MODE                                               :  2;
        unsigned int ENABLE_PERF_EVENT_RD_WR                                      :  1;
        unsigned int PERF_EVENT_RD_WR                                             :  1;
        unsigned int ENABLE_PERF_EVENT_VMID                                       :  1;
        unsigned int PERF_EVENT_VMID                                              :  4;
        unsigned int                                                              :  1;
        unsigned int FORCE_FRAG_2M_TO_64K                                         :  1;
        unsigned int PERM_MODE_OVRD                                               :  1;
        unsigned int LINE_INVALIDATE_OPT                                          :  1;
        unsigned int GPUVM_16K_DEF                                                :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQC_DCACHE_UTCL0_STATUS {
    struct {
        unsigned int FAULT_DETECTED                                               :  1;
        unsigned int RETRY_DETECTED                                               :  1;
        unsigned int PRT_DETECTED                                                 :  1;
        unsigned int                                                              : 29;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQC_DCACHE_UTCL1_CNTL1 {
    struct {
        unsigned int FORCE_4K_L2_RESP                                             :  1;
        unsigned int GPUVM_64K_DEF                                                :  1;
        unsigned int GPUVM_PERM_MODE                                              :  1;
        unsigned int RESP_MODE                                                    :  2;
        unsigned int RESP_FAULT_MODE                                              :  2;
        unsigned int CLIENTID                                                     :  9;
        unsigned int                                                              :  1;
        unsigned int ENABLE_PUSH_LFIFO                                            :  1;
        unsigned int ENABLE_LFIFO_PRI_ARB                                         :  1;
        unsigned int REG_INVALIDATE_VMID                                          :  4;
        unsigned int REG_INVALIDATE_ALL_VMID                                      :  1;
        unsigned int REG_INVALIDATE_TOGGLE                                        :  1;
        unsigned int CLIENT_INVALIDATE_ALL_VMID                                   :  1;
        unsigned int FORCE_MISS                                                   :  1;
        unsigned int FORCE_IN_ORDER                                               :  1;
        unsigned int REDUCE_FIFO_DEPTH_BY_2                                       :  2;
        unsigned int REDUCE_CACHE_SIZE_BY_2                                       :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQC_DCACHE_UTCL1_CNTL2 {
    struct {
        unsigned int SPARE                                                        :  8;
        unsigned int LFIFO_SCAN_DISABLE                                           :  1;
        unsigned int MTYPE_OVRD_DIS                                               :  1;
        unsigned int LINE_VALID                                                   :  1;
        unsigned int DIS_EDC                                                      :  1;
        unsigned int GPUVM_INV_MODE                                               :  1;
        unsigned int SHOOTDOWN_OPT                                                :  1;
        unsigned int FORCE_SNOOP                                                  :  1;
        unsigned int FORCE_GPUVM_INV_ACK                                          :  1;
        unsigned int ARB_BURST_MODE                                               :  2;
        unsigned int ENABLE_PERF_EVENT_RD_WR                                      :  1;
        unsigned int PERF_EVENT_RD_WR                                             :  1;
        unsigned int ENABLE_PERF_EVENT_VMID                                       :  1;
        unsigned int PERF_EVENT_VMID                                              :  4;
        unsigned int                                                              :  1;
        unsigned int FORCE_FRAG_2M_TO_64K                                         :  1;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQC_DCACHE_UTCL1_STATUS {
    struct {
        unsigned int FAULT_DETECTED                                               :  1;
        unsigned int RETRY_DETECTED                                               :  1;
        unsigned int PRT_DETECTED                                                 :  1;
        unsigned int                                                              : 29;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQC_DSM_CNTL {
    struct {
        unsigned int INST_UTCL1_LFIFO_DSM_IRRITATOR_DATA                          :  2;
        unsigned int INST_UTCL1_LFIFO_ENABLE_SINGLE_WRITE                         :  1;
        unsigned int DATA_CU0_WRITE_DATA_BUF_DSM_IRRITATOR_DATA                   :  2;
        unsigned int DATA_CU0_WRITE_DATA_BUF_ENABLE_SINGLE_WRITE                  :  1;
        unsigned int DATA_CU0_UTCL1_LFIFO_DSM_IRRITATOR_DATA                      :  2;
        unsigned int DATA_CU0_UTCL1_LFIFO_ENABLE_SINGLE_WRITE                     :  1;
        unsigned int DATA_CU1_WRITE_DATA_BUF_DSM_IRRITATOR_DATA                   :  2;
        unsigned int DATA_CU1_WRITE_DATA_BUF_ENABLE_SINGLE_WRITE                  :  1;
        unsigned int DATA_CU1_UTCL1_LFIFO_DSM_IRRITATOR_DATA                      :  2;
        unsigned int DATA_CU1_UTCL1_LFIFO_ENABLE_SINGLE_WRITE                     :  1;
        unsigned int DATA_CU2_WRITE_DATA_BUF_DSM_IRRITATOR_DATA                   :  2;
        unsigned int DATA_CU2_WRITE_DATA_BUF_ENABLE_SINGLE_WRITE                  :  1;
        unsigned int DATA_CU2_UTCL1_LFIFO_DSM_IRRITATOR_DATA                      :  2;
        unsigned int DATA_CU2_UTCL1_LFIFO_ENABLE_SINGLE_WRITE                     :  1;
        unsigned int                                                              : 11;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQC_DSM_CNTL2 {
    struct {
        unsigned int INST_UTCL1_LFIFO_ENABLE_ERROR_INJECT                         :  2;
        unsigned int INST_UTCL1_LFIFO_SELECT_INJECT_DELAY                         :  1;
        unsigned int DATA_CU0_WRITE_DATA_BUF_ENABLE_ERROR_INJECT                  :  2;
        unsigned int DATA_CU0_WRITE_DATA_BUF_SELECT_INJECT_DELAY                  :  1;
        unsigned int DATA_CU0_UTCL1_LFIFO_ENABLE_ERROR_INJECT                     :  2;
        unsigned int DATA_CU0_UTCL1_LFIFO_SELECT_INJECT_DELAY                     :  1;
        unsigned int DATA_CU1_WRITE_DATA_BUF_ENABLE_ERROR_INJECT                  :  2;
        unsigned int DATA_CU1_WRITE_DATA_BUF_SELECT_INJECT_DELAY                  :  1;
        unsigned int DATA_CU1_UTCL1_LFIFO_ENABLE_ERROR_INJECT                     :  2;
        unsigned int DATA_CU1_UTCL1_LFIFO_SELECT_INJECT_DELAY                     :  1;
        unsigned int DATA_CU2_WRITE_DATA_BUF_ENABLE_ERROR_INJECT                  :  2;
        unsigned int DATA_CU2_WRITE_DATA_BUF_SELECT_INJECT_DELAY                  :  1;
        unsigned int DATA_CU2_UTCL1_LFIFO_ENABLE_ERROR_INJECT                     :  2;
        unsigned int DATA_CU2_UTCL1_LFIFO_SELECT_INJECT_DELAY                     :  1;
        unsigned int                                                              :  5;
        unsigned int INJECT_DELAY                                                 :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQC_DSM_CNTL2A {
    struct {
        unsigned int INST_TAG_RAM_ENABLE_ERROR_INJECT                             :  2;
        unsigned int INST_TAG_RAM_SELECT_INJECT_DELAY                             :  1;
        unsigned int INST_UTCL1_MISS_FIFO_ENABLE_ERROR_INJECT                     :  2;
        unsigned int INST_UTCL1_MISS_FIFO_SELECT_INJECT_DELAY                     :  1;
        unsigned int INST_MISS_FIFO_ENABLE_ERROR_INJECT                           :  2;
        unsigned int INST_MISS_FIFO_SELECT_INJECT_DELAY                           :  1;
        unsigned int INST_BANK_RAM_ENABLE_ERROR_INJECT                            :  2;
        unsigned int INST_BANK_RAM_SELECT_INJECT_DELAY                            :  1;
        unsigned int DATA_TAG_RAM_ENABLE_ERROR_INJECT                             :  2;
        unsigned int DATA_TAG_RAM_SELECT_INJECT_DELAY                             :  1;
        unsigned int DATA_HIT_FIFO_ENABLE_ERROR_INJECT                            :  2;
        unsigned int DATA_HIT_FIFO_SELECT_INJECT_DELAY                            :  1;
        unsigned int DATA_MISS_FIFO_ENABLE_ERROR_INJECT                           :  2;
        unsigned int DATA_MISS_FIFO_SELECT_INJECT_DELAY                           :  1;
        unsigned int DATA_DIRTY_BIT_RAM_ENABLE_ERROR_INJECT                       :  2;
        unsigned int DATA_DIRTY_BIT_RAM_SELECT_INJECT_DELAY                       :  1;
        unsigned int DATA_BANK_RAM_ENABLE_ERROR_INJECT                            :  2;
        unsigned int DATA_BANK_RAM_SELECT_INJECT_DELAY                            :  1;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQC_DSM_CNTL2B {
    struct {
        unsigned int INST_TAG_RAM_ENABLE_ERROR_INJECT                             :  2;
        unsigned int INST_TAG_RAM_SELECT_INJECT_DELAY                             :  1;
        unsigned int INST_UTCL1_MISS_FIFO_ENABLE_ERROR_INJECT                     :  2;
        unsigned int INST_UTCL1_MISS_FIFO_SELECT_INJECT_DELAY                     :  1;
        unsigned int INST_MISS_FIFO_ENABLE_ERROR_INJECT                           :  2;
        unsigned int INST_MISS_FIFO_SELECT_INJECT_DELAY                           :  1;
        unsigned int INST_BANK_RAM_ENABLE_ERROR_INJECT                            :  2;
        unsigned int INST_BANK_RAM_SELECT_INJECT_DELAY                            :  1;
        unsigned int DATA_TAG_RAM_ENABLE_ERROR_INJECT                             :  2;
        unsigned int DATA_TAG_RAM_SELECT_INJECT_DELAY                             :  1;
        unsigned int DATA_HIT_FIFO_ENABLE_ERROR_INJECT                            :  2;
        unsigned int DATA_HIT_FIFO_SELECT_INJECT_DELAY                            :  1;
        unsigned int DATA_MISS_FIFO_ENABLE_ERROR_INJECT                           :  2;
        unsigned int DATA_MISS_FIFO_SELECT_INJECT_DELAY                           :  1;
        unsigned int DATA_DIRTY_BIT_RAM_ENABLE_ERROR_INJECT                       :  2;
        unsigned int DATA_DIRTY_BIT_RAM_SELECT_INJECT_DELAY                       :  1;
        unsigned int DATA_BANK_RAM_ENABLE_ERROR_INJECT                            :  2;
        unsigned int DATA_BANK_RAM_SELECT_INJECT_DELAY                            :  1;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQC_DSM_CNTLA {
    struct {
        unsigned int INST_TAG_RAM_DSM_IRRITATOR_DATA                              :  2;
        unsigned int INST_TAG_RAM_ENABLE_SINGLE_WRITE                             :  1;
        unsigned int INST_UTCL1_MISS_FIFO_DSM_IRRITATOR_DATA                      :  2;
        unsigned int INST_UTCL1_MISS_FIFO_ENABLE_SINGLE_WRITE                     :  1;
        unsigned int INST_MISS_FIFO_DSM_IRRITATOR_DATA                            :  2;
        unsigned int INST_MISS_FIFO_ENABLE_SINGLE_WRITE                           :  1;
        unsigned int INST_BANK_RAM_DSM_IRRITATOR_DATA                             :  2;
        unsigned int INST_BANK_RAM_ENABLE_SINGLE_WRITE                            :  1;
        unsigned int DATA_TAG_RAM_DSM_IRRITATOR_DATA                              :  2;
        unsigned int DATA_TAG_RAM_ENABLE_SINGLE_WRITE                             :  1;
        unsigned int DATA_HIT_FIFO_DSM_IRRITATOR_DATA                             :  2;
        unsigned int DATA_HIT_FIFO_ENABLE_SINGLE_WRITE                            :  1;
        unsigned int DATA_MISS_FIFO_DSM_IRRITATOR_DATA                            :  2;
        unsigned int DATA_MISS_FIFO_ENABLE_SINGLE_WRITE                           :  1;
        unsigned int DATA_DIRTY_BIT_RAM_DSM_IRRITATOR_DATA                        :  2;
        unsigned int DATA_DIRTY_BIT_RAM_ENABLE_SINGLE_WRITE                       :  1;
        unsigned int DATA_BANK_RAM_DSM_IRRITATOR_DATA                             :  2;
        unsigned int DATA_BANK_RAM_ENABLE_SINGLE_WRITE                            :  1;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQC_DSM_CNTLB {
    struct {
        unsigned int INST_TAG_RAM_DSM_IRRITATOR_DATA                              :  2;
        unsigned int INST_TAG_RAM_ENABLE_SINGLE_WRITE                             :  1;
        unsigned int INST_UTCL1_MISS_FIFO_DSM_IRRITATOR_DATA                      :  2;
        unsigned int INST_UTCL1_MISS_FIFO_ENABLE_SINGLE_WRITE                     :  1;
        unsigned int INST_MISS_FIFO_DSM_IRRITATOR_DATA                            :  2;
        unsigned int INST_MISS_FIFO_ENABLE_SINGLE_WRITE                           :  1;
        unsigned int INST_BANK_RAM_DSM_IRRITATOR_DATA                             :  2;
        unsigned int INST_BANK_RAM_ENABLE_SINGLE_WRITE                            :  1;
        unsigned int DATA_TAG_RAM_DSM_IRRITATOR_DATA                              :  2;
        unsigned int DATA_TAG_RAM_ENABLE_SINGLE_WRITE                             :  1;
        unsigned int DATA_HIT_FIFO_DSM_IRRITATOR_DATA                             :  2;
        unsigned int DATA_HIT_FIFO_ENABLE_SINGLE_WRITE                            :  1;
        unsigned int DATA_MISS_FIFO_DSM_IRRITATOR_DATA                            :  2;
        unsigned int DATA_MISS_FIFO_ENABLE_SINGLE_WRITE                           :  1;
        unsigned int DATA_DIRTY_BIT_RAM_DSM_IRRITATOR_DATA                        :  2;
        unsigned int DATA_DIRTY_BIT_RAM_ENABLE_SINGLE_WRITE                       :  1;
        unsigned int DATA_BANK_RAM_DSM_IRRITATOR_DATA                             :  2;
        unsigned int DATA_BANK_RAM_ENABLE_SINGLE_WRITE                            :  1;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQC_EDC_CNT {
    struct {
        unsigned int DATA_CU0_WRITE_DATA_BUF_SEC_COUNT                            :  2;
        unsigned int DATA_CU0_WRITE_DATA_BUF_DED_COUNT                            :  2;
        unsigned int DATA_CU0_UTCL1_LFIFO_SEC_COUNT                               :  2;
        unsigned int DATA_CU0_UTCL1_LFIFO_DED_COUNT                               :  2;
        unsigned int DATA_CU1_WRITE_DATA_BUF_SEC_COUNT                            :  2;
        unsigned int DATA_CU1_WRITE_DATA_BUF_DED_COUNT                            :  2;
        unsigned int DATA_CU1_UTCL1_LFIFO_SEC_COUNT                               :  2;
        unsigned int DATA_CU1_UTCL1_LFIFO_DED_COUNT                               :  2;
        unsigned int DATA_CU2_WRITE_DATA_BUF_SEC_COUNT                            :  2;
        unsigned int DATA_CU2_WRITE_DATA_BUF_DED_COUNT                            :  2;
        unsigned int DATA_CU2_UTCL1_LFIFO_SEC_COUNT                               :  2;
        unsigned int DATA_CU2_UTCL1_LFIFO_DED_COUNT                               :  2;
        unsigned int DATA_CU3_WRITE_DATA_BUF_SEC_COUNT                            :  2;
        unsigned int DATA_CU3_WRITE_DATA_BUF_DED_COUNT                            :  2;
        unsigned int DATA_CU3_UTCL1_LFIFO_SEC_COUNT                               :  2;
        unsigned int DATA_CU3_UTCL1_LFIFO_DED_COUNT                               :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQC_EDC_CNT2 {
    struct {
        unsigned int INST_BANKA_TAG_RAM_SEC_COUNT                                 :  2;
        unsigned int INST_BANKA_TAG_RAM_DED_COUNT                                 :  2;
        unsigned int INST_BANKA_BANK_RAM_SEC_COUNT                                :  2;
        unsigned int INST_BANKA_BANK_RAM_DED_COUNT                                :  2;
        unsigned int DATA_BANKA_TAG_RAM_SEC_COUNT                                 :  2;
        unsigned int DATA_BANKA_TAG_RAM_DED_COUNT                                 :  2;
        unsigned int DATA_BANKA_BANK_RAM_SEC_COUNT                                :  2;
        unsigned int DATA_BANKA_BANK_RAM_DED_COUNT                                :  2;
        unsigned int INST_BANKA_UTCL1_MISS_FIFO_SED_COUNT                         :  2;
        unsigned int INST_BANKA_MISS_FIFO_SED_COUNT                               :  2;
        unsigned int DATA_BANKA_HIT_FIFO_SED_COUNT                                :  2;
        unsigned int DATA_BANKA_MISS_FIFO_SED_COUNT                               :  2;
        unsigned int DATA_BANKA_DIRTY_BIT_RAM_SED_COUNT                           :  2;
        unsigned int INST_UTCL1_LFIFO_SEC_COUNT                                   :  2;
        unsigned int INST_UTCL1_LFIFO_DED_COUNT                                   :  2;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQC_EDC_CNT3 {
    struct {
        unsigned int INST_BANKB_TAG_RAM_SEC_COUNT                                 :  2;
        unsigned int INST_BANKB_TAG_RAM_DED_COUNT                                 :  2;
        unsigned int INST_BANKB_BANK_RAM_SEC_COUNT                                :  2;
        unsigned int INST_BANKB_BANK_RAM_DED_COUNT                                :  2;
        unsigned int DATA_BANKB_TAG_RAM_SEC_COUNT                                 :  2;
        unsigned int DATA_BANKB_TAG_RAM_DED_COUNT                                 :  2;
        unsigned int DATA_BANKB_BANK_RAM_SEC_COUNT                                :  2;
        unsigned int DATA_BANKB_BANK_RAM_DED_COUNT                                :  2;
        unsigned int INST_BANKB_UTCL1_MISS_FIFO_SED_COUNT                         :  2;
        unsigned int INST_BANKB_MISS_FIFO_SED_COUNT                               :  2;
        unsigned int DATA_BANKB_HIT_FIFO_SED_COUNT                                :  2;
        unsigned int DATA_BANKB_MISS_FIFO_SED_COUNT                               :  2;
        unsigned int DATA_BANKB_DIRTY_BIT_RAM_SED_COUNT                           :  2;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQC_EDC_FUE_CNTL {
    struct {
        unsigned int BLOCK_FUE_FLAGS                                              : 16;
        unsigned int FUE_INTERRUPT_ENABLES                                        : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQC_ICACHE_UTCL0_CNTL1 {
    struct {
        unsigned int FORCE_4K_L2_RESP                                             :  1;
        unsigned int GPUVM_64K_DEF                                                :  1;
        unsigned int GPUVM_PERM_MODE                                              :  1;
        unsigned int RESP_MODE                                                    :  2;
        unsigned int RESP_FAULT_MODE                                              :  2;
        unsigned int CLIENTID                                                     :  9;
        unsigned int                                                              :  1;
        unsigned int ENABLE_PUSH_LFIFO                                            :  1;
        unsigned int ENABLE_LFIFO_PRI_ARB                                         :  1;
        unsigned int REG_INV_VMID                                                 :  4;
        unsigned int REG_INV_ALL_VMID                                             :  1;
        unsigned int REG_INV_TOGGLE                                               :  1;
        unsigned int CLIENT_INVALIDATE_ALL_VMID                                   :  1;
        unsigned int FORCE_MISS                                                   :  1;
        unsigned int FORCE_IN_ORDER                                               :  1;
        unsigned int REDUCE_FIFO_DEPTH_BY_2                                       :  2;
        unsigned int REDUCE_CACHE_SIZE_BY_2                                       :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQC_ICACHE_UTCL0_CNTL2 {
    struct {
        unsigned int SPARE                                                        :  8;
        unsigned int LFIFO_SCAN_DISABLE                                           :  1;
        unsigned int MTYPE_OVRD_DIS                                               :  1;
        unsigned int LINE_VALID                                                   :  1;
        unsigned int DIS_EDC                                                      :  1;
        unsigned int GPUVM_INV_MODE                                               :  1;
        unsigned int SHOOTDOWN_OPT                                                :  1;
        unsigned int FORCE_SNOOP                                                  :  1;
        unsigned int FORCE_GPUVM_INV_ACK                                          :  1;
        unsigned int ARB_BURST_MODE                                               :  2;
        unsigned int ENABLE_PERF_EVENT_RD_WR                                      :  1;
        unsigned int PERF_EVENT_RD_WR                                             :  1;
        unsigned int ENABLE_PERF_EVENT_VMID                                       :  1;
        unsigned int PERF_EVENT_VMID                                              :  4;
        unsigned int                                                              :  1;
        unsigned int FORCE_FRAG_2M_TO_64K                                         :  1;
        unsigned int PERM_MODE_OVRD                                               :  1;
        unsigned int LINE_INVALIDATE_OPT                                          :  1;
        unsigned int GPUVM_16K_DEF                                                :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQC_ICACHE_UTCL0_STATUS {
    struct {
        unsigned int FAULT_DETECTED                                               :  1;
        unsigned int RETRY_DETECTED                                               :  1;
        unsigned int PRT_DETECTED                                                 :  1;
        unsigned int                                                              : 29;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQC_ICACHE_UTCL1_CNTL1 {
    struct {
        unsigned int FORCE_4K_L2_RESP                                             :  1;
        unsigned int GPUVM_64K_DEF                                                :  1;
        unsigned int GPUVM_PERM_MODE                                              :  1;
        unsigned int RESP_MODE                                                    :  2;
        unsigned int RESP_FAULT_MODE                                              :  2;
        unsigned int CLIENTID                                                     :  9;
        unsigned int                                                              :  1;
        unsigned int ENABLE_PUSH_LFIFO                                            :  1;
        unsigned int ENABLE_LFIFO_PRI_ARB                                         :  1;
        unsigned int REG_INVALIDATE_VMID                                          :  4;
        unsigned int REG_INVALIDATE_ALL_VMID                                      :  1;
        unsigned int REG_INVALIDATE_TOGGLE                                        :  1;
        unsigned int CLIENT_INVALIDATE_ALL_VMID                                   :  1;
        unsigned int FORCE_MISS                                                   :  1;
        unsigned int FORCE_IN_ORDER                                               :  1;
        unsigned int REDUCE_FIFO_DEPTH_BY_2                                       :  2;
        unsigned int REDUCE_CACHE_SIZE_BY_2                                       :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQC_ICACHE_UTCL1_CNTL2 {
    struct {
        unsigned int SPARE                                                        :  8;
        unsigned int LFIFO_SCAN_DISABLE                                           :  1;
        unsigned int MTYPE_OVRD_DIS                                               :  1;
        unsigned int LINE_VALID                                                   :  1;
        unsigned int DIS_EDC                                                      :  1;
        unsigned int GPUVM_INV_MODE                                               :  1;
        unsigned int SHOOTDOWN_OPT                                                :  1;
        unsigned int FORCE_SNOOP                                                  :  1;
        unsigned int FORCE_GPUVM_INV_ACK                                          :  1;
        unsigned int ARB_BURST_MODE                                               :  2;
        unsigned int ENABLE_PERF_EVENT_RD_WR                                      :  1;
        unsigned int PERF_EVENT_RD_WR                                             :  1;
        unsigned int ENABLE_PERF_EVENT_VMID                                       :  1;
        unsigned int PERF_EVENT_VMID                                              :  4;
        unsigned int                                                              :  1;
        unsigned int FORCE_FRAG_2M_TO_64K                                         :  1;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQC_ICACHE_UTCL1_STATUS {
    struct {
        unsigned int FAULT_DETECTED                                               :  1;
        unsigned int RETRY_DETECTED                                               :  1;
        unsigned int PRT_DETECTED                                                 :  1;
        unsigned int                                                              : 29;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQC_MISC_CONFIG {
    struct {
        unsigned int PERFTOKEN_DELAY                                              :  5;
        unsigned int                                                              : 27;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  5;
        unsigned int SQC_SPI_TTRACE_FGCG_OVERRIDE                                 :  1;
        unsigned int                                                              : 26;
    } gfx101Plus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQC_WRITEBACK {
    struct {
        unsigned int DWB                                                          :  1;
        unsigned int DIRTY                                                        :  1;
        unsigned int                                                              : 30;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQG_CONFIG {
    struct {
        unsigned int UTCL0_PREFETCH_PAGE                                          :  4;
        unsigned int UTCL0_RETRY_TIMER                                            :  7;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQG_STATUS {
    struct {
        unsigned int REG_BUSY                                                     :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQG_UTCL0_CNTL1 {
    struct {
        unsigned int FORCE_4K_L2_RESP                                             :  1;
        unsigned int GPUVM_64K_DEF                                                :  1;
        unsigned int GPUVM_PERM_MODE                                              :  1;
        unsigned int RESP_MODE                                                    :  2;
        unsigned int RESP_FAULT_MODE                                              :  2;
        unsigned int CLIENTID                                                     :  9;
        unsigned int RESERVED                                                     :  1;
        unsigned int ENABLE_PUSH_LFIFO                                            :  1;
        unsigned int ENABLE_LFIFO_PRI_ARB                                         :  1;
        unsigned int REG_INV_VMID                                                 :  4;
        unsigned int REG_INV_ALL_VMID                                             :  1;
        unsigned int REG_INV_TOGGLE                                               :  1;
        unsigned int CLIENT_INVALIDATE_ALL_VMID                                   :  1;
        unsigned int FORCE_MISS                                                   :  1;
        unsigned int FORCE_IN_ORDER                                               :  1;
        unsigned int REDUCE_FIFO_DEPTH_BY_2                                       :  2;
        unsigned int REDUCE_CACHE_SIZE_BY_2                                       :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQG_UTCL0_CNTL2 {
    struct {
        unsigned int SPARE                                                        :  8;
        unsigned int LFIFO_SCAN_DISABLE                                           :  1;
        unsigned int MTYPE_OVRD_DIS                                               :  1;
        unsigned int LINE_VALID                                                   :  1;
        unsigned int DIS_EDC                                                      :  1;
        unsigned int GPUVM_INV_MODE                                               :  1;
        unsigned int SHOOTDOWN_OPT                                                :  1;
        unsigned int FORCE_SNOOP                                                  :  1;
        unsigned int FORCE_GPUVM_INV_ACK                                          :  1;
        unsigned int ARB_BURST_MODE                                               :  2;
        unsigned int ENABLE_PERF_EVENT_RD_WR                                      :  1;
        unsigned int PERF_EVENT_RD_WR                                             :  1;
        unsigned int ENABLE_PERF_EVENT_VMID                                       :  1;
        unsigned int PERF_EVENT_VMID                                              :  4;
        unsigned int DIS_DUAL_L2_REQ                                              :  1;
        unsigned int FORCE_FRAG_2M_TO_64K                                         :  1;
        unsigned int PERM_MODE_OVRD                                               :  1;
        unsigned int LINE_INVALIDATE_OPT                                          :  1;
        unsigned int GPUVM_16K_DEF                                                :  1;
        unsigned int RESERVED                                                     :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQG_UTCL0_STATUS {
    struct {
        unsigned int FAULT_DETECTED                                               :  1;
        unsigned int RETRY_DETECTED                                               :  1;
        unsigned int PRT_DETECTED                                                 :  1;
        unsigned int RESERVED                                                     :  5;
        unsigned int UNUSED                                                       : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_ALU_CLK_CTRL {
    struct {
        unsigned int FORCE_CU_ON_SH0                                              : 16;
        unsigned int FORCE_CU_ON_SH1                                              : 16;
    } gfx09;
    struct {
        unsigned int FORCE_WGP_ON_SA0                                             : 16;
        unsigned int FORCE_WGP_ON_SA1                                             : 16;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_ARB_CONFIG {
    struct {
        unsigned int WG_RR_INTERVAL                                               :  2;
        unsigned int                                                              :  2;
        unsigned int FWD_PROG_INTERVAL                                            :  2;
        unsigned int                                                              : 26;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  8;
        unsigned int DISABLE_SECOND_TRY                                           :  1;
        unsigned int                                                              : 23;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_BUF_RSRC_WORD0 {
    struct {
        unsigned int BASE_ADDRESS                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_BUF_RSRC_WORD1 {
    struct {
        unsigned int BASE_ADDRESS_HI                                              : 16;
        unsigned int STRIDE                                                       : 14;
        unsigned int CACHE_SWIZZLE                                                :  1;
        unsigned int SWIZZLE_ENABLE                                               :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_BUF_RSRC_WORD2 {
    struct {
        unsigned int NUM_RECORDS                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_BUF_RSRC_WORD3 {
    struct {
        unsigned int DST_SEL_X                                                    :  3;
        unsigned int DST_SEL_Y                                                    :  3;
        unsigned int DST_SEL_Z                                                    :  3;
        unsigned int DST_SEL_W                                                    :  3;
        unsigned int NUM_FORMAT                                                   :  3;
        unsigned int DATA_FORMAT                                                  :  4;
        unsigned int USER_VM_ENABLE                                               :  1;
        unsigned int USER_VM_MODE                                                 :  1;
        unsigned int INDEX_STRIDE                                                 :  2;
        unsigned int ADD_TID_ENABLE                                               :  1;
        unsigned int                                                              :  3;
        unsigned int NV                                                           :  1;
        unsigned int                                                              :  2;
        unsigned int TYPE                                                         :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_CMD {
    struct {
        unsigned int                                                              :  4;
        unsigned int MODE                                                         :  3;
        unsigned int CHECK_VMID                                                   :  1;
        unsigned int DATA                                                         :  4;
        unsigned int                                                              : 12;
        unsigned int QUEUE_ID                                                     :  3;
        unsigned int                                                              :  1;
        unsigned int VM_ID                                                        :  4;
    } bits, bitfields;
    struct {
        unsigned int CMD                                                          :  3;
        unsigned int                                                              : 13;
        unsigned int WAVE_ID                                                      :  4;
        unsigned int SIMD_ID                                                      :  2;
        unsigned int                                                              : 10;
    } gfx09;
    struct {
        unsigned int CMD                                                          :  4;
        unsigned int                                                              : 12;
        unsigned int WAVE_ID                                                      :  5;
        unsigned int                                                              : 11;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_CMD_TIMESTAMP {
    struct {
        unsigned int TIMESTAMP                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_CONFIG {
    struct {
        unsigned int                                                              : 18;
        unsigned int ENABLE_HIPRIO_ON_EXP_RDY_VS                                  :  1;
        unsigned int PRIO_VAL_ON_EXP_RDY_VS                                       :  2;
        unsigned int                                                              :  8;
        unsigned int DISABLE_SP_REDUNDANT_THREAD_GATING                           :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  7;
        unsigned int OVERRIDE_ALU_BUSY                                            :  1;
        unsigned int DEBUG_EN                                                     :  1;
        unsigned int DEBUG_SINGLE_MEMOP                                           :  1;
        unsigned int DEBUG_ONE_INST_CLAUSE                                        :  1;
        unsigned int OVERRIDE_LDS_IDX_BUSY                                        :  1;
        unsigned int EARLY_TA_DONE_DISABLE                                        :  1;
        unsigned int DUA_FLAT_LOCK_ENABLE                                         :  1;
        unsigned int DUA_LDS_BYPASS_DISABLE                                       :  1;
        unsigned int DUA_FLAT_LDS_PINGPONG_DISABLE                                :  1;
        unsigned int DISABLE_VMEM_SOFT_CLAUSE                                     :  1;
        unsigned int DISABLE_SMEM_SOFT_CLAUSE                                     :  1;
        unsigned int                                                              :  3;
        unsigned int REPLAY_SLEEP_CNT                                             :  7;
        unsigned int DISABLE_SP_VGPR_WRITE_SKIP                                   :  1;
        unsigned int                                                              :  1;
        unsigned int DISABLE_FLAT_SOFT_CLAUSE                                     :  1;
        unsigned int DISABLE_MIMG_SOFT_CLAUSE                                     :  1;
    } gfx09;
    struct {
        unsigned int                                                              : 12;
        unsigned int VGPR_SWIZZLE_EN                                              :  1;
        unsigned int                                                              : 19;
    } gfx10;
    struct {
        unsigned int UNUSED                                                       :  7;
        unsigned int                                                              :  2;
        unsigned int DEBUG_SINGLE_MEMOP                                           :  1;
        unsigned int DEBUG_ONE_INST_CLAUSE                                        :  1;
        unsigned int                                                              : 21;
    } gfx101;
    struct {
        unsigned int                                                              : 13;
        unsigned int LDS_BUSY_HYSTERESIS_CNT                                      :  2;
        unsigned int SP_BUSY_HYSTERESIS_CNT                                       :  2;
        unsigned int                                                              : 13;
        unsigned int TA_BUSY_HYSTERESIS_CNT                                       :  2;
    } gfx101Plus;
    struct {
        unsigned int                                                              : 11;
        unsigned int OVERRIDE_LDS_IDX_BUSY                                        :  1;
        unsigned int                                                              :  9;
        unsigned int REPLAY_SLEEP_CNT                                             :  7;
        unsigned int                                                              :  4;
    } nv10;
    struct {
        unsigned int UNUSED                                                       :  7;
        unsigned int                                                              : 25;
    } rv2x;
    struct {
        unsigned int UNUSED                                                       :  7;
        unsigned int                                                              : 25;
    } vg10_Vg12_Rv1x;
    struct {
        unsigned int DISABLE_BARRIER_WAITCNT                                      :  1;
        unsigned int UNUSED                                                       :  6;
        unsigned int                                                              : 25;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_DEBUG_CTRL_LOCAL {
    struct {
        unsigned int UNUSED                                                       :  8;
        unsigned int                                                              : 24;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_DEBUG_STS_GLOBAL {
    struct {
        unsigned int BUSY                                                         :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  1;
        unsigned int INTERRUPT_BUSY                                               :  1;
        unsigned int                                                              :  2;
        unsigned int WAVE_LEVEL_SA0                                               : 12;
        unsigned int WAVE_LEVEL_SA1                                               : 12;
        unsigned int                                                              :  4;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_DEBUG_STS_LOCAL {
    struct {
        unsigned int BUSY                                                         :  1;
        unsigned int                                                              :  3;
        unsigned int WAVE_LEVEL                                                   :  6;
        unsigned int                                                              : 22;
    } most;
    struct {
        unsigned int                                                              : 12;
        unsigned int SQ_BUSY                                                      :  1;
        unsigned int IS_BUSY                                                      :  1;
        unsigned int                                                              :  1;
        unsigned int ARB_BUSY                                                     :  1;
        unsigned int EXP_BUSY                                                     :  1;
        unsigned int BRMSG_BUSY                                                   :  1;
        unsigned int VM_BUSY                                                      :  1;
        unsigned int                                                              : 13;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_DSM_CNTL {
    struct {
        unsigned int WAVEFRONT_STALL_0                                            :  1;
        unsigned int WAVEFRONT_STALL_1                                            :  1;
        unsigned int SPI_BACKPRESSURE_0                                           :  1;
        unsigned int SPI_BACKPRESSURE_1                                           :  1;
        unsigned int                                                              :  4;
        unsigned int SEL_DSM_SGPR_IRRITATOR_DATA0                                 :  1;
        unsigned int SEL_DSM_SGPR_IRRITATOR_DATA1                                 :  1;
        unsigned int SGPR_ENABLE_SINGLE_WRITE                                     :  1;
        unsigned int                                                              :  5;
        unsigned int SEL_DSM_LDS_IRRITATOR_DATA0                                  :  1;
        unsigned int SEL_DSM_LDS_IRRITATOR_DATA1                                  :  1;
        unsigned int LDS_ENABLE_SINGLE_WRITE01                                    :  1;
        unsigned int SEL_DSM_LDS_IRRITATOR_DATA2                                  :  1;
        unsigned int SEL_DSM_LDS_IRRITATOR_DATA3                                  :  1;
        unsigned int LDS_ENABLE_SINGLE_WRITE23                                    :  1;
        unsigned int                                                              :  2;
        unsigned int SEL_DSM_SP_IRRITATOR_DATA0                                   :  1;
        unsigned int SEL_DSM_SP_IRRITATOR_DATA1                                   :  1;
        unsigned int SP_ENABLE_SINGLE_WRITE                                       :  1;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_DSM_CNTL2 {
    struct {
        unsigned int SGPR_ENABLE_ERROR_INJECT                                     :  2;
        unsigned int SGPR_SELECT_INJECT_DELAY                                     :  1;
        unsigned int LDS_D_ENABLE_ERROR_INJECT                                    :  2;
        unsigned int LDS_D_SELECT_INJECT_DELAY                                    :  1;
        unsigned int LDS_I_ENABLE_ERROR_INJECT                                    :  2;
        unsigned int LDS_I_SELECT_INJECT_DELAY                                    :  1;
        unsigned int SP_ENABLE_ERROR_INJECT                                       :  2;
        unsigned int SP_SELECT_INJECT_DELAY                                       :  1;
        unsigned int                                                              :  2;
        unsigned int LDS_INJECT_DELAY                                             :  6;
        unsigned int SP_INJECT_DELAY                                              :  6;
        unsigned int SQ_INJECT_DELAY                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_DS_0 {
    struct {
        unsigned int OFFSET0                                                      :  8;
        unsigned int OFFSET1                                                      :  8;
        unsigned int GDS                                                          :  1;
        unsigned int OP                                                           :  8;
        unsigned int                                                              :  1;
        unsigned int ENCODING                                                     :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_DS_1 {
    struct {
        unsigned int ADDR                                                         :  8;
        unsigned int DATA0                                                        :  8;
        unsigned int DATA1                                                        :  8;
        unsigned int VDST                                                         :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_EDC_CNT {
    struct {
        unsigned int LDS_D_SEC_COUNT                                              :  2;
        unsigned int LDS_D_DED_COUNT                                              :  2;
        unsigned int LDS_I_SEC_COUNT                                              :  2;
        unsigned int LDS_I_DED_COUNT                                              :  2;
        unsigned int SGPR_SEC_COUNT                                               :  2;
        unsigned int SGPR_DED_COUNT                                               :  2;
        unsigned int VGPR0_SEC_COUNT                                              :  2;
        unsigned int VGPR0_DED_COUNT                                              :  2;
        unsigned int VGPR1_SEC_COUNT                                              :  2;
        unsigned int VGPR1_DED_COUNT                                              :  2;
        unsigned int VGPR2_SEC_COUNT                                              :  2;
        unsigned int VGPR2_DED_COUNT                                              :  2;
        unsigned int VGPR3_SEC_COUNT                                              :  2;
        unsigned int VGPR3_DED_COUNT                                              :  2;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_EDC_DED_CNT {
    struct {
        unsigned int LDS_DED                                                      :  8;
        unsigned int SGPR_DED                                                     :  8;
        unsigned int VGPR_DED                                                     :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_EDC_FUE_CNTL {
    struct {
        unsigned int BLOCK_FUE_FLAGS                                              : 16;
        unsigned int FUE_INTERRUPT_ENABLES                                        : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_EDC_INFO {
    struct {
        unsigned int WAVE_ID                                                      :  4;
        unsigned int SIMD_ID                                                      :  2;
        unsigned int SOURCE                                                       :  3;
        unsigned int VM_ID                                                        :  4;
        unsigned int                                                              : 19;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_EDC_SEC_CNT {
    struct {
        unsigned int LDS_SEC                                                      :  8;
        unsigned int SGPR_SEC                                                     :  8;
        unsigned int VGPR_SEC                                                     :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_EXP_0 {
    struct {
        unsigned int EN                                                           :  4;
        unsigned int TGT                                                          :  6;
        unsigned int COMPR                                                        :  1;
        unsigned int DONE                                                         :  1;
        unsigned int VM                                                           :  1;
        unsigned int                                                              : 13;
        unsigned int ENCODING                                                     :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_EXP_1 {
    struct {
        unsigned int VSRC0                                                        :  8;
        unsigned int VSRC1                                                        :  8;
        unsigned int VSRC2                                                        :  8;
        unsigned int VSRC3                                                        :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_FIFO_SIZES {
    struct {
        unsigned int INTERRUPT_FIFO_SIZE                                          :  4;
        unsigned int                                                              : 14;
        unsigned int VMEM_DATA_FIFO_SIZE                                          :  2;
        unsigned int                                                              : 12;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  8;
        unsigned int TTRACE_FIFO_SIZE                                             :  4;
        unsigned int                                                              :  4;
        unsigned int EXPORT_BUF_SIZE                                              :  2;
        unsigned int                                                              : 14;
    } gfx09;
    struct {
        unsigned int                                                              :  8;
        unsigned int TTRACE_FIFO_SIZE                                             :  2;
        unsigned int                                                              :  2;
        unsigned int EXPORT_BUF_VS_RESERVED                                       :  2;
        unsigned int EXPORT_BUF_PS_RESERVED                                       :  2;
        unsigned int EXPORT_BUF_REDUCE                                            :  2;
        unsigned int                                                              : 14;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_FLAT_0 {
    struct {
        unsigned int OFFSET                                                       : 12;
        unsigned int                                                              :  1;
        unsigned int LDS                                                          :  1;
        unsigned int SEG                                                          :  2;
        unsigned int GLC                                                          :  1;
        unsigned int SLC                                                          :  1;
        unsigned int OP                                                           :  7;
        unsigned int                                                              :  1;
        unsigned int ENCODING                                                     :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_FLAT_1 {
    struct {
        unsigned int ADDR                                                         :  8;
        unsigned int DATA                                                         :  8;
        unsigned int SADDR                                                        :  7;
        unsigned int NV                                                           :  1;
        unsigned int VDST                                                         :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_FLAT_SCRATCH_WORD0 {
    struct {
        unsigned int SIZE                                                         : 19;
        unsigned int                                                              : 13;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_FLAT_SCRATCH_WORD1 {
    struct {
        unsigned int OFFSET                                                       : 24;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_GLBL_0 {
    struct {
        unsigned int OFFSET                                                       : 13;
        unsigned int LDS                                                          :  1;
        unsigned int SEG                                                          :  2;
        unsigned int GLC                                                          :  1;
        unsigned int SLC                                                          :  1;
        unsigned int OP                                                           :  7;
        unsigned int                                                              :  1;
        unsigned int ENCODING                                                     :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_GLBL_1 {
    struct {
        unsigned int ADDR                                                         :  8;
        unsigned int DATA                                                         :  8;
        unsigned int SADDR                                                        :  7;
        unsigned int NV                                                           :  1;
        unsigned int VDST                                                         :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_IMG_RSRC_WORD0 {
    struct {
        unsigned int BASE_ADDRESS                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_IMG_RSRC_WORD1 {
    struct {
        unsigned int BASE_ADDRESS_HI                                              :  8;
        unsigned int MIN_LOD                                                      : 12;
        unsigned int DATA_FORMAT                                                  :  6;
        unsigned int NUM_FORMAT                                                   :  4;
        unsigned int NV                                                           :  1;
        unsigned int META_DIRECT                                                  :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_IMG_RSRC_WORD2 {
    struct {
        unsigned int WIDTH                                                        : 14;
        unsigned int HEIGHT                                                       : 14;
        unsigned int PERF_MOD                                                     :  3;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_IMG_RSRC_WORD3 {
    struct {
        unsigned int DST_SEL_X                                                    :  3;
        unsigned int DST_SEL_Y                                                    :  3;
        unsigned int DST_SEL_Z                                                    :  3;
        unsigned int DST_SEL_W                                                    :  3;
        unsigned int BASE_LEVEL                                                   :  4;
        unsigned int LAST_LEVEL                                                   :  4;
        unsigned int SW_MODE                                                      :  5;
        unsigned int                                                              :  3;
        unsigned int TYPE                                                         :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_IMG_RSRC_WORD4 {
    struct {
        unsigned int DEPTH                                                        : 13;
        unsigned int PITCH                                                        : 16;
        unsigned int BC_SWIZZLE                                                   :  3;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_IMG_RSRC_WORD5 {
    struct {
        unsigned int BASE_ARRAY                                                   : 13;
        unsigned int ARRAY_PITCH                                                  :  4;
        unsigned int META_DATA_ADDRESS                                            :  8;
        unsigned int META_LINEAR                                                  :  1;
        unsigned int META_PIPE_ALIGNED                                            :  1;
        unsigned int META_RB_ALIGNED                                              :  1;
        unsigned int MAX_MIP                                                      :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_IMG_RSRC_WORD6 {
    struct {
        unsigned int MIN_LOD_WARN                                                 : 12;
        unsigned int COUNTER_BANK_ID                                              :  8;
        unsigned int LOD_HDW_CNT_EN                                               :  1;
        unsigned int COMPRESSION_EN                                               :  1;
        unsigned int ALPHA_IS_ON_MSB                                              :  1;
        unsigned int COLOR_TRANSFORM                                              :  1;
        unsigned int LOST_ALPHA_BITS                                              :  4;
        unsigned int LOST_COLOR_BITS                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_IMG_RSRC_WORD7 {
    struct {
        unsigned int META_DATA_ADDRESS                                            : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_IMG_SAMP_WORD0 {
    struct {
        unsigned int CLAMP_X                                                      :  3;
        unsigned int CLAMP_Y                                                      :  3;
        unsigned int CLAMP_Z                                                      :  3;
        unsigned int MAX_ANISO_RATIO                                              :  3;
        unsigned int DEPTH_COMPARE_FUNC                                           :  3;
        unsigned int FORCE_UNNORMALIZED                                           :  1;
        unsigned int ANISO_THRESHOLD                                              :  3;
        unsigned int MC_COORD_TRUNC                                               :  1;
        unsigned int FORCE_DEGAMMA                                                :  1;
        unsigned int ANISO_BIAS                                                   :  6;
        unsigned int TRUNC_COORD                                                  :  1;
        unsigned int DISABLE_CUBE_WRAP                                            :  1;
        unsigned int FILTER_MODE                                                  :  2;
        unsigned int COMPAT_MODE                                                  :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_IMG_SAMP_WORD1 {
    struct {
        unsigned int MIN_LOD                                                      : 12;
        unsigned int MAX_LOD                                                      : 12;
        unsigned int PERF_MIP                                                     :  4;
        unsigned int PERF_Z                                                       :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_IMG_SAMP_WORD2 {
    struct {
        unsigned int LOD_BIAS                                                     : 14;
        unsigned int LOD_BIAS_SEC                                                 :  6;
        unsigned int XY_MAG_FILTER                                                :  2;
        unsigned int XY_MIN_FILTER                                                :  2;
        unsigned int Z_FILTER                                                     :  2;
        unsigned int MIP_FILTER                                                   :  2;
        unsigned int MIP_POINT_PRECLAMP                                           :  1;
        unsigned int BLEND_ZERO_PRT                                               :  1;
        unsigned int FILTER_PREC_FIX                                              :  1;
        unsigned int ANISO_OVERRIDE                                               :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_IMG_SAMP_WORD3 {
    struct {
        unsigned int BORDER_COLOR_PTR                                             : 12;
        unsigned int SKIP_DEGAMMA                                                 :  1;
        unsigned int                                                              : 17;
        unsigned int BORDER_COLOR_TYPE                                            :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_IND_DATA {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_IND_INDEX {
    struct {
        unsigned int                                                              : 16;
        unsigned int INDEX                                                        : 16;
    } bits, bitfields;
    struct {
        unsigned int WAVE_ID                                                      :  4;
        unsigned int SIMD_ID                                                      :  2;
        unsigned int THREAD_ID                                                    :  6;
        unsigned int AUTO_INCR                                                    :  1;
        unsigned int FORCE_READ                                                   :  1;
        unsigned int READ_TIMEOUT                                                 :  1;
        unsigned int UNINDEXED                                                    :  1;
        unsigned int                                                              : 16;
    } gfx09;
    struct {
        unsigned int WAVE_ID                                                      :  5;
        unsigned int WORKITEM_ID                                                  :  6;
        unsigned int AUTO_INCR                                                    :  1;
        unsigned int                                                              : 20;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_INST {
    struct {
        unsigned int ENCODING                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_INTERRUPT_AUTO_MASK {
    struct {
        unsigned int MASK                                                         : 24;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_INTERRUPT_MSG_CTRL {
    struct {
        unsigned int STALL                                                        :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_INTERRUPT_WORD_AUTO_CTXID {
    struct {
        unsigned int THREAD_TRACE                                                 :  1;
        unsigned int WLT                                                          :  1;
        unsigned int THREAD_TRACE_BUF_FULL                                        :  1;
        unsigned int REG_TIMESTAMP                                                :  1;
        unsigned int CMD_TIMESTAMP                                                :  1;
        unsigned int HOST_CMD_OVERFLOW                                            :  1;
        unsigned int HOST_REG_OVERFLOW                                            :  1;
        unsigned int IMMED_OVERFLOW                                               :  1;
        unsigned int THREAD_TRACE_UTC_ERROR                                       :  1;
        unsigned int                                                              : 15;
        unsigned int SE_ID                                                        :  2;
        unsigned int ENCODING                                                     :  2;
        unsigned int                                                              :  4;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_INTERRUPT_WORD_AUTO_HI {
    struct {
        unsigned int                                                              :  8;
        unsigned int SE_ID                                                        :  2;
        unsigned int ENCODING                                                     :  2;
        unsigned int                                                              : 20;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_INTERRUPT_WORD_AUTO_LO {
    struct {
        unsigned int THREAD_TRACE                                                 :  1;
        unsigned int WLT                                                          :  1;
        unsigned int THREAD_TRACE_BUF_FULL                                        :  1;
        unsigned int REG_TIMESTAMP                                                :  1;
        unsigned int CMD_TIMESTAMP                                                :  1;
        unsigned int HOST_CMD_OVERFLOW                                            :  1;
        unsigned int HOST_REG_OVERFLOW                                            :  1;
        unsigned int IMMED_OVERFLOW                                               :  1;
        unsigned int THREAD_TRACE_UTC_ERROR                                       :  1;
        unsigned int                                                              : 23;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_INTERRUPT_WORD_CMN_CTXID {
    struct {
        unsigned int                                                              : 24;
        unsigned int SE_ID                                                        :  2;
        unsigned int ENCODING                                                     :  2;
        unsigned int                                                              :  4;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_INTERRUPT_WORD_CMN_HI {
    struct {
        unsigned int                                                              :  8;
        unsigned int SE_ID                                                        :  2;
        unsigned int ENCODING                                                     :  2;
        unsigned int                                                              : 20;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_INTERRUPT_WORD_WAVE_CTXID {
    struct {
        unsigned int DATA                                                         : 12;
        unsigned int SH_ID                                                        :  1;
        unsigned int PRIV                                                         :  1;
        unsigned int WAVE_ID                                                      :  4;
        unsigned int SIMD_ID                                                      :  2;
        unsigned int CU_ID                                                        :  4;
        unsigned int SE_ID                                                        :  2;
        unsigned int ENCODING                                                     :  2;
        unsigned int                                                              :  4;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_INTERRUPT_WORD_WAVE_HI {
    struct {
        unsigned int CU_ID                                                        :  4;
        unsigned int VM_ID                                                        :  4;
        unsigned int SE_ID                                                        :  2;
        unsigned int ENCODING                                                     :  2;
        unsigned int                                                              : 20;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_INTERRUPT_WORD_WAVE_LO {
    struct {
        unsigned int DATA                                                         : 24;
        unsigned int SH_ID                                                        :  1;
        unsigned int PRIV                                                         :  1;
        unsigned int WAVE_ID                                                      :  4;
        unsigned int SIMD_ID                                                      :  2;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_LB_CTR0_CU {
    struct {
        unsigned int SH0_MASK                                                     : 16;
        unsigned int SH1_MASK                                                     : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_LB_CTR1_CU {
    struct {
        unsigned int SH0_MASK                                                     : 16;
        unsigned int SH1_MASK                                                     : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_LB_CTR2_CU {
    struct {
        unsigned int SH0_MASK                                                     : 16;
        unsigned int SH1_MASK                                                     : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_LB_CTR3_CU {
    struct {
        unsigned int SH0_MASK                                                     : 16;
        unsigned int SH1_MASK                                                     : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_LB_CTR_CTRL {
    struct {
        unsigned int START                                                        :  1;
        unsigned int LOAD                                                         :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              : 29;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_LB_CTR_SEL {
    struct {
        unsigned int SEL0                                                         :  4;
        unsigned int SEL1                                                         :  4;
        unsigned int SEL2                                                         :  4;
        unsigned int SEL3                                                         :  4;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_LB_CTR_SEL0 {
    struct {
        unsigned int SEL0                                                         :  8;
        unsigned int                                                              :  7;
        unsigned int DIV0                                                         :  1;
        unsigned int SEL1                                                         :  8;
        unsigned int                                                              :  7;
        unsigned int DIV1                                                         :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_LB_CTR_SEL1 {
    struct {
        unsigned int SEL2                                                         :  8;
        unsigned int                                                              :  7;
        unsigned int DIV2                                                         :  1;
        unsigned int SEL3                                                         :  8;
        unsigned int                                                              :  7;
        unsigned int DIV3                                                         :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_LB_DATA0 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_LB_DATA1 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_LB_DATA2 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_LB_DATA3 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_LDS_CLK_CTRL {
    struct {
        unsigned int FORCE_CU_ON_SH0                                              : 16;
        unsigned int FORCE_CU_ON_SH1                                              : 16;
    } gfx09;
    struct {
        unsigned int FORCE_WGP_ON_SA0                                             : 16;
        unsigned int FORCE_WGP_ON_SA1                                             : 16;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_M0_GPR_IDX_WORD {
    struct {
        unsigned int INDEX                                                        :  8;
        unsigned int                                                              :  4;
        unsigned int VSRC0_REL                                                    :  1;
        unsigned int VSRC1_REL                                                    :  1;
        unsigned int VSRC2_REL                                                    :  1;
        unsigned int VDST_REL                                                     :  1;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_MIMG_0 {
    struct {
        unsigned int OPM                                                          :  1;
        unsigned int                                                              :  7;
        unsigned int DMASK                                                        :  4;
        unsigned int UNORM                                                        :  1;
        unsigned int GLC                                                          :  1;
        unsigned int DA                                                           :  1;
        unsigned int A16                                                          :  1;
        unsigned int TFE                                                          :  1;
        unsigned int LWE                                                          :  1;
        unsigned int OP                                                           :  7;
        unsigned int SLC                                                          :  1;
        unsigned int ENCODING                                                     :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_MIMG_1 {
    struct {
        unsigned int VADDR                                                        :  8;
        unsigned int VDATA                                                        :  8;
        unsigned int SRSRC                                                        :  5;
        unsigned int SSAMP                                                        :  5;
        unsigned int                                                              :  5;
        unsigned int D16                                                          :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_MTBUF_0 {
    struct {
        unsigned int OFFSET                                                       : 12;
        unsigned int OFFEN                                                        :  1;
        unsigned int IDXEN                                                        :  1;
        unsigned int GLC                                                          :  1;
        unsigned int OP                                                           :  4;
        unsigned int DFMT                                                         :  4;
        unsigned int NFMT                                                         :  3;
        unsigned int ENCODING                                                     :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_MTBUF_1 {
    struct {
        unsigned int VADDR                                                        :  8;
        unsigned int VDATA                                                        :  8;
        unsigned int SRSRC                                                        :  5;
        unsigned int                                                              :  1;
        unsigned int SLC                                                          :  1;
        unsigned int TFE                                                          :  1;
        unsigned int SOFFSET                                                      :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_MUBUF_0 {
    struct {
        unsigned int OFFSET                                                       : 12;
        unsigned int OFFEN                                                        :  1;
        unsigned int IDXEN                                                        :  1;
        unsigned int GLC                                                          :  1;
        unsigned int                                                              :  1;
        unsigned int LDS                                                          :  1;
        unsigned int SLC                                                          :  1;
        unsigned int OP                                                           :  7;
        unsigned int                                                              :  1;
        unsigned int ENCODING                                                     :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_MUBUF_1 {
    struct {
        unsigned int VADDR                                                        :  8;
        unsigned int VDATA                                                        :  8;
        unsigned int SRSRC                                                        :  5;
        unsigned int                                                              :  2;
        unsigned int TFE                                                          :  1;
        unsigned int SOFFSET                                                      :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 12;
        unsigned int SQC_BANK_MASK                                                :  4;
        unsigned int                                                              : 16;
    } most;
    struct {
        unsigned int                                                              : 16;
        unsigned int SQC_CLIENT_MASK                                              :  4;
        unsigned int                                                              :  4;
        unsigned int SIMD_MASK                                                    :  4;
        unsigned int                                                              :  4;
    } gfx09;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 12;
        unsigned int SQC_BANK_MASK                                                :  4;
        unsigned int                                                              : 16;
    } most;
    struct {
        unsigned int                                                              : 16;
        unsigned int SQC_CLIENT_MASK                                              :  4;
        unsigned int                                                              :  4;
        unsigned int SIMD_MASK                                                    :  4;
        unsigned int                                                              :  4;
    } gfx09;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 12;
        unsigned int SQC_BANK_MASK                                                :  4;
        unsigned int                                                              : 16;
    } most;
    struct {
        unsigned int                                                              : 16;
        unsigned int SQC_CLIENT_MASK                                              :  4;
        unsigned int                                                              :  4;
        unsigned int SIMD_MASK                                                    :  4;
        unsigned int                                                              :  4;
    } gfx09;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 12;
        unsigned int SQC_BANK_MASK                                                :  4;
        unsigned int                                                              : 16;
    } most;
    struct {
        unsigned int                                                              : 16;
        unsigned int SQC_CLIENT_MASK                                              :  4;
        unsigned int                                                              :  4;
        unsigned int SIMD_MASK                                                    :  4;
        unsigned int                                                              :  4;
    } gfx09;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER4_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER4_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER4_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 12;
        unsigned int SQC_BANK_MASK                                                :  4;
        unsigned int                                                              : 16;
    } most;
    struct {
        unsigned int                                                              : 16;
        unsigned int SQC_CLIENT_MASK                                              :  4;
        unsigned int                                                              :  4;
        unsigned int SIMD_MASK                                                    :  4;
        unsigned int                                                              :  4;
    } gfx09;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER5_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER5_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER5_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 12;
        unsigned int SQC_BANK_MASK                                                :  4;
        unsigned int                                                              : 16;
    } most;
    struct {
        unsigned int                                                              : 16;
        unsigned int SQC_CLIENT_MASK                                              :  4;
        unsigned int                                                              :  4;
        unsigned int SIMD_MASK                                                    :  4;
        unsigned int                                                              :  4;
    } gfx09;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER6_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER6_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER6_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 12;
        unsigned int SQC_BANK_MASK                                                :  4;
        unsigned int                                                              : 16;
    } most;
    struct {
        unsigned int                                                              : 16;
        unsigned int SQC_CLIENT_MASK                                              :  4;
        unsigned int                                                              :  4;
        unsigned int SIMD_MASK                                                    :  4;
        unsigned int                                                              :  4;
    } gfx09;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER7_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER7_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER7_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 12;
        unsigned int SQC_BANK_MASK                                                :  4;
        unsigned int                                                              : 16;
    } most;
    struct {
        unsigned int                                                              : 16;
        unsigned int SQC_CLIENT_MASK                                              :  4;
        unsigned int                                                              :  4;
        unsigned int SIMD_MASK                                                    :  4;
        unsigned int                                                              :  4;
    } gfx09;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER8_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER8_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER8_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 12;
        unsigned int SQC_BANK_MASK                                                :  4;
        unsigned int                                                              : 16;
    } most;
    struct {
        unsigned int                                                              : 16;
        unsigned int SQC_CLIENT_MASK                                              :  4;
        unsigned int                                                              :  4;
        unsigned int SIMD_MASK                                                    :  4;
        unsigned int                                                              :  4;
    } gfx09;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER9_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER9_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER9_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 12;
        unsigned int SQC_BANK_MASK                                                :  4;
        unsigned int                                                              : 16;
    } most;
    struct {
        unsigned int                                                              : 16;
        unsigned int SQC_CLIENT_MASK                                              :  4;
        unsigned int                                                              :  4;
        unsigned int SIMD_MASK                                                    :  4;
        unsigned int                                                              :  4;
    } gfx09;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER10_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER10_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER10_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 12;
        unsigned int SQC_BANK_MASK                                                :  4;
        unsigned int                                                              : 16;
    } most;
    struct {
        unsigned int                                                              : 16;
        unsigned int SQC_CLIENT_MASK                                              :  4;
        unsigned int                                                              :  4;
        unsigned int SIMD_MASK                                                    :  4;
        unsigned int                                                              :  4;
    } gfx09;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER11_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER11_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER11_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 12;
        unsigned int SQC_BANK_MASK                                                :  4;
        unsigned int                                                              : 16;
    } most;
    struct {
        unsigned int                                                              : 16;
        unsigned int SQC_CLIENT_MASK                                              :  4;
        unsigned int                                                              :  4;
        unsigned int SIMD_MASK                                                    :  4;
        unsigned int                                                              :  4;
    } gfx09;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER12_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER12_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER12_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 12;
        unsigned int SQC_BANK_MASK                                                :  4;
        unsigned int                                                              : 16;
    } most;
    struct {
        unsigned int                                                              : 16;
        unsigned int SQC_CLIENT_MASK                                              :  4;
        unsigned int                                                              :  4;
        unsigned int SIMD_MASK                                                    :  4;
        unsigned int                                                              :  4;
    } gfx09;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER13_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER13_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER13_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 12;
        unsigned int SQC_BANK_MASK                                                :  4;
        unsigned int                                                              : 16;
    } most;
    struct {
        unsigned int                                                              : 16;
        unsigned int SQC_CLIENT_MASK                                              :  4;
        unsigned int                                                              :  4;
        unsigned int SIMD_MASK                                                    :  4;
        unsigned int                                                              :  4;
    } gfx09;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER14_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER14_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER14_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 12;
        unsigned int SQC_BANK_MASK                                                :  4;
        unsigned int                                                              : 16;
    } most;
    struct {
        unsigned int                                                              : 16;
        unsigned int SQC_CLIENT_MASK                                              :  4;
        unsigned int                                                              :  4;
        unsigned int SIMD_MASK                                                    :  4;
        unsigned int                                                              :  4;
    } gfx09;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER15_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER15_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER15_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 12;
        unsigned int SQC_BANK_MASK                                                :  4;
        unsigned int                                                              : 16;
    } most;
    struct {
        unsigned int                                                              : 16;
        unsigned int SQC_CLIENT_MASK                                              :  4;
        unsigned int                                                              :  4;
        unsigned int SIMD_MASK                                                    :  4;
        unsigned int                                                              :  4;
    } gfx09;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER_CTRL {
    struct {
        unsigned int PS_EN                                                        :  1;
        unsigned int VS_EN                                                        :  1;
        unsigned int GS_EN                                                        :  1;
        unsigned int ES_EN                                                        :  1;
        unsigned int HS_EN                                                        :  1;
        unsigned int LS_EN                                                        :  1;
        unsigned int CS_EN                                                        :  1;
        unsigned int                                                              :  6;
        unsigned int DISABLE_FLUSH                                                :  1;
        unsigned int                                                              : 18;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  8;
        unsigned int CNTR_RATE                                                    :  5;
        unsigned int                                                              : 19;
    } gfx09;
    struct {
        unsigned int                                                              :  8;
        unsigned int CNTR_RATE                                                    :  2;
        unsigned int                                                              : 22;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER_CTRL2 {
    struct {
        unsigned int FORCE_EN                                                     :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER_MASK {
    struct {
        unsigned int SH0_MASK                                                     : 16;
        unsigned int SH1_MASK                                                     : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_POWER_THROTTLE {
    struct {
        unsigned int MIN_POWER                                                    : 14;
        unsigned int                                                              :  2;
        unsigned int MAX_POWER                                                    : 14;
        unsigned int PHASE_OFFSET                                                 :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_POWER_THROTTLE2 {
    struct {
        unsigned int MAX_POWER_DELTA                                              : 14;
        unsigned int                                                              :  2;
        unsigned int SHORT_TERM_INTERVAL_SIZE                                     : 10;
        unsigned int                                                              :  1;
        unsigned int LONG_TERM_INTERVAL_RATIO                                     :  4;
        unsigned int USE_REF_CLOCK                                                :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_RANDOM_WAVE_PRI {
    struct {
        unsigned int RET                                                          :  7;
        unsigned int RUI                                                          :  3;
        unsigned int                                                              : 22;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 10;
        unsigned int RNG                                                          : 13;
        unsigned int                                                              :  9;
    } gfx09;
    struct {
        unsigned int                                                              : 10;
        unsigned int RNG                                                          : 14;
        unsigned int                                                              :  8;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_REG_CREDITS {
    struct {
        unsigned int SRBM_CREDITS                                                 :  6;
        unsigned int                                                              :  2;
        unsigned int CMD_CREDITS                                                  :  4;
        unsigned int                                                              : 16;
        unsigned int REG_BUSY                                                     :  1;
        unsigned int SRBM_OVERFLOW                                                :  1;
        unsigned int IMMED_OVERFLOW                                               :  1;
        unsigned int CMD_OVERFLOW                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_REG_TIMESTAMP {
    struct {
        unsigned int TIMESTAMP                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_RUNTIME_CONFIG {
    struct {
        unsigned int ENABLE_TEX_ARB_OLDEST                                        :  1;
        unsigned int                                                              : 31;
    } gfx09;
    struct {
        unsigned int UNUSED_REGISTER                                              :  1;
        unsigned int                                                              : 31;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_SCRATCH_0 {
    struct {
        unsigned int OFFSET                                                       : 13;
        unsigned int LDS                                                          :  1;
        unsigned int SEG                                                          :  2;
        unsigned int GLC                                                          :  1;
        unsigned int SLC                                                          :  1;
        unsigned int OP                                                           :  7;
        unsigned int                                                              :  1;
        unsigned int ENCODING                                                     :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_SCRATCH_1 {
    struct {
        unsigned int ADDR                                                         :  8;
        unsigned int DATA                                                         :  8;
        unsigned int SADDR                                                        :  7;
        unsigned int NV                                                           :  1;
        unsigned int VDST                                                         :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_SHADER_TBA_HI {
    struct {
        unsigned int ADDR_HI                                                      :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 31;
        unsigned int TRAP_EN                                                      :  1;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_SHADER_TBA_LO {
    struct {
        unsigned int ADDR_LO                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_SHADER_TMA_HI {
    struct {
        unsigned int ADDR_HI                                                      :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_SHADER_TMA_LO {
    struct {
        unsigned int ADDR_LO                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_SMEM_0 {
    struct {
        unsigned int SBASE                                                        :  6;
        unsigned int SDATA                                                        :  7;
        unsigned int                                                              :  1;
        unsigned int SOFFSET_EN                                                   :  1;
        unsigned int NV                                                           :  1;
        unsigned int GLC                                                          :  1;
        unsigned int IMM                                                          :  1;
        unsigned int OP                                                           :  8;
        unsigned int ENCODING                                                     :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_SMEM_1 {
    struct {
        unsigned int OFFSET                                                       : 21;
        unsigned int                                                              :  4;
        unsigned int SOFFSET                                                      :  7;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_SOP1 {
    struct {
        unsigned int SSRC0                                                        :  8;
        unsigned int OP                                                           :  8;
        unsigned int SDST                                                         :  7;
        unsigned int ENCODING                                                     :  9;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_SOP2 {
    struct {
        unsigned int SSRC0                                                        :  8;
        unsigned int SSRC1                                                        :  8;
        unsigned int SDST                                                         :  7;
        unsigned int OP                                                           :  7;
        unsigned int ENCODING                                                     :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_SOPC {
    struct {
        unsigned int SSRC0                                                        :  8;
        unsigned int SSRC1                                                        :  8;
        unsigned int OP                                                           :  7;
        unsigned int ENCODING                                                     :  9;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_SOPK {
    struct {
        unsigned int SIMM16                                                       : 16;
        unsigned int SDST                                                         :  7;
        unsigned int OP                                                           :  5;
        unsigned int ENCODING                                                     :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_SOPP {
    struct {
        unsigned int SIMM16                                                       : 16;
        unsigned int OP                                                           :  7;
        unsigned int ENCODING                                                     :  9;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_TEX_CLK_CTRL {
    struct {
        unsigned int FORCE_CU_ON_SH0                                              : 16;
        unsigned int FORCE_CU_ON_SH1                                              : 16;
    } gfx09;
    struct {
        unsigned int FORCE_WGP_ON_SA0                                             : 16;
        unsigned int FORCE_WGP_ON_SA1                                             : 16;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_BASE {
    struct {
        unsigned int ADDR                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_BASE2 {
    struct {
        unsigned int ADDR_HI                                                      :  4;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_BUF0_BASE {
    struct {
        unsigned int BASE_LO                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_BUF0_SIZE {
    struct {
        unsigned int BASE_HI                                                      :  4;
        unsigned int                                                              :  4;
        unsigned int SIZE                                                         : 22;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_BUF1_BASE {
    struct {
        unsigned int BASE_LO                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_BUF1_SIZE {
    struct {
        unsigned int BASE_HI                                                      :  4;
        unsigned int                                                              :  4;
        unsigned int SIZE                                                         : 22;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_CNTR {
    struct {
        unsigned int CNTR                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_CTRL {
    struct {
        unsigned int                                                              : 31;
        unsigned int RESET_BUFFER                                                 :  1;
    } gfx09;
    struct {
        unsigned int MODE                                                         :  2;
        unsigned int ALL_VMID                                                     :  1;
        unsigned int CH_PERF_EN                                                   :  1;
        unsigned int INTERRUPT_EN                                                 :  1;
        unsigned int DOUBLE_BUFFER                                                :  1;
        unsigned int HIWATER                                                      :  3;
        unsigned int REG_STALL_EN                                                 :  1;
        unsigned int SPI_STALL_EN                                                 :  1;
        unsigned int SQ_STALL_EN                                                  :  1;
        unsigned int REG_DROP_ON_STALL                                            :  1;
        unsigned int UTIL_TIMER                                                   :  1;
        unsigned int WAVESTART_MODE                                               :  2;
        unsigned int RT_FREQ                                                      :  2;
        unsigned int SYNC_COUNT_MARKERS                                           :  1;
        unsigned int SYNC_COUNT_DRAWS                                             :  1;
        unsigned int                                                              : 10;
        unsigned int CAPTURE_ALL                                                  :  1;
        unsigned int DRAW_EVENT_EN                                                :  1;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_DROPPED_CNTR {
    struct {
        unsigned int CNTR                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_FINISH_DONE_DEBUG {
    struct {
        unsigned int GFX0                                                         : 10;
        unsigned int EXPECT0                                                      :  6;
        unsigned int GFX1                                                         : 10;
        unsigned int EXPECT1                                                      :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_GFX_DRAW_CNTR {
    struct {
        unsigned int CNTR                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_GFX_MARKER_CNTR {
    struct {
        unsigned int CNTR                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_HIWATER {
    struct {
        unsigned int HIWATER                                                      :  3;
        unsigned int                                                              : 29;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_HP3D_DRAW_CNTR {
    struct {
        unsigned int CNTR                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_HP3D_MARKER_CNTR {
    struct {
        unsigned int CNTR                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_MASK {
    struct {
        unsigned int CU_SEL                                                       :  5;
        unsigned int SH_SEL                                                       :  1;
        unsigned int                                                              :  1;
        unsigned int REG_STALL_EN                                                 :  1;
        unsigned int SIMD_EN                                                      :  4;
        unsigned int VM_ID_MASK                                                   :  2;
        unsigned int SPI_STALL_EN                                                 :  1;
        unsigned int SQ_STALL_EN                                                  :  1;
        unsigned int                                                              : 16;
    } gfx09;
    struct {
        unsigned int SIMD_SEL                                                     :  2;
        unsigned int                                                              :  2;
        unsigned int WGP_SEL                                                      :  4;
        unsigned int                                                              :  1;
        unsigned int SA_SEL                                                       :  1;
        unsigned int WTYPE_INCLUDE                                                :  7;
        unsigned int                                                              : 15;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_MODE {
    struct {
        unsigned int MASK_PS                                                      :  3;
        unsigned int MASK_VS                                                      :  3;
        unsigned int MASK_GS                                                      :  3;
        unsigned int MASK_ES                                                      :  3;
        unsigned int MASK_HS                                                      :  3;
        unsigned int MASK_LS                                                      :  3;
        unsigned int MASK_CS                                                      :  3;
        unsigned int MODE                                                         :  2;
        unsigned int CAPTURE_MODE                                                 :  2;
        unsigned int AUTOFLUSH_EN                                                 :  1;
        unsigned int TC_PERF_EN                                                   :  1;
        unsigned int ISSUE_MASK                                                   :  2;
        unsigned int TEST_MODE                                                    :  1;
        unsigned int INTERRUPT_EN                                                 :  1;
        unsigned int WRAP                                                         :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_PERF_MASK {
    struct {
        unsigned int SH0_MASK                                                     : 16;
        unsigned int SH1_MASK                                                     : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_SIZE {
    struct {
        unsigned int SIZE                                                         : 22;
        unsigned int                                                              : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_STATUS {
    struct {
        unsigned int FINISH_PENDING                                               : 10;
        unsigned int                                                              :  6;
        unsigned int FINISH_DONE                                                  : 10;
        unsigned int                                                              :  2;
        unsigned int UTC_ERROR                                                    :  1;
        unsigned int NEW_BUF                                                      :  1;
        unsigned int BUSY                                                         :  1;
        unsigned int FULL                                                         :  1;
    } gfx09;
    struct {
        unsigned int FINISH_PENDING                                               : 12;
        unsigned int FINISH_DONE                                                  : 12;
        unsigned int UTC_ERR                                                      :  1;
        unsigned int BUSY                                                         :  1;
        unsigned int EVENT_CNTR_OVERFLOW                                          :  1;
        unsigned int EVENT_CNTR_STALL                                             :  1;
        unsigned int                                                              :  4;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_TOKEN_MASK {
    struct {
        unsigned int TOKEN_MASK                                                   : 16;
        unsigned int REG_MASK                                                     :  8;
        unsigned int REG_DROP_ON_STALL                                            :  1;
        unsigned int                                                              :  7;
    } gfx09;
    struct {
        unsigned int                                                              : 16;
        unsigned int REG_INCLUDE                                                  :  8;
        unsigned int INST_EXCLUDE                                                 :  2;
        unsigned int                                                              :  5;
        unsigned int REG_DETAIL_ALL                                               :  1;
    } gfx10;
    struct {
        unsigned int TOKEN_EXCLUDE                                                : 12;
        unsigned int                                                              : 20;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_TOKEN_MASK2 {
    struct {
        unsigned int INST_MASK                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_USERDATA_0 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_USERDATA_1 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_USERDATA_2 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_USERDATA_3 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_USERDATA_4 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_USERDATA_5 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_USERDATA_6 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_USERDATA_7 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_WORD_CMN {
    struct {
        unsigned int TOKEN_TYPE                                                   :  4;
        unsigned int TIME_DELTA                                                   :  1;
        unsigned int                                                              : 27;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_WORD_EVENT {
    struct {
        unsigned int TOKEN_TYPE                                                   :  4;
        unsigned int TIME_DELTA                                                   :  1;
        unsigned int SH_ID                                                        :  1;
        unsigned int STAGE                                                        :  3;
        unsigned int                                                              :  1;
        unsigned int EVENT_TYPE                                                   :  6;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_WORD_INST {
    struct {
        unsigned int TOKEN_TYPE                                                   :  4;
        unsigned int TIME_DELTA                                                   :  1;
        unsigned int WAVE_ID                                                      :  4;
        unsigned int SIMD_ID                                                      :  2;
        unsigned int INST_TYPE                                                    :  5;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_WORD_INST_PC_1_OF_2 {
    struct {
        unsigned int TOKEN_TYPE                                                   :  4;
        unsigned int TIME_DELTA                                                   :  1;
        unsigned int WAVE_ID                                                      :  4;
        unsigned int SIMD_ID                                                      :  2;
        unsigned int                                                              :  4;
        unsigned int TRAP_ERROR                                                   :  1;
        unsigned int PC_LO                                                        : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_WORD_INST_PC_2_OF_2 {
    struct {
        unsigned int PC_HI                                                        : 24;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_WORD_INST_USERDATA_1_OF_2 {
    struct {
        unsigned int TOKEN_TYPE                                                   :  4;
        unsigned int TIME_DELTA                                                   :  1;
        unsigned int SH_ID                                                        :  1;
        unsigned int CU_ID                                                        :  4;
        unsigned int WAVE_ID                                                      :  4;
        unsigned int SIMD_ID                                                      :  2;
        unsigned int DATA_LO                                                      : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_WORD_INST_USERDATA_2_OF_2 {
    struct {
        unsigned int DATA_HI                                                      : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_WORD_ISSUE {
    struct {
        unsigned int TOKEN_TYPE                                                   :  4;
        unsigned int TIME_DELTA                                                   :  1;
        unsigned int SIMD_ID                                                      :  2;
        unsigned int                                                              :  1;
        unsigned int INST0                                                        :  2;
        unsigned int INST1                                                        :  2;
        unsigned int INST2                                                        :  2;
        unsigned int INST3                                                        :  2;
        unsigned int INST4                                                        :  2;
        unsigned int INST5                                                        :  2;
        unsigned int INST6                                                        :  2;
        unsigned int INST7                                                        :  2;
        unsigned int INST8                                                        :  2;
        unsigned int INST9                                                        :  2;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_WORD_MISC {
    struct {
        unsigned int TOKEN_TYPE                                                   :  4;
        unsigned int TIME_DELTA                                                   :  8;
        unsigned int SH_ID                                                        :  1;
        unsigned int MISC_TOKEN_TYPE                                              :  3;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_WORD_PERF_1_OF_2 {
    struct {
        unsigned int TOKEN_TYPE                                                   :  4;
        unsigned int TIME_DELTA                                                   :  1;
        unsigned int SH_ID                                                        :  1;
        unsigned int CU_ID                                                        :  4;
        unsigned int CNTR_BANK                                                    :  2;
        unsigned int CNTR0                                                        : 13;
        unsigned int CNTR1_LO                                                     :  7;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_WORD_PERF_2_OF_2 {
    struct {
        unsigned int CNTR1_HI                                                     :  6;
        unsigned int CNTR2                                                        : 13;
        unsigned int CNTR3                                                        : 13;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_WORD_REG_1_OF_2 {
    struct {
        unsigned int TOKEN_TYPE                                                   :  4;
        unsigned int TIME_DELTA                                                   :  1;
        unsigned int PIPE_ID                                                      :  2;
        unsigned int ME_ID                                                        :  2;
        unsigned int REG_DROPPED_PREV                                             :  1;
        unsigned int REG_TYPE                                                     :  3;
        unsigned int                                                              :  1;
        unsigned int REG_PRIV                                                     :  1;
        unsigned int REG_OP                                                       :  1;
        unsigned int REG_ADDR                                                     : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_WORD_REG_2_OF_2 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_WORD_REG_CS_1_OF_2 {
    struct {
        unsigned int TOKEN_TYPE                                                   :  4;
        unsigned int TIME_DELTA                                                   :  1;
        unsigned int PIPE_ID                                                      :  2;
        unsigned int ME_ID                                                        :  2;
        unsigned int REG_ADDR                                                     :  7;
        unsigned int DATA_LO                                                      : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_WORD_REG_CS_2_OF_2 {
    struct {
        unsigned int DATA_HI                                                      : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_WORD_TIMESTAMP_1_OF_2 {
    struct {
        unsigned int TOKEN_TYPE                                                   :  4;
        unsigned int                                                              : 12;
        unsigned int TIME_LO                                                      : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_WORD_TIMESTAMP_2_OF_2 {
    struct {
        unsigned int TIME_HI                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_WORD_WAVE {
    struct {
        unsigned int TOKEN_TYPE                                                   :  4;
        unsigned int TIME_DELTA                                                   :  1;
        unsigned int SH_ID                                                        :  1;
        unsigned int CU_ID                                                        :  4;
        unsigned int WAVE_ID                                                      :  4;
        unsigned int SIMD_ID                                                      :  2;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_WORD_WAVE_START {
    struct {
        unsigned int TOKEN_TYPE                                                   :  4;
        unsigned int TIME_DELTA                                                   :  1;
        unsigned int SH_ID                                                        :  1;
        unsigned int CU_ID                                                        :  4;
        unsigned int WAVE_ID                                                      :  4;
        unsigned int SIMD_ID                                                      :  2;
        unsigned int DISPATCHER                                                   :  5;
        unsigned int VS_NO_ALLOC_OR_GROUPED                                       :  1;
        unsigned int COUNT                                                        :  7;
        unsigned int TG_ID                                                        :  3;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_WPTR {
    struct {
        unsigned int WPTR                                                         : 30;
        unsigned int READ_OFFSET                                                  :  2;
    } gfx09;
    struct {
        unsigned int OFFSET                                                       : 29;
        unsigned int                                                              :  2;
        unsigned int BUFFER_ID                                                    :  1;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_TIME_HI {
    struct {
        unsigned int TIME                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_TIME_LO {
    struct {
        unsigned int TIME                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_UTCL1_CNTL1 {
    struct {
        unsigned int FORCE_4K_L2_RESP                                             :  1;
        unsigned int GPUVM_64K_DEF                                                :  1;
        unsigned int GPUVM_PERM_MODE                                              :  1;
        unsigned int RESP_MODE                                                    :  2;
        unsigned int RESP_FAULT_MODE                                              :  2;
        unsigned int CLIENTID                                                     :  9;
        unsigned int USERVM_DIS                                                   :  1;
        unsigned int ENABLE_PUSH_LFIFO                                            :  1;
        unsigned int ENABLE_LFIFO_PRI_ARB                                         :  1;
        unsigned int REG_INVALIDATE_VMID                                          :  4;
        unsigned int REG_INVALIDATE_ALL_VMID                                      :  1;
        unsigned int REG_INVALIDATE_TOGGLE                                        :  1;
        unsigned int REG_INVALIDATE_ALL                                           :  1;
        unsigned int FORCE_MISS                                                   :  1;
        unsigned int FORCE_IN_ORDER                                               :  1;
        unsigned int REDUCE_FIFO_DEPTH_BY_2                                       :  2;
        unsigned int REDUCE_CACHE_SIZE_BY_2                                       :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_UTCL1_CNTL2 {
    struct {
        unsigned int SPARE                                                        :  8;
        unsigned int LFIFO_SCAN_DISABLE                                           :  1;
        unsigned int MTYPE_OVRD_DIS                                               :  1;
        unsigned int LINE_VALID                                                   :  1;
        unsigned int DIS_EDC                                                      :  1;
        unsigned int GPUVM_INV_MODE                                               :  1;
        unsigned int SHOOTDOWN_OPT                                                :  1;
        unsigned int FORCE_SNOOP                                                  :  1;
        unsigned int FORCE_GPUVM_INV_ACK                                          :  1;
        unsigned int RETRY_TIMER                                                  :  7;
        unsigned int                                                              :  3;
        unsigned int FORCE_FRAG_2M_TO_64K                                         :  1;
        unsigned int                                                              :  1;
        unsigned int PREFETCH_PAGE                                                :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_UTCL1_STATUS {
    struct {
        unsigned int FAULT_DETECTED                                               :  1;
        unsigned int RETRY_DETECTED                                               :  1;
        unsigned int PRT_DETECTED                                                 :  1;
        unsigned int RESERVED                                                     : 13;
        unsigned int UNUSED                                                       : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_VINTRP {
    struct {
        unsigned int VSRC                                                         :  8;
        unsigned int ATTRCHAN                                                     :  2;
        unsigned int ATTR                                                         :  6;
        unsigned int OP                                                           :  2;
        unsigned int VDST                                                         :  8;
        unsigned int ENCODING                                                     :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_VOP1 {
    struct {
        unsigned int SRC0                                                         :  9;
        unsigned int OP                                                           :  8;
        unsigned int VDST                                                         :  8;
        unsigned int ENCODING                                                     :  7;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_VOP2 {
    struct {
        unsigned int SRC0                                                         :  9;
        unsigned int VSRC1                                                        :  8;
        unsigned int VDST                                                         :  8;
        unsigned int OP                                                           :  6;
        unsigned int ENCODING                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_VOP3P_0 {
    struct {
        unsigned int VDST                                                         :  8;
        unsigned int NEG_HI                                                       :  3;
        unsigned int OP_SEL                                                       :  3;
        unsigned int OP_SEL_HI_2                                                  :  1;
        unsigned int CLAMP                                                        :  1;
        unsigned int OP                                                           :  7;
        unsigned int ENCODING                                                     :  9;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_VOP3P_1 {
    struct {
        unsigned int SRC0                                                         :  9;
        unsigned int SRC1                                                         :  9;
        unsigned int SRC2                                                         :  9;
        unsigned int OP_SEL_HI                                                    :  2;
        unsigned int NEG                                                          :  3;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_VOP3_0 {
    struct {
        unsigned int VDST                                                         :  8;
        unsigned int ABS                                                          :  3;
        unsigned int OP_SEL                                                       :  4;
        unsigned int CLAMP                                                        :  1;
        unsigned int OP                                                           : 10;
        unsigned int ENCODING                                                     :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_VOP3_0_SDST_ENC {
    struct {
        unsigned int VDST                                                         :  8;
        unsigned int SDST                                                         :  7;
        unsigned int CLAMP                                                        :  1;
        unsigned int OP                                                           : 10;
        unsigned int ENCODING                                                     :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_VOP3_1 {
    struct {
        unsigned int SRC0                                                         :  9;
        unsigned int SRC1                                                         :  9;
        unsigned int SRC2                                                         :  9;
        unsigned int OMOD                                                         :  2;
        unsigned int NEG                                                          :  3;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_VOPC {
    struct {
        unsigned int SRC0                                                         :  9;
        unsigned int VSRC1                                                        :  8;
        unsigned int OP                                                           :  8;
        unsigned int ENCODING                                                     :  7;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_VOP_DPP {
    struct {
        unsigned int SRC0                                                         :  8;
        unsigned int DPP_CTRL                                                     :  9;
        unsigned int                                                              :  2;
        unsigned int BOUND_CTRL                                                   :  1;
        unsigned int SRC0_NEG                                                     :  1;
        unsigned int SRC0_ABS                                                     :  1;
        unsigned int SRC1_NEG                                                     :  1;
        unsigned int SRC1_ABS                                                     :  1;
        unsigned int BANK_MASK                                                    :  4;
        unsigned int ROW_MASK                                                     :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_VOP_SDWA {
    struct {
        unsigned int SRC0                                                         :  8;
        unsigned int DST_SEL                                                      :  3;
        unsigned int DST_UNUSED                                                   :  2;
        unsigned int CLAMP                                                        :  1;
        unsigned int OMOD                                                         :  2;
        unsigned int SRC0_SEL                                                     :  3;
        unsigned int SRC0_SEXT                                                    :  1;
        unsigned int SRC0_NEG                                                     :  1;
        unsigned int SRC0_ABS                                                     :  1;
        unsigned int                                                              :  1;
        unsigned int S0                                                           :  1;
        unsigned int SRC1_SEL                                                     :  3;
        unsigned int SRC1_SEXT                                                    :  1;
        unsigned int SRC1_NEG                                                     :  1;
        unsigned int SRC1_ABS                                                     :  1;
        unsigned int                                                              :  1;
        unsigned int S1                                                           :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_VOP_SDWA_SDST_ENC {
    struct {
        unsigned int SRC0                                                         :  8;
        unsigned int SDST                                                         :  7;
        unsigned int SD                                                           :  1;
        unsigned int SRC0_SEL                                                     :  3;
        unsigned int SRC0_SEXT                                                    :  1;
        unsigned int SRC0_NEG                                                     :  1;
        unsigned int SRC0_ABS                                                     :  1;
        unsigned int                                                              :  1;
        unsigned int S0                                                           :  1;
        unsigned int SRC1_SEL                                                     :  3;
        unsigned int SRC1_SEXT                                                    :  1;
        unsigned int SRC1_NEG                                                     :  1;
        unsigned int SRC1_ABS                                                     :  1;
        unsigned int                                                              :  1;
        unsigned int S1                                                           :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WATCH0_ADDR_H {
    struct {
        unsigned int ADDR                                                         : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WATCH0_ADDR_L {
    struct {
        unsigned int                                                              :  6;
        unsigned int ADDR                                                         : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WATCH0_CNTL {
    struct {
        unsigned int MASK                                                         : 24;
        unsigned int VMID                                                         :  4;
        unsigned int                                                              :  3;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 29;
        unsigned int MODE                                                         :  2;
        unsigned int                                                              :  1;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WATCH1_ADDR_H {
    struct {
        unsigned int ADDR                                                         : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WATCH1_ADDR_L {
    struct {
        unsigned int                                                              :  6;
        unsigned int ADDR                                                         : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WATCH1_CNTL {
    struct {
        unsigned int MASK                                                         : 24;
        unsigned int VMID                                                         :  4;
        unsigned int                                                              :  3;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 29;
        unsigned int MODE                                                         :  2;
        unsigned int                                                              :  1;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WATCH2_ADDR_H {
    struct {
        unsigned int ADDR                                                         : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WATCH2_ADDR_L {
    struct {
        unsigned int                                                              :  6;
        unsigned int ADDR                                                         : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WATCH2_CNTL {
    struct {
        unsigned int MASK                                                         : 24;
        unsigned int VMID                                                         :  4;
        unsigned int                                                              :  3;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 29;
        unsigned int MODE                                                         :  2;
        unsigned int                                                              :  1;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WATCH3_ADDR_H {
    struct {
        unsigned int ADDR                                                         : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WATCH3_ADDR_L {
    struct {
        unsigned int                                                              :  6;
        unsigned int ADDR                                                         : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WATCH3_CNTL {
    struct {
        unsigned int MASK                                                         : 24;
        unsigned int VMID                                                         :  4;
        unsigned int                                                              :  3;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 29;
        unsigned int MODE                                                         :  2;
        unsigned int                                                              :  1;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WREXEC_EXEC_HI {
    struct {
        unsigned int ADDR_HI                                                      : 16;
        unsigned int                                                              : 10;
        unsigned int FIRST_WAVE                                                   :  1;
        unsigned int                                                              :  1;
        unsigned int MTYPE                                                        :  3;
        unsigned int MSB                                                          :  1;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 27;
        unsigned int ATC                                                          :  1;
        unsigned int                                                              :  4;
    } gfx09;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WREXEC_EXEC_LO {
    struct {
        unsigned int ADDR_LO                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_BLEND_OPT_CONTROL {
    struct {
        unsigned int MRT0_COLOR_OPT_DISABLE                                       :  1;
        unsigned int MRT0_ALPHA_OPT_DISABLE                                       :  1;
        unsigned int                                                              :  2;
        unsigned int MRT1_COLOR_OPT_DISABLE                                       :  1;
        unsigned int MRT1_ALPHA_OPT_DISABLE                                       :  1;
        unsigned int                                                              :  2;
        unsigned int MRT2_COLOR_OPT_DISABLE                                       :  1;
        unsigned int MRT2_ALPHA_OPT_DISABLE                                       :  1;
        unsigned int                                                              :  2;
        unsigned int MRT3_COLOR_OPT_DISABLE                                       :  1;
        unsigned int MRT3_ALPHA_OPT_DISABLE                                       :  1;
        unsigned int                                                              :  2;
        unsigned int MRT4_COLOR_OPT_DISABLE                                       :  1;
        unsigned int MRT4_ALPHA_OPT_DISABLE                                       :  1;
        unsigned int                                                              :  2;
        unsigned int MRT5_COLOR_OPT_DISABLE                                       :  1;
        unsigned int MRT5_ALPHA_OPT_DISABLE                                       :  1;
        unsigned int                                                              :  2;
        unsigned int MRT6_COLOR_OPT_DISABLE                                       :  1;
        unsigned int MRT6_ALPHA_OPT_DISABLE                                       :  1;
        unsigned int                                                              :  2;
        unsigned int MRT7_COLOR_OPT_DISABLE                                       :  1;
        unsigned int MRT7_ALPHA_OPT_DISABLE                                       :  1;
        unsigned int                                                              :  1;
        unsigned int PIXEN_ZERO_OPT_DISABLE                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_BLEND_OPT_EPSILON {
    struct {
        unsigned int MRT0_EPSILON                                                 :  4;
        unsigned int MRT1_EPSILON                                                 :  4;
        unsigned int MRT2_EPSILON                                                 :  4;
        unsigned int MRT3_EPSILON                                                 :  4;
        unsigned int MRT4_EPSILON                                                 :  4;
        unsigned int MRT5_EPSILON                                                 :  4;
        unsigned int MRT6_EPSILON                                                 :  4;
        unsigned int MRT7_EPSILON                                                 :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_DEBUG_1 {
    struct {
        unsigned int SX_DB_QUAD_CREDIT                                            :  7;
        unsigned int                                                              :  1;
        unsigned int DISABLE_BLEND_OPT_DONT_RD_DST                                :  1;
        unsigned int DISABLE_BLEND_OPT_BYPASS                                     :  1;
        unsigned int DISABLE_BLEND_OPT_DISCARD_PIXEL                              :  1;
        unsigned int DISABLE_QUAD_PAIR_OPT                                        :  1;
        unsigned int DISABLE_PIX_EN_ZERO_OPT                                      :  1;
        unsigned int                                                              : 19;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 13;
        unsigned int DISABLE_SX_DB_FGCG                                           :  1;
        unsigned int                                                              : 18;
    } most;
    struct {
        unsigned int                                                              : 14;
        unsigned int DEBUG_DATA                                                   : 18;
    } gfx09;
    struct {
        unsigned int                                                              : 13;
        unsigned int PC_CFG                                                       :  1;
        unsigned int                                                              : 18;
    } gfx09_0;
    struct {
        unsigned int                                                              : 14;
        unsigned int ENABLE_SAME_PC_GDS_CGTS                                      :  1;
        unsigned int DISABLE_RAM_FGCG                                             :  1;
        unsigned int PC_DISABLE_SAME_ADDR_OPT                                     :  1;
        unsigned int                                                              : 15;
    } gfx10;
    struct {
        unsigned int                                                              : 18;
        unsigned int DEBUG_DATA                                                   : 14;
    } gfx101;
    struct {
        unsigned int                                                              : 13;
        unsigned int DISABLE_REP_FGCG                                             :  1;
        unsigned int                                                              :  3;
        unsigned int DISABLE_COL_VAL_READ_OPT                                     :  1;
        unsigned int                                                              : 14;
    } gfx101Plus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_MRT0_BLEND_OPT {
    struct {
        unsigned int COLOR_SRC_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int COLOR_DST_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int COLOR_COMB_FCN                                               :  3;
        unsigned int                                                              :  5;
        unsigned int ALPHA_SRC_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int ALPHA_DST_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int ALPHA_COMB_FCN                                               :  3;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_MRT1_BLEND_OPT {
    struct {
        unsigned int COLOR_SRC_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int COLOR_DST_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int COLOR_COMB_FCN                                               :  3;
        unsigned int                                                              :  5;
        unsigned int ALPHA_SRC_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int ALPHA_DST_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int ALPHA_COMB_FCN                                               :  3;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_MRT2_BLEND_OPT {
    struct {
        unsigned int COLOR_SRC_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int COLOR_DST_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int COLOR_COMB_FCN                                               :  3;
        unsigned int                                                              :  5;
        unsigned int ALPHA_SRC_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int ALPHA_DST_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int ALPHA_COMB_FCN                                               :  3;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_MRT3_BLEND_OPT {
    struct {
        unsigned int COLOR_SRC_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int COLOR_DST_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int COLOR_COMB_FCN                                               :  3;
        unsigned int                                                              :  5;
        unsigned int ALPHA_SRC_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int ALPHA_DST_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int ALPHA_COMB_FCN                                               :  3;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_MRT4_BLEND_OPT {
    struct {
        unsigned int COLOR_SRC_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int COLOR_DST_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int COLOR_COMB_FCN                                               :  3;
        unsigned int                                                              :  5;
        unsigned int ALPHA_SRC_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int ALPHA_DST_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int ALPHA_COMB_FCN                                               :  3;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_MRT5_BLEND_OPT {
    struct {
        unsigned int COLOR_SRC_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int COLOR_DST_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int COLOR_COMB_FCN                                               :  3;
        unsigned int                                                              :  5;
        unsigned int ALPHA_SRC_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int ALPHA_DST_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int ALPHA_COMB_FCN                                               :  3;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_MRT6_BLEND_OPT {
    struct {
        unsigned int COLOR_SRC_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int COLOR_DST_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int COLOR_COMB_FCN                                               :  3;
        unsigned int                                                              :  5;
        unsigned int ALPHA_SRC_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int ALPHA_DST_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int ALPHA_COMB_FCN                                               :  3;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_MRT7_BLEND_OPT {
    struct {
        unsigned int COLOR_SRC_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int COLOR_DST_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int COLOR_COMB_FCN                                               :  3;
        unsigned int                                                              :  5;
        unsigned int ALPHA_SRC_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int ALPHA_DST_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int ALPHA_COMB_FCN                                               :  3;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_PERFCOUNTER0_SELECT {
    struct {
        unsigned int                                                              : 20;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  8;
    } bits, bitfields;
    struct {
        unsigned int PERFCOUNTER_SELECT                                           : 10;
        unsigned int PERFCOUNTER_SELECT1                                          : 10;
        unsigned int                                                              : 12;
    } gfx09_0;
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } gfx09_1x;
    struct {
        unsigned int PERFCOUNTER_SELECT                                           : 10;
        unsigned int PERFCOUNTER_SELECT1                                          : 10;
        unsigned int                                                              : 12;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERFCOUNTER_SELECT2                                          : 10;
        unsigned int PERFCOUNTER_SELECT3                                          : 10;
        unsigned int                                                              : 12;
    } gfx09_0;
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } gfx09_1x;
    struct {
        unsigned int PERFCOUNTER_SELECT2                                          : 10;
        unsigned int PERFCOUNTER_SELECT3                                          : 10;
        unsigned int                                                              : 12;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_PERFCOUNTER1_SELECT {
    struct {
        unsigned int                                                              : 20;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  8;
    } bits, bitfields;
    struct {
        unsigned int PERFCOUNTER_SELECT                                           : 10;
        unsigned int PERFCOUNTER_SELECT1                                          : 10;
        unsigned int                                                              : 12;
    } gfx09_0;
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } gfx09_1x;
    struct {
        unsigned int PERFCOUNTER_SELECT                                           : 10;
        unsigned int PERFCOUNTER_SELECT1                                          : 10;
        unsigned int                                                              : 12;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_PERFCOUNTER1_SELECT1 {
    struct {
        unsigned int PERFCOUNTER_SELECT2                                          : 10;
        unsigned int PERFCOUNTER_SELECT3                                          : 10;
        unsigned int                                                              : 12;
    } gfx09_0;
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } gfx09_1x;
    struct {
        unsigned int PERFCOUNTER_SELECT2                                          : 10;
        unsigned int PERFCOUNTER_SELECT3                                          : 10;
        unsigned int                                                              : 12;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_PERFCOUNTER2_SELECT {
    struct {
        unsigned int                                                              : 20;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  8;
    } bits, bitfields;
    struct {
        unsigned int PERFCOUNTER_SELECT                                           : 10;
        unsigned int PERFCOUNTER_SELECT1                                          : 10;
        unsigned int                                                              : 12;
    } gfx09_0;
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 18;
        unsigned int PERF_MODE                                                    :  4;
    } gfx09_1x;
    struct {
        unsigned int PERFCOUNTER_SELECT                                           : 10;
        unsigned int                                                              : 22;
    } gfx101;
    struct {
        unsigned int                                                              : 10;
        unsigned int PERFCOUNTER_SELECT1                                          : 10;
        unsigned int                                                              : 12;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_PERFCOUNTER3_SELECT {
    struct {
        unsigned int                                                              : 20;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  8;
    } bits, bitfields;
    struct {
        unsigned int PERFCOUNTER_SELECT                                           : 10;
        unsigned int PERFCOUNTER_SELECT1                                          : 10;
        unsigned int                                                              : 12;
    } gfx09_0;
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 18;
        unsigned int PERF_MODE                                                    :  4;
    } gfx09_1x;
    struct {
        unsigned int PERFCOUNTER_SELECT                                           : 10;
        unsigned int                                                              : 22;
    } gfx101;
    struct {
        unsigned int                                                              : 10;
        unsigned int PERFCOUNTER_SELECT1                                          : 10;
        unsigned int                                                              : 12;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_PS_DOWNCONVERT {
    struct {
        unsigned int MRT0                                                         :  4;
        unsigned int MRT1                                                         :  4;
        unsigned int MRT2                                                         :  4;
        unsigned int MRT3                                                         :  4;
        unsigned int MRT4                                                         :  4;
        unsigned int MRT5                                                         :  4;
        unsigned int MRT6                                                         :  4;
        unsigned int MRT7                                                         :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TA_BC_BASE_ADDR {
    struct {
        unsigned int ADDRESS                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TA_BC_BASE_ADDR_HI {
    struct {
        unsigned int ADDRESS                                                      :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TA_CGTT_CTRL {
    struct {
        unsigned int ON_DELAY                                                     :  4;
        unsigned int OFF_HYSTERESIS                                               :  8;
        unsigned int                                                              :  4;
        unsigned int SOFT_STALL_OVERRIDE7                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE6                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE5                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE4                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE3                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE2                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE1                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE0                                         :  1;
        unsigned int SOFT_OVERRIDE7                                               :  1;
        unsigned int SOFT_OVERRIDE6                                               :  1;
        unsigned int SOFT_OVERRIDE5                                               :  1;
        unsigned int SOFT_OVERRIDE4                                               :  1;
        unsigned int SOFT_OVERRIDE3                                               :  1;
        unsigned int SOFT_OVERRIDE2                                               :  1;
        unsigned int SOFT_OVERRIDE1                                               :  1;
        unsigned int SOFT_OVERRIDE0                                               :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TA_CNTL {
    struct {
        unsigned int                                                              : 16;
        unsigned int ALIGNER_CREDIT                                               :  5;
        unsigned int                                                              :  1;
        unsigned int TD_FIFO_CREDIT                                               : 10;
    } bits, bitfields;
    struct {
        unsigned int FX_XNACK_CREDIT                                              :  7;
        unsigned int                                                              : 25;
    } most;
    struct {
        unsigned int                                                              :  9;
        unsigned int SQ_XNACK_CREDIT                                              :  4;
        unsigned int TC_DATA_CREDIT                                               :  3;
        unsigned int                                                              : 16;
    } gfx09;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TA_CNTL_AUX {
    struct {
        unsigned int SCOAL_DSWIZZLE_N                                             :  1;
        unsigned int                                                              :  4;
        unsigned int TFAULT_EN_OVERRIDE                                           :  1;
        unsigned int GATHERH_DST_SEL                                              :  1;
        unsigned int DISABLE_GATHER4_BC_SWIZZLE                                   :  1;
        unsigned int                                                              :  2;
        unsigned int ANISO_HALF_THRESH                                            :  2;
        unsigned int ANISO_ERROR_FP_VBIAS                                         :  1;
        unsigned int ANISO_STEP_ORDER                                             :  1;
        unsigned int ANISO_STEP                                                   :  1;
        unsigned int MINMAG_UNNORM                                                :  1;
        unsigned int ANISO_WEIGHT_MODE                                            :  1;
        unsigned int ANISO_RATIO_LUT                                              :  1;
        unsigned int ANISO_TAP                                                    :  1;
        unsigned int                                                              :  1;
        unsigned int DETERMINISM_RESERVED_DISABLE                                 :  1;
        unsigned int DETERMINISM_OPCODE_STRICT_DISABLE                            :  1;
        unsigned int DETERMINISM_MISC_DISABLE                                     :  1;
        unsigned int DETERMINISM_SAMPLE_C_DFMT_DISABLE                            :  1;
        unsigned int DETERMINISM_SAMPLER_MSAA_DISABLE                             :  1;
        unsigned int DETERMINISM_WRITEOP_READFMT_DISABLE                          :  1;
        unsigned int DETERMINISM_DFMT_NFMT_DISABLE                                :  1;
        unsigned int DISABLE_DWORD_X2_COALESCE                                    :  1;
        unsigned int CUBEMAP_SLICE_CLAMP                                          :  1;
        unsigned int TRUNC_SMALL_NEG                                              :  1;
        unsigned int ARRAY_ROUND_MODE                                             :  2;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  1;
        unsigned int RESERVED                                                     :  3;
        unsigned int                                                              : 28;
    } most;
    struct {
        unsigned int                                                              :  9;
        unsigned int NONIMG_ANISO_BYPASS                                          :  1;
        unsigned int                                                              :  9;
        unsigned int ANISO_MIP_ADJ_MODE                                           :  1;
        unsigned int                                                              : 12;
    } gfx09;
    struct {
        unsigned int                                                              :  9;
        unsigned int AUTO_ALIGN_FORMAT                                            :  1;
        unsigned int                                                              : 22;
    } gfx10;
    struct {
        unsigned int                                                              :  4;
        unsigned int DERIV_ADJUST_DIS                                             :  1;
        unsigned int                                                              : 27;
    } gfx101;
    struct {
        unsigned int                                                              :  8;
        unsigned int ANISO_MAG_STEP_CLAMP                                         :  1;
        unsigned int                                                              : 23;
    } gfx101Plus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TA_CS_BC_BASE_ADDR {
    struct {
        unsigned int ADDRESS                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TA_CS_BC_BASE_ADDR_HI {
    struct {
        unsigned int ADDRESS                                                      :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TA_DEBUG_DATA {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TA_DEBUG_INDEX {
    struct {
        unsigned int INDEX                                                        :  5;
        unsigned int                                                              : 27;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TA_DSM_CNTL {
    struct {
        unsigned int TA_FS_DFIFO_DSM_IRRITATOR_DATA                               :  2;
        unsigned int TA_FS_DFIFO_ENABLE_SINGLE_WRITE                              :  1;
        unsigned int TA_FS_AFIFO_DSM_IRRITATOR_DATA                               :  2;
        unsigned int TA_FS_AFIFO_ENABLE_SINGLE_WRITE                              :  1;
        unsigned int TA_FL_LFIFO_DSM_IRRITATOR_DATA                               :  2;
        unsigned int TA_FL_LFIFO_ENABLE_SINGLE_WRITE                              :  1;
        unsigned int TA_FX_LFIFO_DSM_IRRITATOR_DATA                               :  2;
        unsigned int TA_FX_LFIFO_ENABLE_SINGLE_WRITE                              :  1;
        unsigned int TA_FS_CFIFO_DSM_IRRITATOR_DATA                               :  2;
        unsigned int TA_FS_CFIFO_ENABLE_SINGLE_WRITE                              :  1;
        unsigned int                                                              : 17;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TA_DSM_CNTL2 {
    struct {
        unsigned int TA_FS_DFIFO_ENABLE_ERROR_INJECT                              :  2;
        unsigned int TA_FS_DFIFO_SELECT_INJECT_DELAY                              :  1;
        unsigned int TA_FS_AFIFO_ENABLE_ERROR_INJECT                              :  2;
        unsigned int TA_FS_AFIFO_SELECT_INJECT_DELAY                              :  1;
        unsigned int TA_FL_LFIFO_ENABLE_ERROR_INJECT                              :  2;
        unsigned int TA_FL_LFIFO_SELECT_INJECT_DELAY                              :  1;
        unsigned int TA_FX_LFIFO_ENABLE_ERROR_INJECT                              :  2;
        unsigned int TA_FX_LFIFO_SELECT_INJECT_DELAY                              :  1;
        unsigned int TA_FS_CFIFO_ENABLE_ERROR_INJECT                              :  2;
        unsigned int TA_FS_CFIFO_SELECT_INJECT_DELAY                              :  1;
        unsigned int                                                              : 11;
        unsigned int TA_INJECT_DELAY                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TA_EDC_CNT {
    struct {
        unsigned int TA_FS_DFIFO_SEC_COUNT                                        :  2;
        unsigned int TA_FS_DFIFO_DED_COUNT                                        :  2;
        unsigned int TA_FS_AFIFO_SED_COUNT                                        :  2;
        unsigned int TA_FL_LFIFO_SED_COUNT                                        :  2;
        unsigned int TA_FX_LFIFO_SED_COUNT                                        :  2;
        unsigned int TA_FS_CFIFO_SED_COUNT                                        :  2;
        unsigned int                                                              : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TA_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TA_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TA_PERFCOUNTER0_SELECT {
    struct {
        unsigned int                                                              : 20;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int                                                              :  2;
        unsigned int PERF_SEL1                                                    :  8;
        unsigned int                                                              : 14;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TA_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int                                                              : 24;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;
    struct {
        unsigned int PERF_SEL2                                                    :  8;
        unsigned int                                                              :  2;
        unsigned int PERF_SEL3                                                    :  8;
        unsigned int                                                              : 14;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TA_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TA_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TA_PERFCOUNTER1_SELECT {
    struct {
        unsigned int                                                              : 20;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int                                                              : 24;
    } most;
    struct {
        unsigned int                                                              : 10;
        unsigned int PERF_SEL1                                                    :  8;
        unsigned int                                                              :  6;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int                                                              :  4;
    } gfx09_0;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TA_POWER_CNTL {
    struct {
        unsigned int SAMPLER_CLK_VALID_DELAY                                      :  3;
        unsigned int SAMPLER_CLK_EN_MODE                                          :  1;
        unsigned int                                                              : 12;
        unsigned int NOSAMPLER_CLK_VALID_DELAY                                    :  3;
        unsigned int NOSAMPLER_CLK_EN_MODE                                        :  1;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TA_RESERVED_010C {
    struct {
        unsigned int Unused                                                       : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TA_SCRATCH {
    struct {
        unsigned int SCRATCH                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TA_STATUS {
    struct {
        unsigned int                                                              : 12;
        unsigned int FG_PFIFO_EMPTYB                                              :  1;
        unsigned int FG_LFIFO_EMPTYB                                              :  1;
        unsigned int FG_SFIFO_EMPTYB                                              :  1;
        unsigned int                                                              :  1;
        unsigned int FL_PFIFO_EMPTYB                                              :  1;
        unsigned int FL_LFIFO_EMPTYB                                              :  1;
        unsigned int FL_SFIFO_EMPTYB                                              :  1;
        unsigned int                                                              :  1;
        unsigned int FA_PFIFO_EMPTYB                                              :  1;
        unsigned int FA_LFIFO_EMPTYB                                              :  1;
        unsigned int FA_SFIFO_EMPTYB                                              :  1;
        unsigned int                                                              :  1;
        unsigned int IN_BUSY                                                      :  1;
        unsigned int FG_BUSY                                                      :  1;
        unsigned int LA_BUSY                                                      :  1;
        unsigned int FL_BUSY                                                      :  1;
        unsigned int TA_BUSY                                                      :  1;
        unsigned int FA_BUSY                                                      :  1;
        unsigned int AL_BUSY                                                      :  1;
        unsigned int BUSY                                                         :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCA_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCA_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCA_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCA_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCA_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCA_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCA_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCA_PERFCOUNTER1_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCA_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCA_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCA_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCA_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCA_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCA_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCC_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCC_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCC_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCC_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCC_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCC_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCC_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCC_PERFCOUNTER1_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCC_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCC_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCC_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCC_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCC_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCC_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCI_CNTL_1 {
    struct {
        unsigned int WBINVL1_NUM_CYCLES                                           : 16;
        unsigned int REQ_FIFO_DEPTH                                               :  8;
        unsigned int WDATA_RAM_DEPTH                                              :  8;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCI_CNTL_2 {
    struct {
        unsigned int L1_INVAL_ON_WBINVL2                                          :  1;
        unsigned int TCA_MAX_CREDIT                                               :  8;
        unsigned int                                                              : 23;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCI_DSM_CNTL {
    struct {
        unsigned int WRITE_RAM_IRRITATOR_DATA_SEL                                 :  2;
        unsigned int WRITE_RAM_IRRITATOR_SINGLE_WRITE                             :  1;
        unsigned int                                                              : 29;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCI_DSM_CNTL2 {
    struct {
        unsigned int WRITE_RAM_ENABLE_ERROR_INJECT                                :  2;
        unsigned int WRITE_RAM_SELECT_INJECT_DELAY                                :  1;
        unsigned int                                                              : 23;
        unsigned int TCI_INJECT_DELAY                                             :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCI_EDC_CNT {
    struct {
        unsigned int WRITE_RAM_SED_COUNT                                          :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCI_STATUS {
    struct {
        unsigned int TCI_BUSY                                                     :  1;
        unsigned int                                                              : 31;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCP_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCP_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCP_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCP_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCP_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCP_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCP_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCP_PERFCOUNTER1_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCP_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCP_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCP_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCP_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCP_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCP_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCP_PERFCOUNTER_FILTER {
    struct {
        unsigned int BUFFER                                                       :  1;
        unsigned int FLAT                                                         :  1;
        unsigned int DIM                                                          :  3;
        unsigned int                                                              : 27;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  5;
        unsigned int DATA_FORMAT                                                  :  6;
        unsigned int NUM_FORMAT                                                   :  4;
        unsigned int SW_MODE                                                      :  5;
        unsigned int NUM_SAMPLES                                                  :  2;
        unsigned int OPCODE_TYPE                                                  :  3;
        unsigned int GLC                                                          :  1;
        unsigned int SLC                                                          :  1;
        unsigned int COMPRESSION_ENABLE                                           :  1;
        unsigned int ADDR_MODE                                                    :  3;
        unsigned int                                                              :  1;
    } gfx09;
    struct {
        unsigned int                                                              :  5;
        unsigned int DATA_FORMAT                                                  :  7;
        unsigned int                                                              :  1;
        unsigned int NUM_FORMAT                                                   :  4;
        unsigned int SW_MODE                                                      :  5;
        unsigned int NUM_SAMPLES                                                  :  2;
        unsigned int OPCODE_TYPE                                                  :  3;
        unsigned int SLC                                                          :  1;
        unsigned int DLC                                                          :  1;
        unsigned int GLC                                                          :  1;
        unsigned int COMPRESSION_ENABLE                                           :  1;
        unsigned int                                                              :  1;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCP_PERFCOUNTER_FILTER2 {
    struct {
        unsigned int REQ_MODE                                                     :  3;
        unsigned int                                                              : 29;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCP_PERFCOUNTER_FILTER_EN {
    struct {
        unsigned int BUFFER                                                       :  1;
        unsigned int FLAT                                                         :  1;
        unsigned int DIM                                                          :  1;
        unsigned int DATA_FORMAT                                                  :  1;
        unsigned int NUM_FORMAT                                                   :  1;
        unsigned int SW_MODE                                                      :  1;
        unsigned int NUM_SAMPLES                                                  :  1;
        unsigned int OPCODE_TYPE                                                  :  1;
        unsigned int                                                              : 24;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  8;
        unsigned int GLC                                                          :  1;
        unsigned int SLC                                                          :  1;
        unsigned int COMPRESSION_ENABLE                                           :  1;
        unsigned int ADDR_MODE                                                    :  1;
        unsigned int                                                              : 20;
    } gfx09;
    struct {
        unsigned int                                                              :  8;
        unsigned int SLC                                                          :  1;
        unsigned int DLC                                                          :  1;
        unsigned int GLC                                                          :  1;
        unsigned int COMPRESSION_ENABLE                                           :  1;
        unsigned int REQ_MODE                                                     :  1;
        unsigned int                                                              : 19;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TC_CFG_L1_LOAD_POLICY0 {
    struct {
        unsigned int POLICY_0                                                     :  2;
        unsigned int POLICY_1                                                     :  2;
        unsigned int POLICY_2                                                     :  2;
        unsigned int POLICY_3                                                     :  2;
        unsigned int POLICY_4                                                     :  2;
        unsigned int POLICY_5                                                     :  2;
        unsigned int POLICY_6                                                     :  2;
        unsigned int POLICY_7                                                     :  2;
        unsigned int POLICY_8                                                     :  2;
        unsigned int POLICY_9                                                     :  2;
        unsigned int POLICY_10                                                    :  2;
        unsigned int POLICY_11                                                    :  2;
        unsigned int POLICY_12                                                    :  2;
        unsigned int POLICY_13                                                    :  2;
        unsigned int POLICY_14                                                    :  2;
        unsigned int POLICY_15                                                    :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TC_CFG_L1_LOAD_POLICY1 {
    struct {
        unsigned int POLICY_16                                                    :  2;
        unsigned int POLICY_17                                                    :  2;
        unsigned int POLICY_18                                                    :  2;
        unsigned int POLICY_19                                                    :  2;
        unsigned int POLICY_20                                                    :  2;
        unsigned int POLICY_21                                                    :  2;
        unsigned int POLICY_22                                                    :  2;
        unsigned int POLICY_23                                                    :  2;
        unsigned int POLICY_24                                                    :  2;
        unsigned int POLICY_25                                                    :  2;
        unsigned int POLICY_26                                                    :  2;
        unsigned int POLICY_27                                                    :  2;
        unsigned int POLICY_28                                                    :  2;
        unsigned int POLICY_29                                                    :  2;
        unsigned int POLICY_30                                                    :  2;
        unsigned int POLICY_31                                                    :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TC_CFG_L1_STORE_POLICY {
    struct {
        unsigned int POLICY_0                                                     :  1;
        unsigned int POLICY_1                                                     :  1;
        unsigned int POLICY_2                                                     :  1;
        unsigned int POLICY_3                                                     :  1;
        unsigned int POLICY_4                                                     :  1;
        unsigned int POLICY_5                                                     :  1;
        unsigned int POLICY_6                                                     :  1;
        unsigned int POLICY_7                                                     :  1;
        unsigned int POLICY_8                                                     :  1;
        unsigned int POLICY_9                                                     :  1;
        unsigned int POLICY_10                                                    :  1;
        unsigned int POLICY_11                                                    :  1;
        unsigned int POLICY_12                                                    :  1;
        unsigned int POLICY_13                                                    :  1;
        unsigned int POLICY_14                                                    :  1;
        unsigned int POLICY_15                                                    :  1;
        unsigned int POLICY_16                                                    :  1;
        unsigned int POLICY_17                                                    :  1;
        unsigned int POLICY_18                                                    :  1;
        unsigned int POLICY_19                                                    :  1;
        unsigned int POLICY_20                                                    :  1;
        unsigned int POLICY_21                                                    :  1;
        unsigned int POLICY_22                                                    :  1;
        unsigned int POLICY_23                                                    :  1;
        unsigned int POLICY_24                                                    :  1;
        unsigned int POLICY_25                                                    :  1;
        unsigned int POLICY_26                                                    :  1;
        unsigned int POLICY_27                                                    :  1;
        unsigned int POLICY_28                                                    :  1;
        unsigned int POLICY_29                                                    :  1;
        unsigned int POLICY_30                                                    :  1;
        unsigned int POLICY_31                                                    :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TC_CFG_L1_VOLATILE {
    struct {
        unsigned int VOL                                                          :  4;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TC_CFG_L2_ATOMIC_POLICY {
    struct {
        unsigned int POLICY_0                                                     :  2;
        unsigned int POLICY_1                                                     :  2;
        unsigned int POLICY_2                                                     :  2;
        unsigned int POLICY_3                                                     :  2;
        unsigned int POLICY_4                                                     :  2;
        unsigned int POLICY_5                                                     :  2;
        unsigned int POLICY_6                                                     :  2;
        unsigned int POLICY_7                                                     :  2;
        unsigned int POLICY_8                                                     :  2;
        unsigned int POLICY_9                                                     :  2;
        unsigned int POLICY_10                                                    :  2;
        unsigned int POLICY_11                                                    :  2;
        unsigned int POLICY_12                                                    :  2;
        unsigned int POLICY_13                                                    :  2;
        unsigned int POLICY_14                                                    :  2;
        unsigned int POLICY_15                                                    :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TC_CFG_L2_LOAD_POLICY0 {
    struct {
        unsigned int POLICY_0                                                     :  2;
        unsigned int POLICY_1                                                     :  2;
        unsigned int POLICY_2                                                     :  2;
        unsigned int POLICY_3                                                     :  2;
        unsigned int POLICY_4                                                     :  2;
        unsigned int POLICY_5                                                     :  2;
        unsigned int POLICY_6                                                     :  2;
        unsigned int POLICY_7                                                     :  2;
        unsigned int POLICY_8                                                     :  2;
        unsigned int POLICY_9                                                     :  2;
        unsigned int POLICY_10                                                    :  2;
        unsigned int POLICY_11                                                    :  2;
        unsigned int POLICY_12                                                    :  2;
        unsigned int POLICY_13                                                    :  2;
        unsigned int POLICY_14                                                    :  2;
        unsigned int POLICY_15                                                    :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TC_CFG_L2_LOAD_POLICY1 {
    struct {
        unsigned int POLICY_16                                                    :  2;
        unsigned int POLICY_17                                                    :  2;
        unsigned int POLICY_18                                                    :  2;
        unsigned int POLICY_19                                                    :  2;
        unsigned int POLICY_20                                                    :  2;
        unsigned int POLICY_21                                                    :  2;
        unsigned int POLICY_22                                                    :  2;
        unsigned int POLICY_23                                                    :  2;
        unsigned int POLICY_24                                                    :  2;
        unsigned int POLICY_25                                                    :  2;
        unsigned int POLICY_26                                                    :  2;
        unsigned int POLICY_27                                                    :  2;
        unsigned int POLICY_28                                                    :  2;
        unsigned int POLICY_29                                                    :  2;
        unsigned int POLICY_30                                                    :  2;
        unsigned int POLICY_31                                                    :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TC_CFG_L2_STORE_POLICY0 {
    struct {
        unsigned int POLICY_0                                                     :  2;
        unsigned int POLICY_1                                                     :  2;
        unsigned int POLICY_2                                                     :  2;
        unsigned int POLICY_3                                                     :  2;
        unsigned int POLICY_4                                                     :  2;
        unsigned int POLICY_5                                                     :  2;
        unsigned int POLICY_6                                                     :  2;
        unsigned int POLICY_7                                                     :  2;
        unsigned int POLICY_8                                                     :  2;
        unsigned int POLICY_9                                                     :  2;
        unsigned int POLICY_10                                                    :  2;
        unsigned int POLICY_11                                                    :  2;
        unsigned int POLICY_12                                                    :  2;
        unsigned int POLICY_13                                                    :  2;
        unsigned int POLICY_14                                                    :  2;
        unsigned int POLICY_15                                                    :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TC_CFG_L2_STORE_POLICY1 {
    struct {
        unsigned int POLICY_16                                                    :  2;
        unsigned int POLICY_17                                                    :  2;
        unsigned int POLICY_18                                                    :  2;
        unsigned int POLICY_19                                                    :  2;
        unsigned int POLICY_20                                                    :  2;
        unsigned int POLICY_21                                                    :  2;
        unsigned int POLICY_22                                                    :  2;
        unsigned int POLICY_23                                                    :  2;
        unsigned int POLICY_24                                                    :  2;
        unsigned int POLICY_25                                                    :  2;
        unsigned int POLICY_26                                                    :  2;
        unsigned int POLICY_27                                                    :  2;
        unsigned int POLICY_28                                                    :  2;
        unsigned int POLICY_29                                                    :  2;
        unsigned int POLICY_30                                                    :  2;
        unsigned int POLICY_31                                                    :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TC_CFG_L2_VOLATILE {
    struct {
        unsigned int VOL                                                          :  4;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TD_CGTT_CTRL {
    struct {
        unsigned int ON_DELAY                                                     :  4;
        unsigned int OFF_HYSTERESIS                                               :  8;
        unsigned int                                                              :  4;
        unsigned int SOFT_STALL_OVERRIDE7                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE6                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE5                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE4                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE3                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE2                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE1                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE0                                         :  1;
        unsigned int SOFT_OVERRIDE7                                               :  1;
        unsigned int SOFT_OVERRIDE6                                               :  1;
        unsigned int SOFT_OVERRIDE5                                               :  1;
        unsigned int SOFT_OVERRIDE4                                               :  1;
        unsigned int SOFT_OVERRIDE3                                               :  1;
        unsigned int SOFT_OVERRIDE2                                               :  1;
        unsigned int SOFT_OVERRIDE1                                               :  1;
        unsigned int SOFT_OVERRIDE0                                               :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TD_CNTL {
    struct {
        unsigned int                                                              : 16;
        unsigned int GATHER4_FLOAT_MODE                                           :  1;
        unsigned int                                                              :  3;
        unsigned int DISABLE_POWER_THROTTLE                                       :  1;
        unsigned int                                                              : 11;
    } bits, bitfields;
    struct {
        unsigned int SYNC_PHASE_SH                                                :  2;
        unsigned int                                                              :  2;
        unsigned int SYNC_PHASE_VC_SMX                                            :  2;
        unsigned int                                                              :  2;
        unsigned int PAD_STALL_EN                                                 :  1;
        unsigned int EXTEND_LDS_STALL                                             :  2;
        unsigned int LDS_STALL_PHASE_ADJUST                                       :  2;
        unsigned int                                                              :  2;
        unsigned int PRECISION_COMPATIBILITY                                      :  1;
        unsigned int                                                              :  2;
        unsigned int LD_FLOAT_MODE                                                :  1;
        unsigned int GATHER4_DX9_MODE                                             :  1;
        unsigned int                                                              :  1;
        unsigned int ENABLE_ROUND_TO_ZERO                                         :  1;
        unsigned int                                                              :  1;
        unsigned int DISABLE_2BIT_SIGNED_FORMAT                                   :  1;
        unsigned int                                                              :  8;
    } core;
    struct {
        unsigned int                                                              : 24;
        unsigned int DISABLE_MM_QNAN_COMPARE_RESULT                               :  1;
        unsigned int                                                              :  7;
    } gfx09;
    struct {
        unsigned int                                                              :  3;
        unsigned int DISABLE_SAMPLER_NEG_SCALED_NUM_CLAMP                         :  1;
        unsigned int                                                              :  2;
        unsigned int DISABLE_SAMPLER_MAX_NORM_NUM_CLAMP                           :  1;
        unsigned int                                                              : 18;
        unsigned int ARBITER_OLDEST_PRIORITY                                      :  1;
        unsigned int                                                              :  6;
    } gfx10;
    struct {
        unsigned int                                                              : 22;
        unsigned int DISABLE_ROUND_TO_ZERO_FOR_LARGE_FLOAT_TO_SMALL_FLOAT         :  1;
        unsigned int                                                              :  9;
    } gfx101Plus;
    struct {
        unsigned int                                                              : 26;
        unsigned int DONE_SCOREBOARD_DEPTH                                        :  5;
        unsigned int                                                              :  1;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TD_DEBUG_DATA {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TD_DEBUG_INDEX {
    struct {
        unsigned int INDEX                                                        :  5;
        unsigned int                                                              : 27;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TD_DSM_CNTL {
    struct {
        unsigned int TD_SS_FIFO_LO_DSM_IRRITATOR_DATA                             :  2;
        unsigned int TD_SS_FIFO_LO_ENABLE_SINGLE_WRITE                            :  1;
        unsigned int TD_SS_FIFO_HI_DSM_IRRITATOR_DATA                             :  2;
        unsigned int TD_SS_FIFO_HI_ENABLE_SINGLE_WRITE                            :  1;
        unsigned int TD_CS_FIFO_DSM_IRRITATOR_DATA                                :  2;
        unsigned int TD_CS_FIFO_ENABLE_SINGLE_WRITE                               :  1;
        unsigned int                                                              : 23;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TD_DSM_CNTL2 {
    struct {
        unsigned int TD_SS_FIFO_LO_ENABLE_ERROR_INJECT                            :  2;
        unsigned int TD_SS_FIFO_LO_SELECT_INJECT_DELAY                            :  1;
        unsigned int TD_SS_FIFO_HI_ENABLE_ERROR_INJECT                            :  2;
        unsigned int TD_SS_FIFO_HI_SELECT_INJECT_DELAY                            :  1;
        unsigned int TD_CS_FIFO_ENABLE_ERROR_INJECT                               :  2;
        unsigned int TD_CS_FIFO_SELECT_INJECT_DELAY                               :  1;
        unsigned int                                                              : 17;
        unsigned int TD_INJECT_DELAY                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TD_EDC_CNT {
    struct {
        unsigned int SS_FIFO_LO_SEC_COUNT                                         :  2;
        unsigned int SS_FIFO_LO_DED_COUNT                                         :  2;
        unsigned int SS_FIFO_HI_SEC_COUNT                                         :  2;
        unsigned int SS_FIFO_HI_DED_COUNT                                         :  2;
        unsigned int CS_FIFO_SED_COUNT                                            :  2;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TD_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TD_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TD_PERFCOUNTER0_SELECT {
    struct {
        unsigned int                                                              : 20;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int                                                              :  2;
        unsigned int PERF_SEL1                                                    :  8;
        unsigned int                                                              : 14;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TD_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int                                                              : 24;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;
    struct {
        unsigned int PERF_SEL2                                                    :  8;
        unsigned int                                                              :  2;
        unsigned int PERF_SEL3                                                    :  8;
        unsigned int                                                              : 14;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TD_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TD_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TD_PERFCOUNTER1_SELECT {
    struct {
        unsigned int                                                              : 20;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int                                                              : 24;
    } most;
    struct {
        unsigned int                                                              : 10;
        unsigned int PERF_SEL1                                                    :  8;
        unsigned int                                                              :  6;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int                                                              :  4;
    } gfx09_0;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TD_POWER_CNTL {
    struct {
        unsigned int FORCE_SAMPLER_CLK_TO_CORE                                    :  1;
        unsigned int FORCE_NOFILTER_CLK_TO_CORE                                   :  1;
        unsigned int SAMPLER_CLK_VALID_DELAY                                      :  3;
        unsigned int NOFILTER_CLK_VALID_DELAY                                     :  3;
        unsigned int DISABLE_NOFILTER_FORMATTER_POWER_OPT                         :  1;
        unsigned int FORCE_NOFILTER_D16_FORMATTERS_ON                             :  1;
        unsigned int                                                              : 22;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TD_SCRATCH {
    struct {
        unsigned int SCRATCH                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TD_STATUS {
    struct {
        unsigned int                                                              : 31;
        unsigned int BUSY                                                         :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH0_PerfMonCtl1 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH0_PerfMonCtl2 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH0_PerfMonCtl3 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH0_PerfMonCtl4 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH0_PerfMonCtl5 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH0_PerfMonCtlClk {
    struct {
        unsigned int GlblResetMsk                                                 :  6;
        unsigned int                                                              : 18;
        unsigned int GlblReset                                                    :  1;
        unsigned int GlblMonEn                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int CtrClkEn                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH0_PerfMonCtr1_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH0_PerfMonCtr1_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH0_PerfMonCtr2_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH0_PerfMonCtr2_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH0_PerfMonCtr3_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH0_PerfMonCtr3_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH0_PerfMonCtr4_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH0_PerfMonCtr4_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH0_PerfMonCtr5_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH0_PerfMonCtr5_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH0_PerfMonCtrClk_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH0_PerfMonCtrClk_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH1_PerfMonCtl1 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH1_PerfMonCtl2 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH1_PerfMonCtl3 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH1_PerfMonCtl4 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH1_PerfMonCtl5 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH1_PerfMonCtlClk {
    struct {
        unsigned int GlblResetMsk                                                 :  6;
        unsigned int                                                              : 18;
        unsigned int GlblReset                                                    :  1;
        unsigned int GlblMonEn                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int CtrClkEn                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH1_PerfMonCtr1_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH1_PerfMonCtr1_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH1_PerfMonCtr2_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH1_PerfMonCtr2_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH1_PerfMonCtr3_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH1_PerfMonCtr3_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH1_PerfMonCtr4_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH1_PerfMonCtr4_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH1_PerfMonCtr5_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH1_PerfMonCtr5_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH1_PerfMonCtrClk_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH1_PerfMonCtrClk_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH2_PerfMonCtl1 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH2_PerfMonCtl2 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH2_PerfMonCtl3 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH2_PerfMonCtl4 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH2_PerfMonCtl5 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH2_PerfMonCtlClk {
    struct {
        unsigned int GlblResetMsk                                                 :  6;
        unsigned int                                                              : 18;
        unsigned int GlblReset                                                    :  1;
        unsigned int GlblMonEn                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int CtrClkEn                                                     :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH2_PerfMonCtr1_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH2_PerfMonCtr1_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH2_PerfMonCtr2_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH2_PerfMonCtr2_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH2_PerfMonCtr3_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH2_PerfMonCtr3_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH2_PerfMonCtr4_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH2_PerfMonCtr4_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH2_PerfMonCtr5_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH2_PerfMonCtr5_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH2_PerfMonCtrClk_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH2_PerfMonCtrClk_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH3_PerfMonCtl1 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH3_PerfMonCtl2 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH3_PerfMonCtl3 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH3_PerfMonCtl4 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH3_PerfMonCtl5 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH3_PerfMonCtlClk {
    struct {
        unsigned int GlblResetMsk                                                 :  6;
        unsigned int                                                              : 18;
        unsigned int GlblReset                                                    :  1;
        unsigned int GlblMonEn                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int CtrClkEn                                                     :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH3_PerfMonCtr1_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH3_PerfMonCtr1_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH3_PerfMonCtr2_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH3_PerfMonCtr2_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH3_PerfMonCtr3_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH3_PerfMonCtr3_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH3_PerfMonCtr4_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH3_PerfMonCtr4_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH3_PerfMonCtr5_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH3_PerfMonCtr5_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH3_PerfMonCtrClk_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH3_PerfMonCtrClk_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH4_PerfMonCtl1 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH4_PerfMonCtl2 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH4_PerfMonCtl3 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH4_PerfMonCtl4 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH4_PerfMonCtl5 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH4_PerfMonCtlClk {
    struct {
        unsigned int GlblResetMsk                                                 :  6;
        unsigned int                                                              : 18;
        unsigned int GlblReset                                                    :  1;
        unsigned int GlblMonEn                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int CtrClkEn                                                     :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH4_PerfMonCtr1_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH4_PerfMonCtr1_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH4_PerfMonCtr2_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH4_PerfMonCtr2_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH4_PerfMonCtr3_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH4_PerfMonCtr3_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH4_PerfMonCtr4_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH4_PerfMonCtr4_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH4_PerfMonCtr5_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH4_PerfMonCtr5_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH4_PerfMonCtrClk_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH4_PerfMonCtrClk_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH5_PerfMonCtl1 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH5_PerfMonCtl2 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH5_PerfMonCtl3 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH5_PerfMonCtl4 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH5_PerfMonCtl5 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH5_PerfMonCtlClk {
    struct {
        unsigned int GlblResetMsk                                                 :  6;
        unsigned int                                                              : 18;
        unsigned int GlblReset                                                    :  1;
        unsigned int GlblMonEn                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int CtrClkEn                                                     :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH5_PerfMonCtr1_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH5_PerfMonCtr1_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH5_PerfMonCtr2_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH5_PerfMonCtr2_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH5_PerfMonCtr3_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH5_PerfMonCtr3_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH5_PerfMonCtr4_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH5_PerfMonCtr4_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH5_PerfMonCtr5_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH5_PerfMonCtr5_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH5_PerfMonCtrClk_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH5_PerfMonCtrClk_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH6_PerfMonCtl1 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH6_PerfMonCtl2 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH6_PerfMonCtl3 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH6_PerfMonCtl4 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH6_PerfMonCtl5 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH6_PerfMonCtlClk {
    struct {
        unsigned int GlblResetMsk                                                 :  6;
        unsigned int                                                              : 18;
        unsigned int GlblReset                                                    :  1;
        unsigned int GlblMonEn                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int CtrClkEn                                                     :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH6_PerfMonCtr1_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH6_PerfMonCtr1_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH6_PerfMonCtr2_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH6_PerfMonCtr2_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH6_PerfMonCtr3_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH6_PerfMonCtr3_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH6_PerfMonCtr4_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH6_PerfMonCtr4_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH6_PerfMonCtr5_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH6_PerfMonCtr5_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH6_PerfMonCtrClk_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH6_PerfMonCtrClk_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH7_PerfMonCtl1 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH7_PerfMonCtl2 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH7_PerfMonCtl3 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH7_PerfMonCtl4 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH7_PerfMonCtl5 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH7_PerfMonCtlClk {
    struct {
        unsigned int GlblResetMsk                                                 :  6;
        unsigned int                                                              : 18;
        unsigned int GlblReset                                                    :  1;
        unsigned int GlblMonEn                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int CtrClkEn                                                     :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH7_PerfMonCtr1_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH7_PerfMonCtr1_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH7_PerfMonCtr2_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH7_PerfMonCtr2_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH7_PerfMonCtr3_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH7_PerfMonCtr3_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH7_PerfMonCtr4_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH7_PerfMonCtr4_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH7_PerfMonCtr5_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH7_PerfMonCtr5_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH7_PerfMonCtrClk_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH7_PerfMonCtrClk_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH8_PerfMonCtl1 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH8_PerfMonCtl2 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH8_PerfMonCtl3 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH8_PerfMonCtl4 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH8_PerfMonCtl5 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH8_PerfMonCtlClk {
    struct {
        unsigned int GlblResetMsk                                                 :  6;
        unsigned int                                                              : 18;
        unsigned int GlblReset                                                    :  1;
        unsigned int GlblMonEn                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int CtrClkEn                                                     :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH8_PerfMonCtr1_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH8_PerfMonCtr1_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH8_PerfMonCtr2_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH8_PerfMonCtr2_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH8_PerfMonCtr3_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH8_PerfMonCtr3_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH8_PerfMonCtr4_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH8_PerfMonCtr4_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH8_PerfMonCtr5_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH8_PerfMonCtr5_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH8_PerfMonCtrClk_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH8_PerfMonCtrClk_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH9_PerfMonCtl1 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH9_PerfMonCtl2 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH9_PerfMonCtl3 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH9_PerfMonCtl4 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH9_PerfMonCtl5 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH9_PerfMonCtlClk {
    struct {
        unsigned int GlblResetMsk                                                 :  6;
        unsigned int                                                              : 18;
        unsigned int GlblReset                                                    :  1;
        unsigned int GlblMonEn                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int CtrClkEn                                                     :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH9_PerfMonCtr1_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH9_PerfMonCtr1_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH9_PerfMonCtr2_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH9_PerfMonCtr2_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH9_PerfMonCtr3_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH9_PerfMonCtr3_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH9_PerfMonCtr4_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH9_PerfMonCtr4_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH9_PerfMonCtr5_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH9_PerfMonCtr5_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH9_PerfMonCtrClk_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH9_PerfMonCtrClk_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH10_PerfMonCtl1 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH10_PerfMonCtl2 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH10_PerfMonCtl3 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH10_PerfMonCtl4 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH10_PerfMonCtl5 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH10_PerfMonCtlClk {
    struct {
        unsigned int GlblResetMsk                                                 :  6;
        unsigned int                                                              : 18;
        unsigned int GlblReset                                                    :  1;
        unsigned int GlblMonEn                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int CtrClkEn                                                     :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH10_PerfMonCtr1_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH10_PerfMonCtr1_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH10_PerfMonCtr2_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH10_PerfMonCtr2_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH10_PerfMonCtr3_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH10_PerfMonCtr3_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH10_PerfMonCtr4_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH10_PerfMonCtr4_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH10_PerfMonCtr5_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH10_PerfMonCtr5_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH10_PerfMonCtrClk_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH10_PerfMonCtrClk_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH11_PerfMonCtl1 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH11_PerfMonCtl2 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH11_PerfMonCtl3 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH11_PerfMonCtl4 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH11_PerfMonCtl5 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH11_PerfMonCtlClk {
    struct {
        unsigned int GlblResetMsk                                                 :  6;
        unsigned int                                                              : 18;
        unsigned int GlblReset                                                    :  1;
        unsigned int GlblMonEn                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int CtrClkEn                                                     :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH11_PerfMonCtr1_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH11_PerfMonCtr1_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH11_PerfMonCtr2_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH11_PerfMonCtr2_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH11_PerfMonCtr3_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH11_PerfMonCtr3_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH11_PerfMonCtr4_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH11_PerfMonCtr4_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH11_PerfMonCtr5_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH11_PerfMonCtr5_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH11_PerfMonCtrClk_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH11_PerfMonCtrClk_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH12_PerfMonCtl1 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH12_PerfMonCtl2 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH12_PerfMonCtl3 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH12_PerfMonCtl4 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH12_PerfMonCtl5 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH12_PerfMonCtlClk {
    struct {
        unsigned int GlblResetMsk                                                 :  6;
        unsigned int                                                              : 18;
        unsigned int GlblReset                                                    :  1;
        unsigned int GlblMonEn                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int CtrClkEn                                                     :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH12_PerfMonCtr1_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH12_PerfMonCtr1_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH12_PerfMonCtr2_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH12_PerfMonCtr2_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH12_PerfMonCtr3_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH12_PerfMonCtr3_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH12_PerfMonCtr4_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH12_PerfMonCtr4_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH12_PerfMonCtr5_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH12_PerfMonCtr5_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH12_PerfMonCtrClk_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH12_PerfMonCtrClk_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH13_PerfMonCtl1 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH13_PerfMonCtl2 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH13_PerfMonCtl3 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH13_PerfMonCtl4 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH13_PerfMonCtl5 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH13_PerfMonCtlClk {
    struct {
        unsigned int GlblResetMsk                                                 :  6;
        unsigned int                                                              : 18;
        unsigned int GlblReset                                                    :  1;
        unsigned int GlblMonEn                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int CtrClkEn                                                     :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH13_PerfMonCtr1_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH13_PerfMonCtr1_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH13_PerfMonCtr2_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH13_PerfMonCtr2_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH13_PerfMonCtr3_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH13_PerfMonCtr3_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH13_PerfMonCtr4_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH13_PerfMonCtr4_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH13_PerfMonCtr5_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH13_PerfMonCtr5_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH13_PerfMonCtrClk_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH13_PerfMonCtrClk_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH14_PerfMonCtl1 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH14_PerfMonCtl2 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH14_PerfMonCtl3 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH14_PerfMonCtl4 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH14_PerfMonCtl5 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH14_PerfMonCtlClk {
    struct {
        unsigned int GlblResetMsk                                                 :  6;
        unsigned int                                                              : 18;
        unsigned int GlblReset                                                    :  1;
        unsigned int GlblMonEn                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int CtrClkEn                                                     :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH14_PerfMonCtr1_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH14_PerfMonCtr1_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH14_PerfMonCtr2_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH14_PerfMonCtr2_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH14_PerfMonCtr3_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH14_PerfMonCtr3_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH14_PerfMonCtr4_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH14_PerfMonCtr4_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH14_PerfMonCtr5_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH14_PerfMonCtr5_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH14_PerfMonCtrClk_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH14_PerfMonCtrClk_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH15_PerfMonCtl1 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH15_PerfMonCtl2 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH15_PerfMonCtl3 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH15_PerfMonCtl4 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH15_PerfMonCtl5 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH15_PerfMonCtlClk {
    struct {
        unsigned int GlblResetMsk                                                 :  6;
        unsigned int                                                              : 18;
        unsigned int GlblReset                                                    :  1;
        unsigned int GlblMonEn                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int CtrClkEn                                                     :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH15_PerfMonCtr1_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH15_PerfMonCtr1_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH15_PerfMonCtr2_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH15_PerfMonCtr2_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH15_PerfMonCtr3_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH15_PerfMonCtr3_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH15_PerfMonCtr4_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH15_PerfMonCtr4_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH15_PerfMonCtr5_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH15_PerfMonCtr5_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH15_PerfMonCtrClk_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH15_PerfMonCtrClk_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH16_PerfMonCtl1 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH16_PerfMonCtl2 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH16_PerfMonCtl3 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH16_PerfMonCtl4 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH16_PerfMonCtl5 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH16_PerfMonCtlClk {
    struct {
        unsigned int GlblResetMsk                                                 :  6;
        unsigned int                                                              : 18;
        unsigned int GlblReset                                                    :  1;
        unsigned int GlblMonEn                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int CtrClkEn                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH16_PerfMonCtr1_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH16_PerfMonCtr1_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH16_PerfMonCtr2_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH16_PerfMonCtr2_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH16_PerfMonCtr3_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH16_PerfMonCtr3_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH16_PerfMonCtr4_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH16_PerfMonCtr4_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH16_PerfMonCtr5_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH16_PerfMonCtr5_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH16_PerfMonCtrClk_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH16_PerfMonCtrClk_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH17_PerfMonCtl1 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH17_PerfMonCtl2 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH17_PerfMonCtl3 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH17_PerfMonCtl4 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH17_PerfMonCtl5 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH17_PerfMonCtlClk {
    struct {
        unsigned int GlblResetMsk                                                 :  6;
        unsigned int                                                              : 18;
        unsigned int GlblReset                                                    :  1;
        unsigned int GlblMonEn                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int CtrClkEn                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH17_PerfMonCtr1_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH17_PerfMonCtr1_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH17_PerfMonCtr2_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH17_PerfMonCtr2_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH17_PerfMonCtr3_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH17_PerfMonCtr3_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH17_PerfMonCtr4_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH17_PerfMonCtr4_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH17_PerfMonCtr5_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH17_PerfMonCtr5_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH17_PerfMonCtrClk_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH17_PerfMonCtrClk_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH18_PerfMonCtl1 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH18_PerfMonCtl2 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH18_PerfMonCtl3 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH18_PerfMonCtl4 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH18_PerfMonCtl5 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH18_PerfMonCtlClk {
    struct {
        unsigned int GlblResetMsk                                                 :  6;
        unsigned int                                                              : 18;
        unsigned int GlblReset                                                    :  1;
        unsigned int GlblMonEn                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int CtrClkEn                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH18_PerfMonCtr1_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH18_PerfMonCtr1_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH18_PerfMonCtr2_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH18_PerfMonCtr2_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH18_PerfMonCtr3_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH18_PerfMonCtr3_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH18_PerfMonCtr4_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH18_PerfMonCtr4_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH18_PerfMonCtr5_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH18_PerfMonCtr5_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH18_PerfMonCtrClk_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH18_PerfMonCtrClk_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH19_PerfMonCtl1 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH19_PerfMonCtl2 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH19_PerfMonCtl3 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH19_PerfMonCtl4 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH19_PerfMonCtl5 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH19_PerfMonCtlClk {
    struct {
        unsigned int GlblResetMsk                                                 :  6;
        unsigned int                                                              : 18;
        unsigned int GlblReset                                                    :  1;
        unsigned int GlblMonEn                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int CtrClkEn                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH19_PerfMonCtr1_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH19_PerfMonCtr1_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH19_PerfMonCtr2_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH19_PerfMonCtr2_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH19_PerfMonCtr3_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH19_PerfMonCtr3_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH19_PerfMonCtr4_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH19_PerfMonCtr4_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH19_PerfMonCtr5_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH19_PerfMonCtr5_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH19_PerfMonCtrClk_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH19_PerfMonCtrClk_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH20_PerfMonCtl1 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH20_PerfMonCtl2 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH20_PerfMonCtl3 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH20_PerfMonCtl4 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH20_PerfMonCtl5 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH20_PerfMonCtlClk {
    struct {
        unsigned int GlblResetMsk                                                 :  6;
        unsigned int                                                              : 18;
        unsigned int GlblReset                                                    :  1;
        unsigned int GlblMonEn                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int CtrClkEn                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH20_PerfMonCtr1_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH20_PerfMonCtr1_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH20_PerfMonCtr2_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH20_PerfMonCtr2_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH20_PerfMonCtr3_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH20_PerfMonCtr3_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH20_PerfMonCtr4_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH20_PerfMonCtr4_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH20_PerfMonCtr5_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH20_PerfMonCtr5_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH20_PerfMonCtrClk_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH20_PerfMonCtrClk_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH21_PerfMonCtl1 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH21_PerfMonCtl2 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH21_PerfMonCtl3 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH21_PerfMonCtl4 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH21_PerfMonCtl5 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH21_PerfMonCtlClk {
    struct {
        unsigned int GlblResetMsk                                                 :  6;
        unsigned int                                                              : 18;
        unsigned int GlblReset                                                    :  1;
        unsigned int GlblMonEn                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int CtrClkEn                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH21_PerfMonCtr1_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH21_PerfMonCtr1_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH21_PerfMonCtr2_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH21_PerfMonCtr2_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH21_PerfMonCtr3_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH21_PerfMonCtr3_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH21_PerfMonCtr4_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH21_PerfMonCtr4_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH21_PerfMonCtr5_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH21_PerfMonCtr5_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH21_PerfMonCtrClk_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH21_PerfMonCtrClk_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH22_PerfMonCtl1 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH22_PerfMonCtl2 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH22_PerfMonCtl3 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH22_PerfMonCtl4 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH22_PerfMonCtl5 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH22_PerfMonCtlClk {
    struct {
        unsigned int GlblResetMsk                                                 :  6;
        unsigned int                                                              : 18;
        unsigned int GlblReset                                                    :  1;
        unsigned int GlblMonEn                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int CtrClkEn                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH22_PerfMonCtr1_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH22_PerfMonCtr1_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH22_PerfMonCtr2_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH22_PerfMonCtr2_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH22_PerfMonCtr3_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH22_PerfMonCtr3_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH22_PerfMonCtr4_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH22_PerfMonCtr4_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH22_PerfMonCtr5_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH22_PerfMonCtr5_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH22_PerfMonCtrClk_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH22_PerfMonCtrClk_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH23_PerfMonCtl1 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH23_PerfMonCtl2 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH23_PerfMonCtl3 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH23_PerfMonCtl4 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH23_PerfMonCtl5 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH23_PerfMonCtlClk {
    struct {
        unsigned int GlblResetMsk                                                 :  6;
        unsigned int                                                              : 18;
        unsigned int GlblReset                                                    :  1;
        unsigned int GlblMonEn                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int CtrClkEn                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH23_PerfMonCtr1_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH23_PerfMonCtr1_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH23_PerfMonCtr2_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH23_PerfMonCtr2_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH23_PerfMonCtr3_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH23_PerfMonCtr3_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH23_PerfMonCtr4_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH23_PerfMonCtr4_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH23_PerfMonCtr5_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH23_PerfMonCtr5_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH23_PerfMonCtrClk_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH23_PerfMonCtrClk_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH24_PerfMonCtl1 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH24_PerfMonCtl2 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH24_PerfMonCtl3 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH24_PerfMonCtl4 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH24_PerfMonCtl5 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH24_PerfMonCtlClk {
    struct {
        unsigned int GlblResetMsk                                                 :  6;
        unsigned int                                                              : 18;
        unsigned int GlblReset                                                    :  1;
        unsigned int GlblMonEn                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int CtrClkEn                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH24_PerfMonCtr1_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH24_PerfMonCtr1_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH24_PerfMonCtr2_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH24_PerfMonCtr2_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH24_PerfMonCtr3_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH24_PerfMonCtr3_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH24_PerfMonCtr4_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH24_PerfMonCtr4_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH24_PerfMonCtr5_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH24_PerfMonCtr5_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH24_PerfMonCtrClk_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH24_PerfMonCtrClk_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH25_PerfMonCtl1 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH25_PerfMonCtl2 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH25_PerfMonCtl3 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH25_PerfMonCtl4 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH25_PerfMonCtl5 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH25_PerfMonCtlClk {
    struct {
        unsigned int GlblResetMsk                                                 :  6;
        unsigned int                                                              : 18;
        unsigned int GlblReset                                                    :  1;
        unsigned int GlblMonEn                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int CtrClkEn                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH25_PerfMonCtr1_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH25_PerfMonCtr1_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH25_PerfMonCtr2_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH25_PerfMonCtr2_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH25_PerfMonCtr3_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH25_PerfMonCtr3_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH25_PerfMonCtr4_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH25_PerfMonCtr4_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH25_PerfMonCtr5_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH25_PerfMonCtr5_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH25_PerfMonCtrClk_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH25_PerfMonCtrClk_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH26_PerfMonCtl1 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH26_PerfMonCtl2 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH26_PerfMonCtl3 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH26_PerfMonCtl4 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH26_PerfMonCtl5 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH26_PerfMonCtlClk {
    struct {
        unsigned int GlblResetMsk                                                 :  6;
        unsigned int                                                              : 18;
        unsigned int GlblReset                                                    :  1;
        unsigned int GlblMonEn                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int CtrClkEn                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH26_PerfMonCtr1_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH26_PerfMonCtr1_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH26_PerfMonCtr2_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH26_PerfMonCtr2_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH26_PerfMonCtr3_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH26_PerfMonCtr3_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH26_PerfMonCtr4_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH26_PerfMonCtr4_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH26_PerfMonCtr5_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH26_PerfMonCtr5_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH26_PerfMonCtrClk_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH26_PerfMonCtrClk_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH27_PerfMonCtl1 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH27_PerfMonCtl2 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH27_PerfMonCtl3 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH27_PerfMonCtl4 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH27_PerfMonCtl5 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH27_PerfMonCtlClk {
    struct {
        unsigned int GlblResetMsk                                                 :  6;
        unsigned int                                                              : 18;
        unsigned int GlblReset                                                    :  1;
        unsigned int GlblMonEn                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int CtrClkEn                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH27_PerfMonCtr1_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH27_PerfMonCtr1_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH27_PerfMonCtr2_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH27_PerfMonCtr2_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH27_PerfMonCtr3_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH27_PerfMonCtr3_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH27_PerfMonCtr4_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH27_PerfMonCtr4_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH27_PerfMonCtr5_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH27_PerfMonCtr5_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH27_PerfMonCtrClk_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH27_PerfMonCtrClk_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH28_PerfMonCtl1 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH28_PerfMonCtl2 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH28_PerfMonCtl3 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH28_PerfMonCtl4 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH28_PerfMonCtl5 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH28_PerfMonCtlClk {
    struct {
        unsigned int GlblResetMsk                                                 :  6;
        unsigned int                                                              : 18;
        unsigned int GlblReset                                                    :  1;
        unsigned int GlblMonEn                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int CtrClkEn                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH28_PerfMonCtr1_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH28_PerfMonCtr1_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH28_PerfMonCtr2_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH28_PerfMonCtr2_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH28_PerfMonCtr3_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH28_PerfMonCtr3_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH28_PerfMonCtr4_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH28_PerfMonCtr4_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH28_PerfMonCtr5_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH28_PerfMonCtr5_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH28_PerfMonCtrClk_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH28_PerfMonCtrClk_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH29_PerfMonCtl1 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH29_PerfMonCtl2 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH29_PerfMonCtl3 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH29_PerfMonCtl4 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH29_PerfMonCtl5 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH29_PerfMonCtlClk {
    struct {
        unsigned int GlblResetMsk                                                 :  6;
        unsigned int                                                              : 18;
        unsigned int GlblReset                                                    :  1;
        unsigned int GlblMonEn                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int CtrClkEn                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH29_PerfMonCtr1_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH29_PerfMonCtr1_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH29_PerfMonCtr2_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH29_PerfMonCtr2_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH29_PerfMonCtr3_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH29_PerfMonCtr3_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH29_PerfMonCtr4_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH29_PerfMonCtr4_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH29_PerfMonCtr5_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH29_PerfMonCtr5_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH29_PerfMonCtrClk_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH29_PerfMonCtrClk_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH30_PerfMonCtl1 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH30_PerfMonCtl2 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH30_PerfMonCtl3 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH30_PerfMonCtl4 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH30_PerfMonCtl5 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH30_PerfMonCtlClk {
    struct {
        unsigned int GlblResetMsk                                                 :  6;
        unsigned int                                                              : 18;
        unsigned int GlblReset                                                    :  1;
        unsigned int GlblMonEn                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int CtrClkEn                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH30_PerfMonCtr1_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH30_PerfMonCtr1_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH30_PerfMonCtr2_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH30_PerfMonCtr2_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH30_PerfMonCtr3_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH30_PerfMonCtr3_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH30_PerfMonCtr4_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH30_PerfMonCtr4_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH30_PerfMonCtr5_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH30_PerfMonCtr5_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH30_PerfMonCtrClk_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH30_PerfMonCtrClk_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH31_PerfMonCtl1 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH31_PerfMonCtl2 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH31_PerfMonCtl3 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH31_PerfMonCtl4 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH31_PerfMonCtl5 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int ChipSelMask                                                  :  4;
        unsigned int ChipIDSel                                                    :  4;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH31_PerfMonCtlClk {
    struct {
        unsigned int GlblResetMsk                                                 :  6;
        unsigned int                                                              : 18;
        unsigned int GlblReset                                                    :  1;
        unsigned int GlblMonEn                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int CtrClkEn                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH31_PerfMonCtr1_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH31_PerfMonCtr1_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH31_PerfMonCtr2_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH31_PerfMonCtr2_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH31_PerfMonCtr3_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH31_PerfMonCtr3_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH31_PerfMonCtr4_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH31_PerfMonCtr4_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH31_PerfMonCtr5_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH31_PerfMonCtr5_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH31_PerfMonCtrClk_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UMCCH31_PerfMonCtrClk_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UTCL1_ALOG {
    struct {
        unsigned int UTCL1_ALOG_MODE1_FILTER1_THRESHOLD                           :  3;
        unsigned int UTCL1_ALOG_MODE1_FILTER2_BYPASS                              :  1;
        unsigned int UTCL1_ALOG_ACTIVE                                            :  1;
        unsigned int UTCL1_ALOG_MODE                                              :  1;
        unsigned int UTCL1_ALOG_MODE2_LOCK_WINDOW                                 :  3;
        unsigned int                                                              :  1;
        unsigned int UTCL1_ALOG_MODE2_INTR_THRESHOLD                              :  2;
        unsigned int UTCL1_ALOG_SPACE_EN                                          :  3;
        unsigned int UTCL1_ALOG_CLEAN                                             :  1;
        unsigned int UTCL1_ALOG_IDLE                                              :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  9;
        unsigned int UTCL1_ALOG_ONLY_MISS                                         :  1;
        unsigned int                                                              :  7;
        unsigned int UTCL1_ALOG_TRACK_SEGMENT_SIZE                                :  6;
        unsigned int UTCL1_ALOG_MODE1_FILTER1_BYPASS                              :  1;
        unsigned int UTCL1_ALOG_MODE1_INTR_ON_ALLOC                               :  1;
        unsigned int                                                              :  7;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UTCL1_CGTT_CLK_CTRL {
    struct {
        unsigned int ON_DELAY                                                     :  4;
        unsigned int OFF_HYSTERESIS                                               :  8;
        unsigned int                                                              :  4;
        unsigned int SOFT_STALL_OVERRIDE7                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE6                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE5                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE4                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE3                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE2                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE1                                         :  1;
        unsigned int SOFT_STALL_OVERRIDE0                                         :  1;
        unsigned int                                                              :  1;
        unsigned int SOFT_OVERRIDE6                                               :  1;
        unsigned int SOFT_OVERRIDE5                                               :  1;
        unsigned int SOFT_OVERRIDE4                                               :  1;
        unsigned int SOFT_OVERRIDE3                                               :  1;
        unsigned int SOFT_OVERRIDE2                                               :  1;
        unsigned int SOFT_OVERRIDE1                                               :  1;
        unsigned int SOFT_OVERRIDE0                                               :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UTCL1_CTRL {
    struct {
        unsigned int UTCL1_SMALL_PAGE_SIZE                                        :  1;
        unsigned int UTCL1_LARGE_PAGE_SIZE                                        :  1;
        unsigned int UTCL1_CACHE_CORE_BYPASS                                      :  1;
        unsigned int UTCL1_TCP_BYPASS                                             :  1;
        unsigned int UTCL1_SQCI_BYPASS                                            :  1;
        unsigned int UTCL1_SQCD_BYPASS                                            :  1;
        unsigned int UTCL1_RMI_BYPASS                                             :  1;
        unsigned int UTCL1_SQG_BYPASS                                             :  1;
        unsigned int UTCL1_RMI_DEDICATED_CACHE_CORE                               :  1;
        unsigned int UTCL1_FORCE_RANGE_INV_TO_VMID                                :  1;
        unsigned int UTCL1_FORCE_INV_ALL                                          :  1;
        unsigned int UTCL1_FORCE_INV_ALL_DONE                                     :  1;
        unsigned int UTCL1_UTCL2_FGCG_REPEATERS_OVERRIDE                          :  1;
        unsigned int UTCL1_INV_FILTER_2M                                          :  1;
        unsigned int UTCL1_RANGE_INV_FORCE_CHK_ALL                                :  1;
        unsigned int                                                              :  3;
        unsigned int UTCL1_MH_INV_FRAG_SIZE_OVERRIDE                              :  1;
        unsigned int UTCL1_CACHE_WRITE_PERM                                       :  1;
        unsigned int UTCL1_MH_CAM_DUPLICATE_4K_FILTER                             :  1;
        unsigned int UTCL1_MH_DISABLE_DUPLICATES                                  :  1;
        unsigned int                                                              :  1;
        unsigned int UTCL1_MH_DISABLE_RECENT_BUFFER                               :  1;
        unsigned int UTCL1_MISS_CC_PRIORITY                                       :  2;
        unsigned int UTCL1_REDUCE_CC_SIZE                                         :  2;
        unsigned int                                                              :  4;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 15;
        unsigned int RESERVED                                                     :  3;
        unsigned int                                                              : 14;
    } gfx101;
    struct {
        unsigned int                                                              : 22;
        unsigned int UTCL1_MH_DISABLE_REQUEST_SQUASHING                           :  1;
        unsigned int                                                              :  5;
        unsigned int UTCL1_REDUCE_MH_CFIFO_SIZE                                   :  2;
        unsigned int UTCL1_REDUCE_MH_CAM_SIZE                                     :  2;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UTCL1_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UTCL1_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UTCL1_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 18;
        unsigned int COUNTER_MODE                                                 :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UTCL1_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UTCL1_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UTCL1_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 18;
        unsigned int COUNTER_MODE                                                 :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UTCL1_UTCL0_INVREQ_DISABLE {
    struct {
        unsigned int UTCL1_UTCL0_INVREQ_DISABLE                                   : 25;
        unsigned int                                                              :  7;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UTCL2_CGTT_CLK_CTRL {
    struct {
        unsigned int ON_DELAY                                                     :  4;
        unsigned int OFF_HYSTERESIS                                               :  8;
        unsigned int SOFT_OVERRIDE_EXTRA                                          :  3;
        unsigned int MGLS_OVERRIDE                                                :  1;
        unsigned int SOFT_STALL_OVERRIDE                                          :  8;
        unsigned int SOFT_OVERRIDE                                                :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_CACHE_INVALIDATION {
    struct {
        unsigned int CACHE_INVALIDATION                                           :  2;
        unsigned int                                                              :  2;
        unsigned int DIS_INSTANCING_OPT                                           :  1;
        unsigned int VS_NO_EXTRA_BUFFER                                           :  1;
        unsigned int AUTO_INVLD_EN                                                :  2;
        unsigned int                                                              :  1;
        unsigned int USE_GS_DONE                                                  :  1;
        unsigned int                                                              :  1;
        unsigned int DIS_RANGE_FULL_INVLD                                         :  1;
        unsigned int GS_LATE_ALLOC_EN                                             :  1;
        unsigned int STREAMOUT_FULL_FLUSH                                         :  1;
        unsigned int                                                              :  2;
        unsigned int ES_LIMIT                                                     :  5;
        unsigned int ENABLE_PING_PONG                                             :  1;
        unsigned int OPT_FLOW_CNTL_1                                              :  3;
        unsigned int OPT_FLOW_CNTL_2                                              :  3;
        unsigned int EN_WAVE_MERGE                                                :  1;
        unsigned int ENABLE_PING_PONG_EOI                                         :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_CNTL_STATUS {
    struct {
        unsigned int VGT_BUSY                                                     :  1;
        unsigned int VGT_OUT_INDX_BUSY                                            :  1;
        unsigned int VGT_OUT_BUSY                                                 :  1;
        unsigned int VGT_PT_BUSY                                                  :  1;
        unsigned int VGT_TE_BUSY                                                  :  1;
        unsigned int VGT_VR_BUSY                                                  :  1;
        unsigned int VGT_PI_BUSY                                                  :  1;
        unsigned int VGT_GS_BUSY                                                  :  1;
        unsigned int VGT_HS_BUSY                                                  :  1;
        unsigned int VGT_TE11_BUSY                                                :  1;
        unsigned int VGT_PRIMGEN_BUSY                                             :  1;
        unsigned int                                                              : 21;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DEBUG_DATA {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DEBUG_REG0 {
    struct {
        unsigned int                                                              :  2;
        unsigned int vgt_busy                                                     :  1;
        unsigned int                                                              :  3;
        unsigned int SPARE5                                                       :  1;
        unsigned int SPARE4                                                       :  1;
        unsigned int                                                              :  4;
        unsigned int gs_busy                                                      :  1;
        unsigned int                                                              :  3;
        unsigned int gog_busy                                                     :  1;
        unsigned int                                                              :  3;
        unsigned int SPARE3                                                       :  1;
        unsigned int                                                              :  4;
        unsigned int SPARE2                                                       :  1;
        unsigned int                                                              :  2;
        unsigned int SPARE1                                                       :  1;
        unsigned int sclk_core_vld                                                :  1;
        unsigned int sclk_gs_vld                                                  :  1;
        unsigned int SPARE0                                                       :  1;
    } rv1x;
    struct {
        unsigned int                                                              :  1;
        unsigned int RESERVED0                                                    :  1;
        unsigned int vgt_busy                                                     :  1;
        unsigned int RESERVED1                                                    :  5;
        unsigned int                                                              :  3;
        unsigned int pg_busy                                                      :  1;
        unsigned int gs_busy                                                      :  1;
        unsigned int                                                              :  3;
        unsigned int gog_busy                                                     :  1;
        unsigned int                                                              :  1;
        unsigned int frmt_pg_busy                                                 :  1;
        unsigned int                                                              :  1;
        unsigned int RESERVED2                                                    :  1;
        unsigned int                                                              :  4;
        unsigned int tell_clk_busy                                                :  1;
        unsigned int                                                              :  2;
        unsigned int sclk_tell_vid                                                :  1;
        unsigned int sclk_core_vld                                                :  1;
        unsigned int sclk_gs_vld                                                  :  1;
        unsigned int RESERVED3                                                    :  1;
    } rv2x;
    struct {
        unsigned int                                                              :  2;
        unsigned int vgt_busy                                                     :  1;
        unsigned int                                                              :  3;
        unsigned int SPARE5                                                       :  1;
        unsigned int SPARE4                                                       :  1;
        unsigned int                                                              :  4;
        unsigned int gs_busy                                                      :  1;
        unsigned int                                                              :  3;
        unsigned int gog_busy                                                     :  1;
        unsigned int                                                              :  3;
        unsigned int SPARE3                                                       :  1;
        unsigned int                                                              :  4;
        unsigned int SPARE2                                                       :  1;
        unsigned int                                                              :  2;
        unsigned int SPARE1                                                       :  1;
        unsigned int sclk_core_vld                                                :  1;
        unsigned int sclk_gs_vld                                                  :  1;
        unsigned int SPARE0                                                       :  1;
    } vega;
    struct {
        unsigned int                                                              : 11;
        unsigned int pg_busy                                                      :  1;
        unsigned int                                                              : 20;
    } vg12;
    struct {
        unsigned int                                                              : 11;
        unsigned int pg_busy                                                      :  1;
        unsigned int                                                              : 20;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DEBUG_REG1 {
    struct {
        unsigned int                                                              :  4;
        unsigned int SPARE5                                                       :  1;
        unsigned int SPARE4                                                       :  1;
        unsigned int SPARE3                                                       :  1;
        unsigned int SPARE2                                                       :  1;
        unsigned int SPARE1                                                       :  1;
        unsigned int SPARE0                                                       :  1;
        unsigned int                                                              : 22;
    } most;
    struct {
        unsigned int RESERVED0                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int RESERVED1                                                    :  2;
        unsigned int                                                              :  1;
        unsigned int RESERVED2                                                    :  1;
        unsigned int                                                              :  1;
        unsigned int RESERVED3                                                    :  1;
        unsigned int                                                              :  1;
        unsigned int RESERVED4                                                    :  1;
        unsigned int                                                              :  1;
        unsigned int RESERVED5                                                    :  1;
        unsigned int                                                              :  1;
        unsigned int RESERVED6                                                    :  1;
        unsigned int                                                              :  6;
    } rv2x;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DEBUG_REG2 {
    struct {
        unsigned int                                                              : 29;
        unsigned int SPARE                                                        :  3;
    } most;
    struct {
        unsigned int hs_busy_q                                                    :  1;
        unsigned int hs_noif_busy_q                                               :  1;
        unsigned int tfmmIsBusy_q                                                 :  1;
        unsigned int lsVertIfBusy_0_q                                             :  1;
        unsigned int RESERVED0                                                    :  2;
        unsigned int int_VGT_WD_packet_send_q                                     :  1;
        unsigned int grpModBusy_q                                                 :  1;
        unsigned int                                                              :  1;
        unsigned int RESERVED1                                                    :  1;
        unsigned int                                                              :  1;
        unsigned int hsTgFifoEmpty                                                :  1;
        unsigned int                                                              :  1;
        unsigned int RESERVED2                                                    :  1;
        unsigned int                                                              :  1;
        unsigned int RESERVED3                                                    :  1;
        unsigned int                                                              :  1;
        unsigned int hsTgFifoFull                                                 :  1;
        unsigned int                                                              :  1;
        unsigned int not_VGT_WD_thdgrp_rtr                                        :  1;
        unsigned int                                                              :  7;
        unsigned int RESERVED4                                                    :  5;
    } rv2x;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DEBUG_REG4 {
    struct {
        unsigned int                                                              : 31;
        unsigned int SPARE                                                        :  1;
    } most;
    struct {
        unsigned int                                                              : 29;
        unsigned int hsTgSendFlush                                                :  1;
        unsigned int                                                              :  1;
        unsigned int RESERVED0                                                    :  1;
    } rv2x;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DEBUG_REG5 {
    struct {
        unsigned int SPARE4                                                       :  3;
        unsigned int                                                              :  5;
        unsigned int SPARE3                                                       :  3;
        unsigned int                                                              :  5;
        unsigned int SPARE2                                                       :  3;
        unsigned int                                                              :  5;
        unsigned int SPARE1                                                       :  3;
        unsigned int                                                              :  5;
    } most;
    struct {
        unsigned int RESERVED0                                                    :  3;
        unsigned int hsTgCreditCnt_0                                              :  5;
        unsigned int RESERVED1                                                    :  2;
        unsigned int                                                              :  6;
        unsigned int RESERVED2                                                    :  3;
        unsigned int RESERVED3                                                    :  5;
        unsigned int RESERVED4                                                    :  2;
        unsigned int                                                              :  6;
    } rv2x;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DEBUG_REG7 {
    struct {
        unsigned int                                                              :  5;
        unsigned int SPARE                                                        : 11;
        unsigned int                                                              : 16;
    } most;
    struct {
        unsigned int                                                              :  5;
        unsigned int RESERVED0                                                    : 11;
        unsigned int                                                              : 16;
    } rv2x;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DEBUG_REG9 {
    struct {
        unsigned int                                                              : 30;
        unsigned int SPARE0                                                       :  1;
        unsigned int                                                              :  1;
    } most;
    struct {
        unsigned int                                                              :  7;
        unsigned int RESERVED0                                                    :  4;
        unsigned int                                                              : 12;
        unsigned int RESERVED1                                                    :  1;
        unsigned int                                                              :  1;
        unsigned int RESERVED2                                                    :  2;
        unsigned int                                                              :  3;
        unsigned int RESERVED3                                                    :  1;
        unsigned int RESERVED4                                                    :  1;
    } rv2x;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DEBUG_REG10 {
    struct {
        unsigned int                                                              :  9;
        unsigned int SPARE2                                                       :  2;
        unsigned int                                                              : 21;
    } most;
    struct {
        unsigned int                                                              :  9;
        unsigned int RESERVED0                                                    :  4;
        unsigned int                                                              : 10;
        unsigned int num_gs_waves_r3                                              :  2;
        unsigned int num_es_waves_r3                                              :  3;
        unsigned int RESERVED1                                                    :  4;
    } rv2x;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DEBUG_REG11 {
    struct {
        unsigned int                                                              :  5;
        unsigned int SPARE1                                                       :  1;
        unsigned int                                                              : 10;
        unsigned int gs_issue_rtr                                                 :  1;
        unsigned int                                                              :  1;
        unsigned int SPARE0                                                       :  1;
        unsigned int                                                              : 13;
    } rv1x;
    struct {
        unsigned int                                                              :  3;
        unsigned int RESERVED0                                                    :  1;
        unsigned int                                                              :  1;
        unsigned int RESERVED1                                                    :  1;
        unsigned int                                                              :  1;
        unsigned int RESERVED2                                                    :  3;
        unsigned int                                                              :  4;
        unsigned int VGT_SPI_gssubgrp_rtr                                         :  1;
        unsigned int RESERVED3                                                    :  1;
        unsigned int gs_issue_rtr                                                 :  1;
        unsigned int                                                              :  1;
        unsigned int RESERVED4                                                    :  1;
        unsigned int                                                              :  1;
        unsigned int RESERVED5                                                    :  1;
        unsigned int                                                              :  3;
        unsigned int RESERVED6                                                    :  1;
        unsigned int                                                              :  2;
        unsigned int RESERVED7                                                    :  1;
        unsigned int                                                              :  2;
        unsigned int RESERVED8                                                    :  1;
        unsigned int                                                              :  1;
    } rv2x;
    struct {
        unsigned int                                                              :  5;
        unsigned int SPARE1                                                       :  1;
        unsigned int                                                              : 10;
        unsigned int gs_issue_rtr                                                 :  1;
        unsigned int                                                              :  1;
        unsigned int SPARE0                                                       :  1;
        unsigned int                                                              : 13;
    } vega;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DEBUG_REG12 {
    struct {
        unsigned int                                                              : 31;
        unsigned int SPARE0                                                       :  1;
    } most;
    struct {
        unsigned int                                                              : 30;
        unsigned int RESERVED0                                                    :  1;
        unsigned int RESERVED1                                                    :  1;
    } rv2x;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DEBUG_REG13 {
    struct {
        unsigned int                                                              : 25;
        unsigned int SPARE1                                                       :  1;
        unsigned int SPARE0                                                       :  1;
        unsigned int                                                              :  5;
    } most;
    struct {
        unsigned int                                                              : 24;
        unsigned int RESERVED0                                                    :  1;
        unsigned int RESERVED1                                                    :  1;
        unsigned int RESERVED2                                                    :  1;
        unsigned int                                                              :  5;
    } rv2x;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DEBUG_REG14 {
    struct {
        unsigned int SPARE3                                                       :  4;
        unsigned int                                                              :  8;
        unsigned int SPARE2                                                       :  9;
        unsigned int                                                              :  1;
        unsigned int SPARE                                                        :  3;
        unsigned int                                                              :  3;
        unsigned int SPARE1                                                       :  1;
        unsigned int                                                              :  1;
        unsigned int SPARE0                                                       :  1;
        unsigned int                                                              :  1;
    } most;
    struct {
        unsigned int RESERVED0                                                    :  4;
        unsigned int                                                              :  2;
        unsigned int RESERVED1                                                    :  3;
        unsigned int                                                              :  1;
        unsigned int RESERVED2                                                    :  1;
        unsigned int                                                              :  1;
        unsigned int RESERVED3                                                    : 20;
    } rv2x;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DEBUG_REG15 {
    struct {
        unsigned int                                                              :  3;
        unsigned int output_fifo_full                                             :  1;
        unsigned int                                                              : 28;
    } most;
    struct {
        unsigned int                                                              :  5;
        unsigned int active_sm_q_4_0                                              :  5;
        unsigned int entry_rdptr_q_4_0                                            :  5;
        unsigned int cntr_tbl_wrptr_q_4_0                                         :  5;
        unsigned int RESERVED0                                                    :  6;
        unsigned int                                                              :  3;
        unsigned int RESERVED1                                                    :  3;
    } rv2x;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DEBUG_REG16 {
    struct {
        unsigned int gog_busy                                                     :  1;
        unsigned int                                                              : 31;
    } most;
    struct {
        unsigned int                                                              : 23;
        unsigned int RESERVED0                                                    :  1;
        unsigned int                                                              :  8;
    } rv2x;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DEBUG_REG23 {
    struct {
        unsigned int                                                              : 24;
        unsigned int SPARE                                                        :  8;
    } most;
    struct {
        unsigned int                                                              : 24;
        unsigned int frmt_pg_busy                                                 :  1;
        unsigned int RESERVED                                                     :  7;
    } rv2x;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DEBUG_REG30 {
    struct {
        unsigned int                                                              : 19;
        unsigned int SPARE                                                        : 13;
    } rv1x;
    struct {
        unsigned int                                                              :  4;
        unsigned int pg_busy                                                      :  1;
        unsigned int pg_noif_busy                                                 :  1;
        unsigned int pg_spi_interface_busy                                        :  1;
        unsigned int pg_te_prim_fifo_busy                                         :  1;
        unsigned int                                                              : 11;
        unsigned int non_pg_perfcounter_start                                     :  1;
        unsigned int non_pg_perfcounter_stop                                      :  1;
        unsigned int                                                              :  1;
        unsigned int esvert_fifo_full                                             :  1;
        unsigned int gsprim_fifo_full                                             :  1;
        unsigned int gswave_fifo_full                                             :  1;
        unsigned int eswave_fifo_full                                             :  1;
        unsigned int gsinvoc_fifo_full                                            :  1;
        unsigned int eopg_p2                                                      :  1;
        unsigned int eop_p1_q                                                     :  1;
        unsigned int event_flag_p1_q                                              :  1;
        unsigned int ordered_id_mode_p1_q                                         :  1;
        unsigned int te_vert_fifo_re                                              :  1;
    } rv2x;
    struct {
        unsigned int                                                              : 19;
        unsigned int SPARE                                                        : 13;
    } vega;
    struct {
        unsigned int                                                              :  4;
        unsigned int pg_busy                                                      :  1;
        unsigned int pg_noif_busy                                                 :  1;
        unsigned int pg_spi_interface_busy                                        :  1;
        unsigned int pg_te_prim_fifo_busy                                         :  1;
        unsigned int                                                              : 24;
    } vg12;
    struct {
        unsigned int                                                              :  4;
        unsigned int pg_busy                                                      :  1;
        unsigned int pg_noif_busy                                                 :  1;
        unsigned int pg_spi_interface_busy                                        :  1;
        unsigned int pg_te_prim_fifo_busy                                         :  1;
        unsigned int                                                              : 24;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DEBUG_REG32 {
    struct {
        unsigned int                                                              : 28;
        unsigned int SPARE                                                        :  4;
    } most;
    struct {
        unsigned int                                                              : 30;
        unsigned int su_output_fifo_empty                                         :  1;
        unsigned int su_output_fifo_full                                          :  1;
    } rv2x;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DEBUG_REG35 {
    struct {
        unsigned int RESERVED0                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DISPATCH_DRAW_INDEX {
    struct {
        unsigned int MATCH_INDEX                                                  : 32;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DMA_BASE {
    struct {
        unsigned int BASE_ADDR                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DMA_BASE_HI {
    struct {
        unsigned int BASE_ADDR                                                    : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DMA_CONTROL {
    struct {
        unsigned int PRIMGROUP_SIZE                                               : 16;
        unsigned int                                                              :  1;
        unsigned int IA_SWITCH_ON_EOP                                             :  1;
        unsigned int                                                              :  1;
        unsigned int SWITCH_ON_EOI                                                :  1;
        unsigned int WD_SWITCH_ON_EOP                                             :  1;
        unsigned int                                                              : 11;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 21;
        unsigned int EN_INST_OPT_BASIC                                            :  1;
        unsigned int EN_INST_OPT_ADV                                              :  1;
        unsigned int HW_USE_ONLY                                                  :  1;
        unsigned int                                                              :  8;
    } gfx09;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DMA_DATA_FIFO_DEPTH {
    struct {
        unsigned int DMA_DATA_FIFO_DEPTH                                          :  9;
        unsigned int DMA2DRAW_FIFO_DEPTH                                          : 10;
        unsigned int                                                              : 13;
    } gfx09;
    struct {
        unsigned int DMA_DATA_FIFO_DEPTH                                          : 10;
        unsigned int                                                              : 22;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DMA_EVENT_INITIATOR {
    struct {
        unsigned int EVENT_TYPE                                                   :  6;
        unsigned int                                                              :  4;
        unsigned int ADDRESS_HI                                                   : 17;
        unsigned int EXTENDED_EVENT                                               :  1;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DMA_INDEX_TYPE {
    struct {
        unsigned int INDEX_TYPE                                                   :  2;
        unsigned int SWAP_MODE                                                    :  2;
        unsigned int BUF_TYPE                                                     :  2;
        unsigned int                                                              :  3;
        unsigned int NOT_EOP                                                      :  1;
        unsigned int REQ_PATH                                                     :  1;
        unsigned int                                                              : 21;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  6;
        unsigned int RDREQ_POLICY                                                 :  1;
        unsigned int                                                              :  1;
        unsigned int PRIMGEN_EN                                                   :  1;
        unsigned int                                                              : 23;
    } gfx09;
    struct {
        unsigned int                                                              :  6;
        unsigned int RDREQ_POLICY                                                 :  2;
        unsigned int ATC                                                          :  1;
        unsigned int                                                              :  2;
        unsigned int MTYPE                                                        :  3;
        unsigned int                                                              : 18;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DMA_LS_HS_CONFIG {
    struct {
        unsigned int                                                              :  8;
        unsigned int HS_NUM_INPUT_CP                                              :  6;
        unsigned int                                                              : 18;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DMA_MAX_SIZE {
    struct {
        unsigned int MAX_SIZE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DMA_NUM_INSTANCES {
    struct {
        unsigned int NUM_INSTANCES                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DMA_PRIMITIVE_TYPE {
    struct {
        unsigned int PRIM_TYPE                                                    :  6;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DMA_REQ_FIFO_DEPTH {
    struct {
        unsigned int DMA_REQ_FIFO_DEPTH                                           :  6;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DMA_SIZE {
    struct {
        unsigned int NUM_INDICES                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DRAW_INITIATOR {
    struct {
        unsigned int SOURCE_SELECT                                                :  2;
        unsigned int MAJOR_MODE                                                   :  2;
        unsigned int SPRITE_EN_R6XX                                               :  1;
        unsigned int NOT_EOP                                                      :  1;
        unsigned int USE_OPAQUE                                                   :  1;
        unsigned int                                                              : 22;
        unsigned int REG_RT_INDEX                                                 :  3;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  7;
        unsigned int UNROLLED_INST                                                :  1;
        unsigned int GRBM_SKEW_NO_DEC                                             :  1;
        unsigned int                                                              : 23;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DRAW_INIT_FIFO_DEPTH {
    struct {
        unsigned int DRAW_INIT_FIFO_DEPTH                                         :  6;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DRAW_PAYLOAD_CNTL {
    struct {
        unsigned int                                                              :  1;
        unsigned int EN_REG_RT_INDEX                                              :  1;
        unsigned int                                                              : 30;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  1;
        unsigned int                                                              : 31;
    } most;
    struct {
        unsigned int                                                              :  2;
        unsigned int                                                              :  2;
        unsigned int                                                              : 28;
    } gfx09;
    struct {
        unsigned int                                                              :  3;
        unsigned int                                                              :  1;
        unsigned int EN_DRAW_VP                                                   :  1;
        unsigned int                                                              : 27;
    } gfx10;
    struct {
        unsigned int                                                              :  2;
        unsigned int                                                              :  1;
        unsigned int                                                              : 29;
    } gfx101;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_ENHANCE {
    struct {
        unsigned int MISC                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_ESGS_RING_ITEMSIZE {
    struct {
        unsigned int ITEMSIZE                                                     : 15;
        unsigned int                                                              : 17;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_ESGS_RING_SIZE {
    struct {
        unsigned int MEM_SIZE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_ESGS_RING_SIZE_UMD {
    struct {
        unsigned int MEM_SIZE                                                     : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_ES_PER_GS {
    struct {
        unsigned int ES_PER_GS                                                    : 11;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_EVENT_ADDRESS_REG {
    struct {
        unsigned int ADDRESS_LOW                                                  : 28;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_EVENT_INITIATOR {
    struct {
        unsigned int EVENT_TYPE                                                   :  6;
        unsigned int                                                              :  4;
        unsigned int ADDRESS_HI                                                   : 17;
        unsigned int EXTENDED_EVENT                                               :  1;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_FIFO_DEPTHS {
    struct {
        unsigned int VS_DEALLOC_TBL_DEPTH                                         :  7;
        unsigned int RESERVED_0                                                   :  1;
        unsigned int CLIPP_FIFO_DEPTH                                             : 14;
        unsigned int                                                              : 10;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 22;
        unsigned int HSINPUT_FIFO_DEPTH                                           :  6;
        unsigned int                                                              :  4;
    } gfx09;
    struct {
        unsigned int                                                              : 22;
        unsigned int RESERVED_1                                                   :  1;
        unsigned int HSINPUT_FIFO_DEPTH                                           :  6;
        unsigned int                                                              :  3;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_GROUP_DECR {
    struct {
        unsigned int DECR                                                         :  4;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_GROUP_FIRST_DECR {
    struct {
        unsigned int FIRST_DECR                                                   :  4;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_GROUP_PRIM_TYPE {
    struct {
        unsigned int PRIM_TYPE                                                    :  5;
        unsigned int                                                              :  9;
        unsigned int RETAIN_ORDER                                                 :  1;
        unsigned int RETAIN_QUADS                                                 :  1;
        unsigned int PRIM_ORDER                                                   :  3;
        unsigned int                                                              : 13;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_GROUP_VECT_0_CNTL {
    struct {
        unsigned int COMP_X_EN                                                    :  1;
        unsigned int COMP_Y_EN                                                    :  1;
        unsigned int COMP_Z_EN                                                    :  1;
        unsigned int COMP_W_EN                                                    :  1;
        unsigned int                                                              :  4;
        unsigned int STRIDE                                                       :  8;
        unsigned int SHIFT                                                        :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_GROUP_VECT_0_FMT_CNTL {
    struct {
        unsigned int X_CONV                                                       :  4;
        unsigned int X_OFFSET                                                     :  4;
        unsigned int Y_CONV                                                       :  4;
        unsigned int Y_OFFSET                                                     :  4;
        unsigned int Z_CONV                                                       :  4;
        unsigned int Z_OFFSET                                                     :  4;
        unsigned int W_CONV                                                       :  4;
        unsigned int W_OFFSET                                                     :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_GROUP_VECT_1_CNTL {
    struct {
        unsigned int COMP_X_EN                                                    :  1;
        unsigned int COMP_Y_EN                                                    :  1;
        unsigned int COMP_Z_EN                                                    :  1;
        unsigned int COMP_W_EN                                                    :  1;
        unsigned int                                                              :  4;
        unsigned int STRIDE                                                       :  8;
        unsigned int SHIFT                                                        :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_GROUP_VECT_1_FMT_CNTL {
    struct {
        unsigned int X_CONV                                                       :  4;
        unsigned int X_OFFSET                                                     :  4;
        unsigned int Y_CONV                                                       :  4;
        unsigned int Y_OFFSET                                                     :  4;
        unsigned int Z_CONV                                                       :  4;
        unsigned int Z_OFFSET                                                     :  4;
        unsigned int W_CONV                                                       :  4;
        unsigned int W_OFFSET                                                     :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_GSVS_RING_ITEMSIZE {
    struct {
        unsigned int ITEMSIZE                                                     : 15;
        unsigned int                                                              : 17;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_GSVS_RING_OFFSET_1 {
    struct {
        unsigned int OFFSET                                                       : 15;
        unsigned int                                                              : 17;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_GSVS_RING_OFFSET_2 {
    struct {
        unsigned int OFFSET                                                       : 15;
        unsigned int                                                              : 17;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_GSVS_RING_OFFSET_3 {
    struct {
        unsigned int OFFSET                                                       : 15;
        unsigned int                                                              : 17;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_GSVS_RING_SIZE {
    struct {
        unsigned int MEM_SIZE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_GSVS_RING_SIZE_UMD {
    struct {
        unsigned int MEM_SIZE                                                     : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_GS_INSTANCE_CNT {
    struct {
        unsigned int ENABLE                                                       :  1;
        unsigned int                                                              :  1;
        unsigned int CNT                                                          :  7;
        unsigned int                                                              : 23;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 31;
        unsigned int EN_MAX_VERT_OUT_PER_GS_INSTANCE                              :  1;
    } gfx101Plus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_GS_MAX_PRIMS_PER_SUBGROUP {
    struct {
        unsigned int MAX_PRIMS_PER_SUBGROUP                                       : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_GS_MAX_VERT_OUT {
    struct {
        unsigned int MAX_VERT_OUT                                                 : 11;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_GS_MAX_WAVE_ID {
    struct {
        unsigned int MAX_WAVE_ID                                                  : 12;
        unsigned int                                                              : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_GS_MODE {
    struct {
        unsigned int MODE                                                         :  3;
        unsigned int RESERVED_0                                                   :  1;
        unsigned int CUT_MODE                                                     :  2;
        unsigned int RESERVED_1                                                   :  5;
        unsigned int GS_C_PACK_EN                                                 :  1;
        unsigned int RESERVED_2                                                   :  1;
        unsigned int ES_PASSTHRU                                                  :  1;
        unsigned int                                                              :  3;
        unsigned int PARTIAL_THD_AT_EOI                                           :  1;
        unsigned int SUPPRESS_CUTS                                                :  1;
        unsigned int ES_WRITE_OPTIMIZE                                            :  1;
        unsigned int GS_WRITE_OPTIMIZE                                            :  1;
        unsigned int ONCHIP                                                       :  2;
        unsigned int                                                              :  9;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 14;
        unsigned int RESERVED_3                                                   :  1;
        unsigned int RESERVED_4                                                   :  1;
        unsigned int RESERVED_5                                                   :  1;
        unsigned int                                                              : 15;
    } gfx09;
    struct {
        unsigned int                                                              : 14;
        unsigned int COMPUTE_MODE                                                 :  1;
        unsigned int FAST_COMPUTE_MODE                                            :  1;
        unsigned int ELEMENT_INFO_EN                                              :  1;
        unsigned int                                                              : 15;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_GS_ONCHIP_CNTL {
    struct {
        unsigned int ES_VERTS_PER_SUBGRP                                          : 11;
        unsigned int GS_PRIMS_PER_SUBGRP                                          : 11;
        unsigned int GS_INST_PRIMS_IN_SUBGRP                                      : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_GS_OUT_PRIM_TYPE {
    struct {
        unsigned int OUTPRIM_TYPE                                                 :  6;
        unsigned int                                                              :  2;
        unsigned int OUTPRIM_TYPE_1                                               :  6;
        unsigned int                                                              :  2;
        unsigned int OUTPRIM_TYPE_2                                               :  6;
        unsigned int OUTPRIM_TYPE_3                                               :  6;
        unsigned int                                                              :  3;
        unsigned int UNIQUE_TYPE_PER_STREAM                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_GS_PER_ES {
    struct {
        unsigned int GS_PER_ES                                                    : 11;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_GS_PER_VS {
    struct {
        unsigned int GS_PER_VS                                                    :  4;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_GS_VERTEX_REUSE {
    struct {
        unsigned int VERT_REUSE                                                   :  5;
        unsigned int                                                              : 27;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_GS_VERT_ITEMSIZE {
    struct {
        unsigned int ITEMSIZE                                                     : 15;
        unsigned int                                                              : 17;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_GS_VERT_ITEMSIZE_1 {
    struct {
        unsigned int ITEMSIZE                                                     : 15;
        unsigned int                                                              : 17;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_GS_VERT_ITEMSIZE_2 {
    struct {
        unsigned int ITEMSIZE                                                     : 15;
        unsigned int                                                              : 17;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_GS_VERT_ITEMSIZE_3 {
    struct {
        unsigned int ITEMSIZE                                                     : 15;
        unsigned int                                                              : 17;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_HOS_CNTL {
    struct {
        unsigned int TESS_MODE                                                    :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_HOS_MAX_TESS_LEVEL {
    struct {
        unsigned int MAX_TESS                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_HOS_MIN_TESS_LEVEL {
    struct {
        unsigned int MIN_TESS                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_HOS_REUSE_DEPTH {
    struct {
        unsigned int REUSE_DEPTH                                                  :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_HS_OFFCHIP_PARAM {
    struct {
        unsigned int OFFCHIP_BUFFERING                                            :  9;
        unsigned int OFFCHIP_GRANULARITY                                          :  2;
        unsigned int                                                              : 21;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_HS_OFFCHIP_PARAM_UMD {
    struct {
        unsigned int OFFCHIP_BUFFERING                                            :  9;
        unsigned int OFFCHIP_GRANULARITY                                          :  2;
        unsigned int                                                              : 21;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_IMMED_DATA {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_INDEX_TYPE {
    struct {
        unsigned int INDEX_TYPE                                                   :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  8;
        unsigned int PRIMGEN_EN                                                   :  1;
        unsigned int                                                              : 23;
    } gfx09;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_INDX_OFFSET {
    struct {
        unsigned int INDX_OFFSET                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_INSTANCE_BASE_ID {
    struct {
        unsigned int INSTANCE_BASE_ID                                             : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_INSTANCE_STEP_RATE_0 {
    struct {
        unsigned int STEP_RATE                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_INSTANCE_STEP_RATE_1 {
    struct {
        unsigned int STEP_RATE                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_LAST_COPY_STATE {
    struct {
        unsigned int SRC_STATE_ID                                                 :  3;
        unsigned int                                                              : 13;
        unsigned int DST_STATE_ID                                                 :  3;
        unsigned int                                                              : 13;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_LS_HS_CONFIG {
    struct {
        unsigned int NUM_PATCHES                                                  :  8;
        unsigned int HS_NUM_INPUT_CP                                              :  6;
        unsigned int HS_NUM_OUTPUT_CP                                             :  6;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_MAX_VTX_INDX {
    struct {
        unsigned int MAX_INDX                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_MC_LAT_CNTL {
    struct {
        unsigned int MC_TIME_STAMP_RES                                            :  4;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_MIN_VTX_INDX {
    struct {
        unsigned int MIN_INDX                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_MULTI_PRIM_IB_RESET_EN {
    struct {
        unsigned int RESET_EN                                                     :  1;
        unsigned int MATCH_ALL_BITS                                               :  1;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_MULTI_PRIM_IB_RESET_INDX {
    struct {
        unsigned int RESET_INDX                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_NUM_INDICES {
    struct {
        unsigned int NUM_INDICES                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_NUM_INSTANCES {
    struct {
        unsigned int NUM_INSTANCES                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_OUTPUT_PATH_CNTL {
    struct {
        unsigned int PATH_SELECT                                                  :  3;
        unsigned int                                                              : 29;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_OUT_DEALLOC_CNTL {
    struct {
        unsigned int DEALLOC_DIST                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_PERFCOUNTER1_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int                                                              : 20;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int                                                              : 20;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_PERFCOUNTER_SEID_MASK {
    struct {
        unsigned int PERF_SEID_IGNORE_MASK                                        :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_PRIMITIVEID_EN {
    struct {
        unsigned int PRIMITIVEID_EN                                               :  1;
        unsigned int DISABLE_RESET_ON_EOI                                         :  1;
        unsigned int NGG_DISABLE_PROVOK_REUSE                                     :  1;
        unsigned int                                                              : 29;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_PRIMITIVEID_RESET {
    struct {
        unsigned int VALUE                                                        : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_PRIMITIVE_TYPE {
    struct {
        unsigned int PRIM_TYPE                                                    :  6;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_RESET_DEBUG {
    struct {
        unsigned int GS_DISABLE                                                   :  1;
        unsigned int TESS_DISABLE                                                 :  1;
        unsigned int WD_DISABLE                                                   :  1;
        unsigned int                                                              : 29;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  3;
        unsigned int DISABLE_TE11_DIST_PIPE0                                      :  1;
        unsigned int DISABLE_TE11_DIST_PIPE1                                      :  1;
        unsigned int ENABLE_VMID_RESET_UTCL1                                      :  1;
        unsigned int                                                              : 26;
    } gfx101Plus;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_REUSE_OFF {
    struct {
        unsigned int REUSE_OFF                                                    :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_SHADER_STAGES_EN {
    struct {
        unsigned int LS_EN                                                        :  2;
        unsigned int HS_EN                                                        :  1;
        unsigned int ES_EN                                                        :  2;
        unsigned int GS_EN                                                        :  1;
        unsigned int VS_EN                                                        :  2;
        unsigned int                                                              :  4;
        unsigned int VS_WAVE_ID_EN                                                :  1;
        unsigned int PRIMGEN_EN                                                   :  1;
        unsigned int ORDERED_ID_MODE                                              :  1;
        unsigned int MAX_PRIMGRP_IN_WAVE                                          :  4;
        unsigned int                                                              : 13;
    } bits, bitfields;
    struct {
        unsigned int                                                              :  9;
        unsigned int DISPATCH_DRAW_EN                                             :  1;
        unsigned int DIS_DEALLOC_ACCUM_0                                          :  1;
        unsigned int DIS_DEALLOC_ACCUM_1                                          :  1;
        unsigned int                                                              : 20;
    } most;
    struct {
        unsigned int                                                              : 19;
        unsigned int GS_FAST_LAUNCH                                               :  1;
        unsigned int                                                              : 12;
    } gfx09_0;
    struct {
        unsigned int                                                              : 19;
        unsigned int GS_FAST_LAUNCH                                               :  2;
        unsigned int                                                              : 11;
    } gfx09_1xPlus;
    struct {
        unsigned int                                                              :  8;
        unsigned int DYNAMIC_HS                                                   :  1;
        unsigned int                                                              : 12;
        unsigned int HS_W32_EN                                                    :  1;
        unsigned int GS_W32_EN                                                    :  1;
        unsigned int VS_W32_EN                                                    :  1;
        unsigned int NGG_WAVE_ID_EN                                               :  1;
        unsigned int PRIMGEN_PASSTHRU_EN                                          :  1;
        unsigned int                                                              :  6;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_STRMOUT_BUFFER_CONFIG {
    struct {
        unsigned int STREAM_0_BUFFER_EN                                           :  4;
        unsigned int STREAM_1_BUFFER_EN                                           :  4;
        unsigned int STREAM_2_BUFFER_EN                                           :  4;
        unsigned int STREAM_3_BUFFER_EN                                           :  4;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_STRMOUT_BUFFER_FILLED_SIZE_0 {
    struct {
        unsigned int SIZE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_STRMOUT_BUFFER_FILLED_SIZE_1 {
    struct {
        unsigned int SIZE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_STRMOUT_BUFFER_FILLED_SIZE_2 {
    struct {
        unsigned int SIZE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_STRMOUT_BUFFER_FILLED_SIZE_3 {
    struct {
        unsigned int SIZE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_STRMOUT_BUFFER_OFFSET_0 {
    struct {
        unsigned int OFFSET                                                       : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_STRMOUT_BUFFER_OFFSET_1 {
    struct {
        unsigned int OFFSET                                                       : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_STRMOUT_BUFFER_OFFSET_2 {
    struct {
        unsigned int OFFSET                                                       : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_STRMOUT_BUFFER_OFFSET_3 {
    struct {
        unsigned int OFFSET                                                       : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_STRMOUT_BUFFER_SIZE_0 {
    struct {
        unsigned int SIZE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_STRMOUT_BUFFER_SIZE_1 {
    struct {
        unsigned int SIZE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_STRMOUT_BUFFER_SIZE_2 {
    struct {
        unsigned int SIZE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_STRMOUT_BUFFER_SIZE_3 {
    struct {
        unsigned int SIZE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_STRMOUT_CONFIG {
    struct {
        unsigned int STREAMOUT_0_EN                                               :  1;
        unsigned int STREAMOUT_1_EN                                               :  1;
        unsigned int STREAMOUT_2_EN                                               :  1;
        unsigned int STREAMOUT_3_EN                                               :  1;
        unsigned int RAST_STREAM                                                  :  3;
        unsigned int EN_PRIMS_NEEDED_CNT                                          :  1;
        unsigned int RAST_STREAM_MASK                                             :  4;
        unsigned int                                                              : 19;
        unsigned int USE_RAST_STREAM_MASK                                         :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_STRMOUT_DELAY {
    struct {
        unsigned int SKIP_DELAY                                                   :  8;
        unsigned int SE0_WD_DELAY                                                 :  3;
        unsigned int SE1_WD_DELAY                                                 :  3;
        unsigned int SE2_WD_DELAY                                                 :  3;
        unsigned int SE3_WD_DELAY                                                 :  3;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_STRMOUT_DRAW_OPAQUE_BUFFER_FILLED_SIZE {
    struct {
        unsigned int SIZE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_STRMOUT_DRAW_OPAQUE_OFFSET {
    struct {
        unsigned int OFFSET                                                       : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_STRMOUT_DRAW_OPAQUE_VERTEX_STRIDE {
    struct {
        unsigned int VERTEX_STRIDE                                                :  9;
        unsigned int                                                              : 23;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_STRMOUT_VTX_STRIDE_0 {
    struct {
        unsigned int STRIDE                                                       : 10;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_STRMOUT_VTX_STRIDE_1 {
    struct {
        unsigned int STRIDE                                                       : 10;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_STRMOUT_VTX_STRIDE_2 {
    struct {
        unsigned int STRIDE                                                       : 10;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_STRMOUT_VTX_STRIDE_3 {
    struct {
        unsigned int STRIDE                                                       : 10;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_SYS_CONFIG {
    struct {
        unsigned int DUAL_CORE_EN                                                 :  1;
        unsigned int MAX_LS_HS_THDGRP                                             :  6;
        unsigned int ADC_EVENT_FILTER_DISABLE                                     :  1;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_TESS_DISTRIBUTION {
    struct {
        unsigned int ACCUM_ISOLINE                                                :  8;
        unsigned int ACCUM_TRI                                                    :  8;
        unsigned int ACCUM_QUAD                                                   :  8;
        unsigned int DONUT_SPLIT                                                  :  5;
        unsigned int TRAP_SPLIT                                                   :  3;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_TF_MEMORY_BASE {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_TF_MEMORY_BASE_HI {
    struct {
        unsigned int BASE_HI                                                      :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_TF_MEMORY_BASE_HI_UMD {
    struct {
        unsigned int BASE_HI                                                      :  8;
        unsigned int                                                              : 24;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_TF_MEMORY_BASE_UMD {
    struct {
        unsigned int BASE                                                         : 32;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_TF_PARAM {
    struct {
        unsigned int TYPE                                                         :  2;
        unsigned int PARTITIONING                                                 :  3;
        unsigned int TOPOLOGY                                                     :  3;
        unsigned int RESERVED_REDUC_AXIS                                          :  1;
        unsigned int DEPRECATED                                                   :  1;
        unsigned int                                                              :  4;
        unsigned int DISABLE_DONUTS                                               :  1;
        unsigned int                                                              :  2;
        unsigned int DISTRIBUTION_MODE                                            :  2;
        unsigned int                                                              : 13;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 15;
        unsigned int RDREQ_POLICY                                                 :  1;
        unsigned int                                                              : 16;
    } gfx09;
    struct {
        unsigned int                                                              : 10;
        unsigned int NUM_DS_WAVES_PER_SIMD                                        :  4;
        unsigned int                                                              :  1;
        unsigned int RDREQ_POLICY                                                 :  2;
        unsigned int                                                              :  2;
        unsigned int DETECT_ONE                                                   :  1;
        unsigned int DETECT_ZERO                                                  :  1;
        unsigned int                                                              :  2;
        unsigned int MTYPE                                                        :  3;
        unsigned int                                                              :  6;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_TF_RING_SIZE {
    struct {
        unsigned int SIZE                                                         : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_TF_RING_SIZE_UMD {
    struct {
        unsigned int SIZE                                                         : 16;
        unsigned int                                                              : 16;
    } nv10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_VERTEX_REUSE_BLOCK_CNTL {
    struct {
        unsigned int VTX_REUSE_DEPTH                                              :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_VS_MAX_WAVE_ID {
    struct {
        unsigned int MAX_WAVE_ID                                                  : 12;
        unsigned int                                                              : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_VTX_CNT_EN {
    struct {
        unsigned int VTX_CNT_EN                                                   :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_VTX_VECT_EJECT_REG {
    struct {
        unsigned int PRIM_COUNT                                                   :  7;
        unsigned int                                                              : 25;
    } gfx09;
    struct {
        unsigned int PRIM_COUNT                                                   : 10;
        unsigned int                                                              : 22;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VML2_MEM_ECC_CNTL {
    struct {
        unsigned int INDEX                                                        :  8;
        unsigned int INJECT_DELAY                                                 :  6;
        unsigned int DSM_IRRITATOR_DATA                                           :  2;
        unsigned int ENABLE_SINGLE_WRITE                                          :  1;
        unsigned int ENABLE_ERROR_INJECT                                          :  2;
        unsigned int SELECT_INJECT_DELAY                                          :  1;
        unsigned int SEC_COUNT                                                    :  2;
        unsigned int DED_COUNT                                                    :  2;
        unsigned int WRITE_COUNTERS                                               :  1;
        unsigned int                                                              :  7;
    } vg12;
    struct {
        unsigned int INJECT_DELAY                                                 :  6;
        unsigned int DSM_IRRITATOR_DATA                                           :  2;
        unsigned int ENABLE_SINGLE_WRITE                                          :  1;
        unsigned int ENABLE_ERROR_INJECT                                          :  2;
        unsigned int SELECT_INJECT_DELAY                                          :  1;
        unsigned int SEC_COUNT                                                    :  2;
        unsigned int DED_COUNT                                                    :  2;
        unsigned int WRITE_COUNTERS                                               :  1;
        unsigned int TEST_FUE                                                     :  1;
        unsigned int                                                              : 14;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VML2_MEM_ECC_INDEX {
    struct {
        unsigned int INDEX                                                        :  8;
        unsigned int                                                              : 24;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VML2_WALKER_MEM_ECC_CNTL {
    struct {
        unsigned int INDEX                                                        :  8;
        unsigned int INJECT_DELAY                                                 :  6;
        unsigned int DSM_IRRITATOR_DATA                                           :  2;
        unsigned int ENABLE_SINGLE_WRITE                                          :  1;
        unsigned int ENABLE_ERROR_INJECT                                          :  2;
        unsigned int SELECT_INJECT_DELAY                                          :  1;
        unsigned int SEC_COUNT                                                    :  2;
        unsigned int DED_COUNT                                                    :  2;
        unsigned int WRITE_COUNTERS                                               :  1;
        unsigned int                                                              :  7;
    } vg12;
    struct {
        unsigned int INJECT_DELAY                                                 :  6;
        unsigned int DSM_IRRITATOR_DATA                                           :  2;
        unsigned int ENABLE_SINGLE_WRITE                                          :  1;
        unsigned int ENABLE_ERROR_INJECT                                          :  2;
        unsigned int SELECT_INJECT_DELAY                                          :  1;
        unsigned int SEC_COUNT                                                    :  2;
        unsigned int DED_COUNT                                                    :  2;
        unsigned int WRITE_COUNTERS                                               :  1;
        unsigned int TEST_FUE                                                     :  1;
        unsigned int                                                              : 14;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VML2_WALKER_MEM_ECC_INDEX {
    struct {
        unsigned int INDEX                                                        :  8;
        unsigned int                                                              : 24;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union wbuf_DEBUG_DATA {
    struct {
        unsigned int Reserved0                                                    :  6;
        unsigned int                                                              :  1;
        unsigned int wr_pixel_ptr_nxt                                             :  5;
        unsigned int                                                              :  1;
        unsigned int cstate_3to0                                                  :  4;
        unsigned int                                                              :  5;
        unsigned int Reserved1                                                    : 10;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_BUF_RESOURCE_1 {
    struct {
        unsigned int POS_BUF_SIZE                                                 : 16;
        unsigned int INDEX_BUF_SIZE                                               : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_BUF_RESOURCE_2 {
    struct {
        unsigned int PARAM_BUF_SIZE                                               : 13;
        unsigned int                                                              :  2;
        unsigned int ADDR_MODE                                                    :  1;
        unsigned int CNTL_SB_BUF_SIZE                                             : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_CNTL_SB_BUF_BASE {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_CNTL_SB_BUF_BASE_HI {
    struct {
        unsigned int BASE_HI                                                      :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_CNTL_STATUS {
    struct {
        unsigned int WD_BUSY                                                      :  1;
        unsigned int WD_SPL_DMA_BUSY                                              :  1;
        unsigned int WD_SPL_DI_BUSY                                               :  1;
        unsigned int WD_ADC_BUSY                                                  :  1;
        unsigned int                                                              : 28;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_DEBUG_DATA {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_DEBUG_REG0 {
    struct {
        unsigned int                                                              : 12;
        unsigned int SPARE2                                                       :  1;
        unsigned int                                                              : 19;
    } gfx09_0;
    struct {
        unsigned int                                                              :  2;
        unsigned int wd_busy                                                      :  1;
        unsigned int                                                              : 13;
        unsigned int SPARE3                                                       :  1;
        unsigned int                                                              : 11;
        unsigned int sclk_reg_vld                                                 :  1;
        unsigned int sclk_input_vld                                               :  1;
        unsigned int sclk_core_vld                                                :  1;
        unsigned int                                                              :  1;
    } rv1x;
    struct {
        unsigned int                                                              :  2;
        unsigned int wd_busy                                                      :  1;
        unsigned int                                                              :  9;
        unsigned int wd_ngg_busy                                                  :  1;
        unsigned int                                                              :  3;
        unsigned int RESERVED0                                                    :  1;
        unsigned int                                                              : 11;
        unsigned int sclk_reg_vld                                                 :  1;
        unsigned int sclk_input_vld                                               :  1;
        unsigned int sclk_core_vld                                                :  1;
        unsigned int                                                              :  1;
    } rv2x;
    struct {
        unsigned int                                                              :  2;
        unsigned int wd_busy                                                      :  1;
        unsigned int                                                              : 13;
        unsigned int SPARE3                                                       :  1;
        unsigned int                                                              : 11;
        unsigned int sclk_reg_vld                                                 :  1;
        unsigned int sclk_input_vld                                               :  1;
        unsigned int sclk_core_vld                                                :  1;
        unsigned int                                                              :  1;
    } vega;
    struct {
        unsigned int                                                              : 12;
        unsigned int wd_ngg_busy                                                  :  1;
        unsigned int                                                              : 19;
    } vg12;
    struct {
        unsigned int                                                              : 12;
        unsigned int wd_ngg_busy                                                  :  1;
        unsigned int                                                              : 19;
    } vg20;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_DEBUG_REG1 {
    struct {
        unsigned int                                                              :  8;
        unsigned int SPARE0                                                       :  1;
        unsigned int                                                              : 23;
    } most;
    struct {
        unsigned int                                                              :  8;
        unsigned int reg_xfer_valid                                               :  1;
        unsigned int                                                              : 23;
    } rv2x;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_DEBUG_REG2 {
    struct {
        unsigned int                                                              :  8;
        unsigned int SPARE0                                                       :  1;
        unsigned int                                                              : 23;
    } most;
    struct {
        unsigned int                                                              :  8;
        unsigned int p1_reg_xfer_valid                                            :  1;
        unsigned int                                                              :  3;
        unsigned int p1_grbm_fifo_rdata_reg_id_4_0                                :  5;
        unsigned int p1_grbm_fifo_rdata_state_2_0                                 :  3;
        unsigned int                                                              : 12;
    } rv2x;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_DEBUG_REG3 {
    struct {
        unsigned int                                                              :  1;
        unsigned int SPARE0                                                       :  1;
        unsigned int                                                              : 21;
        unsigned int SPARE1                                                       :  1;
        unsigned int                                                              :  8;
    } most;
    struct {
        unsigned int                                                              :  1;
        unsigned int RESERVED0                                                    :  1;
        unsigned int                                                              :  9;
        unsigned int RESERVED1                                                    :  1;
        unsigned int                                                              : 11;
        unsigned int RESERVED2                                                    :  1;
        unsigned int                                                              :  8;
    } rv2x;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_DEBUG_REG5 {
    struct {
        unsigned int                                                              :  1;
        unsigned int SPARE0                                                       :  1;
        unsigned int                                                              : 21;
        unsigned int SPARE1                                                       :  1;
        unsigned int                                                              :  8;
    } most;
    struct {
        unsigned int                                                              :  1;
        unsigned int RESERVED0                                                    :  1;
        unsigned int                                                              :  9;
        unsigned int RESERVED1                                                    :  1;
        unsigned int                                                              : 11;
        unsigned int RESERVED2                                                    :  1;
        unsigned int                                                              :  8;
    } rv2x;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_DEBUG_REG7 {
    struct {
        unsigned int                                                              :  4;
        unsigned int SE1VGT_WD_thdgrp_send_in                                     :  1;
        unsigned int wd_arb_se1_input_fifo_re                                     :  1;
        unsigned int wd_arb_se1_input_fifo_empty                                  :  1;
        unsigned int wd_arb_se1_input_fifo_full                                   :  1;
        unsigned int SPARE1                                                       :  4;
        unsigned int SPARE2                                                       :  4;
        unsigned int te11_arb_state_q                                             :  3;
        unsigned int SPARE5                                                       :  1;
        unsigned int                                                              :  2;
        unsigned int se1_thdgrp_is_event                                          :  1;
        unsigned int se1_thdgrp_eop                                               :  1;
        unsigned int                                                              :  8;
    } rv1x;
    struct {
        unsigned int                                                              :  4;
        unsigned int SE1VGT_WD_thdgrp_send_in                                     :  1;
        unsigned int wd_arb_se1_input_fifo_re                                     :  1;
        unsigned int wd_arb_se1_input_fifo_empty                                  :  1;
        unsigned int wd_arb_se1_input_fifo_full                                   :  1;
        unsigned int SE2VGT_WD_thdgrp_send_in                                     :  1;
        unsigned int wd_arb_se2_input_fifo_re                                     :  1;
        unsigned int wd_arb_se2_input_fifo_empty                                  :  1;
        unsigned int wd_arb_se2_input_fifo_full                                   :  1;
        unsigned int SE3VGT_WD_thdgrp_send_in                                     :  1;
        unsigned int wd_arb_se3_input_fifo_re                                     :  1;
        unsigned int wd_arb_se3_input_fifo_empty                                  :  1;
        unsigned int wd_arb_se3_input_fifo_full                                   :  1;
        unsigned int te11_arb_state_q                                             :  4;
        unsigned int                                                              :  2;
        unsigned int se1_thdgrp_is_event                                          :  1;
        unsigned int se1_thdgrp_eop                                               :  1;
        unsigned int se2_thdgrp_is_event                                          :  1;
        unsigned int se2_thdgrp_eop                                               :  1;
        unsigned int se3_thdgrp_is_event                                          :  1;
        unsigned int se3_thdgrp_eop                                               :  1;
        unsigned int                                                              :  4;
    } rv2x;
    struct {
        unsigned int                                                              :  4;
        unsigned int SPARE0                                                       :  4;
        unsigned int SPARE1                                                       :  4;
        unsigned int SPARE2                                                       :  4;
        unsigned int SPARE3                                                       :  4;
        unsigned int                                                              :  2;
        unsigned int SPARE4                                                       :  6;
        unsigned int                                                              :  4;
    } vega;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_DEBUG_REG9 {
    struct {
        unsigned int                                                              : 11;
        unsigned int SPARE0                                                       :  5;
        unsigned int                                                              : 11;
        unsigned int SPARE1                                                       :  1;
        unsigned int SPARE2                                                       :  2;
        unsigned int                                                              :  2;
    } most;
    struct {
        unsigned int                                                              :  3;
        unsigned int event_flag_p0_q_or_null_flags_p0_q                           :  1;
        unsigned int                                                              :  7;
        unsigned int RESERVED0                                                    :  5;
        unsigned int                                                              :  8;
        unsigned int RESERVED1                                                    :  1;
        unsigned int                                                              :  2;
        unsigned int RESERVED2                                                    :  1;
        unsigned int RESERVED3                                                    :  2;
        unsigned int                                                              :  2;
    } rv2x;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_DEBUG_REG14 {
    struct {
        unsigned int iUTCL2_WD_ret_tag                                            :  7;
        unsigned int iUTCL2_WD_ret_nack                                           :  2;
        unsigned int iUTCL2_WD_ret_mtype                                          :  2;
        unsigned int iUTCL2_WD_ret_memlog                                         :  1;
        unsigned int iUTCL2_WD_ret_no_pte                                         :  1;
        unsigned int iUTCL2_WD_ret_pte_tmz                                        :  1;
        unsigned int iUTCL2_WD_ret_spa                                            :  1;
        unsigned int iUTCL2_WD_ret_io_steer                                       :  1;
        unsigned int iUTCL2_WD_ret_fragment_size                                  :  6;
        unsigned int iUTCL2_WD_ret_perms_granted                                  :  3;
        unsigned int iUTCL2_WD_ret_perms_requested                                :  3;
        unsigned int iUTCL2_WD_ret_snoop                                          :  1;
        unsigned int iUTCL2_WD_ret_valid                                          :  1;
        unsigned int iUTCL2_WD_ret_physical_address_1_0                           :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_DEBUG_REG15 {
    struct {
        unsigned int iUTCL2_WD_ret_physical_address_33_2                          : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_DEBUG_REG16 {
    struct {
        unsigned int iUTCL2_WD_ret_physical_address_35_34                         :  2;
        unsigned int WD_UTCL2_req_data                                            : 16;
        unsigned int qWD_UTCL2_req_free                                           :  1;
        unsigned int WD_UTCL2_req_send                                            :  1;
        unsigned int qUtcL2XferCycle                                              :  1;
        unsigned int RESERVED0                                                    : 11;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_DEBUG_REG17 {
    struct {
        unsigned int WDUtcL2iuPostArbFifo_Empty                                   :  1;
        unsigned int WDUtcL2iuPostArbFifo_Full                                    :  1;
        unsigned int WDUtcL2Cnt                                                   :  6;
        unsigned int WDUtcL2OutCnt                                                :  6;
        unsigned int qUtcL2XferCycle                                              :  3;
        unsigned int iUTCL10_UTCL2IU_req_rts                                      :  1;
        unsigned int iUTCL11_UTCL2IU_req_rts                                      :  1;
        unsigned int XferId                                                       :  2;
        unsigned int qXferId                                                      :  3;
        unsigned int NewCycleXfer                                                 :  1;
        unsigned int RESERVED0                                                    :  7;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_DEBUG_REG18 {
    struct {
        unsigned int outstanding_tc_accesses_2_0                                  :  3;
        unsigned int utcl2iu_busy                                                 :  1;
        unsigned int utcl11_busy                                                  :  1;
        unsigned int utcl10_se3_busy                                              :  1;
        unsigned int utcl10_se2_busy                                              :  1;
        unsigned int utcl10_se1_busy                                              :  1;
        unsigned int utcl10_se0_busy                                              :  1;
        unsigned int busy                                                         :  1;
        unsigned int UTCL2IU_UTCL1_ret_nack                                       :  2;
        unsigned int UTCL2IU_UTCL1_ret_valid                                      :  1;
        unsigned int TC_WD_ret_valid                                              :  1;
        unsigned int cpf_wd_invreq_clean_out                                      :  1;
        unsigned int cpf_wd_invreq_flushtype_in                                   :  1;
        unsigned int cpf_wd_invreq_start_in                                       :  1;
        unsigned int WD_UTCL2_inv_ack                                             :  1;
        unsigned int UTCL2_WD_inv_valid                                           :  1;
        unsigned int UTCL2_WD_ret_xnack                                           :  2;
        unsigned int UTCL2_WD_ret_valid                                           :  1;
        unsigned int WD_UTCL2_req_free                                            :  1;
        unsigned int WD_UTCL2_req_send                                            :  1;
        unsigned int UTCL1_CLIENT_ret1_ready                                      :  1;
        unsigned int UTCL1_CLIENT_ret1_valid                                      :  1;
        unsigned int CLIENT_UTCL1_req1_ready                                      :  1;
        unsigned int CLIENT_UTCL1_req1_valid                                      :  1;
        unsigned int UTCL1_CLIENT_ret0_ready                                      :  1;
        unsigned int UTCL1_CLIENT_ret0_valid                                      :  1;
        unsigned int CLIENT_UTCL1_req0_ready                                      :  1;
        unsigned int CLIENT_UTCL1_req0_valid                                      :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_DEBUG_REG19 {
    struct {
        unsigned int dma_primgroup_size                                           : 16;
        unsigned int RESERVED0                                                    :  4;
        unsigned int dma_wd_switch_on_eop                                         :  1;
        unsigned int RESERVED1                                                    :  7;
        unsigned int dma_dummy_req_p3_q                                           :  1;
        unsigned int dispatch_draw_mode_p4                                        :  1;
        unsigned int dma_out_of_range_p3_q                                        :  1;
        unsigned int dma_issue_p2                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_DEBUG_REG20 {
    struct {
        unsigned int reg_state_or_event_data_p2_15_0                              : 16;
        unsigned int reg_state_or_event_data_p2_16                                :  1;
        unsigned int reg_state_or_event_data_p2_17                                :  1;
        unsigned int reg_state_or_event_data_p2_18                                :  1;
        unsigned int reg_state_or_event_data_p2_19                                :  1;
        unsigned int reg_state_or_event_data_p2_20                                :  1;
        unsigned int reg_state_or_event_data_p2_21                                :  1;
        unsigned int reg_state_or_event_data_p2_22                                :  1;
        unsigned int reg_state_or_event_data_p2_23                                :  1;
        unsigned int reg_state_or_event_data_p2_31_24                             :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_DEBUG_REG21 {
    struct {
        unsigned int hp_dma_primgroup_size                                        : 16;
        unsigned int RESERVED0                                                    :  4;
        unsigned int hp_dma_wd_switch_on_eop                                      :  1;
        unsigned int RESERVED1                                                    :  7;
        unsigned int hp_dma_dummy_req_p3_q                                        :  1;
        unsigned int hp_dispatch_draw_mode_p4                                     :  1;
        unsigned int hp_dma_out_of_range_p3_q                                     :  1;
        unsigned int hp_dma_issue_p2                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_DEBUG_REG22 {
    struct {
        unsigned int wd_csbm_sm_spi_grant_ordered_id                              : 12;
        unsigned int wd_csbm_sm_gds_data                                          :  2;
        unsigned int wd_ngg_csbm_dma_busy                                         :  1;
        unsigned int csbm_tcarb_tag                                               : 10;
        unsigned int is_event_p3                                                  :  1;
        unsigned int eop_p3                                                       :  1;
        unsigned int null_p3                                                      :  1;
        unsigned int state_id_p3                                                  :  3;
        unsigned int wd_ngg_csbm_no_dma_busy                                      :  1;
    } most;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_ENHANCE {
    struct {
        unsigned int MISC                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_INDEX_BUF_BASE {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_INDEX_BUF_BASE_HI {
    struct {
        unsigned int BASE_HI                                                      :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int                                                              : 20;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int                                                              : 20;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int                                                              : 20;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int                                                              : 20;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_POS_BUF_BASE {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_POS_BUF_BASE_HI {
    struct {
        unsigned int BASE_HI                                                      :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_QOS {
    struct {
        unsigned int DRAW_STALL                                                   :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_UTCL1_CNTL {
    struct {
        unsigned int XNACK_REDO_TIMER_CNT                                         : 20;
        unsigned int                                                              :  3;
        unsigned int VMID_RESET_MODE                                              :  1;
        unsigned int DROP_MODE                                                    :  1;
        unsigned int BYPASS                                                       :  1;
        unsigned int INVALIDATE                                                   :  1;
        unsigned int FRAG_LIMIT_MODE                                              :  1;
        unsigned int FORCE_SNOOP                                                  :  1;
        unsigned int                                                              :  3;
    } bits, bitfields;
    struct {
        unsigned int                                                              : 29;
        unsigned int FORCE_SD_VMID_DIRTY                                          :  1;
        unsigned int                                                              :  2;
    } gfx09;
    struct {
        unsigned int                                                              : 29;
        unsigned int MTYPE_OVERRIDE                                               :  1;
        unsigned int                                                              :  2;
    } gfx10;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_UTCL1_STATUS {
    struct {
        unsigned int FAULT_DETECTED                                               :  1;
        unsigned int RETRY_DETECTED                                               :  1;
        unsigned int PRT_DETECTED                                                 :  1;
        unsigned int                                                              :  5;
        unsigned int FAULT_UTCL1ID                                                :  6;
        unsigned int                                                              :  2;
        unsigned int RETRY_UTCL1ID                                                :  6;
        unsigned int                                                              :  2;
        unsigned int PRT_UTCL1ID                                                  :  6;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

